#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
loadDataFrame <-
  function(url, csvDelimiter, missingDataStrategy) {
    dataFrame <- read.csv(url, sep=csvDelimiter)
    
    return(applyMissingDataStrategy(dataFrame, missingDataStrategy))
}

saveDataFrame <-
  function(dataFrame, dateColumnNr, location) {
    if(class(dataFrame[[dateColumnNr]]) != "Date") {
      stop("column number ", dateColumnNr, " of data frame does not contain dates. This causes invalid data on loading later.")
    }
    write.table(dataFrame, file=location)
}

readDataFrame <- 
  function(location, dateColumnNr) {
    dataFrame <- read.table(file=location)
    dataFrame[[dateColumnNr]] <- as.Date(dataFrame[[dateColumnNr]])
    if(class(dataFrame[[dateColumnNr]]) != "Date") {
      stop("column number ", dateColumnNr, " of loaded data frame does not contain dates")
    }
    
    dataFrame
}

applyDateFormat <-
  function(dataFrame, dateColumnNr, dateFormat="%d.%m.%Y") {
    dataFrame[[dateColumnNr]] <- as.Date(as.character(dataFrame[[dateColumnNr]]), format=dateFormat)
    return(dataFrame)
}

testDateColumnValidity <-
  function(dataFrame, dateColumnNr) {
    result = tryCatch({
      as.Date(dataFrame[[dateColumnNr]])
      naVector <- is.na(dataFrame[[dateColumnNr]])
      naIndices <- which(naVector)
      nas <- dataFrame[[dateColumnNr]][naIndices]
      n <- sum(naVector)
      isValid <- n == 0
      return(list(
        isValid=isValid,
        nas=nas,
        naIndices=naIndices
      ))
    }, error = function(e) {
      naVector <- is.na(dataFrame[[dateColumnNr]])
      naIndices <- which(naVector)
      nas <- dataFrame[[dateColumnNr]][naIndices]
      n <- sum(naVector)
      isValid <- n == 0
      return(list(
        isValid=isValid,
        nas=nas,
        naIndices=naIndices
      ))
    })

    return(result)
}


selectDataFrameByTimeRange <-
  function(dataFrame, dateColumnNr, startDate, endDate) {
    dateColumn <- dataFrame[[dateColumnNr]]
    
    as.Date(dateColumn) #make sure that the dataFrame's dateColumn is a valid
    
    startDateAsDate <- ifelse(class(startDate) == "Date", startDate, as.Date(as.character(startDate)));
    endDateAsDate <- ifelse(class(endDate) == "Date", endDate, as.Date(as.character(endDate)));
    
    newDataFrame <- dataFrame[dateColumn >= startDateAsDate & dateColumn <= endDateAsDate,]
    return(newDataFrame)
  }


applyMissingDataStrategy <-
  function(dataFrame, missingDataStrategy) {
    if(missingDataStrategy == 0) {
      return(dataFrame)
    }else if(missingDataStrategy == 1) {
      #return(na.exclude(dataFrame))
      return(dataFrame[complete.cases(dataFrame),])
    }
    
    stop("unknown missing data strategy chosen")
}


getHeadersForDataFrame <-
  function(dataFrame) {
	return(colnames(dataFrame))
}


getSomeRawValuesFromDataFrame <-
  function(dataFrame, headerName, doReduce=FALSE) {
    if(doReduce == TRUE) {
      head(dataFrame, n=10)[[headerName]]
    }else{
      dataFrame[[headerName]]
    }
    #return(values[[headerName]])
}

getDataFrameOverviewStatistics <-
  function(dataFrame) {
    totalNumberOfDataPoints = (ncol(dataFrame)-1) * nrow(dataFrame)
    numberOfNotAvailableDataPoints = sum(is.na(dataFrame))
    numberOfAvailableDataPoints = totalNumberOfDataPoints - numberOfNotAvailableDataPoints

    statistics <- list(
      totalNumberOfDataPoints=totalNumberOfDataPoints,
      numberOfNotAvailableDataPoints=numberOfNotAvailableDataPoints,
      numberOfAvailableDataPoints=numberOfAvailableDataPoints)
    return (statistics)
  }

#NOTE: this function is obsolete as we no longer plot graph by R but using QCustomPlot within the AT
# createDataFrameInfluenceFactorsPlot <-
#   function(dataFrame, behaviorIndicatorColumnNr, influenceFactorColumnNr, plotLocation) {
#     behaviorIndicator <- dataFrame[[behaviorIndicatorColumnNr]]
#     influenceFactor <- dataFrame[[influenceFactorColumnNr]]
#     
#     xlab <- colnames(dataFrame)[influenceFactorColumnNr]
#     ylab <- colnames(dataFrame)[behaviorIndicatorColumnNr]
#     
#     grDevices::svg(width=6, height=6, pointsize=10, filename=plotLocation)
#     plot(x=influenceFactor, y=behaviorIndicator, xlab=xlab, ylab=ylab, pch=16)
#     dev.off()
#   }


loadDataFrameInfluenceFactors <-
  function(dataFrame, behaviorIndicatorColumnNr, influenceFactorColumnNr) {
    originalBehaviorIndicatorColumn <- dataFrame[[behaviorIndicatorColumnNr]]
    originalInfluenceFactorColumn <- dataFrame[[influenceFactorColumnNr]]

    columns.names <- colnames(dataFrame)
    byColumnsShrinkedDataFrame <- subset(dataFrame, select = c(columns.names[behaviorIndicatorColumnNr], columns.names[influenceFactorColumnNr]))
    operationDataFrame <- na.omit(byColumnsShrinkedDataFrame)
    
    behaviorIndicatorColumn <- operationDataFrame[[1]]
    influenceFactorColumn <- operationDataFrame[[2]]

    
    return(list(
      behaviorIndicatorColumnValues=behaviorIndicatorColumn,
      influenceFactorColumnValues=influenceFactorColumn
    ))
}


loadDataFrameTimeSeriesOverview <- 
  function(dataFrame, dateColumnNr, valueColumnNr) {
    dateColumn <- dataFrame[[dateColumnNr]]
    valueColumn <- dataFrame[[valueColumnNr]]
    
    #xlab <- colnames(dataFrame)[dateColumnNr]
    #ylab <- colnames(dataFrame)[valueColumnNr]
    
    #grDevices::svg(width=6, height=6, pointsize=10, filename=plotLocation)
    #plot(y=valueColumn, x=dateColumn, xlab=xlab, ylab=ylab, pch=16, type="l", col=c("black"))
    #dev.off()
    
    return(list(
      dateValues=dateColumn,
      values=valueColumn
    ))
}