#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
# fromDate and toDate is what the user has chosen in the GUI (exactly that) (same with toDate)
# as some regressors need to have a startup time, the true dataFrame for the regression needs to start earlier so that all the
# regressors are ready at the fromDate. OverallMaxStartupTime represents exatly the number of timesteps (not dates!) we have to
# cut the dataFrame from the beginning at the end of the regression. This is exactly what evaluateExpansion is doing. This is the
# case where the regressors are used to calculate the fit. However, the regressors are also used for further calculations such as
# prediction, applyingFormula and so on. Therefore, all these methods have to cut the results at the latest at the end of their calculations
# with overallMaxStartupTime (making sure that all values are valid as in the case of the regression fit calculation).
loadRegressionFitDataFrame <-
  function(originalDataFrame, originalDateColumnNr, fromDate, toDate, overallMaxStartupTime, selectColumns) {
    #byColumnsShrinkedDataFrame <- subset(originalDataFrame, select = selectColumns)
    
    #dateColumnHeaderName <- getHeadersForDataFrame(originalDataFrame)[originalDateColumnNr]
    #newDateColumnNr <- which(getHeadersForDataFrame(byColumnsShrinkedDataFrame) == dateColumnHeaderName)
    
    #newDateColumn <- byColumnsShrinkedDataFrame[[newDateColumnNr]]
    
    dateColumn <- originalDataFrame[[originalDateColumnNr]]
    fromDateIndex <- which(dateColumn == fromDate)
    
    
    startupDateIndex <- fromDateIndex - overallMaxStartupTime
    if(startupDateIndex <= 0) {
      stop("loadRegressionFitDataFrame : cannot go further back into the past than where the dataFrame starts. FromDate: ", fromDate, " overallMaxStartupTime: ", overallMaxStartupTime)
    }
    
    startupDate <- dateColumn[startupDateIndex]
    
    byDatesShrinkedDataFrame = selectDataFrameByTimeRange(originalDataFrame, originalDateColumnNr, startupDate, toDate)
    byColumnsShrinkedDataFrame <- subset(byDatesShrinkedDataFrame, select = selectColumns)
    
    return(byColumnsShrinkedDataFrame)
  }

setPhaseToCalibration <-
  function() {
    phase <- "c"
    e <- baseenv()
    e$phase <- phase
}

setPhaseToPrediction <-
  function() {
  phase <- "p"
  e <- baseenv()
  e$phase <- phase
}

# use this function to execute concrete regression fit functions
# as here we make sure that before the regression fit function gets executed,
# the phase is always set to calibration and much more important, the phase state
# gets reset to prediction in every case, even if an exception occurs (due to 'finally' statement)
loadProtectedRegressionFit <-
  function(FUN, ...) {
    tryCatch({
      setPhaseToCalibration()
      #inspired by functional::Curry function
      .orig = list(...)
      fun <- function(...) do.call(FUN, c(.orig, list(...)))
      result <- fun()
      
      if (any(is.na(coef(result$rFit)))) 
        stop ("there are aliased coefficients in the model")
      
      return(result)
    }, finally = {
      setPhaseToPrediction()
    })
}


loadRegressionFit <-
  function(dataFrame, formula) {
    result <- 
    fit <- lm(formula=formula, data=dataFrame, x=TRUE)
    
    #regressors <- attr(terms(fit), "term.labels")
    coefficients <- coefficients(fit)
    
    wrappedFit <- list(
      rFit=fit,
      dataFrame=dataFrame,
      coefficients=coefficients
    )
    
    return (wrappedFit)
  }

#######################################################################################################
# Code derived from                                                                                   #
#                                                                                                     #
#    John Fox, Professor                                                                              #
#    Department of Sociology                                                                          #
#    McMaster University                                                                              #
#    Hamilton, Ontario, Canada                                                                        #
#    web: socserv.mcmaster.ca/jfox                                                                    #
#######################################################################################################
loadRegressionFitLmPwi <-
  function(dataFrame, formula, iterations=10) {
    outerModel <- lm(formula=formula, data=dataFrame, x=TRUE)
    X <- model.matrix(outerModel)
    y <- model.response(model.frame(outerModel))
    e <- residuals(outerModel)
    
    innerModel <- outerModel
    
    d.terms <- terms(formula)
    
    #initial values of while loop
    rho_old <- 0
    rho_new <- 1
    i <- 1
    
    while(abs(rho_new-rho_old) > 0.0001) {
      rho_old <- rho_new
      
      #       #inner model CO
      #       n <- length(e)
      #       rho_new <- sum(e[1:(n-1)]*e[2:n])/sum(e^2)
      #       cat("rho: ",rho_new,"\n")
      #       y <- y[2:n] - rho_new * y[1:(n-1)]
      #       X <- X[2:n,] - rho_new * X[1:(n-1),]
      
      #inner (transformed) model PWI (--> problems of rho is closed to 1)
      n <- length(e)
      rho_new <- sum(e[1:(n-1)]*e[2:n])/sum(e[2:(n-1)]^2)
      #cat("rho: ",rho_new,"\n")
      yTrans <- c(y[1] * (1 - rho_new^2)^0.5, y[2:n] - rho_new * y[1:(n-1)])
      XTrans <- rbind(X[1,] * (1 - rho_new^2)^0.5, X[2:n,] - rho_new * X[1:(n-1),])
      
      innerDataFrame <- data.frame(yTrans,XTrans)
      
      innerFormula <- paste0(colnames(innerDataFrame), sep=" + ", collapse="") #make string from colnames(X)
      innerFormula <- substr(innerFormula,1,(nchar(innerFormula)-3)) # delete " + " at the end of the string 
      innerFormula <- sub("\\+", "~", innerFormula)
      innerFormula <- paste0(innerFormula, " - 1")
      
      innerModel <- lm(formula=innerFormula, data=innerDataFrame, x=TRUE) #note: transforms model to e.g. : X.I(dambase.direct(T_H12_DO)) (Intercept is included in XTrans)
      
      #residuals of not transformed (outer) model
      e <- y - X %*% as.numeric(coef(innerModel))
      
      #counter
      i <- i+1
      if (i >= iterations){
        message("iteration of Prais-Winston did not converge")
        break
      }
      
    }
    
    d.x <- innerModel$x
    
    #+
    model <- model.frame(formula, dataFrame)
    
    #next, get matrix out of modelframe
    datamatrix <- data.matrix(model, rownames.force=F)
    datamatrix.dimnames <- dimnames(datamatrix)
    predictor.unscaled <- datamatrix[, 1]
    d.n <- dim(datamatrix)[1]
    
    #create the unscaled X matrix out of the datamatrix
    X.unscaled <- cbind(rep(1,d.n), datamatrix[, 2:dim(datamatrix)[2]])
    d.SST <- t(predictor.unscaled) %*% predictor.unscaled - sum(predictor.unscaled)^2/d.n
    #+
    
    # all back transformed coefficients
    d.Coefficients <- coef(innerModel)
    dimnames.results.withoutintercept <- datamatrix.dimnames[[2]][2:length(datamatrix.dimnames[[2]])]
    dimnames.results <- c("(Intercept)", dimnames.results.withoutintercept)
    names(d.Coefficients) <- dimnames.results #get names from mod$X (original names without the preceding 'X')
    
    # fitted values from back transformed coefficients
    d.fitted.values <- X.unscaled %*% d.Coefficients
    
    
    # residual values from back transformed coefficients
    d.residuals <- predictor.unscaled - d.fitted.values
    d.SSRes <- sum(d.residuals^2)
    d.p <- dim(X.unscaled)[2]
    d.MSRes <- as.numeric(d.SSRes/(d.n-d.p))
    
    
    #rho transformed values
    d.fitted.values.rho <- fitted(innerModel)
    #d.residuals.rho <- predictor.unscaled - d.fitted.values.rho
    d.residuals.rho <- residuals(innerModel)
    d.df.residuals.rho <- df.residual(innerModel)
    d.SSRes.rho <- sum(d.residuals.rho^2)
    d.MSRes.rho <- as.numeric(d.SSRes.rho/(d.n-d.p))
    
    #t-values
    d.t_values <- coef(summary(innerModel))[, "t value"]
    names(d.t_values) <- dimnames.results
    
    #p-values
    d.p_values <- coef(summary(innerModel))[, "Pr(>|t|)"]
    names(d.p_values) <- dimnames.results
    
    # vifs new
    vifs <- as.numeric(calculateVIFs(innerModel))
    vifs <- vifs[2:length(vifs)]
    names(vifs) <- dimnames.results.withoutintercept
    
    
    ### adj.r.squared
    adj.r.squared <- summary(innerModel)$adj.r.squared
    
    ### r.squared
    r.squared <- summary(innerModel)$r.squared
    
    ### fstatistic
    #     d.SSR <- d.SST - d.SSRes
    #     d.k <- d.p - 1
    #     d.fstatistic <- (d.SSR/d.k) / (d.SSRes/(d.n-d.k-1))
    
    d.fstatistic <- summary(innerModel)$fstatistic
    
    ### std errors
    std_errors <- coef(summary(innerModel))[, "Std. Error"]
    names(std_errors) <- dimnames.results
    
    ### std coefficients
    # calculateStdCoefficients provides wrong values, so we provide NaN temporarly
    std_coefficients <- rep(NaN, length(d.Coefficients)-1)
    names(std_coefficients) <- dimnames.results.withoutintercept
    #     
    #     result$cov <- (summary$sigma^2) * summary$cov.unscaled
    #     dimnames(result$cov) <- list(names, names)
    #     result$sigma <- summary$sigma
    #     result$rho <- rho
    
    res <- list(
      coefficients=d.Coefficients,
      rho=rho_new,
      residuals=d.residuals.rho,
      df.residual=d.df.residuals.rho,
      fitted.values=d.fitted.values.rho,
      ssres=d.SSRes,
      msres=d.MSRes,
      ssres.rho=d.SSRes.rho,
      msres.rho=d.MSRes.rho,
      t_values=d.t_values,
      p_values=d.p_values,
      vifs=vifs,
      adj.r.squared=adj.r.squared,
      r.squared=r.squared,
      fstatistic=d.fstatistic,
      std_errors=std_errors,
      std_coefficients=std_coefficients,
      model=innerModel,
      terms=d.terms,
      x=d.x,
      x.unscaled=d.x
    )
    
    class(res) <- 'lmpwi'
    return(res)
  }

loadRegressionFitLmPwiWrapper <-
  function(dataFrame, formula, iterations) {
    fit <- loadRegressionFitLmPwi(dataFrame, formula, iterations)
    
    coefficients <- coefficients(fit)
    
    wrappedFit <- list(
      rFit=fit,
      dataFrame=dataFrame,
      coefficients=coefficients
    )
    
    return (wrappedFit)
  }


####################
# Ridge regression #
####################
loadRegressionFitLmRidge <-
  function(dataFrame, formula, k) {
    # first, get the modelframe which means, apply the formula and end up with a data.frame containing all values of all regressors
    model <- model.frame(formula, dataFrame)
    d.terms <- terms(formula)
    
    #next, get matrix out of modelframe
    datamatrix <- data.matrix(model, rownames.force=F)
    datamatrix.dimnames <- dimnames(datamatrix)
    predictor.unscaled <- datamatrix[, 1]
    d.n <- dim(datamatrix)[1]
    
    #create the unscaled X matrix out of the datamatrix
    X.unscaled <- cbind(rep(1,d.n), datamatrix[, 2:dim(datamatrix)[2]])
    
    #next, scale entire datamatrix with unit length (note, predictor variable is also part of the scaled datamatrix)
    datamatrix.scaled <- matrix(nrow=dim(datamatrix)[1], ncol=dim(datamatrix)[2]) # initialize
    for(column_i in 1:dim(datamatrix.scaled)[2]) {
      datamatrix.scaled[, column_i] <- unitLengthScale(datamatrix[, column_i])
    }
    #restore dimnames on the scaled datamatrix
    dimnames(datamatrix.scaled) <- dimnames(datamatrix)
    
    #create the scaled X matrix out of the datamatrix.scaled
    #by removing predictor variable (mostly Pendulum)
    X.scaled <- datamatrix.scaled[, 2:dim(datamatrix.scaled)[2]]
    
    predictor.scaled <- datamatrix.scaled[, 1]
    
    # regression analysis
    d.I <- diag(dim(t(X.scaled) %*% X.scaled)[1])
    d.Beta_RR <- ginv(t(X.scaled) %*% X.scaled + k*d.I) %*% t(X.scaled) %*% predictor.scaled
    
    # back transformation from d.Beta_RR to unscaled Beta values (Montgomery S.114ff)
    d.SST <- t(predictor.unscaled) %*% predictor.unscaled - sum(predictor.unscaled)^2/d.n
    d.mean <- rep(0, dim(X.scaled)[2]) # initialize
    d.Sjj <- rep(0, dim(X.scaled)[2]) # initialize
    for(ii in 1:(dim(X.scaled)[2])) {
      d.mean[ii] <- mean(X.unscaled[, 1+ii])
      d.Sii <- sum((X.unscaled[, 1+ii]-d.mean[ii])^2)
      d.Sjj[ii] <- d.Sii
    }
    d.Beta <- d.Beta_RR*(d.SST/d.Sjj)^0.5
    d.Intercept <- mean(predictor.unscaled) - sum(d.Beta * d.mean)
    
    
    
    
    # all back transformed coefficients
    d.Coefficients <- c(d.Intercept, d.Beta)
    # dimnames.results holds all the dimnames for values related to intercept and regressors
    dimnames.results.withoutintercept <- datamatrix.dimnames[[2]][2:length(datamatrix.dimnames[[2]])]
    dimnames.results <- c("(Intercept)", dimnames.results.withoutintercept)
    names(d.Coefficients) <- dimnames.results
    
    
    # fitted values from back transformed coefficients
    d.fitted.values <- X.unscaled %*% d.Coefficients
    
    
    # residual values from back transformed coefficients
    d.residuals <- predictor.unscaled - d.fitted.values
    d.SSRes <- sum(d.residuals^2)
    d.p <- dim(X.unscaled)[2]
    d.MSRes <- as.numeric(d.SSRes/(d.n-d.p))
    d.df.residuals <- d.n - dim(X.scaled)[2] - 1
    
    ### t-values except for intercept
    d.Cjj_Snake <- diag(ginv(t(X.scaled) %*% X.scaled + k*d.I))
    d.Syy <- sum((predictor.unscaled - mean(predictor.unscaled))^2)
    d.t_values <- as.vector(d.Beta_RR %*% sqrt(d.Syy)/sqrt(d.MSRes*d.Cjj_Snake))
    d.t_values <- c(NaN, d.t_values) #make sure that intercept has a NaN assigned in order to keep indexing on the application working
    names(d.t_values) <- dimnames.results
    
    ### p-values except for intercept
    d.p_values <- 2*pt(abs(d.t_values), d.df.residuals, lower.tail=FALSE)
    
    ### VIFs
    vifs <- d.Cjj_Snake
    names(vifs) <- dimnames.results.withoutintercept
    
    ### r.squared
    d.r.squared <- 1-(d.SSRes/d.SST)
    
    ### adj.r.squared
    adj.r.squared <- (d.SSRes/(d.n - d.p))/(d.SST/(d.n - 1))
    
    ### fstatistic
    ## note: the following is the MSReg and not MSRes!
    d.MSReg <- sum((d.fitted.values - mean(d.fitted.values))^2)
    #d.fstatistic <- c(d.MSReg/d.MSRes, NaN, NaN)
    d.ResVar <- d.SSRes/d.df.residuals
    d.fstatistic <- c(value=(d.MSReg/(d.p - 1)/d.ResVar), numdf = d.p - 1, dendf = d.df.residuals)
    
    
    ### std errors
    d.Cjj <- diag(ginv(t(X.unscaled)%*%X.unscaled))
    std_errors <- sqrt(d.MSRes*d.Cjj)
    names(std_errors) <- dimnames.results.withoutintercept

    
    res <- list(
      x.unscaled=X.unscaled,
      x=X.unscaled,
      model=model,
      terms=d.terms,
      coefficients=d.Coefficients,
      residuals=d.residuals,
      df.residual=d.df.residuals,
      fitted.values=d.fitted.values,
      sst=d.SST,
      ssres=d.SSRes,
      msres=d.MSRes,
      vifs=vifs,
      t_values=d.t_values,
      p_values=d.p_values,
      r.squared=d.r.squared,
      adj.r.squared=adj.r.squared,
      fstatistic=d.fstatistic,
      std_errors=std_errors
    )
    
    class(res) <- "lmridge"
    return(res)
  }

loadRegressionFitLmRidgeWrapper <-
  function(dataFrame, formula, k) {
    fit <- loadRegressionFitLmRidge(dataFrame, formula, k)
    
    coefficients <- coefficients(fit)
    
    wrappedFit <- list(
      rFit=fit,
      dataFrame=dataFrame,
      coefficients=coefficients
    )
    
    return (wrappedFit)
  }

##################################
# Principal component regression #
##################################
loadRegressionFitLmPcr <-
  function(dataFrame, formula, r) {
    # first, get the modelframe which means, apply the formula and end up with a data.frame containing all values of all regressors
    model <- model.frame(formula, dataFrame)
    d.terms <- terms(formula)
    
    #next, get matrix out of modelframe
    datamatrix <- data.matrix(model, rownames.force=F)
    datamatrix.dimnames <- dimnames(datamatrix)
    predictor.unscaled <- datamatrix[, 1]
    d.n <- dim(datamatrix)[1]
    
    #create the unscaled X matrix out of the datamatrix
    X.unscaled <- cbind(rep(1,d.n), datamatrix[, 2:dim(datamatrix)[2]])
    
    #next, scale entire datamatrix with unit length (note, predictor variable is also part of the scaled datamatrix)
    datamatrix.scaled <- matrix(nrow=dim(datamatrix)[1], ncol=dim(datamatrix)[2]) # initialize
    for(column_i in 1:dim(datamatrix.scaled)[2]) {
      datamatrix.scaled[, column_i] <- unitLengthScale(datamatrix[, column_i])
    }
    #restore dimnames on the scaled datamatrix
    dimnames(datamatrix.scaled) <- dimnames(datamatrix)
    
    #create the scaled X matrix out of the datamatrix.scaled
    #by removing predictor variable (mostly Pendulum)
    X.scaled <- datamatrix.scaled[, 2:dim(datamatrix.scaled)[2]]
    
    predictor.scaled <- datamatrix.scaled[, 1]
    
    #regression analysis
    # eigenvectors and eigenvalues calculation
    t.Eigen <- eigen(t(X.scaled) %*% X.scaled)
    d.T <- t.Eigen$vectors
    d.Lambda <- t.Eigen$values
    
    #principal components
    d.Z <- X.scaled %*% d.T
    
    #regression coefficients
    d.Alpha <- ginv(diag(d.Lambda)) %*% t(d.Z) %*% predictor.scaled
    d.Alpha_PC <- c(d.Alpha[1:r], rep(0, length(d.Alpha)-r))
    d.Beta_PC <- d.T %*% d.Alpha_PC
    
    #back transformation of regression coefficients
    d.SST <- t(predictor.unscaled) %*% predictor.unscaled - sum(predictor.unscaled)^2/d.n
    d.mean <- rep(1, dim(d.Z)[2]) #initialize
    d.Sjj <- rep(1, dim(d.Z)[2]) #initialize
    for(ii in 1:(dim(d.Z)[2])) {
      d.mean[ii] <- mean(X.unscaled[, (1+ii)])
      d.Sii <- sum((X.unscaled[,(1+ii)]-d.mean[ii])^2)
      d.Sjj[ii] <- d.Sii
    }
    d.Beta <- d.Beta_PC*(d.SST/d.Sjj)^0.5
    d.Intercept <- mean(predictor.unscaled) - sum(d.Beta * d.mean)
    
    # coefficients as they are
    # all back transformed coefficients
    d.Coefficients <- c(d.Intercept, d.Beta)
    # dimnames.results holds all the dimnames for values related to intercept and regressors
    dimnames.results.withoutintercept <- datamatrix.dimnames[[2]][2:length(datamatrix.dimnames[[2]])]
    dimnames.results <- c("(Intercept)", dimnames.results.withoutintercept)
    names(d.Coefficients) <- dimnames.results
    
    # fitted values from back transformed coefficients
    d.fitted.values <- X.unscaled %*% d.Coefficients
    
    # residual values from back transformed coefficients
    d.residuals <- predictor.unscaled - d.fitted.values
    d.SSRes <- sum(d.residuals^2)
    d.p <- dim(X.unscaled)[2]
    d.MSRes <- as.numeric(d.SSRes/(d.n-d.p))
    d.df.residuals <- d.n - dim(X.scaled)[2] - 1
    
    ## vif-values
    #------------
    d.Cjj_Snake <- diag(ginv(d.T %*% diag(d.Lambda) %*% t(d.T)))
    vifs <- d.Cjj_Snake
    names(vifs) <- dimnames.results.withoutintercept
    
    ## t-values
    #----------
    d.Syy <- sum((predictor.unscaled - mean(predictor.unscaled))^2)
    d.t_values <- as.vector(d.Beta_PC %*% sqrt(d.Syy)/sqrt(d.MSRes*d.Cjj_Snake))
    d.t_values <- c(NaN, d.t_values) #make sure that intercept has a NaN assigned in order to keep indexing on the application working
    names(d.t_values) <- dimnames.results
    
    ## p-values
    #----------
    d.p_values <- 2*pt(abs(d.t_values), d.df.residuals, lower.tail=FALSE)
    names(d.p_values) <- dimnames.results
    
    ## r.squared
    #---------------
    d.r.squared <- 1-(d.SSRes/d.SST)
    
    ## adj.r.squared
    #---------------
    adj.r.squared <- (d.SSRes/(d.n - d.p))/(d.SST/(d.n - 1))
    
    ## fstatistic
    #------------
    ## note: the following is the MSReg and not MSRes!
    d.MSReg <- sum((d.fitted.values - mean(d.fitted.values))^2)
    #d.fstatistic <- c(d.MSReg/d.MSRes, NaN, NaN)
    d.ResVar <- d.SSRes/d.df.residuals
    d.fstatistic <- c(value=(d.MSReg/(d.p - 1)/d.ResVar), numdf = d.p - 1, dendf = d.df.residuals)
    
    ## std errors
    #------------
    d.Cjj <- diag(ginv(t(X.unscaled)%*%X.unscaled))
    std_errors <- sqrt(d.MSRes*d.Cjj)
    names(std_errors) <- dimnames.results.withoutintercept
    
    res <- list(
      x.unscaled=X.unscaled,
      x=X.unscaled,
      model=model,
      terms=d.terms,
      coefficients=d.Coefficients,
      residuals=d.residuals,
      df.residual=d.df.residuals,
      fitted.values=d.fitted.values,
      sst=d.SST,
      ssres=d.SSRes,
      msres=d.MSRes,
      vifs=vifs,
      t_values=d.t_values,
      p_values=d.p_values,
      r.squared=d.r.squared,
      adj.r.squared=adj.r.squared,
      fstatistic=d.fstatistic,
      std_errors=std_errors
    )
    
    class(res) <- "lmpcr"
    return(res)
  }

loadRegressionFitLmPcrWrapper <-
  function(dataFrame, formula, r) {
    fit <- loadRegressionFitLmPcr(dataFrame, formula, r)
    
    coefficients <- coefficients(fit)
    
    wrappedFit <- list(
      rFit=fit,
      dataFrame=dataFrame,
      coefficients=coefficients
    )
    
    return (wrappedFit)
  }

###
# unit length scaling mostly used by Ridge and Principial Component Regression
###
unitLengthScale <- function(t.j) {
  t.mean <- mean(t.j)
  t.sjj <- sum((t.j-t.mean)^2)
  t.xjj <- (t.j-t.mean)/sqrt(t.sjj)
  return(t.xjj)
}

###
# This is a prediction function which is able to handle algorithm specific behavior more
# appropriate than the provided predict methods. Also allows prediction for regression fits
# not supported by other packages (Ridge regression, Prais-Winsten regression and Principal component regression)
###
predictNewValues <-
  function(regressionFit, newdata) {
    model <- model.frame(terms(regressionFit), newdata, na.action=na.pass)
    
    #next, get matrix out of modelframe
    datamatrix <- data.matrix(model, rownames.force=F)
    datamatrix.dimnames <- dimnames(datamatrix)
    predictor.unscaled <- datamatrix[, 1]
    d.n <- dim(datamatrix)[1]
    message("d.n : ", d.n)
    
    #create the unscaled X matrix out of the datamatrix
    #
    
    X.unscaled <- cbind(rep(1,d.n), datamatrix[, 2:dim(datamatrix)[2]])
    
    predicted.values <- X.unscaled %*% coefficients(regressionFit)

    return(predicted.values)
  }


#formula is a subset of the original regression formula
loadAppliedFormula <-
  function(regressionFit, formula, originalDataFrame, originalDateColumnNr, startDate, endDate, overallMaxStartupTime, responseColumnNr, doApplyCoefficientsAndIntercept) {
    message("loadAppliedFormula");
    #modelframe <- model.frame(as.formula(PENDULUM ~ I(evaluateExpansion("{damreg.seasonal.sin(date_, factor = 1)}", startDiff = 0, finalCutDiff = 0, dateColumn = DATE, userargs = list())) + W_LEVEL), data=dataModel)
    
    operationDataFrame <- selectDataFrameByTimeRange(originalDataFrame, originalDateColumnNr, startDate, endDate)
    
    #get the model frame out of the formula using a given dataFrame
    modelframe <- model.frame(formula, data=operationDataFrame)
    
    #get the coefficients as well as the intercept out of the regressionFit
    fitted.intercept <- coefficients(regressionFit)[1]
    regressors <- as.character(attr(terms(modelframe), "variables")[3:length(attr(terms(modelframe), "variables"))])
    #get a subset of the fitted.coefficients with regard to the columns defined by the modelframe
    fitted.coefficients <- coefficients(regressionFit)[regressors]
    fitted.coefficients.names <- names(fitted.coefficients)
    
    modelframe.ordered <- subset(modelframe, select = fitted.coefficients.names)
    
    #fitted.coefficients * t(modelframe.ordered) = applyed formula to the raw values
    if(doApplyCoefficientsAndIntercept) {
      applyedValues <- fitted.coefficients %*% t(modelframe.ordered)
      applyedValues <- as.vector(t(applyedValues))
      applyedValues <- applyedValues + fitted.intercept
    }else{
      applyedValues <- rep(1, length(fitted.coefficients)) %*% t(modelframe.ordered)
      applyedValues <- as.vector(t(applyedValues))
      applyedValues <- applyedValues
    }
    
    
    response <- operationDataFrame[[responseColumnNr]]
    responseType <- class(response)
    
    
    #cut response and applyedValues with regard to overallMaxStartupTime
    applyedValues <- cutColumnByDiff(column=applyedValues, cutDiff=overallMaxStartupTime)
    response <- cutColumnByDiff(column=response, cutDiff=overallMaxStartupTime)
    
    
    return(
      list(
        #regressors=regressors,
        #fitted.intercept=fitted.intercept,
        #fitted.coefficients=fitted.coefficients,
        #fitted.coefficients.names=fitted.coefficients.names,
        #modelframe.ordered=modelframe.ordered,
        applyedValues=applyedValues,
        response=response,
        responseType=responseType
      )
    )
  }

loadBestFittingRegressionFormula <-
  function(dataFrame, behaviorIndicatorColumnNr) {
    message("loadBestFittingRegressionFormula");
    regressors <- list() #TODO: create a named list such as: MSV1=sin, MSV2=cos
    
    regressionFormula <- list(
      regressors=regressors
    )
  }


loadRegressionFitAnalysisPhysicalSummary <-
  function(regressionFit, dataFrame, dateColumnNr, behaviorIndicatorColumnNr, startDate, regressionEndDate, overallMaxStartupTime) {
    message("loadRegressionFitAnalysisPhysicalSummary");
    #confint<-predict(regressionFit, interval="confidence")
    #     message("loadRegressionFitAnalysisPhysicalSummary startDate : ", startDate)
    #     message("loadRegressionFitAnalysisPhysicalSummary regressionEndDate : ", regressionEndDate)
    behaviorIndicatorColumn <- dataFrame[[behaviorIndicatorColumnNr]]
    dateColumn <- dataFrame[[dateColumnNr]]
    
    endDate <- tail(dateColumn, 1)
	
  	message("loadRegressionFitAnalysisPhysicalSummary - startDate: ", as.character(startDate))
  	message("loadRegressionFitAnalysisPhysicalSummary - endDate: ", as.character(endDate))
    dataFrameForPrediction <- selectDataFrameByTimeRange(dataFrame, dateColumnNr, startDate, endDate)
    predictionDateValues <- dataFrameForPrediction[[dateColumnNr]]
    
    predictions <- predictNewValues(regressionFit=regressionFit, newdata=dataFrameForPrediction)
  	message("loadRegressionFitAnalysisPhysicalSummary - predictions: ", predictions[1], " - length: ", length(predictions))
    
    cis <- calculateCisForPrediction(regressionFit, predictions)
    lowerCi <- cis$lowerCi
    upperCi <- cis$upperCi
    
    #the date values returned need to be cutted by the overMaxStartupTime from the beginning of the column
    predictionDateValuesCutted <- cutColumnByDiff(predictionDateValues, overallMaxStartupTime)
    regressionStartDate <- as.character(startDate)
    regressionEndDate <- as.character(regressionEndDate)
    
    
    #out of limit values
    # all values of the behavior indicator (measurement values) which are either higher than the upperCi or lower than the lowerCi
    #first, cut behaviorIndicatorColumn of the dataFrameForPrediction to the final dates of the prediction
    behaviorIndicatorColumnStartingAtPrediction <- cutColumnByDatesAsColumn(dateColumn, behaviorIndicatorColumn, predictionDateValuesCutted[1])
    outOfLimit <- NULL
    for(i in 1:length(upperCi)) {
      if(is.na(upperCi[i]) || is.na(lowerCi[i])){
        outOfLimit[i] = NaN
      } else if(!is.na(behaviorIndicatorColumnStartingAtPrediction[i]) && upperCi[i] < behaviorIndicatorColumnStartingAtPrediction[i]) {
        outOfLimit[i] = behaviorIndicatorColumnStartingAtPrediction[i]
      }else if(!is.na(behaviorIndicatorColumnStartingAtPrediction[i]) && lowerCi[i] > behaviorIndicatorColumnStartingAtPrediction[i]) {
        outOfLimit[i] = behaviorIndicatorColumnStartingAtPrediction[i]
      }else{
        outOfLimit[i] = NaN
      }
    }    
    
    outOfLimitDates <- predictionDateValuesCutted
    
    X <- cis$X
    #extrapolations
    H <- X%*%ginv((t(X)%*%X))%*%t(X)
    hmax <- max(diag(H))
    #message("hmax : ", hmax)
    
    #calculate H.prediction - from
    
      X.prediction <- calculateXpredictMatrix(regressionFit=regressionFit, newdata=dataFrameForPrediction, na.action=na.omit)
      #message("head(X.prediction)")
      #print(X.prediction[1:5,])
      #H.prediction <- X.prediction%*%ginv((t(X.prediction)%*%X.prediction))%*%t(X.prediction)
      #H.prediction <- t(X.prediction)%*%ginv((t(X)%*%X))%*%X.prediction
      H.prediction <- X.prediction%*%ginv((t(X)%*%X))%*%t(X.prediction)
      H.prediction.diag <- diag(H.prediction)
      hmax.prediction <- max(H.prediction.diag)
      #extrapolation.indices <- numeric()
      #for(i in length(H.prediction.diag)) {
      #  if(H.prediction.diag[i] > hmax) {
      #    extrapolation.indices <- append(extrapolation.indices, i)
      #  }
      #}
#     
    
      extrapolation.indices <- which(H.prediction.diag > hmax)
      extrapolation.values <- predictions[extrapolation.indices]
      extrapolation.dates <- predictionDateValuesCutted[extrapolation.indices]
#    extrapolation.values <- numeric()
#    extrapolation.dates <- character()
    
    
    return(list(
      dateValues=dateColumn,
      predictionDateValues=predictionDateValuesCutted,
      behaviorIndicatorValues=behaviorIndicatorColumn,
      predictions=predictions,
      lowerCi=lowerCi,
      upperCi=upperCi,
      outOfLimit=outOfLimit,
      outOfLimitDates=outOfLimitDates,
      regressionStartDate=regressionStartDate,
      regressionEndDate=regressionEndDate,
      extrapolation.values=extrapolation.values,
      extrapolation.dates=extrapolation.dates
      #H.prediction=H.prediction,
      #hmax=hmax,
      #hmax.prediction=hmax.prediction,
      #extrapolation.indices=extrapolation.indices
    ))
  }

loadRegressionFitResidualNormalProbability <-
  function(regressionFit) {
    message("loadRegressionFitResidualNormalProbability")
    stdres <- rstandard(regressionFit)
    
    #instead of using the qqnorm algorithm, we implement it here because we need a special probabilities steps
    #     mean.stdres <- mean(stdres)
    #     var.stdres <- var(stdres)
    #     sd.stdres <- sd(stdres)
    #     n <- length(stdres)
    #     
    #     probabilities <- (1:n - 0.5)/n
    #     normal.quantiles <- qnorm(probabilities, mean.stdres, sd.stdres)
    
    #line
    probs <- c(0.25, 0.75) #1st and 3rd quartile
    
    #calculate quantiles with algorithm nr 5 which stands for p[k] = (k - 0.5) / n. See ?quantile for more information
    y <- quantile(sort(stdres), probs, names=FALSE, type=5, na.rm=TRUE)
    x <- qnorm(probs)
    slope <- diff(y)/diff(x)
    int <- y[1] - slope * x[1]
    
    x1 = min(stdres)/slope
    y1 = min(stdres) + int
    
    x2 = max(stdres)/slope
    y2 = max(stdres) + int
    
    line = list(c(x1, y1), c(x2, y2))
    
    #     normal.quantiles.sorted = sort(normal.quantiles)
    #     stdres.sorted = sort(stdres)
    
    
    #new approach
    data <- qqnorm(as.numeric(stdres), plot.it=F)
    normal.quantiles.sorted <- data$x
    stdres.sorted <- data$y
    #/new approach
    
    cat("### rstandard : ", min(stdres), max(stdres))
    
    return(list(
      theoreticalQuantiles= normal.quantiles.sorted,
      sampleQuantiles = stdres.sorted,
      line=line
    ))
  }

loadRegressionFitResidualTimeSeries <-
  function(regressionFit, regressionDataFrame, regressionDateColumnNr, predictionBehaviorIndicatorColumnNr, predictionDataFrame, predictionDateColumnNr, predictionStartDate, overallMaxStartupTime) {
    message("loadRegressionFitResidualTimeSeries")
    #if(class(regressionFit) != "lm") stop("Not an object of class 'lm' - further tests are required - glm should also work here")
    
    residuals.values <- residuals(regressionFit)
    if(class(regressionFit$na.action) == "omit") {
      dateColumn <- na.omit(regressionDataFrame)[[regressionDateColumnNr]]
    }else{
      dateColumn <- regressionDataFrame[[regressionDateColumnNr]]
    }
    
    #cut dateColumn with overallMaxStartupTime time steps
    dateColumn <- cutColumnByDiff(dateColumn, overallMaxStartupTime)
    
    if(length(dateColumn) != length(residuals.values)) stop ("unknown missing data strategy used by fitting model")
    
    
    # calculate residuals of the prediction period
    predictionBehaviorIndicatorColumn <- predictionDataFrame[[predictionBehaviorIndicatorColumnNr]]
    predictionDateColumn <- predictionDataFrame[[predictionDateColumnNr]]
    
    predictionEndDate <- tail(predictionDateColumn, 1)
    dataFrameForPrediction <- selectDataFrameByTimeRange(predictionDataFrame, predictionDateColumnNr, predictionStartDate, predictionEndDate)
    predictionDateValues <- dataFrameForPrediction[[predictionDateColumnNr]]
    
    predictionDateValuesCutted <- cutColumnByDiff(predictionDateValues, overallMaxStartupTime)    
    behaviorIndicatorColumnStartingAtPrediction <- cutColumnByDatesAsColumn(predictionDateColumn, predictionBehaviorIndicatorColumn, predictionDateValuesCutted[1])
    
    
    predictions <- predictNewValues(regressionFit=regressionFit, newdata=dataFrameForPrediction)
    message("length(which) : ", length(which(is.na(predictions))))
    #message("foo : ", dataFrameForPrediction$W_LEVEL[1])
    #message("length: ", length(predictions))
    residuals.prediction <- behaviorIndicatorColumnStartingAtPrediction - predictions
    
    cis <- calculateCisForPrediction(regressionFit, predictions)
    lowerCi.absolute <- cis$lowerCi
    upperCi.absolute <- cis$upperCi
    
    lowerCi.relative <- lowerCi.absolute - predictions
    upperCi.relative <- upperCi.absolute - predictions
    
    outOfLimit <- NULL
    message("length(residuals.predicion) : ", length(residuals.prediction))
    for(i in 1:length(upperCi.relative)) {
      if(is.na(residuals.prediction[i])) {
        outOfLimit[i] = NaN
      }else if(!is.na(residuals.prediction[i]) && upperCi.relative[i] < residuals.prediction[i]) {
        outOfLimit[i] = residuals.prediction[i] #behaviorIndicatorColumnStartingAtPrediction[i]
      }else if(!is.na(residuals.prediction[i]) && lowerCi.relative[i] > residuals.prediction[i]) {
        outOfLimit[i] = residuals.prediction[i]
      }else{
        outOfLimit[i] = NaN
      }
    }
    
    outOfLimitDates <- predictionDateValuesCutted
    
    regressionStartDate <- as.character(head(dateColumn, 1))
    regressionEndDate <- as.character(tail(dateColumn, 1))
    
    #see p 23 on the weber's documentation for how to calculate the inverval
    #predictions <- predict(regressionFit, newdata=regressionDataFrame, se.fit=T, interval="predict")$fit
    
    X <- cis$X
    #extrapolations
    H <- X%*%ginv((t(X)%*%X))%*%t(X)
    hmax <- max(diag(H))
    #message("hmax : ", hmax)
    
    #calculate H.prediction - from
          X.prediction <- calculateXpredictMatrix(regressionFit=regressionFit, newdata=dataFrameForPrediction, na.action=na.omit)
          #H.prediction <- X.prediction%*%ginv((t(X.prediction)%*%X.prediction))%*%t(X.prediction)
          #H.prediction <- t(X.prediction)%*%ginv((t(X.prediction)%*%X.prediction))%*%X.prediction
          H.prediction <- X.prediction%*%ginv((t(X)%*%X))%*%t(X.prediction)
          H.prediction.diag <- diag(H.prediction)
          hmax.prediction <- max(H.prediction.diag)
    
          message("hmax: ", hmax, " , hmax.prediction: ", hmax.prediction)
    
          #extrapolation.indices <- numeric()
          #for(i in length(H.prediction.diag)) {
          #  if(H.prediction.diag[i] > hmax) {
          #    extrapolation.indices <- append(extrapolation.indices, i)
          #  }
          #}
    #     
          
          extrapolation.indices <- which(H.prediction.diag > hmax)
    
          #extrapolation.values <- predictions[extrapolation.indices]
          extrapolation.values <- rep(0, length(extrapolation.indices))
          extrapolation.dates <- predictionDateValuesCutted[extrapolation.indices]
    #extrapolation.values <- numeric()
    #extrapolation.dates <- character()
    
    return(list(
      residuals=residuals.values,
      dates=dateColumn,
      predictionResiduals=residuals.prediction,
      predictionDateValues=predictionDateValuesCutted,
      regressionStartDate=regressionStartDate,
      regressionEndDate=regressionEndDate,
      lowerCi=lowerCi.relative,
      upperCi=upperCi.relative,
      outOfLimit=outOfLimit,
      outOfLimitDates=outOfLimitDates,
      extrapolation.values=extrapolation.values,
      extrapolation.dates=extrapolation.dates
    ))
  }


loadRegressorOverview <-
  function(regressionFit) {
    message("loadRegressorOverview")
    #if(class(regressionFit) != "lm") stop("Not an object of class 'lm' - further tests are required")
    #NOTE: this is only valid for lm fits
    #sum <- summary(regressionFit)
    
    coefficients.values <- coefficients(regressionFit)
    names <- names(coefficients.values)
    tstats <- calculateTStats(regressionFit)
    pvalues <- calculatePValues(regressionFit)
    stderrors <- calculateStdErrors(regressionFit)
    #tstats <- coef(sum)[, "t value"] #for linearRidge class: pvals(regressionFit)$tstat
    #pvalues <- coef(sum)[, "Pr(>|t|)"]
    #stderrors <- coef(sum)[, "Std. Error"] #for linearRidge class: pvals(fitSchlR)$se
    vifs <- calculateVIFs(regressionFit)
    stdcoefficients <- calculateStdCoefficients(regressionFit)
    
    overview <- list(
      names=names,
      coefficients=coefficients.values,
      tstats=tstats,
      pvalues=pvalues,
      vifs=vifs,
      stderrors=stderrors,
      stdcoefficients=stdcoefficients
    )
    
    return(overview)
  }



calculateSSResAndMSResDerivedFromPrediction <-
  function(regressionFit, dataFrame, dateColumnNr, behaviorIndicatorColumnNr, predictionStartDate, observationStartDate, overallMaxStartupTime) {
    message("calculateSSResAndMSResDerivedFromPrediction")
    #behaviorIndicatorColumn <- dataFrame[[behaviorIndicatorColumnNr]]
    message("predictionStartDate: ", predictionStartDate)
    message("observationStartDate: ", observationStartDate)
    dateColumn <- dataFrame[[dateColumnNr]]
    
    endDate <- tail(dateColumn, 1)
    message("calculateSSResAndMSResDerivedFromPrediction - startDate: ", as.character(predictionStartDate))
    message("calculateSSResAndMSResDerivedFromPrediction - endDate: ", as.character(endDate))
    dataFrameForPrediction <- selectDataFrameByTimeRange(dataFrame, dateColumnNr, predictionStartDate, endDate)
    predictionDateValues <- cutColumnByDiff(dataFrameForPrediction[[dateColumnNr]], overallMaxStartupTime)
    predictionBehaviorIndicatorValues <- cutColumnByDiff(dataFrameForPrediction[[behaviorIndicatorColumnNr]], overallMaxStartupTime)
    
    
    #todo: cut the date values by overallMax...
    #predictionBehaviorIndicatorValues <- dataFrameForPrediction[[behaviorIndicatorColumnNr]]
    
    #     message("[1]")
    #predictions<-predict(regressionFit, se.fit=FALSE, newdata=dataFrameForPrediction) #these are the regular predictions as done everywhere else
    predictions <- predictNewValues(regressionFit, newdata=dataFrameForPrediction)
    message("predictions - calculateSSResAndMSResDerivedFromPrediction : ", predictions[1], " - length: ", length(predictions))
    #     message("[2]")
    
    predictionsForObservation <- cutColumnByDatesAsColumn(dateColumn=predictionDateValues, column=predictions, startDate=observationStartDate)
    #     message("[3 - predictionDateValues: ", length(predictionDateValues))
    #     message("[3 - predictionBehaviorIndicatorValues: ", length(predictionBehaviorIndicatorValues))
    behaviorIndicatorValuesForObservation <- cutColumnByDatesAsColumn(dateColumn=predictionDateValues, column=predictionBehaviorIndicatorValues, startDate=observationStartDate)
    #     message("[4]")
    
    numberOfObservations <- length(predictionsForObservation)
    
    message("length (predictionsForObservation) : ", length(predictionsForObservation))
    message("length (behaviorIndicatorValuesForObservation) : ", length(behaviorIndicatorValuesForObservation))
    
    ssres <- sum(na.omit(predictionsForObservation - behaviorIndicatorValuesForObservation)^2)
    #msres <- ssres/df.residual(regressionFit)
    msresderived <- ssres / numberOfObservations
    
    result <- list(
      ssres=ssres,
      msresderived=msresderived
    )
    
    return(result)
  }



loadRegressionSummary <-
  function(regressionFit, dataFrame, dateColumnNr, behaviorIndicatorColumnNr, predictionStartDate, observationStartDate, overallMaxStartupTime) {
    message("loadRegressionSummary")
#     message("loadRegressionSummary::predictionStartDate: ", predictionStartDate)
#     message("loadRegressionSummary::observationStartDate: ", observationStartDate)
    
    numberOfObservations <- nobs(regressionFit)
    ssres <- sum(resid(regressionFit)^2) #montgomery (2.16) P.20
    msres <- ssres/df.residual(regressionFit) #montgomery (3.17) P.81 #average variance of a residual
    msresderived <- ssres / numberOfObservations
    predValues <- calculateSSResAndMSResDerivedFromPrediction(regressionFit=regressionFit, dataFrame=dataFrame, dateColumnNr=dateColumnNr, behaviorIndicatorColumnNr=behaviorIndicatorColumnNr, predictionStartDate=predictionStartDate, observationStartDate=observationStartDate, overallMaxStartupTime=overallMaxStartupTime)
    ssresPred <- predValues$ssres
    msresderivedPred <- predValues$msresderived
    
#     message("predValues$ssres : ", predValues$ssres)
#     message("predValues$msresderived : ", predValues$msresderived)
    
    #NOTE, this is only valid for lm fits
    #r2 <- summary(regressionFit)$r.squared
    r2 <- calculateR2(regressionFit)
    r2adj <- calculateR2Adj(regressionFit)
    fstat <- calculateFStat(regressionFit)
    #fstat <- summary(regressionFit)$fstatistic
    fstatpvalue <- pf(fstat[1],fstat[2],fstat[3],lower.tail=FALSE)
    #dwstat <- durbinWatsonTest(regressionFit)$dw
    dwstat <- calculateDWStat(regressionFit)
    #library("car")
    #detach("package:car", unload=TRUE)
    
    summary <- list(
      ssres=ssres,
      msres=msres,
      msresderived=msresderived,
      r2=r2,
      r2adj=r2adj,
      fstat=fstat,
      fstatpvalue=fstatpvalue,
      dwstat=dwstat,
      nobs=numberOfObservations,
      ssresPred=ssresPred,
      msresderivedPred=msresderivedPred
    )
    
    return(summary)
  }

loadRegressionFitACF <- function(regressionFit) {
  message("loadRegressionFitACF")
  x <- residuals(regressionFit)
  #na.action.def=na.actionna.omit
  
  if(class(regressionFit$na.action) == "omit") {
    na.action.def <- na.omit
  }else{
    na.action.def <- na.pass
  }
  acf.result <- acf(x=x, na.action=na.action.def, type="correlation", plot=F, lag.max=365)
  
  acf.result.lag=acf.result$lag
  acf.result.acf=acf.result$acf
  acf.result.n.used=acf.result$n.used
  
  #calculate confidence interval - derived from plot.acf {stats}
  ci.level <- 0.95 #confidence interval level
  clim0 <-
    qnorm((1 + ci.level)/2)/sqrt(acf.result.n.used)
  
  ci.interval=c(-clim0, clim0)
  
  result <- list(
    lag=acf.result.lag,
    acf=acf.result.acf,
    ci.interval=ci.interval,
    n.used=acf.result.n.used
  )
  
  return (result)
}


loadRegressionFitResidualsVersusFitted <- 
  function(regressionFit, externallyStudentizedResiduals=TRUE) {
    message("loadRegressionFitResidualsVersusFitted")
    
    fittedValues <- fitted(regressionFit)
    residualValues <- resolveResiduals(regressionFit, externallyStudentizedResiduals)
    
    return(list(
      parametervalues=fittedValues,
      residuals=residualValues
    ))
  }

loadOverallRegressionQuality <-
  function(regressionFit) {
    message("loadOverallRegressionQuality")
    autocor.dwstat.value <- calculateDWStat(regressionFit)
    
    #define durbin watson p1 expression with regard to the test definitions
    if(calculateDW(regressionFit, alternative="less")$p.value <= 1e-5) {
      autocor.dwstat.degree <- -1; #p1 == -1 -> negative autocorrelation
    }else if(calculateDW(regressionFit, alternative="greater")$p.value <= 1e-5) {
      autocor.dwstat.degree <- 1; #p1 == 1 -> positive autocorrelation
    }else{
      autocor.dwstat.degree <- 0; #p1 == 0 -> no autocorrellation
    }
    
    fit.r2adj.value <- calculateR2Adj(regressionFit)
    if(fit.r2adj.value == 1) {
      fit.r2adj.degree = 0 #perfect fit
    }else if(fit.r2adj.value >= 0.95) { #0.95:1
      fit.r2adj.degree = 1 #ok fit
    }else if(fit.r2adj.value > 0.5) { #0.5:0.95
      fit.r2adj.degree = 2 #bad fit
    }else if(fit.r2adj.value > 0) { #0:0.5
      fit.r2adj.degree = 3 #very bad fit 
    }
    
    #multicol.vifs <- calculateVIFs(regressionFit)
    
    
    return(list(
      autocor.dwstat.value=autocor.dwstat.value,
      autocor.dwstat.degree=autocor.dwstat.degree,
      fit.r2adj.value=fit.r2adj.value,
      fit.r2adj.degree=fit.r2adj.degree
    ))
  }

### the old way:
# evaluateExpansion <-
#   function (exp, pre=function(dateColumn, column){return(list(dateColumn=dateColumn, column=column))}, post=function(dateColumn, column){column}, dateColumn, column=FALSE, userargs=list()) {
#     if(column == FALSE) {
#       pair <- pre(dateColumn=dateColumn, column=FALSE)
#       result <- evaluate(exp, list(date_=pair$dateColumn, arg=userargs))
#       post(dateColumn=pair$dateColumn, column=result)
#       
#     }else{
#       pair <- pre(dateColumn=dateColumn, column=column)
#       result <- evaluate(exp, list(date_=pair$dateColumn, col_=pair$column, arg=userargs))
#       post(dateColumn=pair$dateColumn, column=result)
#     }
# }


# start diff is a positive value used as an index for the start date / start value of the regressor
evaluateExpansion <-
  function (exp, startDiff, finalCutDiff, dateColumn, column, userargs=list(), preprocessors) {
    expstr <- paste(exp, paste(userargs, collapse=", "))
    #message("expstr : ", expstr)
    if(exists(expstr, envir=baseenv())) {
      #message("expstr exists")
      e <- baseenv()
      userargs <- c(userargs, container=get(as.character(expstr), envir=e))
    }else{
      #message("exp does not exist : ")
      e <- baseenv()
      newenv <- new.env()
      ascharacterexp <- as.character(expstr)
      assign(ascharacterexp, newenv, envir=e)
      userargs <- c(userargs, container=newenv)
    }
    
    if(class(dateColumn) != "Date") {
      dateColumn <- as.Date(dateColumn) #try to transform dateColumn into Date objects -> fails if the dateColumn contains misspelled dates
    }
    startDate <- dateColumn[1]
    
    if(missing(column)) {
      #pair <- pre(dateColumn=dateColumn, column=FALSE)
      #result <- evaluate(exp, list(date_=pair$dateColumn, arg=userargs))
      #post(dateColumn=pair$dateColumn, column=result)
      #stop("evaluate expansion for column==FALSE not implemented yet")
      #message("R: startDiff: ", startDiff, " , finalCutDiff : ", finalCutDiff)
      dateColumnForCalculation <- cutColumnByDiff(dateColumn, startDiff)
      result <- evaluate(exp, list(date_=dateColumnForCalculation, arg=userargs))
      cutColumnByDatesWithStartDiffAsColumn(dateColumnForCalculation, column=result, startDiff=finalCutDiff, startDate=dateColumnForCalculation[1])
    }else{
      # pair contains values where the regression including startup time can be calculated on
      pair <- cutColumnByDatesWithStartDiffAsPair(dateColumn=dateColumn, column=column, startDiff=startDiff, startDate=startDate)
      
      # remember the indices of N/As within the original column
      na.positions.vector <- which(is.na(pair$column))
      # now the regressor can calculate the values
      
      # pair -> create a data frame, apply na.omit and get the colums back
      dc <- pair$dateColumn
      vc <- pair$column
      packedDataFrame <- na.omit(data.frame(dc, vc))
      
      #run all preprocessors on date and col
      if(!missing(preprocessors)) {
        if(class(preprocessors) != "list") { #we might introduce a generic function instead of this test
          preprocessors <- list(preprocessors) 
        }
        for(i in length(preprocessors)) {
          packedDataFrame$vc <- evaluate(preprocessors[i], list(date_=packedDataFrame$dc, col_=packedDataFrame$vc))
        }
      }
      #execute the regressor formula 'exp'
      result <- evaluate(exp, list(date_=packedDataFrame$dc, col_=packedDataFrame$vc, arg=userargs))
      
      #unpack: insert N/As to the result as they were stored before
      if(length(na.positions.vector) > 0) {
        result.unpacked <- rep(NA,length(result)+length(na.positions.vector))
        #print(result.unpacked)
        result.unpacked[-na.positions.vector] <- result
        result <- result.unpacked
      }
      
      
      # finally, cut the values from the startup period of the regressor so that the column contains only valid regressor values
      cutColumnByDatesWithStartDiffAsColumn(dateColumn=pair$dateColumn, column=result, startDiff=finalCutDiff, startDate=pair$dateColumn[1])
    }
  }


# evaluate("{ar+ar}", list(ar<-3))
evaluate <-
  function (exp, arg) {
    #ex <- expression(expr)
    #evalq(exp, arg)
    eval(parse(text = exp), arg)
  }

# gets back startDiff steps from the startDate (single steps, not date related steps such as days) at the dateColumn and cuts the column to this date as well as the endDate
# normally, the startDate is the date where the user says 'the regression should start from here' (same for endDate)
# startDiff is positive, if the effective start is later than the startDate. So, if you want to set the effective start date
# earlier than the startDate, startDiff must be negative
cutColumnByDatesWithStartDiffAsPair <-
  function(dateColumn, column, startDiff, startDate) {
    dateIndex <- which(dateColumn == startDate)
    calculatedDateIndex <- dateIndex + startDiff
    if(calculatedDateIndex <= 0) {
      stop("cannot go further back into the past than where the dataFrame starts")
    }
    
    effectiveStartDate <- dateColumn[calculatedDateIndex]
    
    cutColumnByDatesAsPair(dateColumn, column, effectiveStartDate)
  }

cutColumnByDatesWithStartDiffAsColumn <-
  function(dateColumn, column, startDiff, startDate) {
    cutColumnByDatesWithStartDiffAsPair(dateColumn, column, startDiff, startDate)$column 
  }


cutColumnByDatesAsColumn <-
  function(dateColumn, column, startDate) {
    cutColumnByDatesAsPair(dateColumn, column, startDate)$column
  }

cutColumnByDatesAsPair <-
  function(dateColumn, column, startDate) {
    dataFrame <- data.frame(dateColumn, column)
    
    dateColumn <- dataFrame[1]
    column <- dataFrame[2]
    
    startDateAsString <- ifelse(class(startDate) == "Date", as.character(startDate), startDate);
    
    newDataFrame <- dataFrame[dateColumn >= startDateAsString,]
    return(list(dateColumn=newDataFrame$dateColumn, column=newDataFrame$column))
  }

cutColumnByDiff <-
  function(column, cutDiff) {
    finalLength <- length(column)
    
    if(cutDiff < 0) {
      stop("cutDiff cannot be shorter than 0")
    }
    
    if(cutDiff >= finalLength) {
      return(NA)
    }
    
    column[(cutDiff+1):finalLength]
  }

testExpression <-
  function (exp) {
    parse(text = exp)
  }


####################
# Helper functions #
####################

calculateDWStat <-
  function(regressionFit) {
    return(calculateDW(regressionFit)$statistic)
  }

calculateDW <- function(regressionFit, ...) UseMethod("calculateDW", regressionFit)

calculateR2 <- function(regressionFit) UseMethod("calculateR2", regressionFit)

calculateR2Adj <- function(regressionFit) UseMethod("calculateR2Adj", regressionFit)

calculateTStats <- function(regressionFit) UseMethod("calculateTStats", regressionFit)

calculatePValues <- function(regressionFit) UseMethod("calculatePValues", regressionFit)

calculateStdErrors <- function(regressionFit) UseMethod("calculateStdErrors", regressionFit)

calculateStdCoefficients <- function(regressionFit) UseMethod("calculateStdCoefficients", regressionFit)

calculateVIFs <- function(regressionFit) UseMethod("calculateVIFs", regressionFit)

calculateFStat <- function(regressionFit) UseMethod("calculateFStat", regressionFit)

calculateCisForPrediction <- function(regressionFit, ...) UseMethod("calculateCisForPrediction", regressionFit)

calculateXpredictMatrix <- function(regressionFit, ...) UseMethod("calculateXpredictMatrix", regressionFit)

resolveResiduals <-
  function(regressionFit, externallyStudentizedResiduals) {
    if(externallyStudentizedResiduals==TRUE) {
      rstudent(regressionFit)
    }else{
      residuals(regressionFit)
    }
  }

environmentTest <- function(env, mode) {
  if(mode == "c") {
    env$value <- 1234
    return(1)
  }else{
    return(env$value)
  }
}

