#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
######
# Standard lm
######
calculateR2Adj.lm <-
  function(regressionFit) {
    if(class(regressionFit) != "lm") stop("Not an object of class 'lm' - further tests are required")
    return(summary(regressionFit)$adj.r.squared)
}

calculateDW.lm <-
  function(regressionFit, ...) {
    dwtest(regressionFit, ...)
}

calculateTStats.lm <-
  function(regressionFit) {
    coef(summary(regressionFit))[, "t value"]
}

calculatePValues.lm <-
  function(regressionFit) {
    coef(summary(regressionFit))[, "Pr(>|t|)"]
}

calculateStdErrors.lm <-
  function(regressionFit) {
    coef(summary(regressionFit))[, "Std. Error"]
}

# derived from 'car' package v2.0-19
#-------------------------------------------------------------------------------
# Revision history:
# 2009-09-28 by J. Fox 
#-------------------------------------------------------------------------------

# Generalized Variance-Inflation Factors (Henric Nilsson and John Fox)
calculateVIFs.lm <- function(regressionFit) {
  doCalculateVIFs(regressionFit)
}
doCalculateVIFs <- function(mod, ...) {
  if (any(is.na(coef(mod)))) 
    stop ("there are aliased coefficients in the model")
  v <- vcov(mod)
  assign <- attributes(model.matrix(mod))$assign
  if (names(coefficients(mod)[1]) == "(Intercept)") {
    v <- v[-1, -1]
    assign <- assign[-1]
  }
  #else warning("No intercept: vifs may not be sensible.")
  terms <- labels(terms(mod))
  n.terms <- length(terms)
  if (n.terms < 2) stop("model contains fewer than 2 terms - cannot calculate VIF")
  R <- cov2cor(v)
  detR <- det(R)
  result <- matrix(0, n.terms, 3)
  rownames(result) <- terms
  colnames(result) <- c("GVIF", "Df", "GVIF^(1/(2*Df))")
  for (term in 1:n.terms) {
    subs <- which(assign == term)
    result[term, 1] <- det(as.matrix(R[subs, subs])) *
      det(as.matrix(R[-subs, -subs])) / detR
    result[term, 2] <- length(subs)
  }
  if (all(result[, 2] == 1)) result <- result[, 1]
  else result[, 3] <- result[, 1]^(1/(2 * result[, 2]))
  result
}

# derived from QuantPsyc
# Calculate Standardized Coefficients for any regressor but the intercept!
calculateStdCoefficients.lm <- function(regressionFit) {
  b <- summary(regressionFit)$coef[-1, 1]
  sx <- sapply(regressionFit$model[-1], sd)
  sy <- sapply(regressionFit$model[1], sd)
  beta <- b * sx/sy
  return(beta)
}


calculateR2.lm <- function(regressionFit) {
  return(summary(regressionFit)$r.squared)
}

calculateFStat.lm <- function(regressionFit) {
  return(summary(regressionFit)$fstatistic)
}

calculateCisForPrediction.lm <- function(regressionFit, predictions) {
  #for general calculations: see MainForm.cpp (ancient damReg) line 1792
  #ci <- predict(regressionFit, se.fit=FALSE, interval="confidence", level=0.95, newdata=dataFrameForPrediction)
  #lowerCi <- ci[, 2]
  #upperCi <- ci[, 3]
  ssres <- sum(resid(regressionFit)^2) #montgomery (2.16) P.20
  msres <- ssres/df.residual(regressionFit) #montgomery (3.17) P.81 #average variance of a residual
  
  #new approach
  #tvalue <- qt(0.975, regressionFit$df.residual)
  #apply(X, 1, function(x) x%*%ginv((t(X)%*%X))*t(x))
  #apply(X, 1, function(x) sqrt(sigma^2*(1+x%*%ginv((t(X)%*%X))*t(x))))
  X <- regressionFit$x
  sigma.square <- msres
  cis.interval <- apply(X, 1, function(x) sqrt(sigma.square*(1+t(x)%*%ginv((t(X)%*%X))%*%x)))
  t.dist.values <- qt(0.975, regressionFit$df.residual)
  lowerCi <- predictions - t.dist.values*cis.interval
  upperCi <- predictions + t.dist.values*cis.interval
  
  return(list(
      lowerCi=lowerCi,
      upperCi=upperCi,
      X=X
    ))
}

calculateXpredictMatrix.lm <- function(regressionFit, newdata, na.action=na.pass) {
  tt <- terms(regressionFit)
  Terms <- delete.response(tt)
  m <- model.frame(Terms, newdata, na.action = na.action, 
                   xlev = regressionFit$xlevels)
  if (!is.null(cl <- attr(Terms, "dataClasses"))) 
    .checkMFClasses(cl, m)
  X <- model.matrix(Terms, m, contrasts.arg = regressionFit$contrasts)
  X
}


######
# Prais-Winsten delegates
######
residuals.lmpwi <- function(x) {
  return(x$residuals)
}

calculateR2Adj.lmpwi <-
 function(regressionFit) {
   return(summary(regressionFit$mod)$adj.r.squared)
}

calculateDW.lmpwi <-
  function(regressionFit, ...) {
    dwtest(regressionFit$mod, ...)
}

calculateTStats.lmpwi <-
  function(regressionFit) {
    return(regressionFit$t_values)
}

calculatePValues.lmpwi <-
  function(regressionFit) {
    return(regressionFit$p_values)
}

calculateStdErrors.lmpwi <-
  function(regressionFit) {
    return(regressionFit$std_errors)
}

calculateVIFs.lmpwi <-
  function(regressionFit) {
    return(regressionFit$vifs)
}

calculateStdCoefficients.lmpwi <-
  function(regressionFit) {
    return(regressionFit$std_coefficients)
}

nobs.lmpwi <-
 function(object, ...) {
   return(nobs(object$mod, ...))
}


calculateR2.lmpwi <-
  function(regressionFit) {
    return(regressionFit$r.squared)
}

calculateFStat.lmpwi <- function(regressionFit) {
  return(regressionFit$fstatistic)
}

df.residual.lmpwi <- function(object, ...) {
  return(object$df.residual)
}

rstandard.lmpwi <- function(model, ...) {
  return(rstandard(model$mod))
}

rstudent.lmpwi <- function(model, ...) {
  return(rstudent(model$mod, ...))
}

calculateCisForPrediction.lmpwi <- function(regressionFit, predictions) {
  #for general calculations: see MainForm.cpp (ancient damReg) line 1792
  ssres <- regressionFit$ssres
  msres <- regressionFit$msres
  
  #new approach
  #tvalue <- qt(0.975, regressionFit$df.residual)
  #apply(X, 1, function(x) x%*%ginv((t(X)%*%X))*t(x))
  #apply(X, 1, function(x) sqrt(sigma^2*(1+x%*%ginv((t(X)%*%X))*t(x))))
  X <- regressionFit$x
  sigma.square <- msres
  cis.interval <- apply(X, 1, function(x) sqrt(sigma.square*(1+t(x)%*%ginv((t(X)%*%X))%*%x)))
  t.dist.values <- qt(0.975, regressionFit$df.residual)
  lowerCi <- predictions - t.dist.values*cis.interval
  upperCi <- predictions + t.dist.values*cis.interval
  
  return(list(
    lowerCi=lowerCi,
    upperCi=upperCi,
    X=X
  ))
}

calculateXpredictMatrix.lmpwi <- function(regressionFit, newdata, na.action=na.pass) {
  tt <- terms(regressionFit)
  Terms <- delete.response(tt)
  m <- model.frame(Terms, newdata, na.action = na.action, 
                   xlev = regressionFit$xlevels)
  if (!is.null(cl <- attr(Terms, "dataClasses"))) 
    .checkMFClasses(cl, m)
  X <- model.matrix(Terms, m, contrasts.arg = regressionFit$model$contrasts)
  X
}


######
# Ridge delegates
######

# rstudent implementation for ridge regression
# Montgomery p131.ff
rstudent.lmridge <- function(model, ...) {
  #calculate the hat matrix of the model
  x <- model$x.unscaled
  x.hat <- x%*%ginv(t(x)%*%x)%*%t(x)
  
  res.studentized <- residuals(model)/sqrt(model$msres * (1-diag(x.hat)))
  return(res.studentized)
}

rstandard.lmridge <- function(model, ...) {
  return(residuals(model)/sqrt(model$msres))
  #return(rstandard(model$mod))
}

calculateDW.lmridge <-
  function(regressionFit, ...) {
    dwtest(regressionFit, ...)
}

calculateR2.lmridge <-
  function(regressionFit) {
   return(regressionFit$r.squared)
}

calculateFStat.lmridge <-
  function(regressionFit) {
    return(regressionFit$fstatistic)
}

calculateR2Adj.lmridge <-
  function(regressionFit) {
    return(regressionFit$adj.r.squared)
    #return(summary(regressionFit$mod)$adj.r.squared)
    
}

calculateTStats.lmridge <-
  function(regressionFit) {
    return(regressionFit$t_values)
}

calculatePValues.lmridge <-
  function(regressionFit) {
    return(regressionFit$p_values)
}

calculateStdErrors.lmridge <-
  function(regressionFit) {
    return(regressionFit$std_errors)
}

calculateVIFs.lmridge <-
  function(regressionFit) {
    return(regressionFit$vifs)
}

# derived from QuantPsyc
# Calculate Standardized Coefficients for any regressor but the intercept!
calculateStdCoefficients.lmridge <- function(regressionFit) {
  b <- coefficients(regressionFit)[2:length(coefficients(regressionFit))]
  sx <- sapply(regressionFit$model[-1], sd)
  sy <- sapply(regressionFit$model[1], sd)
  beta <- b * sx/sy
  return(beta)
}

nobs.lmridge <-
  function(object, ...) {
   return(NROW(object$residuals))
}


calculateCisForPrediction.lmridge <- function(regressionFit, predictions) {
  #for general calculations: see MainForm.cpp (ancient damReg) line 1792
  ssres <- regressionFit$ssres
  msres <- regressionFit$msres
  
  X <- regressionFit$x
  sigma.square <- msres
  cis.interval <- apply(X, 1, function(x) sqrt(sigma.square*(1+t(x)%*%ginv((t(X)%*%X))%*%x)))
  t.dist.values <- qt(0.975, regressionFit$df.residual)
  lowerCi <- predictions - t.dist.values*cis.interval
  upperCi <- predictions + t.dist.values*cis.interval
  
  return(list(
    lowerCi=lowerCi,
    upperCi=upperCi,
    X=X
  ))
}

calculateXpredictMatrix.lmridge <- function(regressionFit, newdata, na.action=na.pass) {
  tt <- terms(regressionFit)
  Terms <- delete.response(tt)
  m <- model.frame(Terms, newdata, na.action = na.action, 
                   xlev = regressionFit$xlevels)
  if (!is.null(cl <- attr(Terms, "dataClasses"))) 
    .checkMFClasses(cl, m)
  X <- model.matrix(Terms, m, contrasts.arg = regressionFit$model$contrasts)
  X
}


######
# Principal component regression delegates
######
# rstudent implementation for ridge regression
# Montgomery p131.ff
rstudent.lmpcr <- function(model, ...) {
  #calculate the hat matrix of the model
  x <- model$x.unscaled
  x.hat <- x%*%ginv(t(x)%*%x)%*%t(x)
  
  res.studentized <- residuals(model)/sqrt(model$msres * (1-diag(x.hat)))
  return(res.studentized)
}

rstandard.lmpcr <- function(model, ...) {
  return(residuals(model)/sqrt(model$msres))
}

calculateDW.lmpcr <-
  function(regressionFit, ...) {
    dwtest(regressionFit, ...)
  }

calculateR2.lmpcr <-
  function(regressionFit) {
    return(regressionFit$r.squared)
  }

calculateFStat.lmpcr <-
  function(regressionFit) {
    return(regressionFit$fstatistic)
  }

calculateR2Adj.lmpcr <-
  function(regressionFit) {
    return(regressionFit$adj.r.squared)
    #return(summary(regressionFit$mod)$adj.r.squared)
    
  }

calculateTStats.lmpcr <-
  function(regressionFit) {
    return(regressionFit$t_values)
  }

calculatePValues.lmpcr <-
  function(regressionFit) {
    return(regressionFit$p_values)
  }

calculateStdErrors.lmpcr <-
  function(regressionFit) {
    return(regressionFit$std_errors)
  }

calculateVIFs.lmpcr <-
  function(regressionFit) {
    return(regressionFit$vifs)
  }

# derived from QuantPsyc
# Calculate Standardized Coefficients for any regressor but the intercept!
calculateStdCoefficients.lmpcr <- function(regressionFit) {
  b <- coefficients(regressionFit)[2:length(coefficients(regressionFit))]
  sx <- sapply(regressionFit$model[-1], sd)
  sy <- sapply(regressionFit$model[1], sd)
  beta <- b * sx/sy
  return(beta)
}

nobs.lmpcr <-
  function(object, ...) {
    return(NROW(object$residuals))
  }

calculateCisForPrediction.lmpcr <- function(regressionFit, predictions) {
  #for general calculations: see MainForm.cpp (ancient damReg) line 1792
  ssres <- regressionFit$ssres
  msres <- regressionFit$msres
  
  X <- regressionFit$x
  sigma.square <- msres
  cis.interval <- apply(X, 1, function(x) sqrt(sigma.square*(1+t(x)%*%ginv((t(X)%*%X))%*%x)))
  t.dist.values <- qt(0.975, regressionFit$df.residual)
  lowerCi <- predictions - t.dist.values*cis.interval
  upperCi <- predictions + t.dist.values*cis.interval
  
  return(list(
    lowerCi=lowerCi,
    upperCi=upperCi,
    X=X
  ))
}

calculateXpredictMatrix.lmpcr <- function(regressionFit, newdata, na.action=na.pass) {
  tt <- terms(regressionFit)
  Terms <- delete.response(tt)
  m <- model.frame(Terms, newdata, na.action = na.action, 
                   xlev = regressionFit$xlevels)
  if (!is.null(cl <- attr(Terms, "dataClasses"))) 
    .checkMFClasses(cl, m)
  X <- model.matrix(Terms, m, contrasts.arg = regressionFit$model$contrasts)
  X
}