#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
## scaling of regressors
#--------------------------------------------------------------------------------------

# scale between chosen maximum and minimum value (e.g. Max OL, min OL) to (0,1) (see Nedushan (2002)
f.WLev_sc_Nedushan <- function(wlevel, min, max){
  WLev_sc <- (wlevel-min)/(max-min)
  return(WLev_sc)
}

# scale between chosen maximum and minimum value (e.g. Max OL, min OL) to (1,0) (see Tatin et al. (2013), Simon et al. (2013))
f.WLev_sc_EDF <- function(wlevel, min, max){
  WLev_sc <- (max-wlevel)/(max-min)
  return(WLev_sc)
}

# scale Seasonal function according to Verbund, Widmann (1967), Breitenstein et al. (1985)
f.Seas_sc_Verbund <- function(date_){
  library(lubridate)
  f.Seas_sc <- 2*pi/365*yday(date_) #yday gives number of day in the year (from libridate package)
  return(f.Seas_sc)
}


## polynomials
#--------------------------------------------------------------------------------------

#create polynomial of chosen order
f.polynomial <- function(data,ord){
  poly <- data^ord
  return(poly)
}



## delay
#--------------------------------------------------------------------------------------

#delayed effects (see Tatin, 2013, "Thermal Displacements of Concrete Dams:...)
# f.functional_delay_disc_int <- function(d.Phi, d.date, d.T0){
#   d.n <- as.numeric(length(d.Phi)) #size
#   d.dt <- as.numeric(d.date[2:d.n]) - as.numeric(d.date[1:(d.n-1)]) #difference in days
#   #transformation
#   d.F <- rep(0,d.n) #initialise d.F
#   for (ii in 2:d.n){
#     d.F[ii] <- exp(-d.dt[ii-1]/d.T0)*d.F[ii-1] + (1-exp(-d.dt[ii-1]/d.T0))*d.Phi[(ii)]
#   }
#   return(d.F)
# }

## HST Nedushan
#--------------------------------------------------------------------------------------
f.Nedushan_Regr_sin.cos <- function(date_){
  d.Season <- f.Seas_sc_Verbund(date_)
  Regr_sin.cos <- sin(d.Season)*cos(d.Season)
  return(Regr_sin.cos)
}

f.Nedushan_Regr_sin2 <- function(date_){
  d.Season <- f.Seas_sc_Verbund(date_)
  Regr_sin2 <- sin(d.Season)*sin(d.Season)
  return(Regr_sin2)
}



## HSTT Verbund, Widmann (1967), Breitenstein et al. (1985)
#--------------------------------------------------------------------------------------
f.year.Temp  <- function(date_, AirTemp){
  d.Season <- f.Seas_sc_Verbund(date_)
  fit.Temp <- lm(AirTemp ~ sin(d.Season) + cos(d.Season)) #fit of air temperatures
  N.Temp <- fitted(fit.Temp) #normally expected temperature
  return(N.Temp)
}

f.Verbund_Delta.T <- function(AirTemp, date_){
  Delta.T <- AirTemp - f.year.Temp(date_, AirTemp)
  return(Delta.T)
}

f.Verbund_Delta.h <- function(h){
  Delta.h <- h[2:length(h)]-h[1:(length(h)-1)]
  Delta.h <- c(Delta.h[1], Delta.h) #the first value is approximated as the second one (because initial time is not considered)
  return(Delta.h)
}

f.Verbund_Delay <- function(data, date_, dT){ #delay dT days
  #an interpolation of the data to a daily time series is needed
  library(zoo)
  ts.data <- zoo(data, date_) #make time series (needed for interpolation)
  output = seq(min(date_), max(date_), by = "day")
  ts.data.interp <- na.approx(ts.data, x = as.Date, xout=output)
  T.calc <- c(rep(ts.data.interp[1],dT),ts.data.interp) #larger vector with dT-times the same value in front (the first dT values have an approximate interpolation)
  delayed_data <-ts.data.interp #initialise T
  for (ii in 1:length(ts.data.interp)){
    delayed_data[ii] <- mean(T.calc[(ii):(ii+dT)])   #calculate mean data values over the past dT days
  }
  #plot(ts.data.interp)
  #lines(delayed_data,col="red3")
  
  return(delayed_data)
}

f.Verbund_Regr_Delta.T5 <- function(AirTemp, date_){
  Delta.T <- f.Verbund_Delta.T(AirTemp, date_)
  Regr_Delta.T5 <- f.Verbund_Delay(Delta.T, date_, dT=5)
  return(Regr_Delta.T5)
}

f.Verbund_Regr_Delta.T20 <- function(AirTemp, date_){
  Delta.T <- f.Verbund_Delta.T(AirTemp, date_)
  Regr_Delta.T20 <- f.Verbund_Delay(Delta.T, date_, dT=20)
  return(Regr_Delta.T20)
}

f.Verbund_Regr_Delta.T50 <- function(AirTemp, date_){
  Delta.T <- f.Verbund_Delta.T(AirTemp, date_)
  Regr_Delta.T50 <- f.Verbund_Delay(Delta.T, date_, dT=50)
  return(Regr_Delta.T50)
}

f.Verbund_Regr_Delta.h20 <- function(h, date_){
  Delta.h <- f.Verbund_Delta.h(h)
  Regr_Delta.h20 <- f.Verbund_Delay(Delta.h, date_, dT=20)
  return(Regr_Delta.h20)
}


