/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * FactoryImpl.cpp
 *
 *  Created on: Jan 23, 2014
 *      Author: gerberma
 */

#include <QDir>
#include <boost/shared_ptr.hpp>
#include "app/GenericApplicationContext.hpp"
#include "app/Factory.hpp"
#include "app/types.hpp"
#include <stdexcept>

#include "model/DirectoryManager.hpp"
#include "model/DataModelAnalysisSettings.hpp"
#include "model/DataModelAnalysisTimeSeriesOverview.hpp"
#include "model/DataModelOverviewStatistics.hpp"
#include "model/DataModelAnalysisInfluenceFactors.hpp"
#include "model/r/DataFrameTimeSeriesOverviewProvider.hpp"
#include "model/r/IncreasingRegressionFormulaIndexManager.hpp"
#include "model/r/RegressionFitAnalysisRegressorOverviewProvider.hpp"
#include "model/r/RegressionFitAnalysisRegressionSummaryProvider.hpp"
#include "model/r/RegressionFitAnalysisResidualNormalProbabilityProvider.hpp"
#include "model/r/RegressionFitAnalysisResidualTimeSeriesProvider.hpp"
#include "model/r/RegressionFitAnalysisACFProvider.hpp"
#include "model/r/RegressionFitAnalysisResidualVersusRegressionParameterProvider.hpp"
#include "model/r/RegressionFitAnalysisOverallRegressionQualityProvider.hpp"
#include "model/r/RegressionFitAnalysisPhysicalBehaviorProvider.hpp"
#include "model/r/RegressionFitAnalysisRegressorTimeseriesProvider.hpp"
#include "model/r/RegressionFitAnalysisPhysicalSummaryProvider.hpp"
#include "model/RawDataCollector.hpp"


namespace app {

using namespace context;

template<>
DirectoryManager_t *Factory::create<DirectoryManager_t>() {
	using namespace dambase::model;

	return new DirectoryManager(QDir::tempPath() + "/dambase");
}

template<>
Core_Controller_t *Factory::create<Core_Controller_t>(){
	Model_t *model = FETCH_FROM_CTX(Model_t);
	Core_Window_t *coreUi = FETCH_FROM_CTX(Core_Window_t);
	Core_Controller_t *coreController = new Core_Controller_t(model, coreUi, FETCH_FROM_CTX(Project_Controller_t));

	return coreController;
}

template<>
Core_Window_t *Factory::create<Core_Window_t>() {
	Model_t *model = FETCH_FROM_CTX(Model_t);
	Core_Window_t *coreWindow = new Core_Window_t(model);

	return coreWindow;
}

template<>
Analysis_Loader_t *Factory::create<Analysis_Loader_t>() {
	return new Analysis_Loader_t(
			boost::bind(&Analysis_Controller_t::deserialize, FETCH_FROM_CTX(Analysis_Controller_t), _1)
	);
}

template<>
Project_Loader_t *Factory::create<Project_Loader_t>() {
	return new Project_Loader_t(
			boost::bind(&Project_Controller_t::deserialize, FETCH_FROM_CTX(Project_Controller_t), _1),
			FETCH_FROM_CTX(Analysis_Loader_t));
}

template<>
Project_Controller_t *Factory::create<Project_Controller_t>() {
	Project_Wizard_t *projectWizard = FETCH_FROM_CTX(Project_Wizard_t);
	Project_Factory_t *projectFactory = FETCH_FROM_CTX(Project_Factory_t);
	Core_Window_t *coreWindow = FETCH_FROM_CTX(Core_Window_t);
	Project_Overview_Widget_t *projectOverviewWidget = FETCH_FROM_CTX(Project_Overview_Widget_t);
	Navigation_Tree_Widget_t *navigationTreeWidget = FETCH_FROM_CTX(Navigation_Tree_Widget_t);
	Model_t *model = FETCH_FROM_CTX(Model_t);
	Project_Controller_t *projectController = new Project_Controller_t(projectFactory, projectWizard, model, coreWindow, projectOverviewWidget, navigationTreeWidget);

	return projectController;
}

template<>
Analysis_Controller_t *Factory::create<Analysis_Controller_t>() {
	Analysis_Controller_t *analysisController = new Analysis_Controller_t(
			FETCH_FROM_CTX(Core_Window_t),
			FETCH_FROM_CTX(Analysis_Ui_Manager_t),
			FETCH_FROM_CTX(Model_t)->getActivityTracker()
			);

	return analysisController;
}

template<>
AnalysisToolsControllerSubscriptionManager_t *Factory::create<AnalysisToolsControllerSubscriptionManager_t>() {
	return new AnalysisToolsControllerSubscriptionManager_t();
}


template<>
DataModel_Analysis_Settings_t *Factory::create<DataModel_Analysis_Settings_t>() {
	using namespace dambase::model;

	DataModel_Analysis_Settings_t *settings = new DataModelAnalysisSettings();

	boost::shared_ptr<DataModel_Analysis_Tools_Description_t> descriptions = FETCH_FROM_CTX(Model_t)->getDataModelAnalysisToolsDescription();

	//left side overview
	settings->addAutomaticLeftSideOverviewCategoryAnalysisToolDescription(
			descriptions->findOverviewAnalysisToolDescriptionById(DataModelAnalysisTimeSeriesOverview::getStaticToolId())
	);


	//right side overview
	settings->addAutomaticRightSideOverviewCategoryAnalysisToolDescription(
			descriptions->findOverviewAnalysisToolDescriptionById(DataModelOverviewStatistics::getStaticToolId())
	);

	settings->addAutomaticRightSideOverviewCategoryAnalysisToolDescription(
			descriptions->findOverviewAnalysisToolDescriptionById(DataModelAnalysisInfluenceFactors::getStaticToolId())
	);

	return settings;
}

template<>
DataModel_Controller_Internal_Subscription_Manager_Impl_t *Factory::create<DataModel_Controller_Internal_Subscription_Manager_Impl_t>() {
	return new DataModel_Controller_Internal_Subscription_Manager_Impl_t();
}

template<>
DataModel_Controller_Internal_Subscription_Manager_t* Factory::create<DataModel_Controller_Internal_Subscription_Manager_t>() {
	return new DataModel_Controller_Internal_Subscription_Manager_t(
				FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_Impl_t),
				FETCH_FROM_CTX(DataModel_Analysis_Tools_Controller_t)
			);
}

template<>
DataModel_Overview_Statistics_Subscription_Manager_t *Factory::create<DataModel_Overview_Statistics_Subscription_Manager_t>() {
	return new DataModel_Overview_Statistics_Subscription_Manager_t();
}

template<>
DataModel_Analysis_Tools_Ui_Manager_t *Factory::create<DataModel_Analysis_Tools_Ui_Manager_t>() {
	return new DataModel_Analysis_Tools_Ui_Manager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
DataModel_Overview_Statistics_Ui_Manager_t *Factory::create<DataModel_Overview_Statistics_Ui_Manager_t>() {
	return new DataModel_Overview_Statistics_Ui_Manager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
DataModel_Analysis_Tools_Controller_t *Factory::create<DataModel_Analysis_Tools_Controller_t>() {
	return new DataModel_Analysis_Tools_Controller_t(
				FETCH_FROM_CTX(DataModel_Analysis_Tools_Controller_Dispatcher_t)
			);
}

template<>
DataModel_Analysis_Tools_Controller_Dispatcher_t *Factory::create<DataModel_Analysis_Tools_Controller_Dispatcher_t>() {
	return new DataModel_Analysis_Tools_Controller_Dispatcher_t();
}

template<>
DataModel_Analysis_Overview_Summary_Controller_t *Factory::create<DataModel_Analysis_Overview_Summary_Controller_t>() {
	return new DataModel_Analysis_Overview_Summary_Controller_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(DataModel_Overview_Statistics_Ui_Manager_t),
				FETCH_FROM_CTX(DataModel_Overview_Statistics_Subscription_Manager_t)
			);
}

template<>
DataModel_Analysis_Influence_Factors_Ui_Manager_t *Factory::create<DataModel_Analysis_Influence_Factors_Ui_Manager_t>() {
	return new DataModel_Analysis_Influence_Factors_Ui_Manager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t *Factory::create<DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t>() {
	return new DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t();
}

template<>
DataModel_Analysis_Influence_Factors_Controller_t *Factory::create<DataModel_Analysis_Influence_Factors_Controller_t>() {
	return new DataModel_Analysis_Influence_Factors_Controller_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(DataModel_Analysis_Influence_Factors_Ui_Manager_t),
				FETCH_FROM_CTX(DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t)
			);
}

template<>
DataModel_Ui_Manager_t *Factory::create<DataModel_Ui_Manager_t>() {
	return new DataModel_Ui_Manager_t(FETCH_FROM_CTX(Ui_Factory_t), FETCH_FROM_CTX(Core_Window_t));
}

/************************************************
 * Data Model Time Series Analysis Tool related *
 ************************************************/
template<>
DataFrameTimeSeriesOverviewProvider_t *Factory::create<DataFrameTimeSeriesOverviewProvider_t>() {
	using namespace dambase::model::r;

	return new DataFrameTimeSeriesOverviewProvider();
}

template<>
DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t *Factory::create<DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t>() {
	return new DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t();
}

template<>
DataModelAnalysisTimeSeriesOverviewController_t *Factory::create<DataModelAnalysisTimeSeriesOverviewController_t>() {
	return new DataModelAnalysisTimeSeriesOverviewController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(DataModelAnalysisTimeSeriesOverviewUiManager_t),
				FETCH_FROM_CTX(DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t)
			);
}



/***************************
 * RegressionModel related *
 ***************************/
template<>
RegressionModel_Ui_Manager_t *Factory::create<RegressionModel_Ui_Manager_t>() {
	return new RegressionModel_Ui_Manager_t(
			FETCH_FROM_CTX(Ui_Factory_t),
			FETCH_FROM_CTX(HourGlassProgressManager_t),
			FETCH_FROM_CTX(Core_Window_t)
			);
}

template<>
RegressionModel_Internal_Generic_Controller_t *Factory::create<RegressionModel_Internal_Generic_Controller_t>() {
	return new RegressionModel_Internal_Generic_Controller_t(FETCH_FROM_CTX(RegressionModel_Ui_Manager_t));
}

template<>
Regression_Formula_Index_Manager_t *Factory::create<Regression_Formula_Index_Manager_t>() {
	using namespace dambase::model::r;

	return new IncreasingRegressionFormulaIndexManager(0);
}

template<>
RegressionModel_Controller_t *Factory::create<RegressionModel_Controller_t>() {
	return new RegressionModel_Controller_t(
				FETCH_FROM_CTX(Model_t)->getActivityTracker(),
				FETCH_FROM_CTX(RegressionModel_Internal_Generic_Controller_t),
				FETCH_FROM_CTX(RegressionModelAnalysisToolsController_t),
				FETCH_FROM_CTX(Model_t)->getRegressionModelAnalysisToolsDescription(),
				FETCH_FROM_CTX(RegressionModel_Ui_Manager_t)
			);
}

template<>
RegressionModel_Subscription_Manager_t *Factory::create<RegressionModel_Subscription_Manager_t>() {
	return new RegressionModel_Subscription_Manager_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionModel_Controller_t),
				FETCH_FROM_CTX(RegressionModelAnalysisToolsController_t),
				FETCH_FROM_CTX(RegressionFormulaMethodsController_t)
			);
}

template<>
RegressionModelAnalysisToolsControllerDispatcher_t *Factory::create<RegressionModelAnalysisToolsControllerDispatcher_t>(){
	return new RegressionModelAnalysisToolsControllerDispatcher_t();
}

template<>
RegressionModelAnalysisToolsController_t *Factory::create<RegressionModelAnalysisToolsController_t>() {
	return new RegressionModelAnalysisToolsController_t(FETCH_FROM_CTX(RegressionModelAnalysisToolsControllerDispatcher_t));
}

template<>
RegressionFormulaMethodsController_t *Factory::create<RegressionFormulaMethodsController_t>() {
	return new RegressionFormulaMethodsController_t();
}

template<>
R_RegressionFitAnalysisPhysicalSummaryProvider_t *Factory::create<R_RegressionFitAnalysisPhysicalSummaryProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisPhysicalSummaryProvider();
}

template<>
RegressionFitAnalysisPhysicalSummaryUiManager_t *Factory::create<RegressionFitAnalysisPhysicalSummaryUiManager_t>() {
	return new RegressionFitAnalysisPhysicalSummaryUiManager_t(
				FETCH_FROM_CTX(Ui_Factory_t)
			);
}

template<>
RegressionFitAnalysisPhysicalSummaryController_t *Factory::create<RegressionFitAnalysisPhysicalSummaryController_t>() {
	return new RegressionFitAnalysisPhysicalSummaryController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisPhysicalSummaryUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}

template<>
RegressionFitAnalysisRegressorOverviewProvider_t *Factory::create<RegressionFitAnalysisRegressorOverviewProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisRegressorOverviewProvider();
}

template<>
RegressionFitAnalysisRegressorOverviewSubscriptionManager_t *Factory::create<RegressionFitAnalysisRegressorOverviewSubscriptionManager_t>() {
	return new RegressionFitAnalysisRegressorOverviewSubscriptionManager_t(FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t));
}

template<>
RegressionFitAnalysisRegressorOverviewController_t *Factory::create<RegressionFitAnalysisRegressorOverviewController_t>() {
	return new RegressionFitAnalysisRegressorOverviewController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisRegressorOverviewUiManager_t),
				FETCH_FROM_CTX(RegressionFitAnalysisRegressorOverviewSubscriptionManager_t)
			);
}

template<>
RegressionFitAnalysisRegressionSummaryController_t *Factory::create<RegressionFitAnalysisRegressionSummaryController_t>() {
	return new RegressionFitAnalysisRegressionSummaryController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisRegressionSummaryUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}

template<>
RegressionFitAnalysisRegressionSummaryProvider_t *Factory::create<RegressionFitAnalysisRegressionSummaryProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisRegressionSummaryProvider();
}



template<>
RegressionFitAnalysisResidualNormalProbabilityProvider_t *Factory::create<RegressionFitAnalysisResidualNormalProbabilityProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisResidualNormalProbabilityProvider;
}

template<>
RegressionFitAnalysisResidualNormalProbabilityController_t *Factory::create<RegressionFitAnalysisResidualNormalProbabilityController_t>() {
	return new RegressionFitAnalysisResidualNormalProbabilityController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}



template<>
RegressionFitAnalysisResidualTimeSeriesProvider_t *Factory::create<RegressionFitAnalysisResidualTimeSeriesProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisResidualTimeSeriesProvider;
}

template<>
RegressionFitAnalysisResidualTimeSeriesController_t *Factory::create<RegressionFitAnalysisResidualTimeSeriesController_t>() {
	return new RegressionFitAnalysisResidualTimeSeriesController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisResidualTimeSeriesUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}



template<>
RegressionFitAnalysisACFProvider_t *Factory::create<RegressionFitAnalysisACFProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisACFProvider;
}

template<>
RegressionFitAnalysisACFController_t *Factory::create<RegressionFitAnalysisACFController_t>() {
	return new RegressionFitAnalysisACFController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisACFUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}



template<>
RegressionFitAnalysisResidualVersusRegressionParameterProvider_t *Factory::create<RegressionFitAnalysisResidualVersusRegressionParameterProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisResidualVersusRegressionParameterProvider;
}

template<>
RegressionFitAnalysisResidualVersusRegressionParameterController_t *Factory::create<RegressionFitAnalysisResidualVersusRegressionParameterController_t>() {
	return new RegressionFitAnalysisResidualVersusRegressionParameterController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}




template<>
RegressionFitAnalysisOverallRegressionQualityProvider_t *Factory::create<RegressionFitAnalysisOverallRegressionQualityProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisOverallRegressionQualityProvider;
}

template<>
RegressionFitAnalysisOverallRegressionQualityController_t *Factory::create<RegressionFitAnalysisOverallRegressionQualityController_t>() {
	return new RegressionFitAnalysisOverallRegressionQualityController_t(
				FETCH_FROM_CTX(Model_Factory_t),
				FETCH_FROM_CTX(RegressionFitAnalysisOverallRegressionQualityUiManager_t),
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}

template<>
RegressionFitAnalysisPhysicalBehaviorProvider_t *Factory::create<RegressionFitAnalysisPhysicalBehaviorProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisPhysicalBehaviorProvider;
}

template<>
RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t *Factory::create<RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t>() {
	return new RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t(
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}

template<>
RegressionFitAnalysisPhysicalBehaviorController_t *Factory::create<RegressionFitAnalysisPhysicalBehaviorController_t>() {
	return new RegressionFitAnalysisPhysicalBehaviorController_t(
					new RegressionFitAnalysisPhysicalBehaviorGenericController_t(
						FETCH_FROM_CTX(Model_Factory_t),
						FETCH_FROM_CTX(RegressionFitAnalysisPhysicalBehaviorUiManager_t),
						FETCH_FROM_CTX(RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t)
				)
			);
}

template<>
RegressionFitAnalysisRegressorTimeseriesProvider_t *Factory::create<RegressionFitAnalysisRegressorTimeseriesProvider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitAnalysisRegressorTimeseriesProvider();
}

template<>
RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t *Factory::create<RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t>() {
	return new RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t(
				FETCH_FROM_CTX(AnalysisToolsControllerSubscriptionManager_t)
			);
}

template<>
RegressionFitAnalysisRegressorTimeseriesController_t *Factory::create<RegressionFitAnalysisRegressorTimeseriesController_t>() {
	return new RegressionFitAnalysisRegressorTimeseriesController_t(
					new RegressionFitAnalysisRegressorTimeseriesGenericController_t(
						FETCH_FROM_CTX(Model_Factory_t),
						FETCH_FROM_CTX(RegressionFitAnalysisRegressorTimeseriesUiManager_t),
						FETCH_FROM_CTX(RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t)
				)
			);
}


/**
 *  Report related
 */
template<>
ReportInternalGenericController_t *Factory::create<ReportInternalGenericController_t>() {
	return new ReportInternalGenericController_t(FETCH_FROM_CTX(ReportUiManager_t));
}

template<>
ReportController_t *Factory::create<ReportController_t>() {
	return new ReportController_t(
				FETCH_FROM_CTX(ReportInternalGenericController_t),
				FETCH_FROM_CTX(ReportUiManager_t),
				FETCH_FROM_CTX(App_Factory_t)
			);
}

template<>
ReportSubscriptionManager_t *Factory::create<ReportSubscriptionManager_t>() {
	return new ReportSubscriptionManager_t(
				FETCH_FROM_CTX(ReportController_t)
			);
}

/**
 * Export related
 */
template<>
RawDataExportController_t *Factory::create<RawDataExportController_t>() {
	return new RawDataExportController_t(FETCH_FROM_CTX(RawDataExportUiManager_t));
}

template<>
RawDataCollector_t *Factory::create<RawDataCollector_t>() {
	using namespace dambase::model;

	return new RawDataCollector();
}
} /* namespace app */
