/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericApplicationContext.hpp
 *
 *  Created on: Nov 14, 2013
 *      Author: gerberma
 */

#ifndef GENERICAPPLICATIONCONTEXT_HPP_
#define GENERICAPPLICATIONCONTEXT_HPP_

#include <boost/function.hpp>

namespace app {
namespace context {

template<typename T, typename SubType>
T *defaultFactory() {
	return new SubType;
}

template<typename T>
T *defaultFactory() {
	return new T;
}

template<typename T>
void defaultCleanupStrategy(T *t) {
	delete t;
}

template<typename T>
void nullCleanupStrategy(T *t) {
	//empty
}

template<typename T>
class LifetimeStrategy {
public:
	typedef boost::function<T*()> FactoryFunction;
	typedef boost::function<void(T*)> CleanupStrategyFunction;

	LifetimeStrategy() :
			factoryFunction(defaultFactory<T>), cleanupStrategyFunction(
					defaultCleanupStrategy<T>) {
		/*
		 factoryFunction = 0;
		 cleanupStrategyFunction = 0;
		 */

	}

	LifetimeStrategy(FactoryFunction f, CleanupStrategyFunction c) :
			factoryFunction(f), cleanupStrategyFunction(c) {
	}

	LifetimeStrategy(FactoryFunction f) :
			factoryFunction(f), cleanupStrategyFunction(
					defaultCleanupStrategy<T>()) {
	}

	LifetimeStrategy(CleanupStrategyFunction c) :
			factoryFunction(defaultFactory<T>()), cleanupStrategyFunction(c) {
	}

	LifetimeStrategy(const LifetimeStrategy& a) :
			factoryFunction(a.factoryFunction), cleanupStrategyFunction(
					a.cleanupStrategyFunction) {
	}

	void operator =(const LifetimeStrategy& other) {
		factoryFunction = other.factoryFunction;
		cleanupStrategyFunction = other.cleanupStrategyFunction;
	}

	FactoryFunction factoryFunction;
	CleanupStrategyFunction cleanupStrategyFunction;
};

class ThrowLogicalErrorPolicy {
public:
	template<typename T>
	static T *testExpression(T *reference, std::string what = "") {
		if (reference == 0) {
			throw std::logic_error(what);
		} else {
			return reference;
		}
	}
};

/**
 * One context for one type with one Separator has exactly one LifetimeStrategy
 * Note that the Separator can get used to distinguish separate instances of an object of the same type.
 * For example,  GenericApplicationContext<A, ...> and GenericApplicationContext<A, ..., 1> refere to the same type
 * but not to the same object.
 * This separation mechanism can also be used for example to distinguish between Singleton instances and Non-Singleton
 * instances. To do so, you have to specialize this template for both cases where you can specialize to:
 * GenericApplicationContext<T, ErrorStrategy, Separator = Singleton> and GenericApplicationContext<T, ErrorStrategy, Separator = NonSingleton>
 * where Singleton and NonSingleton are either dedicated Classes or enum values.
 */
template<typename T, typename ErrorPolicy = ThrowLogicalErrorPolicy,
		int Separator = 0>
class GenericApplicationContext {
public:
	static void registerLifetimeStrategy(
			LifetimeStrategy<T> *lifetimeStrategy_) {
		lifetimeStrategy = lifetimeStrategy_;
	}

	static void unregisterLifetimeStrategy() {
		if (lifetimeStrategy != 0) {
			delete lifetimeStrategy;
			lifetimeStrategy = 0;
		}
	}

	static void releaseObject() {
		if (object != 0) {
			std::string msg(typeid(T).name());
			ErrorPolicy::testExpression(lifetimeStrategy,
					"no lifetime strategy registered " + msg)->cleanupStrategyFunction(
					object);
			object = 0;
		}
	}

	static T *get() {
		if (object == 0) {
			std::string msg(typeid(T).name());
			object = ErrorPolicy::testExpression(lifetimeStrategy,
					"no lifetime strategy registered " + msg)->factoryFunction();
		}

		return object;
	}

private:
	static LifetimeStrategy<T> *lifetimeStrategy;
	static T *object;
};

/**
 * Contains the lifetime strategy registered for this context
 */
template<typename T, typename ErrorPolicy, int Separator>
LifetimeStrategy<T> *GenericApplicationContext<T, ErrorPolicy, Separator>::lifetimeStrategy =
		0;

/**
 * Stores the singleton object for this context
 */
template<typename T, typename ErrorPolicy, int Separator>
T *GenericApplicationContext<T, ErrorPolicy, Separator>::object = 0;

/**
 * This is the scope for a given GenericApplicationContext.
 * Note that if destroyed, this scope will release the assigned object as well as
 * unregister and delete the lifetime strategy.
 * If you want to use only a scope for the given object then GenericApplicatonContextObjectScope
 * is more what you are looking for.
 */
template<typename ApplicationContext>
struct GenericApplicationContextScope {
	GenericApplicationContextScope() {
	}
	virtual ~GenericApplicationContextScope() {
		ApplicationContext::releaseObject();
		ApplicationContext::unregisterLifetimeStrategy();
	}
};

/**
 * This Scope can be used if you want to give the context object a scope which
 * is different from the scope of the entire application context. If you want to
 * use a scope for the application context as well as for the referenced object then
 * you might better use GenericApplicationContextScope
 */
template<typename ApplicationContext>
struct GenericApplicatonContextObjectScope {
	GenericApplicatonContextObjectScope() {
	}

	virtual ~GenericApplicatonContextObjectScope() {
		ApplicationContext::releaseObject();
	}
};

template<typename Ctx, typename Class>
void registerCtx() {
	Ctx::registerLifetimeStrategy(new LifetimeStrategy<Class>());
}


template<typename Ctx, typename Class>
void registerCtx(boost::function<Class*()> factoryFunction, boost::function<void(Class*)> cleanupFunction) {
	Ctx::registerLifetimeStrategy(new LifetimeStrategy<Class>(factoryFunction, cleanupFunction));
}

template<typename Ctx, typename Class>
const GenericApplicationContextScope<Ctx> registerCtxWithContextScope() {
	registerCtx<Ctx, Class>();
	return GenericApplicationContextScope<Ctx>();
}


template<typename Ctx, typename Class>
const GenericApplicationContextScope<Ctx> registerCtxWithContextScope(boost::function<Class*()> factoryFunction, boost::function<void(Class*)> cleanupFunction) {
	registerCtx<Ctx, Class>(factoryFunction, cleanupFunction);
	return GenericApplicationContextScope<Ctx>();
}

template<typename Ctx, typename Class>
const GenericApplicationContextScope<Ctx> registerCtxWithContextScope(boost::function<Class*()> factoryFunction) {
	registerCtx<Ctx, Class>(factoryFunction, defaultCleanupStrategy<Class>);
	return GenericApplicationContextScope<Ctx>();
}

#define GAC_SCOPE(CLASS) \
	const GenericApplicationContextScope<GenericApplicationContext<CLASS> > scope##CLASS;
#define GAC_DEFAULT_CTOR_INIT(CLASS) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>())
#define REGISTER_FULL_CTX(CLASS, FACTORY, CLEANUP)	\
	registerCtx<GenericApplicationContext<CLASS>, CLASS>(FACTORY, CLEANUP)
#define REGISTER_FULL_DEFAULT_CTX(CLASS)	\
	registerCtx<GenericApplicationContext<CLASS>, CLASS>()
#define REGISTER_SCOPE_FOR_CTX(CLASS) \
	GenericApplicationContextScope<GenericApplicationContext<CLASS> > scope##CLASS
#define REGISTER_FULL_CTX_WITH_CTX_SCOPE(CLASS, FACTORY, CLEANUP) \
	const GenericApplicationContextScope<GenericApplicationContext<CLASS> > scope##CLASS = \
		registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY, CLEANUP);
#define REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(CLASS) \
	const GenericApplicationContextScope<GenericApplicationContext<CLASS> > scope##CLASS = \
		registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>();
#define FETCH_FROM_CTX(CLASS) \
	GenericApplicationContext<CLASS>::get()
}} //namespace app::context

#endif /* GENERICAPPLICATIONCONTEXT_HPP_ */
