/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RawDataExportPoliciesImpl.cpp
 *
 *  Created on: 07.10.2014
 *      Author: gerberma
 */

#include "app/types.hpp"
#include <stdexcept>
#include <boost/shared_ptr.hpp>
#include <QObject>
#include "model/RegressionFitAnalysisRegressorOverview.hpp"
#include "model/RegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/RegressionFitAnalysisPhysicalSummary.hpp"
#include "model/RegressionFitAnalysisACF.hpp"
#include "model/RegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/RegressionFitAnalysisRegressionSummary.hpp"
#include "model/RegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/RegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/RegressionFitAnalysisPhysicalBehavior.hpp"

namespace dambase {
namespace model {

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary> plottable, boost::shared_ptr<RawDataCollector_t> collector) {
	collector->addStatisticsValueColumn(
				plottable->getBehaviorIndicatorName(),
				plottable->getDates(),
				plottable->getBehaviorIndicatorValues()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Prediction"),
				plottable->getPredictionDates(),
				plottable->getPredictions()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Summary (95% Interval Upper)"),
				plottable->getPredictionDates(),
				plottable->getUpperCi()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Summary (95% Interval Lower)"),
				plottable->getPredictionDates(),
				plottable->getLowerCi()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Summary (Out of limit)"),
				plottable->getOutOfLimitDates(),
				plottable->getOutOfLimit()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Summary (Extrapolations)"),
				plottable->getExtrapolationDates(),
				plottable->getExtrapolations()
			);
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualTimeSeries> plottable, boost::shared_ptr<RawDataCollector_t> collector) {
	collector->addStatisticsValueColumn(
				QObject::tr("Residual time series (prediction residuals)"),
				plottable->getPredictionDates(),
				plottable->getPredictionResiduals()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Residual time series (95% Interval Upper)"),
				plottable->getPredictionDates(),
				plottable->getUpperCi()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Residual time series (95% Interval Lower)"),
				plottable->getPredictionDates(),
				plottable->getLowerCi()
			);

	collector->addStatisticsValueColumn(
				QObject::tr("Residual time series (Out of limit)"),
				plottable->getOutOfLimitDates(),
				plottable->getOutOfLimit()
			);
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisACF> plottable, boost::shared_ptr<RawDataCollector_t> collector) {
	boost::shared_ptr<RawDataCollector_t::Collection_t> acfLagsCollection = collector->createNewCollection("ACF values", "Lags");
	acfLagsCollection->addValues(
			QObject::tr("ACF"),
			plottable->getLags(),
			plottable->getAcfs()
	);

	boost::shared_ptr<RawDataCollector_t::Collection_t> acfCiCollection = collector->createNewCollection("ACF 95% Interval", "");
	acfCiCollection->addValue(
			QObject::tr("95% ACF Lower"),
			1,
			plottable->getLowerCi()
	);
	acfCiCollection->addValue(
			QObject::tr("95% ACF Upper"),
			1,
			plottable->getUpperCi()
	);
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview> at, boost::shared_ptr<RawDataCollector_t> collector) {
	boost::shared_ptr<RawDataCollector_t::Collection_t> collection = collector->createNewCollection("Regressor statistics", "Regressor");

	std::vector<QString> regressorNames;
	std::vector<double> tvalues;
	std::vector<double> coefficients;
	std::vector<double> vifs;
	std::vector<double> pvalues;
	std::vector<double> stdCoefficients;
	std::vector<double> stdErrors;

	//intercept item
	regressorNames.push_back(at->getInterceptItem()->getName());
	tvalues.push_back(at->getInterceptItem()->getTStat());
	coefficients.push_back(at->getInterceptItem()->getCoefficient());
	vifs.push_back(at->getInterceptItem()->getVIF());
	pvalues.push_back(at->getInterceptItem()->getPValue());
	stdCoefficients.push_back(at->getInterceptItem()->getStdCoefficient());
	stdErrors.push_back(at->getInterceptItem()->getStdError());

	typedef typename RegressionFitAnalysisRegressorOverview_t::RegressorItem_t RegressorItem_t;

	BOOST_FOREACH(boost::shared_ptr<RegressorItem_t> regressorItem, at->getRegressorItems()) {
		regressorNames.push_back(regressorItem->getName());
		tvalues.push_back(regressorItem->getTStat());
		coefficients.push_back(regressorItem->getCoefficient());
		vifs.push_back(regressorItem->getVIF());
		pvalues.push_back(regressorItem->getPValue());
		stdCoefficients.push_back(regressorItem->getStdCoefficient());
		stdErrors.push_back(regressorItem->getStdError());
	}


	if(at->isScientific()) {
		collection->addValues(
				QObject::tr("t-stat"),
				regressorNames,
				tvalues
		);
		collection->addValues(
					QObject::tr("p-value"),
					regressorNames,
					pvalues
		);
		collection->addValues(
				QObject::tr("VIF"),
				regressorNames,
				vifs
		);
	}

	collection->addValues(
			QObject::tr("Coefficient"),
			regressorNames,
			coefficients
	);

	if(at->isScientific()) {
		collection->addValues(
				QObject::tr("StdErrors"),
				regressorNames,
				stdErrors
		);
		collection->addValues(
				QObject::tr("StdCoefficient"),
				regressorNames,
				stdCoefficients
		);
	}
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary> at, boost::shared_ptr<RawDataCollector_t> collector) {
	boost::shared_ptr<RawDataCollector_t::Collection_t> collection = collector->createNewCollection("Regression summary", "Run");

	if(at->isScientific()) {
		collection->addValue(
				QObject::tr("SSRes"),
				1,
				at->getSSRes()
		);
	}
	collection->addValue(
			QObject::tr("MSRes"),
			1,
			at->getMSRes()
	);
	if(at->isScientific()) {
		collection->addValue(
				QObject::tr("MS'Res"),
				1,
				at->getMSResDerived()
		);

		collection->addValue(
				QObject::tr("R2"),
				1,
				at->getR2()
		);
	}

	collection->addValue(
			QObject::tr("R2adj"),
			1,
			at->getR2Adj()
	);

	if(at->isScientific()) {
		collection->addValue(
				QObject::tr("F-stat"),
				1,
				at->getFStat()
		);
		collection->addValue(
				QObject::tr("signif F"),
				1,
				at->getFStatPValue()
		);
		collection->addValue(
				QObject::tr("DW-stat"),
				1,
				at->getDwStat()
		);
	}

	collection->addValue(
			QObject::tr("Observ"),
			1,
			at->getNobs()
	);

	if(at->isScientific()) {
		collection->addValue(
				QObject::tr("SSResPrediction"),
				1,
				at->getSSResPred()
		);
	}
	collection->addValue(
			QObject::tr("MS'ResPrediction"),
			1,
			at->getMSResDerivedPred()
	);
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualNormalProbability> at, boost::shared_ptr<RawDataCollector_t> collector) {
	boost::shared_ptr<RawDataCollector_t::Collection_t> distributionCollection = collector->createNewCollection("Residual normal probability (Quantiles)", "Probability (Standard Deviation)");

	distributionCollection->addValues(
			QObject::tr("Standardized residuals"),
			at->getTheoreticalQuantiles(),
			at->getSampleQuantiles()
	);


	boost::shared_ptr<RawDataCollector_t::Collection_t> normalProbabilityCollection = collector->createNewCollection("Residual normal probability (normal probability)", "Point");
	normalProbabilityCollection->addValue(
			QObject::tr("x"),
			1,
			at->getNormalLinePointOne().getX()
	);
	normalProbabilityCollection->addValue(
			QObject::tr("y"),
			1,
			at->getNormalLinePointOne().getY()
	);

	normalProbabilityCollection->addValue(
			QObject::tr("x"),
			2,
			at->getNormalLinePointTwo().getX()
	);
	normalProbabilityCollection->addValue(
			QObject::tr("y"),
			2,
			at->getNormalLinePointTwo().getY()
	);
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualVersusRegressionParameter> at, boost::shared_ptr<RawDataCollector_t> collector) {
	boost::shared_ptr<RawDataCollector_t::Collection_t> collection = collector->createNewCollection("Tukey-Anscombe plot (residual vs. fitted)", "fit");

	collection->addValues(
			QObject::tr("Residuals (Studentized)"),
			at->getParameterValues(),
			at->getResiduals()
	);
}

template<>
void ExternalRawDataCollectionPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior> at, boost::shared_ptr<RawDataCollector_t> collector) {
	BOOST_FOREACH(boost::shared_ptr<typename RegressionFitAnalysisPhysicalBehavior_t::Item_t> item, at->getItems()) {
		boost::shared_ptr<RawDataCollector_t::Collection_t> collection = collector->createNewCollection("Physical behavior " + item->getName(), "partial displacement");

		collection->addValues(
				QObject::tr("values"),
				item->getApplyedValues(),
				item->getNumericResponse()
		);
	}
}



/*
 * Note: to enable the export of this AT, choose ExternalRawDataExportPolicy on the GenericAnalysisTool definition on this AT
template<>
void ExternalRawDataCollectionPolicy::build(RegressionFitAnalysisRegressorTimeseries_t *at, RawDataCollector_t *collector) {
	throw std::logic_error("RegressionFitAnalysisRegressorTimeseries_t not implemented yet");
}
*/
}  // namespace model
}  // namespace dambase
