/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportPoliciesImpl.cpp
 *
 *  Created on: Jul 22, 2014
 *      Author: gerberma
 */

#include "app/types.hpp"
#include <QDebug>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include <map>
#include "model/RegressionFitAnalysisRegressorOverview.hpp"
#include "model/RegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/RegressionFitAnalysisPhysicalSummary.hpp"
#include "model/RegressionFitAnalysisACF.hpp"
#include "model/RegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/RegressionFitAnalysisRegressionSummary.hpp"
#include "model/RegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/RegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/RegressionFitAnalysisPhysicalBehavior.hpp"


namespace dambase {
namespace model {

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview> overview, ReportContext_t &reportContext) {
	if(overview->isScientific()) {
		reportContext << "Regressor|t-stat|p-value|VIF|Coefficient|StdError|StdCoefficient" << endl;
		reportContext << "---------|------|-------|---|-----------|--------|--------------" << endl; //use a ':' to mark alignment (e.g. ---: for right alignment)
	}else{
		reportContext << "Regressor|Coefficient" << endl;
		reportContext << "---------|-----------" << endl; //use a ':' to mark alignment (e.g. ---: for right alignment)
	}

	if(overview->isScientific()) {
		reportContext << reportContext.escape(overview->getInterceptItem()->getName())
				<< "|" << overview->getInterceptItem()->getTStat()
				<< "|" << overview->getInterceptItem()->getPValue()
				<< "|" << overview->getInterceptItem()->getVIF()
				<< "|" << overview->getInterceptItem()->getCoefficient()
				<< "|" << overview->getInterceptItem()->getStdError()
				<< "|" << overview->getInterceptItem()->getStdCoefficient()
				<< endl;
	}else{
		reportContext << overview->getInterceptItem()->getName()
				<< "|" << overview->getInterceptItem()->getCoefficient()
				<< endl;
	}

	BOOST_FOREACH(boost::shared_ptr<RegressionFitAnalysisRegressorOverview_t::RegressorItem_t> regressorItem, overview->getRegressorItems()) {
		if(regressorItem->isActive()){
			if(overview->isScientific()) {
				reportContext << reportContext.escape(regressorItem->getName())
						<< "|" << regressorItem->getTStat()
						<< "|" << regressorItem->getPValue()
						<< "|" << regressorItem->getVIF()
						<< "|" << regressorItem->getCoefficient()
						<< "|" << regressorItem->getStdError()
						<< "|" << regressorItem->getStdCoefficient()
						<< endl;
			}else{
				reportContext << regressorItem->getName()
						<< "|" << regressorItem->getCoefficient()
						<< endl;
			}
		}
	}

	reportContext << endl << "Table: " << overview->getName() << endl;

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality> quality, ReportContext_t &reportContext) {
	std::map<int, QString> autocorrMapping; //TODO: move this to the analysis model
	std::map<double, QString> fitMapping; //TODO: move this to the analysis model

	//TODO: move this to the analysis model
	autocorrMapping[-1] = QObject::tr("Negative autocorrelation");
	autocorrMapping[0] = QObject::tr("No autocorrelation");
	autocorrMapping[1] = QObject::tr("Positive autocorrelation");

	//TODO: move this to the analysis model
	fitMapping[0] = QObject::tr("Perfect fit");
	fitMapping[1] = QObject::tr("Good fit");
	fitMapping[2] = QObject::tr("Bad fit");
	fitMapping[3] = QObject::tr("Very bad fit");


	reportContext << "Quality measurement|Result|Value" << endl;
	reportContext << "-------------------|------|-----" << endl;
	reportContext << "Autocorrelation (Durbin-Watson)"
			<< "|" << autocorrMapping[quality->getAutocorrelationDegree()]
			<< "|" << quality->getAutocorrelationValue()
            << endl;
	reportContext << "Overall fit (R2adj)"
			<< "|" << fitMapping[quality->getFitDegree()]
			<< "|" << quality->getFitValue()
			<< endl;

	reportContext << endl << "Table: " << quality->getName() << endl;

	reportContext << endl;
}


template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary> summary, ReportContext_t &reportContext) {
	QString plotFileName = reportContext.getRandomFileNameFor("png");
	QString plotLocation = reportContext.getImageLocationFor(plotFileName); //note, we are not within the working directory of the report yet!!!
	FETCH_FROM_CTX(RegressionFitAnalysisPhysicalSummaryUiManager_t)->getTargetAnalysisToolUiFor(summary)->getPlot()->savePng(
				plotLocation,
				reportContext.getPlotWidth(),
				reportContext.getPlotHeight(),
				reportContext.getPlotScaleFactor(),
				reportContext.getPlotPngQuality()
			);

	reportContext << "![" << summary->getName() << "](" << plotFileName << ")" << endl; //note, when we process the markdown, we are within the working directory of the report!!!

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisACF> acf, ReportContext_t &reportContext) {
	QString plotFileName = reportContext.getRandomFileNameFor("png");
	QString plotLocation = reportContext.getImageLocationFor(plotFileName); //note, we are not within the working directory of the report yet!!!
	FETCH_FROM_CTX(RegressionFitAnalysisACFUiManager_t)->getTargetAnalysisToolUiFor(acf)->getPlot()->savePng(
				plotLocation,
				reportContext.getPlotWidth(),
				reportContext.getPlotHeight(),
				reportContext.getPlotScaleFactor(),
				reportContext.getPlotPngQuality()
			);

	reportContext << "![" << acf->getName() << "](" << plotFileName << ")" << endl; //note, when we process the markdown, we are within the working directory of the report!!!

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualNormalProbability> normalProbability, ReportContext_t &reportContext) {
	QString plotFileName = reportContext.getRandomFileNameFor("png");
	QString plotLocation = reportContext.getImageLocationFor(plotFileName); //note, we are not within the working directory of the report yet!!!
	FETCH_FROM_CTX(RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t)->getTargetAnalysisToolUiFor(normalProbability)->getPlot()->savePng(
				plotLocation,
				reportContext.getPlotWidth(),
				reportContext.getPlotHeight(),
				reportContext.getPlotScaleFactor(),
				reportContext.getPlotPngQuality()
			);

	reportContext << "![" << normalProbability->getName() << "](" << plotFileName << ")" << endl; //note, when we process the markdown, we are within the working directory of the report!!!

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary> summary, ReportContext_t &reportContext) {
	/*
	reportContext << "SS Res|MS Res|MS2 Res|R2|R2Adj|F-stat|signif F|DW-stat|Observ" << endl;
	reportContext << "------|------|-------|--|-----|------|--------|-------|------" << endl;
	reportContext << summary->getSSRes()
					<< "|" << summary->getMSRes()
					<< "MS Res derived" << "|" << summary->getMSResDerived()
					<< "R2" << "|" << summary->getR2()
					<< "R2Adj" << "|" << summary->getR2Adj()
					<< "F-stat" << "|" << summary->getFStat()
					<< "signif F" << "|" << summary->getFStatPValue()
					<< "DW-stat" << "|" << summary->getDwStat()
					<< "Observ" << "|" << summary->getNobs()
					<< endl;

	*/

	reportContext << "Type|Value" << endl;
	reportContext << "----|-----" << endl;
	if(summary->isScientific()) {
		reportContext << "SS Res" << "|" << summary->getSSRes() << endl;
	}

	reportContext << "MS Res" << "|" << summary->getMSRes() << endl;
	if(summary->isScientific()) {
		reportContext << "MS\\' Res" << "|" << summary->getMSResDerived() << endl;
		reportContext << "R2" << "|" << summary->getR2() << endl;
	}

	reportContext << "R2Adj" << "|" << summary->getR2Adj() << endl;

	if(summary->isScientific()) {
		reportContext << "F-stat" << "|" << summary->getFStat() << endl;
		reportContext << "signif F" << "|" << summary->getFStatPValue() << endl;
		reportContext << "DW-stat" << "|" << summary->getDwStat() << endl;
	}

	reportContext << "Observ" << "|" << summary->getNobs() << endl;

	if(summary->isScientific()) {
		reportContext << "SS Res (prediction)" << "|" << summary->getSSResPred() << endl;
	}

	reportContext << "MS\\' Res (prediction)" << "|" << summary->getMSResDerivedPred() << endl;

	reportContext << endl << "Table: " << summary->getName() << endl;

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualTimeSeries> timeSeries, ReportContext_t &reportContext) {
	QString plotFileName = reportContext.getRandomFileNameFor("png");
	QString plotLocation = reportContext.getImageLocationFor(plotFileName); //note, we are not within the working directory of the report yet!!!
	FETCH_FROM_CTX(RegressionFitAnalysisResidualTimeSeriesUiManager_t)->getTargetAnalysisToolUiFor(timeSeries)->getPlot()->savePng(
				plotLocation,
				reportContext.getPlotWidth(),
				reportContext.getPlotHeight(),
				reportContext.getPlotScaleFactor(),
				reportContext.getPlotPngQuality()
			);

	reportContext << "![" << timeSeries->getName() << "](" << plotFileName << ")" << endl; //note, when we process the markdown, we are within the working directory of the report!!!

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualVersusRegressionParameter> versus, ReportContext_t &reportContext) {
	QString plotFileName = reportContext.getRandomFileNameFor("png");
	QString plotLocation = reportContext.getImageLocationFor(plotFileName); //note, we are not within the working directory of the report yet!!!
	FETCH_FROM_CTX(RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t)->getTargetAnalysisToolUiFor(versus)->getPlot()->savePng(
				plotLocation,
				reportContext.getPlotWidth(),
				reportContext.getPlotHeight(),
				reportContext.getPlotScaleFactor(),
				reportContext.getPlotPngQuality()
			);

	reportContext << "![" << versus->getName() << "](" << plotFileName << ")" << endl; //note, when we process the markdown, we are within the working directory of the report!!!

	reportContext << endl;
}

template<>
void ExternalReportPolicy::build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior> at, ReportContext_t &reportContext) {
	QString plotFileName = reportContext.getRandomFileNameFor("png");
	QString plotLocation = reportContext.getImageLocationFor(plotFileName); //note, we are not within the working directory of the report yet!!!
	FETCH_FROM_CTX(RegressionFitAnalysisPhysicalBehaviorUiManager_t)->getTargetAnalysisToolUiFor(at)->getPlot()->savePng(
				plotLocation,
				reportContext.getPlotWidth(),
				reportContext.getPlotHeight(),
				reportContext.getPlotScaleFactor(),
				reportContext.getPlotPngQuality()
			);

	reportContext << "![" << at->getName() << "](" << plotFileName << ")" << endl; //note, when we process the markdown, we are within the working directory of the report!!!

	reportContext << endl;
}

/*
 * Note: to enable the report generation of this AT, choose ExternalReportPolicy on the GenericAnalysisTool definition on this AT
template<>
void ExternalReportPolicy::build(RegressionFitAnalysisRegressorTimeseries_t *regressorTimeseries, ReportContext_t &reportContext) {
	throw std::logic_error("RegressionFitAnalysisRegressorTimeseries_t not implemented yet");
}
*/

} // namespace model
} // namespace dambase
