/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * XmlSerializationPoliciesImpl.cpp
 *
 *  Created on: 21.02.2015
 *      Author: gerberma
 */

#include "app/types.hpp"
#include <QString>
#include <QUrl>
#include <QFile>
#include <iostream>
#include <fstream>
#include <cstdio> // remove
#include <boost/config.hpp>
#if defined(BOOST_NO_STDC_NAMESPACE)
namespace std {
	using ::remove;
}
#endif


#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include <boost/serialization/export.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/set.hpp>
#include <boost/serialization/shared_ptr.hpp>
#include <boost/serialization/void_cast.hpp>

#include <boost/shared_ptr.hpp>

#include "GenericApplicationContext.hpp"

#include "model/api/IProject.hpp"
#include "model/Project.hpp"
#include "model/Analysis.hpp"
#include "model/DataModel.hpp"
#include "model/RegressionModel.hpp"
#include "model/WaterLevelScalingMinMaxOpLevel.hpp"


#include "model/r/DataFrame.hpp"
#include "model/r/DataFrameHeader.hpp"
#include "model/r/DataFrameColumn.hpp"
#include "model/r/DateColumn.hpp"
#include "model/r/TemperatureColumn.hpp"
#include "model/r/WaterLevelColumn.hpp"
#include "model/r/BehaviorIndicatorColumn.hpp"
#include "model/r/MinMaxOpLevelRegressorFormulaBuilderPreprocessor.hpp"
#include "model/r/Regressor.hpp"
#include "model/r/ExpressionRegressorFormulaBuilderUserParameterType.hpp"
#include "model/r/ExpressionRegressorFormulaBuilder.hpp"
#include "model/r/RegressionModelDefinition.hpp"
#include "model/r/RegressionModelTemplate.hpp"
#include "model/r/RegressionFormula.hpp"
#include "model/r/TimeRange.hpp"
#include "model/r/DateDataPoint.hpp"

#include "persistence/XmlSerializationPolicies.hpp"

BOOST_CLASS_EXPORT_GUID(dambase::model::Analysis, "Analysis")
BOOST_CLASS_EXPORT_GUID(dambase::model::WaterLevelScalingMinMaxOpLevel, "WaterLevelScalingMinMaxOpLevel")
BOOST_CLASS_EXPORT_GUID(dambase::model::DataModel, "DataModel")
BOOST_CLASS_EXPORT_GUID(dambase::model::RegressionModel, "RegressionModel")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::DataFrame, "DataFrame")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::Regressor, "Regressor");
BOOST_CLASS_EXPORT_GUID(dambase::model::r::ExpressionRegressorFormulaBuilderUserParameterType, "ExpressionRegressorFormulaBuilderUserParameterType")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::ExpressionRegressorFormulaBuilder, "ExpressionRegressorFormulaBuilder")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::RegressionModelDefinition, "RegressionModelDefinition")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::RegressionModelTemplate, "RegressionModelTemplate")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::RegressionFormula, "RegressionFormula")
BOOST_CLASS_EXPORT_GUID(dambase::model::r::TimeRange, "TimeRange")


namespace boost {
namespace serialization {

template<class Archive>
void save(Archive &ar, const dambase::model::Project &project, const unsigned int){
	ar & boost::serialization::make_nvp("name", project.getName());
	ar & boost::serialization::make_nvp("description", project.getDescription());
	ar & boost::serialization::make_nvp("damOperator", project.getDamOperator());

	ar & boost::serialization::make_nvp("analyses", project.getAnalyses());
}

template<class Archive>
void load(Archive &ar, dambase::model::Project &project, const unsigned int){
	QString name;
	QString description;
	QString damOperator;
	std::vector<boost::shared_ptr<dambase::model::api::IAnalysis> > analyses;


	ar & boost::serialization::make_nvp("name", name);
	ar & boost::serialization::make_nvp("description", description);
	ar & boost::serialization::make_nvp("damOperator", damOperator);
	ar & boost::serialization::make_nvp("analyses", analyses);

	project.setName(name);
	project.setDescription(description);
	project.setDamOperator(damOperator);
	project.setAnalyses(analyses);
}


template<class Archive>
void save(Archive &ar, const dambase::model::Analysis &analysis, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("name", analysis.getName());
	ar & make_nvp("description", analysis.getDescription());
	ar & make_nvp("dataModel", analysis.getDataModel());
	ar & make_nvp("regressionModels", analysis.getRegressionModels());
	ar & make_nvp("waterLevelScalingStrategy", analysis.getWaterLevelScalingStrategy());
}

template<class Archive>
void load(Archive &ar, dambase::model::Analysis &analysis, const unsigned int){
	using namespace boost::serialization;
	QString name;
	QString description;

	boost::shared_ptr<dambase::model::api::IWaterLevelScalingMinMaxOpLevel> waterLevelScalingStrategy;
	boost::shared_ptr<dambase::model::api::IDataModel> dataModel;
	std::vector<boost::shared_ptr<dambase::model::api::IRegressionModel> > regressionModels;

	ar & make_nvp("name", name);
	ar & make_nvp("description", description);
	ar & make_nvp("dataModel", dataModel);
	ar & make_nvp("regressionModels", regressionModels);
	ar & make_nvp("waterLevelScalingStrategy", waterLevelScalingStrategy);


	analysis.setName(name);
	analysis.setDescription(description);
	analysis.setDataModel(dataModel);
	analysis.setRegressionModels(regressionModels);
	analysis.setWaterLevelScalingStrategy(waterLevelScalingStrategy);
}



/**
 * WaterLevelScalingMinMaxOpLevel_t stuff
 */
template<class Archive>
void save(Archive &ar, const dambase::model::WaterLevelScalingMinMaxOpLevel &waterLevelScalingMinMaxOpLevel, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("fullSupplyLevel", waterLevelScalingMinMaxOpLevel.getFullSupplyLevel());
	ar & make_nvp("minimumOperatingLevel", waterLevelScalingMinMaxOpLevel.getMinimumOperatingLevel());
	ar & make_nvp("isEnabled", waterLevelScalingMinMaxOpLevel.isEnabled());
}

template<class Archive>
void load(Archive &ar,  dambase::model::WaterLevelScalingMinMaxOpLevel &waterLevelScalingMinMaxOpLevel, const unsigned int){
	using namespace boost::serialization;

	double minimumOperatingLevel;
	double fullSupplyLevel;
	bool isenabled;

	ar & make_nvp("fullSupplyLevel", fullSupplyLevel);
	ar & make_nvp("minimumOperatingLevel", minimumOperatingLevel);
	ar & make_nvp("isEnabled", isenabled);

	//For pre-release 0.9b to prevent issues with previously saved project files
	if(fullSupplyLevel == 0 and minimumOperatingLevel == 0) {
		isenabled = false;
	}

	waterLevelScalingMinMaxOpLevel.setFullSupplyLevel(fullSupplyLevel);
	waterLevelScalingMinMaxOpLevel.setMinimumOperatingLevel(minimumOperatingLevel);
	waterLevelScalingMinMaxOpLevel.setIsEnabled(isenabled);
}


template<class Archive>
void save(Archive &ar, const dambase::model::DataModel &dataModel, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp("csvDelimiter", dataModel.getCSVDelimiter());
	ar & make_nvp("sourceUri", dataModel.getSourceUri());
	ar & make_nvp("dateFormat", dataModel.getDateFormat());
	ar & make_nvp("dataFrame", dataModel.getDataFrame());
	ar & make_nvp("availableCSVDelimiters", dataModel.getAvailableCSVDelimiters());
}

template<class Archive>
void load(Archive &ar, dambase::model::DataModel &dataModel, const unsigned int) {
	using namespace boost::serialization;
	using namespace app::context;

	QString csvDelimiter;
	QUrl sourceUri;
	QString dateFormat;

	std::map<QString, QString> availableCSVDelimiters;
	boost::shared_ptr<dambase::model::r::api::IDataFrame> dataFrame;


	ar & make_nvp("csvDelimiter", csvDelimiter);
	ar & make_nvp("sourceUri", sourceUri);
	ar & make_nvp("dateFormat", dateFormat);
	ar & make_nvp("dataFrame", dataFrame);
	ar & make_nvp("availableCSVDelimiters", availableCSVDelimiters);

	dataModel.setDataFrameProvider(FETCH_FROM_CTX(Data_Frame_Provider_t));
	dataModel.setCSVDelimiter(csvDelimiter);
	dataModel.setAvailableCSVDelimiters(availableCSVDelimiters);
	dataModel.setSourceUri(sourceUri);
	dataModel.setDateFormat(dateFormat);
	dataModel.setDataFrame(dataFrame);
}

/**
 * DataFrame
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::DataFrame &dataFrame, const unsigned int) {
	using namespace boost::serialization;
	using namespace app::context;

	// make sure that the data frame can get reloaded with the content we serialize here
	if(dataFrame.getDateColumn() == NULL) {
		throw std::logic_error("Invalid serialization state of the data frame. In order to guarantee that this project can get loaded later, you have to assign a valid date column.");
	}

	DirectoryManager_t *directoryManager = FETCH_FROM_CTX(DirectoryManager_t);
	Data_Frame_Provider_t *dataFrameProvider = FETCH_FROM_CTX(Data_Frame_Provider_t);
	QString saveFileLocation = directoryManager->getTemporaryDirectory() + QString("dataFrame.txt");
	qDebug() << "saveFileLocation: " << saveFileLocation;
	if(dataFrame.getDateColumn() == NULL) {
		throw std::logic_error("Cannot save data frame. Assign a date column first.");
	}
	dataFrameProvider->saveDataFrame(dataFrame.getDataFrameConst(), saveFileLocation, dataFrame.getDateColumn()); //we provide the dateColumn here because we can validate the class of the date column to be of "Date"
	QFile toReadFile(saveFileLocation);
	if (!toReadFile.open(QFile::ReadOnly | QFile::Text)) {
		throw std::logic_error("Cannot read temporary file: " + saveFileLocation.toStdString());
	}
	QTextStream in(&toReadFile);

	QString dataFrameAsString(in.readAll());
	toReadFile.close();


	ar & make_nvp("rawData", dataFrameAsString);
	ar & make_nvp("behaviorIndicatorColumns", dataFrame.getBehaviorIndicatorColumns());
	ar & make_nvp("waterLevelColumns", dataFrame.getWaterLevelColumns());
	ar & make_nvp("temperatureColumns", dataFrame.getTemperatureColumns());
	ar & make_nvp("dateColumn", dataFrame.getDateColumn());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::DataFrame &dataFrame, const unsigned int) {
	using namespace boost::serialization;

	QString rawData;
	typename DataModel_t::Behavior_Indicator_Column_Container_t behaviorIndicatorColumns;
	typename DataModel_t::Water_Level_Column_Container_t waterLevelColumns;
	typename DataModel_t::Temperature_Column_Container_t temperatureColumns;
	boost::shared_ptr<DataModel_t::DataFrame_t::DateColumn_t> dateColumn;

	ar & make_nvp("rawData", rawData);
	ar & make_nvp("behaviorIndicatorColumns", behaviorIndicatorColumns);
	ar & make_nvp("waterLevelColumns", waterLevelColumns);
	ar & make_nvp("temperatureColumns", temperatureColumns);
	ar & make_nvp("dateColumn", dateColumn);

	DirectoryManager_t *directoryManager = FETCH_FROM_CTX(DirectoryManager_t);
	Data_Frame_Provider_t *dataFrameProvider = FETCH_FROM_CTX(Data_Frame_Provider_t);
	QString tempDataFrameFileLocation = directoryManager->getTemporaryDirectory() + QString("dataFrame.txt");

	QFile tempDataFrameFile(tempDataFrameFileLocation);
	if (!tempDataFrameFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
		throw std::logic_error("Cannot write temporary file: " + tempDataFrameFileLocation.toStdString());
	}
	QTextStream out(&tempDataFrameFile);
	out << rawData;
	tempDataFrameFile.close();


	dataFrame.setDateColumn(dateColumn);
	dataFrame.setDataFrameFromDeserialization(dataFrameProvider->readDataFrame(tempDataFrameFileLocation, dateColumn));
	dataFrame.setBehaviorIndicatorColumns(behaviorIndicatorColumns);
	dataFrame.setWaterLevelColumns(waterLevelColumns);
	dataFrame.setTemperatureColumns(temperatureColumns);
}


/**
 * Columns
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::DataFrameColumn &dataFrameColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp("dataFrameHeader", dataFrameColumn.getHeader());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::DataFrameColumn &dataFrameColumn, const unsigned int) {
	using namespace boost::serialization;

	dambase::model::r::DataFrameColumn::Header_t header;

	ar & make_nvp("dataFrameHeader", header);
	dataFrameColumn.setHeader(header);
}

template<class Archive>
void save(Archive &ar, const dambase::model::r::BehaviorIndicatorColumn &behaviorIndicatorColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(behaviorIndicatorColumn), base_object<dambase::model::r::BehaviorIndicatorColumn::Base_t>(behaviorIndicatorColumn));
}

template<class Archive>
void load(Archive &ar, dambase::model::r::BehaviorIndicatorColumn &behaviorIndicatorColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(behaviorIndicatorColumn), base_object<dambase::model::r::BehaviorIndicatorColumn::Base_t>(behaviorIndicatorColumn));
}


template<class Archive>
void save(Archive &ar, const dambase::model::r::WaterLevelColumn &waterLevelColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(waterLevelColumn), base_object<dambase::model::r::WaterLevelColumn::Base_t>(waterLevelColumn));
}

template<class Archive>
void load(Archive &ar, dambase::model::r::WaterLevelColumn &waterLevelColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(waterLevelColumn), base_object<dambase::model::r::WaterLevelColumn::Base_t>(waterLevelColumn));
}


template<class Archive>
void save(Archive &ar, const dambase::model::r::TemperatureColumn &temperatureColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(temperatureColumn), base_object<dambase::model::r::TemperatureColumn::Base_t>(temperatureColumn));
}

template<class Archive>
void load(Archive &ar, dambase::model::r::TemperatureColumn &temperatureColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(temperatureColumn), base_object<dambase::model::r::TemperatureColumn::Base_t>(temperatureColumn));
}



template<class Archive>
void save(Archive &ar, const dambase::model::r::DateColumn &dateColumn, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp( BOOST_PP_STRINGIZE(dateColumn), base_object<dambase::model::r::DateColumn::Base_t>(dateColumn));
	ar & make_nvp("values", dateColumn.getValues());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::DateColumn &dateColumn, const unsigned int) {
	using namespace boost::serialization;

	typename DataModel_t::DataFrame_t::Header_t header;
	std::vector<typename DataModel_t::DateColumn_t::Date_Data_Point_t> values;

	ar & make_nvp( BOOST_PP_STRINGIZE(dateColumn), base_object<dambase::model::r::DateColumn::Base_t>(dateColumn));
	ar & make_nvp("values", values);
	dateColumn.setValues(values);
}


/**
 * DataFrameHeader
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::DataFrameHeader &header, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp("name", header.getName());
	ar & make_nvp("index", header.getIndex());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::DataFrameHeader &header, const unsigned int) {
	using namespace boost::serialization;
	QString name;
	int index;

	ar & make_nvp("name", name);
	ar & make_nvp("index", index);

	header.setName(name);
	header.setIndex(index);
}

template<class Archive>
void save(Archive &ar, const dambase::model::r::DateDataPoint &dateDataPoint, const unsigned int) {
	using namespace boost::serialization;

	ar & make_nvp("date", dateDataPoint.getDate());
	ar & make_nvp("index", dateDataPoint.getIndex());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::DateDataPoint &dateDataPoint, const unsigned int) {
	using namespace boost::serialization;

	QString date;
	int index;

	ar & make_nvp("date", date);
	ar & make_nvp("index", index);

	dateDataPoint.setDate(date);
	dateDataPoint.setIndex(index);
}

/**
 * RegressionModel related
 */
template<class Archive>
void save(Archive &ar, const dambase::model::RegressionModel &regressionModel, const unsigned int){
	using namespace boost::serialization;

	//RegressionModel_t::Algorithm algorithm = regressionModel.getAlgorithm();
	int algorithm = regressionModel.getAlgorithmAsInt();

	ar & make_nvp("name", regressionModel.getName());
	ar & make_nvp("description", regressionModel.getDescription());
	ar & make_nvp("timeRange", regressionModel.getTimeRange());
	ar & make_nvp("dataModel", regressionModel.getDataModel());
	ar & make_nvp("regressionFormula", regressionModel.getRegressionFormula());
	ar & make_nvp("currentAlgorithm", algorithm);
	ar & make_nvp("praisWinstenIterations", regressionModel.getPraisWinstenIterations());
	ar & make_nvp("ridgeKValue", regressionModel.getRidgeKValue());
	ar & make_nvp("pcrrValue", regressionModel.getPCRrValue());
}

template<class Archive>
void load(Archive &ar, dambase::model::RegressionModel &regressionModel, const unsigned int){
	using namespace boost::serialization;

	QString name;
	QString description;
	boost::shared_ptr<TimeRange_t> timeRange;
	boost::shared_ptr<DataModel_t> dataModel;
	boost::shared_ptr<Regression_Formula_t> regressionFormula;
	int algorithm;
	unsigned praisWinstenIterations;
	double ridgeKValue;
	unsigned pcrrValue;


	ar & make_nvp("name", name);
	ar & make_nvp("description", description);
	ar & make_nvp("timeRange", timeRange);
	ar & make_nvp("dataModel", dataModel);
	ar & make_nvp("regressionFormula", regressionFormula);
	ar & make_nvp("currentAlgorithm", algorithm);
	ar & make_nvp("praisWinstenIterations", praisWinstenIterations);
	ar & make_nvp("ridgeKValue", ridgeKValue);
	ar & make_nvp("pcrrValue", pcrrValue);

	regressionModel.setRegressionFitProvider(FETCH_FROM_CTX(Regression_Fit_Provider_t));

	regressionModel.setName(name);
	regressionModel.setDescription(description);

	regressionModel.setTimeRange(timeRange);
	regressionModel.setDataModel(dataModel);
	regressionModel.setRegressionFormula(regressionFormula);
	regressionModel.setAlgorithm((RegressionModel_t::Algorithm)algorithm);
	regressionModel.setPraisWinstenIterations(praisWinstenIterations);
	regressionModel.setRidgeKValue(ridgeKValue);
	regressionModel.setPCRrValue(pcrrValue);

	regressionModel.restoreAvailableRegressionTimeWindow();
}

/**
 * TimeRange
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::TimeRange &timeRange, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("fromDate", timeRange.getFromDate());
	ar & make_nvp("toDate", timeRange.getToDate());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::TimeRange &timeRange, const unsigned int){
	using namespace boost::serialization;

	dambase::model::r::TimeRange::DateDataPoint_t fromDateDataPoint;
	dambase::model::r::TimeRange::DateDataPoint_t toDateDataPoint;

	ar & make_nvp("fromDate", fromDateDataPoint);
	ar & make_nvp("toDate", toDateDataPoint);

	timeRange.setFromDate(fromDateDataPoint);
	timeRange.setToDate(toDateDataPoint);
}


/**
 * RegressionFormula related
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::RegressionFormula &regressionFormula, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("dataFrame", regressionFormula.getDataFrame());
	ar & make_nvp("behaviorIndicatorColumn", regressionFormula.getBehaviorIndicatorColumn());
	ar & make_nvp("customRegressionModelDefinitions", regressionFormula.getCustomRegressionModelDefinitions());
	ar & make_nvp("templatedRegressionModelDefinitions", regressionFormula.getTemplatedRegressionModelDefinitions());
	ar & make_nvp("regressorFormulaBuilders", regressionFormula.getRegressorFormulaBuilders());
	ar & make_nvp("regressionModelTemplates", regressionFormula.getAllMethodDescriptionsRegardlessOfTheMode());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::RegressionFormula &regressionFormula, const unsigned int){
	using namespace boost::serialization;

	boost::shared_ptr<DataModel_t::DataFrame_t> dataFrame;
	boost::shared_ptr<DataModel_t::Behavior_Indicator_Column_t> behaviorIndicatorColumn;
	std::vector<boost::shared_ptr<RegressionModelDefinition_t> > customRegressionModelDefinitions;
	std::vector<boost::shared_ptr<RegressionModelDefinition_t> > templatedRegressionModelDefinitions;
	std::vector<boost::shared_ptr<Regression_Formula_t::RegressorFormulaBuilder_t> > regressorFormulaBuilders;
	std::vector<boost::shared_ptr<Regression_Formula_t::RegressionModelTemplate_t> > regressionModelTemplates;

	ar & make_nvp("dataFrame", dataFrame);
	ar & make_nvp("behaviorIndicatorColumn", behaviorIndicatorColumn);
	ar & make_nvp("customRegressionModelDefinitions", customRegressionModelDefinitions);
	ar & make_nvp("templatedRegressionModelDefinitions", templatedRegressionModelDefinitions);
	ar & make_nvp("regressorFormulaBuilders", regressorFormulaBuilders);
	ar & make_nvp("regressionModelTemplates", regressionModelTemplates);


	regressionFormula.setDataFrame(dataFrame);
	regressionFormula.setBehaviorIndicatorColumn(behaviorIndicatorColumn);
	regressionFormula.setCustomRegressionModelDefinitions(customRegressionModelDefinitions);
	regressionFormula.setTemplatedRegressionModelDefinitions(templatedRegressionModelDefinitions);
	regressionFormula.setRegressorFormulaBuilders(regressorFormulaBuilders);
	regressionFormula.setMethodDescriptions(regressionModelTemplates);
}


/**
 * RegressionFormulaMethodDescription_t
 */

template<class Archive>
void save(Archive &ar, const dambase::model::r::RegressionModelTemplate &regressionModelTemplate, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("methodId", regressionModelTemplate.getMethodId());
	ar & make_nvp("name", regressionModelTemplate.getName());
	ar & make_nvp("author", regressionModelTemplate.getAuthor());
	ar & make_nvp("description", regressionModelTemplate.getDescription());
	ar & make_nvp("appearForBasic", regressionModelTemplate.getAppearForBasic());
	ar & make_nvp("appearForScientific", regressionModelTemplate.getAppearForScientific());
	ar & make_nvp("regressorFormulaBuilders", regressionModelTemplate.getRegressorFormulaBuilders());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::RegressionModelTemplate &regressionModelTemplate, const unsigned int){
	using namespace boost::serialization;
	using namespace app::context;

	QString methodId;
	QString name;
	QString author;
	QString description;
	bool appearForBasic;
	bool appearForScientific;
	std::vector<boost::shared_ptr<dambase::model::r::RegressionModelTemplate::RegressorFormulaBuilder_t> > regressorFormulaBuilders;

	ar & make_nvp("methodId", methodId);
	ar & make_nvp("name", name);
	ar & make_nvp("author", author);
	ar & make_nvp("description", description);
	ar & make_nvp("appearForBasic", appearForBasic);
	ar & make_nvp("appearForScientific", appearForScientific);
	ar & make_nvp("regressorFormulaBuilders", regressorFormulaBuilders);

	regressionModelTemplate.setMethodId(methodId);
	regressionModelTemplate.setName(name);
	regressionModelTemplate.setAuthor(author);
	regressionModelTemplate.setDescription(description);
	regressionModelTemplate.setAppearForBasic(appearForBasic);
	regressionModelTemplate.setAppearForScientific(appearForScientific);
	regressionModelTemplate.setRegressorFormulaBuilders(regressorFormulaBuilders);
	regressionModelTemplate.setModeHandler(FETCH_FROM_CTX(Model_t)->getModeSettings()->getModeHandler());
}


/**
 * RegressionModelDefinition
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::RegressionModelDefinition &regressionModelDefinition, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("name", regressionModelDefinition.getName());
	ar & make_nvp("dataFrame", regressionModelDefinition.getDataFrame());
	ar & make_nvp("isEditable", regressionModelDefinition.isEditable());
	ar & make_nvp("regressorFormulaBuilders", regressionModelDefinition.getRegressorFormulaBuilders());
	ar & make_nvp("regressors", regressionModelDefinition.getRegressors());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::RegressionModelDefinition &regressionModelDefinition, const unsigned int){
	using namespace boost::serialization;

	QString name;
	boost::shared_ptr<typename RegressionModelDefinition_t::DataFrame_t> dataFrame;
	bool isEditable;
	std::vector<boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> > regressorFormulaBuilders;
	std::vector<boost::shared_ptr<Regressor_t> > regressors;

	ar & make_nvp("name", name);
	ar & make_nvp("dataFrame", dataFrame);
	ar & make_nvp("isEditable", isEditable);
	ar & make_nvp("regressorFormulaBuilders", regressorFormulaBuilders);
	ar & make_nvp("regressors", regressors);

	regressionModelDefinition.setName(name);
	regressionModelDefinition.setDataFrame(dataFrame);
	regressionModelDefinition.setIsEditable(isEditable);
	regressionModelDefinition.setRegressorFormulaBuilders(regressorFormulaBuilders);
	regressionModelDefinition.setRegressors(regressors);
}

/**
 * ExpressionRegressorFormulaBuilder
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::ExpressionRegressorFormulaBuilder &regressorFormulaBuilder, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("isEditable", regressorFormulaBuilder.isEditable());
	ar & make_nvp("assignmentCategoryIdentifier", regressorFormulaBuilder.getAssignmentCategory()->getIdentifier());
	ar & make_nvp("userParameters", regressorFormulaBuilder.getUserParameters());
	ar & make_nvp("identifier", regressorFormulaBuilder.getIdentifier());
	ar & make_nvp("formula", regressorFormulaBuilder.getFormula());
	ar & make_nvp("formulaDefinition", regressorFormulaBuilder.getFormulaDefinition());
}

template<class Archive>
void load(Archive &ar, dambase::model::r::ExpressionRegressorFormulaBuilder &regressorFormulaBuilder, const unsigned int){
	using namespace boost::serialization;

	bool isEditable;
	QString assignmentCategoryIdentifier;
	ExpressionRegressorFormulaBuilder_t::UserParameters_t userParameters;
	QString identifier;
	QString formula;
	QString formulaDefinition;

	ar & make_nvp("isEditable", isEditable);
	ar & make_nvp("assignmentCategoryIdentifier", assignmentCategoryIdentifier);
	ar & make_nvp("userParameters", userParameters);
	ar & make_nvp("identifier", identifier);
	ar & make_nvp("formula", formula);
	ar & make_nvp("formulaDefinition", formulaDefinition);

	regressorFormulaBuilder.setIsEditable(isEditable);
	Model_t *model = FETCH_FROM_CTX(Model_t);
	boost::shared_ptr<AssignmentCategory_t> assignmentCategory = model->findAssignmentCategoryByIdentifier(assignmentCategoryIdentifier);
	if(assignmentCategory == NULL) {
		throw std::logic_error(std::string("RegressorFormulaBuilder: AssignmentCategory with id ") + assignmentCategoryIdentifier.toStdString() + std::string(" is deprecated"));
	}
	regressorFormulaBuilder.setAssignmentCategory(assignmentCategory);


	//Let's deal with the devil of the const hell!!!!
	const std::vector<boost::shared_ptr<AssignmentCategory_t> > &constAssignmentCategories = model->getAssignmentCategoriesConst();
	std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories;
	assignmentCategories.insert(assignmentCategories.begin(), constAssignmentCategories.begin(), constAssignmentCategories.end());
	regressorFormulaBuilder.setAssignmentCategories(assignmentCategories);

	regressorFormulaBuilder.setUserParameters(userParameters);

	regressorFormulaBuilder.setIdentifier(identifier);
	regressorFormulaBuilder.setFormula(formula);
	regressorFormulaBuilder.setFormulaDefinition(formulaDefinition);
}

/**
 * UserParameter
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::ExpressionRegressorFormulaBuilderUserParameterType &userParameter, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("value", userParameter.getValue());
	ar & make_nvp("isMinimumLeadTime", userParameter.isMinimumLeadTime());
}

template<class Archive>
void load(Archive &ar,  dambase::model::r::ExpressionRegressorFormulaBuilderUserParameterType &userParameter, const unsigned int){
	using namespace boost::serialization;

	QString value;
	bool isMinimumLeadTime;

	ar & make_nvp("value", value);
	ar & make_nvp("isMinimumLeadTime", isMinimumLeadTime);

	userParameter.setValue(value);
	userParameter.setIsMinimumLeadTime(isMinimumLeadTime);
}

/**
 * Regressor_t
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::api::IRegressor &regressor, const unsigned int){
	using namespace boost::serialization;

	boost::shared_ptr<dambase::model::r::api::IExpressionRegressorFormulaBuilder> regressorFormulaBuilder;
	boost::shared_ptr<dambase::model::r::api::IRegressor::Influence_Factor_t> influenceFactor;
	boost::shared_ptr<dambase::model::r::api::IRegressor::DataFrame_t> dataFrame;

	ar & make_nvp("dataFrame", regressor.getDataFrame());
	ar & make_nvp("regressorFormulaBuilder", regressor.getRegressorFormulaBuilder());
	ar & make_nvp("influenceFactor", regressor.getInfluenceFactor());
	ar & make_nvp("assignmentCategoryIdentifier", regressor.getAssignmentCategory()->getIdentifier());
	ar & make_nvp("isEditable", regressor.isEditable());
	ar & make_nvp("isActive", regressor.isActive());
	ar & make_nvp("index", regressor.getIndex());

}

template<class Archive>
void load(Archive &ar,  Regressor_t &regressor, const unsigned int){
	using namespace boost::serialization;

	boost::shared_ptr<dambase::model::r::api::IExpressionRegressorFormulaBuilder> regressorFormulaBuilder;
	boost::shared_ptr<dambase::model::r::api::IRegressor::Influence_Factor_t> influenceFactor;
	boost::shared_ptr<dambase::model::r::api::IRegressor::DataFrame_t> dataFrame;
	QString assignmentCategoryIdentifier;
	bool isEditable;
	bool isActive;
	int index;

	ar & make_nvp("dataFrame", dataFrame);
	ar & make_nvp("regressorFormulaBuilder", regressorFormulaBuilder);
	ar & make_nvp("influenceFactor", influenceFactor);
	ar & make_nvp("assignmentCategoryIdentifier", assignmentCategoryIdentifier);
	ar & make_nvp("isEditable", isEditable);
	ar & make_nvp("isActive", isActive);
	ar & make_nvp("index", index);

	regressor.setDataFrame(dataFrame);
	regressor.setRegressorFormulaBuilder(regressorFormulaBuilder);
	regressor.restoreInfluenceFactor(influenceFactor);
	regressor.setIsEditable(isEditable);
	regressor.setIsActive(isActive);
	regressor.setIndex(index);
}

/**
 * RegressorFormulaBuilderPreprocessor_t stuff
 */
template<class Archive>
void save(Archive &ar, const dambase::model::r::api::IRegressorFormulaBuilderPreprocessor &preprocessor, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp("isActive", preprocessor.isActive());
}

template<class Archive>
void load(Archive &ar,  dambase::model::r::api::IRegressorFormulaBuilderPreprocessor &preprocessor, const unsigned int){
	using namespace boost::serialization;

	bool isactive;

	ar & make_nvp("isActive", isactive);

	preprocessor.setIsActive(isactive);
}

template<class Archive>
void save(Archive &ar, const dambase::model::r::MinMaxOpLevelRegressorFormulaBuilderPreprocessor &preprocessor, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp(BOOST_PP_STRINGIZE(preprocessor), base_object<RegressorFormulaBuilderPreprocessor_t>(preprocessor));

	boost::shared_ptr<dambase::model::api::IWaterLevelScalingMinMaxOpLevel> minMaxOpLevelStrategy;

	ar & make_nvp("name", preprocessor.getName());
	ar & make_nvp("functionCallName", preprocessor.getFunctionCallName());
	ar & make_nvp("minMaxOpLevelStrategy", preprocessor.getMinMaxOpLevelStrategy());
}

template<class Archive>
void load(Archive &ar,  dambase::model::r::MinMaxOpLevelRegressorFormulaBuilderPreprocessor &preprocessor, const unsigned int){
	using namespace boost::serialization;

	ar & make_nvp(BOOST_PP_STRINGIZE(preprocessor), base_object<dambase::model::r::api::IRegressorFormulaBuilderPreprocessor>(preprocessor));

	QString name;
	QString functionCallName;
	boost::shared_ptr<dambase::model::api::IWaterLevelScalingMinMaxOpLevel> minMaxOpLevelStrategy;

	ar & make_nvp("name", name);
	ar & make_nvp("functionCallName", functionCallName);
	ar & make_nvp("minMaxOpLevelStrategy", minMaxOpLevelStrategy);

	preprocessor.setName(name);
	preprocessor.setFunctionCallName(functionCallName);

	preprocessor.setMinMaxOpLevelStrategy(minMaxOpLevelStrategy);
}


/**
 * OTHER STUFF
 */

template<class Archive>
void save(Archive &ar, const QString & s, const unsigned int){
	std::string stdStr(s.toStdString());
    ar << boost::serialization::make_nvp("value", stdStr);
}

template<class Archive>
void load(Archive &ar, QString & s, const unsigned int){
    std::string stdStr;
    ar >> boost::serialization::make_nvp("value", stdStr);
    s = QString::fromStdString(stdStr);
}

template<class Archive>
void save(Archive &ar, const QUrl & url, const unsigned int){
	std::string stdStr(url.toString().toStdString());
    ar << boost::serialization::make_nvp("value", stdStr);
}

template<class Archive>
void load(Archive &ar, QUrl & url, const unsigned int){
    std::string stdStr;
    ar >> boost::serialization::make_nvp("value", stdStr);
    url = QUrl(QString::fromStdString(stdStr));
}




template<class Archive>
void serialize(Archive &ar, dambase::model::Project &project, const unsigned int version) {
	boost::serialization::split_free(ar, project, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::Analysis &analysis, const unsigned int version) {
	boost::serialization::split_free(ar, analysis, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::WaterLevelScalingMinMaxOpLevel &waterLevelScalingMinMaxOpLevel, const unsigned int version) {
	boost::serialization::split_free(ar, waterLevelScalingMinMaxOpLevel, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::DataModel &dataModel, const unsigned int version) {
	boost::serialization::split_free(ar, dataModel, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::DataFrame &dataFrame, const unsigned int version) {
	boost::serialization::split_free(ar, dataFrame, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::BehaviorIndicatorColumn &behaviorIndicatorColumn, const unsigned int version) {
	boost::serialization::split_free(ar, behaviorIndicatorColumn, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::WaterLevelColumn &waterLevelColumn, const unsigned int version) {
	boost::serialization::split_free(ar, waterLevelColumn, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::TemperatureColumn &temperatureColumn, const unsigned int version) {
	boost::serialization::split_free(ar, temperatureColumn, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::DateColumn &dateColumn, const unsigned int version) {
	boost::serialization::split_free(ar, dateColumn, version);
}


template<class Archive>
void serialize(Archive &ar, dambase::model::r::DateDataPoint &dateDataPoint, const unsigned int version) {
	boost::serialization::split_free(ar, dateDataPoint, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::DataFrameHeader &dataFrameHeader, const unsigned int version) {
	boost::serialization::split_free(ar, dataFrameHeader, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::RegressionModel &regressionModel, const unsigned int version) {
	boost::serialization::split_free(ar, regressionModel, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::TimeRange &timeRange, const unsigned int version) {
	boost::serialization::split_free(ar, timeRange, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::RegressionFormula &regressionFormula, const unsigned int version) {
	boost::serialization::split_free(ar, regressionFormula, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::RegressionModelDefinition &regressionModelDefinition, const unsigned int version) {
	boost::serialization::split_free(ar, regressionModelDefinition, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::ExpressionRegressorFormulaBuilder &regressorFormulaBuilder, const unsigned int version) {
	boost::serialization::split_free(ar, regressorFormulaBuilder, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::ExpressionRegressorFormulaBuilderUserParameterType &userParameter, const unsigned int version) {
	boost::serialization::split_free(ar, userParameter, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::Regressor &regressor, const unsigned int version) {
	boost::serialization::split_free(ar, regressor, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::Regressor::Influence_Factor_t &influenceFactor, const unsigned int version) {
	boost::serialization::split_free(ar, influenceFactor, version);
}

template<class Archive>
void serialize(Archive &ar, dambase::model::r::RegressionModelTemplate &regressionModelTemplate, const unsigned int version) {
	boost::serialization::split_free(ar, regressionModelTemplate, version);
}

template<class Archive>
void serialize(Archive &ar, MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t &minMaxOpLevelRegressorFormulaBuilderPreprocessor, const unsigned int version) {
	boost::serialization::split_free(ar, minMaxOpLevelRegressorFormulaBuilderPreprocessor, version);
}

/**
 * OTHER STUFF
 */
template<class Archive>
void serialize( Archive& ar, QString& s, const unsigned int version )
{
	boost::serialization::split_free(ar, s, version);
}


template<class Archive>
void serialize( Archive& ar, QUrl& url, const unsigned int version )
{
	boost::serialization::split_free(ar, url, version);
}
}} /* namespace boost::serialization */



namespace persistence {

XmlSerializationPolicy::XmlSerializationPolicy(){;}
XmlSerializationPolicy::~XmlSerializationPolicy(){;}

template<>
void XmlSerializationPolicy::serialize(boost::shared_ptr<dambase::model::api::IProject> parentProject, QString filename) {
	std::ofstream ofs(filename.toStdString().c_str());
	boost::archive::xml_oarchive oa(ofs);

	registerDerivedBasePairs();

	boost::shared_ptr<dambase::model::Project> project = boost::static_pointer_cast<dambase::model::Project>(parentProject);
	oa << BOOST_SERIALIZATION_NVP(project);

	std::cout << "saved to " << filename.toStdString() << std::endl;
}


template<>
void XmlSerializationPolicy::deserialize(boost::shared_ptr<dambase::model::api::IProject>& parentProject, QString filename) {
	std::ifstream ifs(filename.toStdString().c_str());
	boost::archive::xml_iarchive ia(ifs);

	registerDerivedBasePairs();

	boost::shared_ptr<dambase::model::Project> project; // note, all the object creation happens within the deserialization process.

	ia >> BOOST_SERIALIZATION_NVP(project);

	parentProject = project;
}


void
XmlSerializationPolicy::registerDerivedBasePairs() {
	using namespace boost::serialization;
	/*
	 * see: http://www.boost.org/doc/libs/1_57_0/libs/serialization/doc/serialization.html and http://www.boost.org/doc/libs/1_47_0/libs/serialization/doc/void_cast.html
	 */
	void_cast_register<dambase::model::Analysis, dambase::model::api::IAnalysis>();
	void_cast_register<dambase::model::WaterLevelScalingMinMaxOpLevel, dambase::model::api::IWaterLevelScalingMinMaxOpLevel>();
	void_cast_register<dambase::model::DataModel, dambase::model::api::IDataModel>();
	void_cast_register<dambase::model::RegressionModel, dambase::model::api::IRegressionModel>();

	void_cast_register<dambase::model::r::DataFrame, dambase::model::r::api::IDataFrame>();
	void_cast_register<dambase::model::r::Regressor, dambase::model::r::api::IRegressor>();
	void_cast_register<dambase::model::r::ExpressionRegressorFormulaBuilderUserParameterType, dambase::model::r::api::IExpressionRegressorFormulaBuilderUserParameterType>();
	void_cast_register<dambase::model::r::ExpressionRegressorFormulaBuilder, dambase::model::r::api::IExpressionRegressorFormulaBuilder>();
	void_cast_register<dambase::model::r::RegressionModelDefinition, dambase::model::r::api::IRegressionModelDefinition>();
	void_cast_register<dambase::model::r::RegressionModelTemplate, dambase::model::r::api::IRegressionModelTemplate>();
	void_cast_register<dambase::model::r::RegressionFormula, dambase::model::r::api::IRegressionFormula>();
	void_cast_register<dambase::model::r::TimeRange, dambase::model::r::api::ITimeRange>();
}

}
