/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolFactoryImpl.cpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#include <boost/shared_ptr.hpp>
#include "app/GenericApplicationContext.hpp"
#include "app/Factory.hpp"
#include "app/types.hpp"
#include <vector>

#include "model/AnalysisToolDescription.hpp"
#include "model/AnalysisToolsDescriptions.hpp"
#include "model/AnalysisToolModeDescription.hpp"
#include "model/AnalysisToolsModeDescription.hpp"
#include "model/DataModelAnalysisToolsDescription.hpp"
#include "model/RegressionModelAnalysisToolsDescription.hpp"
#include "model/DataModelOverviewStatistics.hpp"
#include "model/DataModelAnalysisInfluenceFactors.hpp"
#include "model/DataModelAnalysisTimeSeriesOverview.hpp"
#include "model/RegressionFitAnalysisPhysicalSummary.hpp"
#include "model/RegressionFitAnalysisRegressorOverview.hpp"
#include "model/RegressionFitAnalysisRegressionSummary.hpp"
#include "model/RegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/RegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/RegressionFitAnalysisACF.hpp"
#include "model/RegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/RegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/RegressionFitAnalysisPhysicalBehavior.hpp"
#include "model/RegressionFitAnalysisRegressorTimeseries.hpp"

using namespace app::context;

namespace app {

template<>
DataModel_Analysis_Tools_Description_t *Factory::create<DataModel_Analysis_Tools_Description_t, ModeSettings_t>(boost::shared_ptr<ModeSettings_t> modeSettings) {
	using namespace dambase::model;
	//TODO: define the descriptions from values provided by the corresponding models such as the DataModelAnalysisOverviewStatistics

	std::vector<boost::shared_ptr<Analysis_Tool_Description_t> > overviewToolsDescription;
	std::vector<boost::shared_ptr<Analysis_Tool_Description_t> > distributionToolsDescription;

	boost::shared_ptr<Analysis_Tool_Description_t> summary(new AnalysisToolDescription("Summary", DataModelOverviewStatistics::getStaticToolId()));
	boost::shared_ptr<Analysis_Tool_Description_t> influenceFactors(new AnalysisToolDescription("Influence factors", DataModelAnalysisInfluenceFactors::getStaticToolId()));
	boost::shared_ptr<Analysis_Tool_Description_t> timeSeriesOverview(new AnalysisToolDescription("Time series overview", DataModelAnalysisTimeSeriesOverview::getStaticToolId()));

	overviewToolsDescription.push_back(summary);
	overviewToolsDescription.push_back(influenceFactors);
	overviewToolsDescription.push_back(timeSeriesOverview);

	return new DataModelAnalysisToolsDescription(overviewToolsDescription, distributionToolsDescription, modeSettings->getModeHandler());
}

template<>
RegressionModel_Analysis_Tools_Description_t *Factory::create<RegressionModel_Analysis_Tools_Description_t>(boost::shared_ptr<ModeSettings_t> modeSettings) {
	using namespace dambase::model;

	return new RegressionModelAnalysisToolsDescription(
				modeSettings->getRegressionModelAnalysisToolsModeDescription(),
				modeSettings->getModeHandler()
			);
}

template<>
AnalysisToolModeDescription_t *Factory::create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription) {
	using namespace dambase::model;

	return new AnalysisToolModeDescription(
		analysisToolDescription
	);
}

template<>
AnalysisToolsModeDescription_t *Factory::create<AnalysisToolsModeDescription_t, ModeHandler_t>(boost::shared_ptr<ModeHandler_t> modeHandler) {
	using namespace dambase::model;

	return new AnalysisToolsModeDescription(
			modeHandler
	);
}

template<>
Analysis_Tools_Descriptions_t *Factory::create<Analysis_Tools_Descriptions_t>(boost::shared_ptr<ModeSettings_t> modeSettings) {
	using namespace context;
	using namespace dambase::model;

	boost::shared_ptr<DataModel_Analysis_Tools_Description_t> dataModelDescription(FETCH_FROM_CTX(Model_Factory_t)->create<DataModel_Analysis_Tools_Description_t>(modeSettings));
	boost::shared_ptr<RegressionModel_Analysis_Tools_Description_t> regressionModelDescription(FETCH_FROM_CTX(Model_Factory_t)->create<RegressionModel_Analysis_Tools_Description_t>(modeSettings));
	return new AnalysisToolsDescriptions(dataModelDescription, regressionModelDescription);
}

template<>
Analysis_Tools_Factory_t *Factory::create<Analysis_Tools_Factory_t>() {
	return new Analysis_Tools_Factory_t();
}

template<>
DataModel_Overview_Statistics_t *Factory::create<DataModel_Overview_Statistics_t, DataModel_t>(boost::shared_ptr<DataModel_t> dataModel) {
	using namespace dambase::model;
	return new DataModelOverviewStatistics(
				dataModel,
				FETCH_FROM_CTX(DataFrame_Overview_Statistics_Provider_t)
			);
}

template<>
DataModel_Influence_Factors_t *Factory::create<DataModel_Influence_Factors_t, DataModel_t>(boost::shared_ptr<DataModel_t> dataModel) {
	using namespace dambase::model;
	return new DataModelAnalysisInfluenceFactors(
				dataModel,
				FETCH_FROM_CTX(DataFrame_Influence_Factors_Provider_t)
			);
}

template<>
DataModelAnalysisTimeSeriesOverview_t *Factory::create<DataModelAnalysisTimeSeriesOverview_t, DataModel_t>(boost::shared_ptr<DataModel_t> dataModel) {
	using namespace dambase::model;
	return new DataModelAnalysisTimeSeriesOverview(
				dataModel,
				FETCH_FROM_CTX(DataFrameTimeSeriesOverviewProvider_t)
			);
}


//RegressionModel related
template<>
RegressionFitAnalysisPhysicalSummary_t *Factory::create<RegressionFitAnalysisPhysicalSummary_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;
	RegressionFitAnalysisPhysicalSummary *summary = new RegressionFitAnalysisPhysicalSummary;

	summary->setName(QObject::tr("Regression plot"));
	summary->setRegressionModel(regressionModel);
	summary->setProvider(FETCH_FROM_CTX(R_RegressionFitAnalysisPhysicalSummaryProvider_t));

	return summary;
}

template<>
RegressionFitAnalysisRegressorOverview_t *Factory::create<RegressionFitAnalysisRegressorOverview_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisRegressorOverview *overview = new RegressionFitAnalysisRegressorOverview;
	overview->setName(QObject::tr("Regressor overview")); //name can also be resolved in the context of the app
	overview->setRegressionModel(regressionModel);
	overview->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisRegressorOverviewProvider_t));
	overview->setModeHandler(FETCH_FROM_CTX(Model_t)->getModeSettings()->getModeHandler());

	return overview;
}

template<>
RegressionFitAnalysisRegressionSummary_t *Factory::create<RegressionFitAnalysisRegressionSummary_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisRegressionSummary *summary = new RegressionFitAnalysisRegressionSummary;
	summary->setRegressionModel(regressionModel);
	summary->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisRegressionSummaryProvider_t));
	summary->setName(QObject::tr("Regression summary"));
	summary->setModeHandler(FETCH_FROM_CTX(Model_t)->getModeSettings()->getModeHandler());

	return summary;
}


template<>
RegressionFitAnalysisResidualNormalProbability_t *Factory::create<RegressionFitAnalysisResidualNormalProbability_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisResidualNormalProbability *normalProbability = new RegressionFitAnalysisResidualNormalProbability;
	normalProbability->setRegressionModel(regressionModel);
	normalProbability->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisResidualNormalProbabilityProvider_t));
	normalProbability->setName(QObject::tr("Residual normal probability"));

	return normalProbability;
}


template<>
RegressionFitAnalysisResidualTimeSeries_t *Factory::create<RegressionFitAnalysisResidualTimeSeries_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisResidualTimeSeries *timeSeries = new RegressionFitAnalysisResidualTimeSeries;
	timeSeries->setRegressionModel(regressionModel);
	timeSeries->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisResidualTimeSeriesProvider_t));
	timeSeries->setName("Residual time series");

	return timeSeries;
}

template<>
RegressionFitAnalysisACF_t *Factory::create<RegressionFitAnalysisACF_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisACF *acf = new RegressionFitAnalysisACF;
	acf->setRegressionModel(regressionModel);
	acf->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisACFProvider_t));
	acf->setName(QObject::tr("Residual ACF"));
	return acf;
}


template<>
RegressionFitAnalysisResidualVersusRegressionParameter_t *Factory::create<RegressionFitAnalysisResidualVersusRegressionParameter_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisResidualVersusRegressionParameter *versus = new RegressionFitAnalysisResidualVersusRegressionParameter;
	versus->setRegressionModel(regressionModel);
	versus->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisResidualVersusRegressionParameterProvider_t));
	versus->setName(QObject::tr("Tukey-Anscombe plot (residual vs. fitted)"));

	return versus;
}


template<>
RegressionFitAnalysisOverallRegressionQuality_t *Factory::create<RegressionFitAnalysisOverallRegressionQuality_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisOverallRegressionQuality *quality = new RegressionFitAnalysisOverallRegressionQuality;
	quality->setName(QObject::tr("Regression overall quality")); //name can also be resolved in the context of the app
	quality->setRegressionModel(regressionModel);
	quality->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisOverallRegressionQualityProvider_t));

	return quality;
}

template<>
RegressionFitAnalysisPhysicalBehavior_t *Factory::create<RegressionFitAnalysisPhysicalBehavior_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisPhysicalBehavior *physicalBehavior = new RegressionFitAnalysisPhysicalBehavior;
	Model_t *model = FETCH_FROM_CTX(Model_t);

	physicalBehavior->setName(QObject::tr("Regression physical behavior"));
	physicalBehavior->setRegressionModel(regressionModel);
	physicalBehavior->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisPhysicalBehaviorProvider_t));
	physicalBehavior->setAssignmentCategories(model->getAssignmentCategories());
	physicalBehavior->setAssignmentCategory(model->findAssignmentCategoryByIdentifier("water_level"));

	return physicalBehavior;
}

template<>
RegressionFitAnalysisRegressorTimeseries_t *Factory::create<RegressionFitAnalysisRegressorTimeseries_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	using namespace dambase::model;

	RegressionFitAnalysisRegressorTimeseries *regressorTimeseries = new RegressionFitAnalysisRegressorTimeseries;
	Model_t *model = FETCH_FROM_CTX(Model_t);

	regressorTimeseries->setName(QObject::tr("Regressor time series"));
	regressorTimeseries->setRegressionModel(regressionModel);
	regressorTimeseries->setProvider(FETCH_FROM_CTX(RegressionFitAnalysisRegressorTimeseriesProvider_t));
	regressorTimeseries->setIsPolynom(false);
	regressorTimeseries->setIsScaled(true);

	return regressorTimeseries;
}


} /* namespace app */

