/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ModelFactoryImpl.cpp
 *
 *  Created on: Dec 17, 2013
 *      Author: gerberma
 */

#include <qDebug>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include <vector>
#include "app/GenericApplicationContext.hpp"
#include "app/Factory.hpp"
#include "app/types.hpp"
#include "model/r/RManager.hpp"
#include "model/AssignmentCategory.hpp"
#include "model/ModeHandler.hpp"
#include "model/ModeSettings.hpp"
#include "model/AnalysisToolDescription.hpp"
#include "model/RegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/RegressionFitAnalysisRegressorOverview.hpp"
#include "model/RegressionFitAnalysisPhysicalSummary.hpp"
#include "model/RegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/RegressionFitAnalysisPhysicalBehavior.hpp"
#include "model/RegressionFitAnalysisRegressorTimeseries.hpp"
#include "model/RegressionFitAnalysisRegressionSummary.hpp"
#include "model/RegressionFitAnalysisACF.hpp"
#include "model/RegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/RegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/ActivityTracker.hpp"
#include "model/Model.hpp"
#include "model/Project.hpp"
#include "model/Analysis.hpp"
#include "model/WaterLevelScalingMinMaxOpLevel.hpp"

typedef dambase::model::r::RManager R_Manager_t;

namespace app {

Factory::Factory() {
}

Factory::~Factory() {
}

template<>
ModeHandler_t *Factory::create<ModeHandler_t>() {
	using namespace dambase::model;

	return new ModeHandler(0); //initialize with basic user mode
}

template<>
ModeSettings_t *Factory::create<ModeSettings_t>() {
	using namespace dambase::model;

	boost::shared_ptr<ModeHandler_t> modeHandler(FETCH_FROM_CTX(Model_Factory_t)->create<ModeHandler_t>());

	boost::shared_ptr<AnalysisToolsModeDescription_t> regressionModelAnalysisToolsModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolsModeDescription_t, ModeHandler_t>(modeHandler));
	boost::shared_ptr<AnalysisToolsModeDescription_t> dataModelAnalysisToolsModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolsModeDescription_t, ModeHandler_t>(modeHandler));


	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Tukey-Anscombe Plot (Residual vs. Fitted)", RegressionFitAnalysisResidualVersusRegressionParameter::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> scientificAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		boost::shared_ptr<AnalysisToolModeDescription_t> basicAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));

		basicAnalysisToolModeDescription->setAppearLeft(false);
		basicAnalysisToolModeDescription->setAppearAutomatically(false);
		basicAnalysisToolModeDescription->setForMode(0);
		basicAnalysisToolModeDescription->setForTabNumber(0);

		scientificAnalysisToolModeDescription->setAppearLeft(false);
		scientificAnalysisToolModeDescription->setAppearAutomatically(true);
		scientificAnalysisToolModeDescription->setForMode(1);
		scientificAnalysisToolModeDescription->setForTabNumber(0);


		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					scientificAnalysisToolModeDescription
				);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					basicAnalysisToolModeDescription
				);
	}

	/**
	 * This is the regressor overview part
	 */
	boost::shared_ptr<Analysis_Tool_Description_t> regressorOverviewAnalysisToolDescription(new AnalysisToolDescription("Regressor overview", RegressionFitAnalysisRegressorOverview::getStaticToolId()));
	{
			//Note, the scientific description is further below!!!
		boost::shared_ptr<AnalysisToolModeDescription_t>basicToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(regressorOverviewAnalysisToolDescription));

		basicToolModeDescription->setAppearLeft(true);
		basicToolModeDescription->setAppearAutomatically(true);
		basicToolModeDescription->setForMode(0);
		basicToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					basicToolModeDescription
				);
	}
	/**
	 * Setup RegressionModel AnalysisToolsModeDescriptions
	 */
	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Summary", RegressionFitAnalysisPhysicalSummary::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t>scientificAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		scientificAnalysisToolModeDescription->setAppearLeft(true);
		scientificAnalysisToolModeDescription->setAppearAutomatically(true);
		scientificAnalysisToolModeDescription->setForMode(1);
		scientificAnalysisToolModeDescription->setForTabNumber(0);

		boost::shared_ptr<AnalysisToolModeDescription_t> basicAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		basicAnalysisToolModeDescription->setAppearLeft(true);
		basicAnalysisToolModeDescription->setAppearAutomatically(true);
		basicAnalysisToolModeDescription->setForMode(0);
		basicAnalysisToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					scientificAnalysisToolModeDescription
				);
		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					basicAnalysisToolModeDescription
				);
	}
	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Regression overall quality", RegressionFitAnalysisOverallRegressionQuality::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> scientificAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		scientificAnalysisToolModeDescription->setAppearLeft(true);
		scientificAnalysisToolModeDescription->setAppearAutomatically(true);
		scientificAnalysisToolModeDescription->setForMode(1);
		scientificAnalysisToolModeDescription->setForTabNumber(0);

		boost::shared_ptr<AnalysisToolModeDescription_t> basicAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		basicAnalysisToolModeDescription->setAppearLeft(true);
		basicAnalysisToolModeDescription->setAppearAutomatically(true);
		basicAnalysisToolModeDescription->setForMode(0);
		basicAnalysisToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					scientificAnalysisToolModeDescription
				);
		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					basicAnalysisToolModeDescription
				);
	}

	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Regression physical behavior", RegressionFitAnalysisPhysicalBehavior::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> scientificAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		scientificAnalysisToolModeDescription->setAppearLeft(false);
		scientificAnalysisToolModeDescription->setAppearAutomatically(false);
		scientificAnalysisToolModeDescription->setForMode(1);
		scientificAnalysisToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					scientificAnalysisToolModeDescription
				);


		boost::shared_ptr<AnalysisToolModeDescription_t> basicAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		basicAnalysisToolModeDescription->setAppearLeft(false);
		basicAnalysisToolModeDescription->setAppearAutomatically(true);
		basicAnalysisToolModeDescription->setForMode(0);
		basicAnalysisToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					basicAnalysisToolModeDescription
				);
	}

	{
		boost::shared_ptr<Analysis_Tool_Description_t>regressorTimeseriesAnalysisToolDescription(new AnalysisToolDescription("Regressor timeseries", RegressionFitAnalysisRegressorTimeseries::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t>basicAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(regressorTimeseriesAnalysisToolDescription));
		boost::shared_ptr<AnalysisToolModeDescription_t>scientificAnalysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(regressorTimeseriesAnalysisToolDescription));

		scientificAnalysisToolModeDescription->setAppearLeft(false);
		scientificAnalysisToolModeDescription->setAppearAutomatically(true);
		scientificAnalysisToolModeDescription->setForMode(1);
		scientificAnalysisToolModeDescription->setForTabNumber(0);

		basicAnalysisToolModeDescription->setAppearLeft(false);
		basicAnalysisToolModeDescription->setAppearAutomatically(true);
		basicAnalysisToolModeDescription->setForMode(0);
		basicAnalysisToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					basicAnalysisToolModeDescription
				);
		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
				scientificAnalysisToolModeDescription
				);
	}

	/**
	 * This is for the regressor overview scientific part
	 */
	{
		boost::shared_ptr<AnalysisToolModeDescription_t>scientificToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(regressorOverviewAnalysisToolDescription));

		scientificToolModeDescription->setAppearLeft(false);
		scientificToolModeDescription->setAppearAutomatically(true);
		scientificToolModeDescription->setForMode(1);
		scientificToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					scientificToolModeDescription
				);
	}
	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Regression summary", RegressionFitAnalysisRegressionSummary::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> scientificToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		scientificToolModeDescription->setAppearLeft(false);
		scientificToolModeDescription->setAppearAutomatically(true);
		scientificToolModeDescription->setForMode(1);
		scientificToolModeDescription->setForTabNumber(0);


		boost::shared_ptr<AnalysisToolModeDescription_t>basicToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		basicToolModeDescription->setAppearLeft(false);
		basicToolModeDescription->setAppearAutomatically(true);
		basicToolModeDescription->setForMode(0);
		basicToolModeDescription->setForTabNumber(0);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					scientificToolModeDescription
				);
		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
				basicToolModeDescription
		);
	}
	{
		boost::shared_ptr<Analysis_Tool_Description_t>analysisToolDescription(new AnalysisToolDescription("Residual ACF", RegressionFitAnalysisACF::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> analysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		analysisToolModeDescription->setAppearLeft(false);
		analysisToolModeDescription->setAppearAutomatically(true);
		analysisToolModeDescription->setForMode(1);
		analysisToolModeDescription->setForTabNumber(1);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					analysisToolModeDescription
				);
	}
	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Residual normal probability", RegressionFitAnalysisResidualNormalProbability::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> analysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		analysisToolModeDescription->setAppearLeft(false);
		analysisToolModeDescription->setAppearAutomatically(true);
		analysisToolModeDescription->setForMode(1);
		analysisToolModeDescription->setForTabNumber(1);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					analysisToolModeDescription
				);
	}
	{
		boost::shared_ptr<Analysis_Tool_Description_t> analysisToolDescription(new AnalysisToolDescription("Residual time series", RegressionFitAnalysisResidualTimeSeries::getStaticToolId()));
		boost::shared_ptr<AnalysisToolModeDescription_t> analysisToolModeDescription(FETCH_FROM_CTX(Model_Factory_t)->create<AnalysisToolModeDescription_t, Analysis_Tool_Description_t>(analysisToolDescription));
		analysisToolModeDescription->setAppearLeft(false);
		analysisToolModeDescription->setAppearAutomatically(true);
		analysisToolModeDescription->setForMode(1);
		analysisToolModeDescription->setForTabNumber(1);

		regressionModelAnalysisToolsModeDescription->addAnalysisToolModeDescription(
					analysisToolModeDescription
				);
	}


	return new ModeSettings(
			modeHandler,
			regressionModelAnalysisToolsModeDescription,
			dataModelAnalysisToolsModeDescription
	);
}

template<>
ActivityTracker_t *Factory::create<ActivityTracker_t>() {
	using namespace dambase::model;

	return new ActivityTracker();
}

template<>
Model_t* Factory::create<Model_t>() {
	using namespace context;
	using namespace dambase::model;

	qDebug() << __FUNCTION__ << " Model_t* Factory::create<Model_t>()  : 1 ";
	int assignmentIndex = 0;
	boost::shared_ptr<AssignmentCategory_t> waterLevelAssignmentCategory(new model::AssignmentCategory("water_level", "Water Level", assignmentIndex++, true));
	boost::shared_ptr<AssignmentCategory_t> seasonalAssignmentCategory(new model::AssignmentCategory("seasonal", "Seasonal", assignmentIndex++, false));
	boost::shared_ptr<AssignmentCategory_t> driftAssignmentCategory(new model::AssignmentCategory("drift", "Drift", assignmentIndex++, false));
	boost::shared_ptr<AssignmentCategory_t> creepAssignmentCategory(new model::AssignmentCategory("creep", "Creep", assignmentIndex++, true));
	boost::shared_ptr<AssignmentCategory_t> temperatureAssignmentCategory(new model::AssignmentCategory("temperature", "Temperature", assignmentIndex++, true));
	boost::shared_ptr<AssignmentCategory_t> averageAssignmentCategory(new model::AssignmentCategory("average", "Average", assignmentIndex++, true));
	boost::shared_ptr<AssignmentCategory_t> delayAssignmentCategory(new model::AssignmentCategory("delay", "Delay", assignmentIndex++, true));
	boost::shared_ptr<AssignmentCategory_t> anyAssignmentCategory(new model::AssignmentCategory("any", "Any influence factor", assignmentIndex++, true));

	qDebug() << __FUNCTION__ << " Model_t* Factory::create<Model_t>()  : 2 ";

	std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories;
	assignmentCategories.push_back(waterLevelAssignmentCategory);
	assignmentCategories.push_back(seasonalAssignmentCategory);
	assignmentCategories.push_back(creepAssignmentCategory);
	assignmentCategories.push_back(driftAssignmentCategory);
	assignmentCategories.push_back(temperatureAssignmentCategory);
	assignmentCategories.push_back(averageAssignmentCategory);
	assignmentCategories.push_back(delayAssignmentCategory);
	assignmentCategories.push_back(anyAssignmentCategory);

	qDebug() << __FUNCTION__ << " Model_t* Factory::create<Model_t>()  : 3 ";
	boost::shared_ptr<ActivityTracker_t> activityTracker(FETCH_FROM_CTX(Model_Factory_t)->create<ActivityTracker_t>());

	qDebug() << __FUNCTION__ << " Model_t* Factory::create<Model_t>()  : 4 ";
	boost::shared_ptr<ModeSettings_t> modeSettings(FETCH_FROM_CTX(Model_Factory_t)->create<ModeSettings_t>());

	boost::shared_ptr<Analysis_Tools_Descriptions_t> analysisToolsDescriptions(FETCH_FROM_CTX(Model_Factory_t)->create<Analysis_Tools_Descriptions_t>(modeSettings));

	qDebug() << __FUNCTION__ << " Model_t* Factory::create<Model_t>()  : 5 ";
	return new Model(
				activityTracker,
				analysisToolsDescriptions,
				assignmentCategories,
				modeSettings
			);
}

template<>
Project_t* Factory::create<Project_t>() {
	using namespace dambase::model;
	return new Project(
			FETCH_FROM_CTX(Model_t)->getModeSettings()->getModeHandler()
			);
}

template<>
Project_t* Factory::create<Project_t>(boost::shared_ptr<ModeHandler_t> modeHandler) {
	using namespace dambase::model;
	return new Project(modeHandler);
}

template<>
WaterLevelScalingMinMaxOpLevel_t *Factory::create<WaterLevelScalingMinMaxOpLevel_t>() {
	using namespace dambase::model;
	return new WaterLevelScalingMinMaxOpLevel;
}

template<>
Analysis_t* Factory::create<Analysis_t>() {
	using namespace dambase::model;
	Analysis *analysis = new Analysis;

	boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> waterLevelScalingStrategy(FETCH_FROM_CTX(Model_Factory_t)->create<WaterLevelScalingMinMaxOpLevel_t>());
	analysis->setWaterLevelScalingStrategy(waterLevelScalingStrategy);

	return analysis;
}

template<>
R_Manager_t* Factory::create<R_Manager_t>(int argc, char *argv[]) {
	using namespace dambase::model;
	return new R_Manager_t(argc, argv);
}



} /* namespace app */
