/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelFactoryImpl.cpp
 *
 *  Created on: Mar 4, 2014
 *      Author: gerberma
 */

#include <stdexcept>
#include <vector>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "app/GenericApplicationContext.hpp"
#include "app/Factory.hpp"
#include "app/types.hpp"
#include "applicationcontext/scopes.hpp"

#include "model/RegressionModel.hpp"
#include "model/r/TimeRange.hpp"
#include "model/r/Regressor.hpp"
#include "model/r/RegressionModelDefinition.hpp"
#include "model/r/RegressionModelTemplate.hpp"
#include "model/r/RegressionFormula.hpp"
#include "model/r/RegressionFitProvider.hpp"

namespace app {


template<>
TimeRange_t *Factory::create<TimeRange_t>(boost::shared_ptr<DataModel_t> dataModel) {
	using namespace dambase::model::r;

	return new TimeRange(
			dataModel->getDateColumn()->getFirstValue(),
			dataModel->getDateColumn()->getLastValue()
	);
}

template<>
Regressor_t *Factory::create<Regressor_t, ExpressionRegressorFormulaBuilder_t, DataModel_t::DataFrame_t>(boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> regressorFormulaBuilder, boost::shared_ptr<DataModel_t::DataFrame_t> dataFrame) {
	using namespace dambase::model::r;

	return new Regressor(
				regressorFormulaBuilder,
				dataFrame,
				FETCH_FROM_CTX(Regression_Formula_Index_Manager_t)->getNextAvailableIndex()
			);
}

template<>
Regressor_t *Factory::create<Regressor_t>(int index) {
	throw std::logic_error("Factory::create<Regressor_t>(int index) is deprecated");
	//return new Regressor_t(index);
}

template<>
RegressionModelDefinition_t *Factory::create<RegressionModelDefinition_t, DataModel_t, QString>(boost::shared_ptr<DataModel_t> dataModel, QString name) {
	using namespace dambase::model::r;

	return new RegressionModelDefinition(name, dataModel->getDataFrame());
}

template<>
RegressionModelDefinition_t *Factory::create<RegressionModelDefinition_t, DataModel_t::DataFrame_t, QString, std::vector<boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> > >(boost::shared_ptr<DataModel_t::DataFrame_t> dataFrame, QString name, std::vector<boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> > formulaBuilders) {
	using namespace dambase::model::r;

	RegressionModelDefinition_t *modelDefinition = new RegressionModelDefinition(name, dataFrame);

	modelDefinition->addRegressorFormulaBuilders(formulaBuilders);

	return modelDefinition;
}

template<>
Regression_Formula_t *Factory::create<Regression_Formula_t, DataModel_t::DataFrame_t, TimeRange_t, Analysis_t>(boost::shared_ptr<DataModel_t::DataFrame_t> dataFrame, boost::shared_ptr<TimeRange_t> timeRange, boost::shared_ptr<Analysis_t> analysis) {
	using namespace dambase::model::r;

	std::vector<boost::shared_ptr<RegressionModelTemplate_t> > regressionModelTemplates;
	boost::shared_ptr<ModeHandler_t> modeHandler = FETCH_FROM_CTX(Model_t)->getModeSettings()->getModeHandler();

	bool templateDoesAppearInBasicMode = true;
	bool templateDoesAppearInScientificMode = true;
	bool templateDoesNotAppearInBasicMode = false;
	bool templateDoesNotAppearInScientificMode = false;


	boost::shared_ptr<RegressionModelTemplate_t> hstEdfModel(new RegressionModelTemplate(
			modeHandler,
			"f88f5ba7-5966-43f7-b2a7-1d78a9a0c297",
			QObject::tr("Hydrostatic Seasonal Time (EDF)"),
			QObject::tr("Tatin et al. (2013)\nSimon et al. (2013)"),
			QObject::tr("Hydrostatic seasonal time model according to EDF with minimum and full supply level scaling"),
			templateDoesAppearInScientificMode,
			templateDoesAppearInBasicMode
		));

	boost::shared_ptr<RegressionModelTemplate_t> hstModel(new RegressionModelTemplate(
			modeHandler,
			"8a31679e-2be6-42b9-8065-f9ef8d0abf26",
			QObject::tr("Hydrostatic Seasonal"),
			QObject::tr("no specific reference"),
			QObject::tr("Model with chebyshev polynom for water level and seasonal functions"),
			templateDoesAppearInScientificMode,
			templateDoesAppearInBasicMode
		));

	boost::shared_ptr<RegressionModelTemplate_t> httModel(new RegressionModelTemplate(
			modeHandler,
			"de467a4a-1697-4a42-9878-e61c21ffe5d5",
			QObject::tr("Hydrostatic Thermal"),
			QObject::tr("no specific reference"),
			QObject::tr("Model with chebyshev polynom and direct temperature"),
			templateDoesAppearInScientificMode,
			templateDoesAppearInBasicMode
		));

	boost::shared_ptr<RegressionModelTemplate_t> powerPolynom3rdDegree(new RegressionModelTemplate(
			modeHandler,
			"4053564f-1c35-429a-a808-0ea4b5097305",
			QObject::tr("Water level polynom 3rd degree"),
			QObject::tr("no specific reference"),
			QObject::tr("Water level polynom of 3rd degree without any special scaling"),
			templateDoesAppearInScientificMode,
			templateDoesNotAppearInBasicMode
		));

	boost::shared_ptr<RegressionModelTemplate_t> powerPolynom4thDegree(new RegressionModelTemplate(
			modeHandler,
			"bf88c257-cc9a-4f93-92f3-299fe547582d",
			QObject::tr("Water level polynom 4th degree"),
			QObject::tr("no specific reference"),
			QObject::tr("Water level polynom of 4th degree without any special scaling"),
			templateDoesAppearInScientificMode,
			templateDoesNotAppearInBasicMode
		));

	boost::shared_ptr<RegressionModelTemplate_t> seasonal2S(new RegressionModelTemplate(
			modeHandler,
			"11de61b8-4600-425d-b8eb-37bfe6a06e93",
			QObject::tr("Seasonal with scaling 2"),
			QObject::tr("no specific reference"),
			QObject::tr("Polynom of seasonal sin/cos functions up to scaling factor 2"),
			templateDoesAppearInScientificMode,
			templateDoesNotAppearInBasicMode
		));


	regressionModelTemplates.push_back(hstEdfModel);
	regressionModelTemplates.push_back(hstModel);
	regressionModelTemplates.push_back(httModel);
	regressionModelTemplates.push_back(powerPolynom3rdDegree);
	regressionModelTemplates.push_back(powerPolynom4thDegree);
	regressionModelTemplates.push_back(seasonal2S);

	using namespace app::context;

	std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories = FETCH_FROM_CTX(Model_t)->getAssignmentCategories();

	Model_t *model = FETCH_FROM_CTX(Model_t);
	boost::shared_ptr<AssignmentCategory_t> waterLevelAssignmentCategory = model->findAssignmentCategoryByIdentifier("water_level");
	boost::shared_ptr<AssignmentCategory_t> seasonalAssignmentCategory = model->findAssignmentCategoryByIdentifier("seasonal");
	boost::shared_ptr<AssignmentCategory_t> driftAssignmentCategory = model->findAssignmentCategoryByIdentifier("drift");
	boost::shared_ptr<AssignmentCategory_t> creepAssignmentCategory = model->findAssignmentCategoryByIdentifier("creep");
	boost::shared_ptr<AssignmentCategory_t> temperatureAssignmentCategory = model->findAssignmentCategoryByIdentifier("temperature");
	boost::shared_ptr<AssignmentCategory_t> averageAssignmentCategory = model->findAssignmentCategoryByIdentifier("average");
	boost::shared_ptr<AssignmentCategory_t> delayAssignmentCategory = model->findAssignmentCategoryByIdentifier("delay");
	boost::shared_ptr<AssignmentCategory_t> anyAssignmentCategory = model->findAssignmentCategoryByIdentifier("any");


	std::vector<boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> > customRegressorFormulaBuilders;

	// SEASONAL SIN
	for(int i = 1; i <= 4; ++i) {

		boost::shared_ptr<SeasonalFormulaBuilderDefinition_t> seasonal(FETCH_FROM_CTX(Model_Factory_t)->create<SeasonalFormulaBuilderDefinition_t>(true, i));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
										seasonal,
										seasonalAssignmentCategory, //get one from the data frame
										assignmentCategories
									));

		if(i == 1) {
			hstModel->addRegressorFormulaBuilder(builder);
		}
		if(i <= 2) {
			seasonal2S->addRegressorFormulaBuilder(builder);
		}

		if(i <= 2) {
			hstEdfModel->addRegressorFormulaBuilder(builder);
		}

		customRegressorFormulaBuilders.push_back(builder);
	}

	// SEASONAL COS
	for(int i = 1; i <= 4; ++i) {
		boost::shared_ptr<SeasonalFormulaBuilderDefinition_t> seasonal(FETCH_FROM_CTX(Model_Factory_t)->create<SeasonalFormulaBuilderDefinition_t>(false, i));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
										seasonal,
										seasonalAssignmentCategory, //get one from the data frame
										assignmentCategories
									));
		customRegressorFormulaBuilders.push_back(builder);

		if(i == 1) {
			hstModel->addRegressorFormulaBuilder(builder);
		}
		if(i <= 2) {
			seasonal2S->addRegressorFormulaBuilder(builder);
		}

		if(i <= 2) {
			hstEdfModel->addRegressorFormulaBuilder(builder);
		}
	}

	// POWER & CREEP(POWER)
	for(int i = 1; i <= 5; ++i) {
		boost::shared_ptr<PowerFormulaBuilderDefinition_t> p(FETCH_FROM_CTX(Model_Factory_t)->create<PowerFormulaBuilderDefinition_t>(i));

		std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > emptyPreprocessors;
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				p,
				waterLevelAssignmentCategory,
				assignmentCategories,
				emptyPreprocessors
			));


		boost::shared_ptr<CreepPowerFormulaBuilderDefinition_t> creepPower(FETCH_FROM_CTX(Model_Factory_t)->create<CreepPowerFormulaBuilderDefinition_t, PowerFormulaBuilderDefinition_t>(p));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> creepPowerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
					creepPower,
					creepAssignmentCategory,
					assignmentCategories,
					emptyPreprocessors
				));

		customRegressorFormulaBuilders.push_back(creepPowerBuilder);

		if(i <= 3) {
			powerPolynom3rdDegree->addRegressorFormulaBuilder(builder);
		}
		if(i <= 4) {
			powerPolynom4thDegree->addRegressorFormulaBuilder(builder);
		}

		customRegressorFormulaBuilders.push_back(builder);
	}

	// POWER and CREEP(POWER) special scaling
	for(int i = 1; i <= 5; ++i) {
		boost::shared_ptr<PowerFormulaBuilderDefinition_t> p(FETCH_FROM_CTX(Model_Factory_t)->create<PowerFormulaBuilderDefinition_t>(i));

		//create cheby with a preprocessor for min/max operating level regressor formula builder preprocessors
		std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > minMaxPreprocessors;
		boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t> minMax(FETCH_FROM_CTX(Model_Factory_t)->create<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t>(analysis->getWaterLevelScalingStrategy(), QString("Minimum / Full supply level scaling (max-wlevel)/(max-min)"), QString("f.WLev_sc_EDF")));
		minMaxPreprocessors.push_back(minMax);
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> minMaxPreprocessorPowerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
			p,
			waterLevelAssignmentCategory, //get one from the data frame
			assignmentCategories,
			minMaxPreprocessors
		));

		minMaxPreprocessorPowerBuilder->setIdentifier("scaled - " + minMaxPreprocessorPowerBuilder->getIdentifier() + " (min oplevel and full supply level)");
		minMaxPreprocessorPowerBuilder->setFormula(minMaxPreprocessorPowerBuilder->getFormula() + "{min/max}");

		customRegressorFormulaBuilders.push_back(minMaxPreprocessorPowerBuilder);


		boost::shared_ptr<CreepPowerFormulaBuilderDefinition_t> creepPower(FETCH_FROM_CTX(Model_Factory_t)->create<CreepPowerFormulaBuilderDefinition_t, PowerFormulaBuilderDefinition_t>(p));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> minMaxPreprocessorCreepPowerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
					creepPower,
					creepAssignmentCategory,
					assignmentCategories,
					minMaxPreprocessors
				));

		minMaxPreprocessorCreepPowerBuilder->setIdentifier("scaled - " + minMaxPreprocessorCreepPowerBuilder->getIdentifier() + " (min oplevel and full supply level)");
		minMaxPreprocessorCreepPowerBuilder->setFormula(minMaxPreprocessorCreepPowerBuilder->getFormula() + "{min/max}");

		customRegressorFormulaBuilders.push_back(minMaxPreprocessorCreepPowerBuilder);


		if(i <= 4) {
			hstEdfModel->addRegressorFormulaBuilder(minMaxPreprocessorPowerBuilder);
		}
	}

	// CHEBYSHEV and CREEP(CHEBYSHEV)
	for(int i = 1; i <= 5; ++i) {
		boost::shared_ptr<ChebyshevFormulaBuilderDefinition_t> cheby(FETCH_FROM_CTX(Model_Factory_t)->create<ChebyshevFormulaBuilderDefinition_t>(i));

		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> noPreprocessorChebyBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
									cheby,
									waterLevelAssignmentCategory, //get one from the data frame
									assignmentCategories
								));

		customRegressorFormulaBuilders.push_back(noPreprocessorChebyBuilder);

		boost::shared_ptr<CreepChebyshevFormulaBuilderDefinition_t> creepCheby(FETCH_FROM_CTX(Model_Factory_t)->create<CreepChebyshevFormulaBuilderDefinition_t, ChebyshevFormulaBuilderDefinition_t>(cheby));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> creepChebyBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
					creepCheby,
					creepAssignmentCategory,
					assignmentCategories
				));

		customRegressorFormulaBuilders.push_back(creepChebyBuilder);

		if(i <= 4)
		{
			hstModel->addRegressorFormulaBuilder(noPreprocessorChebyBuilder);
			httModel->addRegressorFormulaBuilder(noPreprocessorChebyBuilder);
		}
	}

	// DRIFT Square root
	{
		boost::shared_ptr<SquareRootBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<SquareRootBuilderDefinition_t>());
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> squareBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				driftAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(squareBuilder);
	}

	// DRIFT Logarithmus Naturalis
	{
		boost::shared_ptr<LogarithmusNaturalisBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<LogarithmusNaturalisBuilderDefinition_t>());

		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> logarithmNaturalisBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				driftAssignmentCategory, //get one from the data frame
				assignmentCategories
			));

		customRegressorFormulaBuilders.push_back(logarithmNaturalisBuilder);
	}

	// DRIFT Power (direct and up to 4)
	for(int power = 1; power <= 4; ++power) {
		boost::shared_ptr<DriftPowerBuilderDefinition_t> driftPower(FETCH_FROM_CTX(Model_Factory_t)->create<DriftPowerBuilderDefinition_t>(power));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				driftPower,
				driftAssignmentCategory, //get one from the data frame
				assignmentCategories
			));

		customRegressorFormulaBuilders.push_back(builder);

		if(power <= 4) {
			hstEdfModel->addRegressorFormulaBuilder(builder);
		}
	}

	// DRIFT Exponential
	for(int divisor = 1; divisor <= 8;) {
		boost::shared_ptr<ExponentialBuilderDefinition_t> drift(FETCH_FROM_CTX(Model_Factory_t)->create<ExponentialBuilderDefinition_t>(divisor));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
									drift,
									driftAssignmentCategory, //get one from the data frame
									assignmentCategories
								));

		customRegressorFormulaBuilders.push_back(builder);

		if(divisor == 2) {
			hstEdfModel->addRegressorFormulaBuilder(builder, "Adjust argument divisor manually to get the best fit.");
		}


		if(divisor == 1) {
			divisor += 1;
		}else{
			divisor += 2;
		}
	}

	// AVERAGE
	{
		// DIRECT
		boost::shared_ptr<DirectExpressionFormulaBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<DirectExpressionFormulaBuilderDefinition_t>(QString("average"), QString("avg")));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> averageDirect(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				averageAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(averageDirect);


		std::vector<int> timesteps;
		timesteps.push_back(7);
		timesteps.push_back(14);
		timesteps.push_back(30);
		timesteps.push_back(60);
		timesteps.push_back(120);
		timesteps.push_back(180);
		timesteps.push_back(360);

		BOOST_FOREACH(int timestep, timesteps) {
			boost::shared_ptr<AverageExpressionFormulaBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<AverageExpressionFormulaBuilderDefinition_t>(timestep));
			boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
					builder,
					averageAssignmentCategory, //get one from the data frame
					assignmentCategories
				));
			customRegressorFormulaBuilders.push_back(innerBuilder);
		}
	}

	// DELAY
	{
		for(int timestep = 1; timestep <=4; ++timestep) {
			boost::shared_ptr<DelayExpressionFormulaBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<DelayExpressionFormulaBuilderDefinition_t>(timestep));
			boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
					builder,
					delayAssignmentCategory, //get one from the data frame
					assignmentCategories
				));
			customRegressorFormulaBuilders.push_back(innerBuilder);
		}
	}

	// EXTRA Direct
	{
		boost::shared_ptr<DirectExpressionFormulaBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<DirectExpressionFormulaBuilderDefinition_t>(QString("any"), QString("any")));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				anyAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(innerBuilder);
	}

	// EXTRA ExtraSqrtRampBuilderDefinition_t
	{
		boost::shared_ptr<ExtraSqrtRampBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExtraSqrtRampBuilderDefinition_t>(100));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				anyAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(innerBuilder);
	}

	// EXTRA ExtraColumnTimesStepBuilderDefinition_t
	{
		boost::shared_ptr<ExtraColumnTimesStepBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExtraColumnTimesStepBuilderDefinition_t>(100));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				anyAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(innerBuilder);
	}

	// EXTRA SplineExpressionFormulaBuilderDefinition_t
	{
		std::vector<double> x;
		std::vector<double> y;

		x.push_back(1);
		x.push_back(2);
		x.push_back(3);

		y.push_back(1);
		y.push_back(2);
		y.push_back(4);

		boost::shared_ptr<SplineExpressionFormulaBuilderDefinition_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<SplineExpressionFormulaBuilderDefinition_t>(x, y));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builder,
				anyAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(innerBuilder);
	}

	// TEMPERATURE
	{
		// DIRECT
		boost::shared_ptr<DirectExpressionFormulaBuilderDefinition_t> builderDefinition(FETCH_FROM_CTX(Model_Factory_t)->create<DirectExpressionFormulaBuilderDefinition_t>(QString("temperature"), QString("temp")));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> builder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builderDefinition,
				temperatureAssignmentCategory, //get one from the data frame
				assignmentCategories
			));

		customRegressorFormulaBuilders.push_back(builder);

		httModel->addRegressorFormulaBuilder(builder);

	}
	for(int depth = 2; depth <= 6; depth+=2) {
		boost::shared_ptr<TemperatureOneDimensionalHeatEquation_t> builderDefinition(FETCH_FROM_CTX(Model_Factory_t)->create<TemperatureOneDimensionalHeatEquation_t>(depth));
		boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> innerBuilder(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilder_t>(
				builderDefinition,
				temperatureAssignmentCategory, //get one from the data frame
				assignmentCategories
			));
		customRegressorFormulaBuilders.push_back(innerBuilder);
	}


	boost::shared_ptr<RegressionModelDefinition_t> customRegressionModelDefinition(FETCH_FROM_CTX(Model_Factory_t)->create<RegressionModelDefinition_t, DataModel_t::DataFrame_t, QString, std::vector<boost::shared_ptr<ExpressionRegressorFormulaBuilder_t > > >(
				dataFrame,
				QObject::tr("Custom model definition"),
				customRegressorFormulaBuilders
			));


	Regression_Formula_t *regressionFormula = new RegressionFormula(
			regressionModelTemplates,
			customRegressorFormulaBuilders,
			dataFrame
			);

	regressionFormula->addCustomRegressionModelDefinition(customRegressionModelDefinition);

	return regressionFormula;
}

template<>
Regression_Fit_Provider_t *Factory::create<Regression_Fit_Provider_t>() {
	using namespace dambase::model::r;

	return new RegressionFitProvider();
}

template<>
RegressionModel_t *Factory::create<RegressionModel_t, DataModel_t, Analysis_t>(boost::shared_ptr<DataModel_t> dataModel, boost::shared_ptr<Analysis_t> analysis) {
	using namespace app::context;
	using namespace dambase::model;

	boost::shared_ptr<TimeRange_t> timeRange(FETCH_FROM_CTX(App_Factory_t)->create<TimeRange_t>(dataModel));

	boost::shared_ptr<Regression_Formula_t> regressionFormula(FETCH_FROM_CTX(App_Factory_t)->create<Regression_Formula_t>(dataModel->getDataFrame(), timeRange, analysis));

	return new RegressionModel(
				dataModel,
				timeRange,
				regressionFormula,
				FETCH_FROM_CTX(Regression_Fit_Provider_t)
			);
}

} /* namespace app */
