/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisController.hpp
 *
 *  Created on: Nov 28, 2013
 *      Author: gerberma
 */

#ifndef ANALYSISCONTROLLER_HPP_
#define ANALYSISCONTROLLER_HPP_

#include <vector>
#include <iostream>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "applicationcontext/scopes.hpp"
#include "communication/Signals.hpp"

namespace controller {

template<
	typename A,
	typename WorkingAreaContainer,
	typename AnalysisUiManager,
	typename ActivityTracker
>
class AnalysisController {
public:
	AnalysisController(WorkingAreaContainer *workingAreaContainer, AnalysisUiManager *analysisUiManager, boost::shared_ptr<ActivityTracker> activityTracker)
		:workingAreaContainer(workingAreaContainer), analysisUiManager(analysisUiManager), activityTracker(activityTracker){;}

	virtual ~AnalysisController(){;}

	template<typename P, typename AnalysisFactory>
	void createFor(boost::shared_ptr<P> project, AnalysisFactory *analysisFactory) {
		boost::shared_ptr<A> analysis(analysisFactory->template create<A>());

		typedef typename AnalysisUiManager::wizard_ui_type wizard_ui_t;
		app::context::LocalScope<wizard_ui_t> scope(analysisUiManager->getWizardFor(analysis), boost::bind<void>(&AnalysisUiManager::destroyWizardFor, analysisUiManager, analysis));
		wizard_ui_t *wizard = scope.get();

		wizard->start();

		if(wizard->accepted() == false) {
			return;
		}

		project->addAnalysis(analysis);

		typename AnalysisUiManager::overview_ui_type *analysisOverviewUi;
		analysisOverviewUi = analysisUiManager->getOverviewUiFor(analysis);

		//subscribe for edit
		analysisOverviewUi->subscribeForEdit(
				boost::bind(&AnalysisController<A, WorkingAreaContainer, AnalysisUiManager, ActivityTracker>::edit, this, _1)
		);

		COM_SUBSCRIBE_FOR(analysisOverviewUi, EditWaterLevelScaling,
				boost::bind(&AnalysisController<A, WorkingAreaContainer, AnalysisUiManager, ActivityTracker>::editWaterLevelScaling, this, _1)
		);

		activityTracker->trackChanged();

		//show overview ui
		show(analysis);
	}

	template<typename P>
	void remove(boost::shared_ptr<A> analysis, boost::shared_ptr<P> project) {
		using namespace app::context;
		typedef typename AnalysisUiManager::info_dialog_ui_type info_dialog_ui_t;

		LocalScope<info_dialog_ui_t> infoDialog(analysisUiManager->getInfoDialogUiFor(analysis), boost::bind<void>(&AnalysisUiManager::destroyInfoDialogUiFor, analysisUiManager, analysis));

		if(infoDialog->confirmRemoveQuestionFor(analysis) == false) {
			return;
		}

		analysisUiManager->destroyOverviewUiFor(analysis);

		//TODO: analysisUiManager should delegate destroy for each regressionModel and the dataModel one as well
		// or: we get the RegressionModel's uiManager as well as the dataModel's here
		// maybe we can provide a visitor pattern to this function here

		//TODO: delegate to RegressionModel controller as well as DataModel controller to remove objects properly and guarantee that the ownership structures of the uiManagers are save
		// I should really use a visitor here to do this job

		project->removeAnalysis(analysis);

		activityTracker->trackChanged();
	}

	void edit(boost::shared_ptr<A> analysis) {
		typedef typename AnalysisUiManager::wizard_ui_type wizard_ui_t;
		app::context::LocalScope<wizard_ui_t> scope(analysisUiManager->getWizardFor(analysis), boost::bind<void>(&AnalysisUiManager::destroyWizardFor, analysisUiManager, analysis));
		wizard_ui_t *wizard = scope.get();

		wizard->start();
	}

	void editWaterLevelScaling(boost::shared_ptr<A> analysis) {
		typedef typename AnalysisUiManager::wizard_ui_type wizard_ui_t;
		app::context::LocalScope<wizard_ui_t> scope(analysisUiManager->getWizardFor(analysis), boost::bind<void>(&AnalysisUiManager::destroyWizardFor, analysisUiManager, analysis));
		wizard_ui_t *wizard = scope.get();

		wizard->editWaterLevelScaling();
	}

	void show(boost::shared_ptr<A> analysis) {
		typename AnalysisUiManager::overview_ui_type *analysisOverviewUi;
		analysisOverviewUi = analysisUiManager->getOverviewUiFor(analysis);

		analysisOverviewUi->show();
		workingAreaContainer->setWorkingArea(analysisOverviewUi);
	}

	void deserialize(boost::shared_ptr<A> analysis) {
		std::cout << "AnalysisController " <<  __FUNCTION__ << std::endl;
		//setup overview uis for analysis and connect them
		typename AnalysisUiManager::overview_ui_type *analysisOverviewUi;
		analysisOverviewUi = analysisUiManager->getOverviewUiFor(analysis);

		//subscribe for edit
		analysisOverviewUi->subscribeForEdit(
				boost::bind(&AnalysisController<A, WorkingAreaContainer, AnalysisUiManager, ActivityTracker>::edit, this, _1)
		);
		analysisOverviewUi->subscribeForEditWaterLevelScaling(
				boost::bind(&AnalysisController<A, WorkingAreaContainer, AnalysisUiManager, ActivityTracker>::editWaterLevelScaling, this, _1)
		);
	}

private:
	WorkingAreaContainer *workingAreaContainer;
	AnalysisUiManager *analysisUiManager;
	boost::shared_ptr<ActivityTracker> activityTracker;
};

} /* namespace controller */


#endif /* ANALYSISCONTROLLER_HPP_ */
