/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelController.hpp
 *
 *  Created on: Dec 6, 2013
 *      Author: gerberma
 */

#ifndef DATAMODELCONTROLLER_HPP_
#define DATAMODELCONTROLLER_HPP_

#include <iostream>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <QString>
#include <QDebug>
#include "applicationcontext/scopes.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "DataModelControllerInternalSubscriptionManager.hpp"
#include "communication/Signals.hpp"

namespace controller {

template<
	typename DataModel,
	typename ActivityTracker,
	typename AnalysisToolsController,
	typename DataModelAnalysisSettings,
	typename DataModelUiManager,
	typename SubscriptionManager
>
class DataModelController {
	typedef typename DataModelUiManager::wizard_ui_type wizard_ui_t;
public:
	DataModelController(boost::shared_ptr<ActivityTracker> activityTracker, AnalysisToolsController *analysisToolsController, DataModelAnalysisSettings *analysisSettings, DataModelUiManager *dataModelUiManager, SubscriptionManager *subscriptionManager)
		:activityTracker(activityTracker), analysisToolsController(analysisToolsController), analysisSettings(analysisSettings), dataModelUiManager(dataModelUiManager), subscriptionManager(subscriptionManager)
	{
		if(analysisToolsController == NULL) {
			qDebug() << __FUNCTION__ << " :  analysisToolsController == NULL";
		}
		if(analysisSettings == NULL) {
			qDebug() << __FUNCTION__ << " :  analysisSettings == NULL";
		}
		if(dataModelUiManager == NULL) {
			qDebug() << __FUNCTION__ << " :  dataModelUiManager == NULL";
		}
		if(subscriptionManager == NULL) {
			qDebug() << __FUNCTION__ << " :  subscriptionManager == NULL";
		}
	}

	virtual ~DataModelController(){;}

	template<typename Analysis, typename DataModelFactory>
	void createFor(boost::shared_ptr<Analysis> analysis, DataModelFactory *dataModelFactory){
		boost::shared_ptr<DataModel> dataModel(dataModelFactory->template create<DataModel>());

		app::context::LocalScope<wizard_ui_t> scope(createWizardFor(dataModel), boost::bind<void>(&DataModelUiManager::destroyWizardFor, dataModelUiManager, dataModel));
		wizard_ui_t *wizard = scope.get();

		wizard->start();

		if(wizard->accepted() == false) {
			return;
		}

		analysis->setDataModel(dataModel);

		activityTracker->trackChanged();
		show(dataModel, analysis);
	}

	template<typename Analysis>
	void instantiateAllAnalysisTools(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<Analysis> analysis) {
		typedef typename DataModelAnalysisSettings::AnalysisToolDescription_t AnalysisToolDescription_t;
		//left side overview
		qDebug() << __FUNCTION__ << " : 1";
		BOOST_FOREACH(boost::shared_ptr<AnalysisToolDescription_t> analysisToolDescription, analysisSettings->getAutomaticLeftSideOverviewCategoryAnalysisToolDescriptions()) {
			analysisToolsController->addAnalysisTool(
				analysis,
				analysisToolDescription,
				dataModelUiManager->getLeftSideOverviewCategoryWidgetFor(dataModel)->getUiLocationCallback()
			);
		}

		qDebug() << __FUNCTION__ << " : 2";
		//right side overview
		BOOST_FOREACH(boost::shared_ptr<AnalysisToolDescription_t> analysisToolDescription, analysisSettings->getAutomaticRightSideOverviewCategoryAnalysisToolDescriptions()) {
			analysisToolsController->addAnalysisTool(
				analysis,
				analysisToolDescription,
				dataModelUiManager->getRightSideOverviewCategoryWidgetFor(dataModel)->getUiLocationCallback()
			);
		}

		qDebug() << __FUNCTION__ << " : 3";
	}

	template<typename Analysis>
	void show(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<Analysis> analysis){
		typename DataModelUiManager::overview_ui_type *dataModelOverviewUi;
		bool newCreated = false;
		dataModelOverviewUi = dataModelUiManager->getOverviewUiFor(dataModel, newCreated);

		bool isNull = dataModelOverviewUi == NULL;

		qDebug() << __FUNCTION__ << " dataModelOverviewUi :  " << isNull;

		if(newCreated) {
			qDebug() << __FUNCTION__ << " 2 ";
			subscriptionManager->subscribeOverviewUi(dataModelOverviewUi, analysis, this);
			qDebug() << __FUNCTION__ << " 3 ";
			subscriptionManager->subscribeOverviewUiDelegates(dataModelOverviewUi, analysis);
			qDebug() << __FUNCTION__ << " 4 ";
			instantiateAllAnalysisTools(dataModel, analysis);
			qDebug() << __FUNCTION__ << " 5 ";
		}

		qDebug() << __FUNCTION__ << " 6 ";
		dataModelOverviewUi->show();

		qDebug() << __FUNCTION__ << " 7 ";
		dataModelUiManager->getWorkingAreaContainerUi()->setWorkingArea(dataModelOverviewUi);

		qDebug() << __FUNCTION__ << " 8 ";
	}

	void requestSourceUri(boost::shared_ptr<DataModel> dataModel){
		typedef typename DataModelUiManager::uri_dialog_ui_type uri_dialog_ui_t;
		app::context::LocalScope<uri_dialog_ui_t> scope(dataModelUiManager->getUriDialogUiFor(dataModel), boost::bind<void>(&DataModelUiManager::destroyUriDialogUiFor, dataModelUiManager, dataModel));
		uri_dialog_ui_t *uriDialog = scope.get();

		typename DataModel::source_uri_t uri = uriDialog->getOpenUrl(QString("Data model"), QString("csv"));

		if(!uri.isEmpty()) {
			dataModel->setSourceUri(uri);
			loadDataModel(dataModel);
		}
	}

	void setCSVDelimiter(boost::shared_ptr<DataModel> dataModel, QString csvDelimiter){
		dataModel->setCSVDelimiter(csvDelimiter);
		activityTracker->trackChanged();

		if(!dataModel->getSourceUri().isEmpty()) {
			loadDataModel(dataModel);
		}
	}

	void dateColumnChange(boost::shared_ptr<DataModel> dataModel, unsigned index){
		std::cout << __FUNCTION__ << " - " << index << std::endl;
		dataModel->setDateColumn(index);
		activityTracker->trackChanged();
	}

	void applyDateFormat(boost::shared_ptr<DataModel> dataModel, QString dateFormat){
		std::cout << __FUNCTION__ << std::endl;
		dataModel->applyDateFormat(dateFormat);
		activityTracker->trackChanged();
	}

	void behaviorIndicatorAssignmentChanged(boost::shared_ptr<DataModel> dataModel, int index, bool assignment){
		std::cout << __FUNCTION__ << " index: " << index << " , assignment: " << assignment << std::endl;
		if(assignment == true) {
			dataModel->assignBehaviorIndicatorColumn(index);
		}else{
			dataModel->removeBehaviorIndicatorColumn(index);
		}

		activityTracker->trackChanged();
	}

	void waterLevelAssignmentChanged(boost::shared_ptr<DataModel> dataModel, int index, bool assignment){
		std::cout << __FUNCTION__ << " index: " << index << " , assignment: " << assignment << std::endl;
		if(assignment == true) {
			dataModel->assignWaterLevelColumn(index);
		}else{
			dataModel->removeWaterLevelColumn(index);
		}

		activityTracker->trackChanged();
	}

	void temperatureAssignmentChanged(boost::shared_ptr<DataModel> dataModel, int index, bool assignment){
		std::cout << __FUNCTION__ << " index: " << index << " , assignment: " << assignment << std::endl;
		if(assignment == true) {
			dataModel->assignTemperatureColumn(index);
		}else{
			dataModel->removeTemperatureColumn(index);
		}

		activityTracker->trackChanged();
	}

	template<typename Analysis>
	void editCommonParameters(boost::shared_ptr<DataModel>dataModel, boost::shared_ptr<Analysis> analysis){
		if(!analysis->getRegressionModels().empty()) {
			throw std::logic_error("Editing data model is prevented as there is already a regression model using this data model");
		}
		app::context::LocalScope<wizard_ui_t> scope(createWizardFor(dataModel), boost::bind<void>(&DataModelUiManager::destroyWizardFor, dataModelUiManager, dataModel));
		wizard_ui_t *wizard = scope.get();

		wizard->editCommonParameters();

		//show(dataModel);
	}

	template<typename Analysis>
	void editTimeSeries(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<Analysis> analysis){
		if(!analysis->getRegressionModels().empty()) {
			throw std::logic_error("Editing data model is prevented as there is already a regression model using this data model");
		}
		app::context::LocalScope<wizard_ui_t> scope(createWizardFor(dataModel), boost::bind<void>(&DataModelUiManager::destroyWizardFor, dataModelUiManager, dataModel));
		wizard_ui_t *wizard = scope.get();

		wizard->editTimeSeries();

		//show(dataModel);
	}

	template<typename Analysis>
	void editAssignment(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<Analysis> analysis){
		if(!analysis->getRegressionModels().empty()) {
			throw std::logic_error("Editing data model is prevented as there is already a regression model using this data model");
		}

		app::context::LocalScope<wizard_ui_t> scope(createWizardFor(dataModel), boost::bind<void>(&DataModelUiManager::destroyWizardFor, dataModelUiManager, dataModel));
		wizard_ui_t *wizard = scope.get();

		wizard->editAssignment();

		//show(dataModel);
	}

	typename DataModelUiManager::wizard_ui_type *createWizardFor(boost::shared_ptr<DataModel> dataModel){
		wizard_ui_t *wizard = dataModelUiManager->getWizardFor(dataModel);
		subscriptionManager->subscribeWizard(wizard, this);
		return wizard;
	}

private:
	void loadDataModel(boost::shared_ptr<DataModel> dataModel){
		if(!dataModel->getSourceUri().isEmpty()) {
			dataModel->loadDataFrame();
			dataModel->estimateDateColumn();
		}
	}

private:
	boost::shared_ptr<ActivityTracker> activityTracker;
	AnalysisToolsController *analysisToolsController;
	DataModelAnalysisSettings *analysisSettings;
	DataModelUiManager *dataModelUiManager;
	SubscriptionManager *subscriptionManager;
};




} /* namespace controller */

#endif /* DATAMODELCONTROLLER_HPP_ */
