/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericController.hpp
 *
 *  Created on: Feb 21, 2014
 *      Author: gerberma
 */

#ifndef GENERICCONTROLLER_HPP_
#define GENERICCONTROLLER_HPP_

#include <boost/mpl/eval_if.hpp>
#include <boost/mpl/identity.hpp>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "applicationcontext/scopes.hpp"


namespace controller {

namespace detail {

template<class T> struct hasWizardDecisionMaker {
    template<class U> static char (&test(typename U::wizard_ui_t const*))[1];
    template<class U> static char (&test(...))[2];
    static const bool value = (sizeof(test<T>(0)) == 1);
};

template<class T> struct hasWorkingDialogDecisionMaker {
    template<class U> static char (&test(typename U::working_dialog_ui_t const*))[1];
    template<class U> static char (&test(...))[2];
    static const bool value = (sizeof(test<T>(0)) == 1);
};

template<class T> struct hasOverviewUiDecisionMaker {
    template<class U> static char (&test(typename U::overview_ui_t const*))[1];
    template<class U> static char (&test(...))[2];
    static const bool value = (sizeof(test<T>(0)) == 1);
};

template<class T> struct hasWorkingAreaContainerDecisionMaker {
    template<class U> static char (&test(typename U::working_area_container_ui_t const*))[1];
    template<class U> static char (&test(...))[2];
    static const bool value = (sizeof(test<T>(0)) == 1);
};

template<typename T>
void nullFunction(boost::shared_ptr<T>){;}

}

template<
	typename T,
	typename UiManager
>
class GenericController {
	struct HasWizard {
		template<typename SubscriptionManager>
		static bool invoke(boost::shared_ptr<T> t, UiManager *uiManager, SubscriptionManager *subscriptionManager) {
			using namespace boost;
			app::context::LocalScope<typename UiManager::wizard_ui_t> wizard(uiManager->getWizardFor(t, subscriptionManager), bind<void>(&UiManager::destroyWizardFor, uiManager, t));
			wizard->start();

			return wizard->accepted();
		}
	};

	struct HasNoWizard {
		template<typename SubscriptionManager>
		static bool invoke(boost::shared_ptr<T>, UiManager *, SubscriptionManager *) {
			return true;
		}
	};

	struct HasWorkingDialog {
		template<typename SubscriptionManager>
		static void invoke(boost::shared_ptr<T> t, UiManager *uiManager, SubscriptionManager *subscriptionManager) {
			using namespace boost;
			app::context::LocalScope<typename UiManager::working_dialog_ui_t> workingDialog(uiManager->getWorkingDialogFor(t, subscriptionManager), bind<void>(&UiManager::destroyWorkingDialogFor, uiManager, t));
			workingDialog->exec();
			return;
		}
	};

	struct HasNoWorkingDialog {
		template<typename SubscriptionManager>
		static void invoke(boost::shared_ptr<T> t, UiManager *uiManager, SubscriptionManager *subscriptionManager) {
			return;
		}
	};

	struct HasOverviewUi {
		template<typename SubscriptionManager, typename Controller>
		static void invoke(boost::shared_ptr<T> t, SubscriptionManager *subscriptionManager, Controller *controller) {
			controller->show(t, subscriptionManager);
		}
	};

	struct HasNoOverviewUi {
		template<typename SubscriptionManager, typename Controller>
		static void invoke(boost::shared_ptr<T> t, SubscriptionManager *subscriptionManager, Controller *controller) {
			return;
		}
	};

	struct HasWorkingAreaContainer {
		static void invoke(UiManager *uiManager, typename UiManager::overview_ui_t *overviewUi) {
			uiManager->getWorkingAreaContainerUi()->setWorkingArea(overviewUi);
		}
	};

	struct HasNoWorkingAreaContainer {
		static void invoke(UiManager *, typename UiManager::overview_ui_t *) {;}
	};

public:
	GenericController(UiManager *uiManager)
		:uiManager(uiManager){;}

	virtual ~GenericController(){;}

	template<typename SubscriptionManager>
	void createFor(
			boost::function<void(boost::shared_ptr<T>)> parentCallback,
			boost::function<T*()> factory,
			SubscriptionManager *subscriptionManager,
			boost::function<void(boost::shared_ptr<T>)> postCreateCallback = boost::bind(detail::nullFunction<T>, _1)
	){
		namespace bmpl = boost::mpl;

		boost::shared_ptr<T> t(factory());
		postCreateCallback(t);

		/**
		 * Wizard case
		 */
		typedef BOOST_DEDUCED_TYPENAME bmpl::eval_if<
				detail::hasWizardDecisionMaker<UiManager>,
				bmpl::identity<HasWizard>,
				bmpl::identity<HasNoWizard>
		>::type handleWizardCase;

		if(handleWizardCase::invoke(t, uiManager, subscriptionManager) == false) {
			return;
		}

		/**
		 * Working Dialog case
		 */
		typedef BOOST_DEDUCED_TYPENAME bmpl::eval_if<
				detail::hasWorkingDialogDecisionMaker<UiManager>,
				bmpl::identity<HasWorkingDialog>,
				bmpl::identity<HasNoWorkingDialog>
		>::type handleWorkingDialogCase;

		handleWorkingDialogCase::invoke(t, uiManager, subscriptionManager);


		parentCallback(t);


		/**
		 * Overview ui case
		 * Call 'show' only if an overview ui has been defined by the UiManager
		 */
		typedef BOOST_DEDUCED_TYPENAME bmpl::eval_if<
				detail::hasOverviewUiDecisionMaker<UiManager>,
				bmpl::identity<HasOverviewUi>,
				bmpl::identity<HasNoOverviewUi>
		>::type handleOverviewUiCase;
		handleOverviewUiCase::invoke(t, subscriptionManager, this);
	}

	template<typename SubscriptionManager>
	void show(boost::shared_ptr<T> t, SubscriptionManager *subscriptionManager) {
		namespace bmpl = boost::mpl;

		typename UiManager::overview_ui_t *overviewUi = uiManager->getOverviewUiFor(t, subscriptionManager);
		overviewUi->show();

		typedef BOOST_DEDUCED_TYPENAME bmpl::eval_if<
				detail::hasWorkingAreaContainerDecisionMaker<UiManager>,
				bmpl::identity<HasWorkingAreaContainer>,
				bmpl::identity<HasNoWorkingAreaContainer>
		>::type handleWorkingAreaContainer;

		handleWorkingAreaContainer::invoke(uiManager, overviewUi);
	}

private:
	UiManager *uiManager;
};


} /* namespace controller */


#endif /* GENERICCONTROLLER_HPP_ */
