/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectController.hpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#ifndef PROJECTCONTROLLER_HPP_
#define PROJECTCONTROLLER_HPP_

#include <iostream>
#include <QDebug>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include "applicationcontext/scopes.hpp"
#include "mpl/Type2Type.hpp"

namespace controller {

/**
 * P = Project
 * PF = ProjectFactory
 * PW = ProjectWizard
 */
template<
	typename P,
	typename PF,
	typename PW,
	typename Model,
	typename WorkingAreaContainer,
	typename ProjectOverviewUi,
	typename NavigationUi,
	typename FileDialogUi,
	typename SerializationPolicy,
	typename ExceptionPolicy
>
class ProjectController : public SerializationPolicy {
public:
	ProjectController(PF *projectFactory, PW *projectWizard, Model *model, WorkingAreaContainer *workingAreaContainer, ProjectOverviewUi *projectOverviewUi, NavigationUi *navigationUi)
		:projectFactory(projectFactory),
		 projectWizard(projectWizard),
		 model(model),
		 workingAreaContainer(workingAreaContainer),
		 projectOverviewUi(projectOverviewUi),
		 navigationUi(navigationUi)
	{
		qDebug() << __FUNCTION__;
	}

	virtual ~ProjectController(){}

	void create() {
		using namespace app::context;

		boost::shared_ptr<P> project(projectFactory->template create<P>(model->getModeSettings()->getModeHandler()));

		projectWizard->start(project);

		if(projectWizard->accepted() == false) {
			projectWizard->resetProject();
			return;
		}

		model->setProject(project);
		projectOverviewUi->show(model->getProject());
		navigationUi->show(model->getProject());
		workingAreaContainer->setWorkingArea(projectOverviewUi);
		workingAreaContainer->setNavigationArea(navigationUi);

		model->getActivityTracker()->trackChanged();
	}

	void edit() {
		projectWizard->editCommonProperties(model->getProject());
	}

	void edited() {
		/**
		 * We expect that the ProjectOverviewUi (as well as other ui components) has been subscribed for model changes so we don't update it here
		 */
		workingAreaContainer->setWorkingArea(projectOverviewUi);
		model->getActivityTracker()->trackChanged();
	}

	void save() {
		//TODO: FileDialogUi should provide a save method which sets and therefore translates the strings by itself

		QString projectFileName = FileDialogUi::getSaveFileName(QString("Project"));

		if(projectFileName.isEmpty()) {
			return;
		}


		try{
			SerializationPolicy::template serialize<P>(model->getProject(), projectFileName);
		}catch(std::exception &ex){
			ExceptionPolicy::handleSerializationException(mpl::Type2Type<P>(), projectFileName, ex.what());
			return;
		}

		model->getActivityTracker()->resetChanges();

	}

	template<typename Loader>
	void load(Loader *loader) {

		QString projectFileName = FileDialogUi::getOpenFileName("Project");

		if(projectFileName.isEmpty()) {
			return;
		}

		boost::shared_ptr<P> project; // note, all the object creation happens within the deserialization process.

		try {
			SerializationPolicy::template deserialize<P>(project, projectFileName);
		}catch(std::exception &ex){
			ExceptionPolicy::handleDeserializationException(mpl::Type2Type<P>(), projectFileName, ex.what());
			return;
		}

		model->setProject(project);

		projectOverviewUi->show(model->getProject());
		navigationUi->show(model->getProject());
		workingAreaContainer->setWorkingArea(projectOverviewUi); //show project overview on working area
		workingAreaContainer->setNavigationArea(navigationUi); //show navigation ui on working area


		//call loader
		loader->execute(project);

	}

	void show() {
		projectOverviewUi->show(model->getProject());
		workingAreaContainer->setWorkingArea(projectOverviewUi); //show project overview on working area
	}

	void deserialize(boost::shared_ptr<P> project) {
		std::cout << "ProjectController " <<  __FUNCTION__ << std::endl;
	}

	void modeChanged(int newMode) {
		model->getModeSettings()->getModeHandler()->setMode(newMode);
	}

private:
	PF *projectFactory;
	PW *projectWizard;
	Model *model;
	WorkingAreaContainer *workingAreaContainer;
	ProjectOverviewUi *projectOverviewUi;
	NavigationUi *navigationUi;
};

} /* namespace controller */


#endif /* PROJECTCONTROLLER_HPP_ */
