/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelController.hpp
 *
 *  Created on: Feb 21, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELCONTROLLER_HPP_
#define REGRESSIONMODELCONTROLLER_HPP_

#include <stdexcept>
#include <algorithm>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <QString>
#include <QDebug>
#include "applicationcontext/scopes.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "GenericController.hpp"

namespace controller {

template<
	typename RegressionModel,
	typename RegressionModelAnalysisToolsDescription,
	typename ActivityTracker,
	typename AnalysisToolsController,
	typename UiManager,
	typename InternalGenericController = GenericController<RegressionModel, UiManager>
>
class RegressionModelController {
public:
	RegressionModelController(
			boost::shared_ptr<ActivityTracker> activityTracker,
			InternalGenericController *genericController,
			AnalysisToolsController *analysisToolsController,
			boost::shared_ptr<RegressionModelAnalysisToolsDescription> analysisToolsDescription,
			UiManager *uiManager)
		:activityTracker(activityTracker), genericController(genericController), analysisToolsController(analysisToolsController), analysisToolsDescription(analysisToolsDescription), uiManager(uiManager)
	{;}

	virtual ~RegressionModelController() {;}

	template
	<
		typename Analysis,
		typename SubscriptionManager
	>
	void createFor(boost::shared_ptr<Analysis> analysis, boost::function<RegressionModel*(boost::shared_ptr<typename Analysis::DataModel_t>, boost::shared_ptr<Analysis>)> factory, SubscriptionManager *subscriptionManager) {
		using namespace boost;

		if(analysis->getDataModel() == NULL) {
			throw std::logic_error("please first load data model");
		}

		genericController->createFor(
				bind(&RegressionModelController<RegressionModel,RegressionModelAnalysisToolsDescription, ActivityTracker, AnalysisToolsController, UiManager, InternalGenericController>::template postAcceptedCreate<Analysis, SubscriptionManager>, this, analysis, _1, subscriptionManager),
				bind(factory, analysis->getDataModel(), analysis),
				subscriptionManager
		);
	}

	template
	<
		typename Analysis,
		typename SubscriptionManager
	>
	void remove(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<Analysis> analysis, SubscriptionManager *subscriptionManager) {
		using namespace app::context;
		using namespace boost;
		typedef typename UiManager::info_dialog_ui_type info_dialog_ui_t;

		LocalScope<info_dialog_ui_t> infoDialog(uiManager->getInfoDialogUiFor(regressionModel), bind<void>(&UiManager::destroyInfoDialogUiFor, uiManager, regressionModel));

		if(infoDialog->confirmRemoveQuestionFor(regressionModel) == false) {
			return;
		}
		//first, unsubscribe from the subscriptionManager

		//then:
		uiManager->destroyOverviewUiFor(regressionModel);

		//TODO: destroy all ATs from their UiManagers
		analysis->removeRegressionModel(regressionModel);

		activityTracker->trackChanged();
	}

	void setupBestFittingRegressionFormulaFor(boost::shared_ptr<RegressionModel> regressionModel) {
		//it might happen that we have to provide an empty RegressionFormula created from a factory
		//or what's even better is when the factory responsible for creating the regressionModel would create a dedicated empty RegressionFormula  (forced by the regression model's ctor which requires a regressionFormula object) (forced by the regression model's ctor which requires a regressionFormula object)
		regressionModel->setupBestFittingRegressionFormula();
	}

	void setName(boost::shared_ptr<RegressionModel> regressionModel, QString &name) {
		regressionModel->setName(name);

		activityTracker->trackChanged();
	}

	void setDescription(boost::shared_ptr<RegressionModel> regressionModel, QString &description) {
		regressionModel->setDescription(description);

		activityTracker->trackChanged();
	}

	void fromDateChanged(boost::shared_ptr<RegressionModel> regressionModel, int dateColumnIndex) {
		regressionModel->setFromDate(dateColumnIndex);

		activityTracker->trackChanged();
	}

	void toDateChanged(boost::shared_ptr<RegressionModel> regressionModel, int dateColumnIndex) {
		regressionModel->setToDate(dateColumnIndex);

		activityTracker->trackChanged();
	}

	void behaviorIndicatorChanged(boost::shared_ptr<RegressionModel> regressionModel, int behaviorIndicatorColumnIndex) {
		regressionModel->setBehaviorIndicatorColumn(behaviorIndicatorColumnIndex);

		activityTracker->trackChanged();
	}

	template<typename Analysis, typename SubscriptionManager>
	void postAcceptedCreate(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		calculateRegressionFit(regressionModel);
		analysis->addRegressionModel(regressionModel);

		subscribeDelegates(analysis, regressionModel, subscriptionManager);

		instantiateAllAnalysisTools(analysis, regressionModel, subscriptionManager);

		activityTracker->trackChanged();
	}

	template<typename Analysis, typename SubscriptionManager>
	void instantiateAllAnalysisTools(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		typedef typename RegressionModelAnalysisToolsDescription::AnalysisToolDescription_t AnalysisToolDescription_t;
		//left side physics
		BOOST_FOREACH(boost::shared_ptr<AnalysisToolDescription_t> analysisToolDescription, analysisToolsDescription->getAutomaticLeftSidePhysicalCategoryAnalysisToolDescriptions()) {
			analysisToolsController->addAnalysisTool(
					analysis,
					regressionModel,
					analysisToolDescription,
					uiManager->getLeftSidePhysicsCategoryWidgetFor(regressionModel, subscriptionManager)->getUiLocationCallback()
				);
		}
		//left side statistics
		BOOST_FOREACH(boost::shared_ptr<AnalysisToolDescription_t> analysisToolDescription, analysisToolsDescription->getAutomaticLeftSideStatisticsCategoryAnalysisToolDescriptions()) {
			analysisToolsController->addAnalysisTool(
					analysis,
					regressionModel,
					analysisToolDescription,
					uiManager->getLeftSideStatisticsCategoryWidgetFor(regressionModel, subscriptionManager)->getUiLocationCallback()
				);
		}

		//right side physics
		BOOST_FOREACH(boost::shared_ptr<AnalysisToolDescription_t> analysisToolDescription, analysisToolsDescription->getAutomaticRightSidePhysicalCategoryAnalysisToolDescriptions()) {
			analysisToolsController->addAnalysisTool(
					analysis,
					regressionModel,
					analysisToolDescription,
					uiManager->getPhysicsCategoryWidgetFor(regressionModel, subscriptionManager)->getUiLocationCallback()
				);
		}
		//right side statistics
		BOOST_FOREACH(boost::shared_ptr<AnalysisToolDescription_t> analysisToolDescription, analysisToolsDescription->getAutomaticRightSideStatisticsCategoryAnalysisToolDescriptions()) {
			analysisToolsController->addAnalysisTool(
					analysis,
					regressionModel,
					analysisToolDescription,
					uiManager->getStatisticsCategoryWidgetFor(regressionModel, subscriptionManager)->getUiLocationCallback()
				);
		}
	}

	template<typename Analysis, typename SubscriptionManager>
	void show(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		bool containsOverviewUi = uiManager->containsOverviewUiFor(regressionModel);

		genericController->show(regressionModel, subscriptionManager); //this instantiates the overview ui, so the next time we don't delegate subscriptions
		if(containsOverviewUi == false) {
			//we come here for example after clicking on the regression model item of the navigation tree if we have loaded by model by deserialization
			calculateRegressionFit(regressionModel);
			subscribeDelegates(analysis, regressionModel, subscriptionManager);
			instantiateAllAnalysisTools(analysis, regressionModel, subscriptionManager);
		}
	}

	template<typename Analysis, typename SubscriptionManager>
	void subscribeDelegates(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		subscriptionManager->subscribeDelegates(uiManager->getPhysicsCategoryWidgetFor(regressionModel, subscriptionManager), analysis, regressionModel);
		subscriptionManager->subscribeDelegates(uiManager->getStatisticsCategoryWidgetFor(regressionModel, subscriptionManager), analysis, regressionModel);
		subscriptionManager->subscribeDelegates(uiManager->getLeftSidePhysicsCategoryWidgetFor(regressionModel, subscriptionManager), analysis, regressionModel);
		subscriptionManager->subscribeDelegates(uiManager->getLeftSideStatisticsCategoryWidgetFor(regressionModel, subscriptionManager), analysis, regressionModel);
	}

	template<typename SubscriptionManager>
	void editRegressionFormula(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace app::context;
		using namespace boost;

		LocalScope<typename UiManager::wizard_ui_t> wizard(uiManager->getWizardFor(regressionModel, subscriptionManager), bind<void>(&UiManager::destroyWizardFor, uiManager, regressionModel));
		wizard->editRegressionFormula();

		if(wizard->accepted()) {
			calculateRegressionFit(regressionModel);
		}
	}

	template<typename SubscriptionManager>
	void editAlgorithm(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace app::context;
		using namespace boost;

		LocalScope<typename UiManager::wizard_ui_t> wizard(uiManager->getWizardFor(regressionModel, subscriptionManager), bind<void>(&UiManager::destroyWizardFor, uiManager, regressionModel));
		wizard->editAlgorithm();

		if(wizard->accepted()) {
			calculateRegressionFit(regressionModel);
		}
	}

	template<typename SubscriptionManager>
	void editTimeRange(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace app::context;
		using namespace boost;

		LocalScope<typename UiManager::wizard_ui_t> wizard(uiManager->getWizardFor(regressionModel, subscriptionManager), bind<void>(&UiManager::destroyWizardFor, uiManager, regressionModel));
		wizard->editTimeRange();

		if(wizard->accepted()) {
			calculateRegressionFit(regressionModel);
		}
	}

	template<typename SubscriptionManager>
	void editCommonParameters(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace app::context;
		using namespace boost;

		LocalScope<typename UiManager::wizard_ui_t> wizard(uiManager->getWizardFor(regressionModel, subscriptionManager), bind<void>(&UiManager::destroyWizardFor, uiManager, regressionModel));
		wizard->editCommonParameters();

		//NOTE: we don't have to recalculate the regression as we don't change any regression related parameters here
		//we might change this in the future but up to now, I guess it would irritate the user as it means that the plots will change as well (at least in the focus)
	}

	template<typename SubscriptionManager>
	void editBehaviorIndicator(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace app::context;
		using namespace boost;

		LocalScope<typename UiManager::wizard_ui_t> wizard(uiManager->getWizardFor(regressionModel, subscriptionManager), bind<void>(&UiManager::destroyWizardFor, uiManager, regressionModel));
		wizard->editBehaviorIndicator();

		if(wizard->accepted()) {
			calculateRegressionFit(regressionModel);
		}
	}


	void calculateRegressionFit(boost::shared_ptr<RegressionModel> regressionModel) {
		uiManager->getProgressManager()->showOperationInProgress(boost::bind(&RegressionModel::calculateRegressionFit, regressionModel));
	}

	template<typename ModelDefinition, typename Factory, typename SubscriptionManager>
	void addRegressor(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<typename RegressionModel::Regression_Formula_t> regressionFormula, boost::shared_ptr<ModelDefinition> modelDefinition, QString formulaBuilderIdentifier, Factory *factory, SubscriptionManager *subscriptionManager, QString instantiationMessage = QString("")) {
		using namespace app::context;
		using namespace boost;

		boost::shared_ptr<typename ModelDefinition::RegressorFormulaBuilder_t> formulaBuilder = modelDefinition->findRegressorFormulaBuilderByIdentifier(formulaBuilderIdentifier);
		if(formulaBuilder == NULL) {
			throw std::logic_error("no formula builder found for " + formulaBuilderIdentifier.toStdString());
		}

		boost::shared_ptr<typename ModelDefinition::Regressor_t> regressor(factory->template create<typename ModelDefinition::Regressor_t>(formulaBuilder, modelDefinition->getDataFrame()));
		if(formulaBuilder->requiresInfluenceFactor()) {
			if(regressor->getAvailableInfluenceFactors().empty()) {
				throw std::logic_error(std::string("This regressor requires an influence factor of type ") + formulaBuilder->getAssignmentCategory()->getName().toStdString() + std::string(" but there is no one assigned"));
			}

			regressor->setInfluenceFactor(regressor->getAvailableInfluenceFactors().front());

			if(regressor->getAvailableInfluenceFactors().size() > 1) {
				LocalScope<typename UiManager::influence_factor_selection_ui_t> influenceFactorSelectionUi(
							uiManager->getInfluenceFactorSelectionUiFor(regressor, subscriptionManager),
							bind<void>(&UiManager::destroyInfluenceFactorSelectionUiFor, uiManager, regressor)
						);
				influenceFactorSelectionUi->exec();
			}
		}

		//TODO: test if a regressor with such a formula already exists
		if(regressionFormula->containsRegressor(regressor)) {
			boost::shared_ptr<ModelDefinition> modelDefinitionContainingTheRegressor = regressionFormula->getRegressionModelDefinitionForRegressor(regressor);
			if(modelDefinitionContainingTheRegressor == NULL) {
				throw std::logic_error("Cannot add regressor. Expected to find a regression model definition containing regressor: " + regressor->getFormula().toStdString());
			}
			throw std::logic_error("Regressor : " + regressor->getFormula().toStdString() + " already exists in \"" + modelDefinitionContainingTheRegressor->getName().toStdString() + "\"");
		}

		//show any instantiation messages
		qDebug() << __FUNCTION__ << " : 0";
		if(!instantiationMessage.isEmpty()) {
			qDebug() << "##### " << __FUNCTION__ << " : 1";
			LocalScope<typename UiManager::regressor_influence_factor_instantiation_message_ui_t> instantiationMessageUi(
				uiManager->getRegressorInfluenceFactorInstantiationMessageUiFor(regressor),
				bind<void>(&UiManager::destroyRegressorInfluenceFactorInstantiationMessageUiFor, uiManager, regressor)
			);

			instantiationMessageUi->execWithMessage(instantiationMessage);
		}

		modelDefinition->addRegressor(regressor);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged(); //note: we track changes only if no exception was thrown!
	}

	/**
	 * Adds a regressor with a given (preselected) influenceFactor (required by template instantiations where the template defines all the influence factors needed for a given regressor of a certain formulaBuilder
	 */
	template<typename ModelDefinition, typename Factory, typename SubscriptionManager>
	void addRegressor(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<typename RegressionModel::Regression_Formula_t> regressionFormula, boost::shared_ptr<ModelDefinition> modelDefinition, boost::shared_ptr<typename ModelDefinition::Regressor_t::Influence_Factor_t> influenceFactor, QString formulaBuilderIdentifier, Factory *factory, SubscriptionManager *subscriptionManager, QString instantiationMessage = QString("")) {
		using namespace app::context;

		typedef typename ModelDefinition::Regressor_t::Influence_Factor_t Influence_Factor_t;

		boost::shared_ptr<typename ModelDefinition::RegressorFormulaBuilder_t> formulaBuilder = modelDefinition->findRegressorFormulaBuilderByIdentifier(formulaBuilderIdentifier);
		if(formulaBuilder == NULL) {
			throw std::logic_error("no formula builder found for " + formulaBuilderIdentifier.toStdString());
		}

		boost::shared_ptr<typename ModelDefinition::Regressor_t> regressor(factory->template create<typename ModelDefinition::Regressor_t>(formulaBuilder, modelDefinition->getDataFrame()));
		if(formulaBuilder->requiresInfluenceFactor()) {
			using namespace app::context;
			using namespace boost;

			if(regressor->getAvailableInfluenceFactors().empty()) {
				throw std::logic_error(std::string("This regressor requires an influence factor of type ") + formulaBuilder->getAssignmentCategory()->getName().toStdString() + std::string(" but there is no one assigned"));
			}

		}

		regressor->setInfluenceFactor(influenceFactor); //test on whether this is valid or not has to be done by the regressor

		//TODO: test if a regressor with such a formula already exists
		if(regressionFormula->containsRegressor(regressor)) {
			boost::shared_ptr<ModelDefinition> modelDefinitionContainingTheRegressor = regressionFormula->getRegressionModelDefinitionForRegressor(regressor);
			if(modelDefinitionContainingTheRegressor == NULL) {
				throw std::logic_error("Cannot add regressor. Expected to find a regression model definition containing regressor: " + regressor->getFormula().toStdString());
			}
			throw std::logic_error("Regressor : " + regressor->getFormula().toStdString() + " already exists in \"" + modelDefinitionContainingTheRegressor->getName().toStdString() + "\"");
		}

		modelDefinition->addRegressor(regressor);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged(); //note: we track changes only if no exception was thrown!
	}

	template<typename Factory, typename SubscriptionManager>
	void instantiateModelTemplate(
			boost::shared_ptr<RegressionModel> regressionModel,
			boost::shared_ptr<typename RegressionModel::Regression_Formula_t::RegressionModelTemplate_t> templateModelDescription,
			Factory *factory,
			SubscriptionManager *subscriptionManager)
	{
		using namespace app::context;

		typedef typename RegressionModel::Regression_Formula_t::RegressionModelDefinition_t RegressionModelDefinition_t;
		typedef typename RegressionModelDefinition_t::RegressorFormulaBuilder_t RegressorFormulaBuilder_t;

		boost::shared_ptr<RegressionModelDefinition_t> modelDefinition(factory->template create<RegressionModelDefinition_t>(regressionModel->getDataModel(), templateModelDescription->getName())); //factory needs to get DataFrame from DataModel
		modelDefinition->freezeModification();

		//TODO: this is a hack and should be handled in a general way supported by the modelDefinition structure (e.g. a special category for 'getForeachColumnRegressorFormulaBuilders()'
		if(templateModelDescription->getMethodId() == "de467a4a-1697-4a42-9878-e61c21ffe5d5") { //this is the htt case and needs special treatment
			if(regressionModel->getDataModel()->getTemperatureColumns().size() == 0) {
				throw std::logic_error("There are no temperature columns assigned which is a requirement for the HTT model.");
			}
			BOOST_FOREACH(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, templateModelDescription->getRegressorFormulaBuilders()) {
				if(regressorFormulaBuilder->getIdentifier() == "direct temperature") {
					BOOST_FOREACH(boost::shared_ptr<typename RegressionModel::DataModel_t::Column_t> temperatureColumn, regressionModel->getDataModel()->getTemperatureColumns()) {
						//add a regressor for each temperature column with the given regressorFormulaBuilder (direct temperature)
						modelDefinition->addRegressorFormulaBuilder(regressorFormulaBuilder);
						addRegressor(regressionModel, regressionModel->getRegressionFormula(), modelDefinition, temperatureColumn, regressorFormulaBuilder->getIdentifier(), factory, subscriptionManager);
					}
				}else{
					modelDefinition->addRegressorFormulaBuilder(regressorFormulaBuilder);
					addRegressor(regressionModel, regressionModel->getRegressionFormula(), modelDefinition, regressorFormulaBuilder->getIdentifier(), factory, subscriptionManager);
				}
			}
		}else{
			BOOST_FOREACH(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, templateModelDescription->getRegressorFormulaBuilders()) {
				QString instantiationMessage;
				if(templateModelDescription->hasInstantiationMessageFor(regressorFormulaBuilder)) {
					instantiationMessage = templateModelDescription->resolveInstantiationMessageFor(regressorFormulaBuilder);
				}
				modelDefinition->addRegressorFormulaBuilder(regressorFormulaBuilder);
				addRegressor(regressionModel, regressionModel->getRegressionFormula(), modelDefinition, regressorFormulaBuilder->getIdentifier(), factory, subscriptionManager, instantiationMessage);
			}
		}

		regressionModel->getRegressionFormula()->addTemplatedRegressionModelDefinition(modelDefinition);

		activityTracker->trackChanged(); //note: we track changes only if no exception was thrown!
	}

	void releaseForModification(
			boost::shared_ptr<typename RegressionModel::Regression_Formula_t> regressionFormula,
			boost::shared_ptr<typename RegressionModel::Regression_Formula_t::RegressionModelDefinition_t> modelDefinition)
	{
		regressionFormula->releaseRegressionModelDefinitionForModification(modelDefinition);

		activityTracker->trackChanged();
	}

	void removeRegressionModelDefinition(
			boost::shared_ptr<typename RegressionModel::Regression_Formula_t> regressionFormula,
			boost::shared_ptr<typename RegressionModel::Regression_Formula_t::RegressionModelDefinition_t> modelDefinition)
	{
		uiManager->destroyRegressionModelDefinitionEditUiFor(modelDefinition);
		regressionFormula->removeRegressionModelDefinition(modelDefinition);

		activityTracker->trackChanged();
	}

	template<typename ModelDefinition>
	void removeRegressor(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<ModelDefinition> modelDefinition, boost::shared_ptr<typename ModelDefinition::Regressor_t> regressor) {
		modelDefinition->removeRegressor(regressor);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged();
	}

	template<typename Regressor>
	void setRegressorInfluenceFactor(boost::shared_ptr<Regressor> regressor, int influenceFactorIndex) {
		regressor->setInfluenceFactor(influenceFactorIndex);

		activityTracker->trackChanged();
	}

	/**
	 * RegressorFormulaBuilder related
	 */
	template<typename RegressorFormulaBuilder>
	void setRegressorFormulaBuilderIdentifier(boost::shared_ptr<RegressorFormulaBuilder> formulaBuilder, QString identifier) {
		formulaBuilder->setIdentifier(identifier);

		activityTracker->trackChanged();
	}

	template<typename RegressorFormulaBuilder>
	void setRegressorFormulaBuilderFormula(boost::shared_ptr<RegressorFormulaBuilder> formulaBuilder, QString formula) {
		formulaBuilder->setFormula(formula);

		activityTracker->trackChanged();
	}


	template<typename RegressorFormulaBuilder>
	void setRegressorFormulaBuilderAssignmentCategory(boost::shared_ptr<RegressorFormulaBuilder> formulaBuilder, QString identifier) {
		if(formulaBuilder->findAssignmentCategoryByIdentifier(identifier) == NULL) {
			throw std::logic_error("No assignment category found for identifier " + identifier.toStdString());
		}

		formulaBuilder->setAssignmentCategory(formulaBuilder->findAssignmentCategoryByIdentifier(identifier));

		activityTracker->trackChanged();
	}

	template<typename RegressorFormulaBuilderPreprocessor>
	void setRegressorFormulaBuilderPreprocessorIsActive(boost::shared_ptr<RegressorFormulaBuilderPreprocessor> preprocessor, bool activeState) {
		preprocessor->setIsActive(activeState);

		activityTracker->trackChanged();
	}


	template<typename RegressorFormulaBuilder>
	void setRegressorFormulaBuilderFormulaDefinition(boost::shared_ptr<RegressorFormulaBuilder> formulaBuilder, QString formulaDefinition) {
		formulaBuilder->setFormulaDefinition(formulaDefinition);

		activityTracker->trackChanged();
	}

	template<
		typename RegressorFormulaBuilder,
		typename Factory
	>
	void deriveRegressorFormulaBuilder(boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder, boost::shared_ptr<RegressionModel> regressionModel, Factory *factory) {
		boost::shared_ptr<RegressorFormulaBuilder> derivedFormulaBuilder(factory->template create<RegressorFormulaBuilder>(
					regressorFormulaBuilder,
					regressionModel->getRegressionFormula()->getNewIdentifierFor(regressorFormulaBuilder),
					regressionModel->getRegressionFormula()->getNewFormulaFor(regressorFormulaBuilder)
				));

		regressionModel->getRegressionFormula()->addRegressorFormulaBuilder(derivedFormulaBuilder);

		activityTracker->trackChanged();
	}

	template<
			typename RegressorFormulaBuilder,
			typename Factory
		>
	void createUserParameter(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder, Factory *factory) {
		using namespace app::context;
		using namespace boost;
		typedef typename RegressorFormulaBuilder::UserParameterType_t user_parameter_type_t;

		boost::shared_ptr<user_parameter_type_t> userParameterType(factory->template create<user_parameter_type_t>());

		LocalScope<typename UiManager::user_parameter_name_ui_t> userParameterUi(uiManager->getUserParameterNameUiFor(regressorFormulaBuilder), bind<void>(&UiManager::destroyUserParameterNameUiFor, uiManager, regressorFormulaBuilder));
		QString name = userParameterUi->getValue();

		regressorFormulaBuilder->addUserParameter(name, userParameterType);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged();
	}

	template<typename RegressorFormulaBuilder>
	void removeUserParameter(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder, typename RegressorFormulaBuilder::UserParameter_t userParameter) {
		regressorFormulaBuilder->removeUserParameter(userParameter);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged();
	}

	template<typename UserParameter>
	void setUserParameterValue(boost::shared_ptr<RegressionModel> regressionModel, UserParameter userParameter, QString value) {
		userParameter.second->setValue(value);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged();
	}

	template<typename UserParameter>
	void setUserParameterIsMinimumLeadTime(boost::shared_ptr<RegressionModel> regressionModel, UserParameter userParameter, bool state) {
		userParameter.second->setIsMinimumLeadTime(state);
		regressionModel->updateFromFormulaChanges();

		activityTracker->trackChanged();
	}

	void algorithmChanged(boost::shared_ptr<RegressionModel> regressionModel, int algorithm) {
		regressionModel->setAlgorithm(algorithm);

		activityTracker->trackChanged();
	}

	void setPraisWinstenIterations(boost::shared_ptr<RegressionModel> regressionModel, int iterations) {
		regressionModel->setPraisWinstenIterations(iterations);

		activityTracker->trackChanged();
	}

	void setRidgeKValue(boost::shared_ptr<RegressionModel> regressionModel, double k) {
		regressionModel->setRidgeKValue(k);

		activityTracker->trackChanged();
	}

	void setPCRrValue(boost::shared_ptr<RegressionModel> regressionModel, unsigned r) {
		regressionModel->setPCRrValue(r);

		activityTracker->trackChanged();
	}

private:
	boost::shared_ptr<ActivityTracker> activityTracker;
	InternalGenericController *genericController;
	AnalysisToolsController *analysisToolsController;
	boost::shared_ptr<RegressionModelAnalysisToolsDescription> analysisToolsDescription;
	UiManager *uiManager;
};

} /* namespace controller */

#endif /* REGRESSIONMODELCONTROLLER_HPP_ */
