/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportController.hpp
 *
 *  Created on: Jul 10, 2014
 *      Author: gerberma
 */

#ifndef REPORTCONTROLLER_HPP_
#define REPORTCONTROLLER_HPP_

#include <QDebug>
#include <boost/foreach.hpp>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "applicationcontext/scopes.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "GenericController.hpp"

namespace controller {

template<
	typename Report,
	typename UiManager,
	typename Factory,
	typename InternalGenericController
>
class ReportController {
public:
	ReportController(InternalGenericController *genericController, UiManager *uiManager, Factory *factory)
		:genericController(genericController), uiManager(uiManager), factory(factory)
	{;}

	template<
		typename Project,
		typename Analysis,
		typename RegressionModel,
		typename SubscriptionManager
	>
	void createFor(
			boost::shared_ptr<Project> project,
			boost::shared_ptr<Analysis> analysis,
			boost::shared_ptr<RegressionModel> regressionModel,
			/* boost::function<Report*(boost::shared_ptr<Project>, boost::shared_ptr<Analysis>, boost::shared_ptr<RegressionModel>)> factory, */
			SubscriptionManager *subscriptionManager)
	{
		using namespace boost;

		//Report *report = factory(project, analysis, regressionModel);
		genericController->createFor(
					bind(&ReportController<Report, UiManager, Factory, InternalGenericController>::nofunction, this, _1),
					bind((Report *(Factory::*)(boost::shared_ptr<Project>, boost::shared_ptr<Analysis>, boost::shared_ptr<RegressionModel>))&Factory::template create<Report, Project, Analysis, RegressionModel>, factory, project, analysis, regressionModel),
					subscriptionManager,
					bind(&ReportController<Report, UiManager, Factory, InternalGenericController>::template setupReportListElements<Project, Analysis, RegressionModel>, this, _1, project, analysis, regressionModel)
				);
	}

	void nofunction(boost::shared_ptr<Report>) {
	}


	void build(boost::shared_ptr<Report> report) {
		using namespace app::context;
		using namespace boost;
		//get destination from user input

		typedef typename UiManager::save_dialog_ui_t save_dialog_ui_t;
		LocalScope<save_dialog_ui_t> saveDialog(uiManager->getSaveDialogUiFor(report), bind<void>(&UiManager::destroySaveDialogUiFor, uiManager, report));
		QString reportLocation = saveDialog->getSaveLocation();

		if(reportLocation.isEmpty()) {
			return;
		}

		//build report
		report->setOutputFileName(reportLocation);
		report->build();
		bool state = report->write();
		if(state == true) {
			if(report->hasFailed()) {
				report->writeFailed();
			}else{
				report->writeSuccessful();
			}
		}else{
			//TODO: log data such as process' exit code and std error
			report->writeTimedOut();
		}
	}

	template<
			typename Project,
			typename Analysis,
			typename RegressionModel
	>
	void setupReportListElements(boost::shared_ptr<Report> report,
			boost::shared_ptr<Project> project,
			boost::shared_ptr<Analysis> analysis,
			boost::shared_ptr<RegressionModel> regressionModel)
	{
		int index = 0;
		using namespace boost;

		typename Report::ReportListElementContainer_t reportListElements;

		shared_ptr<typename Report::TitleReportListElement_t> titleListElement(factory->template create<typename Report::TitleReportListElement_t>(index++));
		reportListElements.push_back(titleListElement);
		report->setTitleReportListElement(titleListElement);

		shared_ptr<typename Report::DescriptionReportListElement_t> descriptionListElement(factory->template create<typename Report::DescriptionReportListElement_t>(index++));
		reportListElements.push_back(descriptionListElement);
		report->setDescriptionReportListElement(descriptionListElement);


		shared_ptr<typename Report::ProjectReportListElement_t> projectReportListElement(factory->template create<typename Report::ProjectReportListElement_t, Project, Analysis, RegressionModel>(project, analysis, regressionModel, index++));
		reportListElements.push_back(projectReportListElement);
		report->setProjectReportListElement(projectReportListElement);

		shared_ptr<typename Report::AlgorithmReportListElement_t> algorithmReportListElement(factory->template create<typename Report::AlgorithmReportListElement_t, RegressionModel>(regressionModel, index++));
		reportListElements.push_back(algorithmReportListElement);
		report->setAlgorithmReportListElement(algorithmReportListElement);

		typename Report::AnalysisToolListElementContainer_t analysisToolListElements;
		typedef typename RegressionModel::AnalysisTool_t AnalysisTool_t;
		BOOST_FOREACH(boost::shared_ptr<AnalysisTool_t> analysisTool, regressionModel->getAnalysisTools()) {
			if(analysisTool->supportsReporting() == true) {
				boost::shared_ptr<typename Report::AnalysisToolReportListElement_t> analysisToolListElement(factory->template create<typename Report::AnalysisToolReportListElement_t, AnalysisTool_t>(analysisTool, index++));
				reportListElements.push_back(analysisToolListElement);
				analysisToolListElements.push_back(analysisToolListElement);
			}
		}
		report->setAnalysisToolReportListElements(analysisToolListElements);

		report->setReportListElements(reportListElements);
	}

	template<typename TextReportListElement>
	void setText(boost::shared_ptr<TextReportListElement> textReportListElement, QString text) {
		textReportListElement->setText(text);
	}

	template<typename ReportListElement>
	void setSelectState(boost::shared_ptr<ReportListElement> reportListElement, bool state) {
		reportListElement->setIsSelected(state);
	}

	template<typename ReportListElement>
	void moveUp(boost::shared_ptr<Report> report, boost::shared_ptr<ReportListElement> reportListElement) {
		if(report == NULL) {
			qDebug() << __FUNCTION__ << " - report == NULL";
			return;
		}
		if(reportListElement == NULL) {
			qDebug() << __FUNCTION__ << " - reportListElement == NULL";
			return;
		}
		report->moveUp(reportListElement);
	}

	template<typename ReportListElement>
	void moveDown(boost::shared_ptr<Report> report, boost::shared_ptr<ReportListElement> reportListElement) {
		if(report == NULL) {
			qDebug() << __FUNCTION__ << " - report == NULL";
			return;
		}
		if(reportListElement == NULL) {
			qDebug() << __FUNCTION__ << " - reportListElement == NULL";
			return;
		}
		report->moveDown(reportListElement);
	}

	void setPlotScaleFactor(boost::shared_ptr<Report> report, double scaleFactor) {
		report->setPlotScaleFactor(scaleFactor);
	}

	void setFontSize(boost::shared_ptr<Report> report, int fontSize) {
		report->setFontSize(fontSize);
	}

private:
	InternalGenericController *genericController;
	UiManager *uiManager;
	Factory *factory;
};

}  // namespace controller


#endif /* REPORTCONTROLLER_HPP_ */
