/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisCategoryGenericWidget.hpp
 *
 *  Created on: Jan 21, 2014
 *      Author: gerberma
 */

#ifndef ANALYSISCATEGORYGENERICWIDGET_HPP_
#define ANALYSISCATEGORYGENERICWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QPushButton>
#include <QSpacerItem>
#include <QScrollArea>
#include <vector>
#include <boost/function.hpp>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include "Delegates.hpp"
#include "communication/Signals.hpp"
#include "ComboBox.hpp"
#include "MoveToolbarButton.hpp"
#include "AnalysisToolWidget.hpp"

namespace gui {

template<typename AnalysisCategoryProvider>
class AnalysisCategoryGenericWidget : public QWidget {
public:
	AnalysisCategoryGenericWidget(AnalysisCategoryProvider *analysisCategoryWidget, QWidget *parent = 0);
	virtual ~AnalysisCategoryGenericWidget();

	COM_SUBSCRIPTION_FUNCTION(AddAnalysisTool, communication::Signal<
					void(boost::shared_ptr<typename AnalysisCategoryProvider::tool_description_t>, boost::function<void(AnalysisToolWidget *)>)
				>);

	QString getCategoryName() {
		return analysisCategoryProvider->getCategoryName();
	}

	boost::function<void(AnalysisToolWidget *)> getUiLocationCallback() {
		return boost::bind(&AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::addAnalysisToolWidgetCallback, this, _1);
	}

private:
	void setupLayout();
	void connectLocal();
	void updateValues();

	void addAnalysisToolWidgetCallback(AnalysisToolWidget *analysisToolWidget);
	void addAnalysisToolButtonClicked();

	void moveAnalysisToolWidgetUp(AnalysisToolWidget *analysisToolWidget);
	void moveAnalysisToolWidgetDown(AnalysisToolWidget *analysisToolWidget);

private:
	AnalysisCategoryProvider *analysisCategoryProvider;
	QVBoxLayout *vboxLayout;
	QScrollArea *analysisToolsScrollArea;
	QWidget *analysisToolsContainerWidget;
	QVBoxLayout *analysisToolsContainerWidgetLayout;
	//QPushButton *configureComparisonButton;
	MoveToolbarButton *addButton;
	ComboBox *chooseAnalysisToolComboBox;
	COM_CONNECTION modeChangedConnection;
};

template<typename AnalysisCategoryProvider>
inline AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::AnalysisCategoryGenericWidget(AnalysisCategoryProvider* analysisCategoryWidget, QWidget* parent)
	:QWidget(parent), analysisCategoryProvider(analysisCategoryWidget)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename AnalysisCategoryProvider>
inline AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::~AnalysisCategoryGenericWidget() {
}

template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::addAnalysisToolWidgetCallback(AnalysisToolWidget *analysisToolWidget) {
	using namespace boost;
	COM_SUBSCRIBE_FOR(analysisToolWidget->getToolBar(), MoveUpButtonClicked, bind(&AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::moveAnalysisToolWidgetUp, this, analysisToolWidget));
	COM_SUBSCRIBE_FOR(analysisToolWidget->getToolBar(), MoveDownButtonClicked, bind(&AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::moveAnalysisToolWidgetDown, this, analysisToolWidget));

	analysisToolWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);

	analysisToolsContainerWidgetLayout->insertWidget(0, analysisToolWidget);
	analysisToolsScrollArea->setWidgetResizable(true);
}

template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::moveAnalysisToolWidgetUp(AnalysisToolWidget *analysisToolWidget) {
	int index = analysisToolsContainerWidgetLayout->indexOf(analysisToolWidget);

	if(index > 0) {
		analysisToolsContainerWidgetLayout->removeWidget(analysisToolWidget);
		analysisToolsContainerWidgetLayout->insertWidget(index - 1, analysisToolWidget);
	}
}

template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::moveAnalysisToolWidgetDown(AnalysisToolWidget *analysisToolWidget) {
	int index = analysisToolsContainerWidgetLayout->indexOf(analysisToolWidget);

	if(index >= 0 && index < analysisToolsContainerWidgetLayout->count()) {
		analysisToolsContainerWidgetLayout->removeWidget(analysisToolWidget);
		analysisToolsContainerWidgetLayout->insertWidget(index + 1, analysisToolWidget);
	}
}



template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::setupLayout() {
	analysisToolsContainerWidgetLayout = new QVBoxLayout(this);
	analysisToolsContainerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);
	analysisToolsContainerWidgetLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));
	analysisToolsContainerWidget = new QWidget(this);
	analysisToolsContainerWidget->setLayout(analysisToolsContainerWidgetLayout);
	analysisToolsScrollArea = new QScrollArea(this);
	//analysisToolsScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	analysisToolsScrollArea->setWidget(analysisToolsContainerWidget);
	analysisToolsScrollArea->setWidgetResizable(true);
	analysisToolsScrollArea->setFrameShape(QFrame::NoFrame);

	/*
	configureComparisonButton = new QPushButton(tr("Configure comparison"), this);
	configureComparisonButton->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	configureComparisonButton->hide();
	*/
	addButton = new MoveToolbarButton("+", this);

	chooseAnalysisToolComboBox = new ComboBox(this);


	QGridLayout *gridLayout = new QGridLayout(this);
	gridLayout->addWidget(chooseAnalysisToolComboBox, 0, 0, Qt::AlignLeft);
	gridLayout->addWidget(addButton, 0, 1, Qt::AlignLeft);
	QSpacerItem *horizontalSpacer = new QSpacerItem(1,1,QSizePolicy::Expanding, QSizePolicy::Ignored);
	gridLayout->addItem(horizontalSpacer, 0, 1);
	//gridLayout->addWidget(configureComparisonButton, 0, 2, Qt::AlignRight);

	QWidget *toolsSelectionWidget = new QWidget(this);
	toolsSelectionWidget->setLayout(gridLayout);

	vboxLayout = new QVBoxLayout(this);
	vboxLayout->addWidget(toolsSelectionWidget);
	vboxLayout->addWidget(analysisToolsScrollArea);

	setLayout(vboxLayout);
}

template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::addAnalysisToolButtonClicked() {
	if(chooseAnalysisToolComboBox->currentIndex() == 0) {
		return;
	}

	QString currentToolId = chooseAnalysisToolComboBox->itemData(chooseAnalysisToolComboBox->currentIndex()).toString();

	typedef typename AnalysisCategoryProvider::tool_description_t tool_description_t;
	typename AnalysisCategoryProvider::container_t toolDescriptions = analysisCategoryProvider->getCategoryRelatedAnalysisToolsDescription();
	BOOST_FOREACH(boost::shared_ptr<tool_description_t> toolDescription, toolDescriptions) {
		if(toolDescription->getId() == currentToolId) {
			AddAnalysisToolSignal(toolDescription,
					boost::bind(&AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::addAnalysisToolWidgetCallback, this, _1));
			return;
		}
	}
}

template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::connectLocal() {
	using namespace boost;
	using namespace communication;

	communication::QtBoostDelegate *addButtonDelegate = new QtBoostDelegate(bind(&AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::addAnalysisToolButtonClicked, this), this);
	connect(addButton, SIGNAL(clicked()), addButtonDelegate, SLOT(execute()));

	modeChangedConnection = COM_SUBSCRIBE_FOR(analysisCategoryProvider->getAnalysisToolsDescription()->getModeHandler(), ModeChanged, bind(&AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::updateValues, this));
}

template<typename AnalysisCategoryProvider>
inline void AnalysisCategoryGenericWidget<AnalysisCategoryProvider>::updateValues() {
	typedef typename AnalysisCategoryProvider::tool_description_t tool_description_t;
	typename AnalysisCategoryProvider::container_t toolDescriptions = analysisCategoryProvider->getCategoryRelatedAnalysisToolsDescription();

	chooseAnalysisToolComboBox->clear();
	chooseAnalysisToolComboBox->addItem(tr("choose analysis tool"), -1);
	chooseAnalysisToolComboBox->disableSelectionFor(0);
	chooseAnalysisToolComboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	BOOST_FOREACH(boost::shared_ptr<tool_description_t> toolDescription, toolDescriptions) {
		chooseAnalysisToolComboBox->addItem(toolDescription->getName(), toolDescription->getId());
	}
}

} /* namespace gui */

#endif /* ANALYSISCATEGORYGENERICWIDGET_HPP_ */
