/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisOverviewWidget.hpp
 *
 *  Created on: Nov 29, 2013
 *      Author: gerberma
 */

#ifndef ANALYSISOVERVIEWWIDGET_HPP_
#define ANALYSISOVERVIEWWIDGET_HPP_

#include <QWidget>
#include <QObject>
#include <QLabel>
#include <QPushButton>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QSpacerItem>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/signals2/signal.hpp>
#include "communication/Signals.hpp"
#include "Delegates.hpp"
#include <iostream>
#include <boost/shared_ptr.hpp>

namespace gui {

namespace priv {

class AnalysisOverviewWidgetSignalsSlots : public QWidget {
	Q_OBJECT
private slots:
	virtual void qtEditSlot() = 0;
};

class DirtyAnalysisOverviewWidgetBaseClass : public AnalysisOverviewWidgetSignalsSlots {
};

} /* namespace priv */


template<typename A>
class AnalysisOverviewWidget : public priv::DirtyAnalysisOverviewWidgetBaseClass {
public:
	typedef boost::signals2::signal<void(boost::shared_ptr<A>)> edit_signal_t;
	COM_SUBSCRIPTION_FUNCTION(EditWaterLevelScaling, communication::Signal<void(boost::shared_ptr<A>)>);
public:
	AnalysisOverviewWidget(boost::shared_ptr<A>analysis);
	virtual ~AnalysisOverviewWidget();


	boost::signals2::connection subscribeForEdit(const typename edit_signal_t::slot_type &subscriber);

private:
	void setupLayout();
	void updateValues();
	void qtEditSlot();

private:
	boost::shared_ptr<A> analysis;
	QLabel *nameLabel;
	QLabel *nameValueLabel;
	QLabel *descriptionLabel;
	QLabel *descriptionValueLabel;
	QLabel *damOperatorLabel;
	QLabel *damOperatorValueLabel;
	QPushButton *commonInformationEditButton;
	QLabel *waterLevelScalingMinimumOpLevelLabel;
	QLabel *waterLevelScalingFullSupplyOpLevelLabel;
	QLabel *waterLevelScalingMinimumOpLevelValueLabel;
	QLabel *waterLevelScalingFullSupplyOpLevelValueLabel;
	QLabel *waterLevelScalingEnabledLabel;
	QLabel *waterLevelScalingEnabledValueLabel;
	QPushButton *waterLevelScalingEditButton;
	boost::signals2::connection commonParameterChangedConnection;
	COM_CONNECTION waterLevelStrategyChangedConnection;
	edit_signal_t editSignal;

};

template<typename A>
inline AnalysisOverviewWidget<A>::AnalysisOverviewWidget(boost::shared_ptr<A> analysis)
	:analysis(analysis)
{
	using namespace boost;

	setupLayout();
	commonParameterChangedConnection = analysis->subscribe(bind(&AnalysisOverviewWidget<A>::updateValues, this));
	waterLevelStrategyChangedConnection = COM_SUBSCRIBE_FOR(analysis->getWaterLevelScalingStrategy(), Changed, bind(&AnalysisOverviewWidget<A>::updateValues, this));
	updateValues();
}

template<typename A>
inline AnalysisOverviewWidget<A>::~AnalysisOverviewWidget() {
	std::cout << __FUNCTION__ << std::endl;
	commonParameterChangedConnection.disconnect();
}


template<typename A>
inline void AnalysisOverviewWidget<A>::setupLayout() {
	using namespace boost;
	using namespace communication;

	//setStyleSheet("background-color:white;");
	nameLabel = new QLabel(tr("Name"));
	nameLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	nameValueLabel = new QLabel;
	nameValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	nameLabel->setBuddy(nameValueLabel);

	descriptionLabel = new QLabel(tr("Description"));
	descriptionLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	descriptionValueLabel = new QLabel;
	descriptionValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	descriptionLabel->setBuddy(descriptionValueLabel);

	commonInformationEditButton = new QPushButton(tr("edit"), this);
	commonInformationEditButton->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

	connect(commonInformationEditButton, SIGNAL(clicked()), this, SLOT(qtEditSlot()));

	QVBoxLayout *mainLayout = new QVBoxLayout;

	QGroupBox *commonInformationGroupBox = new QGroupBox(tr("General information"), this);
		QGridLayout *commonInformationGrid = new QGridLayout;
		commonInformationGrid->addWidget(nameLabel, 0, 0);
		commonInformationGrid->addWidget(nameValueLabel, 0, 1, Qt::AlignLeft);
		commonInformationGrid->addWidget(descriptionLabel, 1, 0, Qt::AlignTop|Qt::AlignLeft);
		commonInformationGrid->addWidget(descriptionValueLabel, 1, 1, Qt::AlignTop|Qt::AlignLeft);
		commonInformationGrid->addWidget(commonInformationEditButton, 2, 2, Qt::AlignRight);
	commonInformationGroupBox->setLayout(commonInformationGrid);
	commonInformationGroupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);





	/**
	 * WaterLevel scaling strategy
	 */
	waterLevelScalingMinimumOpLevelLabel = new QLabel(tr("Minimum operating level"), this);
	waterLevelScalingMinimumOpLevelLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

	waterLevelScalingMinimumOpLevelValueLabel = new QLabel(this);
	waterLevelScalingMinimumOpLevelValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	waterLevelScalingFullSupplyOpLevelLabel = new QLabel(tr("Full supply level"), this);
	waterLevelScalingFullSupplyOpLevelLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

	waterLevelScalingFullSupplyOpLevelValueLabel = new QLabel(this);
	waterLevelScalingFullSupplyOpLevelValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	waterLevelScalingEnabledLabel = new QLabel(tr("Is enabled"), this);
	waterLevelScalingEnabledLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

	waterLevelScalingEnabledValueLabel = new QLabel(this);
	waterLevelScalingEnabledValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	waterLevelScalingEditButton = new QPushButton(tr("edit"), this);
	waterLevelScalingEditButton->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

	QGroupBox *waterlevelScalingGroupBox = new QGroupBox("Water level scaling", this);
		QGridLayout *waterlevelScalingGrid = new QGridLayout;
		waterlevelScalingGrid->addWidget(waterLevelScalingEnabledLabel, 0, 0, Qt::AlignTop|Qt::AlignLeft);
		waterlevelScalingGrid->addWidget(waterLevelScalingEnabledValueLabel, 0, 1, Qt::AlignTop|Qt::AlignLeft);
		waterlevelScalingGrid->addWidget(waterLevelScalingFullSupplyOpLevelLabel, 1, 0, Qt::AlignTop|Qt::AlignLeft);
		waterlevelScalingGrid->addWidget(waterLevelScalingFullSupplyOpLevelValueLabel, 1, 1, Qt::AlignTop|Qt::AlignLeft);
		waterlevelScalingGrid->addWidget(waterLevelScalingMinimumOpLevelLabel, 2, 0);
		waterlevelScalingGrid->addWidget(waterLevelScalingMinimumOpLevelValueLabel, 2, 1, Qt::AlignLeft);
		waterlevelScalingGrid->addWidget(waterLevelScalingEditButton, 2, 2, Qt::AlignRight);
		waterlevelScalingGroupBox->setLayout(waterlevelScalingGrid);
		waterlevelScalingGroupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	QtBoostDelegate *waterLevelEditDelegate = new QtBoostDelegate(bind(ref(EditWaterLevelScalingSignal), analysis), this);
	connect(waterLevelScalingEditButton, SIGNAL(clicked()), waterLevelEditDelegate, SLOT(execute()));

	QSpacerItem *spacer = new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding);

	mainLayout->addWidget(commonInformationGroupBox);
	mainLayout->addWidget(waterlevelScalingGroupBox);
	mainLayout->addSpacerItem(spacer);

	setLayout(mainLayout);
}

template<typename A>
inline void AnalysisOverviewWidget<A>::updateValues() {
	nameValueLabel->setText(analysis->getName());
	descriptionValueLabel->setText(analysis->getDescription());
	waterLevelScalingMinimumOpLevelValueLabel->setText(QString::number(analysis->getWaterLevelScalingStrategy()->getMinimumOperatingLevel()));
	waterLevelScalingFullSupplyOpLevelValueLabel->setText(QString::number(analysis->getWaterLevelScalingStrategy()->getFullSupplyLevel()));

	QString enabledState;
	if(analysis->getWaterLevelScalingStrategy()->isEnabled() == true) {
		enabledState = tr("yes");
	}else{
		enabledState = tr("no");
	}
	waterLevelScalingEnabledValueLabel->setText(enabledState);
}


template<typename A>
inline boost::signals2::connection AnalysisOverviewWidget<A>::subscribeForEdit(const typename edit_signal_t::slot_type &subscriber) {
	return editSignal.connect(subscriber);
}


template<typename A>
inline void AnalysisOverviewWidget<A>::qtEditSlot() {
	editSignal(analysis);
}

} /* namespace gui */
#endif /* ANALYSISOVERVIEWWIDGET_HPP_ */
