/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisWaterLevelScalingWindow.hpp
 *
 *  Created on: 12.11.2014
 *      Author: gerberma
 */

#ifndef ANALYSISWATERLEVELSCALINGWINDOW_HPP_
#define ANALYSISWATERLEVELSCALINGWINDOW_HPP_

#include <QWidget>
#include <QWizardPage>
#include <QLabel>
#include <QLineEdit>
#include <QCheckBox>
#include <QGridLayout>
#include <QRegExpValidator>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"

namespace gui {

template<typename A>
class AnalysisWaterLevelScalingWindow : public QWizardPage {
public:
	AnalysisWaterLevelScalingWindow(QWidget *parent, boost::shared_ptr<A> analysis)
		:QWizardPage(parent), analysis(analysis)
	{
		setupLayout();
		prepare();
		connectLocal();
	}

	virtual ~AnalysisWaterLevelScalingWindow(){;}

	bool isComplete() const {
		QString fullSupplyLevelValue = fullSupplyLevelLineEdit->text();
		QString minOpLevelValue = minOpLevelLineEdit->text();
		int fullSupplyLevelValueLength = fullSupplyLevelValue.length();
		int minOpLevelValueLength = minOpLevelValue.length();

		if(fullSupplyLevelLineEdit->validator()->validate(fullSupplyLevelValue, fullSupplyLevelValueLength) == QValidator::Acceptable
				&& minOpLevelLineEdit->validator()->validate(minOpLevelValue, minOpLevelValueLength) == QValidator::Acceptable
				&& minOpLevelValue.toDouble() < fullSupplyLevelValue.toDouble()
				)
		{
			return true;
		}

		return false;
	}

private:
	void setupLayout() {
		QRegExp rx("^[0-9][0-9]*$");
		QValidator *numberValidator = new QRegExpValidator(rx, this);

		setTitle(tr("Water level scaling strategy"));
		setSubTitle(tr("For the water level scaling, you can define the minimum operating level as well as the full supply level of the dam"));

		minOpLevelLineEdit = new QLineEdit;
		minOpLevelLineEdit->setValidator(numberValidator);
		fullSupplyLevelLineEdit = new QLineEdit;
		fullSupplyLevelLineEdit->setValidator(numberValidator);

		minOpLevelLabel = new QLabel(tr("Minimum operating level"), this);
		fullSupplyLevelLabel = new QLabel(tr("Full supply level"), this);

		isEnabledLabel = new QLabel(tr("Is enabled"), this);
		isEnabledCheckBox = new QCheckBox(this);

		registerField("fullsupplylevel", fullSupplyLevelLineEdit);
		registerField("minoplevel", minOpLevelLineEdit);
		registerField("isenabled", isEnabledCheckBox);

		QGridLayout *layout = new QGridLayout;
		layout->addWidget(isEnabledLabel, 0, 0);
		layout->addWidget(isEnabledCheckBox, 0, 1);
		layout->addWidget(fullSupplyLevelLabel, 1, 0);
		layout->addWidget(fullSupplyLevelLineEdit, 1, 1);
		layout->addWidget(minOpLevelLabel, 2, 0);
		layout->addWidget(minOpLevelLineEdit, 2, 1);

		setLayout(layout);
	}

	void prepare() {
		isEnabledCheckBox->setChecked(analysis->getWaterLevelScalingStrategy()->isEnabled());
		fullSupplyLevelLineEdit->setText(QString::number(analysis->getWaterLevelScalingStrategy()->getFullSupplyLevel()));
		minOpLevelLineEdit->setText(QString::number(analysis->getWaterLevelScalingStrategy()->getMinimumOperatingLevel()));
		enabledStateChanged();
	}

	void enabledStateChanged() {
		fullSupplyLevelLineEdit->setEnabled(isEnabledCheckBox->isChecked());
		minOpLevelLineEdit->setEnabled(isEnabledCheckBox->isChecked());
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		QtBoostDelegate *enabledStateChangedDelegate = new QtBoostDelegate(bind(&AnalysisWaterLevelScalingWindow<A>::enabledStateChanged, this), this);
		connect(isEnabledCheckBox, SIGNAL(stateChanged(int)), enabledStateChangedDelegate, SLOT(execute()));

		connect(fullSupplyLevelLineEdit, SIGNAL(textChanged(const QString &)), this, SIGNAL(completeChanged()));
		connect(minOpLevelLineEdit, SIGNAL(textChanged(const QString &)), this, SIGNAL(completeChanged()));
	}

private:
	boost::shared_ptr<A> analysis;
	QLabel *minOpLevelLabel;
	QLabel *fullSupplyLevelLabel;
	QLabel *isEnabledLabel;
	QLineEdit *minOpLevelLineEdit;
	QLineEdit *fullSupplyLevelLineEdit;
	QCheckBox *isEnabledCheckBox;
};

} //namespace gui

#endif /* ANALYSISWATERLEVELSCALINGWINDOW_HPP_ */
