/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * MainWindowDecl.hpp
 *
 *  Created on: Oct 30, 2013
 *      Author: gerberma
 */

#ifndef COREWINDOW_HPP_
#define COREWINDOW_HPP_

#include <QDebug>
#include <QMainWindow>
#include <QDialog>
#include <QDialogButtonBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QObject>
#include <QAction>
#include <QListWidget>
#include <QMenu>
#include <QTextEdit>
#include <QSplitter>
#include <QScrollArea>
#include <QWidget>
#include <QStackedWidget>
#include <QStatusBar>
#include <QMenuBar>
#include <QLabel>
#include <QActionGroup>
#include <QFile>
#include <boost/bind.hpp>
#include <boost/signals2/signal.hpp>
#include "communication/Signals.hpp"
#include "Delegates.hpp"


namespace gui {

class AboutDialog : public QDialog {
public:
	AboutDialog(QWidget *parent):QDialog(parent){
	 setupLayout();
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;

	    QFile file("about.html"); //we expect it to be in the current working directory
	    if (!file.open(QFile::ReadOnly | QFile::Text)) throw std::logic_error("Cannot open about.html file. Please check that file exists in the path of the executable");
	    QTextStream in(&file);
		QString htmlMessage(in.readAll());

		QTextEdit *message = new QTextEdit(this);
		message->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
		message->setHtml(htmlMessage);
		message->setReadOnly(true);

		buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);
		buttonBox->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
		QHBoxLayout *buttonBoxLayout = new QHBoxLayout();
		buttonBoxLayout->addWidget(buttonBox, 1, Qt::AlignRight);

		mainLayout->addWidget(message);
		mainLayout->addLayout(buttonBoxLayout);
		//mainLayout->addWidget(buttonBox, 1, Qt::AlignBottom);

		connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
		setLayout(mainLayout);
	}

private:
	QDialogButtonBox *buttonBox;
};

namespace priv {
	class DirtyBaseForCoreWindow : public QMainWindow {
	};

	class CoreWindowSignalsAndSlots : public DirtyBaseForCoreWindow {
		Q_OBJECT
	signals:
		void closeSignal();
		void createProjectSignal();
		void saveProjectSignal();
		void loadProjectSignal();

	private slots:
		virtual void closeSlot() = 0;
		virtual void createProjectSlot() = 0;
		virtual void loadProjectSlot() = 0;
		virtual void saveProjectSlot() = 0;
		virtual void showAboutDialogSlot() = 0;
	};
}

template<typename Model>
class CoreWindow: public priv::CoreWindowSignalsAndSlots {
public:
	typedef boost::signals2::signal<void()> close_signal_t;
	typedef boost::signals2::signal<void()> create_project_signal_t;
	typedef boost::signals2::signal<void()> load_project_signal_t;
	typedef boost::signals2::signal<void()> save_project_signal_t;
	typedef boost::signals2::signal<void()> show_project_signal_t;
	COM_SUBSCRIPTION_FUNCTION(ModeChanged, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(CloseRequested, communication::Signal<void(QCloseEvent *)>);

public:

	void start();

	void closeEvent(QCloseEvent *event) {
		CloseRequestedSignal(event);
	}

	CoreWindow(Model *model);
	virtual ~CoreWindow();

	void setWorkingArea(QWidget *workingAreaUi);
	void setNavigationArea(QWidget *navigationAreaUi);

	boost::signals2::connection subscribeForClose(const close_signal_t::slot_type &subscriber);
	boost::signals2::connection subscribeForCreate(const create_project_signal_t::slot_type &subscriber);
	boost::signals2::connection subscribeForLoad(const load_project_signal_t::slot_type &subscriber);
	boost::signals2::connection subscribeForSave(const save_project_signal_t::slot_type &subscriber);

private:
	void setupLayout();
	void setupMenu();
	void setupStatusBar();
	void updateFromModeChanges();
	void basicModeTriggered();
	void scientificModeTriggered();
	void newProjectAssignedToModel();
	void projectChanged();
	void updateWindowTitle(QString projectName = "");

	void closeSlot();
	void createProjectSlot();
	void loadProjectSlot();
	void saveProjectSlot();
	void showAboutDialogSlot();

private:
	Model *model;
	QMenu *fileMenu;
	QMenu *projectMenu;
	QMenu *modeMenu;
	QMenu *aboutMenu;
	QAction *closeAction;
	QAction *createProjectAction;
	QAction *saveProjectAction;
	QAction *loadProjectAction;
	QAction *basicModeAction;
	QAction *scientificModeAction;
	QAction *aboutDialogAction;
	QLabel *modeStatusLabel;
	QActionGroup *modeStatusActionGroup;
	QSplitter *splitter;
	QStackedWidget *workingAreaWidgetContainer;
	QStackedWidget *navigationAreaWidgetContainer;
	QScrollArea *workingAreaWidget;
	QScrollArea *navigationAreaWidget;
	boost::signals2::connection modelConnection;
	boost::signals2::connection projectConnection;
	COM_CONNECTION modeChangedConnection;
	communication::QtBoostDelegate *basicModeActionDelegate;
	communication::QtBoostDelegate *scientificModeActionDelegate;
	close_signal_t closeSignal;
	create_project_signal_t createProjectSignal;
	load_project_signal_t loadProjectSignal;
	save_project_signal_t saveProjectSignal;
};

template<typename Model>
inline CoreWindow<Model>::CoreWindow(Model* model)
	:model(model)
{
	using namespace boost;

	setupLayout();
	setupMenu();
	setupStatusBar();
	updateFromModeChanges();

	modelConnection = model->subscribe(boost::bind(&CoreWindow<Model>::newProjectAssignedToModel, this));
	modeChangedConnection = COM_SUBSCRIBE_FOR(model->getModeSettings()->getModeHandler(), ModeChanged, bind(&CoreWindow<Model>::updateFromModeChanges, this));

	updateWindowTitle();
}

template<typename Model>
inline CoreWindow<Model>::~CoreWindow() {
	modelConnection.disconnect();
	projectConnection.disconnect();
}

template<typename Model>
inline void CoreWindow<Model>::setWorkingArea(QWidget* workingAreaUi) {
	if(workingAreaWidgetContainer->indexOf(workingAreaUi) == -1) {
		workingAreaWidgetContainer->addWidget(workingAreaUi);
	}

	workingAreaWidgetContainer->setCurrentWidget(workingAreaUi);
}

template<typename Model>
inline void CoreWindow<Model>::setNavigationArea(QWidget* navigationAreaUi) {
	if(navigationAreaWidgetContainer->indexOf(navigationAreaUi) == -1) {
		navigationAreaWidgetContainer->addWidget(navigationAreaUi);
	}

	navigationAreaWidgetContainer->setCurrentWidget(navigationAreaUi);
	//navigationAreaWidget->setWidget(navigationAreaUi);
}

template<typename Model>
inline void CoreWindow<Model>::closeSlot() {
	closeSignal();
}

template<typename Model>
inline void CoreWindow<Model>::createProjectSlot() {
	createProjectSignal();
}

template<typename Model>
inline void CoreWindow<Model>::loadProjectSlot() {
	loadProjectSignal();
}

template<typename Model>
inline void CoreWindow<Model>::saveProjectSlot() {
	saveProjectSignal();
}

template<typename Model>
inline void CoreWindow<Model>::showAboutDialogSlot() {
	AboutDialog *aboutDialog = new AboutDialog(this);
	aboutDialog->exec();
}

template<typename Model>
inline boost::signals2::connection CoreWindow<Model>::subscribeForClose(const close_signal_t::slot_type& subscriber) {
	return closeSignal.connect(subscriber);
}

template<typename Model>
inline boost::signals2::connection CoreWindow<Model>::subscribeForCreate(const create_project_signal_t::slot_type& subscriber) {
	return createProjectSignal.connect(subscriber);
}

template<typename Model>
inline boost::signals2::connection CoreWindow<Model>::subscribeForLoad(const load_project_signal_t::slot_type& subscriber) {
	return loadProjectSignal.connect(subscriber);
}

template<typename Model>
inline boost::signals2::connection CoreWindow<Model>::subscribeForSave(const save_project_signal_t::slot_type& subscriber) {
	return saveProjectSignal.connect(subscriber);
}


template<typename Model>
inline void CoreWindow<Model>::newProjectAssignedToModel() {
	if(model->getProject() != NULL) {
		saveProjectAction->setEnabled(true);
		updateWindowTitle(model->getProject()->getName());
		projectConnection.disconnect(); //for any older projects
		projectConnection = model->getProject()->subscribe(boost::bind(&CoreWindow<Model>::projectChanged, this));
	}else{
		saveProjectAction->setEnabled(false);
		updateWindowTitle();
	}

	loadProjectAction->setEnabled(model->isLoadingProjectAllowed());

	createProjectAction->setEnabled(model->isCreatingNewProjectAllowed());
}

template<typename Model>
inline void CoreWindow<Model>::setupLayout() {
	splitter = new QSplitter(Qt::Horizontal, this);

	workingAreaWidget = new QScrollArea(this);
	workingAreaWidget->setWidgetResizable(true);
	navigationAreaWidget = new QScrollArea(this);
	navigationAreaWidget->setWidgetResizable(true);

	workingAreaWidgetContainer = new QStackedWidget(this);
	workingAreaWidget->setWidget(workingAreaWidgetContainer);

	navigationAreaWidgetContainer = new QStackedWidget(this);
	navigationAreaWidget->setWidget(navigationAreaWidgetContainer);

	splitter->insertWidget(0, navigationAreaWidget);
	splitter->insertWidget(1, workingAreaWidget);

	splitter->setStretchFactor(0, 1);
	splitter->setStretchFactor(1, 10);

	setMinimumSize(800, 450);
	showMaximized();

	statusBar()->show();

	setCentralWidget (splitter);
}

template<typename Model>
inline void CoreWindow<Model>::setupMenu() {
	fileMenu = menuBar()->addMenu(tr("&File"));
	projectMenu = menuBar()->addMenu(tr("&Project"));
	modeMenu = menuBar()->addMenu(tr("&Mode"));
	aboutMenu = menuBar()->addMenu(tr("&About"));

	closeAction = new QAction(tr("&Close"), this);
	createProjectAction = new QAction(tr("&Create project"), this);
	saveProjectAction = new QAction(tr("&Save project"), this);
	saveProjectAction->setEnabled(false);
	loadProjectAction = new QAction(tr("&Load project"), this);

	fileMenu->addAction(closeAction);
	projectMenu->addAction(createProjectAction);
	projectMenu->addAction(loadProjectAction);
	projectMenu->addAction(saveProjectAction);

	basicModeAction = new QAction(tr("&Basic"), this);
	scientificModeAction = new QAction(tr("&Scientific"), this);
	modeStatusActionGroup = new QActionGroup(this);
	basicModeAction->setEnabled(true);
	basicModeAction->setCheckable(true);
	scientificModeAction->setCheckable(true);
	scientificModeAction->setChecked(true);
	basicModeAction->setActionGroup(modeStatusActionGroup);
	scientificModeAction->setActionGroup(modeStatusActionGroup);
	modeMenu->addAction(basicModeAction);
	modeMenu->addAction(scientificModeAction);

	aboutDialogAction = new QAction(tr("&About DamBASE"), this);
	aboutMenu->addAction(aboutDialogAction);

	connect(closeAction, SIGNAL(triggered()), this, SLOT(closeSlot()));
	connect(createProjectAction, SIGNAL(triggered()), this, SLOT(createProjectSlot()));
	connect(loadProjectAction, SIGNAL(triggered()), this, SLOT(loadProjectSlot()));
	connect(saveProjectAction, SIGNAL(triggered()), this, SLOT(saveProjectSlot()));
	connect(aboutDialogAction, SIGNAL(triggered()), this, SLOT(showAboutDialogSlot()));

	using namespace communication;
	using namespace boost;
	basicModeActionDelegate = new QtBoostDelegate(bind(&CoreWindow<Model>::basicModeTriggered, this), this);
	connect(basicModeAction, SIGNAL(triggered()), basicModeActionDelegate, SLOT(execute()));
	scientificModeActionDelegate = new QtBoostDelegate(bind(&CoreWindow<Model>::scientificModeTriggered, this), this);
	connect(scientificModeAction, SIGNAL(triggered()), scientificModeActionDelegate, SLOT(execute()));
}

template<typename Model>
inline void CoreWindow<Model>::setupStatusBar() {
	statusBar()->showMessage(tr("Ready"));
	modeStatusLabel = new QLabel(this);
	modeStatusLabel->setIndent(3);

	statusBar()->addPermanentWidget(modeStatusLabel);
}

template<typename Model>
inline void CoreWindow<Model>::updateFromModeChanges() {
	disconnect(basicModeAction, SIGNAL(triggered()), basicModeActionDelegate, SLOT(execute()));
	disconnect(scientificModeAction, SIGNAL(triggered()), scientificModeActionDelegate, SLOT(execute()));

	if(model->getModeSettings()->getModeHandler()->isScientific()) {
		modeStatusLabel->setText(tr("Scientific"));
		scientificModeAction->setChecked(true);
		basicModeAction->setChecked(false);
	}else{
		modeStatusLabel->setText(tr("Basic"));
		scientificModeAction->setChecked(false);
		basicModeAction->setChecked(true);
	}

	connect(basicModeAction, SIGNAL(triggered()), basicModeActionDelegate, SLOT(execute()));
	connect(scientificModeAction, SIGNAL(triggered()), scientificModeActionDelegate, SLOT(execute()));

}

template<typename Model>
inline void CoreWindow<Model>::basicModeTriggered() {
	ModeChangedSignal(0);
}

template<typename Model>
inline void CoreWindow<Model>::scientificModeTriggered() {
	ModeChangedSignal(1);
}


template<typename Model>
inline void CoreWindow<Model>::start() {
	qDebug() << __FUNCTION__;
	QMainWindow::show();
}

template<typename Model>
inline void CoreWindow<Model>::projectChanged() {
	if(model->getProject() != NULL) {
		updateWindowTitle(model->getProject()->getName());
	}
}

template<typename Model>
inline void CoreWindow<Model>::updateWindowTitle(QString projectName) {
	QString windowTitle = tr("DamBASE (v _CURRENT_DAMBASE_VERSION_)");
	if(projectName.isEmpty()) {
		setWindowTitle(windowTitle);
	}else{
		setWindowTitle(windowTitle + QString(" - ") + projectName);
	}
}

} /* namespace gui */


#endif /* COREWINDOW_HPP_ */
