/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactorsConfigurationWidget.hpp
 *
 *  Created on: Feb 4, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISINFLUENCEFACTORSCONFIGURATIONWIDGET_HPP_
#define DATAMODELANALYSISINFLUENCEFACTORSCONFIGURATIONWIDGET_HPP_

#include <QWidget>
#include <QGridLayout>
#include <QLabel>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "ComboBox.hpp"
#include "Delegates.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename InfluenceFactors>
class DataModelAnalysisInfluenceFactorsConfigurationWidget : public QWidget {
public:
	DataModelAnalysisInfluenceFactorsConfigurationWidget(boost::shared_ptr<InfluenceFactors> influenceFactors, QWidget *parent = 0);
	virtual ~DataModelAnalysisInfluenceFactorsConfigurationWidget();

	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorChanged, communication::Signal<void(boost::shared_ptr<InfluenceFactors>, int)>);
	COM_SUBSCRIPTION_FUNCTION(InfluenceFactorChanged, communication::Signal<void(boost::shared_ptr<InfluenceFactors>, int)>);

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

	void behaviorIndicatorChanged(int index);
	void influenceFactorChanged(int index);

private:
	boost::shared_ptr<InfluenceFactors> influenceFactors;
	QLabel *behaviorIndicatorLabel;
	QLabel *influenceFactorsLabel;
	ComboBox *behaviorIndicatorComboBox;
	ComboBox *influenceFactorsComboBox;
	communication::QtBoostIndexDelegate *behaviorIndicatorChangedDelegate;
	communication::QtBoostIndexDelegate *influenceFactorChangedDelegate;
	COM_CONNECTION anyInfluenceFactorChangedConnection;
	COM_CONNECTION behavioIndicatorsChangedConnection;
};

template<typename InfluenceFactors>
inline DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::DataModelAnalysisInfluenceFactorsConfigurationWidget(boost::shared_ptr<InfluenceFactors> influenceFactors, QWidget* parent)
	:QWidget(parent), influenceFactors(influenceFactors)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename InfluenceFactors>
inline DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::~DataModelAnalysisInfluenceFactorsConfigurationWidget() {
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::setupLayout() {
	QGridLayout *gridLayout = new QGridLayout(this);

	behaviorIndicatorLabel = new QLabel(tr("Behavior indicator"), this);
	behaviorIndicatorComboBox = new ComboBox(this);
	behaviorIndicatorComboBox->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	behaviorIndicatorLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	behaviorIndicatorLabel->setBuddy(behaviorIndicatorComboBox);

	influenceFactorsLabel = new QLabel(tr("Influence factor"), this);
	influenceFactorsComboBox = new ComboBox(this);
	influenceFactorsComboBox->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	influenceFactorsLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	influenceFactorsLabel->setBuddy(influenceFactorsComboBox);

	gridLayout->addWidget(behaviorIndicatorLabel, 0, 0);
	gridLayout->addWidget(behaviorIndicatorComboBox, 0, 1);
	gridLayout->addWidget(influenceFactorsLabel, 1, 0);
	gridLayout->addWidget(influenceFactorsComboBox, 1, 1);

	setLayout(gridLayout);
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::updateValues() {
	//
	//

	disconnect(behaviorIndicatorComboBox, SIGNAL(currentIndexChanged(int)), behaviorIndicatorChangedDelegate, SLOT(execute(int)));
	behaviorIndicatorComboBox->clear();

	BOOST_FOREACH(boost::shared_ptr<typename InfluenceFactors::Column_t> behaviorIndicator, influenceFactors->getBehaviorIndicatorColumns()) {
		behaviorIndicatorComboBox->addItem(behaviorIndicator->getName(), behaviorIndicator->getIndex());
	}

	//make sure that the succeeding setCurrentByValue will cause a signal to get called
	connect(behaviorIndicatorComboBox, SIGNAL(currentIndexChanged(int)), behaviorIndicatorChangedDelegate, SLOT(execute(int)));
	if(influenceFactors->getCurrentBehaviorIndicatorColumn() != NULL) {
		behaviorIndicatorComboBox->setCurrentByValue(influenceFactors->getCurrentBehaviorIndicatorColumn()->getIndex());
	}else{
		behaviorIndicatorComboBox->setCurrentByValue(influenceFactors->getAnEstimatedBehaviorIndicator());
	}


	disconnect(influenceFactorsComboBox, SIGNAL(currentIndexChanged(int)),influenceFactorChangedDelegate, SLOT(execute(int)));
	influenceFactorsComboBox->clear();

	influenceFactorsComboBox->addItem(tr("Water level"), -1);
	BOOST_FOREACH(boost::shared_ptr<typename InfluenceFactors::Column_t> waterLevelColumn, influenceFactors->getWaterLevelColumns()) {
		influenceFactorsComboBox->addItem(" " + waterLevelColumn->getName(), waterLevelColumn->getIndex());
	}
	influenceFactorsComboBox->addItem(tr("Temperature"), -1);
	BOOST_FOREACH(boost::shared_ptr<typename InfluenceFactors::Column_t> temperatureColumn, influenceFactors->getTemperatureColumns()) {
		influenceFactorsComboBox->addItem(" " + temperatureColumn->getName(), temperatureColumn->getIndex());
	}

	//make sure that the succeeding setCurrentByValue will cause a signal to get called
	connect(influenceFactorsComboBox, SIGNAL(currentIndexChanged(int)),influenceFactorChangedDelegate, SLOT(execute(int)));
	if(influenceFactors->getCurrentInfluenceFactorColumn() != NULL) {
		influenceFactorsComboBox->setCurrentByValue(influenceFactors->getCurrentInfluenceFactorColumn()->getIndex());
	}else{
		influenceFactorsComboBox->setCurrentByValue(influenceFactors->getAnEstimatedInfluenceFactor());
	}
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::connectLocal() {
	behaviorIndicatorChangedDelegate = new communication::QtBoostIndexDelegate(boost::bind(&DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::behaviorIndicatorChanged, this,  _1), this);
	connect(behaviorIndicatorComboBox, SIGNAL(currentIndexChanged(int)), behaviorIndicatorChangedDelegate, SLOT(execute(int)));

	influenceFactorChangedDelegate = new communication::QtBoostIndexDelegate(boost::bind(&DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::influenceFactorChanged, this,  _1), this);
	connect(influenceFactorsComboBox, SIGNAL(currentIndexChanged(int)),influenceFactorChangedDelegate, SLOT(execute(int)));


	anyInfluenceFactorChangedConnection = COM_SUBSCRIBE_FOR(influenceFactors, AnyInfluenceFactorChanged, boost::bind(&DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::updateValues, this));
	behavioIndicatorsChangedConnection = COM_SUBSCRIBE_FOR(influenceFactors, BehaviorIndicatorsChanged, boost::bind(&DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::updateValues, this));
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::behaviorIndicatorChanged(int index) {
	if(behaviorIndicatorComboBox->itemData(behaviorIndicatorComboBox->currentIndex(), Qt::UserRole) != -1) {
		BehaviorIndicatorChangedSignal(influenceFactors, behaviorIndicatorComboBox->itemData(behaviorIndicatorComboBox->currentIndex(), Qt::UserRole).toString().toInt());
	}
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>::influenceFactorChanged(int index) {
	if(influenceFactorsComboBox->itemData(influenceFactorsComboBox->currentIndex(), Qt::UserRole) != -1) {
		InfluenceFactorChangedSignal(influenceFactors, influenceFactorsComboBox->itemData(influenceFactorsComboBox->currentIndex(), Qt::UserRole).toString().toInt());
	}
}


} /* namespace gui */


#endif /* DATAMODELANALYSISINFLUENCEFACTORSCONFIGURATIONWIDGET_HPP_ */
