/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactorsDisplayWidget.hpp
 *
 *  Created on: Jan 30, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISINFLUENCEFACTORSDISPLAYWIDGET_HPP_
#define DATAMODELANALYSISINFLUENCEFACTORSDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QLabel>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QSpacerItem>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "AnalysisToolToolBar.hpp"
#include "DataModelAnalysisInfluenceFactorsConfigurationWidget.hpp"
#include "ComboBox.hpp"
#include "Delegates.hpp"
#include "AnalysisToolWidget.hpp"
#include "qcustomplot.h"

namespace gui {

template<typename InfluenceFactors>
class DataModelAnalysisInfluenceFactorsDisplayWidget : public AnalysisToolWidget {
public:
	typedef AnalysisToolWidget base_ui_type;
public:
	DataModelAnalysisInfluenceFactorsDisplayWidget(boost::shared_ptr<InfluenceFactors> influenceFactors, QWidget *parent = 0);
	virtual ~DataModelAnalysisInfluenceFactorsDisplayWidget();


	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>, configurationWidget, BehaviorIndicatorChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>, configurationWidget, InfluenceFactorChanged);
	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysisTool, communication::Signal<void(boost::shared_ptr<InfluenceFactors>)>);

private:
	void setupLayout();
	void connectLocal();
	void updateValues();

private:
	boost::shared_ptr<InfluenceFactors> influenceFactors;
	DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors> *configurationWidget;
	ComboBox *behaviorIndicatorComboBox;
	QCustomPlot *plot;
	COM_CONNECTION analysisToolToolBarRemoveButtonClickedConnection;
	COM_CONNECTION loadedConnection;
};

template<typename InfluenceFactors>
inline DataModelAnalysisInfluenceFactorsDisplayWidget<InfluenceFactors>::DataModelAnalysisInfluenceFactorsDisplayWidget(boost::shared_ptr<InfluenceFactors> influenceFactors, QWidget* parent)
	:AnalysisToolWidget(parent), influenceFactors(influenceFactors)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename InfluenceFactors>
inline DataModelAnalysisInfluenceFactorsDisplayWidget<InfluenceFactors>::~DataModelAnalysisInfluenceFactorsDisplayWidget() {
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsDisplayWidget<InfluenceFactors>::setupLayout() {
	QVBoxLayout *mainLayout = new QVBoxLayout;
	QVBoxLayout *contentLayout = new QVBoxLayout;
	configurationWidget = new DataModelAnalysisInfluenceFactorsConfigurationWidget<InfluenceFactors>(influenceFactors, this);

	//fill elements
	QGridLayout *dataLayout = new QGridLayout(this);
	QWidget *dataWidget = new QWidget(this);
	dataWidget->setLayout(dataLayout);
	contentLayout->addWidget(AnalysisToolWidget::getToolBar(), 0, Qt::AlignRight | Qt::AlignTop);
	contentLayout->addWidget(configurationWidget, 0, Qt::AlignLeft);
	contentLayout->addWidget(dataWidget);



	plot = new QCustomPlot(this);
	plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);

	/*
	 * legend
	QCPLayoutElement *placeholder = new QCPLayoutElement;
	QCPLayoutGrid *subLayout = new QCPLayoutGrid;
	plot->plotLayout()->addElement(0, 1, subLayout);
	plot->plotLayout()->setColumnStretchFactor(0, 5);
	subLayout->addElement(0, 0, plot->legend);
	subLayout->addElement(1, 0, placeholder);
	subLayout->setRowStretchFactor(0, 0.01); //legend cell in row 0 should take up as little vertical space as possible
	plot->legend->setVisible(true);
	*/
	dataLayout->addWidget(plot);
	// /fill elements



	QGroupBox *groupBox = new QGroupBox(QObject::tr("Influence factors"), this);
	groupBox->setLayout(contentLayout);
	groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	QSpacerItem *verticalExpandingSpacer = new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding);
	mainLayout->addWidget(groupBox);
	mainLayout->addSpacerItem(verticalExpandingSpacer);



	setLayout(mainLayout);
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsDisplayWidget<InfluenceFactors>::connectLocal() {
	analysisToolToolBarRemoveButtonClickedConnection = COM_SUBSCRIBE_FOR(AnalysisToolWidget::getToolBar(), RemoveButtonClicked, (boost::bind(boost::ref(RemoveAnalysisToolSignal), influenceFactors)));
	loadedConnection = COM_SUBSCRIBE_FOR(influenceFactors, Loaded, boost::bind(&DataModelAnalysisInfluenceFactorsDisplayWidget<InfluenceFactors>::updateValues, this));
}

template<typename InfluenceFactors>
inline void DataModelAnalysisInfluenceFactorsDisplayWidget<InfluenceFactors>::updateValues() {
	using namespace boost;

	QVector<double> behaviorIndicatorValues;
	BOOST_FOREACH(double value, influenceFactors->getBehaviorIndicatorColumnValues()) {
		behaviorIndicatorValues.push_back(value);
	}

	QVector<double> influenceFactorValues;
	BOOST_FOREACH(double value, influenceFactors->getInfluenceFactorColumnValues()) {
		influenceFactorValues.push_back(value);
	}

	plot->clearGraphs();

	plot->addGraph();
	plot->graph(0)->setData(behaviorIndicatorValues, influenceFactorValues);
	plot->graph(0)->setLineStyle(QCPGraph::lsNone);
	plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCircle, Qt::black, Qt::white, 5));

	if(influenceFactors->getCurrentInfluenceFactorColumn() != NULL) {
		plot->yAxis->setLabel(influenceFactors->getCurrentInfluenceFactorColumn()->getName());
	}else{
		plot->yAxis->setLabel("");
	}

	if(influenceFactors->getCurrentBehaviorIndicatorColumn() != NULL) {
		plot->xAxis->setLabel(influenceFactors->getCurrentBehaviorIndicatorColumn()->getName());
	}else{
		plot->xAxis->setLabel("");
	}

	plot->setMinimumHeight(500);
	plot->rescaleAxes();
	plot->replot();
}

} /* namespace gui */


#endif /* DATAMODELANALYSISINFLUENCEFACTORSDISPLAYWIDGET_HPP_ */
