/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverviewConfigurationWidget.hpp
 *
 *  Created on: Mar 11, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISTIMESERIESOVERVIEWCONFIGURATIONWIDGET_HPP_
#define DATAMODELANALYSISTIMESERIESOVERVIEWCONFIGURATIONWIDGET_HPP_

#include <QWidget>
#include <QGridLayout>
#include <QLabel>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "ComboBox.hpp"
#include "Delegates.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename TimeSeriesOverview>
class DataModelAnalysisTimeSeriesOverviewConfigurationWidget : public QWidget {
public:
	DataModelAnalysisTimeSeriesOverviewConfigurationWidget(boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview, QWidget *parent = 0);
	virtual ~DataModelAnalysisTimeSeriesOverviewConfigurationWidget();

	//COM_SUBSCRIPTION_FUNCTION(ValueColumnChanged, communication::Signal<void(TimeSeriesOverview *, int)>);
	typedef communication::Signal<void(boost::shared_ptr<TimeSeriesOverview>, int)>  __ValueColumnChanged_signal_t;
	communication::Signal<void(boost::shared_ptr<TimeSeriesOverview>, int)>  ValueColumnChangedSignal;
	boost::signals2::connection subscribeForValueColumnChanged(const typename communication::Signal<void(boost::shared_ptr<TimeSeriesOverview>, int)>::slot_type &subscriber) {
		return ValueColumnChangedSignal.connect(subscriber);
	}

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

	void valueColumnChanged(int index);

private:
	boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview;
	QLabel *valueColumnLabel;
	ComboBox *valueColumnComboBox;
	communication::QtBoostIndexDelegate *valueColumnChangedDelegate;
	COM_CONNECTION changedConnection;
};

template<typename TimeSeriesOverview>
inline DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::DataModelAnalysisTimeSeriesOverviewConfigurationWidget(boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview, QWidget* parent)
	:QWidget(parent), timeSeriesOverview(timeSeriesOverview)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename TimeSeriesOverview>
inline DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::~DataModelAnalysisTimeSeriesOverviewConfigurationWidget() {
}

template<typename TimeSeriesOverview>
inline void DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::setupLayout() {
	QGridLayout *gridLayout = new QGridLayout(this);

	valueColumnLabel = new QLabel(tr("Value"), this);
	valueColumnComboBox = new ComboBox(this);
	valueColumnComboBox->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	valueColumnLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	valueColumnLabel->setBuddy(valueColumnComboBox);

	gridLayout->addWidget(valueColumnLabel, 0, 0);
	gridLayout->addWidget(valueColumnComboBox, 0, 1);

	setLayout(gridLayout);
}

template<typename TimeSeriesOverview>
inline void DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::updateValues() {
	//
	//

	disconnect(valueColumnComboBox, SIGNAL(currentIndexChanged(int)),valueColumnChangedDelegate, SLOT(execute(int)));
	valueColumnComboBox->clear();

	int currentComboBoxIndex = 0;
	valueColumnComboBox->addItem(tr("Behavior indicator"), -1);
	valueColumnComboBox->disableSelectionFor(currentComboBoxIndex);
	++currentComboBoxIndex;

	BOOST_FOREACH(boost::shared_ptr<typename TimeSeriesOverview::Column_t> behaviorIndicatorColumn, timeSeriesOverview->getDataModel()->getBehaviorIndicatorColumns()) {
		valueColumnComboBox->addItem(" " + behaviorIndicatorColumn->getName(), behaviorIndicatorColumn->getIndex());
		++currentComboBoxIndex;
	}

	valueColumnComboBox->addItem(tr("Water level"), -1);
	valueColumnComboBox->disableSelectionFor(currentComboBoxIndex);
	++currentComboBoxIndex;

	BOOST_FOREACH(boost::shared_ptr<typename TimeSeriesOverview::Column_t> waterLevelColumn, timeSeriesOverview->getDataModel()->getWaterLevelColumns()) {
		valueColumnComboBox->addItem(" " + waterLevelColumn->getName(), waterLevelColumn->getIndex());
		++currentComboBoxIndex;
	}
	valueColumnComboBox->addItem(tr("Temperature"), -1);
	valueColumnComboBox->disableSelectionFor(currentComboBoxIndex);
	++currentComboBoxIndex;
	BOOST_FOREACH(boost::shared_ptr<typename TimeSeriesOverview::Column_t> temperatureColumn, timeSeriesOverview->getDataModel()->getTemperatureColumns()) {
		valueColumnComboBox->addItem(" " + temperatureColumn->getName(), temperatureColumn->getIndex());
		++currentComboBoxIndex;
	}

	//make sure that the succeeding setCurrentByValue will cause a signal to get called
	//
	if(timeSeriesOverview->getValueColumn() != NULL) {
		valueColumnComboBox->setCurrentByValue(timeSeriesOverview->getValueColumn()->getIndex());
		++currentComboBoxIndex;
	}
	connect(valueColumnComboBox, SIGNAL(currentIndexChanged(int)),valueColumnChangedDelegate, SLOT(execute(int)));
}

template<typename TimeSeriesOverview>
inline void DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::connectLocal() {
	valueColumnChangedDelegate = new communication::QtBoostIndexDelegate(boost::bind(&DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::valueColumnChanged, this,  _1), this);
	connect(valueColumnComboBox, SIGNAL(currentIndexChanged(int)),valueColumnChangedDelegate, SLOT(execute(int)));

	changedConnection = COM_SUBSCRIBE_FOR(timeSeriesOverview, Changed, boost::bind(&DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::updateValues, this));
}

template<typename TimeSeriesOverview>
inline void DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview>::valueColumnChanged(int index) {
	if(valueColumnComboBox->itemData(valueColumnComboBox->currentIndex(), Qt::UserRole) != -1) {
		int value = valueColumnComboBox->itemData(valueColumnComboBox->currentIndex(), Qt::UserRole).toString().toInt();
		ValueColumnChangedSignal(timeSeriesOverview, value);
	}
}


} /* namespace gui */



#endif /* DATAMODELANALYSISTIMESERIESOVERVIEWCONFIGURATIONWIDGET_HPP_ */
