/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverviewDisplayWidget.hpp
 *
 *  Created on: Mar 11, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISTIMESERIESOVERVIEWDISPLAYWIDGET_HPP_
#define DATAMODELANALYSISTIMESERIESOVERVIEWDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QLabel>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QSpacerItem>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include "AnalysisToolToolBar.hpp"
#include "DataModelAnalysisTimeSeriesOverviewConfigurationWidget.hpp"
#include "ComboBox.hpp"
#include "Delegates.hpp"
#include "AnalysisToolWidget.hpp"
#include "qcustomplot.h"

namespace gui {

template
<
	typename TimeSeriesOverview,
	typename ConfigurationWidget
>
class DataModelAnalysisTimeSeriesOverviewDisplayWidget : public AnalysisToolWidget {
public:
	typedef AnalysisToolWidget base_ui_type;
	typedef ConfigurationWidget configuration_widget_t;
public:
	DataModelAnalysisTimeSeriesOverviewDisplayWidget(boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview, QWidget *parent = 0)
		:AnalysisToolWidget(parent), timeSeriesOverview(timeSeriesOverview), configurationWidget(NULL)
	{;}
	virtual ~DataModelAnalysisTimeSeriesOverviewDisplayWidget(){;}

	void setConfigurationWidget(ConfigurationWidget *configurationWidget) {
		this->configurationWidget = configurationWidget;
	}

	void init() {
		/*
		if(configurationWidget == NULL) {
			throw std::logic_error("no configuration widget set");
		}
		*/

		setupLayout();
		updateValues();
		connectLocal();
		plot->rescaleAxes();
	}

	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysisTool, communication::Signal<void(boost::shared_ptr<TimeSeriesOverview>)>);

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;
		QVBoxLayout *contentLayout = new QVBoxLayout;

		//fill elements
		QGridLayout *dataLayout = new QGridLayout(this);
		QWidget *dataWidget = new QWidget(this);
		dataWidget->setLayout(dataLayout);
		contentLayout->addWidget(AnalysisToolWidget::getToolBar(), 0, Qt::AlignRight | Qt::AlignTop);
		if(configurationWidget != NULL) {
			contentLayout->addWidget(configurationWidget, 0, Qt::AlignLeft);
		}
		contentLayout->addWidget(dataWidget);
		//svgPlotWidget = new SvgPlotWidget<TimeSeriesOverview>(timeSeriesOverview, this);

		plot = new QCustomPlot(this);
		plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);
		QCPLayoutElement *placeholder = new QCPLayoutElement;
		QCPLayoutGrid *subLayout = new QCPLayoutGrid;
		plot->plotLayout()->addElement(0, 1, subLayout);
		plot->plotLayout()->setColumnStretchFactor(0, 5);
		subLayout->addElement(0, 0, plot->legend);
		subLayout->addElement(1, 0, placeholder);
		subLayout->setRowStretchFactor(0, 0.01); //legend cell in row 0 should take up as little vertical space as possible
		plot->legend->setVisible(true);

		dataLayout->addWidget(plot);


		//dataLayout->addWidget(svgPlotWidget, 1, 0, Qt::AlignCenter);
		// /fill elements

		QGroupBox *groupBox = new QGroupBox(QObject::tr("Time series overview"), this);
		groupBox->setLayout(contentLayout);
		groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

		QSpacerItem *verticalExpandingSpacer = new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding);
		mainLayout->addWidget(groupBox);
		mainLayout->addSpacerItem(verticalExpandingSpacer);

		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;

		QVector<double> values;
		BOOST_FOREACH(double value, timeSeriesOverview->getValues()) {
			values.push_back(value);
		}

		QVector<double> dates;
		posix_time::ptime time_t_epoch(gregorian::date(1970,1,1));
		BOOST_FOREACH(std::string stringDate, timeSeriesOverview->getDates()) {
			qDebug() << "date: " << QString::fromStdString(stringDate);
			gregorian::date thedate(gregorian::from_string(stringDate));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			dates.push_back(duration.total_seconds());
		}

		plot->clearGraphs();

		plot->addGraph();
		plot->graph(0)->setData(dates, values);
		plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
		plot->graph(0)->setName(timeSeriesOverview->getValueColumnName());

		plot->xAxis->setTickLabelType(QCPAxis::ltDateTime);
		//plot->xAxis->setDateTimeFormat("yyyy");
		plot->xAxis->setDateTimeFormat("dd.MM\nyyyy");
		plot->xAxis->setAutoTickStep(true);
		//plot->xAxis->setTickStep(31556927+60*60+60); //2629743.83 seconds for 1 month //31556927+60*60+60 for one year (empirical estimation for this qcustom library)
		//plot->xAxis->setSubTickCount(11);

		plot->xAxis->setLabel(tr("date"));
		plot->yAxis->setLabel(timeSeriesOverview->getValueColumnName());
		plot->setMinimumHeight(200);
		plot->rescaleAxes();
		plot->replot();
	}

	void connectLocal() {
		analysisToolToolBarRemoveButtonClickedConnection = COM_SUBSCRIBE_FOR(AnalysisToolWidget::getToolBar(), RemoveButtonClicked, (boost::bind(boost::ref(RemoveAnalysisToolSignal), timeSeriesOverview)));
		analysisToolLoadedConnection = COM_SUBSCRIBE_FOR(timeSeriesOverview, Loaded, (boost::bind(&DataModelAnalysisTimeSeriesOverviewDisplayWidget<TimeSeriesOverview, ConfigurationWidget>::updateValues, this)));
	}

private:
	boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview;
	//SvgPlotWidget<TimeSeriesOverview> *svgPlotWidget;
	ConfigurationWidget *configurationWidget;
	QCustomPlot *plot;
	COM_CONNECTION analysisToolToolBarRemoveButtonClickedConnection;
	COM_CONNECTION analysisToolLoadedConnection;
};

} /* namespace gui */


#endif /* DATAMODELANALYSISTIMESERIESOVERVIEWDISPLAYWIDGET_HPP_ */
