/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelCommonParameterWidget.hpp
 *
 *  Created on: Dec 6, 2013
 *      Author: gerberma
 */

#ifndef DATAMODELCOMMONPARAMETERWIDGET_HPP_
#define DATAMODELCOMMONPARAMETERWIDGET_HPP_

#include <iostream>
#include <algorithm>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <QRegExpValidator>
#include <QWidget>
#include <QWizardPage>
#include <QSplitter>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QSpacerItem>
#include <QPushButton>
#include <QLineEdit>
#include <QLabel>
#include <QComboBox>
#include <QPushButton>
#include <QObject>
#include "DataModelRawDataDisplayWidget.hpp"
#include "ComboBox.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "Delegates.hpp"
#include "conversion/converter.hpp"
#include "gui/guiconverter.hpp"
#include "communication/Signals.hpp"

namespace gui {

namespace priv {
class DirtyBaseForDataModelEditWidgetSignalsSlots : public QWidget {
	public:
		DirtyBaseForDataModelEditWidgetSignalsSlots(QWidget *parent)
			:QWidget(parent)
		{}

		virtual ~DirtyBaseForDataModelEditWidgetSignalsSlots()
		{}
	};

	class DataModelEditWidgetSignalsSlots : public DirtyBaseForDataModelEditWidgetSignalsSlots {
		Q_OBJECT
	public:
		DataModelEditWidgetSignalsSlots(QWidget *parent)
			:DirtyBaseForDataModelEditWidgetSignalsSlots(parent)
		{}

		virtual ~DataModelEditWidgetSignalsSlots()
		{}

	};
}

template<typename DataModel>
class DataModelEditWidget : public priv::DataModelEditWidgetSignalsSlots {
public:
	typedef boost::signals2::signal<void(boost::shared_ptr<DataModel>)> request_source_uri_signal_t;
	typedef boost::signals2::signal<void(void)> complete_changed_signal_t;
	COM_SUBSCRIPTION_FUNCTION(CSVDelimiterChanged, communication::Signal<void(boost::shared_ptr<DataModel>, QString)>);
public:
	DataModelEditWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent = 0)
		:priv::DataModelEditWidgetSignalsSlots(parent), dataModel(dataModel)
	{
		setupLayout();
		connectLocal();
		subscribeForModelChanges();
		updateValues();
	}

	virtual ~DataModelEditWidget(){
	}

	boost::signals2::connection subscribeForRequestSourceUri(const typename request_source_uri_signal_t::slot_type &subscriber);

	boost::signals2::connection subscribeForCompleteChanged(const typename complete_changed_signal_t::slot_type &subscriber);

	bool isComplete();

public:
	void missingStrategyIndexChanged(int index);

private:
	void csvDelimiterChanged();

private:
	void setupLayout();
	void connectLocal();
	void subscribeForModelChanges();
	void updateValues();

private:
	boost::shared_ptr<DataModel> dataModel;
	QPushButton *loadButton;
	QLabel *uriLabel;
	QLabel *csvDelimiterLabel;
	ComboBox *csvDelimiterComboBox;
	DataModelRawDataDisplayWidget<DataModel> *rawDataWidget;
	request_source_uri_signal_t requestSourceUriSignal;
	complete_changed_signal_t completeChangedSignal;
	COM_CONNECTION dataModelChangedConnection;
};

template<typename DataModel>
inline void gui::DataModelEditWidget<DataModel>::setupLayout() {
	loadButton = new QPushButton(tr("Load"), this);
	loadButton->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	uriLabel = new QLabel(this);

	//Source URI
	uriLabel->setText(dataModel->getSourceUri().toString());

	/*
	QRegExp notEmptyRegex("^(?!\\s*$).+");
	QValidator *notEmptyValidator = new QRegExpValidator(notEmptyRegex, this);
	uriLabel->setValidator(notEmptyValidator);
	*/

	csvDelimiterLabel = new QLabel(tr("CSV delimiter"), this);
	csvDelimiterComboBox = new ComboBox(this);
	csvDelimiterComboBox->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum);
	csvDelimiterComboBox->clear();

	{
		typedef std::pair<QString, QString> pair_t;
		std::vector<QString> sortedPairs;
		BOOST_FOREACH(pair_t csvDelimiterPair, dataModel->getAvailableCSVDelimiters()) {
			csvDelimiterComboBox->addItem(csvDelimiterPair.first, csvDelimiterPair.second);
		}
		csvDelimiterComboBox->setCurrentByValue(dataModel->getCSVDelimiter());
	}

	rawDataWidget = new DataModelRawDataDisplayWidget<DataModel>(dataModel, this);

	GroupBoxLayoutBuilder dataModelLayoutBuilder(tr("Data model"));
	int index = 0;
	dataModelLayoutBuilder.addWidget(loadButton, index, 0);
	dataModelLayoutBuilder.addWidget(uriLabel, index, 1);
	dataModelLayoutBuilder.addWidget(csvDelimiterLabel, ++index, 0);
	dataModelLayoutBuilder.addWidget(csvDelimiterComboBox, index, 1, Qt::AlignLeft);
//	dataModelLayoutBuilder.addWidget(rawDataWidget, ++index, 0, 3, 2);
	QWidget *dataModelWidget = new QWidget(this);
	QVBoxLayout *dataModelWidgetLayout = dataModelLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed), false);
	dataModelWidgetLayout->setAlignment(Qt::AlignTop);
	dataModelWidget->setLayout(dataModelWidgetLayout);

	GroupBoxLayoutBuilder rawDataLayoutBuilder(tr("Data"));
	rawDataLayoutBuilder.addWidget(rawDataWidget, 0, 0, 3, 2);
	QWidget *rawDataWidget = new QWidget(this);
	QVBoxLayout *rawDataWidgetLayout = rawDataLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding), false);
	rawDataWidgetLayout->setAlignment(Qt::AlignTop);
	rawDataWidget->setLayout(rawDataWidgetLayout);

	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	mainLayout->addWidget(dataModelWidget);
	mainLayout->addWidget(rawDataWidget);
	mainLayout->setAlignment(Qt::AlignTop);

	setLayout(mainLayout);
}


template<typename DataModel>
inline void gui::DataModelEditWidget<DataModel>::csvDelimiterChanged() {
	CSVDelimiterChangedSignal(dataModel, csvDelimiterComboBox->itemData(csvDelimiterComboBox->currentIndex(), Qt::UserRole).toString());
}

template<typename DataModel>
inline void gui::DataModelEditWidget<DataModel>::connectLocal() {
	using namespace boost;
	using namespace communication;

	communication::QtBoostDelegate *completeChangedDelegate;
	completeChangedDelegate = new communication::QtBoostDelegate(boost::bind(boost::ref(completeChangedSignal)), this);
	connect(loadButton, SIGNAL(clicked()), completeChangedDelegate, SLOT(execute()));

	communication::QtBoostDelegate *loadButtonClickedDelegate;
	loadButtonClickedDelegate = new communication::QtBoostDelegate(boost::bind(boost::ref(requestSourceUriSignal), dataModel), this);
	connect(loadButton, SIGNAL(clicked()), loadButtonClickedDelegate, SLOT(execute()));

	QtBoostDelegate *csvDelimiterChangedDelegate;
	csvDelimiterChangedDelegate = new QtBoostDelegate(bind(&gui::DataModelEditWidget<DataModel>::csvDelimiterChanged, this), this);
	connect(csvDelimiterComboBox, SIGNAL(currentIndexChanged(int)), csvDelimiterChangedDelegate, SLOT(execute()));
}

template<typename DataModel>
inline boost::signals2::connection DataModelEditWidget<DataModel>::subscribeForRequestSourceUri(const typename request_source_uri_signal_t::slot_type& subscriber) {
	return requestSourceUriSignal.connect(subscriber);
}

template<typename DataModel>
inline void DataModelEditWidget<DataModel>::subscribeForModelChanges() {
	dataModelChangedConnection = dataModel->subscribe(boost::bind(&DataModelEditWidget<DataModel>::updateValues, this));
}


template<typename DataModel>
inline boost::signals2::connection DataModelEditWidget<DataModel>::subscribeForCompleteChanged(const typename complete_changed_signal_t::slot_type& subscriber) {
	return completeChangedSignal.connect(subscriber);
}

template<typename DataModel>
inline bool DataModelEditWidget<DataModel>::isComplete() {
	bool state = false;

	//TODO: 'ask' data model (via rules) about completeness as the data model has been loaded anyway until we are here
	QString nameText(uriLabel->text());
	int nameLength = uriLabel->text().length();
	QRegExp notEmptyRegex("^(?!\\s*$).+");
	QValidator *notEmptyValidator = new QRegExpValidator(notEmptyRegex, this);
	if(notEmptyValidator->validate(nameText, nameLength) == QValidator::Acceptable) {
		state = true;
	}else{
		state = false;
	}


	return state;
}

template<typename DataModel>
inline void gui::DataModelEditWidget<DataModel>::updateValues() {
	if(dataModel->getSourceUri().isEmpty()) {
		uriLabel->setText(tr("<no data file loaded>"));
	}else{
		uriLabel->setText(dataModel->getSourceUri().toString());
	}

	completeChangedSignal();
}

template
<
	typename DataModel,
	typename NextIdStrategy
>
class DataModelCommonParameterWidget : public QWizardPage {
public:
	typedef typename DataModelEditWidget<DataModel>::request_source_uri_signal_t request_source_uri_signal_t;
public:
	DataModelCommonParameterWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId);
	virtual ~DataModelCommonParameterWidget();

	boost::signals2::connection subscribeForRequestSourceUri(const typename request_source_uri_signal_t::slot_type &subscriber);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelEditWidget<DataModel>, dataModelWidget, CSVDelimiterChanged);


	bool isComplete() const;
	int nextId() const;
private:
	void setupLayout(boost::shared_ptr<DataModel> dataModel);
	void connectLocal();

private:
	boost::shared_ptr<DataModel> dataModel;
	SplittedLayoutBuilder splittedWizardPage;
	DataModelEditWidget<DataModel> *dataModelWidget;
	COM_CONNECTION dataModelWidgetIsCompleteConnection;
	NextIdStrategy *nextIdStrategy;
	typename NextIdStrategy::type_t myId;
};


template<typename DataModel, typename NextIdStrategy>
inline DataModelCommonParameterWidget<DataModel, NextIdStrategy>::DataModelCommonParameterWidget(QWidget* parent, boost::shared_ptr<DataModel> dataModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId)
	:QWizardPage(parent), dataModel(dataModel), nextIdStrategy(nextIdStrategy), myId(myId)
{
	setupLayout(dataModel);
	connectLocal();
}

template<typename DataModel, typename NextIdStrategy>
inline DataModelCommonParameterWidget<DataModel, NextIdStrategy>::~DataModelCommonParameterWidget() {
}

template<typename DataModel, typename NextIdStrategy>
inline boost::signals2::connection DataModelCommonParameterWidget<DataModel, NextIdStrategy>::subscribeForRequestSourceUri(const typename request_source_uri_signal_t::slot_type& subscriber) {
	return dataModelWidget->subscribeForRequestSourceUri(subscriber);
}


template<typename DataModel, typename NextIdStrategy>
inline int DataModelCommonParameterWidget<DataModel, NextIdStrategy>::nextId() const {
	return nextIdStrategy->nextFrom(myId);
}

template<typename DataModel, typename NextIdStrategy>
inline bool DataModelCommonParameterWidget<DataModel, NextIdStrategy>::isComplete() const {
	return dataModelWidget->isComplete();
}

template<typename DataModel, typename NextIdStrategy>
inline void DataModelCommonParameterWidget<DataModel, NextIdStrategy>::setupLayout(boost::shared_ptr<DataModel> dataModel) {
	dataModelWidget = new DataModelEditWidget<DataModel>(dataModel, this);

	setLayout(splittedWizardPage.build(this, dataModelWidget, NULL));
}

template<typename DataModel, typename NextIdStrategy>
inline void DataModelCommonParameterWidget<DataModel, NextIdStrategy>::connectLocal() {
	communication::BoostQtSimpleDelegate *completeChangedDelegate = new communication::BoostQtSimpleDelegate(this, SIGNAL(completeChanged()), this);
	dataModelWidget->subscribeForCompleteChanged(boost::bind(&communication::BoostQtSimpleDelegate::execute, completeChangedDelegate));
}

} /* namespace gui */


#endif /* DATAMODELCOMMONPARAMETERWIDGET_HPP_ */
