/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelOverviewWidget.hpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELOVERVIEWWIDGET_HPP_
#define DATAMODELOVERVIEWWIDGET_HPP_

#include <QWidget>
#include <QSpacerItem>
#include <QGroupBox>
#include <boost/shared_ptr.hpp>
#include "GroupBoxLayoutBuilder.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "DataModelCommonParameterDisplayWidget.hpp"
#include "DataModelTimeseriesDisplayWidget.hpp"
#include "DataModelAssignmentDisplayWidget.hpp"
#include "DataModelAnalysisWidget.hpp"

namespace gui {

template<typename DataModel, typename AnalysisToolsDescription>
class DataModelOverviewWidget : public QWidget {
public:
	typedef typename DataModelAnalysisWidget<AnalysisToolsDescription>::OverviewCategoryWidget_t OverviewCategoryWidget_t;
public:
	DataModelOverviewWidget(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<AnalysisToolsDescription> analysisToolsDescription);
	virtual ~DataModelOverviewWidget();

	COM_SUBSCRIPTION_DELEGATE_NAMED_FUNCTION(DataModelCommonParameterDisplayWidget<DataModel>, commonParameterDisplayWidget, Edit, EditCommonParameters);
	COM_SUBSCRIPTION_DELEGATE_NAMED_FUNCTION(DataModelTimeseriesDisplayWidget<DataModel>, timeSeriesDisplayWidget, Edit, EditTimeSeries);
	COM_SUBSCRIPTION_DELEGATE_NAMED_FUNCTION(DataModelAssignmentDisplayWidget<DataModel>, assignmentDisplayWidget, Edit, EditAssignment);

	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelAnalysisWidget<AnalysisToolsDescription>, dataModelAnalysisWidget, AddAnalysisTool);
	COM_SUBSCRIPTION_DELEGATE_NAMED_FUNCTION(DataModelAnalysisWidget<AnalysisToolsDescription>, leftSideDataModelAnalysisWidget, AddAnalysisTool, AddLeftSideAnalysisTool);

	OverviewCategoryWidget_t *getLeftSideOverviewCategoryWidget() {
		return leftSideDataModelAnalysisWidget->getOverviewCategoryWidget();
	}

	OverviewCategoryWidget_t *getRightSideOverviewCategoryWidget() {
		return dataModelAnalysisWidget->getOverviewCategoryWidget();
	}

private:
	void setupLayout();

private:
	boost::shared_ptr<DataModel> dataModel;
	DataModelCommonParameterDisplayWidget<DataModel> *commonParameterDisplayWidget;
	DataModelTimeseriesDisplayWidget<DataModel> *timeSeriesDisplayWidget;
	DataModelAssignmentDisplayWidget<DataModel> *assignmentDisplayWidget;

	DataModelAnalysisWidget<AnalysisToolsDescription> *dataModelAnalysisWidget;
	DataModelAnalysisWidget<AnalysisToolsDescription> *leftSideDataModelAnalysisWidget;

	QWidget *dataDisplayWidgetContainer;
	QWidget *dataAnalysisWidgetContainer;
	boost::shared_ptr<AnalysisToolsDescription> analysisToolsDescription;
};

template<typename DataModel, typename AnalysisToolsDescription>
inline DataModelOverviewWidget<DataModel, AnalysisToolsDescription>::DataModelOverviewWidget(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<AnalysisToolsDescription> analysisToolsDescription)
	:dataModel(dataModel), analysisToolsDescription(analysisToolsDescription)
{
	setupLayout();
}

template<typename DataModel, typename AnalysisToolsDescription>
inline DataModelOverviewWidget<DataModel, AnalysisToolsDescription>::~DataModelOverviewWidget() {
}


template<typename DataModel, typename AnalysisToolsDescription>
inline void DataModelOverviewWidget<DataModel, AnalysisToolsDescription>::setupLayout() {
	commonParameterDisplayWidget = new DataModelCommonParameterDisplayWidget<DataModel>(dataModel, this);
	timeSeriesDisplayWidget = new DataModelTimeseriesDisplayWidget<DataModel>(dataModel, this);
	assignmentDisplayWidget = new DataModelAssignmentDisplayWidget<DataModel>(dataModel, this);

	QGroupBox *dataGroupBox = new QGroupBox(tr("Data"), this);
	QVBoxLayout *dataGroupBoxLayout = new QVBoxLayout(this);
	dataGroupBoxLayout->addWidget(commonParameterDisplayWidget);
	dataGroupBoxLayout->addWidget(timeSeriesDisplayWidget);
	dataGroupBoxLayout->addWidget(assignmentDisplayWidget);

	//LEFT SIDE ANALYSIS
	leftSideDataModelAnalysisWidget = new DataModelAnalysisWidget<AnalysisToolsDescription>(analysisToolsDescription, this);
	dataGroupBoxLayout->addWidget(leftSideDataModelAnalysisWidget);
	// /LEFT SIDE ANALYSIS

	//dataGroupBoxLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));
	dataGroupBox->setLayout(dataGroupBoxLayout);

	/*
	GroupBoxLayoutBuilder dataOverviewGroupBoxLayoutBuilder(tr("Data"));
	dataOverviewGroupBoxLayoutBuilder.addWidget(commonParameterDisplayWidget);
	dataOverviewGroupBoxLayoutBuilder.addWidget(timeSeriesDisplayWidget);
	dataOverviewGroupBoxLayoutBuilder.addWidget(assignmentDisplayWidget);
	*/

	QVBoxLayout *dataDisplayWidgetContainerLayout = new QVBoxLayout(this);
	dataDisplayWidgetContainerLayout->addWidget(dataGroupBox);
	dataDisplayWidgetContainer = new QWidget(this);
	//dataDisplayWidgetContainerLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));
	dataDisplayWidgetContainer->setLayout(dataDisplayWidgetContainerLayout);


	dataModelAnalysisWidget = new DataModelAnalysisWidget<AnalysisToolsDescription>(analysisToolsDescription, this);
	GroupBoxLayoutBuilder dataModelAnalysisGroupBoxLayoutBuilder(tr("Analysis"));
	dataModelAnalysisGroupBoxLayoutBuilder.addWidget(dataModelAnalysisWidget);

	dataAnalysisWidgetContainer = new QWidget(this);
	dataAnalysisWidgetContainer->setLayout(dataModelAnalysisGroupBoxLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding), false));

	//dataAnalysisWidgetContainer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

	SplittedLayoutBuilder splittedLayoutBuilder;
	setLayout(splittedLayoutBuilder.build(this, dataDisplayWidgetContainer, dataAnalysisWidgetContainer));
}

} /* namespace gui */


#endif /* DATAMODELOVERVIEWWIDGET_HPP_ */
