/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelWizard.hpp
 *
 *  Created on: Dec 6, 2013
 *      Author: gerberma
 */

#ifndef DATAMODELWIZARD_HPP_
#define DATAMODELWIZARD_HPP_

#include <iostream>
#include <QWizard>
#include <boost/signals2/signal.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "BaseWizard.hpp"
#include "DataModelCommonParameterWidget.hpp"
#include "DataModelTimeSeriesWidget.hpp"
#include "DataModelAssignmentWizardPage.hpp"
#include "DataModelWizardPages.hpp"
#include "WizardNextIdStrategy.hpp"

namespace gui {

template<
	typename DataModel
>
class DataModelWizard : public BaseWizard {
public:
	typedef DataModelCommonParameterWidget<DataModel, WizardNextIdStrategy<DataModelWizardPages::Pages> > _common_prameter_widget_t;
	typedef DataModelTimeSeriesWidget<DataModel, WizardNextIdStrategy<DataModelWizardPages::Pages> > _timeseries_widget_t;
	typedef DataModelAssignmentWizardPage<DataModel, WizardNextIdStrategy<DataModelWizardPages::Pages> > _assignment_wizard_page_t;
	typedef typename _common_prameter_widget_t::request_source_uri_signal_t request_source_uri_signal_t;

	typedef boost::signals2::signal<void()> created_signal_t;
public:
	DataModelWizard(boost::shared_ptr<DataModel> dataModel);
	virtual ~DataModelWizard();

	boost::signals2::connection subscribeForRequestSourceUri(const typename request_source_uri_signal_t::slot_type &subscriber);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_common_prameter_widget_t, commonParameterWidget, CSVDelimiterChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_timeseries_widget_t, timeSeriesWidget, DateColumnChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_timeseries_widget_t, timeSeriesWidget, ApplyDateFormat);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_assignment_wizard_page_t, assignmentWizardPage, BehaviorIndicatorAssignmentChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_assignment_wizard_page_t, assignmentWizardPage, WaterLevelAssignmentChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_assignment_wizard_page_t, assignmentWizardPage, TemperatureAssignmentChanged);
	boost::signals2::connection subscribeForCreated(const created_signal_t::slot_type &subscriber);

	void start();
	void editCommonParameters();
	void editTimeSeries();
	void editAssignment();
	bool accepted();

private:
	boost::shared_ptr<DataModel> dataModel;
	_common_prameter_widget_t *commonParameterWidget;
	_timeseries_widget_t *timeSeriesWidget;
	_assignment_wizard_page_t *assignmentWizardPage;
	WizardNextIdStrategy<DataModelWizardPages::Pages> *nextIdStrategy;
	created_signal_t createdSignal;
};

template<typename DataModel>
inline DataModelWizard<DataModel>::DataModelWizard(boost::shared_ptr<DataModel> dataModel)
	:dataModel(dataModel)
{
	setWizardStyle(ModernStyle);

	nextIdStrategy = new WizardNextIdStrategy<DataModelWizardPages::Pages>(DataModelWizardPages::Final, this);

	commonParameterWidget = new _common_prameter_widget_t(this, dataModel, nextIdStrategy, DataModelWizardPages::CommonParameter);
	commonParameterWidget->setVisible(false);

	timeSeriesWidget = new _timeseries_widget_t(this, dataModel, nextIdStrategy, DataModelWizardPages::TimeSeries);
	timeSeriesWidget->setVisible(false);

	assignmentWizardPage = new _assignment_wizard_page_t(this, dataModel, nextIdStrategy, DataModelWizardPages::Assignment);
	assignmentWizardPage->setVisible(false);
}

template<typename DataModel>
inline DataModelWizard<DataModel>::~DataModelWizard() {
}


template<typename DataModel>
inline boost::signals2::connection DataModelWizard<DataModel>::subscribeForCreated(const created_signal_t::slot_type& subscriber) {
	return createdSignal.connect(subscriber);
}


template<typename DataModel>
inline boost::signals2::connection gui::DataModelWizard<DataModel>::subscribeForRequestSourceUri(const typename request_source_uri_signal_t::slot_type& subscriber) {
	return commonParameterWidget->subscribeForRequestSourceUri(subscriber);
}


template<typename DataModel>
inline void DataModelWizard<DataModel>::start() {
	if(commonParameterWidget == NULL
			|| timeSeriesWidget == NULL
			|| assignmentWizardPage == NULL)
	{
		return;
	}
	nextIdStrategy->setTransition(DataModelWizardPages::CommonParameter, DataModelWizardPages::TimeSeries);
	nextIdStrategy->setTransition(DataModelWizardPages::TimeSeries, DataModelWizardPages::Assignment);
	nextIdStrategy->setTransition(DataModelWizardPages::Assignment, DataModelWizardPages::Final);

	setPage(DataModelWizardPages::CommonParameter, commonParameterWidget);
	setPage(DataModelWizardPages::TimeSeries, timeSeriesWidget);
	setPage(DataModelWizardPages::Assignment, assignmentWizardPage);
	setStartId(DataModelWizardPages::CommonParameter);

	exec(); //execute synchronously
}


template<typename DataModel>
inline void DataModelWizard<DataModel>::editCommonParameters() {
	start();
}


template<typename DataModel>
inline void DataModelWizard<DataModel>::editTimeSeries() {
	if(timeSeriesWidget == NULL) {
		return;
	}

	nextIdStrategy->setTransition(DataModelWizardPages::TimeSeries, DataModelWizardPages::Final);

	/*
	removePage(DataModelWizardPages::CommonParameter);
	removePage(DataModelWizardPages::TimeSeries);
	removePage(DataModelWizardPages::Assignment);
	*/

	setPage(DataModelWizardPages::TimeSeries, timeSeriesWidget);
	setStartId(DataModelWizardPages::TimeSeries);

	exec(); //execute synchronously
}


template<typename DataModel>
inline void DataModelWizard<DataModel>::editAssignment() {
	if(assignmentWizardPage == NULL) {
		return;
	}

	nextIdStrategy->setTransition(DataModelWizardPages::Assignment, DataModelWizardPages::Final);

	setPage(DataModelWizardPages::Assignment, assignmentWizardPage);
	setStartId(DataModelWizardPages::Assignment);

	exec(); //execute synchronously
}

template<typename DataModel>
inline bool DataModelWizard<DataModel>::accepted() {
	return result() == Accepted;
}

//button(QWizard::NextButton)

} /* namespace gui */


#endif /* DATAMODELWIZARD_HPP_ */
