/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * NavigationTreeWidget.hpp
 *
 *  Created on: Nov 27, 2013
 *      Author: gerberma
 */

#ifndef NAVIGATIONTREEWIDGET_HPP_
#define NAVIGATIONTREEWIDGET_HPP_

#include <QObject>
#include <QTreeWidget>
#include <QAction>
#include <QMouseEvent>
#include <QApplication>
#include <QMenu>
#include <QString>
#include <QDebug>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/signals2/signal.hpp>
#include <boost/functional/value_factory.hpp>
#include <iostream>
#include <algorithm>
#include "Delegates.hpp"
#include "communication/Signals.hpp"

namespace gui {


/**
 * This is a helper class which adds type safety to selected item stuff for QTreeWidget which by itself
 * is not able to say which model has been selected by which tree item. With the help of a lot of boost bind
 * we are able to call member functions of a derived QTreeWidget with models assigned in a type safe way.
 */
class NavigationTreeItem : public QTreeWidgetItem {
public:
	typedef boost::function<QString()> TextProvider_t;
	typedef boost::function<void(const QPoint &, QMenu &)> ContextMenuProvider_t;
	typedef boost::signals2::signal<void(NavigationTreeItem *, int)> selected_signal_t;
public:
	NavigationTreeItem(QString prefix, TextProvider_t textProvider, ContextMenuProvider_t contextMenuProvider)
		:QTreeWidgetItem(), prefix(prefix), textProvider(textProvider), contextMenuProvider(contextMenuProvider)
	{
		updateValues();
	}

	virtual ~NavigationTreeItem() {
		connection.disconnect();
	}

	void setConnection(boost::signals2::connection connection) {
		this->connection = connection;
	}

	void updateValues() {
		QString value = QString("(") + prefix + QString(") ") + textProvider();
		setText(0, value);
	}

	boost::signals2::connection subscribeForSelected(const selected_signal_t::slot_type &subscriber) {
		return selectedSignal.connect(subscriber);
	}

	void selected(int column) {
		selectedSignal(this, column);
	}

	void showContextMenu(const QPoint &pos, QMenu &ctxMenu) {
		contextMenuProvider(pos, ctxMenu);
	}

	void setTextProvider(TextProvider_t textProvider) {
		this->textProvider = textProvider;
	}

private:
	QString prefix;
	TextProvider_t textProvider;
	ContextMenuProvider_t contextMenuProvider;
	boost::signals2::connection connection;
	selected_signal_t selectedSignal;
};

namespace priv {


class NavigationTreeWidgetQtSignalsAndSlots : public QTreeWidget
{
	Q_OBJECT
public:
	NavigationTreeWidgetQtSignalsAndSlots(QWidget *parent = 0)
		:QTreeWidget(parent){}
	virtual ~NavigationTreeWidgetQtSignalsAndSlots(){}

private slots:
	virtual void showContextMenu(const QPoint &) = 0;
	virtual void createAnalysisSlot() = 0;
	virtual void qtreeWigetItemClicked(QTreeWidgetItem *, int) = 0;

};

}/* namespace priv */

template<typename P>
class NavigationTreeWidget : public priv::NavigationTreeWidgetQtSignalsAndSlots {
	typedef typename P::Analysis_t Analysis_t;
	typedef typename Analysis_t::DataModel_t DataModel_t;
	typedef typename Analysis_t::RegressionModel_t RegressionModel_t;
public:
	typedef boost::signals2::signal<void(boost::shared_ptr<P>)> create_analysis_signal_t;
	typedef typename create_analysis_signal_t::slot_type slot_t;
	typedef boost::signals2::signal<void(boost::shared_ptr<Analysis_t>)> analysis_clicked_signal_t;
	typedef typename analysis_clicked_signal_t::slot_type analysis_clicked_signal_slot_t;
	typedef boost::signals2::signal<void(boost::shared_ptr<Analysis_t>)>create_data_model_signal_t;
	typedef typename create_data_model_signal_t::slot_type create_data_model_signal_slot_t;

public:
	NavigationTreeWidget(QWidget *parent = 0);
	virtual ~NavigationTreeWidget();


	void show(boost::shared_ptr<P> project);

	boost::signals2::connection subscribeForCreateAnalysis(const slot_t &subscriber);
	boost::signals2::connection subscribeForAnalysisDoubleClicked(const analysis_clicked_signal_slot_t &subscriber);
	boost::signals2::connection subscribeForCreateDataModel(const create_data_model_signal_slot_t &subscriber);
	COM_SUBSCRIPTION_FUNCTION(ProjectDoubleClicked, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(DataModelDoubleClicked, communication::Signal<void(boost::shared_ptr<DataModel_t>, boost::shared_ptr<Analysis_t>)>);
	COM_SUBSCRIPTION_FUNCTION(CreateRegressionModel, communication::Signal<void(boost::shared_ptr<Analysis_t>)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysis, communication::Signal<void(boost::shared_ptr<Analysis_t>, boost::shared_ptr<P>)>);
	COM_SUBSCRIPTION_FUNCTION(RegressionModelDoubleClicked, communication::Signal<void(boost::shared_ptr<RegressionModel_t>, boost::shared_ptr<Analysis_t>)>);
	COM_SUBSCRIPTION_FUNCTION(CreateReport, communication::Signal<void(boost::shared_ptr<P>, boost::shared_ptr<Analysis_t>, boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(CreateExport, communication::Signal<void(boost::shared_ptr<P>, boost::shared_ptr<Analysis_t>, boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(LoadDataModelSource, communication::Signal<void(boost::shared_ptr<DataModel_t>,boost::shared_ptr<Analysis_t>)>);
	COM_SUBSCRIPTION_FUNCTION(EditDataModelTimeSeries, communication::Signal<void(boost::shared_ptr<DataModel_t>, boost::shared_ptr<Analysis_t>)>);
	COM_SUBSCRIPTION_FUNCTION(EditDataModelAssignment, communication::Signal<void(boost::shared_ptr<DataModel_t>, boost::shared_ptr<Analysis_t>)>);
	COM_SUBSCRIPTION_FUNCTION(EditRegressionCommonParameters, communication::Signal<void(boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(EditRegressionTimeRange, communication::Signal<void(boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(EditRegressionBehaviorIndicator, communication::Signal<void(boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(EditRegressionFormula, communication::Signal<void(boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(ExecuteRegression, communication::Signal<void(boost::shared_ptr<RegressionModel_t>)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveRegression, communication::Signal<void(boost::shared_ptr<RegressionModel_t>, boost::shared_ptr<Analysis_t>)>);

private:
	void setupRootItem();
	void setupLayout();
	void analysisAdded(boost::shared_ptr<Analysis_t> analysis);
	void removeAnalysis(boost::shared_ptr<Analysis_t>, boost::shared_ptr<Analysis_t>, QTreeWidgetItem *analysisItem);
	void dataModelSet(boost::shared_ptr<Analysis_t>analysis, QTreeWidgetItem *analysisItem);
	void regressionModelAdded(boost::shared_ptr<Analysis_t>analysis, boost::shared_ptr<RegressionModel_t> regressionModel, QTreeWidgetItem *analysisItem);
	void removeRegressionModel(boost::shared_ptr<RegressionModel_t>, boost::shared_ptr<RegressionModel_t>, QTreeWidgetItem *analysisItem);
	void showContextMenu(const QPoint &);
	void showContextMenuFor(boost::shared_ptr<Analysis_t>analysis, const QPoint &pos, QMenu &ctxMenu);
	void showContextMenuFor(boost::shared_ptr<DataModel_t>dataModel, boost::shared_ptr<Analysis_t>analysis, const QPoint &pos, QMenu &ctxMenu);
	void showContextMenuFor(boost::shared_ptr<RegressionModel_t> regressionModel, boost::shared_ptr<Analysis_t> analysis, const QPoint &pos, QMenu &ctxMenu);
	void showContextMenuForProject(const QPoint &pos, QMenu &ctxMenu);
	void createAnalysisSlot();
	void qtreeWigetItemClicked(QTreeWidgetItem *item, int column);
	void navigationTreeItemDoubleClicked(boost::shared_ptr<P> project);
	void navigationTreeItemDoubleClicked(NavigationTreeItem *item, int column, boost::shared_ptr<P> project);
	void navigationTreeItemDoubleClicked(NavigationTreeItem *item, int column, boost::shared_ptr<Analysis_t> analysis);
	void navigationTreeItemDoubleClicked(NavigationTreeItem *item, int column, boost::shared_ptr<DataModel_t> dataModel, boost::shared_ptr<Analysis_t> analysis);
	void navigationTreeItemDoubleClicked(NavigationTreeItem *item, int column, boost::shared_ptr<RegressionModel_t> regressionModel, boost::shared_ptr<Analysis_t> analysis);


private:
	boost::shared_ptr<P> project;
	QAction *createAnalysisAction;
	create_analysis_signal_t createAnalysisSignal;
	analysis_clicked_signal_t analysisDoubleClickedSignal;
	create_data_model_signal_t createDataModelSignal;
	QTreeWidgetItem *rootItem;
};

} /* namespace gui */

template<typename P>
inline gui::NavigationTreeWidget<P>::NavigationTreeWidget(QWidget* parent)
	:NavigationTreeWidgetQtSignalsAndSlots(parent)
{
	setupLayout();
}

template<typename P>
inline gui::NavigationTreeWidget<P>::~NavigationTreeWidget() {
}


template<typename P>
inline void gui::NavigationTreeWidget<P>::setupRootItem() {
	QTreeWidgetItem* headerItem = new QTreeWidgetItem();
	headerItem->setText(0, tr("Navigation"));
	setHeaderItem(headerItem);

	rootItem = new NavigationTreeItem(
			QString("p"),
			boost::bind(boost::value_factory<QString>(), tr("Project")),
			boost::bind(&NavigationTreeWidget<P>::showContextMenuForProject, this, _1, _2)
			);
	rootItem->setExpanded(true);
	addTopLevelItem(rootItem);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::show(boost::shared_ptr<P> project) {
	clear();

	setupRootItem();
	this->project = project;

	//update local model
	BOOST_FOREACH(boost::shared_ptr<Analysis_t> analysis, project->getAnalyses()) {
		analysisAdded(analysis);
	}
	/*
	std::for_each(project->getAnalyses().begin(), project->getAnalyses().end(),
			boost::bind(&NavigationTreeWidget<P>::analysisAdded, this, _1)
	);
	*/
	setCurrentItem(rootItem);

	NavigationTreeItem *rootItemNavigationTreeItem = dynamic_cast<NavigationTreeItem *>(rootItem);
	rootItemNavigationTreeItem->setTextProvider(boost::bind(&P::getName, project));
	rootItemNavigationTreeItem->updateValues();
	rootItemNavigationTreeItem->setConnection(project->subscribe(boost::bind(&NavigationTreeItem::updateValues, rootItemNavigationTreeItem)));
	rootItemNavigationTreeItem->subscribeForSelected(boost::bind((void(NavigationTreeWidget<P>::*)(NavigationTreeItem*, int, boost::shared_ptr<P>))&NavigationTreeWidget<P>::navigationTreeItemDoubleClicked, this, _1, _2, project));

	project->subscribeForAnalysisAdded(boost::bind(&NavigationTreeWidget<P>::analysisAdded, this, _1));
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::setupLayout() {
	QTreeWidgetItem* headerItem = new QTreeWidgetItem();
	setupRootItem();

	setStyleSheet("background-color:transparent;");
	setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(showContextMenu(const QPoint&)));

	createAnalysisAction = new QAction(tr("Create analysis"), this);
	createAnalysisAction->setEnabled(false);
	connect(createAnalysisAction, SIGNAL(triggered()), this, SLOT(createAnalysisSlot()));

	//style
	setExpandsOnDoubleClick(false);
	setIndentation(10);
	setWordWrap(true);
	//setRootIsDecorated(false);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::showContextMenu(const QPoint &pos) {
	QMenu ctxMenu;
	NavigationTreeItem *currentNavigationTreeItem = static_cast<NavigationTreeItem *>(currentItem());
	currentNavigationTreeItem->showContextMenu(mapToGlobal(pos), ctxMenu);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::showContextMenuForProject(const QPoint &pos, QMenu &ctxMenu) {
	using namespace boost;
	using namespace communication;

	std::cout << __FUNCTION__ << " - project " << std::endl;
	if(project == NULL) {
		return;
	}


	ctxMenu.addAction(createAnalysisAction);
	createAnalysisAction->setEnabled(project->isNewAnalysisAllowed());

	QAction *showProjectAction = new QAction(tr("Show project"), &ctxMenu);
	QtBoostDelegate *showProjectDelegate = new QtBoostDelegate(bind<void>(ref(ProjectDoubleClickedSignal)), &ctxMenu);
	connect(showProjectAction, SIGNAL(triggered()), showProjectDelegate, SLOT(execute()));
	ctxMenu.addAction(showProjectAction);

	ctxMenu.exec(pos);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::showContextMenuFor(boost::shared_ptr<Analysis_t> analysis, const QPoint &pos, QMenu &ctxMenu) {
	using namespace communication;
	using namespace boost;

	QAction *createDataModelAction = new QAction(tr("Create data model"), &ctxMenu);
	createDataModelAction->setEnabled(analysis->isNewDataModelAllowed());

	QAction *createRegressionModelAction = new QAction(tr("Create regression model"), &ctxMenu);
	createRegressionModelAction->setEnabled(analysis->isNewRegressionModelAllowed());

	QtBoostDelegate *createDelegate = new QtBoostDelegate(bind<void>(ref(createDataModelSignal), analysis), &ctxMenu);
	connect(createDataModelAction, SIGNAL(triggered()), createDelegate, SLOT(execute()));

	QtBoostDelegate *createRegressionModelDelegate = new QtBoostDelegate(bind<void>(ref(CreateRegressionModelSignal), analysis), &ctxMenu);
	connect(createRegressionModelAction, SIGNAL(triggered()), createRegressionModelDelegate, SLOT(execute()));

	QAction *removeAnalysisAction = new QAction(tr("Remove analysis"), &ctxMenu);
	QtBoostDelegate *removeAnalysisDelegate = new QtBoostDelegate(bind<void>(ref(RemoveAnalysisSignal), analysis, project), &ctxMenu);
	connect(removeAnalysisAction, SIGNAL(triggered()), removeAnalysisDelegate, SLOT(execute()));

	QAction *showAnalysisAction = new QAction(tr("Show analysis"), &ctxMenu);
	QtBoostDelegate *showAnalysisDelegate = new QtBoostDelegate(bind<void>(ref(analysisDoubleClickedSignal), analysis), &ctxMenu);
	connect(showAnalysisAction, SIGNAL(triggered()), showAnalysisDelegate, SLOT(execute()));

	ctxMenu.addAction(createDataModelAction);
	ctxMenu.addAction(createRegressionModelAction);
	ctxMenu.addAction(removeAnalysisAction);
	ctxMenu.addAction(showAnalysisAction);

	ctxMenu.exec(pos);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::showContextMenuFor(boost::shared_ptr<DataModel_t> dataModel, boost::shared_ptr<Analysis_t> analysis, const QPoint &pos, QMenu &ctxMenu) {
	using namespace communication;
	using namespace boost;


	QAction *loadSourceAction = new QAction(tr("Load from source"), &ctxMenu);
	QtBoostDelegate *loadSourceDelegate = new QtBoostDelegate(bind<void>(ref(LoadDataModelSourceSignal), dataModel, analysis), &ctxMenu);
	connect(loadSourceAction, SIGNAL(triggered()), loadSourceDelegate, SLOT(execute()));
	ctxMenu.addAction(loadSourceAction);

	QAction *editDateColumnAction = new QAction(tr("Edit date column"), &ctxMenu);
	QtBoostDelegate *editDateColumnDelegate = new QtBoostDelegate(bind<void>(ref(EditDataModelTimeSeriesSignal), dataModel, analysis), &ctxMenu);
	connect(editDateColumnAction, SIGNAL(triggered()), editDateColumnDelegate, SLOT(execute()));
	ctxMenu.addAction(editDateColumnAction);

	QAction *editAssignmentAction = new QAction(tr("Edit assignment"), &ctxMenu);
	QtBoostDelegate *editAssignmentDelegate = new QtBoostDelegate(bind<void>(ref(EditDataModelAssignmentSignal), dataModel, analysis), &ctxMenu);
	connect(editAssignmentAction, SIGNAL(triggered()), editAssignmentDelegate, SLOT(execute()));
	ctxMenu.addAction(editAssignmentAction);

	QAction *showDataModelAction = new QAction(tr("Show data model"), &ctxMenu);
	QtBoostDelegate *showDataModelDelegate = new QtBoostDelegate(bind<void>(ref(DataModelDoubleClickedSignal), dataModel, analysis), &ctxMenu);
	connect(showDataModelAction, SIGNAL(triggered()), showDataModelDelegate, SLOT(execute()));
	ctxMenu.addAction(showDataModelAction);

	ctxMenu.exec(pos);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::showContextMenuFor(boost::shared_ptr<RegressionModel_t> regressionModel, boost::shared_ptr<Analysis_t> analysis, const QPoint &pos, QMenu &ctxMenu) {
	using namespace communication;
	using namespace boost;

	QAction *createReportAction = new QAction(tr("Create report"), &ctxMenu);
	QtBoostDelegate *createReportDelegate = new QtBoostDelegate(bind<void>(ref(CreateReportSignal), project, analysis, regressionModel), &ctxMenu);
	connect(createReportAction, SIGNAL(triggered()), createReportDelegate, SLOT(execute()));
	ctxMenu.addAction(createReportAction);

	QAction *createExportAction = new QAction(tr("Export Regression Results"), &ctxMenu);
	QtBoostDelegate *createExportDelegate = new QtBoostDelegate(bind<void>(ref(CreateExportSignal), project, analysis, regressionModel), &ctxMenu);
	connect(createExportAction, SIGNAL(triggered()), createExportDelegate, SLOT(execute()));
	ctxMenu.addAction(createExportAction);

	ctxMenu.addAction(ctxMenu.addSeparator());

	QAction *editRegressionCommonParametersAction = new QAction(tr("Edit Common parameters"), &ctxMenu);
	QtBoostDelegate *editRegressionCommonParametersDelegate = new QtBoostDelegate(bind<void>(ref(EditRegressionCommonParametersSignal), regressionModel), &ctxMenu);
	connect(editRegressionCommonParametersAction, SIGNAL(triggered()), editRegressionCommonParametersDelegate, SLOT(execute()));
	ctxMenu.addAction(editRegressionCommonParametersAction);


	QAction *editRegressionTimeRangeAction = new QAction(tr("Edit Time range"), &ctxMenu);
	QtBoostDelegate *editRegressionTimeRangeDelegate = new QtBoostDelegate(bind<void>(ref(EditRegressionTimeRangeSignal), regressionModel), &ctxMenu);
	connect(editRegressionTimeRangeAction, SIGNAL(triggered()), editRegressionTimeRangeDelegate, SLOT(execute()));
	ctxMenu.addAction(editRegressionTimeRangeAction);


	QAction *editRegressionBehaviorIndicatorAction = new QAction(tr("Edit Behavior indicator"), &ctxMenu);
	QtBoostDelegate *editRegressionBehaviorIndicatorDelegate = new QtBoostDelegate(bind<void>(ref(EditRegressionBehaviorIndicatorSignal), regressionModel), &ctxMenu);
	connect(editRegressionBehaviorIndicatorAction, SIGNAL(triggered()), editRegressionBehaviorIndicatorDelegate, SLOT(execute()));
	ctxMenu.addAction(editRegressionBehaviorIndicatorAction);

	QAction *editRegressionFormulaAction = new QAction(tr("Edit Formula"), &ctxMenu);
	QtBoostDelegate *editRegressionFormulaDelegate = new QtBoostDelegate(bind<void>(ref(EditRegressionFormulaSignal), regressionModel), &ctxMenu);
	connect(editRegressionFormulaAction, SIGNAL(triggered()), editRegressionFormulaDelegate, SLOT(execute()));
	ctxMenu.addAction(editRegressionFormulaAction);

	QAction *executeRegressionAction = new QAction(tr("Execute Regression"), &ctxMenu);
	QtBoostDelegate *executeRegressionDelegate = new QtBoostDelegate(bind<void>(ref(ExecuteRegressionSignal), regressionModel), &ctxMenu);
	connect(executeRegressionAction, SIGNAL(triggered()), executeRegressionDelegate, SLOT(execute()));
	ctxMenu.addAction(executeRegressionAction);

	QAction *removeRegressionAction = new QAction(tr("Remove Regression"), &ctxMenu);
	QtBoostDelegate *removeRegressionDelegate = new QtBoostDelegate(bind<void>(ref(RemoveRegressionSignal), regressionModel, analysis), &ctxMenu);
	connect(removeRegressionAction, SIGNAL(triggered()), removeRegressionDelegate, SLOT(execute()));
	ctxMenu.addAction(removeRegressionAction);

	QAction *showRegressionAction = new QAction(tr("Show Regression"), &ctxMenu);
	QtBoostDelegate *showRegressionDelegate = new QtBoostDelegate(bind<void>(ref(RegressionModelDoubleClickedSignal), regressionModel, analysis), &ctxMenu);
	connect(showRegressionAction, SIGNAL(triggered()), showRegressionDelegate, SLOT(execute()));
	ctxMenu.addAction(showRegressionAction);

	ctxMenu.exec(pos);
}

template<typename P>
inline boost::signals2::connection gui::NavigationTreeWidget<P>::subscribeForCreateAnalysis(const slot_t &subscriber) {
	return createAnalysisSignal.connect(subscriber);
}

template<typename P>
inline boost::signals2::connection gui::NavigationTreeWidget<P>::subscribeForAnalysisDoubleClicked(const analysis_clicked_signal_slot_t &subscriber) {
	return analysisDoubleClickedSignal.connect(subscriber);
}

template<typename P>
inline boost::signals2::connection gui::NavigationTreeWidget<P>::subscribeForCreateDataModel(const create_data_model_signal_slot_t &subscriber) {
	return createDataModelSignal.connect(subscriber);
}


template<typename P>
inline void gui::NavigationTreeWidget<P>::analysisAdded(boost::shared_ptr<Analysis_t> analysis) {
	NavigationTreeItem *navigationTreeItem;
	navigationTreeItem = new NavigationTreeItem(
			QString("a"),
			boost::bind(&Analysis_t::getName, analysis),
			boost::bind((void(NavigationTreeWidget<P>::*)(boost::shared_ptr<Analysis_t>,  const QPoint &, QMenu &))&NavigationTreeWidget<P>::showContextMenuFor, this, analysis, _1, _2)
	);
	navigationTreeItem->setConnection(analysis->subscribe(boost::bind(&NavigationTreeItem::updateValues, navigationTreeItem)));
	navigationTreeItem->subscribeForSelected(boost::bind((void(NavigationTreeWidget<P>::*)(NavigationTreeItem*, int, boost::shared_ptr<Analysis_t>))&NavigationTreeWidget<P>::navigationTreeItemDoubleClicked, this, _1, _2, analysis));
	QTreeWidgetItem *analysisItem = navigationTreeItem;
	analysisItem->setHidden(false);
	analysisItem->setExpanded(true);

	connect(this, SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int )), this, SLOT(qtreeWigetItemClicked(QTreeWidgetItem *, int)));

	analysis->subscribeForDataModelSet(boost::bind(&NavigationTreeWidget<P>::dataModelSet, this, analysis, analysisItem));
	analysis->subscribeForRegressionModelAdded(boost::bind(&NavigationTreeWidget<P>::regressionModelAdded, this, analysis, _1, analysisItem));

	if(analysis->getDataModel()) {
		dataModelSet(analysis, analysisItem);
	}

	BOOST_FOREACH(boost::shared_ptr<RegressionModel_t> regressionModel, analysis->getRegressionModels()) {
		regressionModelAdded(analysis, regressionModel, analysisItem);
	}

	project->subscribeForRemoveAnalysis(boost::bind(&NavigationTreeWidget<P>::removeAnalysis, this, analysis, _1, analysisItem));

	rootItem->addChild(analysisItem);
	rootItem->setExpanded(true);
	setCurrentItem(analysisItem);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::removeAnalysis(boost::shared_ptr<Analysis_t> originalAnalysis, boost::shared_ptr<Analysis_t> analysisToBeRemoved, QTreeWidgetItem *analysisItem) {
	if(originalAnalysis != analysisToBeRemoved) {
		return;
	}

	setCurrentItem(rootItem);
	navigationTreeItemDoubleClicked(project);
	int i = indexOfTopLevelItem(analysisItem);
	takeTopLevelItem(i);
	delete analysisItem;
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::dataModelSet(boost::shared_ptr<Analysis_t> analysis, QTreeWidgetItem *analysisItem) {
	boost::shared_ptr<DataModel_t> dataModel = analysis->getDataModel();

	NavigationTreeItem *navigationTreeItem;
	navigationTreeItem = new NavigationTreeItem(
			QString("d"),
			boost::bind(boost::value_factory<QString>(), tr("data model")),
			boost::bind((void(NavigationTreeWidget<P>::*)(boost::shared_ptr<DataModel_t>, boost::shared_ptr<Analysis_t>, const QPoint &, QMenu &))&NavigationTreeWidget<P>::showContextMenuFor, this, dataModel, analysis, _1, _2)
	);
	//navigationTreeItem->setConnection(dataModel->subscribe(boost::bind(&NavigationTreeItem::updateValues, navigationTreeItem)));
	navigationTreeItem->subscribeForSelected(boost::bind((void(NavigationTreeWidget<P>::*)(NavigationTreeItem*, int, boost::shared_ptr<DataModel_t>, boost::shared_ptr<Analysis_t>))&NavigationTreeWidget<P>::navigationTreeItemDoubleClicked, this, _1, _2, dataModel, analysis));
	QTreeWidgetItem *dataModelItem = navigationTreeItem;
	dataModelItem->setHidden(false);
	dataModelItem->setExpanded(true);

	analysisItem->addChild(dataModelItem);
	analysisItem->setExpanded(true);
	setCurrentItem(dataModelItem);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::regressionModelAdded(boost::shared_ptr<Analysis_t> analysis, boost::shared_ptr<RegressionModel_t> regressionModel, QTreeWidgetItem *analysisItem) {
	using namespace boost;
	qDebug() << __FUNCTION__ << " : " << regressionModel->getName();

	NavigationTreeItem *navigationTreeItem;
	navigationTreeItem = new NavigationTreeItem(
			QString("r"),
			//bind(value_factory<QString>(), regressionModel->getName()),
			bind(&RegressionModel_t::getName, regressionModel),
			bind((void(NavigationTreeWidget<P>::*)(boost::shared_ptr<RegressionModel_t>, boost::shared_ptr<Analysis_t>, const QPoint &, QMenu &))&NavigationTreeWidget<P>::showContextMenuFor, this, regressionModel, analysis, _1, _2)
	);
	navigationTreeItem->setConnection(regressionModel->subscribe(boost::bind(&NavigationTreeItem::updateValues, navigationTreeItem)));
	navigationTreeItem->subscribeForSelected(bind((void(NavigationTreeWidget<P>::*)(NavigationTreeItem *, int, boost::shared_ptr<RegressionModel_t>, boost::shared_ptr<Analysis_t>))&NavigationTreeWidget<P>::navigationTreeItemDoubleClicked, this, _1, _2, regressionModel, analysis));
	QTreeWidgetItem *regressionModelItem = navigationTreeItem;
	regressionModelItem->setHidden(false);
	regressionModelItem->setExpanded(true);

	analysisItem->addChild(regressionModelItem);
	analysisItem->setExpanded(true);
	setCurrentItem(regressionModelItem);

	analysis->subscribeForRemoveRegressionModel(boost::bind(&NavigationTreeWidget<P>::removeRegressionModel, this, regressionModel, _1, regressionModelItem));
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::removeRegressionModel(boost::shared_ptr<RegressionModel_t> originalRegressionModel, boost::shared_ptr<RegressionModel_t> regressionModelToBeRemoved, QTreeWidgetItem *regressionModelItem) {
	if(originalRegressionModel != regressionModelToBeRemoved) {
		return;
	}

	setCurrentItem(rootItem);
	navigationTreeItemDoubleClicked(project);
	int i = indexOfTopLevelItem(regressionModelItem);
	takeTopLevelItem(i);
	delete regressionModelItem;
}


template<typename P>
inline void gui::NavigationTreeWidget<P>::qtreeWigetItemClicked(QTreeWidgetItem* item, int column) {
	NavigationTreeItem *i = static_cast<NavigationTreeItem *>(item);
	i->selected(column);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::createAnalysisSlot() {
	createAnalysisSignal(project);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::navigationTreeItemDoubleClicked(boost::shared_ptr<P>) {
	ProjectDoubleClickedSignal();
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::navigationTreeItemDoubleClicked(NavigationTreeItem* , int, boost::shared_ptr<P>) {
	ProjectDoubleClickedSignal();
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::navigationTreeItemDoubleClicked(NavigationTreeItem* , int, boost::shared_ptr<Analysis_t> analysis) {
	//now we are type save again :-)
	analysisDoubleClickedSignal(analysis);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::navigationTreeItemDoubleClicked(NavigationTreeItem* , int, boost::shared_ptr<DataModel_t> dataModel, boost::shared_ptr<Analysis_t> analysis) {
	DataModelDoubleClickedSignal(dataModel, analysis);
}

template<typename P>
inline void gui::NavigationTreeWidget<P>::navigationTreeItemDoubleClicked(NavigationTreeItem* , int, boost::shared_ptr<RegressionModel_t> regressionModel, boost::shared_ptr<Analysis_t> analysis) {
	RegressionModelDoubleClickedSignal(regressionModel, analysis);
}

#endif /* NAVIGATIONTREEWIDGET_HPP_ */
