/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Plottable.hpp
 *
 *  Created on: 29.04.2015
 *      Author: gerberma
 */

#ifndef PLOTTABLE_HPP_
#define PLOTTABLE_HPP_

#include <QObject>
#include <QMouseEvent>
#include <QMenu>
#include <boost/bind.hpp>
#include <boost/random.hpp>
#include <boost/generator_iterator.hpp>
#include <map>
#include <vector>
#include "qcustomplot.h"

namespace gui {

class Plottable : public QObject {
public:
	Plottable(QWidget *widgetParent) : QObject(widgetParent), widgetParent(widgetParent) {
		setupLayout();
		connectLocal();
	}

	virtual ~Plottable(){;}

	QCustomPlot *getPlot() {
		return plot;
	}

	void rescaleAxes() {
		plot->rescaleAxes();
	}

	void replot() {
		plot->replot();
	}

	void preUpdate() {
		//track state of each graph before cleaning up:
		for(int graphIndex = 0; graphIndex < plot->graphCount(); ++graphIndex) {
			QCPGraph *graph = plot->graph(graphIndex);
			currentGraphStates[graph->name()] = graph->visible();
		}

		itemGraphMap.clear();
		plot->clearGraphs();
		plot->clearPlottables();
	}

	void postUpdate() {
		//recreate current graph state

		for(int graphIndex = 0; graphIndex < plot->graphCount(); ++graphIndex) {
			QCPGraph *graph = plot->graph(graphIndex);
			if(currentGraphStates.find(graph->name()) != currentGraphStates.end()) {
				QCPPlottableLegendItem *item = plot->legend->itemWithPlottable(graph);
				if(item != NULL) {
					enableDisableGraph(item, graph, currentGraphStates[graph->name()]);
				}
			}
		}

		replot();
	}

	void addGraphToLastLegendItem(QCPGraph *graph) {
		itemGraphMap[plot->legend->item(plot->legend->itemCount() - 1)].push_back(graph);
	}

	// from: http://stackoverflow.com/questions/43044/algorithm-to-randomly-generate-an-aesthetically-pleasing-color-palette
	QColor getRandomColor() {
		QColor mix = QColor::fromRgb(128, 128, 128);
		/*
	    int red = getRandomNumber();
	    int green = getRandomNumber();
	    int blue = getRandomNumber();
	    */
		int red = std::rand() % 255;
		int green = std::rand() % 255;
		int blue = std::rand() % 255;

	    // mix the color
		red = ((red + mix.red())%255);
		green = ((green + mix.green())%255);
		blue = ((blue + mix.blue())%255);

	    return QColor::fromRgb(red, green, blue);
	}
private:
	void setupLayout() {
		plot = new QCustomPlot(widgetParent);
		//plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom | QCP::iSelectLegend | QCP::iSelectPlottables);
		plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);


		QCPLayoutElement *placeholder = new QCPLayoutElement;
		QCPLayoutGrid *subLayout = new QCPLayoutGrid;
		plot->plotLayout()->addElement(0, 1, subLayout);
		plot->plotLayout()->setColumnStretchFactor(0, 5);
		subLayout->addElement(0, 0, plot->legend);
		subLayout->addElement(1, 0, placeholder);
		subLayout->setRowStretchFactor(0, 0.01); //legend cell in row 0 should take up as little vertical space as possible
		plot->legend->setVisible(true);
		plot->legend->setSelectableParts(QCPLegend::spItems);
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		QtBoostDelegate *selectionChangedDelegate = new QtBoostDelegate(bind(&Plottable::selectionChanged, this), this);
		connect(plot, SIGNAL(selectionChangedByUser()), selectionChangedDelegate, SLOT(execute()));

		QtBoostQCustomPlotContextMenuDelegate *contextMenuRequestedDelegate = new QtBoostQCustomPlotContextMenuDelegate(bind(&Plottable::contextMenuRequestedOnLegendItem, this, _1, _2, _3), this);
		connect(plot, SIGNAL(legendClick (QCPLegend *, QCPAbstractLegendItem *, QMouseEvent *)), contextMenuRequestedDelegate, SLOT(execute(QCPLegend *, QCPAbstractLegendItem *, QMouseEvent *)));
	}

	void contextMenuRequestedOnLegendItem(QCPLegend *legend, QCPAbstractLegendItem *abstractLegendItem, QMouseEvent *mouseEvent) {
		using namespace boost;
		using namespace communication;

		for (int i=0; i< plot->graphCount(); ++i)
		  {
			QCPGraph *graph = plot->graph(i);
			QCPPlottableLegendItem *item = plot->legend->itemWithPlottable(graph);
			if(item != NULL && item == abstractLegendItem) {
				if(mouseEvent->button() == Qt::RightButton) {
					QMenu *menu = new QMenu(widgetParent);
					menu->setAttribute(Qt::WA_DeleteOnClose);
					QtBoostDelegate *enableDisableDelegate = new QtBoostDelegate(bind(&Plottable::toggleEnableDisableGraph, this, item, graph), this);
					menu->addAction(graph->visible() ? tr("Disable") : tr("Enable"), enableDisableDelegate, SLOT(execute()));
					menu->popup(mouseEvent->globalPos());
				}
			}
		  }
	}

	void toggleEnableDisableGraph(QCPPlottableLegendItem *item, QCPGraph *graph) {
		enableDisableGraph(item, graph, !graph->visible());
		plot->replot();
	}

	void enableDisableGraph(QCPPlottableLegendItem *item, QCPGraph *, bool shouldBeVisible) {
		if(shouldBeVisible == true) {
			item->setTextColor(QColor(0,0,0));
		}else{
			item->setTextColor(QColor(200,200,200));
		}

		BOOST_FOREACH(QCPGraph *graph, itemGraphMap[item]) {
			graph->setVisible(shouldBeVisible);
		}
	}

	//derived from the interaction example of the QCustomPlot distribution
	void selectionChanged() {
	  // synchronize selection of graphs with selection of corresponding legend items:
	  for (int i=0; i< plot->graphCount(); ++i)
	  {
		QCPGraph *graph = plot->graph(i);
		QCPPlottableLegendItem *item = plot->legend->itemWithPlottable(graph);
		if(item != NULL) {
			if (item->selected() || graph->selected())
			{
			  item->setSelected(true);
			  graph->setSelected(true);
			}
		}
	  }
	}

	int getRandomNumber() {
	  using namespace std;
	  typedef boost::mt19937 RNGType;
	  RNGType rng;
	  boost::uniform_int<> one_to_six( 1, 255);
	  boost::variate_generator< RNGType, boost::uniform_int<> >
					dice(rng, one_to_six);

	  return dice();
	}

private:
	QWidget *widgetParent;
	QCustomPlot *plot;
	std::map<QCPAbstractLegendItem *, std::vector<QCPGraph *> > itemGraphMap;
	std::map<QString, bool> currentGraphStates;
};

}  // namespace gui


#endif /* PLOTTABLE_HPP_ */
