/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectOverviewWindow.hpp
 *
 *  Created on: Nov 7, 2013
 *      Author: gerberma
 */

#ifndef PROJECTOVERVIEWWIDGET_HPP_
#define PROJECTOVERVIEWWIDGET_HPP_

#include <QWidget>
#include <QObject>
#include <QLabel>
#include <QPushButton>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QSpacerItem>
#include <boost/bind.hpp>
#include <boost/signals2/signal.hpp>
#include <boost/shared_ptr.hpp>

namespace gui {

namespace priv {

class ProjectOverviewWidgetSignalsSlots : public QWidget {
	Q_OBJECT
private slots:
	virtual void qtEditSlot() = 0;
};

class DirtyProjectOverviewWidgetBaseClass : public ProjectOverviewWidgetSignalsSlots {
};

} /* namespace priv */


template<typename P>
class ProjectOverviewWidget : public priv::DirtyProjectOverviewWidgetBaseClass {
public:
	typedef boost::signals2::signal<void()> edit_signal_t;
public:
	ProjectOverviewWidget();
	virtual ~ProjectOverviewWidget();

	void show(boost::shared_ptr<P> project);

	boost::signals2::connection subscribeForEdit(const edit_signal_t::slot_type &subscriber);

private:
	void setupLayout();
	void updateValues();
	void qtEditSlot();

private:
	boost::shared_ptr<P> project;
	QLabel *nameLabel;
	QLabel *nameValueLabel;
	QLabel *descriptionLabel;
	QLabel *descriptionValueLabel;
	QLabel *damOperatorLabel;
	QLabel *damOperatorValueLabel;
	QPushButton *editButton;
	boost::signals2::connection connection;
	edit_signal_t editSignal;

};

template<typename P>
inline ProjectOverviewWidget<P>::ProjectOverviewWidget()
{
	setupLayout();
}

template<typename P>
inline ProjectOverviewWidget<P>::~ProjectOverviewWidget() {
	connection.disconnect();
}

template<typename P>
inline void ProjectOverviewWidget<P>::show(boost::shared_ptr<P> project) {
	this->project = project;
	connection = project->subscribe(boost::bind(&ProjectOverviewWidget<P>::updateValues, this));
	updateValues();
}

template<typename P>
inline void ProjectOverviewWidget<P>::setupLayout() {
	//setStyleSheet("background-color:white;");
	nameLabel = new QLabel(tr("Name"));
	nameLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	nameValueLabel = new QLabel;
	nameValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	nameLabel->setBuddy(nameValueLabel);

	descriptionLabel = new QLabel(tr("Description"));
	descriptionLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	descriptionValueLabel = new QLabel;
	descriptionValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	descriptionLabel->setBuddy(descriptionValueLabel);

	damOperatorLabel = new QLabel(tr("Dam operator"));
	damOperatorLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	damOperatorValueLabel = new QLabel;
	damOperatorValueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	damOperatorLabel->setBuddy(nameValueLabel);

	editButton = new QPushButton(tr("edit"), this);
	editButton->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

	connect(editButton, SIGNAL(clicked()), this, SLOT(qtEditSlot()));

	QVBoxLayout *mainLayout = new QVBoxLayout;

	QGroupBox *commonInformationGroupBox = new QGroupBox(tr("General information"), this);
		QGridLayout *commonInformationGrid = new QGridLayout;
		commonInformationGrid->addWidget(nameLabel, 0, 0);
		commonInformationGrid->addWidget(nameValueLabel, 0, 1, Qt::AlignLeft);
		commonInformationGrid->addWidget(descriptionLabel, 1, 0, Qt::AlignTop|Qt::AlignLeft);
		commonInformationGrid->addWidget(descriptionValueLabel, 1, 1, Qt::AlignTop|Qt::AlignLeft);
		commonInformationGrid->addWidget(damOperatorLabel, 2, 0, Qt::AlignTop|Qt::AlignLeft);
		commonInformationGrid->addWidget(damOperatorValueLabel, 2, 1, Qt::AlignTop|Qt::AlignLeft);
		commonInformationGrid->addWidget(editButton, 2, 2, Qt::AlignRight);
	commonInformationGroupBox->setLayout(commonInformationGrid);
	commonInformationGroupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	QSpacerItem *spacer = new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding);

	mainLayout->addWidget(commonInformationGroupBox);
	mainLayout->addSpacerItem(spacer);

	setLayout(mainLayout);
}

template<typename P>
inline void ProjectOverviewWidget<P>::updateValues() {
	nameValueLabel->setText(project->getName());
	descriptionValueLabel->setText(project->getDescription());
	damOperatorValueLabel->setText(project->getDamOperator());
}

template<typename P>
inline boost::signals2::connection ProjectOverviewWidget<P>::subscribeForEdit(const edit_signal_t::slot_type &subscriber) {
	return editSignal.connect(subscriber);
}

} /* namespace gui */

template<typename P>
inline void gui::ProjectOverviewWidget<P>::qtEditSlot() {
	editSignal();
}

#endif /* PROJECTOVERVIEWWIDGET_HPP_ */
