/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalBehaviorDisplayWidget.hpp
 *
 *  Created on: 21.10.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISPHYSICALBEHAVIORDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISPHYSICALBEHAVIORDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QSizePolicy>
#include <QVector>
#include <QDebug>
#include <stdlib.h> //for rand()
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include "Delegates.hpp"
#include "communication/Signals.hpp"
#include "qcustomplot.h"
#include "ComboBox.hpp"
#include "Plottable.hpp"

namespace gui {

template<typename PhysicalBehavior>
class RegressionFitAnalysisPhysicalBehaviorDisplayWidget : public QWidget {
public:
	typedef PhysicalBehavior type_t;
	typedef typename PhysicalBehavior::AssignmentCategory_t AssignmentCategory_t;
public:
	COM_SUBSCRIPTION_FUNCTION(SetAssignmentCategory, communication::Signal<void(boost::shared_ptr<PhysicalBehavior>, QString)>);
public:
	RegressionFitAnalysisPhysicalBehaviorDisplayWidget(boost::shared_ptr<PhysicalBehavior> physicalBehavior, QWidget *parent = 0)
		:QWidget(parent), physicalBehavior(physicalBehavior)
	{
		setupLayout();
		connectLocal();
		updateValues();
		plottable->rescaleAxes();
	}

	QString getName() {
		return tr("Physical behavior");
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);

		categoryComboBox = new ComboBox(this);
		categoryComboBox->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

		plottable = new Plottable(this);

		mainLayout->addWidget(categoryComboBox, Qt::AlignRight);
		mainLayout->addWidget(plottable->getPlot());

		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;

		disconnect(categoryComboBox, SIGNAL(currentIndexChanged(int)), categoryDelegate, SLOT(execute(int)));
		/**
		 * Update assignment categories
		 */
		categoryComboBox->clear();
		qDebug() << __FUNCTION__;
		BOOST_FOREACH(boost::shared_ptr<AssignmentCategory_t> assignmentCategory, physicalBehavior->getAssignmentCategories()) {
			categoryComboBox->addItem(assignmentCategory->getName(), assignmentCategory->getIdentifier());
		}
		categoryComboBox->setCurrentByValue(physicalBehavior->getAssignmentCategory()->getIdentifier());
		connect(categoryComboBox, SIGNAL(currentIndexChanged(int)), categoryDelegate, SLOT(execute(int)));


		plottable->preUpdate();
		QCustomPlot *plot = plottable->getPlot();
		int graphIndex = 0;
		typedef typename PhysicalBehavior::Item_t Item_t;
		BOOST_FOREACH(boost::shared_ptr<Item_t> item, physicalBehavior->getItems()) {
			QVector<double> applyedValues;
			BOOST_FOREACH(double value, item->getApplyedValues()) {
				applyedValues.push_back(value);
			}

			QVector<double> numericResponse;
			BOOST_FOREACH(double value, item->getNumericResponse()) {
				numericResponse.push_back(value);
			}

			posix_time::ptime time_t_epoch(gregorian::date(1970,1,1));
			QVector<double> dateResponse;
			BOOST_FOREACH(std::string stringDate, item->getDateResponse()) {
				gregorian::date thedate(gregorian::from_string(stringDate));
				posix_time::ptime t(thedate);
				posix_time::time_duration duration = t - time_t_epoch;
				dateResponse.push_back(duration.total_seconds());
			}


			plot->addGraph(); //response
			//plot->graph(graphIndex)->setName(item->getName());
			plot->graph(graphIndex)->setName(item->getName());
			plot->graph(graphIndex)->setPen(QPen(plottable->getRandomColor()));
			if(item->getResponseType() == "Date") {
				plot->graph(graphIndex)->setData(applyedValues, dateResponse);
				plot->yAxis->setTickLabelType(QCPAxis::ltDateTime);
				plot->xAxis->setTickLabelType(QCPAxis::ltNumber);
				//plot->xAxis->setDateTimeFormat("yyyy");
				plot->yAxis->setDateTimeFormat("dd.MM\nyyyy");
				plot->yAxis->setAutoTickStep(true);
				plot->yAxis->setLabel(tr("date"));
			}else if(item->getResponseType() == "numeric") {
				plot->graph(graphIndex)->setData(applyedValues, numericResponse);
				plot->yAxis->setTickLabelType(QCPAxis::ltNumber);
				plot->xAxis->setTickLabelType(QCPAxis::ltNumber);
				//plot->yAxis->setLabel(item->getName()); //Note: the name of the y axis cannot be set as it's a merge of all item values
			}
			plottable->addGraphToLastLegendItem(plot->graph(graphIndex));

			graphIndex++;
		}

		plot->xAxis->setLabel(tr("Partial displacement"));
		plot->setMinimumHeight(400);
		plottable->rescaleAxes();
		plottable->postUpdate();
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		categoryDelegate = new QtBoostIndexDelegate(
				bind(&RegressionFitAnalysisPhysicalBehaviorDisplayWidget<PhysicalBehavior>::categorySelectionChanged, this, _1)
			, this);
		connect(categoryComboBox, SIGNAL(currentIndexChanged(int)), categoryDelegate, SLOT(execute(int)));

	}

	void categorySelectionChanged(int index) {
		SetAssignmentCategorySignal(physicalBehavior, categoryComboBox->itemData(index, Qt::UserRole).toString());
	}

	boost::shared_ptr<type_t> get() {
		return physicalBehavior;
	}

	QCustomPlot *getPlot() {
		return plottable->getPlot();
	}

private:
	boost::shared_ptr<PhysicalBehavior> physicalBehavior;
	Plottable *plottable;
	ComboBox *categoryComboBox;
	communication::QtBoostIndexDelegate *categoryDelegate;
};

}  // namespace gui


#endif /* REGRESSIONFITANALYSISPHYSICALBEHAVIORDISPLAYWIDGET_HPP_ */
