/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalSummaryDisplayWidget.hpp
 *
 *  Created on: Jan 30, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISPHYSICALSUMMARYDISPLAYWIDGET_H_
#define REGRESSIONFITANALYSISPHYSICALSUMMARYDISPLAYWIDGET_H_

#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <QWidget>
#include <QMenu>
#include <QVBoxLayout>
#include <QVector>
#include <QDebug>
#include <QString>
#include <string>
#include <vector>
#include <map>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "qcustomplot.h"
#include "Plottable.hpp"


namespace gui {

template<typename RegressionFitAnalysisPhysicalSummary>
class RegressionFitAnalysisPhysicalSummaryDisplayWidget : public QWidget {
public:
	typedef RegressionFitAnalysisPhysicalSummary type_t;
public:
	RegressionFitAnalysisPhysicalSummaryDisplayWidget(boost::shared_ptr<RegressionFitAnalysisPhysicalSummary> physicalSummary, QWidget *parent = 0);
	virtual ~RegressionFitAnalysisPhysicalSummaryDisplayWidget();

	void setupLayout();
	void updateValues();

	QString getName() {
		return tr("Summary");
	}

	boost::shared_ptr<RegressionFitAnalysisPhysicalSummary> get() {
		return physicalSummary;
	}

	QCustomPlot *getPlot() {
		return plottable->getPlot();
	}

	void connectLocal() {

	}

private:
	boost::shared_ptr<RegressionFitAnalysisPhysicalSummary> physicalSummary;
	Plottable *plottable;
};

template<typename RegressionFitAnalysisPhysicalSummary>
inline RegressionFitAnalysisPhysicalSummaryDisplayWidget<RegressionFitAnalysisPhysicalSummary>::RegressionFitAnalysisPhysicalSummaryDisplayWidget(boost::shared_ptr<RegressionFitAnalysisPhysicalSummary> physicalSummary, QWidget* parent)
	:QWidget(parent), physicalSummary(physicalSummary)
{
	setupLayout();
	updateValues();
	connectLocal();
	plottable->rescaleAxes();
}

template<typename RegressionFitAnalysisPhysicalSummary>
inline RegressionFitAnalysisPhysicalSummaryDisplayWidget<RegressionFitAnalysisPhysicalSummary>::~RegressionFitAnalysisPhysicalSummaryDisplayWidget() {
}

template<typename RegressionFitAnalysisPhysicalSummary>
inline void RegressionFitAnalysisPhysicalSummaryDisplayWidget<RegressionFitAnalysisPhysicalSummary>::setupLayout() {

	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	plottable = new Plottable(this);
	mainLayout->addWidget(plottable->getPlot());
	setLayout(mainLayout);
}

template<typename RegressionFitAnalysisPhysicalSummary>
inline void RegressionFitAnalysisPhysicalSummaryDisplayWidget<RegressionFitAnalysisPhysicalSummary>::updateValues() {
	using namespace boost;

	QVector<double> behaviorIndicatorValues;
	BOOST_FOREACH(double value, physicalSummary->getBehaviorIndicatorValues()) {
		behaviorIndicatorValues.push_back(value);
	}

	QVector<double> predictions;
	BOOST_FOREACH(double value, physicalSummary->getPredictions()) {
		predictions.push_back(value);
	}

	posix_time::ptime time_t_epoch(gregorian::date(1970,1,1));

	QVector<double> dates;
	BOOST_FOREACH(std::string stringDate, physicalSummary->getDates()) {
		gregorian::date thedate(gregorian::from_string(stringDate));
		posix_time::ptime t(thedate);
		posix_time::time_duration duration = t - time_t_epoch;
		dates.push_back(duration.total_seconds());
	}

	QVector<double> predictionDates;
	BOOST_FOREACH(std::string stringDate, physicalSummary->getPredictionDates()) {
		gregorian::date thedate(gregorian::from_string(stringDate));
		posix_time::ptime t(thedate);
		posix_time::time_duration duration = t - time_t_epoch;
		predictionDates.push_back(duration.total_seconds());
	}

	/**
	 * Confidence interval
	 */
	QVector<double> ciX;
	QVector<double> lowerCiY;
	QVector<double> upperCiY;

	ciX = predictionDates;
	BOOST_FOREACH(double ci, physicalSummary->getLowerCi()) {
		lowerCiY.push_back(ci);
	}
	BOOST_FOREACH(double ci, physicalSummary->getUpperCi()) {
		upperCiY.push_back(ci);
	}

	/**
	 * Out of limit
	 */
	QVector<double> outOfLimit;
	QVector<double> outOfLimitDates;
	BOOST_FOREACH(double outOfLimitValue, physicalSummary->getOutOfLimit()) {
		outOfLimit.push_back(outOfLimitValue);
	}
	BOOST_FOREACH(std::string stringDate, physicalSummary->getOutOfLimitDates()) {
		gregorian::date thedate(gregorian::from_string(stringDate));
		posix_time::ptime t(thedate);
		posix_time::time_duration duration = t - time_t_epoch;
		outOfLimitDates.push_back(duration.total_seconds());
	}

	/**
	 * Extrapolations
	 */
	QVector<double> extrapolations;
	QVector<double> extrapolationDates;
	BOOST_FOREACH(double extrapolationValue, physicalSummary->getExtrapolations()) {
		extrapolations.push_back(extrapolationValue);
	}
	BOOST_FOREACH(std::string stringDate, physicalSummary->getExtrapolationDates()) {
		gregorian::date thedate(gregorian::from_string(stringDate));
		posix_time::ptime t(thedate);
		posix_time::time_duration duration = t - time_t_epoch;
		extrapolationDates.push_back(duration.total_seconds());
	}

	plottable->preUpdate();
	QCustomPlot *plot = plottable->getPlot();

	plot->addGraph();
	plot->graph(0)->setName(physicalSummary->getBehaviorIndicatorName());
	plot->graph(0)->setData(dates, behaviorIndicatorValues);
	plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
	plot->graph(0)->setPen(QPen(QColor(0, 0, 0)));
	plottable->addGraphToLastLegendItem(plot->graph(0));

	plot->addGraph();
	plot->graph(1)->setName(tr("Prediction"));
	plot->graph(1)->setData(predictionDates, predictions);
	plot->graph(1)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
	plottable->addGraphToLastLegendItem(plot->graph(1));

	plot->addGraph();
	plot->graph(2)->setData(ciX, lowerCiY);
	plot->graph(2)->setName(tr("95% Interval"));
	plot->graph(2)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
	plot->graph(2)->setPen( QPen ( Qt::green));
	plot->addGraph();
	plot->graph(3)->setData(ciX, upperCiY);
	plot->graph(3)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
	plot->graph(3)->setPen( QPen ( Qt::green));
	plot->legend->removeItem(plot->legend->itemCount()-1);
	plottable->addGraphToLastLegendItem(plot->graph(2));
	plottable->addGraphToLastLegendItem(plot->graph(3));

	plot->addGraph();
	plot->graph(4)->setData(outOfLimitDates, outOfLimit);
	plot->graph(4)->setName(tr("Out of limit"));
	plot->graph(4)->setLineStyle(QCPGraph::lsNone);
	plot->graph(4)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCross, Qt::red, Qt::red, 5));
	plottable->addGraphToLastLegendItem(plot->graph(4));

	plot->addGraph();
	plot->graph(5)->setData(extrapolationDates, extrapolations);
	plot->graph(5)->setName(tr("Extrapolations"));
	plot->graph(5)->setLineStyle(QCPGraph::lsNone);
	plot->graph(5)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssSquare, Qt::black, Qt::yellow, 5));
	plottable->addGraphToLastLegendItem(plot->graph(5));


	QVector<double> regressionStartDate;
	QVector<double> regressionEndDate;
	{
		gregorian::date thedate(gregorian::from_string(physicalSummary->getRegressionStartDate()));
		posix_time::ptime t(thedate);
		posix_time::time_duration duration = t - time_t_epoch;
		regressionStartDate.push_back(duration.total_seconds());
		regressionStartDate.push_back(duration.total_seconds());
	}
	{
		gregorian::date thedate(gregorian::from_string(physicalSummary->getRegressionEndDate()));
		posix_time::ptime t(thedate);
		posix_time::time_duration duration = t - time_t_epoch;
		regressionEndDate.push_back(duration.total_seconds());
		regressionEndDate.push_back(duration.total_seconds());
	}
	QVector<double> verticalValues;
	std::vector<double> stdLowerCiValues = physicalSummary->getLowerCi();
	std::vector<double> stdUpperCiValues = physicalSummary->getUpperCi();
	double minValue = *std::min_element(stdLowerCiValues.begin(), stdLowerCiValues.end());
	double maxValue = *std::max_element(stdUpperCiValues.begin(), stdUpperCiValues.end());
	verticalValues << minValue << maxValue;

	plot->addGraph();
	plot->graph(6)->setData(regressionStartDate, verticalValues);
	plot->graph(6)->setPen( QPen ( Qt::gray));
	plot->legend->removeItem(plot->legend->itemCount()-1);
	plottable->addGraphToLastLegendItem(plot->graph(6));

	plot->addGraph();
	plot->graph(7)->setData(regressionEndDate, verticalValues);
	plot->graph(7)->setPen( QPen ( Qt::gray));
	plot->legend->removeItem(plot->legend->itemCount()-1);
	plottable->addGraphToLastLegendItem(plot->graph(7));

	plot->xAxis->setTickLabelType(QCPAxis::ltDateTime);
	//plot->xAxis->setDateTimeFormat("yyyy");
	plot->xAxis->setDateTimeFormat("dd.MM\nyyyy");
	plot->xAxis->setAutoTickStep(true);
	//plot->xAxis->setTickStep(31556927+60*60+60); //2629743.83 seconds for 1 month //31556927+60*60+60 for one year (empirical estimation for this qcustom library)
	//plot->xAxis->setSubTickCount(11);

	plot->xAxis->setLabel(tr("date"));
	plot->yAxis->setLabel(physicalSummary->getBehaviorIndicatorName());
	plot->setMinimumHeight(200);

	plottable->postUpdate();
}

} /* namespace gui */


#endif /* REGRESSIONFITANALYSISPHYSICALSUMMARYDISPLAYWIDGET_H_ */
