/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorTimeseriesDisplayWidget.hpp
 *
 *  Created on: 28.10.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSORTIMESERIESDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISREGRESSORTIMESERIESDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QVector>
#include <QColor>
#include <QDebug>
#include <QRadioButton>
#include <QButtonGroup>
#include <stdlib.h> //for rand()
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/bind.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"
#include "qcustomplot.h"
#include "Plottable.hpp"

namespace gui {

template<typename RegressorTimeseries>
class RegressionFitAnalysisRegressorTimeseriesDisplayWidget : public QWidget {
public:
	COM_SUBSCRIPTION_FUNCTION(SetPolynomState, communication::Signal<void(boost::shared_ptr<RegressorTimeseries>, bool)>);
	COM_SUBSCRIPTION_FUNCTION(SetScalingState, communication::Signal<void(boost::shared_ptr<RegressorTimeseries>, bool)>);
public:
	typedef RegressorTimeseries type_t;
public:
	RegressionFitAnalysisRegressorTimeseriesDisplayWidget(boost::shared_ptr<RegressorTimeseries> regressorTimeseries, QWidget *parent = 0)
		:QWidget(parent), regressorTimeseries(regressorTimeseries)
	{
		setupLayout();
		connectLocal();
		updateValues();
		plottable->rescaleAxes();
	}

	QString getName() {
		return tr("Regressor time series");
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);

		asSplittedRadioButton = new QRadioButton(tr("splitted"), this);
		asPolynomRadioButton = new QRadioButton(tr("by assignment category"), this);
		polynomButtonGroup = new QButtonGroup(this);

		GroupBoxLayoutBuilder polynomLayout(tr("Grouping"));
		polynomLayout.addWidget(asSplittedRadioButton, 0, 0);
		polynomLayout.addWidget(asPolynomRadioButton, 0, 1);
		QWidget *polynomWidget = new QWidget(this);
		polynomWidget->setLayout(polynomLayout.build(this));


		isScaledRadioButton = new QRadioButton(tr("scaled"), this);
		isNotScaledRadioButton = new QRadioButton(tr("not scaled"), this);
		scaledButtonGroup = new QButtonGroup(this);

		GroupBoxLayoutBuilder scaledLayout(tr("Scaling"));
		scaledLayout.addWidget(isScaledRadioButton, 0, 0);
		scaledLayout.addWidget(isNotScaledRadioButton, 0, 1);
		QWidget *scaledWidget = new QWidget(this);
		scaledWidget->setLayout(scaledLayout.build(this));


		plottable = new Plottable(this);

		mainLayout->addWidget(polynomWidget);
		mainLayout->addWidget(scaledWidget);
		mainLayout->addWidget(plottable->getPlot());
		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;

		plottable->preUpdate();
		QCustomPlot *plot = plottable->getPlot();

		int graphIndex = 0;
		typedef typename RegressorTimeseries::Item_t Item_t;
		BOOST_FOREACH(boost::shared_ptr<Item_t> item, regressorTimeseries->getItems()) {
			QVector<double> applyedValues;
			BOOST_FOREACH(double value, item->getApplyedValues()) {
				applyedValues.push_back(value);
			}

			posix_time::ptime time_t_epoch(gregorian::date(1970,1,1));
			QVector<double> dateResponse;
			BOOST_FOREACH(std::string stringDate, item->getDateResponse()) {
				gregorian::date thedate(gregorian::from_string(stringDate));
				posix_time::ptime t(thedate);
				posix_time::time_duration duration = t - time_t_epoch;
				dateResponse.push_back(duration.total_seconds());
			}


			plot->addGraph();
			//plot->graph(graphIndex)->setName(item->getName());
			plot->graph(graphIndex)->setName(item->getName());
			plot->graph(graphIndex)->setData(dateResponse, applyedValues);
			plot->graph(graphIndex)->setPen(QPen(plottable->getRandomColor()));
			plot->xAxis->setTickLabelType(QCPAxis::ltDateTime);
			plot->xAxis->setDateTimeFormat("dd.MM\nyyyy");
			plot->xAxis->setAutoTickStep(true);
			plot->xAxis->setLabel(tr("date"));

			plottable->addGraphToLastLegendItem(plot->graph(graphIndex));

			graphIndex++;
		}

		plot->xAxis->setLabel(tr("date"));
		plot->yAxis->setLabel(tr("regressor values (") + (regressorTimeseries->isScaled() ? tr("scaled") : tr("not scaled")) + ")");
		plot->setMinimumHeight(200);
		plottable->postUpdate();

		asPolynomRadioButton->setChecked(regressorTimeseries->isPolynom());
		asSplittedRadioButton->setChecked(!regressorTimeseries->isPolynom());

		isScaledRadioButton->setChecked(regressorTimeseries->isScaled());
		isNotScaledRadioButton->setChecked(!regressorTimeseries->isScaled());
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		QtBoostDelegate *polynomDelegate = new QtBoostDelegate(bind(&RegressionFitAnalysisRegressorTimeseriesDisplayWidget<RegressorTimeseries>::polynomStateChangedHandler, this), this);
		connect(asSplittedRadioButton, SIGNAL(toggled(bool)), polynomDelegate, SLOT(execute()));

		QtBoostDelegate *scalingDelegate = new QtBoostDelegate(bind(&RegressionFitAnalysisRegressorTimeseriesDisplayWidget<RegressorTimeseries>::scalingStateChangedHandler, this), this);
		connect(isScaledRadioButton, SIGNAL(toggled(bool)), scalingDelegate, SLOT(execute()));
	}


	void polynomStateChangedHandler() {
		if(asSplittedRadioButton->isChecked()) {
			SetPolynomStateSignal(regressorTimeseries, false);
		}else if(asPolynomRadioButton->isChecked()) {
			SetPolynomStateSignal(regressorTimeseries, true);
		}

		plottable->rescaleAxes();
		plottable->replot();
	}

	void scalingStateChangedHandler() {
		if(isScaledRadioButton->isChecked()) {
			SetScalingStateSignal(regressorTimeseries, true);
		}else if(isNotScaledRadioButton->isChecked()) {
			SetScalingStateSignal(regressorTimeseries, false);
		}

		plottable->rescaleAxes();
		plottable->replot();
	}

	boost::shared_ptr<type_t> get() {
		return regressorTimeseries;
	}

	QCustomPlot *getPlot() {
		return plottable->getPlot();
	}

private:
	boost::shared_ptr<RegressorTimeseries> regressorTimeseries;
	Plottable *plottable;

	QRadioButton *asSplittedRadioButton;
	QRadioButton *asPolynomRadioButton;
	QButtonGroup *polynomButtonGroup;

	QRadioButton *isScaledRadioButton;
	QRadioButton *isNotScaledRadioButton;
	QButtonGroup *scaledButtonGroup;
};

}  // namespace gui


#endif /* REGRESSIONFITANALYSISREGRESSORTIMESERIESDISPLAYWIDGET_HPP_ */
