/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualTimeSeriesDisplayWidget.hpp
 *
 *  Created on: Apr 4, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISRESIDUALTIMESERIESDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISRESIDUALTIMESERIESDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QVector>
#include <QDebug>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include "Delegates.hpp"
#include "qcustomplot.h"
#include "Plottable.hpp"

namespace gui {

template<typename ResidualTimeSeries>
class RegressionFitAnalysisResidualTimeSeriesDisplayWidget : public QWidget {
public:
	typedef ResidualTimeSeries type_t;
public:
	RegressionFitAnalysisResidualTimeSeriesDisplayWidget(boost::shared_ptr<ResidualTimeSeries> residualTimeSeries, QWidget *parent = 0)
		:QWidget(parent), residualTimeSeries(residualTimeSeries)
	{
		setupLayout();
		updateValues();
		connectLocal();
		plottable->rescaleAxes();
	}

	boost::shared_ptr<ResidualTimeSeries> get() {
		return residualTimeSeries;
	}

	QString getName() {
		return "Residual time series plot";
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);
		plottable = new Plottable(this);
		mainLayout->addWidget(plottable->getPlot());
		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;

		posix_time::ptime time_t_epoch(gregorian::date(1970,1,1));

		QVector<double> residuals;
		BOOST_FOREACH(double value, residualTimeSeries->getResiduals()) {
			residuals.push_back(value);
		}

		QVector<double> predictionResiduals;
		BOOST_FOREACH(double value, residualTimeSeries->getPredictionResiduals()) {
			predictionResiduals.push_back(value);
		}

		QVector<double> dates;
		BOOST_FOREACH(std::string stringDate, residualTimeSeries->getDates()) {
			gregorian::date thedate(gregorian::from_string(stringDate));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			dates.push_back(duration.total_seconds());
		}

		QVector<double> predictionDates;
		BOOST_FOREACH(std::string stringDate, residualTimeSeries->getPredictionDates()) {
			gregorian::date thedate(gregorian::from_string(stringDate));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			predictionDates.push_back(duration.total_seconds());
		}


		QVector<double> regressionStartDate;
		QVector<double> regressionEndDate;
		{
			gregorian::date thedate(gregorian::from_string(residualTimeSeries->getRegressionStartDate()));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			regressionStartDate.push_back(duration.total_seconds());
			regressionStartDate.push_back(duration.total_seconds());
		}
		{
			gregorian::date thedate(gregorian::from_string(residualTimeSeries->getRegressionEndDate()));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			regressionEndDate.push_back(duration.total_seconds());
			regressionEndDate.push_back(duration.total_seconds());
		}
		QVector<double> verticalValues;
		double minValue = *std::min_element(predictionResiduals.begin(), predictionResiduals.end());
		double maxValue = *std::max_element(predictionResiduals.begin(), predictionResiduals.end());
		verticalValues << minValue << maxValue;

		/**
		 * Confidence interval
		 */
		QVector<double> ciX;
		QVector<double> lowerCiY;
		QVector<double> upperCiY;

		ciX = predictionDates;
		BOOST_FOREACH(double ci, residualTimeSeries->getLowerCi()) {
			lowerCiY.push_back(ci);
		}
		BOOST_FOREACH(double ci, residualTimeSeries->getUpperCi()) {
			upperCiY.push_back(ci);
		}

		/**
		 * Out of limit
		 */
		QVector<double> outOfLimit;
		QVector<double> outOfLimitDates;
		BOOST_FOREACH(double outOfLimitValue, residualTimeSeries->getOutOfLimit()) {
			outOfLimit.push_back(outOfLimitValue);
		}
		BOOST_FOREACH(std::string stringDate, residualTimeSeries->getOutOfLimitDates()) {
			gregorian::date thedate(gregorian::from_string(stringDate));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			outOfLimitDates.push_back(duration.total_seconds());
		}

		/**
		 * Extrapolations
		 */
		QVector<double> extrapolations;
		QVector<double> extrapolationDates;
		BOOST_FOREACH(double extrapolationValue, residualTimeSeries->getExtrapolations()) {
			extrapolations.push_back(extrapolationValue);
		}
		BOOST_FOREACH(std::string stringDate, residualTimeSeries->getExtrapolationDates()) {
			gregorian::date thedate(gregorian::from_string(stringDate));
			posix_time::ptime t(thedate);
			posix_time::time_duration duration = t - time_t_epoch;
			extrapolationDates.push_back(duration.total_seconds());
		}


		plottable->preUpdate();
		QCustomPlot *plot = plottable->getPlot();

		/**
		 * Residual graph
		 */
		/*
		plot->addGraph();
		plot->graph(0)->setData(dates, residuals);
		plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
		plot->graph(0)->setPen( QPen ( Qt::black));
		plot->graph(0)->setName(tr("Residuals"));
		*/

		/**
		 * Prediction graph
		 */
		plot->addGraph();
		plot->graph(0)->setData(predictionDates, predictionResiduals);
		plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
		plot->graph(0)->setPen( QPen ( Qt::blue));
		plot->graph(0)->setName(tr("Residuals (prediction)"));
		plottable->addGraphToLastLegendItem(plot->graph(0));


		/**
		 * Vertical lines
		 */
		plot->addGraph();
		plot->graph(1)->setData(regressionStartDate, verticalValues);
		plot->graph(1)->setPen( QPen ( Qt::gray));
		plot->legend->removeItem(plot->legend->itemCount()-1);
		plot->addGraph();
		plot->graph(2)->setData(regressionEndDate, verticalValues);
		plot->graph(2)->setPen( QPen ( Qt::gray));
		plot->legend->removeItem(plot->legend->itemCount()-1);


		plot->addGraph();
		plot->graph(3)->setData(ciX, lowerCiY);
		plot->graph(3)->setName(tr("95% Interval"));
		plot->graph(3)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
		plot->graph(3)->setPen( QPen ( Qt::green));
		plot->addGraph();
		plot->graph(4)->setData(ciX, upperCiY);
		plot->graph(4)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssNone, Qt::black, Qt::white, 5));
		plot->graph(4)->setPen( QPen ( Qt::green));
		plot->legend->removeItem(plot->legend->itemCount()-1);
		plottable->addGraphToLastLegendItem(plot->graph(3));
		plottable->addGraphToLastLegendItem(plot->graph(4));

		plot->addGraph();
		plot->graph(5)->setData(outOfLimitDates, outOfLimit);
		plot->graph(5)->setName(tr("Out of limit"));
		plot->graph(5)->setLineStyle(QCPGraph::lsNone);
		plot->graph(5)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCross, Qt::red, Qt::red, 5));
		plottable->addGraphToLastLegendItem(plot->graph(5));

		plot->addGraph();
		plot->graph(6)->setData(extrapolationDates, extrapolations);
		plot->graph(6)->setName(tr("Extrapolations"));
		plot->graph(6)->setLineStyle(QCPGraph::lsNone);
		plot->graph(6)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssSquare, Qt::black, Qt::yellow, 5));
		plottable->addGraphToLastLegendItem(plot->graph(6));

		plot->xAxis->setTickLabelType(QCPAxis::ltDateTime);
		//plot->xAxis->setDateTimeFormat("yyyy");
		plot->xAxis->setDateTimeFormat("dd.MM\nyyyy");
		plot->xAxis->setAutoTickStep(true);

		plot->xAxis->setLabel(tr("date"));
		plot->yAxis->setLabel(tr("Residuals"));
		plot->setMinimumHeight(200);

		plottable->postUpdate();
	}

	QCustomPlot *getPlot() {
		return plottable->getPlot();
	}

	void connectLocal() {
	}

private:
	boost::shared_ptr<ResidualTimeSeries> residualTimeSeries;
	Plottable *plottable;
};

}  // namespace gui


#endif /* REGRESSIONFITANALYSISRESIDUALTIMESERIESDISPLAYWIDGET_HPP_ */
