/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelAlgorithmSelectionEditWidget.hpp
 *
 *  Created on: 19.01.2015
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELALGORITHMSELECTIONEDITWIDGET_HPP_
#define REGRESSIONMODELALGORITHMSELECTIONEDITWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QRadioButton>
#include <QButtonGroup>
#include <QSpinBox>
#include <QRegExpValidator>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "GroupBoxLayoutBuilder.hpp"


namespace gui {

template<typename RegressionModel>
class RegressionModelAlgorithmSelectionEditWidget : public QWidget {
public:
	COM_SUBSCRIPTION_FUNCTION(AlgorithmChanged, communication::Signal<void(boost::shared_ptr<RegressionModel>, int)>);
	COM_SUBSCRIPTION_FUNCTION(PraisWinstenIterationsChanged, communication::Signal<void(boost::shared_ptr<RegressionModel>, unsigned)>);
	COM_SUBSCRIPTION_FUNCTION(RidgeKValueChanged, communication::Signal<void(boost::shared_ptr<RegressionModel>, double)>);
	COM_SUBSCRIPTION_FUNCTION(PCRrValueChanged, communication::Signal<void(boost::shared_ptr<RegressionModel>, unsigned)>);
public:
	RegressionModelAlgorithmSelectionEditWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent = 0)
		:QWidget(parent), regressionModel(regressionModel)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);

		{
			standardContainerWidget = new QWidget(this);
			GroupBoxLayoutBuilder layoutBuilder(tr("OLS (ordinary least square)"));
			standardAlgorithmRadioButton = new QRadioButton(tr("disabled"), this);
			layoutBuilder.addWidget(standardAlgorithmRadioButton);
			standardContainerWidget->setLayout(layoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed), false));
			mainLayout->addWidget(standardContainerWidget);
		}

		{
			praisWinstenContainerWidget = new QWidget(this);
			GroupBoxLayoutBuilder layoutBuilder(tr("Prais-Winsten"));

			praisWinstenAlgorithmRadioButton = new QRadioButton(tr("disabled"), this);
			QLabel *iterationsLabel = new QLabel(tr("Max iterations"), this);
			praisWinstenIterationsSpinBox = new QSpinBox(this);
			praisWinstenIterationsSpinBox->setMinimum(1);
			praisWinstenIterationsSpinBox->setMaximum(10);

			layoutBuilder.addWidget(praisWinstenAlgorithmRadioButton);
			layoutBuilder.addWidget(iterationsLabel, 1, 0, 1, 1, Qt::AlignLeft);
			layoutBuilder.addWidget(praisWinstenIterationsSpinBox, 1, 1, 1, 1, Qt::AlignLeft);
			praisWinstenContainerWidget->setLayout(layoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed), false));
			mainLayout->addWidget(praisWinstenContainerWidget);
		}

		{
			ridgeContainerWidget = new QWidget(this);
			GroupBoxLayoutBuilder layoutBuilder(tr("Ridge"));

			ridgeAlgorithmRadioButton = new QRadioButton(tr("disabled"), this);
			QLabel *kValueLabel = new QLabel(tr("k"), this);
			ridgeKValueLineEdit = new QLineEdit(this);
			ridgeKValueLineEdit->setValidator(new QDoubleValidator(this));

			layoutBuilder.addWidget(ridgeAlgorithmRadioButton);
			layoutBuilder.addWidget(kValueLabel, 1, 0, 1, 1, Qt::AlignLeft);
			layoutBuilder.addWidget(ridgeKValueLineEdit, 1, 1, 1, 1, Qt::AlignLeft);
			ridgeContainerWidget->setLayout(layoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed), false));
			mainLayout->addWidget(ridgeContainerWidget);
		}

		{
			pcrContainerWidget = new QWidget(this);
			GroupBoxLayoutBuilder layoutBuilder(tr("Principal Component Regression"));

			pcrAlgorithmRadioButton = new QRadioButton(tr("disabled"), this);
			QLabel *rLabel = new QLabel(tr("r"), this);
			pcrRValueSpinBox = new QSpinBox(this);
			pcrRValueSpinBox->setMinimum(1);

			layoutBuilder.addWidget(pcrAlgorithmRadioButton);
			layoutBuilder.addWidget(rLabel, 1, 0, 1, 1, Qt::AlignLeft);
			layoutBuilder.addWidget(pcrRValueSpinBox, 1, 1, 1, 1, Qt::AlignLeft);
			pcrContainerWidget->setLayout(layoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed), false));
			mainLayout->addWidget(pcrContainerWidget);
		}

		algorithmButtonGroup = new QButtonGroup(this);
		algorithmButtonGroup->addButton(standardAlgorithmRadioButton);
		algorithmButtonGroup->addButton(praisWinstenAlgorithmRadioButton);
		algorithmButtonGroup->addButton(ridgeAlgorithmRadioButton);
		algorithmButtonGroup->addButton(pcrAlgorithmRadioButton);



		mainLayout->setAlignment(Qt::AlignTop);

		setLayout(mainLayout);
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		QtBoostDelegate *algorithmChangedDelegate = new QtBoostDelegate(bind(&RegressionModelAlgorithmSelectionEditWidget<RegressionModel>::algorithmChangedHandler, this), this);
		QtBoostDelegate *praisWinstenIterationsChangedDelegate = new QtBoostDelegate(bind(ref(PraisWinstenIterationsChangedSignal), regressionModel,
					bind(&QSpinBox::value, praisWinstenIterationsSpinBox)
				), this);
		QtBoostDelegate *ridgeKValueChangedDelegate = new QtBoostDelegate(bind(&RegressionModelAlgorithmSelectionEditWidget<RegressionModel>::ridgeKValueChangedHandler, this), this);
		QtBoostDelegate *pcrRValueChangedDelegate = new QtBoostDelegate(bind(ref(PCRrValueChangedSignal), regressionModel,
							bind(&QSpinBox::value, pcrRValueSpinBox)
						), this);

		connect(standardAlgorithmRadioButton, SIGNAL(clicked(bool)), algorithmChangedDelegate, SLOT(execute()));

		connect(praisWinstenAlgorithmRadioButton, SIGNAL(clicked(bool)), algorithmChangedDelegate, SLOT(execute()));
		connect(praisWinstenIterationsSpinBox, SIGNAL(valueChanged(int)), praisWinstenIterationsChangedDelegate, SLOT(execute()));

		connect(ridgeAlgorithmRadioButton, SIGNAL(clicked(bool)), algorithmChangedDelegate, SLOT(execute()));
		connect(ridgeKValueLineEdit, SIGNAL(editingFinished()), ridgeKValueChangedDelegate, SLOT(execute()));

		connect(pcrAlgorithmRadioButton, SIGNAL(clicked(bool)), algorithmChangedDelegate, SLOT(execute()));
		connect(pcrRValueSpinBox, SIGNAL(valueChanged(int)), pcrRValueChangedDelegate, SLOT(execute()));

		algorithmChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, AlgorithmChanged, bind(&RegressionModelAlgorithmSelectionEditWidget<RegressionModel>::updateValues, this));
		pcrrValueChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, PCRrValueChanged, bind(&RegressionModelAlgorithmSelectionEditWidget<RegressionModel>::updateValues, this));
		regressionFormulaChangedConnection = COM_SUBSCRIBE_FOR(regressionModel->getRegressionFormula(), Changed,
					bind(&RegressionModelAlgorithmSelectionEditWidget<RegressionModel>::updateValues, this));
	}

	void updateValues() {
		praisWinstenIterationsSpinBox->setEnabled(false);
		ridgeKValueLineEdit->setEnabled(false);
		pcrRValueSpinBox->setEnabled(false);
		pcrRValueSpinBox->setMaximum(regressionModel->getMaxPCRrValue());
		praisWinstenIterationsSpinBox->setValue(regressionModel->getPraisWinstenIterations());
		ridgeKValueLineEdit->setText(QString::number(regressionModel->getRidgeKValue()));
		pcrRValueSpinBox->setValue(regressionModel->getPCRrValue());

		standardAlgorithmRadioButton->setText(tr("disabled"));
		praisWinstenAlgorithmRadioButton->setText(tr("disabled"));
		ridgeAlgorithmRadioButton->setText(tr("disabled"));
		pcrAlgorithmRadioButton->setText(tr("disabled"));

		if(regressionModel->isStandardAlgorithm()) {
			standardAlgorithmRadioButton->setChecked(true);
			standardAlgorithmRadioButton->setText(tr("enabled"));
		}else if(regressionModel->isPraisWinstenAlgorithm()) {
			praisWinstenAlgorithmRadioButton->setChecked(true);
			praisWinstenIterationsSpinBox->setEnabled(true);
			praisWinstenAlgorithmRadioButton->setText(tr("enabled"));
		}else if(regressionModel->isRidgeAlgorithm()) {
			ridgeAlgorithmRadioButton->setChecked(true);
			ridgeKValueLineEdit->setEnabled(true);
			ridgeAlgorithmRadioButton->setText(tr("enabled"));
		}else if(regressionModel->isPCRAlgorithm()) {
			pcrAlgorithmRadioButton->setChecked(true);
			pcrRValueSpinBox->setEnabled(true);
			pcrAlgorithmRadioButton->setText(tr("enabled"));
		}
		//TODO: continue with other algorithms
	}

	void algorithmChangedHandler() {
		if(standardAlgorithmRadioButton->isChecked()) {
			AlgorithmChangedSignal(regressionModel, RegressionModel::Standard);
		}else if(praisWinstenAlgorithmRadioButton->isChecked()) {
			AlgorithmChangedSignal(regressionModel, RegressionModel::PraisWinsten);
		}else if(ridgeAlgorithmRadioButton->isChecked()) {
			AlgorithmChangedSignal(regressionModel, RegressionModel::Ridge);
		}else if(pcrAlgorithmRadioButton->isChecked()) {
			AlgorithmChangedSignal(regressionModel, RegressionModel::Pcr);
		}
	}

	void ridgeKValueChangedHandler() {
		RidgeKValueChangedSignal(regressionModel, ridgeKValueLineEdit->text().toDouble(NULL));
	}

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	QButtonGroup *algorithmButtonGroup;
	QWidget *standardContainerWidget;
	QRadioButton *standardAlgorithmRadioButton;
	QRadioButton *praisWinstenAlgorithmRadioButton;
	QSpinBox *praisWinstenIterationsSpinBox;
	QWidget *praisWinstenContainerWidget;
	QWidget *ridgeContainerWidget;
	QRadioButton *ridgeAlgorithmRadioButton;
	QLineEdit *ridgeKValueLineEdit;
	QWidget *pcrContainerWidget;
	QSpinBox *pcrRValueSpinBox;
	QRadioButton *pcrAlgorithmRadioButton;
	COM_CONNECTION algorithmChangedConnection;
	COM_CONNECTION regressionFormulaChangedConnection;
	COM_CONNECTION pcrrValueChangedConnection;
};

}  // namespace gui


#endif /* REGRESSIONMODELALGORITHMSELECTIONEDITWIDGET_HPP_ */
