/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelDefinitionEditWidget.hpp
 *
 *  Created on: Aug 6, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELDEFINITIONEDITWIDGET_HPP_
#define REGRESSIONMODELDEFINITIONEDITWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QLabel>
#include <QWidget>
#include <QScrollArea>
#include <QStandardItemModel>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <map>
#include <set>
#include <algorithm>
#include "RegressorEditWidget.hpp"
#include "ComboBox.hpp"
#include "Delegates.hpp"
#include "MoveToolbarButton.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionFormula, typename ModelDefinition>
class RegressionModelDefinitionEditWidget : public QWidget {
public:
	typedef ModelDefinition ModelDefinition_t;
	typedef typename ModelDefinition::Regressor_t Regressor_t;
	typedef typename ModelDefinition::RegressorFormulaBuilder_t RegressorFormulaBuilder_t;
public:
	COM_SUBSCRIPTION_FUNCTION(AddRegressor, communication::Signal<void(boost::shared_ptr<RegressionFormula>, boost::shared_ptr<ModelDefinition>, QString)>); //identifier of the RegressorFormulaBuilder
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressor, communication::Signal<void(boost::shared_ptr<ModelDefinition>, boost::shared_ptr<Regressor_t>)>);
	COM_SUBSCRIPTION_FUNCTION(Remove, communication::Signal<void(boost::shared_ptr<RegressionFormula>, boost::shared_ptr<ModelDefinition>)>);
	COM_SUBSCRIPTION_FUNCTION(Modify, communication::Signal<void(boost::shared_ptr<RegressionFormula>,boost::shared_ptr<ModelDefinition>)>);

public:
	RegressionModelDefinitionEditWidget(boost::shared_ptr<RegressionFormula> regressionFormula, boost::shared_ptr<ModelDefinition> modelDefinition, QWidget *parent = 0)
		:QWidget(parent), regressionFormula(regressionFormula), modelDefinition(modelDefinition)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	virtual ~RegressionModelDefinitionEditWidget() {
		typedef std::pair<RegressorEditWidget<Regressor_t> *, boost::signals2::connection > pair_t;
		BOOST_FOREACH(pair_t pair, connectionMapping) {
			pair.second.disconnect();
		}
	}

private:
	void setupLayout() {
		mainLayout = new QVBoxLayout();
		nameLabel = new QLabel(this);
		removeButton = new MoveToolbarButton("-", this);
		QWidget *titleWidget = new QWidget(this);
		QHBoxLayout *titleWidgetLayout = new QHBoxLayout(this);
		titleWidgetLayout->addWidget(nameLabel, 1, Qt::AlignLeft);
		titleWidgetLayout->addWidget(removeButton, 1, Qt::AlignRight);
		titleWidget->setLayout(titleWidgetLayout);

		modifyButton = new MoveToolbarButton("m", this);

		chooseRegressorWidget = new QWidget(this);
		chooseRegressorWidgetLayout = new QHBoxLayout(chooseRegressorWidget);
		selectRegressorFormulaBuilderComboBox = new ComboBox(chooseRegressorWidget);
		createRegressorButton = new MoveToolbarButton("+", chooseRegressorWidget);
		chooseRegressorWidgetLayout->addWidget(selectRegressorFormulaBuilderComboBox);
		chooseRegressorWidgetLayout->addWidget(createRegressorButton);
		chooseRegressorWidgetLayout->addWidget(modifyButton, 1, Qt::AlignRight);
		chooseRegressorWidget->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum);

		regressorContainerWidgetLayout = new QVBoxLayout(); //this is the layout new regressor edit widgets can be added
		regressorContainerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);
		regressorContainerWidgetLayout->setAlignment(Qt::AlignTop|Qt::AlignLeft);
		regressorContainerWidget = new QWidget(this);
		regressorContainerWidget->setLayout(regressorContainerWidgetLayout);
		scrollArea = new QScrollArea(this);
		scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
		scrollArea->setWidget(regressorContainerWidget);
		scrollArea->setWidgetResizable(true);

		mainLayout->addWidget(titleWidget);
		mainLayout->addWidget(chooseRegressorWidget);
		mainLayout->addWidget(scrollArea);
		setLayout(mainLayout);
	}

	void regressorAdded(boost::shared_ptr<Regressor_t> regressor) {
		/*
		QLabel *regressorLabel = new QLabel();
		regressorLabel->setText(regressor->getName());
		regressorContainerWidgetLayout->addWidget(regressorLabel, Qt::AlignLeft);
		*/
		using namespace boost;

		RegressorEditWidget<Regressor_t> *regressorEditWidget = new RegressorEditWidget<Regressor_t>(regressor);
		regressorContainerWidgetLayout->addWidget(regressorEditWidget);
		connectionMapping[regressorEditWidget] = COM_SUBSCRIBE_FOR(regressorEditWidget, RemoveRegressor,
				bind(ref(RemoveRegressorSignal), modelDefinition, _1));
	}

	/*
	void regressorRemoved(Regressor_t *regressor) {
		for(int i=0; i < regressorContainerWidgetLayout->count(); ++i) {
			QLayoutItem *item = regressorContainerWidgetLayout->itemAt(i);
			RegressorEditWidget<Regressor_t> *regressorEditWidget = dynamic_cast<RegressorEditWidget<Regressor_t> *>(item->widget());
			if(regressorEditWidget != NULL && regressorEditWidget->getRegressor() == regressor ) {
				deleteRegressorEditWidget(regressorEditWidget);
				return;
			}
		}
	}
	*/

	void releaseRegressorEditWidget(RegressorEditWidget<Regressor_t> *regressorEditWidget) {
		connectionMapping[regressorEditWidget].disconnect();
		connectionMapping.erase(regressorEditWidget);
	}

	void regressorFormulaBuilderAdded(int index, boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) {
		/*
		QString additionalInfo;
		if(regressorFormulaBuilder->getIdentifier() != regressorFormulaBuilder->getFormula()) {
			additionalInfo = QString(" [ ") + regressorFormulaBuilder->getFormula() + QString(" ]");
		}
		*/
		selectRegressorFormulaBuilderComboBox->insertItem(index, QString("- ") + regressorFormulaBuilder->resolveExtendedIdentifier(), regressorFormulaBuilder->getIdentifier());
	}

	template<typename AssignmentCategory>
	struct AssignmentCategoryComparator {
		bool operator() (boost::shared_ptr<AssignmentCategory> const &lhs, boost::shared_ptr<AssignmentCategory> const &rhs) const {
			return lhs->getIndex() < rhs->getIndex();
		}
	};

	template<typename RegressorFormulaBuilder>
	struct RegressorFormulaBuilderComparator {
		bool operator() (boost::shared_ptr<RegressorFormulaBuilder> const &lhs, boost::shared_ptr<RegressorFormulaBuilder> const &rhs) const {
			return lhs->getIdentifier() < rhs->getIdentifier();
		}
	};

	void updateValues() {
		nameLabel->setText(modelDefinition->getName());

		QLayoutItem *item;
		while((item = regressorContainerWidgetLayout->takeAt(0)) != 0) {
			regressorContainerWidgetLayout->removeItem(item);
			QWidget *widget = item->widget();
			RegressorEditWidget<Regressor_t> *regressorEditWidget = dynamic_cast<RegressorEditWidget<Regressor_t> *>(widget);
			if(regressorEditWidget != NULL) {
				qDebug() << "going to release regressor edit widget";
				releaseRegressorEditWidget(regressorEditWidget);
			}
			delete widget;
			delete item;
		}
		BOOST_FOREACH(boost::shared_ptr<Regressor_t> regressor, modelDefinition->getRegressors()) {
			regressorAdded(regressor);
		}

		selectRegressorFormulaBuilderComboBox->clear();

		typedef typename RegressorFormulaBuilder_t::AssignmentCategory_t AssignmentCategory_t;
		typedef std::pair<boost::shared_ptr<AssignmentCategory_t>, std::vector<boost::shared_ptr<RegressorFormulaBuilder_t> > > assignment_pair_t;
		std::map<boost::shared_ptr<AssignmentCategory_t>, std::set<boost::shared_ptr<RegressorFormulaBuilder_t> , RegressorFormulaBuilderComparator<RegressorFormulaBuilder_t> > > assignmentCategoryMap;
		std::set<boost::shared_ptr<AssignmentCategory_t>, AssignmentCategoryComparator<AssignmentCategory_t> > orderedAssignmentCategories;

		BOOST_FOREACH(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, modelDefinition->getRegressorFormulaBuilders()) {
			assignmentCategoryMap[regressorFormulaBuilder->getAssignmentCategory()].insert(regressorFormulaBuilder);
			orderedAssignmentCategories.insert(regressorFormulaBuilder->getAssignmentCategory());
		}

		int index = 0;
		BOOST_FOREACH(boost::shared_ptr<AssignmentCategory_t> assignmentCategory, orderedAssignmentCategories) {

			selectRegressorFormulaBuilderComboBox->insertItem(index, assignmentCategory->getName(), -1);
			QStandardItemModel *selectRegressorSelectionModel = qobject_cast<QStandardItemModel *>(selectRegressorFormulaBuilderComboBox->model());
			QModelIndex modelIndex = selectRegressorSelectionModel->index(index, selectRegressorFormulaBuilderComboBox->modelColumn(), selectRegressorFormulaBuilderComboBox->rootModelIndex());
			selectRegressorSelectionModel->itemFromIndex(modelIndex)->setSelectable(false);
			//QMap<int, QVariant> colorMap;
			//colorMap[Qt::BackgroundColorRole] = QColor(Qt::gray);
			//colorMap[Qt::TextColorRole] = QColor(Qt::blue);
			//selectRegressorSelectionModel->setItemData(modelIndex, colorMap);

			++index;
			BOOST_FOREACH(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, assignmentCategoryMap[assignmentCategory]) {
				regressorFormulaBuilderAdded(index++, regressorFormulaBuilder);
			}
		}

		if(modelDefinition->isEditable() == false) {
			selectRegressorFormulaBuilderComboBox->hide();
			createRegressorButton->hide();
			modifyButton->show();
		}else{
			selectRegressorFormulaBuilderComboBox->show();
			createRegressorButton->show();
			modifyButton->hide();
		}

		update();
	}

	void addRegressor() {
		QStandardItemModel *selectRegressorSelectionModel = qobject_cast<QStandardItemModel *>(selectRegressorFormulaBuilderComboBox->model());
		QModelIndex modelIndex = selectRegressorSelectionModel->index(selectRegressorFormulaBuilderComboBox->currentIndex(), selectRegressorFormulaBuilderComboBox->modelColumn(), selectRegressorFormulaBuilderComboBox->rootModelIndex());
		if(selectRegressorSelectionModel->itemFromIndex(modelIndex)->isSelectable() == false) {
			return;
		}

		AddRegressorSignal(regressionFormula, modelDefinition, selectRegressorFormulaBuilderComboBox->itemData(selectRegressorFormulaBuilderComboBox->currentIndex(), Qt::UserRole).toString());
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		createRegressorButtonDelegate = new QtBoostDelegate(bind(&RegressionModelDefinitionEditWidget<RegressionFormula, ModelDefinition>::addRegressor, this), this);
		connect(createRegressorButton, SIGNAL(clicked()), createRegressorButtonDelegate, SLOT(execute()));

		modifyModelButtonDelegate = new QtBoostDelegate(bind(ref(ModifySignal), regressionFormula, modelDefinition), this);
		connect(modifyButton, SIGNAL(clicked()), modifyModelButtonDelegate, SLOT(execute()));

		removeButtonDelegate = new QtBoostDelegate(bind(ref(RemoveSignal), regressionFormula, modelDefinition), this);
		connect(removeButton, SIGNAL(clicked()), removeButtonDelegate, SLOT(execute()));

		//regressorAddedConnection = COM_SUBSCRIBE_FOR(modelDefinition, RegressorAdded, bind(&RegressionModelDefinitionEditWidget<ModelDefinition>::regressorAdded, this, _1));
		//TODO: sign up for primary changes and delegate to updateValues() regressorFormulaBuilderAddedConnection = COM_SUBSCRIBE_FOR(modelDefinition, RegressorFormulaBuilderAdded, bind(&RegressionModelDefinitionEditWidget<ModelDefinition>::regressorFormulaBuilderAdded, this, _1));
		changeConnection = COM_SUBSCRIBE_FOR(modelDefinition, Changed, bind(&RegressionModelDefinitionEditWidget<RegressionFormula, ModelDefinition>::updateValues, this));
	}

private:
	boost::shared_ptr<RegressionFormula> regressionFormula;
	boost::shared_ptr<ModelDefinition> modelDefinition;
	ComboBox *selectRegressorFormulaBuilderComboBox;
	communication::QtBoostDelegate *createRegressorButtonDelegate;
	communication::QtBoostDelegate *modifyModelButtonDelegate;
	communication::QtBoostDelegate *removeButtonDelegate;
	COM_CONNECTION changeConnection;
	QLabel *nameLabel;
	QVBoxLayout *mainLayout;
	QWidget *chooseRegressorWidget;
	QHBoxLayout *chooseRegressorWidgetLayout;
	MoveToolbarButton *createRegressorButton;
	MoveToolbarButton *modifyButton;
	MoveToolbarButton *removeButton;
	QVBoxLayout *regressorContainerWidgetLayout;
	QWidget *regressorContainerWidget;
	QScrollArea *scrollArea;
	std::map<RegressorEditWidget<Regressor_t> *, boost::signals2::connection> connectionMapping;
};

}  // namespace gui


#endif /* REGRESSIONMODELDEFINITIONEDITWIDGET_HPP_ */
