/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelTimeRangeWizardPage.hpp
 *
 *  Created on: Feb 18, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELTIMERANGEWIZARDPAGE_HPP_
#define REGRESSIONMODELTIMERANGEWIZARDPAGE_HPP_

#include <QWizardPage>
#include <QLabel>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QSpacerItem>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "ComboBox.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
class RegressionModelTimeRangeWizardPage : public QWizardPage
{
public:
	RegressionModelTimeRangeWizardPage(QWidget *parent, boost::shared_ptr<RegressionModel> regressionModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId);
	virtual ~RegressionModelTimeRangeWizardPage();

	COM_SUBSCRIPTION_FUNCTION(FromDateChanged, communication::IndexChangedSignal<boost::shared_ptr<RegressionModel> >);
	COM_SUBSCRIPTION_FUNCTION(ToDateChanged, communication::IndexChangedSignal<boost::shared_ptr<RegressionModel> >);

private:
	void setupLayout();
	void connectLocal();
	void updateValues();

private:
	QLabel *descriptionLabel;
	QLabel *fromLabel;
	QLabel *toLabel;
	ComboBox *fromComboBox;
	ComboBox *toComboBox;
	boost::shared_ptr<RegressionModel> regressionModel;
	NextIdStrategy* nextIdStrategy;
	typename NextIdStrategy::type_t myId;
	communication::QtBoostIndexDelegate *fromDateChangedDelegate;
	communication::QtBoostIndexDelegate *toDateChangedDelegate;
	COM_CONNECTION regressionModelTimeValuesChangedConnection;
	COM_CONNECTION regressionModelAvailableRegressionTimeWindowChangedConnection;
};

template<typename RegressionModel, typename NextIdStrategy>
inline RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::RegressionModelTimeRangeWizardPage(
			QWidget* parent, boost::shared_ptr<RegressionModel> regressionModel,
			NextIdStrategy* nextIdStrategy, typename NextIdStrategy::type_t myId)
		:QWizardPage(parent), regressionModel(regressionModel), nextIdStrategy(nextIdStrategy), myId(myId)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename RegressionModel, typename NextIdStrategy>
inline RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::~RegressionModelTimeRangeWizardPage() {
}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::setupLayout() {
	QVBoxLayout *_mainLayout = new QVBoxLayout();

	descriptionLabel = new QLabel(tr("Please choose the time range for which the regression should be applied."), this);
	descriptionLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	descriptionLabel->setWordWrap(true);


	QGroupBox *groupBox = new QGroupBox(tr("Time range"), this);

	QVBoxLayout *groupBoxLayout = new QVBoxLayout();
	groupBoxLayout->addWidget(descriptionLabel);


	fromLabel = new QLabel(tr("From"), this);
	fromLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Ignored);
	toLabel = new QLabel(tr("To"), this);
	toLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Ignored);

	fromComboBox = new ComboBox(this);
	toComboBox = new ComboBox(this);

	fromLabel->setBuddy(fromComboBox);
	toLabel->setBuddy(toComboBox);

	QWidget *fromToWidget = new QWidget(this);
		QGridLayout *fromToWidgetLayout = new QGridLayout(fromToWidget);
		fromToWidgetLayout->addWidget(fromLabel, 0, 0);
		fromToWidgetLayout->addWidget(fromComboBox, 0, 1, Qt::AlignLeft);
		fromToWidgetLayout->addWidget(toLabel, 1, 0);
		fromToWidgetLayout->addWidget(toComboBox, 1, 1, Qt::AlignLeft|Qt::AlignTop);
	fromToWidget->setLayout(fromToWidgetLayout);

	groupBoxLayout->addWidget(fromToWidget);

	groupBox->setLayout(groupBoxLayout);
	groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	_mainLayout->addWidget(groupBox);
	_mainLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));

	setLayout(_mainLayout);
}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::connectLocal() {
	using namespace communication;
	using namespace boost;

	fromDateChangedDelegate = new QtBoostIndexDelegate(bind(ref(FromDateChangedSignal), regressionModel, _1), this);
	connect(fromComboBox, SIGNAL(currentIndexChanged(int)), fromDateChangedDelegate, SLOT(execute(int)));

	toDateChangedDelegate = new QtBoostIndexDelegate(bind(ref(ToDateChangedSignal), regressionModel, _1), this);
	connect(toComboBox, SIGNAL(currentIndexChanged(int)), toDateChangedDelegate, SLOT(execute(int)));

    regressionModelTimeValuesChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, TimeValuesChanged, bind(&RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::updateValues, this));
    regressionModelAvailableRegressionTimeWindowChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, AvailableRegressionTimeWindowChanged, bind(&RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::updateValues, this));
}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelTimeRangeWizardPage<RegressionModel, NextIdStrategy>::updateValues() {
	disconnect(fromComboBox, SIGNAL(currentIndexChanged(int)), fromDateChangedDelegate, SLOT(execute(int)));
	disconnect(toComboBox, SIGNAL(currentIndexChanged(int)), toDateChangedDelegate, SLOT(execute(int)));

	fromComboBox->clear();
	toComboBox->clear();

	//BOOST_FOREACH(typename RegressionModel::DataModel_t::Date_Data_Point_t dateDataPoint, regressionModel->getDataModel()->getDateColumn()->getValues()) {
	BOOST_FOREACH(typename RegressionModel::DataModel_t::Date_Data_Point_t dateDataPoint, regressionModel->getAvailableRegressionTimeWindow()) {
		fromComboBox->addItem(dateDataPoint.getDate(), dateDataPoint.getIndex());
		toComboBox->addItem(dateDataPoint.getDate(), dateDataPoint.getIndex());
	}

	fromComboBox->setCurrentByValue(regressionModel->getFromDate().getIndex());
	toComboBox->setCurrentByValue(regressionModel->getToDate().getIndex());

	connect(fromComboBox, SIGNAL(currentIndexChanged(int)), fromDateChangedDelegate, SLOT(execute(int)));
	connect(toComboBox, SIGNAL(currentIndexChanged(int)), toDateChangedDelegate, SLOT(execute(int)));
}

} /* namespace gui */


#endif /* REGRESSIONMODELTIMERANGEWIZARDPAGE_HPP_ */
