/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelWizard.hpp
 *
 *  Created on: Feb 11, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELWIZARD_HPP_
#define REGRESSIONMODELWIZARD_HPP_

#include <QWizard>
#include <boost/shared_ptr.hpp>
#include "BaseWizard.hpp"
#include "WizardNextIdStrategy.hpp"
#include "RegressionModelWizardPages.hpp"
#include "WizardNameDescriptionWizardPage.hpp"
#include "RegressionModelTimeRangeWizardPage.hpp"
#include "RegressionModelBehaviorIndicatorWizardPage.hpp"
#include "RegressionModelRegressionFormulaWizardPage.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelWizard : public BaseWizard {
public:
	typedef WizardNameDescriptionWizardPage<RegressionModel, WizardNextIdStrategy<RegressionModelWizardPages::Pages> > _common_parameter_wizard_page;
	typedef RegressionModelTimeRangeWizardPage<RegressionModel, WizardNextIdStrategy<RegressionModelWizardPages::Pages> > _time_range_wizard_page;
	typedef RegressionModelBehaviorIndicatorWizardPage<RegressionModel, WizardNextIdStrategy<RegressionModelWizardPages::Pages> > _behavior_indicator_wizard_page;
	typedef RegressionModelRegressionFormulaWizardPage<RegressionModel, WizardNextIdStrategy<RegressionModelWizardPages::Pages> > _regression_formula_wizard_page;
public:
	RegressionModelWizard(boost::shared_ptr<RegressionModel> regressionModel);
	virtual ~RegressionModelWizard();

	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_common_parameter_wizard_page, commonParameterWidget, SetName);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_common_parameter_wizard_page, commonParameterWidget, SetDescription);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_time_range_wizard_page, timeRangeWidget, FromDateChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_time_range_wizard_page, timeRangeWidget, ToDateChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_behavior_indicator_wizard_page, behaviorIndicatorWidget, BehaviorIndicatorChanged);
	//COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, RemoveRegressor);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, AddRegressionFormulaMethodInstance);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, AlgorithmChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, PraisWinstenIterationsChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, RidgeKValueChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, PCRrValueChanged);

	/*
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, CreateWaterLevelRegressor);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, CreateTemperatureRegressor);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, WaterLevelRegressorFormulaEdited);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(_regression_formula_wizard_page, regressionFormulaWidget, TemperatureRegressorFormulaEdited);
	*/

	void start();
	void editCommonParameters();
	void editBehaviorIndicator();
	void editRegressionFormula();
	void editAlgorithm();
	void editTimeRange();
	bool accepted();

	typename _regression_formula_wizard_page::_regression_model_definitions_widget_t *getRegressionModelDefinitionsWidget() {
		return regressionFormulaWidget->getRegressionModelDefinitionWidget();
	}

	typename _regression_formula_wizard_page::_regressor_formula_builders_widget_t *getRegressorFormulaBuildersWidget() {
		return regressionFormulaWidget->getRegressorFormulaBuildersWidget();
	}
private:
	boost::shared_ptr<RegressionModel> regressionModel;
	WizardNextIdStrategy<RegressionModelWizardPages::Pages> *nextIdStrategy;
	_common_parameter_wizard_page *commonParameterWidget;
	_time_range_wizard_page *timeRangeWidget;
	_behavior_indicator_wizard_page *behaviorIndicatorWidget;
	_regression_formula_wizard_page *regressionFormulaWidget;
};

template<typename RegressionModel>
inline RegressionModelWizard<RegressionModel>::RegressionModelWizard(boost::shared_ptr<RegressionModel> regressionModel)
	:regressionModel(regressionModel)
{
	setWizardStyle(ModernStyle);

	nextIdStrategy = new WizardNextIdStrategy<RegressionModelWizardPages::Pages>(RegressionModelWizardPages::Final, this);

	commonParameterWidget = new _common_parameter_wizard_page(this, regressionModel, nextIdStrategy, RegressionModelWizardPages::CommonParameter, tr("Regression information"));
	commonParameterWidget->setVisible(false);

	timeRangeWidget = new _time_range_wizard_page(this, regressionModel, nextIdStrategy, RegressionModelWizardPages::TimeRange);
	timeRangeWidget->setVisible(false);

	behaviorIndicatorWidget = new _behavior_indicator_wizard_page(this, regressionModel, nextIdStrategy, RegressionModelWizardPages::BehaviorIndicator);
	behaviorIndicatorWidget->setVisible(false);

	regressionFormulaWidget = new _regression_formula_wizard_page(this, regressionModel, nextIdStrategy, RegressionModelWizardPages::BehaviorIndicator);
	regressionFormulaWidget->setVisible(false);
}

template<typename RegressionModel>
inline RegressionModelWizard<RegressionModel>::~RegressionModelWizard() {
}

template<typename RegressionModel>
inline void RegressionModelWizard<RegressionModel>::start() {
	if(commonParameterWidget == NULL) {
		return;
	}

	nextIdStrategy->setTransition(RegressionModelWizardPages::CommonParameter, RegressionModelWizardPages::TimeRange);
	nextIdStrategy->setTransition(RegressionModelWizardPages::TimeRange, RegressionModelWizardPages::BehaviorIndicator);
	nextIdStrategy->setTransition(RegressionModelWizardPages::BehaviorIndicator, RegressionModelWizardPages::RegressionFormula);
	nextIdStrategy->setTransition(RegressionModelWizardPages::RegressionFormula, RegressionModelWizardPages::Final);

	setPage(RegressionModelWizardPages::CommonParameter, commonParameterWidget);
	setPage(RegressionModelWizardPages::TimeRange, timeRangeWidget);
	setPage(RegressionModelWizardPages::BehaviorIndicator, behaviorIndicatorWidget);
	setPage(RegressionModelWizardPages::RegressionFormula, regressionFormulaWidget);

	setStartId(RegressionModelWizardPages::CommonParameter);

	exec();
}

template<typename RegressionModel>
inline void RegressionModelWizard<RegressionModel>::editCommonParameters() {
	nextIdStrategy->setTransition(RegressionModelWizardPages::CommonParameter, RegressionModelWizardPages::Final);

	setPage(RegressionModelWizardPages::CommonParameter, commonParameterWidget);
	setStartId(RegressionModelWizardPages::CommonParameter);

	exec();
}

template<typename RegressionModel>
inline void RegressionModelWizard<RegressionModel>::editBehaviorIndicator() {
	nextIdStrategy->setTransition(RegressionModelWizardPages::BehaviorIndicator, RegressionModelWizardPages::Final);

	setPage(RegressionModelWizardPages::BehaviorIndicator, behaviorIndicatorWidget);
	setStartId(RegressionModelWizardPages::BehaviorIndicator);

	exec();
}

template<typename RegressionModel>
inline void RegressionModelWizard<RegressionModel>::editRegressionFormula() {
	nextIdStrategy->setTransition(RegressionModelWizardPages::RegressionFormula, RegressionModelWizardPages::Final);

	setPage(RegressionModelWizardPages::RegressionFormula, regressionFormulaWidget);
	setStartId(RegressionModelWizardPages::RegressionFormula);

	exec();
}

template<typename RegressionModel>
inline void RegressionModelWizard<RegressionModel>::editAlgorithm() {
	editRegressionFormula();
}


template<typename RegressionModel>
inline void RegressionModelWizard<RegressionModel>::editTimeRange() {
	nextIdStrategy->setTransition(RegressionModelWizardPages::TimeRange, RegressionModelWizardPages::Final);

	setPage(RegressionModelWizardPages::TimeRange, timeRangeWidget);
	setStartId(RegressionModelWizardPages::TimeRange);

	exec();
}


template<typename RegressionModel>
inline bool RegressionModelWizard<RegressionModel>::accepted() {
	return result() == Accepted;
}

} /* namespace gui */

#endif /* REGRESSIONMODELWIZARD_HPP_ */
