/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelsDefinitionEditWidget.hpp
 *
 *  Created on: Aug 6, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELSDEFINITIONEDITWIDGET_HPP_
#define REGRESSIONMODELSDEFINITIONEDITWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QScrollArea>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"

namespace gui {

/**
 * Widget containing a list of model definitions
 */
template<typename RegressionFormula>
class RegressionModelsDefinitionEditWidget : public QWidget {
public:
	typedef typename RegressionFormula::RegressionModelDefinition_t RegressionModelDefinition_t;
public:
	RegressionModelsDefinitionEditWidget(boost::shared_ptr<RegressionFormula> regressionFormula, QWidget *parent = 0)
		:QWidget(parent), regressionFormula(regressionFormula)
	{;}

	virtual ~RegressionModelsDefinitionEditWidget() {
		typedef std::pair<boost::shared_ptr<RegressionModelDefinition_t>, boost::signals2::connection > pair_t;
		BOOST_FOREACH(pair_t pair, regressionModelDefinitionConnections) {
			pair.second.disconnect();
		}
	}

	/*
	 * The UiManager itself can define a function acting as factory or a boost function which delegates to the factory owned by the UiManager
	 * A destroy function is not necessary as this widget will be owned by a parent and destroyed by qt's memory management. Well, this is an
	 * assumption (and therefor a suggestion), but the UiManager can handle this as he wants.
	 */
	void init(boost::function<QWidget*(boost::shared_ptr<RegressionFormula>, boost::shared_ptr<RegressionModelDefinition_t>)> widgetFactory) {
		setupLayout();
		updateValues(widgetFactory);
		connectLocal(widgetFactory);
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;

		modelDefinitionsContainerWidgetLayout = new QVBoxLayout;
		modelDefinitionsContainerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);

		elementContainerWidget = new QWidget(this);
		elementContainerWidget->setLayout(modelDefinitionsContainerWidgetLayout);

		elementScrollArea = new QScrollArea(this);
		elementScrollArea->setWidget(elementContainerWidget);
		elementScrollArea->setWidgetResizable(true);
		elementScrollArea->setFrameShape(QFrame::NoFrame);

		mainLayout->addWidget(elementScrollArea);

		setLayout(mainLayout);
	}

	void updateValues(boost::function<QWidget*(boost::shared_ptr<RegressionFormula>, boost::shared_ptr<RegressionModelDefinition_t>)> widgetFactory) {
		BOOST_FOREACH(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition, regressionFormula->getAllRegressionModelDefinitions()) {
			/*
			QWidget *widget = widgetFactory(regressionFormula, regressionModelDefinition);
			modelDefinitionsContainerWidgetLayout->addWidget(widget, Qt::AlignLeft);
			*/
			regressionModelDefinitionAdded(widgetFactory, regressionModelDefinition);
		}
	}

	void removeRegressionModelDefinition(QWidget *widget, boost::shared_ptr<RegressionModelDefinition_t> assignedRegressionModelDefinition, boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) {
		if(assignedRegressionModelDefinition != regressionModelDefinition) {
			return;
		}

		regressionModelDefinitionConnections[regressionModelDefinition].disconnect();
		regressionModelDefinitionConnections.erase(regressionModelDefinition);
		qDebug() << __FUNCTION__ << " Widget : " << widget;
		qDebug() << "Widget: " << __FUNCTION__ << " : " << regressionModelDefinition->getName();
		delete widget;
		/*
		for(int i = 0; i < modelDefinitionsContainerWidgetLayout->count(); ++i) {
			QLayoutItem *item = modelDefinitionsContainerWidgetLayout->itemAt(i);
			if(item->widget() == widget) {
				delete item->widget();
				delete item;
				return;
			}
		}
		*/
	}

	void regressionModelDefinitionAdded(boost::function<QWidget*(boost::shared_ptr<RegressionFormula>, boost::shared_ptr<RegressionModelDefinition_t>)> widgetFactory, boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) {
		using namespace boost;

		QWidget *widget = widgetFactory(regressionFormula, regressionModelDefinition);
		widget->setParent(this);
		qDebug() << __FUNCTION__ << " Widget : " << widget;
		modelDefinitionsContainerWidgetLayout->insertWidget(0, widget, Qt::AlignLeft);
		regressionModelDefinitionConnections[regressionModelDefinition] = COM_SUBSCRIBE_FOR(regressionFormula, RemoveRegressionModelDefinition,
				bind(&RegressionModelsDefinitionEditWidget<RegressionFormula>::removeRegressionModelDefinition, this, widget, regressionModelDefinition, _1));
	}



	void connectLocal(boost::function<QWidget*(boost::shared_ptr<RegressionFormula>, boost::shared_ptr<RegressionModelDefinition_t>)> widgetFactory) {
		using namespace boost;

		regressionModelDefinitionAddedConnection = COM_SUBSCRIBE_FOR(
					regressionFormula,
					RegressionModelDefinitionAdded,
					bind(&RegressionModelsDefinitionEditWidget<RegressionFormula>::regressionModelDefinitionAdded, this, widgetFactory, _1)
				);
	}

private:
	boost::shared_ptr<RegressionFormula> regressionFormula;
	QVBoxLayout *modelDefinitionsContainerWidgetLayout;
	QWidget *elementContainerWidget;
	QScrollArea *elementScrollArea;
	COM_CONNECTION regressionModelDefinitionAddedConnection;
	std::map<boost::shared_ptr<RegressionModelDefinition_t>, boost::signals2::connection> regressionModelDefinitionConnections;
};

}  // namespace gui


#endif /* REGRESSIONMODELSDEFINITIONEDITWIDGET_HPP_ */
