/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderUserParametersEditWidget.hpp
 *
 *  Created on: 20.08.2014
 *      Author: gerberma
 */

#ifndef REGRESSORFORMULABUILDERUSERPARAMETERSEDITWIDGET_HPP_
#define REGRESSORFORMULABUILDERUSERPARAMETERSEDITWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QCheckBox>
#include <QVBoxLayout>
#include <QScrollArea>
#include <QHBoxLayout>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <map>
#include "MoveToolbarButton.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename UserParameter> //UserParameter is expected to be of an std::pair<QString, UserParameterType *>
class RegressorFormulaBuilderUserParameterEditWidget : public QWidget {
public:
	COM_SUBSCRIPTION_FUNCTION(Remove, communication::Signal<void(UserParameter)>);
	COM_SUBSCRIPTION_FUNCTION(SetValue, communication::Signal<void(UserParameter, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetIsMinimumLeadTime, communication::Signal<void(UserParameter, bool)>);

public:
	RegressorFormulaBuilderUserParameterEditWidget(UserParameter userParameter, QWidget *parent = 0)
		:QWidget(parent), userParameter(userParameter)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	virtual ~RegressorFormulaBuilderUserParameterEditWidget(){;}

	UserParameter &getUserParameter() {
		return userParameter;
	}

	void setEditability(bool iseditable) {
		removeButton->setEnabled(iseditable);
		isMinLeadTimeCheckBox->setEnabled(iseditable);
	}

private:
	void setupLayout() {
		QGridLayout *mainLayout = new QGridLayout(this);

		nameLabel = new QLabel(this);
		valueLineEdit = new QLineEdit(this);
		isMinLeadTimeLabel = new QLabel(tr("is startup time"));
		isMinLeadTimeCheckBox = new QCheckBox(this);
		//nameLineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);

		removeButton = new MoveToolbarButton(tr("-"), this);


		mainLayout->addWidget(removeButton, 0, 3, Qt::AlignRight);
		mainLayout->addWidget(nameLabel, 0, 0, 1, 1, Qt::AlignLeft);
		mainLayout->addWidget(valueLineEdit, 0, 1, 1, 2, Qt::AlignLeft);
		mainLayout->addWidget(isMinLeadTimeLabel, 1, 0, 1, 1, Qt::AlignLeft);
		mainLayout->addWidget(isMinLeadTimeCheckBox, 1, 1, 1, 2, Qt::AlignLeft);


		setLayout(mainLayout);
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		removeButtonDelegate = new QtBoostDelegate(bind(ref(RemoveSignal), userParameter));
		connect(removeButton, SIGNAL(clicked()), removeButtonDelegate, SLOT(execute()));

		valueDelegate = new QtBoostDelegate(bind(ref(SetValueSignal), userParameter,
					bind<QString>(&QLineEdit::text, ref(valueLineEdit))
				), this);
		connect(valueLineEdit, SIGNAL(editingFinished()), valueDelegate, SLOT(execute()));

		isMinLeadTimeCheckBoxDelegate = new QtBoostBooleanDelegate(bind(ref(SetIsMinimumLeadTimeSignal), userParameter, _1), this);
		connect(isMinLeadTimeCheckBox, SIGNAL(stateChanged(int)), isMinLeadTimeCheckBoxDelegate, SLOT(execute(int)));
	}

	void updateValues() {
		nameLabel->setText(userParameter.first);
		valueLineEdit->setText(userParameter.second->getValue());
		isMinLeadTimeCheckBox->setCheckState(userParameter.second->isMinimumLeadTime() ?  Qt::Checked : Qt::Unchecked);
	}


private:
	UserParameter userParameter;
	QLabel *nameLabel;
	QLineEdit *valueLineEdit;
	QLabel *isMinLeadTimeLabel;
	QCheckBox *isMinLeadTimeCheckBox;
	MoveToolbarButton *removeButton;
	communication::QtBoostDelegate *removeButtonDelegate;
	communication::QtBoostDelegate *valueDelegate;
	communication::QtBoostBooleanDelegate *isMinLeadTimeCheckBoxDelegate;
};



template<typename RegressorFormulaBuilder>
class RegressorFormulaBuilderUserParametersEditWidget : public QWidget {
public:
	typedef typename RegressorFormulaBuilder::UserParameter_t UserParameter_t;
	typedef RegressorFormulaBuilderUserParameterEditWidget<typename RegressorFormulaBuilder::UserParameter_t> Edit_Widget_t;
public:
	COM_SUBSCRIPTION_FUNCTION(CreateUserParameter, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveUserParameter, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>, UserParameter_t)>);
	COM_SUBSCRIPTION_FUNCTION(UserParameterSetValue, communication::Signal<void(UserParameter_t, QString)>);
	COM_SUBSCRIPTION_FUNCTION(UserParameterSetIsMinimumLeadTime, communication::Signal<void(UserParameter_t, bool)>);
public:
	RegressorFormulaBuilderUserParametersEditWidget(boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder, QWidget *parent = 0)
		:QWidget(parent), regressorFormulaBuilder(regressorFormulaBuilder)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	virtual ~RegressorFormulaBuilderUserParametersEditWidget() {
		typedef std::pair<Edit_Widget_t *, std::vector<boost::signals2::connection> > pair_t;
		BOOST_FOREACH(pair_t pair, editWidgetConnections) {
			BOOST_FOREACH(boost::signals2::connection connection, pair.second) {
				connection.disconnect();
			}
		}
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;

		createUserParameterButton = new MoveToolbarButton("+", this);

		containerWidgetLayout = new QVBoxLayout;
		containerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);
		containerWidgetLayout->setAlignment(Qt::AlignTop|Qt::AlignLeft);

		elementContainerWidget = new QWidget(this);
		elementContainerWidget->setLayout(containerWidgetLayout);

		/*
		elementScrollArea = new QScrollArea(this);
		elementScrollArea->setWidget(elementContainerWidget);
		elementScrollArea->setWidgetResizable(true);
		elementScrollArea->setFrameShape(QFrame::NoFrame);
		*/

		mainLayout->addWidget(createUserParameterButton, 1, Qt::AlignRight);
		//mainLayout->addWidget(elementScrollArea);
		mainLayout->addWidget(elementContainerWidget);
		mainLayout->setAlignment(Qt::AlignTop|Qt::AlignLeft);

		setLayout(mainLayout);
	}

	void userParameterAdded(UserParameter_t userParameter) {
		using namespace boost;

		Edit_Widget_t *editWidget = new Edit_Widget_t(userParameter, this);
		//editWidget->setEnabled(regressorFormulaBuilder->isEditable());
		editWidget->setEditability(regressorFormulaBuilder->isEditable());

		editWidgetConnections[editWidget].push_back(COM_SUBSCRIBE_FOR(editWidget, SetValue, bind(ref(UserParameterSetValueSignal), _1, _2)));
		editWidgetConnections[editWidget].push_back(COM_SUBSCRIBE_FOR(editWidget, SetIsMinimumLeadTime, bind(ref(UserParameterSetIsMinimumLeadTimeSignal), _1, _2)));
		editWidgetConnections[editWidget].push_back(COM_SUBSCRIBE_FOR(editWidget, Remove, bind(ref(RemoveUserParameterSignal), regressorFormulaBuilder, _1)));

		containerWidgetLayout->addWidget(editWidget);
	}

	void removeUserParameter(UserParameter_t userParameter) {
		QLayoutItem *item;
		for(int index = 0; index < containerWidgetLayout->count(); ++index) {
			item = containerWidgetLayout->itemAt(index);
			Edit_Widget_t *editWidget = dynamic_cast<Edit_Widget_t *>(item->widget());
			if(editWidget != NULL && editWidget->getUserParameter().first == userParameter.first) {
				BOOST_FOREACH(boost::signals2::connection connection, editWidgetConnections[editWidget]) {
					connection.disconnect();
				}
				editWidgetConnections.erase(editWidget);

				delete editWidget;
				return;
			}
		}
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		createUserParameterButtonDelegate = new QtBoostDelegate(bind(ref(CreateUserParameterSignal), regressorFormulaBuilder), this);
		connect(createUserParameterButton, SIGNAL(clicked()), createUserParameterButtonDelegate, SLOT(execute()));

		userParameterAddedConnection = COM_SUBSCRIBE_FOR(regressorFormulaBuilder, UserParameterAdded, bind(&RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>::userParameterAdded, this, _1));
		userParameterRemoveConnection = COM_SUBSCRIBE_FOR(regressorFormulaBuilder, RemoveUserParameter, bind(&RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>::removeUserParameter, this, _1));
	}

	void updateValues() {
		using namespace boost;
		BOOST_FOREACH(typename RegressorFormulaBuilder::UserParameter_t userParameter, regressorFormulaBuilder->getUserParameters()) {
			userParameterAdded(userParameter);
		}

		createUserParameterButton->setEnabled(regressorFormulaBuilder->isEditable());
	}

private:
	boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder;
	QVBoxLayout *containerWidgetLayout;
	QWidget *elementContainerWidget;
	QScrollArea *elementScrollArea;
	QLabel *createUserParameterLabel;
	MoveToolbarButton *createUserParameterButton;
	communication::QtBoostDelegate *createUserParameterButtonDelegate;
	std::map<Edit_Widget_t *, std::vector<boost::signals2::connection> > editWidgetConnections;
	COM_CONNECTION userParameterAddedConnection;
	COM_CONNECTION userParameterRemoveConnection;
};

}  // namespace gui



#endif /* REGRESSORFORMULABUILDERUSERPARAMETERSEDITWIDGET_HPP_ */
