/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorInfluenceFactorSelectionWidget.hpp
 *
 *  Created on: Aug 12, 2014
 *      Author: gerberma
 */

#ifndef REGRESSORINFLUENCEFACTORSELECTIONWIDGET_HPP_
#define REGRESSORINFLUENCEFACTORSELECTIONWIDGET_HPP_

#include <QVBoxLayout>
#include <QDialog>
#include <QLabel>
#include <QSpacerItem>
#include <QDialogButtonBox>
#include <QStandardItemModel>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "ComboBox.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename Regressor>
class RegressorInfluenceFactorSelectionWidget : public QDialog {
public:
	typedef Regressor Regressor_t;
public:
	COM_SUBSCRIPTION_FUNCTION(SetInfluenceFactor, communication::IndexChangedSignal<boost::shared_ptr<Regressor_t> >);
public:
	RegressorInfluenceFactorSelectionWidget(boost::shared_ptr<Regressor_t> regressor, QWidget *parent = 0)
		:QDialog(parent), regressor(regressor)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;

		influenceFactorSelection = new ComboBox(this);
		buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);

		QLabel *infoLabel = new QLabel(tr("Select an influence factor for: "), this);
		QLabel *formulBuilderExtendedIdentifierLabel = new QLabel(regressor->getRegressorFormulaBuilder()->resolveExtendedIdentifier(), this);

		QVBoxLayout *elementLayout = new QVBoxLayout;
		QWidget *elementWidget = new QWidget;
		elementLayout->addWidget(infoLabel);
		elementLayout->addWidget(formulBuilderExtendedIdentifierLabel);
		elementLayout->addWidget(influenceFactorSelection);
		elementLayout->setAlignment(Qt::AlignTop);
		elementWidget->setLayout(elementLayout);

		mainLayout->addWidget(elementWidget, Qt::AlignTop);
		mainLayout->addWidget(buttonBox, Qt::AlignBottom);

		setWindowTitle(tr("Influence factor selection"));
		setLayout(mainLayout);
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		influenceFactorSelectionDelegate = new QtBoostIndexDelegate(bind(&RegressorInfluenceFactorSelectionWidget<Regressor>::influenceFactorSelectionChanged, this, _1), this);
		connect(influenceFactorSelection, SIGNAL(currentIndexChanged(int)), influenceFactorSelectionDelegate, SLOT(execute(int)));

		connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
	}

	void influenceFactorSelectionChanged(int comboBoxIndex) {
		SetInfluenceFactorSignal(regressor, influenceFactorSelection->itemData(comboBoxIndex, Qt::UserRole).toInt());
	}

	void updateValues() {
		BOOST_FOREACH(boost::shared_ptr<typename Regressor::Influence_Factor_t> influenceFactor, regressor->getAvailableInfluenceFactors()) {
			influenceFactorSelection->addItem(influenceFactor->getName(), influenceFactor->getIndex());
		}

		if(regressor->getInfluenceFactor() == NULL) {
			influenceFactorSelection->addItem(tr("Influence factors"), -1);
			QStandardItemModel* influenceFactorSelectionModel = qobject_cast<QStandardItemModel*>(influenceFactorSelection->model());
			QModelIndex firstIndex = influenceFactorSelectionModel->index(0, influenceFactorSelection->modelColumn(), influenceFactorSelection->rootModelIndex());
			influenceFactorSelectionModel->itemFromIndex(firstIndex)->setSelectable(false);
		}
		influenceFactorSelection->setCurrentByValue(regressor->getInfluenceFactor()->getIndex());
	}

private:
	boost::shared_ptr<Regressor_t> regressor;
	ComboBox *influenceFactorSelection;
	QDialogButtonBox *buttonBox;
	communication::QtBoostIndexDelegate *influenceFactorSelectionDelegate;
};

}  // namespace gui


#endif /* REGRESSORINFLUENCEFACTORSELECTIONWIDGET_HPP_ */
