/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportWidget.hpp
 *
 *  Created on: Jul 23, 2014
 *      Author: gerberma
 */

#ifndef REPORTWIDGET_HPP_
#define REPORTWIDGET_HPP_

#include <QDialog>
#include <QMessageBox>
#include <QVBoxLayout>
#include <QScrollArea>
#include <qstandarditemmodel.h>
#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "ComboBox.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename Report>
class ReportWidget : public QDialog {
public:
	COM_SUBSCRIPTION_FUNCTION(CreateReport, communication::Signal<void(boost::shared_ptr<Report>)>);
	COM_SUBSCRIPTION_FUNCTION(PlotScaleFactorChanged, communication::Signal<void(boost::shared_ptr<Report>, double)>);
	COM_SUBSCRIPTION_FUNCTION(FontSizeChanged, communication::Signal<void(boost::shared_ptr<Report>, int)>);
public:
	ReportWidget(boost::shared_ptr<Report>report, QWidget *parent = 0)
		:QDialog(parent), report(report)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	void addReportListElementWidget(QWidget *listElementWidget) {
		reportListElementContainerWidgetLayout->addWidget(listElementWidget);
	}

	void moveReportListElementWidget(int fromIndex, int toIndex) {
		QWidget *widget = reportListElementContainerWidgetLayout->itemAt(fromIndex)->widget();
//		QWidget *toWidget = reportListElementContainerWidgetLayout->itemAt(toIndex)->widget();

		if(widget == NULL) {
			return;
		}

		reportListElementContainerWidgetLayout->removeWidget(widget);
		reportListElementContainerWidgetLayout->insertWidget(toIndex, widget);

	}

	virtual ~ReportWidget(){;}



private:
	void setupLayout() {
		setModal(true); //this is a model dialog

		QVBoxLayout *mainLayout = new QVBoxLayout;

		QWidget *plotScaleWidget = new QWidget(this);
		QHBoxLayout *plotScaleWidgetLayout = new QHBoxLayout(this);
		plotScaleWidget->setLayout(plotScaleWidgetLayout);
		plotScaleComboBox = new ComboBox(this);
		plotScaleComboBox->setFixedWidth(40);
		plotScaleLabel = new QLabel(tr("Plot scale factor"), this);
		plotScaleWidgetLayout->addWidget(plotScaleLabel, 1, Qt::AlignLeft);
		plotScaleWidgetLayout->addWidget(plotScaleComboBox, 1, Qt::AlignLeft);

		QWidget *fontSizeWidget = new QWidget(this);
		QHBoxLayout *fontSizeWidgetLayout = new QHBoxLayout(this);
		fontSizeWidget->setLayout(fontSizeWidgetLayout);
		fontSizeComboBox = new ComboBox(this);
		fontSizeComboBox->setFixedWidth(40);
		//fontSizeComboBox->addItem("10", 10);
		fontSizeLabel = new QLabel(tr("Font size (for PDF only)"), this);
		fontSizeWidgetLayout->addWidget(fontSizeLabel, 1, Qt::AlignLeft);
		fontSizeWidgetLayout->addWidget(fontSizeComboBox, 1, Qt::AlignLeft);

		mainLayout->addWidget(plotScaleWidget);
		mainLayout->addWidget(fontSizeWidget);


		reportListElementContainerWidgetLayout = new QVBoxLayout;
		reportListElementContainerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);

		elementContainerWidget = new QWidget(this);
		elementContainerWidget->setLayout(reportListElementContainerWidgetLayout);

		elementScrollArea = new QScrollArea(this);
		elementScrollArea->setWidget(elementContainerWidget);
		elementScrollArea->setWidgetResizable(true);
		elementScrollArea->setFrameShape(QFrame::NoFrame);

		mainLayout->addWidget(elementScrollArea);

		//menu
		menuBar = new QMenuBar(this);
		fileMenu = menuBar->addMenu(tr("&File"));
		createReportAction = new QAction(tr("&Create report"), this);
		closeAction = new QAction(tr("&Close"), this);
		fileMenu->addAction(createReportAction);
		fileMenu->addSeparator();
		fileMenu->addAction(closeAction);

		mainLayout->setMenuBar(menuBar);

		setLayout(mainLayout);
	}


	void updateValues() {
		BOOST_FOREACH(double plotSize, report->getAvailablePlotScaleFactors()) {
			plotScaleComboBox->addItem(QString::number(plotSize), plotSize);
		}
		plotScaleComboBox->setCurrentByValue(report->getPlotScaleFactor());

		BOOST_FOREACH(int fontSize, report->getAvailableFontSizes()) {
			fontSizeComboBox->addItem(QString::number(fontSize), fontSize);
		}
		fontSizeComboBox->setCurrentByValue(report->getFontSize());
	}

	void plotScaleChanged(int index) {
		PlotScaleFactorChangedSignal(report, plotScaleComboBox->itemData(index, Qt::UserRole).toDouble());
	}

	void fontSizeChanged(int index) {
		FontSizeChangedSignal(report, fontSizeComboBox->itemData(index, Qt::UserRole).toInt());
	}

	void writeSuccessful() {
		QMessageBox::information(this, tr("Report"), tr("Report has been written successfully to %1").arg(report->getOutputFileName()), QMessageBox::Ok, QMessageBox::Ok);
	}

	void writeTimedOut(int timeoutLimit) {
		QMessageBox::information(this, tr("Report"), tr("Failed to write report due to a timeout of %1 seconds.").arg(timeoutLimit / 1000), QMessageBox::Ok, QMessageBox::Ok);
	}

	void writeFailed() {
		QMessageBox::critical(this, tr("Report"), tr("Failed to write report: %1").arg(report->getFailMessage()), QMessageBox::Ok, QMessageBox::Ok);
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		createReportDelegate = new QtBoostDelegate(bind(ref(CreateReportSignal), report), this);
		connect(createReportAction, SIGNAL(triggered()), createReportDelegate, SLOT(execute()));

		imageScaleChangedDelegate = new QtBoostIndexDelegate(bind(&ReportWidget<Report>::plotScaleChanged, this, _1), this);
		connect(plotScaleComboBox, SIGNAL(currentIndexChanged(int)), imageScaleChangedDelegate, SLOT(execute(int)));

		fontSizeChangedDelegate = new QtBoostIndexDelegate(bind(&ReportWidget<Report>::fontSizeChanged, this, _1), this);
		connect(fontSizeComboBox, SIGNAL(currentIndexChanged(int)), fontSizeChangedDelegate, SLOT(execute(int)));

		moveReportListElementConnection = COM_SUBSCRIBE_FOR(report, MoveReportListElement, bind(&ReportWidget<Report>::moveReportListElementWidget, this, _1, _2));
		writeSuccessfulConnection = COM_SUBSCRIBE_FOR(report, WriteSuccessful, bind(&ReportWidget<Report>::writeSuccessful, this));
		writeFailedConnection = COM_SUBSCRIBE_FOR(report, WriteFailed, bind(&ReportWidget<Report>::writeFailed, this));
		writeTimedOutConnection = COM_SUBSCRIBE_FOR(report, WriteTimedOut, bind(&ReportWidget<Report>::writeTimedOut, this, _1));
	}

	//move ReportListElement up / down methods which get called from signals caused by the report itself

private:
	boost::shared_ptr<Report> report;
	QVBoxLayout *reportListElementContainerWidgetLayout;
	QWidget *elementContainerWidget;
	QScrollArea *elementScrollArea;
	QMenuBar *menuBar;
	QMenu *fileMenu;
	QAction *createReportAction;
	QAction *closeAction;
	ComboBox *plotScaleComboBox;
	QLabel *plotScaleLabel;
	ComboBox *fontSizeComboBox;
	QLabel *fontSizeLabel;
	communication::QtBoostDelegate *createReportDelegate;
	communication::QtBoostIndexDelegate *imageScaleChangedDelegate;
	communication::QtBoostIndexDelegate *fontSizeChangedDelegate;
	COM_CONNECTION moveReportListElementConnection;
	COM_CONNECTION writeSuccessfulConnection;
	COM_CONNECTION writeTimedOutConnection;
	COM_CONNECTION writeFailedConnection;
};

}  // namespace gui


#endif /* REPORTWIDGET_HPP_ */
