/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * TimeSeriesEditWidget.hpp
 *
 *  Created on: Jan 10, 2014
 *      Author: gerberma
 */

#ifndef TIMESERIESEDITWIDGET_HPP_
#define TIMESERIESEDITWIDGET_HPP_

#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QGroupBox>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/signals2/signal.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "DataModelRawDataDisplayWidget.hpp"
#include "ComboBox.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "DataModelWizardPages.hpp"
#include "Delegates.hpp"

namespace gui {

namespace priv {
class DirtyBaseForTimeSeriesEditWidgetSignalSlots : public QWidget {
public:
	DirtyBaseForTimeSeriesEditWidgetSignalSlots(QWidget *parent)
		:QWidget(parent)
	{}

	virtual ~DirtyBaseForTimeSeriesEditWidgetSignalSlots()
	{}
};

class TimeSeriesEditWidgetQtSignalsSlots : public DirtyBaseForTimeSeriesEditWidgetSignalSlots {
	Q_OBJECT
public:
	TimeSeriesEditWidgetQtSignalsSlots(QWidget *parent)
		:DirtyBaseForTimeSeriesEditWidgetSignalSlots(parent)
	{}

	virtual ~TimeSeriesEditWidgetQtSignalsSlots()
	{}

public slots:
	virtual void timeSeriesIndexChanged(int index) = 0;
	virtual void predefinedDateFormatIndexChanged(int index) = 0;
	virtual void dateFormatChanged() = 0;
};
} /* namespace priv */



template<typename DataModel>
class TimeSeriesEditWidget : public priv::TimeSeriesEditWidgetQtSignalsSlots {
public:
	COM_SUBSCRIPTION_FUNCTION(DateColumnChanged, communication::IndexChangedSignal<boost::shared_ptr<DataModel> >)
	COM_SUBSCRIPTION_FUNCTION(ApplyDateFormat, communication::Signal<void(boost::shared_ptr<DataModel>, QString)>)
private:
	typedef typename DataModel::DateColumn_t DateColumn_t;
public:
	TimeSeriesEditWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel);
	virtual ~TimeSeriesEditWidget();

private:
	void setupLayout();
	void updateValues();
	void updateCurrentIndex();
	void connectLocal();

	void timeSeriesIndexChanged(int index);
	void predefinedDateFormatIndexChanged(int index);
	void dateFormatChanged();

private:
	boost::shared_ptr<DataModel> dataModel;
	QLabel *timeSeriesLabel;
	ComboBox *timeSeriesComboBox;
	ComboBox *predefinedDateFormatsComboBox;
	QLineEdit *dateFormatLineEdit;
	QLabel *dateFormatLabel;
	QLabel *firstDateLabel;
	QLabel *firstDateValueLabel;
	QLabel *lastDateLabel;
	QLabel *lastDateValueLabel;
	QPushButton *applyFormatButton;
	DataModelRawDataDisplayWidget<DataModel> *rawDataWidget;
	communication::QtBoostDelegate *applyFormatButtonDelegate;
	COM_CONNECTION dataModelLoadedConnection;
	COM_CONNECTION dateColumnChangedConnection;
	COM_CONNECTION behaviorIndicatorAssignmentChangedConnection;
	COM_CONNECTION waterLevelAssignmentChangedConnection;
	COM_CONNECTION temperatureAssignmentChangedConnection;
};



template<typename DataModel>
inline gui::TimeSeriesEditWidget<DataModel>::TimeSeriesEditWidget(QWidget* parent, boost::shared_ptr<DataModel> dataModel)
	:priv::TimeSeriesEditWidgetQtSignalsSlots(parent), dataModel(dataModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}


template<typename DataModel>
inline gui::TimeSeriesEditWidget<DataModel>::~TimeSeriesEditWidget() {
}

template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::setupLayout() {
	timeSeriesLabel = new QLabel(this);
	timeSeriesLabel->setText(tr("Column"));

	timeSeriesComboBox = new ComboBox(this);

	dateFormatLabel = new QLabel(tr("Format"), this);
	dateFormatLineEdit = new QLineEdit(this);
	predefinedDateFormatsComboBox = new ComboBox(this);

	firstDateLabel = new QLabel(tr("First date"), this);
	firstDateValueLabel = new QLabel(this);
	lastDateLabel = new QLabel(tr("Last date"), this);
	lastDateValueLabel = new QLabel(this);

	applyFormatButton = new QPushButton(tr("Apply"), this);

	rawDataWidget = new DataModelRawDataDisplayWidget<DataModel>(dataModel, this);

	GroupBoxLayoutBuilder dataModelLayoutBuilder(tr("Column"));
	int lineIndex = 0;
	dataModelLayoutBuilder.addWidget(timeSeriesLabel, lineIndex, 0);
	dataModelLayoutBuilder.addWidget(timeSeriesComboBox, lineIndex, 1);
	dataModelLayoutBuilder.addWidget(dateFormatLabel, ++lineIndex, 0);
	dataModelLayoutBuilder.addWidget(dateFormatLineEdit, lineIndex, 1);
	dataModelLayoutBuilder.addWidget(predefinedDateFormatsComboBox, lineIndex, 2);
	dataModelLayoutBuilder.addWidget(firstDateLabel, ++lineIndex, 0);
	dataModelLayoutBuilder.addWidget(firstDateValueLabel, lineIndex, 1);
	dataModelLayoutBuilder.addWidget(lastDateLabel, ++lineIndex, 0);
	dataModelLayoutBuilder.addWidget(lastDateValueLabel, lineIndex, 1);
	dataModelLayoutBuilder.addWidget(applyFormatButton, ++lineIndex, 2);
	//dataModelLayoutBuilder.addWidget(rawDataWidget, ++lineIndex, 0, 3, 3);
	QWidget *dataModelWidget = new QWidget(this);
	QVBoxLayout *dataModelWidgetLayout = dataModelLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum), false);
	dataModelWidgetLayout->setAlignment(Qt::AlignTop);
	dataModelWidget->setLayout(dataModelWidgetLayout);

	GroupBoxLayoutBuilder rawDataLayoutBuilder(tr("Data"));
	rawDataLayoutBuilder.addWidget(rawDataWidget, 0, 0, 3, 2);
	QWidget *rawDataWidget = new QWidget(this);
	QVBoxLayout *rawDataWidgetLayout = rawDataLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding), false);
	rawDataWidgetLayout->setAlignment(Qt::AlignTop);
	rawDataWidget->setLayout(rawDataWidgetLayout);

	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	mainLayout->addWidget(dataModelWidget);
	mainLayout->addWidget(rawDataWidget);
	mainLayout->setAlignment(Qt::AlignTop);

	setLayout(mainLayout);
}

template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::updateValues() {
	typedef typename DataModel::Header_t Header_t;

	dateFormatLineEdit->setEnabled(dataModel->isDateColumnEditable());
	predefinedDateFormatsComboBox->setEnabled(dataModel->isDateColumnEditable());
	applyFormatButton->setEnabled(dataModel->isDateColumnEditable());
	timeSeriesComboBox->setEnabled(dataModel->isDateColumnEditable());

	disconnect(timeSeriesComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(timeSeriesIndexChanged(int)));
	timeSeriesComboBox->clear();

	BOOST_FOREACH(Header_t const &header, dataModel->getAssignableDateColumnHeaders()) {

		timeSeriesComboBox->addItem(header.getName(), header.getIndex());
		//timeSeriesComboBox->setItemData(index)
	}
	connect(timeSeriesComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(timeSeriesIndexChanged(int)));

	disconnect(predefinedDateFormatsComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(predefinedDateFormatIndexChanged(int)));
	predefinedDateFormatsComboBox->clear();
	std::vector<QString> predefinedDateFormats;
	predefinedDateFormats.push_back("%d.%m.%Y");
	predefinedDateFormats.push_back("%d/%m/%Y");
	BOOST_FOREACH(QString dateFormat, predefinedDateFormats) {
		predefinedDateFormatsComboBox->addItem(dateFormat, dateFormat);
	}

	connect(predefinedDateFormatsComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(predefinedDateFormatIndexChanged(int)));

	disconnect(dateFormatLineEdit, SIGNAL(textChanged(QString)), this, SLOT(dateFormatChanged()));
	dateFormatLineEdit->setText(dataModel->getDateFormat());
	connect(dateFormatLineEdit, SIGNAL(textChanged(QString)), this, SLOT(dateFormatChanged()));

	updateCurrentIndex();
}

template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::updateCurrentIndex() {
	disconnect(timeSeriesComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(timeSeriesIndexChanged(int)));

	typedef typename DataModel::Header_Container_t::const_iterator it_t;
	boost::shared_ptr<DateColumn_t> dateColumn;
	if(dataModel->isDateColumnEditable()) {
		dateColumn = dataModel->getDateColumn(dateFormatLineEdit->text());
	}else {
		dateColumn = dataModel->getDateColumn();
	}

	if(dateColumn != NULL) {
		timeSeriesComboBox->setCurrentByValue(dateColumn->getHeader().getIndex());
		firstDateValueLabel->setText(dateColumn->getValues().front().getDate());
		lastDateValueLabel->setText(dateColumn->getValues().back().getDate());
	}

	connect(timeSeriesComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(timeSeriesIndexChanged(int)));
}


template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::timeSeriesIndexChanged(int index) {
	int headerIndex = timeSeriesComboBox->itemData(index, Qt::UserRole).toString().toInt();

	DateColumnChangedSignal(dataModel, headerIndex);
}


template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::predefinedDateFormatIndexChanged(int index) {
	dateFormatLineEdit->setText(predefinedDateFormatsComboBox->itemData(index, Qt::UserRole).toString());
	updateCurrentIndex();
}

template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::dateFormatChanged() {
	//DateFormatChangedSignal(dataModel, dateFormatLineEdit->text());
	boost::shared_ptr<DateColumn_t> dateColumn;
	if(dataModel->isDateColumnEditable()) {
		dateColumn = dataModel->getDateColumn(dateFormatLineEdit->text());
	}else {
		dateColumn = dataModel->getDateColumn();
	}

	if(dateColumn != NULL) {
		firstDateValueLabel->setText(dateColumn->getValues().front().getDate());
		lastDateValueLabel->setText(dateColumn->getValues().back().getDate());
	}
}


template<typename DataModel>
inline void gui::TimeSeriesEditWidget<DataModel>::connectLocal() {
	//connect(timeSeriesComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(timeSeriesIndexChanged(int)));

	using namespace communication;
	using namespace boost;
	applyFormatButtonDelegate = new QtBoostDelegate(bind(ref(ApplyDateFormatSignal), dataModel,
				bind<QString>(&QLineEdit::text, ref(dateFormatLineEdit))
			), this);
	connect(applyFormatButton, SIGNAL(clicked()), applyFormatButtonDelegate, SLOT(execute()));

	dataModelLoadedConnection = COM_SUBSCRIBE_FOR(dataModel, DataModelLoaded, (boost::bind(&TimeSeriesEditWidget<DataModel>::updateValues, this)));
	dateColumnChangedConnection = COM_SUBSCRIBE_FOR(dataModel, DateColumnChanged, (boost::bind(&TimeSeriesEditWidget<DataModel>::updateCurrentIndex, this)));
	behaviorIndicatorAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, BehaviorIndicatorAssignmentChanged, (boost::bind(&TimeSeriesEditWidget<DataModel>::updateValues, this)));
	waterLevelAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, WaterLevelAssignmentChanged, (boost::bind(&TimeSeriesEditWidget<DataModel>::updateValues, this)));
	temperatureAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, TemperatureAssignmentChanged, (boost::bind(&TimeSeriesEditWidget<DataModel>::updateValues, this)));
}


} /* namespace gui */


#endif /* TIMESERIESEDITWIDGET_HPP_ */
