/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * WizardNameDescriptionWizardPage.hpp
 *
 *  Created on: Nov 28, 2013
 *      Author: gerberma
 */

#ifndef WIZARDNAMEDESCRIPTIONWIZARDPAGE_HPP_
#define WIZARDNAMEDESCRIPTIONWIZARDPAGE_HPP_

#include <QWidget>
#include <QWizardPage>
#include <QLabel>
#include <QLineEdit>
#include <QTextEdit>
#include <QGridLayout>
#include <QRegExpValidator>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "communication/Signals.hpp"

namespace gui {

template
<
	typename T,
	typename NextIdStrategy
>
class WizardNameDescriptionWizardPage : public QWizardPage {
public:
	WizardNameDescriptionWizardPage(QWidget *parent, boost::shared_ptr<T> t, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId, QString const &title);
	virtual ~WizardNameDescriptionWizardPage();

	COM_SUBSCRIPTION_FUNCTION(SetName, communication::Signal<void(boost::shared_ptr<T>, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetDescription, communication::Signal<void(boost::shared_ptr<T>, QString)>);

	bool isComplete() const;

private:
	void setupLayout(QString const &title);
	void prepare();
	void nameLineEditFinished();
	void descriptionTextEditFinished();
	void connectLocal();

private:
	boost::shared_ptr<T> nameDescriptionObject;
	NextIdStrategy *nextIdStrategy;
	typename NextIdStrategy::type_t myId;
	QLabel *nameLabel;
	QLineEdit *nameLineEdit;
	QLabel *descriptionLabel;
	QTextEdit *descriptionTextEdit;
	communication::QtBoostDelegate *nameEditedDelegate;
	communication::QtBoostDelegate *descriptionEditedDelegate;
};


template
<
	typename T,
	typename NextIdStrategy
>
inline WizardNameDescriptionWizardPage<T, NextIdStrategy>::WizardNameDescriptionWizardPage(QWidget *parent, boost::shared_ptr<T> t, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId, QString const &title)
	:QWizardPage(parent), nameDescriptionObject(t), nextIdStrategy(nextIdStrategy), myId(myId)
{
	setupLayout(title);
	prepare();
	connectLocal();
}

template
<
	typename T,
	typename NextIdStrategy
>
inline WizardNameDescriptionWizardPage<T, NextIdStrategy>::~WizardNameDescriptionWizardPage() {
}

template
<
	typename T,
	typename NextIdStrategy
>
inline bool WizardNameDescriptionWizardPage<T, NextIdStrategy>::isComplete() const {
	QString nameText(nameLineEdit->text());
	int nameLength = nameLineEdit->text().length();
	if(nameLineEdit->validator()->validate(nameText, nameLength) == QValidator::Acceptable) {
		return true;
	}

	return false;
}

template
<
	typename T,
	typename NextIdStrategy
>
inline void WizardNameDescriptionWizardPage<T, NextIdStrategy>::prepare() {
	nameLineEdit->setText(nameDescriptionObject->getName());
	descriptionTextEdit->setText(nameDescriptionObject->getDescription());
}

template
<
	typename T,
	typename NextIdStrategy
>
inline void WizardNameDescriptionWizardPage<T, NextIdStrategy>::nameLineEditFinished() {
	SetNameSignal(nameDescriptionObject, nameLineEdit->text());
}

template
<
	typename T,
	typename NextIdStrategy
>
inline void WizardNameDescriptionWizardPage<T, NextIdStrategy>::descriptionTextEditFinished() {
	SetDescriptionSignal(nameDescriptionObject, descriptionTextEdit->toPlainText());
}

template
<
	typename T,
	typename NextIdStrategy
>
inline void WizardNameDescriptionWizardPage<T, NextIdStrategy>::connectLocal() {
	using namespace boost;
	using namespace communication;

	nameEditedDelegate = new QtBoostDelegate(bind(&WizardNameDescriptionWizardPage<T, NextIdStrategy>::nameLineEditFinished, this));
	connect(nameLineEdit, SIGNAL(editingFinished()), nameEditedDelegate, SLOT(execute()));

	descriptionEditedDelegate = new QtBoostDelegate(bind(&WizardNameDescriptionWizardPage<T, NextIdStrategy>::descriptionTextEditFinished, this));
	connect(descriptionTextEdit, SIGNAL(textChanged()), descriptionEditedDelegate, SLOT(execute()));
}

template
<
	typename T,
	typename NextIdStrategy
>
inline void WizardNameDescriptionWizardPage<T, NextIdStrategy>::setupLayout(QString const &title) {
	setTitle(title);

	QRegExp rx("^(?!\\s*$).+");
	QValidator *validator = new QRegExpValidator(rx, this);
	nameLabel = new QLabel(tr("Name"));
	nameLineEdit = new QLineEdit;
	nameLineEdit->setValidator(validator);
	connect(nameLineEdit, SIGNAL(textChanged(const QString &)), this, SIGNAL(completeChanged()));

	nameLabel->setBuddy(nameLineEdit);
	descriptionLabel = new QLabel(tr("Description"));
	descriptionTextEdit = new QTextEdit;
	descriptionLabel->setBuddy(descriptionTextEdit);

	registerField("name", nameLineEdit);
	registerField("description", descriptionTextEdit, "plainText");

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(nameLabel, 0, 0);
	layout->addWidget(nameLineEdit, 0, 1);
	layout->addWidget(descriptionLabel, 1, 0, Qt::AlignTop);
	layout->addWidget(descriptionTextEdit, 1, 1);

	setLayout(layout);
}


} /* namespace gui */

#endif /* WIZARDNAMEDESCRIPTIONWIZARDPAGE_HPP_ */
