/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * OwnerhshipPolicies.hpp
 *
 *  Created on: Nov 29, 2013
 *      Author: gerberma
 */

#ifndef OWNERHSHIPPOLICIES_HPP_
#define OWNERHSHIPPOLICIES_HPP_

#include <iostream>
#include <set>
#include <vector>
#include <map>
#include <algorithm>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>

namespace memory {

class DeleteOnDeleteChildPolicy {
public:
	template<typename T>
	void deleteChild(T *t) {
		if(t != NULL) {
			delete t;
		}
	}
};

class IgnoreOnDeleteChildPolicy {
public:
	template<typename T>
	void deleteChild(T *) {
	}
};

template<
	class T,
	class DeleteChildPolicy = IgnoreOnDeleteChildPolicy
>
class IgnoreOwnershipPolicy : public DeleteChildPolicy {
public:
	void overtakeOwnership(T *) {
	}

	void overtakeOwnership(const std::vector<T *> &) {
	}

	template<typename U>
	void overtakeOwnership(const std::set<T *, U> &container) {
	}

	template<typename Key, typename Value>
	void overtakeOwnership(const std::map<Key, Value *> &container) {
	}

	bool containsChild(T *) {
		return false;
	}

	void deleteChild(T *) {
	}

	void deleteChildren(const std::vector<T *> &) {
	}

	void deleteAllChildren() {
	}

	IgnoreOwnershipPolicy()
	{}


	virtual ~IgnoreOwnershipPolicy(){
	}
};



template<
	class T,
	class DeleteChildPolicy = DeleteOnDeleteChildPolicy
>
class CustomOwnershipPolicy : public DeleteChildPolicy {
public:
	void overtakeOwnership(T *t) {
		children.insert(t);
	}

	void overtakeOwnership(const std::vector<T *> &container) {
		children.insert(container.begin(), container.end());
	}

	template<typename U>
	void overtakeOwnership(const std::set<T *, U> &container) {
		children.insert(container.begin(), container.end());
	}

	template<typename Key, typename Value>
	void overtakeOwnership(const std::map<Key, Value *> &container) {
		std::vector<Value *> valueVector;

		typedef std::pair<Key, Value *> pair_t;
		BOOST_FOREACH(pair_t pair, container) {
			valueVector.push_back(pair.second);
		}

		overtakeOwnership(valueVector);
	}

	bool containsChild(T *t) {
		return children.find(t) != children.end();
	}

	void deleteChild(T *t) {
		//std::cout << __FUNCTION__ << std::endl;
		children.erase(t);
		DeleteChildPolicy::deleteChild(t);
	}

	void deleteChildren(const std::vector<T *> &deletableChildren) {
		BOOST_FOREACH(T *deletableChild, deletableChildren) {
			deleteChild(deletableChild);
		}
	}

	void deleteAllChildren() {
		std::for_each(children.begin(), children.end(), boost::bind(&CustomOwnershipPolicy<T, DeleteChildPolicy>::deleteChild, this, _1));
	}

	CustomOwnershipPolicy()
	{}


	virtual ~CustomOwnershipPolicy(){
		//std::cout << __FUNCTION__ << std::endl;
		std::for_each(children.begin(), children.end(), boost::bind(&CustomOwnershipPolicy<T, DeleteChildPolicy>::deleteChild, this, _1));
	}
private:
	std::set<T *> children;

};


/*
class QtParentChildOwnershipPolicy : public QObject {
public:
	template<typename T>
	void overtakeOwnership(T *child) {child->setParent(this);}

	template<typename T>
	bool containsChild(T *child) {
		return children().contains(child);
	}

	template<typename T>
	void deleteChild(T *child) {
		return delete child;
	}

	QtParentChildOwnershipPolicy(){}
	virtual ~QtParentChildOwnershipPolicy(){}
};
*/

} /* namespace ownership */


#endif /* OWNERHSHIPPOLICIES_HPP_ */
