/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModel.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include <algorithm>
#include <stdexcept>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include "DataModel.hpp"

namespace dambase {
namespace model {

DataModel::DataModel()
	:dataFrameProvider(NULL), dateFormat("%d.%m.%Y"), dateFormatApplied(false)
{;}

DataModel::DataModel(
		api::IDataModel::DataFrameProvider_t *dataFrameProvider,
		std::map<QString, QString> availableCSVDelimiters,
		QString csvDelimiter,
		QString dateFormat)
	:dataFrameProvider(dataFrameProvider),
	 availableCSVDelimiters(availableCSVDelimiters),
	 csvDelimiter(csvDelimiter),
	 dateFormat(dateFormat),
	 dateFormatApplied(false)
{
}

DataModel::~DataModel(){;}

void
DataModel::setAvailableCSVDelimiters(std::map<QString, QString> availableCSVDelimiters) {
	this->availableCSVDelimiters = availableCSVDelimiters;
}

const std::map<QString, QString> &
DataModel::getAvailableCSVDelimiters() const {
	return availableCSVDelimiters;
}

const QString &
DataModel::getCSVDelimiter() const {
	return csvDelimiter;
}

void
DataModel::setCSVDelimiter(QString csvDelimiter) {
	this->csvDelimiter = csvDelimiter;
}

const api::IDataModel::source_uri_t &
DataModel::getSourceUri() const {
	return sourceURI;
}

void
DataModel::setSourceUri(const api::IDataModel::source_uri_t &sourceUri) {
	sourceURI = sourceUri;
	api::IDataModel::fire();
}


void
DataModel::setDataFrameProvider(api::IDataModel::DataFrameProvider_t *dataFrameProvider) {
	this->dataFrameProvider = dataFrameProvider;
}


void
DataModel::loadDataFrame() {
	setDataFrame(dataFrameProvider->load(sourceURI, getCSVDelimiter()));
}

/**
 * This is used by loadDataFrame as well as the deserialization process
 */
void
DataModel::setDataFrame(boost::shared_ptr<api::IDataModel::DataFrame_t> newDataFrame) {
	this->dataFrame = newDataFrame;

	dateFormatApplied = false;
	DataModelSignals::fireDataModelLoaded();

	updateAnalysisTools();
}


boost::shared_ptr<api::IDataModel::DateColumn_t>
DataModel::getDateColumn() {
	if(dataFrame != NULL) {
		return dataFrame->getDateColumn();
	}

	boost::shared_ptr<DateColumn_t> nullReturn;
	return nullReturn;
}

boost::shared_ptr<api::IDataModel::DateColumn_t>
DataModel::getDateColumn(QString dateFormat) {
	if(dataFrame != NULL && getDateColumn() != NULL) {
		Rcpp::DataFrame newDataFrame = dataFrameProvider->applyDateFormat(dataFrame->getDataFrame(), getDateColumn()->getIndex(), dateFormat); //TODO: load a dataFrame as done somewhere else
		return dataFrame->getDateColumn(newDataFrame);
	}

	boost::shared_ptr<api::IDataModel::DateColumn_t> nullReturn;
	return nullReturn;
}

void
DataModel::setDateColumn(int index) {
	if(dataFrame != NULL) {
		dataFrame->setDateColumn(index);
	}
	DataModelSignals::fireDateColumnChanged();

	updateAnalysisTools();
}

void
DataModel::estimateDateColumn() {
	if(dataFrame != NULL) {
		int estimatedDateColumnIndex = dataFrame->estimateDateColumn();
		if(estimatedDateColumnIndex != -1) {
			setDateColumn(estimatedDateColumnIndex);
		}
	}
}

bool
DataModel::isColumnAssignedWithIndex(int index) {
	return
			   isBehaviorIndicatorColumnAssignedFor(index)
			|| isWaterLevelColumnAssignedFor(index)
			|| isTemperatureColumnAssignedFor(index);
}

bool
DataModel::assignBehaviorIndicatorColumn(int index) {
	if(dataFrame != NULL) {
		bool ret = dataFrame->assignBehaviorIndicatorColumn(index);
		if (ret == true) {
			api::IDataModel::fireBehaviorIndicatorColumnAssignmentChanged();
			updateAnalysisTools();
		}
		return ret;
	}else{
		return false;
	}
}

bool
DataModel::removeBehaviorIndicatorColumn(int index) {
	if(dataFrame != NULL) {
		bool ret = dataFrame->removeBehaviorIndicatorColumn(index);
		if(ret == true) {
			api::IDataModel::fireBehaviorIndicatorColumnAssignmentChanged();
			updateAnalysisTools();
		}

		return ret;
	}else{
		return false;
	}
}

bool
DataModel::isBehaviorIndicatorColumnAssignedFor(int index) {
	if(dataFrame != NULL) {
		return dataFrame->isBehaviorIndicatorColumnAssignedFor(index);
	}else{
		return false;
	}
}

bool
DataModel::assignWaterLevelColumn(int index) {
	if(dataFrame != NULL) {
		bool ret = dataFrame->assignWaterLevelColumn(index);
		if (ret == true) {
			api::IDataModel::fireWaterLevelColumnAssignmentChanged();
			updateAnalysisTools();
		}
		return ret;
	}else{
		return false;
	}
}

bool
DataModel::removeWaterLevelColumn(int index) {
	if(dataFrame != NULL) {
		bool ret = dataFrame->removeWaterLevelColumn(index);
		if(ret == true) {
			api::IDataModel::fireWaterLevelColumnAssignmentChanged();
			updateAnalysisTools();
		}

		return ret;
	}else{
		return false;
	}
}

bool
DataModel::isWaterLevelColumnAssignedFor(int index) {
	if(dataFrame != NULL) {
		return dataFrame->isWaterLevelColumnAssignedFor(index);
	}else{
		return false;
	}
}


bool
DataModel::assignTemperatureColumn(int index) {
	if(dataFrame != NULL) {
		bool ret = dataFrame->assignTemperatureColumn(index);
		if (ret == true) {
			api::IDataModel::fireTemperatureColumnAssignmentChanged();
			updateAnalysisTools();
		}
		return ret;
	}else{
		return false;
	}
}

bool
DataModel::removeTemperatureColumn(int index) {
	if(dataFrame != NULL) {
		bool ret = dataFrame->removeTemperatureColumn(index);
		if(ret == true) {
			api::IDataModel::fireTemperatureColumnAssignmentChanged();
			updateAnalysisTools();
		}

		return ret;
	}else{
		return false;
	}
}

bool
DataModel::isTemperatureColumnAssignedFor(int index) {
	if(dataFrame != NULL) {
		return dataFrame->isTemperatureColumnAssignedFor(index);
	}else{
		return false;
	}
}

api::IDataModel::Header_Container_t&
DataModel::getHeaders() {
	if(dataFrame != NULL) {
		return dataFrame->getHeaders();
	}
	return nullHeaders;
}


boost::shared_ptr<api::IDataModel::Column_t>
DataModel::getColumnByIndex(int index) {
	if(dataFrame != NULL) {
		return dataFrame->getColumnByIndex(index);
	}else{
		boost::shared_ptr<api::IDataModel::Column_t> nullReturn;
		return nullReturn;
	}
}

api::IDataModel::Assignable_Header_Container_t
DataModel::getAssignableDateColumnHeaders() {
	if(dataFrame != NULL) {
		return dataFrame->getAssignableDateColumnHeaders();
	}
	return api::IDataModel::Assignable_Header_Container_t();
}

api::IDataModel::Assignable_Header_Container_t
DataModel::getAssignableBehaviorIndicatorHeaders() {
	if(dataFrame != NULL) {
		return dataFrame->getAssignableBehaviorIndicatorHeaders();
	}
	return api::IDataModel::Assignable_Header_Container_t();
}

api::IDataModel::Assignable_Header_Container_t
DataModel::getAssignableWaterLevelHeaders() {
	if(dataFrame != NULL) {
		return dataFrame->getAssignableWaterLevelHeaders();
	}
	return api::IDataModel::Assignable_Header_Container_t();
}

api::IDataModel::Assignable_Header_Container_t
DataModel::getAssignableTemperatureHeaders() {
	if(dataFrame != NULL) {
		return dataFrame->getAssignableTemperatureHeaders();
	}
	return api::IDataModel::Assignable_Header_Container_t();
}

const api::IDataModel::Behavior_Indicator_Column_Container_t&
DataModel::getBehaviorIndicatorColumns() {
	if(dataFrame != NULL) {
		return dataFrame->getBehaviorIndicatorColumns();
	}else{
		return nullBehaviorIndicatorColumns;
	}
}

boost::shared_ptr<api::IDataModel::Behavior_Indicator_Column_t>
DataModel::getBehaviorIndicatorColumnWith(int index) {
	if(dataFrame != NULL) {
		BOOST_FOREACH(boost::shared_ptr<api::IDataModel::Behavior_Indicator_Column_t> behaviorIndicator, getBehaviorIndicatorColumns()) {
			if(behaviorIndicator->getIndex() == index) {
				return behaviorIndicator;
			}
		}
		boost::shared_ptr<api::IDataModel::Behavior_Indicator_Column_t> nullReturn;
		return nullReturn;
	}else{
		boost::shared_ptr<api::IDataModel::Behavior_Indicator_Column_t> nullReturn;
		return nullReturn;
	}
}

const api::IDataModel::Water_Level_Column_Container_t&
DataModel::getWaterLevelColumns() {
	if(dataFrame != NULL) {
		return dataFrame->getWaterLevelColumns();
	}else{
		return nullWaterLevelColumns;
	}
}

const api::IDataModel::Temperature_Column_Container_t&
DataModel::getTemperatureColumns() {
	if(dataFrame != NULL) {
		return dataFrame->getTemperatureColumns();
	}else{
		return nullTemperatureColumns;
	}
}

boost::shared_ptr<api::IDataModel::Column_t>
DataModel::getAnyAssignedColumn() {
	if(getBehaviorIndicatorColumns().size() != 0) {
		return (*getBehaviorIndicatorColumns().begin());
	}

	if(getTemperatureColumns().size() != 0) {
		return (*getTemperatureColumns().begin());
	}

	if(getWaterLevelColumns().size() != 0) {
		return (*getWaterLevelColumns().begin());
	}

	boost::shared_ptr<api::IDataModel::Column_t> nullReturn;
	return nullReturn;
}


const boost::shared_ptr<api::IDataModel::DataFrame_t> &
DataModel::getDataFrame() const {
	return dataFrame;
}

void
DataModel::applyDateFormat(QString dateFormat) {
	this->dateFormat = dateFormat;
	dataFrame->updateFromApplyDateFormat(dataFrameProvider->applyDateFormat(dataFrame->getDataFrame(), getDateColumn()->getIndex(), dateFormat));
	dateFormatApplied = true;
	api::IDataModel::fireDataModelLoaded();
	updateAnalysisTools();
}

bool
DataModel::isDateColumnEditable() {
	return !dateFormatApplied;
}

bool
DataModel::isDateColumnAssignmentComplete() {
	return dateFormatApplied;
}

const QString &
DataModel::getDateFormat() const {
	return dateFormat;
}

void
DataModel::setDateFormat(QString dateFormat) {
	this->dateFormat = dateFormat;
}

void
DataModel::addDataModelAnalysis(boost::shared_ptr<api::IDataModel::AnalysisTool_t> dataModelAnalysis) {
	dataModelAnalysesTools.push_back(dataModelAnalysis);
}

void
DataModel::removeDataModelAnalysis(boost::shared_ptr<api::IDataModel::AnalysisTool_t> dataModelAnalysis) {
	dataModelAnalysesTools.erase(std::remove(dataModelAnalysesTools.begin(), dataModelAnalysesTools.end(), dataModelAnalysis));
}

void
DataModel::updateAnalysisTools() {
	BOOST_FOREACH(boost::shared_ptr<api::IDataModel::AnalysisTool_t> analysisTool, dataModelAnalysesTools) {
		analysisTool->configurationUpdateRequested();
		analysisTool->execute();
	}
}

} //namespace model
} //namespace dambase
