/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactors.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */


#include "DataModelAnalysisInfluenceFactors.hpp"

namespace dambase {
namespace model {

DataModelAnalysisInfluenceFactors::DataModelAnalysisInfluenceFactors(boost::shared_ptr<DataModelAnalysisInfluenceFactors::DataModel_t> dataModel, Provider_t *provider)
	:dataModel(dataModel), provider(provider), currentBehaviorIndicatorColumnIndex(-1), currentInfluenceIndicatorColumnIndex(-1)
{
	using namespace boost;
	dataModelBehaviorIndicatorsChangedConnection = COM_SUBSCRIBE_FOR(dataModel, BehaviorIndicatorAssignmentChanged, bind(&api::IDataModelAnalysisInfluenceFactors::fireBehaviorIndicatorsChanged, this));
	dataModelWaterLevelAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, WaterLevelAssignmentChanged, bind(&api::IDataModelAnalysisInfluenceFactors::fireAnyInfluenceFactorChanged, this));
	dataModelTemperatureAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, TemperatureAssignmentChanged, bind(&api::IDataModelAnalysisInfluenceFactors::fireAnyInfluenceFactorChanged, this));
}

DataModelAnalysisInfluenceFactors::~DataModelAnalysisInfluenceFactors(){;}


void
DataModelAnalysisInfluenceFactors::execute() {
	if(getCurrentBehaviorIndicatorColumn() != NULL && getCurrentInfluenceFactorColumn() != NULL) {
		rDataFrameInfluenceFactors = provider->load(dataModel->getDataFrame(), getCurrentBehaviorIndicatorColumn(), getCurrentInfluenceFactorColumn());
		api::IDataModelAnalysisInfluenceFactors::fireLoaded();

	}else{
		if(rDataFrameInfluenceFactors != NULL) {
			api::IDataModelAnalysisInfluenceFactors::fireLoaded();
		}
	}
}

bool
DataModelAnalysisInfluenceFactors::supportsReporting() {
	return false;
}

bool
DataModelAnalysisInfluenceFactors::supportRawDataExport() {
	return false;
}

void
DataModelAnalysisInfluenceFactors::configurationUpdateRequested() {
	qDebug() << "DataModelAnalysisInfluenceFactors::" << __FUNCTION__;
}

int
DataModelAnalysisInfluenceFactors::getAnEstimatedBehaviorIndicator() {
	if(!getBehaviorIndicatorColumns().empty()) {
		return (*(getBehaviorIndicatorColumns().begin()))->getIndex();
	}

	return -1;
}

void
DataModelAnalysisInfluenceFactors::estimateCurrentBehaviorIndicator() {
	currentBehaviorIndicatorColumnIndex = getAnEstimatedBehaviorIndicator();
}

int
DataModelAnalysisInfluenceFactors::getAnEstimatedInfluenceFactor() {
	if(!getWaterLevelColumns().empty()) {
		return (*(getWaterLevelColumns().begin()))->getIndex();
	}else if(!getTemperatureColumns().empty()) {
		return (*(getTemperatureColumns().begin()))->getIndex();
	}

	return -1;
}

void
DataModelAnalysisInfluenceFactors::estimateCurrentInfluenceFactor() {
	currentInfluenceIndicatorColumnIndex = getAnEstimatedInfluenceFactor();
}

void
DataModelAnalysisInfluenceFactors::setCurrentBehaviorIndicator(int index) {
	currentBehaviorIndicatorColumnIndex = index;
}

boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t>
DataModelAnalysisInfluenceFactors::getCurrentBehaviorIndicatorColumn() {
	typedef api::IDataModelAnalysisInfluenceFactors::Column_t behavior_t;

	BOOST_FOREACH(boost::shared_ptr<behavior_t> behaviorIndicator, dataModel->getBehaviorIndicatorColumns()) {
		if(behaviorIndicator->getIndex() == currentBehaviorIndicatorColumnIndex) {
			return behaviorIndicator;
		}
	}

	boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t> nullReturn;
	return nullReturn;
}

void
DataModelAnalysisInfluenceFactors::setCurrentInfluenceFactor(int index) {
	currentInfluenceIndicatorColumnIndex = index;
}

boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t>
DataModelAnalysisInfluenceFactors::getCurrentInfluenceFactorColumn() {
	typedef api::IDataModelAnalysisInfluenceFactors::Column_t column_t;

	BOOST_FOREACH(boost::shared_ptr<column_t> column, dataModel->getWaterLevelColumns()) {
		if(column->getIndex() == currentInfluenceIndicatorColumnIndex) {
			return column;
		}
	}

	BOOST_FOREACH(boost::shared_ptr<column_t> column, dataModel->getTemperatureColumns()) {
		if(column->getIndex() == currentInfluenceIndicatorColumnIndex) {
			return column;
		}
	}

	boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t> nullReturn;
	return nullReturn;
}

api::IDataModelAnalysisInfluenceFactors::ValueContainer_t
DataModelAnalysisInfluenceFactors::getBehaviorIndicatorColumnValues() {
	if(rDataFrameInfluenceFactors == NULL) {
		return nullBehaviorIndicatorColumnValues;
	}

	return rDataFrameInfluenceFactors->getBehaviorIndicatorColumnValues();
}

api::IDataModelAnalysisInfluenceFactors::ValueContainer_t
DataModelAnalysisInfluenceFactors::getInfluenceFactorColumnValues() {
	if(rDataFrameInfluenceFactors == NULL) {
		return nullInfluenceFactorColumnValues;
	}

	return rDataFrameInfluenceFactors->getInfluenceFactorColumnValues();
}

const api::IDataModelAnalysisInfluenceFactors::Behavior_Indicator_Column_Container_t&
DataModelAnalysisInfluenceFactors::getBehaviorIndicatorColumns() {
	return dataModel->getBehaviorIndicatorColumns();
}

const DataModelAnalysisInfluenceFactors::DataModel_t::Water_Level_Column_Container_t&
DataModelAnalysisInfluenceFactors::getWaterLevelColumns() {
	return dataModel->getWaterLevelColumns();
}

const DataModelAnalysisInfluenceFactors::DataModel_t::Temperature_Column_Container_t&
DataModelAnalysisInfluenceFactors::getTemperatureColumns() {
	return dataModel->getTemperatureColumns();
}

} //namespace model
} //namespace dambase
