/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RawDataCollector.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#include <stdexcept>
#include "RawDataCollector.hpp"

namespace dambase {
namespace model {


RawDataCollector::Collection::Collection(QString name, QString keyName):name(name), keyName(keyName){;}
RawDataCollector::Collection::~Collection(){;}

void
RawDataCollector::Collection::addValues(QString name, std::vector<std::string> keys, std::vector<double> values) {
	for(int i = 0; i < keys.size(); ++i) {
		collection[QString::fromStdString(keys.at(i))].push_back(RawDataCollector::DataRow(name, values.at(i)));
	}
}

void
RawDataCollector::Collection::addValues(QString name, std::vector<QString> keys, std::vector<double> values) {
	std::vector<std::string> stdStringKeys;
	BOOST_FOREACH(QString key, keys) {
		stdStringKeys.push_back(key.toStdString());
	}

	addValues(name, stdStringKeys, values);
}

void
RawDataCollector::Collection::addValues(QString name, std::vector<unsigned> keys, std::vector<double> values) {
	std::vector<std::string> keysAsString;

	unsigned maxKey = *std::max_element(keys.begin(), keys.end());
	unsigned numberOfPlaceholders = calculateNrOfPlaceholdersFor(maxKey);

	BOOST_FOREACH(unsigned key, keys) {
		//keysAsString.push_back(QString::number(key).toStdString());
		//QString number = QString("%1").arg(key, 8, QChar('0')).toUpper();
		QString number = QString("%1").arg(key).rightJustified(numberOfPlaceholders, '0', false);
		//qDebug() << __FUNCTION__ << " " << QString::number(key) <<  " : " << number;
		keysAsString.push_back(number.toStdString());
	}

	addValues(name, keysAsString, values);
}

void
RawDataCollector::Collection::addValues(QString name, std::vector<double> keys, std::vector<double> values) {
	std::vector<std::string> keysAsString;
	BOOST_FOREACH(double key, keys) {
		keysAsString.push_back(QString::number(key).toStdString());
	}

	addValues(name, keysAsString, values);
}

void
RawDataCollector::Collection::addValue(QString name, unsigned key, double value) {
	std::vector<std::string> keysAsString;
	QString number = QString("%1").arg(key).rightJustified(calculateNrOfPlaceholdersFor(UINT_MAX), '0', false);
	//qDebug() << __FUNCTION__ << " " << QString::number(key) <<  " : " << number;
	keysAsString.push_back(number.toStdString());

	std::vector<double> values;
	values.push_back(value);

	addValues(name, keysAsString, values);
}

QString
RawDataCollector::Collection::getKeyName() {
	return keyName;
}

QString
RawDataCollector::Collection::getName() {
	return name;
}

std::map<QString, std::vector<api::IRawDataCollector::DataRow> > &
RawDataCollector::Collection::getCollection() {
	return collection;
}

unsigned
RawDataCollector::Collection::calculateNrOfPlaceholdersFor(unsigned u) {
	 return u > 0 ? (int) log10 ((double) u) + 1 : 0;
}

/**
 *
 */

RawDataCollector::DataRow::DataRow(QString name, double value):name(name), value(value){;}
RawDataCollector::DataRow::~DataRow(){;}

double
RawDataCollector::DataRow::getValue() {return value;}

QString
RawDataCollector::DataRow::getName(){return name;}

/**
 *
 */

RawDataCollector::RawDataCollector(){;}
RawDataCollector::~RawDataCollector(){;}

void
RawDataCollector::addStatisticsValueColumn(QString name, std::vector<std::string> dates, std::vector<double> values) {
	for(int i = 0; i < dates.size(); ++i) {
		collection[QString::fromStdString(dates.at(i))].push_back(DataRow(name, values.at(i)));
	}
}

void
RawDataCollector::writeHeader(QFile *outputFile, boost::shared_ptr<IRawDataCollector::Project_t> project, boost::shared_ptr<IRawDataCollector::Analysis_t> analysis, boost::shared_ptr<IRawDataCollector::RegressionModel_t> regressionModel) {
	QTextStream textStream(outputFile);

	textStream << QObject::tr("Export of ");
	textStream << project->getName();
	textStream << " / " << analysis->getName();
	textStream << " / " << regressionModel->getName();
	textStream << QObject::tr(" at ") << QDateTime::currentDateTime().toString();
	textStream << endl << endl;

	//selected algorithm
	textStream << "Algorithm: ";
	if(regressionModel->isStandardAlgorithm()) {
		textStream << "Ordinary least square" << endl << endl;
	}else if(regressionModel->isPraisWinstenAlgorithm()) {
		textStream << "Prais Winsten with rho = " << QString::number(regressionModel->getPraisWinstenRho()) << endl << endl;
	}else if(regressionModel->isRidgeAlgorithm()) {
		textStream << "Ridge Regression with k = " << QString::number(regressionModel->getRidgeKValue()) << endl << endl;
	}else if(regressionModel->isPCRAlgorithm()) {
		textStream << "Principal Component Regression with r = " << QString::number(regressionModel->getPCRrValue()) << endl << endl;
	}else{
		throw std::logic_error("No known algorithm");
	}
}

void
RawDataCollector::write(QFile *outputFile) {
	typedef std::pair<QString, std::vector<IRawDataCollector::DataRow> > pair_t;
	QTextStream textStream(outputFile);

	int rowIndex = 0;
	textStream << QObject::tr("Regression values");
	BOOST_FOREACH(pair_t pair, collection) {
		//first, plot the headers (date and names of the columns)
		if(rowIndex == 0) {
			int columnIndex = 0;
			BOOST_FOREACH(IRawDataCollector::DataRow &dataRow, pair.second) {
				//on the first row, at the first columnIndex, plot header 'date'
				if(columnIndex == 0) {
					textStream << QObject::tr("Date");
				}
				//TODO: find all headers down all dataRows (if the current dataRow has no data for this date (pair), no header will be written
				++columnIndex;
				//then follow with the names of the columns
				textStream << '\t' << dataRow.getName();
			}
			textStream << '\n';
		}

		textStream << pair.first; //date
		BOOST_FOREACH(IRawDataCollector::DataRow &dataRow, pair.second) {
			textStream << '\t' << dataRow.getValue();
		}

		textStream << '\n';
		++rowIndex;
	}

	//then, individual collections
	BOOST_FOREACH(boost::shared_ptr<IRawDataCollector::Collection_t> collection, individualCollections){
		int rowIndex = 0;
		textStream << '\n' << collection->getName() << '\n';
		BOOST_FOREACH(pair_t pair, collection->getCollection()) {
			//first, plot the headers (date and names of the columns)
			if(rowIndex == 0) {
				int columnIndex = 0;
				BOOST_FOREACH(IRawDataCollector::DataRow &dataRow, pair.second) {
					//on the first row, at the first columnIndex, plot header 'keyName of the collection'
					if(columnIndex == 0) {
						textStream << collection->getKeyName();
					}
					++columnIndex;
					//then follow with the names of the columns
					textStream << '\t' << dataRow.getName();
				}
				textStream << '\n';
			}

			textStream << pair.first; //key
			BOOST_FOREACH(IRawDataCollector::DataRow &dataRow, pair.second) {
				textStream << '\t' << dataRow.getValue();
			}

			textStream << '\n';
			++rowIndex;
		}
	}
}

QString
RawDataCollector::getSupportedFileTypesAsString() {
	return "Text (*.txt)";
}

boost::shared_ptr<api::IRawDataCollector::Collection_t>
RawDataCollector::createNewCollection(QString name, QString keyName) {
	boost::shared_ptr<IRawDataCollector::Collection_t> collection(new Collection(name, keyName));
	individualCollections.push_back(collection);

	return collection;
}


} //namespace model
} //namespace dambase
