/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalBehavior.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include "RegressionFitAnalysisPhysicalBehavior.hpp"
#include "api/IDataModel.hpp"


namespace dambase {
namespace model {

RegressionFitAnalysisPhysicalBehavior::RegressionFitAnalysisPhysicalBehavior():provider(NULL){;}

RegressionFitAnalysisPhysicalBehavior::RegressionFitAnalysisPhysicalBehavior(
		boost::shared_ptr<RegressionModel_t> regressionModel,
		std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> > assignmentCategories,
		boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> assignmentCategory,
		Provider_t *provider)
	:regressionModel(regressionModel), assignmentCategories(assignmentCategories), assignmentCategory(assignmentCategory), provider(provider)
{;}

RegressionFitAnalysisPhysicalBehavior::~RegressionFitAnalysisPhysicalBehavior()
{;}

void
RegressionFitAnalysisPhysicalBehavior::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisPhysicalBehavior::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}


std::vector<boost::shared_ptr<r::api::IRRegressionFitAnalysisPhysicalBehavior> >
RegressionFitAnalysisPhysicalBehavior::getItems() {
	return items;
}

void
RegressionFitAnalysisPhysicalBehavior::setAssignmentCategories(std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> > assignmentCategories) {
	this->assignmentCategories = assignmentCategories;
}

std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> >
RegressionFitAnalysisPhysicalBehavior::getAssignmentCategories() {
	return regressionModel->getRegressionFormula()->getInvolvedAssignmentCategoriesWithAnInfluenceFactor();
}

void
RegressionFitAnalysisPhysicalBehavior::setAssignmentCategory(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> assignmentCategory) {
	if(assignmentCategory == NULL) {
		return;
	}

	this->assignmentCategory = assignmentCategory;
}

boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t>
RegressionFitAnalysisPhysicalBehavior::getAssignmentCategory() {
	return assignmentCategory;
}

void
RegressionFitAnalysisPhysicalBehavior::execute() {
	//find responseColumn for the given assignment category
	//TODO: right at the moment, we use only Water level which means that we cannot deal with seasonal
	typedef api::IDataModel::Influence_Factor_t InfluenceFactor_t;

	if(assignmentCategory == NULL) {
		throw std::logic_error("No assignment category set");
	}

	items.clear();

	if(assignmentCategory->hasColumns() == true) {
		std::vector<boost::shared_ptr<InfluenceFactor_t> > influenceFactorsOfThisAssignmentCategory;
		influenceFactorsOfThisAssignmentCategory = regressionModel->getRegressionFormula()->findInfluenceFactorsOfRegressorsAssignedTo(
					regressionModel->getRegressionFormula()->getActiveRegressors(),
					assignmentCategory
				);

		BOOST_FOREACH(boost::shared_ptr<InfluenceFactor_t> influenceFactor, influenceFactorsOfThisAssignmentCategory) {
			boost::shared_ptr<r::api::IRRegressionFitAnalysisPhysicalBehavior> item = provider->load(
				regressionModel->getRegressionFit(),
				regressionModel->getRegressionFormula()->buildFormulaForInfluenceFactorWithinAssignmentCategory(influenceFactor, assignmentCategory),
				//we choose the prediction frame as the data frame of interest here but we can make this configurable
				regressionModel->getRegressionFormula()->getDataFrame()->getDataFrame(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn(),
				regressionModel->getRegressionFit()->getRegressionFitDataFrame()->getDateColumn()->getFirstValue().getDate(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn()->getLastValue().getDate(),
				regressionModel->getRegressionFit()->getOverallMaxStartupTime(),
				influenceFactor,
				influenceFactor->getName()
			);
			items.push_back(item);
		}
	}else{
	}

	api::IRegressionFitAnalysisPhysicalBehavior::fireLoaded();
}

boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t>
RegressionFitAnalysisPhysicalBehavior::findAssignmentCategoryByIdentifier(QString identifier) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> assignmentCategory, assignmentCategories) {
		if(assignmentCategory->getIdentifier() == identifier) {
			return assignmentCategory;
		}
	}

	boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> nullReturn;
	return nullReturn;
}

void
RegressionFitAnalysisPhysicalBehavior::build(api::IRegressionFitAnalysisPhysicalBehavior::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisPhysicalBehavior>::shared_from_this(), rc);
}

void
RegressionFitAnalysisPhysicalBehavior::build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisPhysicalBehavior>::shared_from_this(), collector);
}

void
RegressionFitAnalysisPhysicalBehavior::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisPhysicalBehavior::getName() {
	return this->name;
}

void
RegressionFitAnalysisPhysicalBehavior::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisPhysicalBehavior::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisPhysicalBehavior::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
