/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModel.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <QDebug>
#include <QString>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include <vector>
#include <algorithm>
#include <cmath>
#include "RegressionModel.hpp"

namespace dambase {
namespace model {


RegressionModel::RegressionModel(): currentAlgorithm(api::IRegressionModel::Standard), praisWinstenIterations(10), ridgeKValue(0.06), pcrrValue(1), regressionFitProvider(NULL)
{;}

template<typename DateDataPoint>
struct RemoveDateDataPointIfEarlier {
	RemoveDateDataPointIfEarlier(DateDataPoint &reference):reference(reference){;}

	bool operator()(DateDataPoint const &l) const {
		return l < reference;
	}

	DateDataPoint reference;
};

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModel::setDataModel(boost::shared_ptr<api::IRegressionModel::DataModel_t> dataModel) {
	this->dataModel = dataModel;
}

/**
 * This is primarly used by the serialization process
 */
const boost::shared_ptr<api::IRegressionModel::TimeRange_t> &
RegressionModel::getTimeRange() const {
	return timeRange;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModel::setTimeRange(boost::shared_ptr<api::IRegressionModel::TimeRange_t> timeRange) {
	this->timeRange = timeRange;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModel::setRegressionFormula(boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> regressionFormula) {
	this->regressionFormula = regressionFormula;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModel::setRegressionFitProvider(api::IRegressionModel::RegressionFitProvider_t *regressionFitProvider) {
	this->regressionFitProvider = regressionFitProvider;
}

RegressionModel::RegressionModel(boost::shared_ptr<api::IRegressionModel::DataModel_t> dataModel, boost::shared_ptr<api::IRegressionModel::TimeRange_t> timeRange, boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> regressionFormula, api::IRegressionModel::RegressionFitProvider_t *regressionFitProvider)
	:dataModel(dataModel),
	 regressionFormula(regressionFormula),
	 regressionFitProvider(regressionFitProvider),
	 timeRange(timeRange),
	 currentAlgorithm(api::IRegressionModel::Standard),
	 praisWinstenIterations(10),
	 ridgeKValue(0.06),
	 pcrrValue(1)
{
	if(dataModel->getBehaviorIndicatorColumns().size() == 0) {
		throw std::logic_error("no behavior indicator defined to get from data model");
	}

	regressionFormula->setBehaviorIndicatorColumn(*dataModel->getBehaviorIndicatorColumns().begin());

	availableRegressionTimeWindow = dataModel->getDateColumn()->getValues();
}

RegressionModel::~RegressionModel() {;}

const QString &
RegressionModel::getDescription() const {
	return description;
}

void
RegressionModel::setDescription(QString& description) {
	this->description = description;
	api::IRegressionModel::fireDescriptionChanged();
}

const QString &
RegressionModel::getName() const {
	return name;
}

void
RegressionModel::setName(QString& name) {
	this->name = name;
	api::IRegressionModel::fireNameChanged();
}

void
RegressionModel::setFromDate(int index, bool fireSignal) {
	timeRange->setFromDate(dataModel->getDateColumn()->getValueAt(index));
	if(fireSignal == true) {
		api::IRegressionModel::fireFromDateChanged();
	}
}

void
RegressionModel::setFromDate(api::IRegressionModel::DataModel_t::Date_Data_Point_t &fromDate, bool fireSignal = true) {
	timeRange->setFromDate(fromDate);
	if(fireSignal == true) {
		api::IRegressionModel::fireFromDateChanged();
	}
}

void
RegressionModel::setToDate(int index) {
	timeRange->setToDate(dataModel->getDateColumn()->getValueAt(index));
	api::IRegressionModel::fireToDateChanged();
}

/**
 * The date within the availableRegressionTimeWindow the user can choose from to set the beginning of the regression
 */
const api::IRegressionModel::DataModel_t::Date_Data_Point_t &
RegressionModel::getFromDate() const {
	return timeRange->getFromDate();
}

/**
 * The latest date within the availableRegressionTimeWindow the user can choose from to set the end of the regression calibration date
 */
const api::IRegressionModel::DataModel_t::Date_Data_Point_t &
RegressionModel::getToDate() const {
	return timeRange->getToDate();
}

/**
 * Returns a list of dates from which the user can choose from. Considers startup times of the formula
 */
std::vector<api::IRegressionModel::DataModel_t::Date_Data_Point_t>
RegressionModel::getAvailableRegressionTimeWindow() {
	return availableRegressionTimeWindow;
}

/**
 * This is primarly used by the deserialization process. For other purposes, use #updateAvailableRegressionTimeWindow
 */
void
RegressionModel::restoreAvailableRegressionTimeWindow() {
	availableRegressionTimeWindow = dataModel->getDateColumn()->getValues();
	updateAvailableRegressionTimeWindow();
}

/**
 * The aim of this method is to provide an access point which allows updating values (such as available regression time window, maximum pcr r value ...)
 * from the controller.
 */
void
RegressionModel::updateFromFormulaChanges() {
	updateAvailableRegressionTimeWindow();
	updatePCRrValue();
}

void
RegressionModel::updateAvailableRegressionTimeWindow() {
	//setFromDate to the minimumStartDate if currentFromDate is earlier than the beginning of the dataModel's dateColumn
	//second, set availableRegressionTimeWindow new if first date is later than current fromDate

	typedef api::IRegressionModel::DataModel_t::Date_Data_Point_t DateDataPoint_t;

	if(availableRegressionTimeWindow.size() == 0) {
		throw std::logic_error("available regression time window has length 0");
	}

	DateDataPoint_t dateColumnStartDate = dataModel->getDateColumn()->getFirstValue();
	int maxStartupTime = regressionFormula->calculateOverallMaxStartupTime();

	DateDataPoint_t earliestPossibleStartDate = dataModel->getDateColumn()->getValueAt(dateColumnStartDate, maxStartupTime);
	if(earliestPossibleStartDate > availableRegressionTimeWindow.front()) {
		//shorten availableRegressionTimeWindow to the earliestPossibleStartDate
		availableRegressionTimeWindow.erase(
			std::remove_if(availableRegressionTimeWindow.begin(), availableRegressionTimeWindow.end(), RemoveDateDataPointIfEarlier<DateDataPoint_t>(earliestPossibleStartDate)),
			availableRegressionTimeWindow.end()
		);
		bool fromDateUpdated = false;
		if(getFromDate() < earliestPossibleStartDate) {
			setFromDate(earliestPossibleStartDate, false);
			fromDateUpdated = true;
		}

		api::IRegressionModel::fireAvailableRegressionTimeWindowChanged();
		if(fromDateUpdated) {
			api::IRegressionModel::fireFromDateChanged();
		}

	}else if(earliestPossibleStartDate < availableRegressionTimeWindow.front()){
		//we also might enlarge the available regression time window if the earliestPossible
		availableRegressionTimeWindow.clear();
		std::vector<IRegressionModel::DataModel_t::Date_Data_Point_t> newValues = dataModel->getDateColumn()->getValues();
		availableRegressionTimeWindow.insert(availableRegressionTimeWindow.begin(), newValues.begin(), newValues.end());
		availableRegressionTimeWindow.erase(
			std::remove_if(availableRegressionTimeWindow.begin(), availableRegressionTimeWindow.end(), RemoveDateDataPointIfEarlier<DateDataPoint_t>(earliestPossibleStartDate)),
			availableRegressionTimeWindow.end()
		);

		api::IRegressionModel::fireAvailableRegressionTimeWindowChanged();
	}

	qDebug() << __FUNCTION__ << "begin : " << availableRegressionTimeWindow.front().getDate() << " : end: " << availableRegressionTimeWindow.back().getDate();
}

void
RegressionModel::setBehaviorIndicatorColumn(int behaviorIndicatorColumnIndex) {
	if(dataModel->getBehaviorIndicatorColumnWith(behaviorIndicatorColumnIndex) == NULL) {
		throw std::logic_error("behavior indicator with such index does not exist");
	}
	regressionFormula->setBehaviorIndicatorColumn(dataModel->getBehaviorIndicatorColumnWith(behaviorIndicatorColumnIndex));
	api::IRegressionModel::fireBehaviorIndicatorChanged();
}

boost::shared_ptr<api::IRegressionModel::DataModel_t::Behavior_Indicator_Column_t>
RegressionModel::getBehaviorIndicatorColumn() {
	return regressionFormula->getBehaviorIndicatorColumn();
}

void
RegressionModel::setupBestFittingRegressionFormula() {
	throw std::logic_error("RegressionModel::setupBestFittingRegressionFormula : not implemented yet");
}

const boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> &
RegressionModel::getRegressionFormula() const {
	return regressionFormula;
}

void
RegressionModel::calculateRegressionFit() {
	/**
	 * Check some preconditions before calculating the new values
	 */
	if(timeRange->getFromDate() > timeRange->getToDate()) {
		throw std::logic_error("from date (" + timeRange->getFromDate().getDate().toStdString() + ") is later than to date (" + timeRange->getToDate().getDate().toStdString() + ").");
	}
	if(timeRange->getFromDate() == timeRange->getToDate()) {
		throw std::logic_error("from date (" + timeRange->getFromDate().getDate().toStdString() + ") and to date (" + timeRange->getToDate().getDate().toStdString() + ") cannot be at the same date.");
	}

	if(isStandardAlgorithm()) {
		qDebug() << __FUNCTION__ << " 1";
		regressionFit = regressionFitProvider->load(
					regressionFitProvider->loadRegressionFitDataFrame(
								dataModel->getDataFrame(),
								regressionFormula,
								timeRange->getFromDate().getDate(),
								timeRange->getToDate().getDate()
							),
					regressionFormula
				);
	}else if(isPraisWinstenAlgorithm()){
		qDebug() << __FUNCTION__ << " 2";
		regressionFit = regressionFitProvider->loadPraisWinsten(
						regressionFitProvider->loadRegressionFitDataFrame(
									dataModel->getDataFrame(),
									regressionFormula,
									timeRange->getFromDate().getDate(),
									timeRange->getToDate().getDate()
								),
						regressionFormula,
						praisWinstenIterations
					);
	}else if(isRidgeAlgorithm()){
		qDebug() << __FUNCTION__ << " 2";
		regressionFit = regressionFitProvider->loadRidge(
						regressionFitProvider->loadRegressionFitDataFrame(
									dataModel->getDataFrame(),
									regressionFormula,
									timeRange->getFromDate().getDate(),
									timeRange->getToDate().getDate()
								),
						regressionFormula,
						ridgeKValue
					);
	}else if(isPCRAlgorithm()){
		qDebug() << __FUNCTION__ << " 2";
		regressionFit = regressionFitProvider->loadPCR(
						regressionFitProvider->loadRegressionFitDataFrame(
									dataModel->getDataFrame(),
									regressionFormula,
									timeRange->getFromDate().getDate(),
									timeRange->getToDate().getDate()
								),
						regressionFormula,
						pcrrValue
					);
	}else{
		throw std::logic_error("unknown algorithm selected");
	}

	api::IRegressionModel::fireCalculated();

	qDebug() << __FUNCTION__ << " 3";
	//regressionFit = regressionFitProvider->load(dataModel->getDataFrame()->getDataFrame(), regressionFormula);

	//recalculate all registered analysis tools to make sure that each of them is up to date with the new calculated fit
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModel::AnalysisTool_t> analysisTool, analysisTools) {
		analysisTool->execute();
	}
	qDebug() << __FUNCTION__ << " 4";
}

boost::shared_ptr<api::IRegressionModel::RegressionFit_t>
RegressionModel::getRegressionFit() {
	return regressionFit;
}

const boost::shared_ptr<api::IRegressionModel::DataModel_t> &
RegressionModel::getDataModel() const {
	return dataModel;
}

//TODO: rename after refactoring has finished
void
RegressionModel::addRegressionModelAnalysis(boost::shared_ptr<api::IRegressionModel::AnalysisTool_t> analysis) {
	qDebug() << "RegressionModel::" << __FUNCTION__;
	analysisTools.push_back(analysis);
}

api::IRegressionModel::AnalysisToolsContainer_t &
RegressionModel::getAnalysisTools() {
	return analysisTools;
}

//TODO: rename after refactoring has finished
void
RegressionModel::removeRegressionModelAnalysis(boost::shared_ptr<api::IRegressionModel::AnalysisTool_t> analysis) {
	qDebug() << "RegressionModel::" << __FUNCTION__;
	analysisTools.erase(std::remove(analysisTools.begin(), analysisTools.end(), analysis));
}

bool
RegressionModel::isStandardAlgorithm() {
	return currentAlgorithm == Standard;
}

bool
RegressionModel::isPraisWinstenAlgorithm() {
	return currentAlgorithm == PraisWinsten;
}

bool
RegressionModel::isRidgeAlgorithm() {
	return currentAlgorithm == Ridge;
}

bool
RegressionModel::isPCRAlgorithm() {
	return currentAlgorithm == Pcr;
}

const api::IRegressionModel::Algorithm &
RegressionModel::getAlgorithm() const {
	return currentAlgorithm;
}

const int
RegressionModel::getAlgorithmAsInt() const {
	return (const int)currentAlgorithm;
}

void
RegressionModel::setAlgorithm(int algorithm) {
	this->currentAlgorithm = (api::IRegressionModel::Algorithm)algorithm;
	api::IRegressionModel::fireAlgorithmChanged();
}

double
RegressionModel::getPraisWinstenRho() {
	if(regressionFit == NULL) {
		qDebug() << "RegressionModel::" << __FUNCTION__ << " : " << "regressoinFit == NULL";
		return NAN;
	}

	return regressionFit->getPraisWinstenRho();
}

const unsigned &
RegressionModel::getPraisWinstenIterations() const {
	return praisWinstenIterations;
}

void
RegressionModel::setPraisWinstenIterations(unsigned praisWinstenIterations) {
	this->praisWinstenIterations = praisWinstenIterations;
}

const double &
RegressionModel::getRidgeKValue() const {
	return ridgeKValue;
}

void
RegressionModel::setRidgeKValue(double k) {
	ridgeKValue = k;
}

unsigned
RegressionModel::getMaxPCRrValue() {
	return regressionFormula->getActiveRegressors().size();
}

const unsigned &
RegressionModel::getPCRrValue() const {
	return pcrrValue;
}

void
RegressionModel::setPCRrValue(unsigned r) {
	pcrrValue = r;
	api::IRegressionModel::firePCRrValueChanged();
}

/**
 * This method gets called from updateFromFormulaChanges in order to make sure that the r value set does not exceed maxPCRrValue
 */
void
RegressionModel::updatePCRrValue() {
	unsigned maxPCRrValue = getMaxPCRrValue();
	if(pcrrValue > maxPCRrValue) {
		setPCRrValue(maxPCRrValue);
	}
}



} //namespace model
} //namespace dambase
