/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModel.hpp
 *
 *  Created on: Feb 19, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODEL_HPP_
#define REGRESSIONMODEL_HPP_

#include <QDebug>
#include <QString>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include <vector>
#include <algorithm>
#include <cmath>
#include "api/IRegressionModel.hpp"


namespace dambase {
namespace model {

class RegressionModel : public virtual api::IRegressionModel {
public:
	RegressionModel();
	virtual ~RegressionModel();

	enum Algorithm {
		Standard,
		PraisWinsten,
		Ridge,
		Pcr
	};

	/**
	 * This is primarly used by the deserialization process
	 */
	void setDataModel(boost::shared_ptr<api::IRegressionModel::DataModel_t> dataModel);

	/**
	 * This is primarly used by the serialization process
	 */
	const boost::shared_ptr<api::IRegressionModel::TimeRange_t> &getTimeRange() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setTimeRange(boost::shared_ptr<api::IRegressionModel::TimeRange_t> timeRange);

	/**
	 * This is primarly used by the deserialization process
	 */
	void setRegressionFormula(boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> regressionFormula);
	/**
	 * This is primarly used by the deserialization process
	 */
	void setRegressionFitProvider(api::IRegressionModel::RegressionFitProvider_t *regressionFitProvider);

	RegressionModel(boost::shared_ptr<api::IRegressionModel::DataModel_t> dataModel, boost::shared_ptr<api::IRegressionModel::TimeRange_t> timeRange, boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> regressionFormula, api::IRegressionModel::RegressionFitProvider_t *regressionFitProvider);

	const QString &getDescription() const;

	void setDescription(QString& description);

	const QString &getName() const;

	void setName(QString& name);

	void setFromDate(int index, bool fireSignal);

	void setFromDate(api::IRegressionModel::DataModel_t::Date_Data_Point_t &fromDate, bool fireSignal);

	void setToDate(int index);
	/**
	 * The date within the availableRegressionTimeWindow the user can choose from to set the beginning of the regression
	 */
	const api::IRegressionModel::DataModel_t::Date_Data_Point_t &getFromDate() const;

	/**
	 * The latest date within the availableRegressionTimeWindow the user can choose from to set the end of the regression calibration date
	 */
	const api::IRegressionModel::DataModel_t::Date_Data_Point_t &getToDate() const;

	/**
	 * Returns a list of dates from which the user can choose from. Considers startup times of the formula
	 */
	std::vector<api::IRegressionModel::DataModel_t::Date_Data_Point_t> getAvailableRegressionTimeWindow();

	/**
	 * This is primarly used by the deserialization process. For other purposes, use #updateAvailableRegressionTimeWindow
	 */
	void restoreAvailableRegressionTimeWindow();

	/**
	 * The aim of this method is to provide an access point which allows updating values (such as available regression time window, maximum pcr r value ...)
	 * from the controller.
	 */
	void updateFromFormulaChanges();

	void updateAvailableRegressionTimeWindow();

	void setBehaviorIndicatorColumn(int behaviorIndicatorColumnIndex);

	boost::shared_ptr<api::IRegressionModel::DataModel_t::Behavior_Indicator_Column_t> getBehaviorIndicatorColumn();

	void setupBestFittingRegressionFormula();

	const boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> &getRegressionFormula() const;

	void calculateRegressionFit();

	boost::shared_ptr<api::IRegressionModel::RegressionFit_t> getRegressionFit();

	const boost::shared_ptr<api::IRegressionModel::DataModel_t> &getDataModel() const;

	//TODO: rename after refactoring has finished
	void addRegressionModelAnalysis(boost::shared_ptr<api::IRegressionModel::AnalysisTool_t> analysis);

	api::IRegressionModel::AnalysisToolsContainer_t &getAnalysisTools();

	//TODO: rename after refactoring has finished
	void removeRegressionModelAnalysis(boost::shared_ptr<api::IRegressionModel::AnalysisTool_t> analysis);

	bool isStandardAlgorithm();

	bool isPraisWinstenAlgorithm();

	bool isRidgeAlgorithm();

	bool isPCRAlgorithm();

	const api::IRegressionModel::Algorithm &getAlgorithm() const;

	const int getAlgorithmAsInt() const;

	void setAlgorithm(int algorithm);

	double getPraisWinstenRho();

	const unsigned &getPraisWinstenIterations() const;

	void setPraisWinstenIterations(unsigned praisWinstenIterations);

	const double &getRidgeKValue() const;

	void setRidgeKValue(double k);

	unsigned getMaxPCRrValue();

	const unsigned &getPCRrValue() const;

	void setPCRrValue(unsigned r);

private:
	/**
	 * This method gets called from updateFromFormulaChanges in order to make sure that the r value set does not exceed maxPCRrValue
	 */
	void updatePCRrValue();

private:
	boost::shared_ptr<api::IRegressionModel::DataModel_t> dataModel;
	boost::shared_ptr<api::IRegressionModel::Regression_Formula_t> regressionFormula;
	api::IRegressionModel::RegressionFitProvider_t *regressionFitProvider;
	boost::shared_ptr<api::IRegressionModel::RegressionFit_t> regressionFit;
	QString name;
	QString description;
	boost::shared_ptr<api::IRegressionModel::TimeRange_t> timeRange;
	std::vector<api::IRegressionModel::DataModel_t::Date_Data_Point_t> availableRegressionTimeWindow;
	api::IRegressionModel::AnalysisToolsContainer_t analysisTools;
	api::IRegressionModel::Algorithm currentAlgorithm;
	unsigned praisWinstenIterations;
	double ridgeKValue;
	unsigned pcrrValue;
};


}} /* namespace dambase::model */

#endif /* REGRESSIONMODEL_HPP_ */
