/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Report.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include <QDebug>
#include <QString>
#include <QApplication>
#include <stdexcept>
#include <vector>
#include <algorithm>
#include <boost/foreach.hpp>
#include "Report.hpp"

namespace dambase {
namespace model {

Report::Report(boost::shared_ptr<api::IReport::Project_t> project, boost::shared_ptr<api::IReport::Analysis_t> analysis, boost::shared_ptr<api::IReport::RegressionModel_t> regressionModel,
		QString baseDirectory, QString supportedFileTypes, int plotWidth, int plotHeight)
	: project(project),
	  analysis(analysis),
	  regressionModel(regressionModel),
	  //factory(factory),
	  workingDirectory(baseDirectory),
	  supportedFileTypes(supportedFileTypes),
	  plotWidth(plotWidth),
	  plotHeight(plotHeight),
	  plotScaleFactor(1.0),
	  fontSize(10)
{
	availablePlotScaleFactors.push_back(0.5);
	availablePlotScaleFactors.push_back(1.0);
	availablePlotScaleFactors.push_back(2.0);
	availablePlotScaleFactors.push_back(3.0);
	availablePlotScaleFactors.push_back(4.0);

	availableFontSizes.push_back(10);
	availableFontSizes.push_back(11);
	availableFontSizes.push_back(12);
}

Report::~Report(){;}

QString
Report::getOutputFileName() {
	return outputFileName;
}

void
Report::setOutputFileName(QString outputFileName) {
	this->outputFileName = outputFileName;
}

void
Report::build() {
	markdownFileLocation = workingDirectory + "/report.md";
	qDebug() << "write markdown to : " << markdownFileLocation;

	QFile file(markdownFileLocation);
	file.open(QIODevice::WriteOnly | QIODevice::Text);

	api::IReport::ReportListElement_t::ReportContext_t reportContext(&file, workingDirectory);
	reportContext.setPlotScaleFactor(getPlotScaleFactor());
	reportContext.setPlotWidth(plotWidth);
	reportContext.setPlotHeight(plotHeight);

	/**
	 * First, write document variables
	 */
	QString subTitle;
	subTitle = reportContext.escape(project->getName())
			+ QString(" / ") + reportContext.escape(analysis->getName())
			+ QString(" / ") + reportContext.escape(regressionModel->getName())
			+ QString(" at ") + reportContext.escape(QDateTime::currentDateTime().toString());

	reportContext << "---" << endl;
	reportContext << "lang: english" << endl;
	reportContext << "documentclass: scrartcl" << endl;
	reportContext << "title: " << "DamBASE Report" << endl;
	reportContext << "author: " << subTitle << endl;
	reportContext << "---" << endl;
	reportContext << endl;

	BOOST_FOREACH(boost::shared_ptr<api::IReport::ReportListElement_t> listElement, reportListElements) {
		if(listElement->isSelected()) {
			listElement->build(reportContext);
		}
	}


	file.close();

	qDebug() << "... done";
}


bool
Report::write() {
	qDebug() << "write report to " << outputFileName;

	QString dataDir = QApplication::applicationDirPath() + "/reporting/pandoc/datadir";

	QStringList pandocArguments;
	pandocArguments.push_back("-s");
	pandocArguments.push_back("report.md");
	pandocArguments.push_back("--toc");
	pandocArguments.push_back("--data-dir=" + dataDir);
	pandocArguments.push_back("-V");
	pandocArguments.push_back("fontsize=" + QString::number(fontSize) + "pt");
	pandocArguments.push_back("-V");
	pandocArguments.push_back("geometry=textwidth=17cm,textheight=25cm");
	pandocArguments.push_back("--reference-docx");
	pandocArguments.push_back(dataDir + "/templates/reference.docx");
	pandocArguments.push_back("-o");
	pandocArguments.push_back(outputFileName);

	writeProcess.setWorkingDirectory(workingDirectory);

	writeProcess.start("pandoc.exe", pandocArguments);

	return writeProcess.waitForFinished(60000);
}

boost::shared_ptr<api::IReport::TitleReportListElement_t>
Report::getTitleReportListElement() {
	return titleListElement;
}

boost::shared_ptr<api::IReport::DescriptionReportListElement_t>
Report::getDescriptionReportListElement() {
	return descriptionListElement;
}

bool
Report::isDescriptionReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element) {
	return element == descriptionListElement;
}

boost::shared_ptr<api::IReport::ProjectReportListElement_t>
Report::getProjectReportListElement() {
	return projectReportListElement;
}

bool
Report::isProjectReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element) {
	return element == projectReportListElement;
}

api::IReport::ReportListElementContainer_t &
Report::getReportListElements() {
	return reportListElements;
}

bool
Report::isAnalysisToolListElement(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) {
	BOOST_FOREACH(boost::shared_ptr<api::IReport::AnalysisToolReportListElement_t> analysisToolListElement, analysisToolListElements) {
		if(reportListElement == analysisToolListElement) {
			return true;
		}
	}

	return false;
}

boost::shared_ptr<api::IReport::AnalysisToolReportListElement_t>
Report::getAnalysisToolListElementFor(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) {
	BOOST_FOREACH(boost::shared_ptr<AnalysisToolReportListElement_t> analysisToolListElement, analysisToolListElements) {
		if(reportListElement == analysisToolListElement) {
			return analysisToolListElement;
		}
	}

	boost::shared_ptr<api::IReport::AnalysisToolReportListElement_t> nullReturn;
	return nullReturn;
}

QString
Report::getSupportedFileTypesAsString() {
	return supportedFileTypes;
}

void
Report::moveUp(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) {
	int fromIndex = findIndexOf(reportListElement);
	if(fromIndex <= 0) {
		return; // we cannot move up
	}

	int toIndex = fromIndex - 1;
	exchange(fromIndex, toIndex);
	api::IReport::fireMoveReportListElement(fromIndex, toIndex);
}

void
Report::moveDown(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) {
	int fromIndex = findIndexOf(reportListElement);
	if(fromIndex >= reportListElements.size() || fromIndex == -1) {
		return; // we cannot move down
	}

	int toIndex = fromIndex + 1;
	exchange(fromIndex, toIndex);
	api::IReport::fireMoveReportListElement(fromIndex, toIndex);
}

int
Report::findIndexOf(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) {
	typedef api::IReport::ReportListElementContainer_t::iterator it_t;
	it_t it = std::find(reportListElements.begin(), reportListElements.end(), reportListElement);
	if(it == reportListElements.end()) {
		return -1;
	}

	return std::distance(reportListElements.begin(), it);
}

void
Report::exchange(int elementIndex, int withElementIndex) {
	boost::shared_ptr<api::IReport::ReportListElement_t> element = reportListElements.at(elementIndex);
	boost::shared_ptr<api::IReport::ReportListElement_t> with = reportListElements.at(withElementIndex);

	reportListElements.at(elementIndex) = with;
	reportListElements.at(withElementIndex) = element;
}

std::vector<double>
Report::getAvailablePlotScaleFactors() {
	return availablePlotScaleFactors;
}

double
Report::getPlotScaleFactor() {
	return plotScaleFactor;
}

void
Report::setPlotScaleFactor(double plotSize) {
	this->plotScaleFactor = plotSize;
}

std::vector<int>
Report::getAvailableFontSizes() {
	return availableFontSizes;
}

int
Report::getFontSize() {
	return fontSize;
}

void
Report::setFontSize(int fontSize) {
	this->fontSize = fontSize;
}

void
Report::writeSuccessful() {
	WriteSuccessfulSignal();
}

void
Report::writeTimedOut() {
	WriteTimedOutSignal(60000);
}

void
Report::writeFailed() {
	WriteFailedSignal();
}

bool
Report::hasFailed() {
	return writeProcess.exitCode() != 0;
}

void Report::setTitleReportListElement(boost::shared_ptr<api::IReport::TitleReportListElement_t> titleReportListElement) {
	this->titleListElement = titleReportListElement;
}

void Report::setDescriptionReportListElement(boost::shared_ptr<api::IReport::DescriptionReportListElement_t> descriptionReportListElement) {
	this->descriptionListElement = descriptionReportListElement;
}

void Report::setProjectReportListElement(boost::shared_ptr<api::IReport::ProjectReportListElement_t> projectReportListElement) {
	this->projectReportListElement = projectReportListElement;
}

boost::shared_ptr<api::IReport::AlgorithmReportListElement_t>
Report::getAlgorithmReportListElement() {
	return algorithmReportListElement;
}

void Report::setAlgorithmReportListElement(boost::shared_ptr<AlgorithmReportListElement_t> algorithmReportListElement) {
	this->algorithmReportListElement = algorithmReportListElement;
}

bool Report::isAlgorithmReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element) {
	return element == algorithmReportListElement;
}

void Report::setReportListElements(api::IReport::ReportListElementContainer_t& reportListElements) {
	this->reportListElements = reportListElements;
}

void Report::setAnalysisToolReportListElements(api::IReport::AnalysisToolListElementContainer_t& analysisToolListElements) {
	this->analysisToolListElements = analysisToolListElements;
}

QString
Report::getFailMessage() {
	return writeProcess.readAllStandardError();
}

} //namespace model
} //namespace dambase

