/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IDataModel.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IDATAMODEL_HPP_
#define IDATAMODEL_HPP_

#include <QUrl>
#include <QString>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <map>
#include "model/DataModelActionRules.hpp"
#include "model/SignalPolicies.hpp"
#include "r/api/IDataFrameProvider.hpp"
#include "r/api/IDataFrame.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {

class DataModelSignals : public communication::BoostSignalPolicy {
public:
	DataModelSignals(){}
	virtual ~DataModelSignals(){}

	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(DataModelLoaded, communication::BoostSignalPolicy::changed_signal_t)
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(DateColumnChanged, communication::BoostSignalPolicy::changed_signal_t)
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(BehaviorIndicatorAssignmentChanged, communication::BoostSignalPolicy::changed_signal_t)
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(WaterLevelAssignmentChanged, communication::BoostSignalPolicy::changed_signal_t)
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(TemperatureAssignmentChanged, communication::BoostSignalPolicy::changed_signal_t)


	void fireDataModelLoaded() {
		fire();
		DataModelLoadedSignal();
	}

	void fireDateColumnChanged() {
		fire();
		DateColumnChangedSignal();
	}

	void fireBehaviorIndicatorColumnAssignmentChanged() {
		fire();
		BehaviorIndicatorAssignmentChangedSignal();
	}

	void fireWaterLevelColumnAssignmentChanged() {
		fire();
		WaterLevelAssignmentChangedSignal();
	}

	void fireTemperatureColumnAssignmentChanged() {
		fire();
		TemperatureAssignmentChangedSignal();
	}

	void fire() {
		communication::BoostSignalPolicy::fire();
	}
};

class IDataModel : public DataModelSignals, model::DataModelActionRules  {
public:
	typedef QUrl source_uri_t;
	typedef r::api::IDataFrame DataFrame_t;
	typedef DataFrame_t::DateColumn_t DateColumn_t;
	typedef DateColumn_t::Date_Data_Point_t Date_Data_Point_t;
	typedef DataFrame_t::Behavior_Indicator_Column_t Behavior_Indicator_Column_t;
	typedef DataFrame_t::Water_Level_Column_t Water_Level_Column_t;
	typedef DataFrame_t::Temperature_Column_t Temperature_Column_t;
	typedef DataFrame_t::Behavior_Indicator_Column_Container_t Behavior_Indicator_Column_Container_t;
	typedef DataFrame_t::Water_Level_Column_Container_t Water_Level_Column_Container_t;
	typedef DataFrame_t::Temperature_Column_Container_t Temperature_Column_Container_t;
	typedef DataFrame_t::Header_t Header_t;
	typedef DataFrame_t::Column_t Column_t;
	typedef DataFrame_t::Header_Container_t Header_Container_t;
	typedef DataFrame_t::Assignable_Header_Container_t Assignable_Header_Container_t;
	typedef DataFrame_t::Influence_Factor_t Influence_Factor_t;
	typedef r::api::IDataFrameProvider DataFrameProvider_t;
	typedef IAnalysisTool AnalysisTool_t;
public:
	IDataModel(){;}
	virtual ~IDataModel(){;}

	void setAvailableCSVDelimiters(std::map<QString, QString> availableCSVDelimiters);

	virtual const std::map<QString, QString> &getAvailableCSVDelimiters() const = 0;

	virtual const QString &getCSVDelimiter() const = 0;

	virtual void setCSVDelimiter(QString csvDelimiter) = 0;

	virtual const source_uri_t &getSourceUri() const = 0;

	virtual void setSourceUri(const source_uri_t &sourceUri) = 0;

	virtual void setDataFrameProvider(DataFrameProvider_t *dataFrameProvider) = 0;

	virtual void loadDataFrame() = 0;

	/**
	 * This is used by loadDataFrame as well as the deserialization process
	 */
	virtual void setDataFrame(boost::shared_ptr<DataFrame_t> newDataFrame) = 0;

	virtual boost::shared_ptr<DateColumn_t> getDateColumn() = 0;

	virtual boost::shared_ptr<DateColumn_t> getDateColumn(QString dateFormat)= 0;

	virtual void setDateColumn(int index) = 0;

	virtual void estimateDateColumn() = 0;

	virtual bool isColumnAssignedWithIndex(int index) = 0;

	virtual bool assignBehaviorIndicatorColumn(int index) = 0;

	virtual bool removeBehaviorIndicatorColumn(int index) = 0;

	virtual bool isBehaviorIndicatorColumnAssignedFor(int index) = 0;

	virtual bool assignWaterLevelColumn(int index) = 0;

	virtual bool removeWaterLevelColumn(int index) = 0;

	virtual bool isWaterLevelColumnAssignedFor(int index) = 0;

	virtual bool assignTemperatureColumn(int index) = 0;

	virtual bool removeTemperatureColumn(int index) = 0;

	virtual bool isTemperatureColumnAssignedFor(int index) = 0;

	virtual Header_Container_t& getHeaders() = 0;

	virtual boost::shared_ptr<Column_t> getColumnByIndex(int index) = 0;

	virtual Assignable_Header_Container_t getAssignableDateColumnHeaders() = 0;

	virtual Assignable_Header_Container_t getAssignableBehaviorIndicatorHeaders() = 0;

	virtual Assignable_Header_Container_t getAssignableWaterLevelHeaders() = 0;

	virtual Assignable_Header_Container_t getAssignableTemperatureHeaders() = 0;

	virtual const Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns() = 0;

	virtual boost::shared_ptr<Behavior_Indicator_Column_t> getBehaviorIndicatorColumnWith(int index) = 0;

	virtual const Water_Level_Column_Container_t& getWaterLevelColumns() = 0;

	virtual const Temperature_Column_Container_t& getTemperatureColumns() = 0;

	virtual boost::shared_ptr<Column_t> getAnyAssignedColumn() = 0;

	virtual const boost::shared_ptr<DataFrame_t> &getDataFrame() const = 0;

	virtual void applyDateFormat(QString dateFormat) = 0;

	virtual bool isDateColumnEditable() = 0;

	virtual bool isDateColumnAssignmentComplete() = 0;

	virtual const QString &getDateFormat() const = 0;

	virtual void setDateFormat(QString dateFormat) = 0;

	virtual void addDataModelAnalysis(boost::shared_ptr<AnalysisTool_t> dataModelAnalysis) = 0;

	virtual void removeDataModelAnalysis(boost::shared_ptr<AnalysisTool_t> dataModelAnalysis) = 0;
};

} //namespace api
} //namespace model
} //namespace



#endif /* IDATAMODEL_HPP_ */
