/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionModel.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONMODEL_HPP_
#define IREGRESSIONMODEL_HPP_

#include <boost/shared_ptr.hpp>
#include <vector>
#include "model/SignalPolicies.hpp"
#include "IDataModel.hpp"
#include "r/api/ITimeRange.hpp"
#include "IAnalysisTool.hpp"
#include "r/api/IRegressionFormula.hpp"
#include "r/api/IRegressionFit.hpp"
#include "r/api/IRegressionFitProvider.hpp"

namespace dambase {
namespace model {
namespace api {

namespace detail {

class RegressionModelSignalPolicy : public communication::BoostSignalPolicy {
public:
	COM_SUBSCRIPTION_FUNCTION(Calculated, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(CommonParametersChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(AlgorithmChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(AvailableRegressionTimeWindowChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(TimeValuesChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(PCRrValueChanged, communication::Signal<void()>);
public:
	void fireCalculated() {
		communication::BoostSignalPolicy::fire();
		CalculatedSignal();
	}

	void fireNameChanged() {
		communication::BoostSignalPolicy::fire();
		CommonParametersChangedSignal();
	}

	void fireDescriptionChanged() {
		communication::BoostSignalPolicy::fire();
		CommonParametersChangedSignal();
	}

	void fireBehaviorIndicatorChanged() {
		communication::BoostSignalPolicy::fire();
		BehaviorIndicatorChangedSignal();
	}

	void fireAlgorithmChanged() {
		communication::BoostSignalPolicy::fire();
		AlgorithmChangedSignal();
	}

	void fireFromDateChanged() {
		communication::BoostSignalPolicy::fire();
		TimeValuesChangedSignal();
	}

	void fireToDateChanged() {
		communication::BoostSignalPolicy::fire();
		TimeValuesChangedSignal();
	}

	void fireAvailableRegressionTimeWindowChanged() {
		communication::BoostSignalPolicy::fire();
		AvailableRegressionTimeWindowChangedSignal();
	}

	void firePCRrValueChanged() {
		communication::BoostSignalPolicy::fire();
		PCRrValueChangedSignal();
	}
};

}  // namespace detail

class IRegressionModel : public detail::RegressionModelSignalPolicy {
public:
	typedef IDataModel DataModel_t;
	typedef r::api::ITimeRange TimeRange_t;
	typedef IAnalysisTool AnalysisTool_t;
	typedef std::vector<boost::shared_ptr<AnalysisTool_t> > AnalysisToolsContainer_t;
	typedef r::api::IRegressionFormula Regression_Formula_t;
	typedef r::api::IRegressionFit RegressionFit_t;
	typedef r::api::IRegressionFitProvider RegressionFitProvider_t;
	typedef IDataModel::Behavior_Indicator_Column_t Behavior_Indicator_Column_t;

public:
	IRegressionModel(){;}
	virtual ~IRegressionModel(){;}

	enum Algorithm {
		Standard,
		PraisWinsten,
		Ridge,
		Pcr
	};

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setDataModel(boost::shared_ptr<DataModel_t> dataModel) = 0;

	/**
	 * This is primarly used by the serialization process
	 */
	virtual const boost::shared_ptr<TimeRange_t> &getTimeRange() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setTimeRange(boost::shared_ptr<TimeRange_t> timeRange) = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setRegressionFormula(boost::shared_ptr<Regression_Formula_t> regressionFormula) = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setRegressionFitProvider(RegressionFitProvider_t *regressionFitProvider) = 0;

	virtual const QString &getDescription() const = 0;

	virtual void setDescription(QString& description) = 0;

	virtual const QString &getName() const = 0;

	virtual void setName(QString& name) = 0;

	virtual void setFromDate(int index, bool fireSignal = true) = 0;

	virtual void setFromDate(DataModel_t::Date_Data_Point_t &fromDate, bool fireSignal = true) = 0;

	virtual void setToDate(int index) = 0;

	/**
	 * The date within the availableRegressionTimeWindow the user can choose from to set the beginning of the regression
	 */
	virtual const DataModel_t::Date_Data_Point_t &getFromDate() const = 0;

	/**
	 * The latest date within the availableRegressionTimeWindow the user can choose from to set the end of the regression calibration date
	 */
	virtual const DataModel_t::Date_Data_Point_t &getToDate() const = 0;

	/**
	 * Returns a list of dates from which the user can choose from. Considers startup times of the formula
	 */
	virtual std::vector<DataModel_t::Date_Data_Point_t> getAvailableRegressionTimeWindow() = 0;

	/**
	 * This is primarly used by the deserialization process. For other purposes, use #updateAvailableRegressionTimeWindow
	 */
	virtual void restoreAvailableRegressionTimeWindow() = 0;

	/**
	 * The aim of this method is to provide an access point which allows updating values (such as available regression time window, maximum pcr r value ...)
	 * from the controller.
	 */
	virtual void updateFromFormulaChanges() = 0;

	virtual void updateAvailableRegressionTimeWindow() = 0;

	virtual void setBehaviorIndicatorColumn(int behaviorIndicatorColumnIndex) = 0;

	virtual boost::shared_ptr<DataModel_t::Behavior_Indicator_Column_t> getBehaviorIndicatorColumn() = 0;

	virtual void setupBestFittingRegressionFormula() = 0;

	virtual const boost::shared_ptr<Regression_Formula_t> &getRegressionFormula() const = 0;

	virtual void calculateRegressionFit() = 0;

	virtual boost::shared_ptr<RegressionFit_t> getRegressionFit() = 0;

	virtual const boost::shared_ptr<DataModel_t> &getDataModel() const = 0;

	//TODO: rename after refactoring has finished
	virtual void addRegressionModelAnalysis(boost::shared_ptr<AnalysisTool_t> analysis) = 0;

	virtual AnalysisToolsContainer_t &getAnalysisTools() = 0;

	//TODO: rename after refactoring has finished
	virtual void removeRegressionModelAnalysis(boost::shared_ptr<AnalysisTool_t> analysis) = 0;

	virtual bool isStandardAlgorithm() = 0;

	virtual bool isPraisWinstenAlgorithm() = 0;

	virtual bool isRidgeAlgorithm() = 0;

	virtual bool isPCRAlgorithm() = 0;

	virtual const Algorithm &getAlgorithm() const = 0;

	virtual const int getAlgorithmAsInt() const = 0;

	virtual void setAlgorithm(int algorithm) = 0;

	virtual double getPraisWinstenRho() = 0;

	virtual const unsigned &getPraisWinstenIterations() const = 0;

	virtual void setPraisWinstenIterations(unsigned praisWinstenIterations) = 0;

	virtual const double &getRidgeKValue() const = 0;

	virtual void setRidgeKValue(double k) = 0;

	virtual unsigned getMaxPCRrValue() = 0;

	virtual const unsigned &getPCRrValue() const = 0;

	virtual void setPCRrValue(unsigned r) = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IREGRESSIONMODEL_HPP_ */
