/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrame.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */


#include <Rcpp.h>
#include <boost/foreach.hpp>
#include "DataFrame.hpp"

namespace dambase {
namespace model {
namespace r {

DataFrame::DataFrame(){;}

DataFrame::DataFrame(Rcpp::DataFrame &dataFrame)
	:dataFrame(dataFrame)
{
	calculateNonPointable();
}

DataFrame::~DataFrame(){;}

void
DataFrame::updateSomeRawValues() {
	//some values
	someRawValues.clear();
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function getHeadersFromDataFrame = damBaseEnv["getHeadersForDataFrame"];
	Rcpp::Function getSomeRawValuesFromDataFrame = damBaseEnv["getSomeRawValuesFromDataFrame"];

	Rcpp::CharacterVector rheaders = Rcpp::as<Rcpp::CharacterVector >(getHeadersFromDataFrame(
			Rcpp::Named("dataFrame", dataFrame))
			);

	BOOST_FOREACH(Rcpp::CharacterVector::value_type rheader, rheaders) {
		std::string rheaderStr = Rcpp::as<std::string>(rheader);
		Rcpp::CharacterVector someValues = Rcpp::as<Rcpp::CharacterVector >(getSomeRawValuesFromDataFrame(
				Rcpp::Named("dataFrame", dataFrame),
				Rcpp::Named("headerName", rheaderStr)
				));

		BOOST_FOREACH(Rcpp::CharacterVector::value_type rvalue, someValues) {
			Rcpp::String theRHeaderName = rheader;
			QString headerName = QString::fromStdString(theRHeaderName);

			Rcpp::String theRValue = rvalue;
			QString valueAsString = QString::fromStdString(theRValue);
			someRawValues[headerName].push_back(valueAsString);
		}
	}
}

void
DataFrame::updateFromApplyDateFormat(Rcpp::DataFrame dataFrame) {
	this->dataFrame = dataFrame;
	if(getDateColumn() != NULL) {
		setDateColumn(getDateColumn()->getIndex(), true);
	}
	updateSomeRawValues();
}

DataFrameHeader &
DataFrame::getHeader(int at) {
	return headers.at(at);
}

api::IDataFrame::Header_Container_t &
DataFrame::getHeaders() {
	return headers;
}

std::vector<QString>
DataFrame::getSomeRawValuesFor(QString headerName) {
	return someRawValues[headerName];
}

const std::map<QString, std::vector<QString> > &
DataFrame::getRawValues() const {
	return someRawValues;
}

boost::shared_ptr<api::IDataFrame::Column_t>
DataFrame::getColumnByIndex(int index) {
	std::vector<boost::shared_ptr<api::IDataFrame::Column_t> > allColumns;

	allColumns.insert(allColumns.begin(), waterLevelColumns.begin(), waterLevelColumns.end());
	allColumns.insert(allColumns.begin(), temperatureColumns.begin(), temperatureColumns.end());
	allColumns.insert(allColumns.begin(), behaviorIndicatorColumns.begin(), behaviorIndicatorColumns.end());
	if(dateColumn != NULL) {
		allColumns.push_back(dateColumn);
	}

	BOOST_FOREACH(boost::shared_ptr<api::IDataFrame::Column_t> column, allColumns) {
		if(column->getIndex() == index) {
			return column;
		}
	}

	boost::shared_ptr<api::IDataFrame::Column_t> nullResult;
	return nullResult;
}

const api::IDataFrame::Assignable_Header_Container_t
DataFrame::getAssignableDateColumnHeaders() {
	api::IDataFrame::Assignable_Header_Container_t alreadyAssignedHeaders;
	api::IDataFrame::Assignable_Header_Container_t assignableHeaders;

	BOOST_FOREACH(boost::shared_ptr<BehaviorIndicatorColumn> behaviorIndicatorColumn, behaviorIndicatorColumns) {
		alreadyAssignedHeaders.push_back(behaviorIndicatorColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<WaterLevelColumn> waterLevelColumn, waterLevelColumns) {
		alreadyAssignedHeaders.push_back(waterLevelColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<TemperatureColumn> temperatureColumn, temperatureColumns) {
		alreadyAssignedHeaders.push_back(temperatureColumn->getHeader());
	}

	std::sort(alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(), api::BeforeColumnComparatorForReferences<DataFrameHeader>());

	std::set_difference(
			headers.begin(), headers.end(),
			alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(),
			std::back_inserter<Assignable_Header_Container_t>(assignableHeaders),
			api::BeforeColumnComparatorForReferences<DataFrameHeader>()
			);

	return assignableHeaders;
}

const api::IDataFrame::Assignable_Header_Container_t
DataFrame::getAssignableBehaviorIndicatorHeaders() {
	api::IDataFrame::Assignable_Header_Container_t alreadyAssignedHeaders;
	api::IDataFrame::Assignable_Header_Container_t assignableHeaders;


	if(dateColumn != NULL) {
		alreadyAssignedHeaders.push_back(dateColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<WaterLevelColumn> waterLevelColumn, waterLevelColumns) {
		alreadyAssignedHeaders.push_back(waterLevelColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<TemperatureColumn> temperatureColumn, temperatureColumns) {
		alreadyAssignedHeaders.push_back(temperatureColumn->getHeader());
	}

	std::sort(alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(), api::BeforeColumnComparatorForReferences<DataFrameHeader>());

	std::set_difference(
			headers.begin(), headers.end(),
			alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(),
			std::back_inserter<Assignable_Header_Container_t>(assignableHeaders),
			api::BeforeColumnComparatorForReferences<DataFrameHeader>()
			);

	return assignableHeaders;
}

const api::IDataFrame::Assignable_Header_Container_t
DataFrame::getAssignableWaterLevelHeaders() {
	api::IDataFrame::Assignable_Header_Container_t alreadyAssignedHeaders;
	api::IDataFrame::Assignable_Header_Container_t assignableHeaders;


	if(dateColumn != NULL) {
		alreadyAssignedHeaders.push_back(dateColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<BehaviorIndicatorColumn> behaviorIndicatorColumn, behaviorIndicatorColumns) {
		alreadyAssignedHeaders.push_back(behaviorIndicatorColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<TemperatureColumn> temperatureColumn, temperatureColumns) {
		alreadyAssignedHeaders.push_back(temperatureColumn->getHeader());
	}

	std::sort(alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(), api::BeforeColumnComparatorForReferences<DataFrameHeader>());

	std::set_difference(
			headers.begin(), headers.end(),
			alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(),
			std::back_inserter<Assignable_Header_Container_t>(assignableHeaders),
			api::BeforeColumnComparatorForReferences<DataFrameHeader>()
			);

	return assignableHeaders;
}

const api::IDataFrame::Assignable_Header_Container_t
DataFrame::getAssignableTemperatureHeaders() {
	api::IDataFrame::Assignable_Header_Container_t alreadyAssignedHeaders;
	api::IDataFrame::Assignable_Header_Container_t assignableHeaders;


	if(dateColumn != NULL) {
		alreadyAssignedHeaders.push_back(dateColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<BehaviorIndicatorColumn> behaviorIndicatorColumn, behaviorIndicatorColumns) {
		alreadyAssignedHeaders.push_back(behaviorIndicatorColumn->getHeader());
	}

	BOOST_FOREACH(boost::shared_ptr<WaterLevelColumn> waterLevelColumn, waterLevelColumns) {
		alreadyAssignedHeaders.push_back(waterLevelColumn->getHeader());
	}

	std::sort(alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(), api::BeforeColumnComparatorForReferences<DataFrameHeader>());

	std::set_difference(
			headers.begin(), headers.end(),
			alreadyAssignedHeaders.begin(), alreadyAssignedHeaders.end(),
			std::back_inserter<Assignable_Header_Container_t>(assignableHeaders),
			api::BeforeColumnComparatorForReferences<DataFrameHeader>()
			);

	return assignableHeaders;
}

int
DataFrame::getNrOfRows() {
	return dataFrame.nrows();
}

int
DataFrame::getNrOfColumns() {
	return headers.size();
}

bool
DataFrame::setDateColumn(int fromIndex, bool forceUpdate) {
	if(dateColumn != NULL) {
		if(dateColumn->getIndex() == fromIndex && forceUpdate == false) {
			return false; //not set as it's the same
		}
		dateColumn = boost::shared_ptr<DateColumn>();
		//dateColumn.reset();
	}

	dateColumn = boost::shared_ptr<DateColumn>(new DateColumn(headers.at(fromIndex), Rcpp::as<Rcpp::CharacterVector>(dataFrame[fromIndex])));
	return true;
}

void
DataFrame::setDateColumn(boost::shared_ptr<DateColumn> dateColumn) {
	this->dateColumn = dateColumn;
}

const boost::shared_ptr<DateColumn> &
DataFrame::getDateColumn() const {
	return dateColumn;
}

boost::shared_ptr<DateColumn>
DataFrame::getDateColumn(Rcpp::DataFrame forDataFrame) {
	if(dateColumn == NULL) {
		boost::shared_ptr<DateColumn> nullReturn;
		return nullReturn;
	}

	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function getHeadersFromDataFrame = damBaseEnv["getHeadersForDataFrame"];

	Rcpp::CharacterVector rheaders = Rcpp::as<Rcpp::CharacterVector >(getHeadersFromDataFrame(
			Rcpp::Named("dataFrame", dataFrame))
			);

	int index = 0;
	std::vector<DataFrameHeader> tempHeaders;
	BOOST_FOREACH(Rcpp::CharacterVector::value_type rheader, rheaders) {
		tempHeaders.push_back(model::r::DataFrameHeader(rheader, index++));
	}


	int fromIndex = dateColumn->getIndex();


	boost::shared_ptr<DateColumn> result(new DateColumn(tempHeaders.at(fromIndex), Rcpp::as<Rcpp::CharacterVector>(forDataFrame[fromIndex])));
	return result;
}

int
DataFrame::estimateDateColumn() {
	if(headers.size() == 0) {
		return -1;
	}else{
		return 0; //TODO: use an R algorithm for real estimation of the date column or try and error with boost date on each column and first value
	}
}

bool
DataFrame::assignBehaviorIndicatorColumn(int fromIndex) {
	boost::shared_ptr<BehaviorIndicatorColumn> behaviorIndicatorColumn(new BehaviorIndicatorColumn(headers.at(fromIndex)));

	if(behaviorIndicatorColumns.insert(behaviorIndicatorColumn).second == false) {
		return false;
	}

	return true;
}

bool
DataFrame::isBehaviorIndicatorColumnAssignedFor(int index) {
	BOOST_FOREACH(boost::shared_ptr<BehaviorIndicatorColumn> behaviorIndicatorColumn, behaviorIndicatorColumns) {
		if(behaviorIndicatorColumn->getIndex() == index) {
			return true;
		}
	}

	return false;
}

bool
DataFrame::removeBehaviorIndicatorColumn(int fromIndex) {

	BOOST_FOREACH(boost::shared_ptr<BehaviorIndicatorColumn> column, behaviorIndicatorColumns) {
		if(column->getHeader().getIndex() == fromIndex) {
			behaviorIndicatorColumns.erase(column);
			return true;
		}
	}

	return false;
}

const api::IDataFrame::Behavior_Indicator_Column_Container_t &
DataFrame::getBehaviorIndicatorColumns() const {
	return behaviorIndicatorColumns;
}

void
DataFrame::setBehaviorIndicatorColumns(api::IDataFrame::Behavior_Indicator_Column_Container_t &columns) {
	this->behaviorIndicatorColumns = columns;
}

bool
DataFrame::assignWaterLevelColumn(int fromIndex) {
	boost::shared_ptr<WaterLevelColumn> waterLevelColumn(new WaterLevelColumn(headers.at(fromIndex)));

	if(waterLevelColumns.insert(waterLevelColumn).second == false) {
		return false;
	}

	return true;
}

bool
DataFrame::isWaterLevelColumnAssignedFor(int index) {
	BOOST_FOREACH(boost::shared_ptr<WaterLevelColumn> waterLevelColumn, waterLevelColumns) {
		if(waterLevelColumn->getIndex() == index) {
			return true;
		}
	}

	return false;
}


bool
DataFrame::removeWaterLevelColumn(int fromIndex) {
	BOOST_FOREACH(boost::shared_ptr<WaterLevelColumn> column, waterLevelColumns) {
		if(column->getHeader().getIndex() == fromIndex) {
			waterLevelColumns.erase(column);
			return true;
		}
	}

	return false;
}

boost::shared_ptr<WaterLevelColumn>
DataFrame::findWaterLevelColumnByName(QString name) {
	BOOST_FOREACH(boost::shared_ptr<WaterLevelColumn> waterLevelColumn, getWaterLevelColumns()) {
		if(waterLevelColumn->getName() == name) {
			return waterLevelColumn;
		}
	}

	boost::shared_ptr<WaterLevelColumn> nullReturn;
	return nullReturn;
}

const api::IDataFrame::Water_Level_Column_Container_t &
DataFrame::getWaterLevelColumns() const {
	return waterLevelColumns;
}

void
DataFrame::setWaterLevelColumns(api::IDataFrame::Water_Level_Column_Container_t &columns) {
	this->waterLevelColumns = columns;
}


bool
DataFrame::assignTemperatureColumn(int fromIndex) {
	boost::shared_ptr<TemperatureColumn> temperatureColumn(new TemperatureColumn(headers.at(fromIndex)));

	if(temperatureColumns.insert(temperatureColumn).second == false) {
		return false;
	}

	return true;
}

bool
DataFrame::isTemperatureColumnAssignedFor(int index) {
	BOOST_FOREACH(boost::shared_ptr<TemperatureColumn> temperatureColumn, temperatureColumns) {
		if(temperatureColumn->getIndex() == index) {
			return true;
		}
	}

	return false;
}


bool
DataFrame::removeTemperatureColumn(int fromIndex) {
	using namespace app::context;

	BOOST_FOREACH(boost::shared_ptr<TemperatureColumn> column, temperatureColumns) {
		if(column->getHeader().getIndex() == fromIndex) {
			temperatureColumns.erase(column);
			return true;
		}
	}

	return false;
}

const api::IDataFrame::Temperature_Column_Container_t &
DataFrame::getTemperatureColumns() const {
	return temperatureColumns;
}

void
DataFrame::setTemperatureColumns(api::IDataFrame::Temperature_Column_Container_t &columns) {
	this->temperatureColumns = columns;
}

api::IDataFrame::Influence_Factor_Container_t
DataFrame::getAssignedInfluenceFactorsFor(boost::shared_ptr<api::IAssignmentCategory> assignmentCategory) {
	api::IDataFrame::Influence_Factor_Container_t influenceFactors;

	if(assignmentCategory->getIdentifier() == "water_level"
			|| assignmentCategory->getIdentifier() == "creep"
			) {
		influenceFactors.insert(influenceFactors.begin(), waterLevelColumns.begin(), waterLevelColumns.end());
	}else if(assignmentCategory->getIdentifier() == "temperature") {
		influenceFactors.insert(influenceFactors.begin(), temperatureColumns.begin(), temperatureColumns.end());
	}else if(assignmentCategory->getIdentifier() == "any"
			|| assignmentCategory->getIdentifier() == "average"
			|| assignmentCategory->getIdentifier() == "delay"
			) {
		influenceFactors.insert(influenceFactors.begin(), waterLevelColumns.begin(), waterLevelColumns.end());
		influenceFactors.insert(influenceFactors.begin(), temperatureColumns.begin(), temperatureColumns.end());
	}

	return influenceFactors;
}

void
DataFrame::setDataFrameFromDeserialization(Rcpp::DataFrame dataFrame) {
	this->dataFrame = dataFrame;
	calculateNonPointable();
}

void
DataFrame::calculateNonPointable(){
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function getHeadersFromDataFrame = damBaseEnv["getHeadersForDataFrame"];
	Rcpp::Function getSomeRawValuesFromDataFrame = damBaseEnv["getSomeRawValuesFromDataFrame"];

	Rcpp::CharacterVector rheaders = Rcpp::as<Rcpp::CharacterVector >(getHeadersFromDataFrame(
			Rcpp::Named("dataFrame", dataFrame))
			);

		int index = 0;
		BOOST_FOREACH(Rcpp::CharacterVector::value_type rheader, rheaders) {
			headers.push_back(model::r::DataFrameHeader(rheader, index++));

		}

	updateSomeRawValues();
}

Rcpp::DataFrame &
DataFrame::getDataFrame() {
	return dataFrame;
}

const Rcpp::DataFrame &
DataFrame::getDataFrameConst() const {
	return dataFrame;
}

} //namespace r
} //namespace model
} //namespace dambase


