/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ExpressionRegressorFormulaBuilder.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <QString>
#include <boost/algorithm/string/join.hpp>
#include <vector>
#include <map>
#include <algorithm>
#include <stdexcept>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/shared_ptr.hpp>
#include "ExpressionRegressorFormulaBuilder.hpp"

namespace dambase {
namespace model {
namespace r {


ExpressionRegressorFormulaBuilder::ExpressionRegressorFormulaBuilder()
		:iseditable(false)
	{;}

ExpressionRegressorFormulaBuilder::ExpressionRegressorFormulaBuilder(QString identifier, std::vector<boost::shared_ptr<api::IAssignmentCategory> > assignmentCategories, boost::shared_ptr<api::IAssignmentCategory> assignmentCategory, std::vector<boost::shared_ptr<api::IRegressorFormulaBuilderPreprocessor> > preprocessors, bool iseditable)
		:identifier(identifier), assignmentCategories(assignmentCategories), assignmentCategory(assignmentCategory), preprocessors(preprocessors), iseditable(iseditable)
	{
	}

ExpressionRegressorFormulaBuilder::ExpressionRegressorFormulaBuilder(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder> original, QString newIdentifier, QString newFormula, bool iseditable) {
	this->identifier = newIdentifier;
	this->assignmentCategories = original->getAssignmentCategories();
	this->assignmentCategory = original->getAssignmentCategory();
	this->formula = newFormula;
	this->formulaDefinition = original->getFormulaDefinition();
	this->iseditable = iseditable;
}

ExpressionRegressorFormulaBuilder::~ExpressionRegressorFormulaBuilder(){;}

const QString &
ExpressionRegressorFormulaBuilder::getIdentifier() const {
	return identifier;
}

void
ExpressionRegressorFormulaBuilder::setIdentifier(QString identifier) {
	this->identifier = identifier;
	api::IExpressionRegressorFormulaBuilder::firePrimaryChange();
}

QString
ExpressionRegressorFormulaBuilder::resolveExtendedIdentifier() {
	QString extendedIdentifier = getIdentifier();

	if(getIdentifier() != getFormula()) {
		extendedIdentifier += QString(" [ ") + getFormula() + QString(" ]");
	}

	return extendedIdentifier;
}

QString
ExpressionRegressorFormulaBuilder::resolveName(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::InfluenceFactor_t> influenceFactor) {
	//use formula and replace '$' by the influenceFactor's header name
	//up to now, we just support one influenceFactor

	using namespace std;
	using namespace boost;

	string resolvedName = getFormula().toStdString();
	replace_all(resolvedName, "$", influenceFactor->getName().toStdString());

	return QString::fromStdString(resolvedName);
}

QString
ExpressionRegressorFormulaBuilder::resolveName() {
	return getFormula();
}

void
ExpressionRegressorFormulaBuilder::setFormula(QString formula) {
	this->formula = formula;
	api::IExpressionRegressorFormulaBuilder::firePrimaryChange();
}

const QString &
ExpressionRegressorFormulaBuilder::getFormula() const {
	return formula;
}

void
ExpressionRegressorFormulaBuilder::setFormulaDefinition(QString formulaDefinition) {
	this->formulaDefinition = formulaDefinition;
}

const QString &
ExpressionRegressorFormulaBuilder::getFormulaDefinition() const {
	return formulaDefinition;
}

QString
ExpressionRegressorFormulaBuilder::resolveFormulaDefinition(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::DataFrame_t> dataFrame, int overallMaxStartupTime) {
	using namespace std;
	using namespace boost;

	return QString("I(evaluateExpansion(")
						+ QString("\"{") + getEnhancedFormulaDefinition() + QString("}\"")
						+ QString(", startDiff = ") + QString::number(calculateStartDiffFromBeginningOfTheRegressionFrame(overallMaxStartupTime))
						+ QString(", finalCutDiff = ") + QString::number(calculateFinalCutDiff(overallMaxStartupTime))
						+ QString(", dateColumn = ") + dataFrame->getDateColumn()->getName()
						+ QString(", userargs = ") + getEnhancedUserParameters()
						+ QString(")") + QString(")");
}


/**
 * overallMaxStartupTime is the maximum startup time of all involved regressors
 * Therefore, the dateColumn has to start at the date where the overallMaxStartupTime added points to the startup time of the regression
 * So we calculate here a difference which tells us at what date we can start and provide this difference to the evaluateExpansion function
 */
QString
ExpressionRegressorFormulaBuilder::resolveFormulaDefinition(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::DataFrame_t> dataFrame, boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::InfluenceFactor_t> influenceFactor, int overallMaxStartupTime) {
	using namespace std;
	using namespace boost;

	QString preprocessorsDefinition;
	if(hasActivePreprocessors()) {
		preprocessorsDefinition = ", preprocessors = " + resolvePreprocessorsDefinition();
	}

	return QString("I(evaluateExpansion(")
						+ QString("\"{") + getEnhancedFormulaDefinition() + QString("}\"")
						+ QString(", startDiff = ") + QString::number(calculateStartDiffFromBeginningOfTheRegressionFrame(overallMaxStartupTime))
						+ QString(", finalCutDiff = ") + QString::number(calculateFinalCutDiff(overallMaxStartupTime))
						+ QString(", dateColumn = ") + dataFrame->getDateColumn()->getName()
						+ QString(", column = ") + influenceFactor->getName()
						+ QString(", userargs = ") + getEnhancedUserParameters()
						+ preprocessorsDefinition
						+ QString(")") + QString(")");
}

bool
ExpressionRegressorFormulaBuilder::hasPreprocessors() {
	return preprocessors.empty();
}

bool
ExpressionRegressorFormulaBuilder::requiresInfluenceFactor() {
	return assignmentCategory->hasColumns();
}

boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t>
ExpressionRegressorFormulaBuilder::findAssignmentCategoryByIdentifier(QString identifier) {
	BOOST_FOREACH(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t>assignmentCategory, assignmentCategories) {
		if(assignmentCategory->getIdentifier() == identifier) {
			return assignmentCategory;
		}
	}

	boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t> nullReturn;
	return nullReturn;
}

/**
 * This is primarly used by the deserialization process.
 */
void
ExpressionRegressorFormulaBuilder::setAssignmentCategories(std::vector<boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t> > &assignmentCategories) {
	this->assignmentCategories = assignmentCategories;
}

std::vector<boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t> > &
ExpressionRegressorFormulaBuilder::getAssignmentCategories() {
	return assignmentCategories;
}

void
ExpressionRegressorFormulaBuilder::setAssignmentCategory(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t> assignmentCategory) {
	this->assignmentCategory = assignmentCategory;
}

boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::AssignmentCategory_t>
ExpressionRegressorFormulaBuilder::getAssignmentCategory() const {
	return assignmentCategory;
}

api::IExpressionRegressorFormulaBuilder::DataFrame_t::Influence_Factor_Container_t
ExpressionRegressorFormulaBuilder::getAvailableInfluenceFactors(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::DataFrame_t> dataFrame) {
	if(dataFrame == NULL) {
		throw std::logic_error("ExpressionRegressorFormulaBuilder::getAvailable... dataFrame == NULL");
	}
	return dataFrame->getAssignedInfluenceFactorsFor(assignmentCategory);
}

/**
 * This is primarly used by the deserialization process
 */
void
ExpressionRegressorFormulaBuilder::setIsEditable(bool iseditable) {
	this->iseditable = iseditable;
}

const bool &
ExpressionRegressorFormulaBuilder::isEditable() const {
	return iseditable;
}

/**
 * This is primarly used by the deserialization process.
 */
void
ExpressionRegressorFormulaBuilder::setUserParameters(api::IExpressionRegressorFormulaBuilder::UserParameters_t &userParameters) {
	this->userParameters = userParameters;
}

void
ExpressionRegressorFormulaBuilder::addUserParameter(QString name, boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::UserParameterType_t> userParameterType) {
	userParameters[name] = userParameterType;
	api::IExpressionRegressorFormulaBuilder::fireUserParameterAdded(std::pair<QString, boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::UserParameterType_t> >(name, userParameterType));
}

const std::map<QString, boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::UserParameterType_t> > &
ExpressionRegressorFormulaBuilder::getUserParameters() const {
	return userParameters;
}

void
ExpressionRegressorFormulaBuilder::removeUserParameter(api::IExpressionRegressorFormulaBuilder::UserParameter_t userParameter) {
	api::IExpressionRegressorFormulaBuilder::fireRemoveUserParameter(userParameter);
	userParameters.erase(userParameter.first);
}

int
ExpressionRegressorFormulaBuilder::calculateStartupTimesteps() {
	std::vector<int> minimumLeafTimes;
	minimumLeafTimes.push_back(0); //make sure that in case of having no minimumLeafTime set on any user parameter, we have at least 0 as the maximum startupTimestep

	BOOST_FOREACH(UserParameter_t userParameter, userParameters) {
		if(userParameter.second->isMinimumLeadTime()) {
			minimumLeafTimes.push_back(userParameter.second->getValue().toInt());
		}
	}

	return *std::max_element(minimumLeafTimes.begin(), minimumLeafTimes.end());
}

const std::vector<boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::RegressorFormulaBuilderPreprocessor_t> >&
ExpressionRegressorFormulaBuilder::getPreprocessors() const {
	return preprocessors;
}

/**
 * This is primarly used by the deserialization process as well as the clone process
 */
void
ExpressionRegressorFormulaBuilder::setPreprocessors(std::vector<boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::RegressorFormulaBuilderPreprocessor_t> >& preprocessors) {
	this->preprocessors = preprocessors;
}

int
ExpressionRegressorFormulaBuilder::calculateStartDiffFromBeginningOfTheRegressionFrame(int overallMaxStartupTime) {
	return overallMaxStartupTime - calculateStartupTimesteps();
}

int
ExpressionRegressorFormulaBuilder::calculateFinalCutDiff(int overallMaxStartupTime, int startDiffFromBeginningOfTheRegressionFrame) {
	return overallMaxStartupTime - startDiffFromBeginningOfTheRegressionFrame;
}

int
ExpressionRegressorFormulaBuilder::calculateFinalCutDiff(int overallMaxStartupTime) {
	return overallMaxStartupTime - calculateStartDiffFromBeginningOfTheRegressionFrame(overallMaxStartupTime);
}

QString
ExpressionRegressorFormulaBuilder::resolvePreprocessorsDefinition() {
	using namespace boost;

	std::vector<std::string> preprocessorDefinitionsAsStdStringVector;
	BOOST_FOREACH(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::RegressorFormulaBuilderPreprocessor_t> preprocessor, preprocessors) {
		if(preprocessor->isActive()) {
			preprocessorDefinitionsAsStdStringVector.push_back("\"" + preprocessor->resolveFormula().toStdString() + "\"");
		}
	}

	std::string preprocessorDefinitionsAsStdString = algorithm::join(preprocessorDefinitionsAsStdStringVector, ", ");
	return QString("list(") + QString::fromStdString(preprocessorDefinitionsAsStdString) + QString(")");
}

bool
ExpressionRegressorFormulaBuilder::hasActivePreprocessors() {
	bool state = false;
	BOOST_FOREACH(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder::RegressorFormulaBuilderPreprocessor_t> preprocessor, preprocessors) {
		if(preprocessor->isActive()) {
			state = true;
		}
	}

	return state;
}

QString
ExpressionRegressorFormulaBuilder::getEnhancedUserParameters() {
	using namespace boost;
	typedef UserParameter_t pair_t;
	std::vector<std::string> userParameterElements;

	BOOST_FOREACH(pair_t pair, userParameters) {
		userParameterElements.push_back(pair.first.toStdString() + std::string(" = ") + pair.second->getValue().toStdString());
	}

	std::string userParametersInListFormat = algorithm::join(userParameterElements, ", ");
	return QString("list(") + QString::fromStdString(userParametersInListFormat) + QString(")");
}

QString
ExpressionRegressorFormulaBuilder::getEnhancedFormulaDefinition() {
	std::string enhancedFormulaDefinition(getFormulaDefinition().toStdString());

	//replace every newline with a ';'
	using namespace boost;
#ifdef _WIN32
	replace_all(enhancedFormulaDefinition, "\n", ";");
#endif
	return QString::fromStdString(enhancedFormulaDefinition);
}

} //namespace r
} //namespace model
} //namespace dambase
