/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitProvider.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include <Rcpp.h>
#include <QDebug>
#include "RegressionFitProvider.hpp"

namespace dambase {
namespace model {
namespace r {

RegressionFitProvider::RegressionFitProvider(){;}
RegressionFitProvider::~RegressionFitProvider(){;}

/**
 * DataFrame is the final dataFrame on which the regression takes place.
 * This means that the dataFrame has already been shrinked to the final data by any prior processing.
 * Therefore, the returned RegressionFit<RegressionFormula> contains a reference to this dataFrame as it's the one used for the regression
 */
boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t>
RegressionFitProvider::load(boost::shared_ptr<api::IRegressionFitProvider::RegressionFitDataFrame_t> regressionFitDataFrame, boost::shared_ptr<api::IRegressionFitProvider::RegressionFormula_t> formula) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function protectedRegressionFitFunction = damBaseEnv["loadProtectedRegressionFit"];
	Rcpp::Function loadRegressionFit = damBaseEnv["loadRegressionFit"];

	Rcpp::List wrappedFit = Rcpp::as<Rcpp::List>(protectedRegressionFitFunction(
				Rcpp::Named("FUN", loadRegressionFit),
				Rcpp::Named("dataFrame", regressionFitDataFrame->getDataFrame()),
				Rcpp::Named("formula", formula->buildFormula())
			));

	boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t> result(new RegressionFit(wrappedFit, formula, regressionFitDataFrame));
	return result;
}

boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t>
RegressionFitProvider::loadPraisWinsten(boost::shared_ptr<api::IRegressionFitProvider::RegressionFitDataFrame_t> regressionFitDataFrame,boost::shared_ptr<api::IRegressionFitProvider::RegressionFormula_t> formula, unsigned iterations) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function protectedRegressionFitFunction = damBaseEnv["loadProtectedRegressionFit"];
	Rcpp::Function loadRegressionFit = damBaseEnv["loadRegressionFitLmPwiWrapper"];

	Rcpp::List wrappedFit = Rcpp::as<Rcpp::List>(protectedRegressionFitFunction(
				Rcpp::Named("FUN", loadRegressionFit),
				Rcpp::Named("dataFrame", regressionFitDataFrame->getDataFrame()),
				Rcpp::Named("formula", formula->buildFormula()),
				Rcpp::Named("iterations", iterations)
			));

	boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t> result(new RegressionFit(wrappedFit, formula, regressionFitDataFrame));
	return result;
}

boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t>
RegressionFitProvider::loadRidge(boost::shared_ptr<api::IRegressionFitProvider::RegressionFitDataFrame_t> regressionFitDataFrame, boost::shared_ptr<api::IRegressionFitProvider::RegressionFormula_t> formula, double k) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function protectedRegressionFitFunction = damBaseEnv["loadProtectedRegressionFit"];
	Rcpp::Function loadRegressionFit = damBaseEnv["loadRegressionFitLmRidgeWrapper"];

	Rcpp::List wrappedFit = Rcpp::as<Rcpp::List>(protectedRegressionFitFunction(
				Rcpp::Named("FUN", loadRegressionFit),
				Rcpp::Named("dataFrame", regressionFitDataFrame->getDataFrame()),
				Rcpp::Named("formula", formula->buildFormula()),
				Rcpp::Named("k", k)
			));

	boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t> result(new RegressionFit(wrappedFit, formula, regressionFitDataFrame));
	return result;
}

boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t>
RegressionFitProvider::loadPCR(boost::shared_ptr<api::IRegressionFitProvider::RegressionFitDataFrame_t> regressionFitDataFrame, boost::shared_ptr<api::IRegressionFitProvider::RegressionFormula_t> formula, unsigned r) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function protectedRegressionFitFunction = damBaseEnv["loadProtectedRegressionFit"];
	Rcpp::Function loadRegressionFit = damBaseEnv["loadRegressionFitLmPcrWrapper"];

	Rcpp::List wrappedFit = Rcpp::as<Rcpp::List>(protectedRegressionFitFunction(
				Rcpp::Named("FUN", loadRegressionFit),
				Rcpp::Named("dataFrame", regressionFitDataFrame->getDataFrame()),
				Rcpp::Named("formula", formula->buildFormula()),
				Rcpp::Named("r", r)
			));

	boost::shared_ptr<api::IRegressionFitProvider::RegressionFit_t> result(new RegressionFit(wrappedFit, formula, regressionFitDataFrame));
	return result;
}

boost::shared_ptr<api::IRegressionFitProvider::RegressionFitDataFrame_t>
RegressionFitProvider::loadRegressionFitDataFrame(boost::shared_ptr<api::IRegressionFitProvider::DataFrame_t> dataFrame, boost::shared_ptr<api::IRegressionFitProvider::RegressionFormula_t> formula, QString fromDate, QString toDate) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function loadRegressionFitDataFrameFunction = damBaseEnv["loadRegressionFitDataFrame"];

	Rcpp::CharacterVector selectColumns;
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFitProvider::DataFrame_t::Column_t> column, formula->getInvolvedColumns()) {
		selectColumns.push_back(column->getName().toStdString());
	}

	int overallMaxStartupTime = formula->calculateOverallMaxStartupTime();
	Rcpp::DataFrame regressionFitDataFrame = Rcpp::as<Rcpp::DataFrame>(loadRegressionFitDataFrameFunction(
				Rcpp::Named("originalDataFrame", dataFrame->getDataFrame()),
				Rcpp::Named("originalDateColumnNr", dataFrame->getDateColumn()->getIndex() + 1),
				Rcpp::Named("fromDate", fromDate.toStdString()),
				Rcpp::Named("toDate", toDate.toStdString()),
				Rcpp::Named("overallMaxStartupTime", overallMaxStartupTime),
				Rcpp::Named("selectColumns", selectColumns)
			));

	boost::shared_ptr<api::IRegressionFitProvider::RegressionFitDataFrame_t> result(new RegressionFitDataFrame(regressionFitDataFrame, dataFrame->getDateColumn(), formula->getBehaviorIndicatorColumn(), overallMaxStartupTime));
	return result;
}

} //namespace r
} //namespace model
} //namespace dambase


