/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFormula.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <iostream>
#include <stdexcept>
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/shared_ptr.hpp>
#include <Rcpp.h>
#include <QDebug>
#include "RegressionModelDefinition.hpp"
#include "RegressionFormula.hpp"

namespace dambase {
namespace model {
namespace r {

template<typename T>
struct BeforeAssignmentCategoryComparator
{
	bool operator() (boost::shared_ptr<T> const &lhs, boost::shared_ptr<T> const &rhs) const
	{
		return lhs->getIdentifier() < rhs->getIdentifier();
	}
};

template<typename T>
struct BeforeInfluenceFactorComparator
{
	bool operator() (boost::shared_ptr<T> const &lhs, boost::shared_ptr<T> const &rhs) const
	{
		return lhs->getIndex() < rhs->getIndex();
	}
};


RegressionFormula::RegressionFormula(){;}
RegressionFormula::RegressionFormula(
		std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > methodDescriptions,
		std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > regressorFormulaBuilders,
		boost::shared_ptr<api::IRegressionFormula::DataFrame_t> dataFrame
)
	:methodDescriptions(methodDescriptions), regressorFormulaBuilders(regressorFormulaBuilders), dataFrame(dataFrame)
{
}

RegressionFormula::~RegressionFormula(){
	typedef std::pair<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t>, boost::signals2::connection> pair_t;
	BOOST_FOREACH(pair_t pair, regressionModelDefinitionConnections) {
		pair.second.disconnect();
	}
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionFormula::setDataFrame(boost::shared_ptr<api::IRegressionFormula::DataFrame_t> dataFrame) {
	this->dataFrame = dataFrame;
}

api::IRegressionFormula::DataFrame_t::Column_Container_t
RegressionFormula::getInvolvedColumns() {
	api::IRegressionFormula::DataFrame_t::Column_Container_t involvedColumns;

	if(getBehaviorIndicatorColumn() != NULL) {
		involvedColumns.insert(getBehaviorIndicatorColumn());
	}

	if(dataFrame->getDateColumn() != NULL) {
		involvedColumns.insert(dataFrame->getDateColumn());
	}

	//iterate over each regressor and add influenceFactor if it contains/uses one
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getActiveRegressors()) {
		if(regressor->getInfluenceFactor() != NULL) {
			involvedColumns.insert(regressor->getInfluenceFactor());
		}
	}

	return involvedColumns;
}

void
RegressionFormula::setBehaviorIndicatorColumn(boost::shared_ptr<api::IRegressionFormula::DataFrame_t::Behavior_Indicator_Column_t> behaviorIndicatorColumn) {
	this->behaviorIndicatorColumn = behaviorIndicatorColumn;
	api::IRegressionFormula::RegressionFormulaSignals::fire();
}


const boost::shared_ptr<api::IRegressionFormula::DataFrame_t::Behavior_Indicator_Column_t> &
RegressionFormula::getBehaviorIndicatorColumn() const {
	return behaviorIndicatorColumn;
}

Rcpp::Formula
RegressionFormula::buildFormula() {
	return Rcpp::Formula(buildFormulaAsRString());
}

Rcpp::Formula
RegressionFormula::buildFormulaForSingleRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor) {
	std::string formula = buildFormulaAsRStringForSingleRegressor(regressor);
	qDebug() << "FORMULA: " << QString::fromStdString(formula);
	return Rcpp::Formula(formula);
}

Rcpp::Formula
RegressionFormula::buildFormulaForInfluenceFactorWithinAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> influenceFactor, boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory) {
	std::string formula = buildFormulaAsRStringForInfluenceFactorWithinAssignmentCategory(influenceFactor, assignmentCategory);
	qDebug() << "FORMULA: " << QString::fromStdString(formula);
	return Rcpp::Formula(formula);
}


Rcpp::Formula
RegressionFormula::buildFormulaForAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory) {
	std::string formula = buildFormulaAsRStringForAssignmentCategory(assignmentCategory);
	qDebug() << "FORMULA: " << QString::fromStdString(formula);
	return Rcpp::Formula(formula);
}


int
RegressionFormula::calculateOverallMaxStartupTime() {
	std::vector<int> startupTimes;
	startupTimes.push_back(0);

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getActiveRegressors()) {
		startupTimes.push_back(regressor->getRegressorFormulaBuilder()->calculateStartupTimesteps());
	}

	int overallMaxStartupTime = *std::max_element(startupTimes.begin(), startupTimes.end());
	qDebug() << "overallMaxStartupTime : " << overallMaxStartupTime;

	return overallMaxStartupTime;
}

QString
RegressionFormula::resolveRFormulaFor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor) {
	return regressor->getRFormula(calculateOverallMaxStartupTime());
}

std::string
RegressionFormula::buildFormulaAsRStringForSingleRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, QString delimiter) {
	if(behaviorIndicatorColumn == NULL) {
		throw std::logic_error("no behavior indicator defined");
	}
	std::vector<std::string> regressorsAsString;

	regressorsAsString.push_back(resolveRFormulaFor(regressor).toStdString());

	QString behaviorIndicatorFormula = QString("cutColumnByDiff(")
			+ QString("column = ") + behaviorIndicatorColumn->getName()
			+ QString(", cutDiff = ") + QString::number(calculateOverallMaxStartupTime())
			+ QString(")");

	std::string formula = behaviorIndicatorFormula.toStdString() + std::string(" ~ ") + boost::algorithm::join(regressorsAsString, delimiter.toStdString());
	return formula;
}

std::string
RegressionFormula::buildFormulaAsRString(QString delimiter) {
	if(behaviorIndicatorColumn == NULL) {
		throw std::logic_error("no behavior indicator defined");
	}
	std::vector<std::string> regressorsAsString;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getActiveRegressors()) {
		regressorsAsString.push_back(resolveRFormulaFor(regressor).toStdString());
	}

	QString behaviorIndicatorFormula = QString("cutColumnByDiff(")
			+ QString("column = ") + behaviorIndicatorColumn->getName()
			+ QString(", cutDiff = ") + QString::number(calculateOverallMaxStartupTime())
			+ QString(")");

	std::string formula = behaviorIndicatorFormula.toStdString() + std::string(" ~ ") + boost::algorithm::join(regressorsAsString, delimiter.toStdString());
	return formula;
}


std::string
RegressionFormula::buildFormulaAsRStringForInfluenceFactorWithinAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> influenceFactor, boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory, QString delimiter) {
	if(behaviorIndicatorColumn == NULL) {
		throw std::logic_error("no behavior indicator defined");
	}
	std::vector<std::string> regressorsAsString;
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getActiveRegressors()) {
		boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> regressorsInfluenceFactor = regressor->getInfluenceFactor();
		if(regressorsInfluenceFactor == NULL) {
			continue;
		}

		if(regressor->getAssignmentCategory()->getIdentifier() == assignmentCategory->getIdentifier()
				&& regressorsInfluenceFactor->getIndex() == influenceFactor->getIndex()
				&& regressorsInfluenceFactor->getName() == influenceFactor->getName()
				)
		{
			regressorsAsString.push_back(resolveRFormulaFor(regressor).toStdString());
		}
	}

	QString behaviorIndicatorFormula = QString("cutColumnByDiff(")
			+ QString("column = ") + behaviorIndicatorColumn->getName()
			+ QString(", cutDiff = ") + QString::number(calculateOverallMaxStartupTime())
			+ QString(")");

	std::string formula = behaviorIndicatorFormula.toStdString() + std::string(" ~ ") + boost::algorithm::join(regressorsAsString, delimiter.toStdString());
	return formula;

}


std::string
RegressionFormula::buildFormulaAsRStringForAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory, QString delimiter) {
	if(behaviorIndicatorColumn == NULL) {
		throw std::logic_error("no behavior indicator defined");
	}
	std::vector<std::string> regressorsAsString;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getActiveRegressors()) {
		if(regressor->getAssignmentCategory()->getName() == assignmentCategory->getName()) {
			regressorsAsString.push_back(resolveRFormulaFor(regressor).toStdString());
		}
	}

	QString behaviorIndicatorFormula = QString("cutColumnByDiff(")
			+ QString("column = ") + behaviorIndicatorColumn->getName()
			+ QString(", cutDiff = ") + QString::number(calculateOverallMaxStartupTime())
			+ QString(")");

	std::string formula = behaviorIndicatorFormula.toStdString() + std::string(" ~ ") + boost::algorithm::join(regressorsAsString, delimiter.toStdString());
	return formula;
}


QString
RegressionFormula::toString(QString delimiter) {
	if(behaviorIndicatorColumn == NULL) {
		throw std::logic_error("no behavior indicator defined");
	}
	std::vector<std::string> regressorsAsString;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getActiveRegressors()) {
		regressorsAsString.push_back(regressor->getFormula().toStdString());
	}

	std::string formula = behaviorIndicatorColumn->getName().toStdString() + std::string(" ~ ") + boost::algorithm::join(regressorsAsString, delimiter.toStdString());
	return QString::fromStdString(formula);
}


std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> >
RegressionFormula::getMethodDescriptions() {
	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > availableMethodDescriptionsForCurrentMode;
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> methodDescription, methodDescriptions) {
		if(methodDescription->availableInCurrentMode()) {
			availableMethodDescriptionsForCurrentMode.push_back(methodDescription);
		}
	}
	return availableMethodDescriptionsForCurrentMode;
}

/**
 * Primarly used by the serialization process
 */
const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > &
RegressionFormula::getAllMethodDescriptionsRegardlessOfTheMode() const {
	return methodDescriptions;
}

/**
 * Primarly used by the deserialization process
 */
void
RegressionFormula::setMethodDescriptions(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > &methodDescriptions) {
	this->methodDescriptions = methodDescriptions;
}

/**
 * Primarly used by the deserialization process
 */
void
RegressionFormula::setCustomRegressionModelDefinitions(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &regressionModelDefinitions) {
	this->customRegressionModelDefinitions = regressionModelDefinitions;
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition, this->customRegressionModelDefinitions) {
		keepRegressionModelDefinition(regressionModelDefinition);
	}
}

void
RegressionFormula::addCustomRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition) {
	customRegressionModelDefinitions.push_back(regressionModelDefinition);
	keepRegressionModelDefinition(regressionModelDefinition);
}

/**
 * Primarly used by the deserialization process
 */
void
RegressionFormula::setTemplatedRegressionModelDefinitions(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &regressionModelDefinitions) {
	this->templatedRegressionModelDefinitions = regressionModelDefinitions;
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition, this->templatedRegressionModelDefinitions) {
		keepRegressionModelDefinition(regressionModelDefinition);
	}
}

void
RegressionFormula::addTemplatedRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition) {
	templatedRegressionModelDefinitions.push_back(regressionModelDefinition);
	keepRegressionModelDefinition(regressionModelDefinition);
}

void
RegressionFormula::removeRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition) {
	if(removeRegressionModelDefinition(regressionModelDefinition, templatedRegressionModelDefinitions)) {
		return;
	}
	removeRegressionModelDefinition(regressionModelDefinition, customRegressionModelDefinitions);
}

const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &
RegressionFormula::getCustomRegressionModelDefinitions() const {
	return customRegressionModelDefinitions;
}

const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &
RegressionFormula::getTemplatedRegressionModelDefinitions() const {
	return templatedRegressionModelDefinitions;
}

std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> >
RegressionFormula::getAllRegressionModelDefinitions() {
	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > allRegressionModelDefinitions;
	allRegressionModelDefinitions.insert(allRegressionModelDefinitions.begin(), templatedRegressionModelDefinitions.begin(), templatedRegressionModelDefinitions.end());
	allRegressionModelDefinitions.insert(allRegressionModelDefinitions.begin(), customRegressionModelDefinitions.begin(), customRegressionModelDefinitions.end());

	return allRegressionModelDefinitions;
}


void
RegressionFormula::releaseRegressionModelDefinitionForModification(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> templatedModelDefinition, templatedRegressionModelDefinitions) {
		if(regressionModelDefinition == templatedModelDefinition) {
			customRegressionModelDefinitions.push_back(regressionModelDefinition);
			templatedRegressionModelDefinitions.erase(std::remove(templatedRegressionModelDefinitions.begin(), templatedRegressionModelDefinitions.end(), templatedModelDefinition));
			regressionModelDefinition->releaseForModification(getRegressorFormulaBuilders());
			return;
		}
	}

	throw std::logic_error("regressionModelDefinition (" + regressionModelDefinition->getName().toStdString() + ") is not a templated model definition and can not be released therefore. This is a bug.");
}


/**
 * This is primarly used by the deserialization process
 */
void
RegressionFormula::setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > &regressorFormulaBuilders) {
	this->regressorFormulaBuilders = regressorFormulaBuilders;
}

void
RegressionFormula::addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> regressorFormulaBuilder) {
	regressorFormulaBuilders.push_back(regressorFormulaBuilder);

	api::IRegressionFormula::RegressionFormulaSignals::RegressorFormulaBuilderAddedSignal(regressorFormulaBuilder);
	api::IRegressionFormula::RegressionFormulaSignals::fire();

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> modelDefinition, customRegressionModelDefinitions) {
		modelDefinition->addRegressorFormulaBuilder(regressorFormulaBuilder);
	}
}

const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > &
RegressionFormula::getRegressorFormulaBuilders() const {
	return regressorFormulaBuilders;
}

boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t>
RegressionFormula::findRegressorFormulaBuilderByIdentifier(QString identifier) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> formulaBuilder, regressorFormulaBuilders) {
		if(formulaBuilder->getIdentifier() == identifier) {
			return formulaBuilder;
		}
	}

	boost::shared_ptr<RegressorFormulaBuilder_t> nullReturn;
	return nullReturn;
}

boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t>
RegressionFormula::findRegressorFormulaBuilderByFormula(QString formula) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> formulaBuilder, regressorFormulaBuilders) {
		if(formulaBuilder->getFormula() == formula) {
			return formulaBuilder;
		}
	}

	boost::shared_ptr<RegressorFormulaBuilder_t> nullReturn;
	return nullReturn;
}

QString
RegressionFormula::getNewIdentifierFor(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> regressorFormulaBuilder) {
	int maxConcatenations = 20;
	QString derivePattern = "_";
	QString identifierPattern = derivePattern + regressorFormulaBuilder->getIdentifier();

	QString beginPattern;
	for(int loops = 1; loops <= maxConcatenations; ++loops) {
		beginPattern += derivePattern;
	}
	if(boost::starts_with(identifierPattern, beginPattern)) {
		throw std::logic_error(std::string("please rename the identifiers for ") + identifierPattern.toStdString());
	}

	for(int loops = 1; loops <= maxConcatenations; ++loops) {
		if(findRegressorFormulaBuilderByIdentifier(identifierPattern) == NULL) {
			return identifierPattern;
		}else{
			identifierPattern = derivePattern + identifierPattern;
		}
	};

	throw std::logic_error(std::string("please rename the identifiers for ") + identifierPattern.toStdString());
}

QString
RegressionFormula::getNewFormulaFor(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> regressorFormulaBuilder) {
	QString derivePattern = "_";
	QString formulaPattern = derivePattern + regressorFormulaBuilder->getFormula();

	int maxConcatenations = 20;

	QString beginPattern;
	for(int loops = 1; loops <= maxConcatenations; ++loops) {
		beginPattern += derivePattern;
	}
	if(boost::starts_with(formulaPattern, beginPattern)) {
		throw std::logic_error(std::string("please rename the formula for ") + formulaPattern.toStdString());
	}

	for(int loops = 1; loops <= maxConcatenations; ++loops) {
		if(findRegressorFormulaBuilderByFormula(formulaPattern) == NULL) {
			return formulaPattern;
		}else{
			formulaPattern = derivePattern + formulaPattern;
		}
	};

	throw std::logic_error(std::string("please rename the formula for ") + formulaPattern.toStdString());
}

std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> >
RegressionFormula::getActiveRegressors() {
	std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > activeRegressors;
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getRegressors()) {
		if(regressor->isActive()) {
			activeRegressors.push_back(regressor);
		}
	}

	return activeRegressors;
}

std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> >
RegressionFormula::getRegressors() {
	std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > regressors;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition, getAllRegressionModelDefinitions()) {
		BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, regressionModelDefinition->getRegressors()) {
			regressors.push_back(regressor);
		}
	}

	return regressors;
}

boost::shared_ptr<api::IRegressionFormula::Regressor_t>
RegressionFormula::findRegressorByIndex(int index) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getRegressors()) {
		if(regressor->getIndex() == index) {
			return regressor;
		}
	}

	boost::shared_ptr<api::IRegressionFormula::Regressor_t> nullReturn;
	return nullReturn;
}

boost::shared_ptr<api::IRegressionFormula::Regressor_t>
RegressionFormula::findRegressorByFormula(QString formula) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, getRegressors()) {
		if(regressor->getFormula() == formula) {
			return regressor;
		}
	}

	boost::shared_ptr<Regressor_t> nullReturn;
	return nullReturn;
}

std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> >
RegressionFormula::getInvolvedAssignmentCategoriesWithAnInfluenceFactor() {
	std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> > involvedAssignmentCategoriesWithAnInfluenceFactor;

	std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> > allAssignmentCategories = findAssignmentCategoriesFor(getActiveRegressors());

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory, allAssignmentCategories) {
		if(assignmentCategory->hasColumns()) {
			involvedAssignmentCategoriesWithAnInfluenceFactor.push_back(assignmentCategory);
		}
	}

	return involvedAssignmentCategoriesWithAnInfluenceFactor;
}

std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> >
RegressionFormula::findAssignmentCategoriesFor(std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > regressors) {
	std::set<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> , BeforeAssignmentCategoryComparator<api::IRegressionFormula::AssignmentCategory_t> > resultAsSet;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, regressors) {
		resultAsSet.insert(regressor->getAssignmentCategory());
	}

	std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> > resultAsVector;
	resultAsVector.insert(resultAsVector.begin(), resultAsSet.begin(), resultAsSet.end());

	return resultAsVector;
}


/**
 * Returns a vector containing all (unique) influence factors of all regressors being assigned to a given assignmentcategory
 */
std::vector<boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> >
RegressionFormula::findInfluenceFactorsOfRegressorsAssignedTo(std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > regressors, boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory) {
	std::set<boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> , BeforeInfluenceFactorComparator<api::IRegressionFormula::InfluenceFactor_t> > resultAsSet;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, regressors) {
		boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> regressorAssignmentCategory = regressor->getAssignmentCategory();
		if(regressorAssignmentCategory->getIdentifier() == assignmentCategory->getIdentifier()) {
			boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> influenceFactor = regressor->getInfluenceFactor();
			if(influenceFactor != NULL) {
				resultAsSet.insert(influenceFactor);
			}
		}
	}

	std::vector<boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> > resultAsVector;
	resultAsVector.insert(resultAsVector.begin(), resultAsSet.begin(), resultAsSet.end());

	return resultAsVector;
}

boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t>
RegressionFormula::getRegressionModelDefinitionForRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition, getAllRegressionModelDefinitions()) {
		if(regressionModelDefinition->containsRegressor(regressor)) {
			return regressionModelDefinition;
		}
	}

	boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> nullReturn;
	return nullReturn;
}

bool
RegressionFormula::containsRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor) {
	return findRegressorByFormula(regressor->getFormula()) != NULL;
}


const boost::shared_ptr<api::IRegressionFormula::DataFrame_t> &
RegressionFormula::getDataFrame() const {
	return dataFrame;
}

void
RegressionFormula::keepRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition) {
	using namespace boost;

	api::IRegressionFormula::RegressionFormulaSignals::RegressionModelDefinitionAddedSignal(regressionModelDefinition);
	api::IRegressionFormula::RegressionFormulaSignals::fire();

	regressionModelDefinitionConnections[regressionModelDefinition] = COM_SUBSCRIBE_FOR(regressionModelDefinition, Changed, (bind(&api::IRegressionFormula::RegressionFormulaSignals::fire, this)));
}

bool
RegressionFormula::removeRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition, std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &modelDefinitions) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> temporaryModelDefinition, modelDefinitions) {
		if(regressionModelDefinition == temporaryModelDefinition) {
			modelDefinitions.erase(std::remove(modelDefinitions.begin(), modelDefinitions.end(), regressionModelDefinition));
			regressionModelDefinitionConnections[regressionModelDefinition].disconnect();
			regressionModelDefinitionConnections.erase(regressionModelDefinition);

			qDebug() << "RegressionFormula::removeRegressionModelDefinition going to remove";
			api::IRegressionFormula::RegressionFormulaSignals::RemoveRegressionModelDefinitionSignal(regressionModelDefinition);
			api::IRegressionFormula::RegressionFormulaSignals::fire();
			return true;
		}
	}

	return false;
}


} //namespace r
} //namespace model
} //namespace dambase
