/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelDefinition.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */


#include "RegressionModelDefinition.hpp"

namespace dambase {
namespace model {
namespace r {

RegressionModelDefinition::RegressionModelDefinition() : iseditable(true)
{;}

RegressionModelDefinition::RegressionModelDefinition(QString name, boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> dataFrame, bool iseditable):name(name), dataFrame(dataFrame), iseditable(iseditable)
{;}

RegressionModelDefinition::~RegressionModelDefinition() {
	typedef std::pair<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t>, boost::signals2::connection > formula_builder_pair_t;
	BOOST_FOREACH(formula_builder_pair_t pair, regressorFormulaBuilderConnections) {
		pair.second.disconnect();
	}

	typedef std::pair<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t>, boost::signals2::connection> regressor_pair_t;
	BOOST_FOREACH(regressor_pair_t pair, regressorConnections) {
		pair.second.disconnect();
	}
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelDefinition::setDataFrame(boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> dataFrame) {
	this->dataFrame = dataFrame;
}

/**
 * This is primarly used by the deserialization process. For other purposes, use #freezeModification
 */
void
RegressionModelDefinition::setIsEditable(bool iseditable) {
	this->iseditable = iseditable;
}

const bool &
RegressionModelDefinition::isEditable() const {
	return iseditable;
}

void
RegressionModelDefinition::freezeModification() {
	iseditable = false;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor, getRegressors()) {
		regressor->freezeModification();
	}

	api::IRegressionModelDefinition::fireModificationChanged();
}

void
RegressionModelDefinition::releaseForModification(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > additionalFormulaBuilders) {
	iseditable = true;
	setName("* derived from " + getName());

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor, getRegressors()) {
		regressor->releaseForModification();
	}

	//merge additionalFormulaBuilders with local ones
	std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > localFormulaBuilders = regressorFormulaBuilders;
	std::sort(localFormulaBuilders.begin(), localFormulaBuilders.end());
	std::sort(additionalFormulaBuilders.begin(), additionalFormulaBuilders.end());

	std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > notContainedFormulaBuilders;
	std::set_difference(
			additionalFormulaBuilders.begin(), additionalFormulaBuilders.end(),
			localFormulaBuilders.begin(), localFormulaBuilders.end(),
			std::back_inserter(notContainedFormulaBuilders)
			);

	addRegressorFormulaBuilders(notContainedFormulaBuilders);

	api::IRegressionModelDefinition::fireModificationChanged();
}

void
RegressionModelDefinition::setName(QString name) {
	this->name = name;
	api::IRegressionModelDefinition::fire();
}

const QString &
RegressionModelDefinition::getName() const {
	return name;
}

const std::vector<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> > &
RegressionModelDefinition::getRegressors() const {
	return regressors;
}

/**
 * This is primarly used by deserialization process. For other purposes, use #addRegressorFormulaBuilders
 */
void
RegressionModelDefinition::setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > &formulaBuilders) {
	using namespace boost;

	this->regressorFormulaBuilders = formulaBuilders;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder, this->regressorFormulaBuilders) {
		regressorFormulaBuilderConnections[formulaBuilder] = COM_SUBSCRIBE_FOR(formulaBuilder, Changed, bind(&api::IRegressionModelDefinition::fire, this));
	}
}

void
RegressionModelDefinition::addRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > formulaBuilders) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder, formulaBuilders) {
		addRegressorFormulaBuilder(formulaBuilder);
	}
}

void
RegressionModelDefinition::addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder) {
	using namespace boost;

	regressorFormulaBuilders.push_back(formulaBuilder);
	api::IRegressionModelDefinition::fireRegressorFormulaBuilderAdded(formulaBuilder);
	regressorFormulaBuilderConnections[formulaBuilder] = COM_SUBSCRIBE_FOR(formulaBuilder, Changed, bind(&api::IRegressionModelDefinition::fire, this));
}

void
RegressionModelDefinition::removeRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder) {

	throw std::logic_error("removeRegressorFormulaBuilder : not completely implemented yet. However, keep track of the connection as described in the implementation");
	//if(regressorFormulaBuilderConnections[formulaBuilder] != NULL) {
		regressorFormulaBuilderConnections[formulaBuilder].disconnect();
		regressorFormulaBuilderConnections.erase(formulaBuilder);
	//}
}

const std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > &
RegressionModelDefinition::getRegressorFormulaBuilders() const {
	return regressorFormulaBuilders;
}

boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t>
RegressionModelDefinition::findRegressorFormulaBuilderByIdentifier(QString identifier) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> regressorFormulaBuilder, getRegressorFormulaBuilders()) {
		if(regressorFormulaBuilder->getIdentifier() == identifier) {
			return regressorFormulaBuilder;
		}
	}

	boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> nullReturn;
	return nullReturn;
}

/**
 * This is primary used by the deserialization process
 */
void
RegressionModelDefinition::setRegressors(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> > regressors) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor, regressors) {
		addRegressor(regressor);
	}
	//this->regressors = regressors;
}

void
RegressionModelDefinition::addRegressor(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor) {
	using namespace boost;
	regressors.push_back(regressor);

	if(iseditable == false) {
		regressor->freezeModification();
	}

	regressorConnections[regressor] = COM_SUBSCRIBE_FOR(regressor, Changed, bind(&api::IRegressionModelDefinition::fire, this));
	api::IRegressionModelDefinition::fireRegressorAdded(regressor);
}

bool
RegressionModelDefinition::containsRegressor(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor) {
	BOOST_FOREACH(boost::shared_ptr<Regressor_t> localRegressor, regressors) {
		if(regressor->getFormula() == localRegressor->getFormula() || regressor == localRegressor) {
			return true;
		}
	}

	return false;
}

void
RegressionModelDefinition::removeRegressor(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor) {
	regressors.erase(std::remove(regressors.begin(), regressors.end(), regressor));

	api::IRegressionModelDefinition::fireRemoveRegressor(regressor);
	if(regressorConnections.count(regressor) != 0) {
		regressorConnections[regressor].disconnect();
		regressorConnections.erase(regressor);
	}
}

const boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> &
RegressionModelDefinition::getDataFrame() const {
	return dataFrame;
}



} //namespace r
} //namespace model
} //namespace dambase
