/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Regressor.cpp
 *
 *  Created on: 13.02.2015
 *      Author: gerberma
 */


#include <QDebug>
#include "Regressor.hpp"

namespace dambase {
namespace model {
namespace r {


Regressor::Regressor()
		:isactive(true), iseditable(true), index(0)
	{;}

Regressor::Regressor(boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> regressorFormulaBuilder, boost::shared_ptr<api::IRegressor::DataFrame_t> dataFrame, int index, bool isactive, bool iseditable)
		:regressorFormulaBuilder(regressorFormulaBuilder), dataFrame(dataFrame), isactive(isactive), iseditable(iseditable), index(index)
	{
	}

Regressor::~Regressor()
	{;}

	/**
	 * This is primarly used by the serialization process
	 */
const boost::shared_ptr<api::IRegressor::DataFrame_t> &
Regressor::getDataFrame() const {
	return dataFrame;
}

/**
 * This is primarly used by the deserialization process
 */
void
Regressor::setDataFrame(boost::shared_ptr<DataFrame_t> dataFrame) {
	this->dataFrame = dataFrame;
}

//mostly for gui use only as the R formula for example should be resolved internally
const boost::shared_ptr<api::IRegressor::Influence_Factor_t> &
Regressor::getInfluenceFactor() const {
	return influenceFactor;
}

/**
 * This is primarly used by the deserialization process.
 */
void
Regressor::setRegressorFormulaBuilder(boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> regressorFormulaBuilder) {
	this->regressorFormulaBuilder = regressorFormulaBuilder;
}

const boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> &
Regressor::getRegressorFormulaBuilder() const {
	return regressorFormulaBuilder;
}

boost::shared_ptr<api::IRegressor::AssignmentCategory_t>
Regressor::getAssignmentCategory() const {
	return regressorFormulaBuilder->getAssignmentCategory();
}

QString
Regressor::getRFormula(int overallMaxStartupTime) {
	if(regressorFormulaBuilder == NULL) {
		//throw std::logic_error("No regressorFormulaBuilder defined. Cannot handle getRFormula()");
		return "";
	}

	QString rFormula;
	if(influenceFactor == NULL) {
		rFormula = regressorFormulaBuilder->resolveFormulaDefinition(dataFrame, overallMaxStartupTime);
	}else{
		rFormula = regressorFormulaBuilder->resolveFormulaDefinition(dataFrame, influenceFactor, overallMaxStartupTime);
	}

	qDebug() << "rFormula: " << rFormula;

	return rFormula;
}

QString
Regressor::getFormula() {
	if(regressorFormulaBuilder == NULL) {
		//throw std::logic_error("No regressorFormulaBuilder defined. Cannot handle getFormula()");
		return "";
	}

	if(influenceFactor == NULL) {
		return regressorFormulaBuilder->resolveName();
	}else{
		return regressorFormulaBuilder->resolveName(influenceFactor);
	}
}

QString
Regressor::getName() {
	return getFormula(); // TODO: introduce a name
}

typename api::IRegressor::DataFrame_t::Influence_Factor_Container_t
Regressor::getAvailableInfluenceFactors() {
	return regressorFormulaBuilder->getAvailableInfluenceFactors(dataFrame); //TODO: use formulaBuilder to resolve the available columns
}

void
Regressor::setInfluenceFactor(int influenceFactorIndex) {
	setInfluenceFactor(dataFrame->getColumnByIndex(influenceFactorIndex));
}

void
Regressor::setInfluenceFactor(boost::shared_ptr<api::IRegressor::Influence_Factor_t> influenceFactorToBeSet) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressor::Influence_Factor_t> availableInfluenceFactor, getAvailableInfluenceFactors()) {
		if(availableInfluenceFactor->getName() == influenceFactorToBeSet->getName() && availableInfluenceFactor->getIndex() == influenceFactorToBeSet->getIndex()) {
			this->influenceFactor = influenceFactorToBeSet;
			return;
		}
	}

	if(influenceFactorToBeSet != NULL) {
		throw std::logic_error("Influence factor (" + influenceFactorToBeSet->getName().toStdString() + ") for this regressor (" + getName().toStdString() + ") is not assignable");
	}
}

/**
 * This is primary used by the deserialization process
 */
void
Regressor::restoreInfluenceFactor(boost::shared_ptr<api::IRegressor::Influence_Factor_t> influenceFactor) {
	this->influenceFactor = influenceFactor;
}

void
Regressor::freezeModification() {
	iseditable = false;
}

void
Regressor::releaseForModification() {
	iseditable = true;
}

const bool &
Regressor::isEditable() const {
	return iseditable;
}

/**
 * This is primary used by the deserialization process. Use either #releaseForModification or #freezeModification for other purposes
 */
void
Regressor::setIsEditable(bool isEditable) {
	this->iseditable = isEditable;
}

void
Regressor::setIsActive(bool isactive) {
	this->isactive = isactive;
	communication::BoostSignalPolicy::fire();
}

const bool &
Regressor::isActive() const {
	return isactive;
}

/**
 * This is primary used by the deserialization process
 */
void
Regressor::setIndex(int index) {
	this->index = index;
}

const int &
Regressor::getIndex() const {
	return index;
}


} //namespace r
} //namespace model
} //namespace dambase
