/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IDataFrame.hpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */

#ifndef IDATAFRAME_HPP_
#define IDATAFRAME_HPP_

#include <vector>
#include <set>
#include <boost/shared_ptr.hpp>
#include "IAssignmentCategory.hpp"
#include "model/r/DateColumn.hpp"
#include "model/r/BehaviorIndicatorColumn.hpp"
#include "model/r/WaterLevelColumn.hpp"
#include "model/r/TemperatureColumn.hpp"
#include "model/r/DataFrameHeader.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

template<typename T>
struct BeforeColumnComparator
{
	bool operator() (boost::shared_ptr<T> const &lhs, boost::shared_ptr<T> const &rhs) const
	{
		return lhs->getIndex() < rhs->getIndex();
	}
};


template<typename T>
struct BeforeColumnComparatorForReferences
{
	bool operator() (T const &lhs, T const &rhs) const
	{
		return lhs.getIndex() < rhs.getIndex();
	}
};

class IDataFrame {
public:
	typedef model::r::DateColumn DateColumn_t;
	typedef model::r::DataFrameHeader Header_t;
	typedef model::r::DataFrameColumn Column_t;
	typedef BeforeColumnComparator<Column_t> ColumnComparator_t;
	typedef std::vector<model::r::DataFrameHeader> Header_Container_t;
	typedef Header_Container_t Assignable_Header_Container_t;
	typedef BehaviorIndicatorColumn Behavior_Indicator_Column_t;
	typedef model::r::WaterLevelColumn Water_Level_Column_t;
	typedef model::r::TemperatureColumn Temperature_Column_t;
	typedef Column_t Influence_Factor_t;
	typedef std::vector<boost::shared_ptr<Column_t> > Influence_Factor_Container_t;
	typedef std::set<boost::shared_ptr<Column_t>, ColumnComparator_t> Column_Container_t;
	typedef std::set<boost::shared_ptr<Behavior_Indicator_Column_t>, BeforeColumnComparator<BehaviorIndicatorColumn> > Behavior_Indicator_Column_Container_t;
	typedef std::set<boost::shared_ptr<Water_Level_Column_t>, BeforeColumnComparator<WaterLevelColumn> > Water_Level_Column_Container_t;
	typedef std::set<boost::shared_ptr<Temperature_Column_t>, BeforeColumnComparator<TemperatureColumn> > Temperature_Column_Container_t;
public:
	virtual void updateSomeRawValues() = 0;

	virtual void updateFromApplyDateFormat(Rcpp::DataFrame dataFrame) = 0;

	virtual DataFrameHeader &getHeader(int at) = 0;

	virtual Header_Container_t& getHeaders() = 0;

	virtual std::vector<QString> getSomeRawValuesFor(QString headerName) = 0;

	virtual const std::map<QString, std::vector<QString> > &getRawValues() const = 0;

	virtual boost::shared_ptr<Column_t> getColumnByIndex(int index) = 0;

	virtual const Assignable_Header_Container_t getAssignableDateColumnHeaders() = 0;

	virtual const Assignable_Header_Container_t getAssignableBehaviorIndicatorHeaders() = 0;

	virtual const Assignable_Header_Container_t getAssignableWaterLevelHeaders() = 0;

	virtual const Assignable_Header_Container_t getAssignableTemperatureHeaders() = 0;

	virtual int getNrOfRows() = 0;

	virtual int getNrOfColumns() = 0;

	/*
	 * disabled temporarly
	getRow(int at);
	*/

	virtual bool setDateColumn(int fromIndex, bool forceUpdate = false) = 0;

	/**
	 * This is called from the deserialization process
	 */
	virtual void setDateColumn(boost::shared_ptr<DateColumn> dateColumn) = 0;

	virtual const boost::shared_ptr<DateColumn> &getDateColumn() const = 0;

	virtual boost::shared_ptr<DateColumn> getDateColumn(Rcpp::DataFrame forDataFrame) = 0;

	virtual int estimateDateColumn() = 0;


	virtual bool assignBehaviorIndicatorColumn(int fromIndex) = 0;

	virtual bool isBehaviorIndicatorColumnAssignedFor(int index) = 0;

	virtual bool removeBehaviorIndicatorColumn(int fromIndex) = 0;

	virtual const Behavior_Indicator_Column_Container_t &getBehaviorIndicatorColumns() const = 0;

	virtual void setBehaviorIndicatorColumns(Behavior_Indicator_Column_Container_t &columns) = 0;

	virtual bool assignWaterLevelColumn(int fromIndex) = 0;

	virtual bool isWaterLevelColumnAssignedFor(int index) = 0;

	virtual bool removeWaterLevelColumn(int fromIndex) = 0;

	virtual boost::shared_ptr<WaterLevelColumn> findWaterLevelColumnByName(QString name) = 0;

	virtual const Water_Level_Column_Container_t &getWaterLevelColumns() const = 0;

	virtual void setWaterLevelColumns(Water_Level_Column_Container_t &columns) = 0;

	virtual bool assignTemperatureColumn(int fromIndex) = 0;

	virtual bool isTemperatureColumnAssignedFor(int index) = 0;

	virtual bool removeTemperatureColumn(int fromIndex) = 0;

	virtual const Temperature_Column_Container_t &getTemperatureColumns() const = 0;

	virtual void setTemperatureColumns(Temperature_Column_Container_t &columns) = 0;

	virtual Influence_Factor_Container_t getAssignedInfluenceFactorsFor(boost::shared_ptr<IAssignmentCategory> assignmentCategory) = 0;

	virtual void setDataFrameFromDeserialization(Rcpp::DataFrame dataFrame) = 0;

	virtual void calculateNonPointable() = 0;

	/**
	 * Use this only for methods based on the real dataFrame for other calculations
	 */
	virtual Rcpp::DataFrame &getDataFrame() = 0;

	/**
	 * For deserialization
	 */
	virtual const Rcpp::DataFrame &getDataFrameConst() const = 0;
public:
	IDataFrame(){;}
	virtual ~IDataFrame(){;}
};

} //namespace api
} //namespace r
} //namespace model
} //namespace



#endif /* IDATAFRAME_HPP_ */
