/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IExpressionRegressorFormulaBuilder.hpp
 *
 *  Created on: 13.02.2015
 *      Author: gerberma
 */

#ifndef IEXPRESSIONREGRESSORFORMULABUILDER_HPP_
#define IEXPRESSIONREGRESSORFORMULABUILDER_HPP_

#include <QString>
#include <map>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "model/SignalPolicies.hpp"
#include "IDataFrame.hpp"
#include "IRegressorFormulaBuilderPreprocessor.hpp"
#include "IExpressionRegressorFormulaBuilderUserParameterType.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

namespace detail {

class ExpressionRegressorFormulaBuilderSignals : public communication::BoostSignalPolicy {
public:
	typedef std::pair<QString, boost::shared_ptr<IExpressionRegressorFormulaBuilderUserParameterType> > UserParameter_t;
public:
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(PrimaryChange, communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(UserParameterAdded, communication::Signal<void(UserParameter_t)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveUserParameter, communication::Signal<void(UserParameter_t)>);

	void firePrimaryChange() {
		communication::BoostSignalPolicy::fire();
		PrimaryChangeSignal();
	}

	void fireUserParameterAdded(UserParameter_t userParameter) {
		communication::BoostSignalPolicy::fire();
		UserParameterAddedSignal(userParameter);
	}

	void fireRemoveUserParameter(UserParameter_t userParameter) {
		communication::BoostSignalPolicy::fire();
		RemoveUserParameterSignal(userParameter);
	}
};

}  // namespace detail

class IExpressionRegressorFormulaBuilder : public detail::ExpressionRegressorFormulaBuilderSignals {
public:
	typedef IAssignmentCategory AssignmentCategory_t;
	typedef IRegressorFormulaBuilderPreprocessor RegressorFormulaBuilderPreprocessor_t;
	typedef api::IExpressionRegressorFormulaBuilderUserParameterType UserParameterType_t;
	typedef std::pair<QString, boost::shared_ptr<UserParameterType_t> > UserParameter_t;
	typedef std::map<QString, boost::shared_ptr<UserParameterType_t> > UserParameters_t;
	typedef IDataFrame DataFrame_t;
	typedef IDataFrame::Influence_Factor_t InfluenceFactor_t;
public:
	IExpressionRegressorFormulaBuilder(){;}
	virtual ~IExpressionRegressorFormulaBuilder(){;}

	virtual const QString &getIdentifier() const = 0;

	virtual void setIdentifier(QString identifier) = 0;

	virtual QString resolveExtendedIdentifier() = 0;

	virtual QString resolveName(boost::shared_ptr<InfluenceFactor_t> influenceFactor) = 0;

	virtual QString resolveName() = 0;

	virtual void setFormula(QString formula) = 0;

	virtual const QString &getFormula() const = 0;

	virtual void setFormulaDefinition(QString formulaDefinition) = 0;

	virtual const QString &getFormulaDefinition() const = 0;

	virtual QString resolveFormulaDefinition(boost::shared_ptr<DataFrame_t> dataFrame, int overallMaxStartupTime) = 0;


	/**
	 * overallMaxStartupTime is the maximum startup time of all involved regressors
	 * Therefore, the dateColumn has to start at the date where the overallMaxStartupTime added points to the startup time of the regression
	 * So we calculate here a difference which tells us at what date we can start and provide this difference to the evaluateExpansion function
	 */
	virtual QString resolveFormulaDefinition(boost::shared_ptr<DataFrame_t> dataFrame, boost::shared_ptr<InfluenceFactor_t> influenceFactor, int overallMaxStartupTime) = 0;

	virtual bool hasPreprocessors() = 0;

	virtual bool requiresInfluenceFactor() = 0;

	virtual boost::shared_ptr<AssignmentCategory_t> findAssignmentCategoryByIdentifier(QString identifier) = 0;

	/**
	 * This is primarly used by the deserialization process.
	 */
	virtual void setAssignmentCategories(std::vector<boost::shared_ptr<AssignmentCategory_t> > &assignmentCategories) = 0;

	virtual std::vector<boost::shared_ptr<AssignmentCategory_t> > &getAssignmentCategories() = 0;

	virtual void setAssignmentCategory(boost::shared_ptr<AssignmentCategory_t> assignmentCategory) = 0;

	virtual boost::shared_ptr<AssignmentCategory_t> getAssignmentCategory() const = 0;

	virtual DataFrame_t::Influence_Factor_Container_t getAvailableInfluenceFactors(boost::shared_ptr<DataFrame_t> dataFrame) = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setIsEditable(bool iseditable) = 0;

	virtual const bool &isEditable() const = 0;

	/**
	 * This is primarly used by the deserialization process.
	 */
	virtual void setUserParameters(UserParameters_t &userParameters) = 0;

	virtual void addUserParameter(QString name, boost::shared_ptr<UserParameterType_t> userParameterType) = 0;

	virtual const std::map<QString, boost::shared_ptr<UserParameterType_t> > &getUserParameters() const = 0;

	virtual void removeUserParameter(UserParameter_t userParameter) = 0;

	virtual int calculateStartupTimesteps() = 0;

	virtual const std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> >& getPreprocessors() const = 0;

	/**
	 * This is primarly used by the deserialization process as well as the clone process
	 */
	virtual void setPreprocessors(std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> >& preprocessors) = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace dambase

#endif /* IEXPRESSIONREGRESSORFORMULABUILDER_HPP_ */
