/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFormula.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFORMULA_HPP_
#define IREGRESSIONFORMULA_HPP_

#include <boost/shared_ptr.hpp>
#include <vector>
#include "IAssignmentCategory.hpp"
#include "IRegressor.hpp"
#include "IDataFrame.hpp"
#include "IRegressionModelTemplate.hpp"
#include "IRegressionModelDefinition.hpp"
#include "IExpressionRegressorFormulaBuilder.hpp"
#include "model/SignalPolicies.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

namespace detail {
class RegressionFormulaSignals : public communication::BoostSignalPolicy {
public:
	typedef IRegressionModelDefinition RegressionModelDefinition_t;
	typedef IExpressionRegressorFormulaBuilder RegressorFormulaBuilder_t;
public:
	COM_SUBSCRIPTION_FUNCTION(WaterLevelRegressorAdded, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(TemperatureRegressorAdded, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(RegressorRemoved, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(RegressionModelDefinitionAdded, communication::Signal<void(boost::shared_ptr<RegressionModelDefinition_t>)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressionModelDefinition, communication::Signal<void(boost::shared_ptr<RegressionModelDefinition_t>)>);
	COM_SUBSCRIPTION_FUNCTION(RegressorFormulaBuilderAdded, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder_t>)>);
};
} // namespace detail

class IRegressionFormula : public virtual detail::RegressionFormulaSignals {
public:
	typedef IRegressionModelTemplate RegressionModelTemplate_t;
	typedef IRegressionModelDefinition RegressionModelDefinition_t;
	typedef IExpressionRegressorFormulaBuilder RegressorFormulaBuilder_t;
	typedef IAssignmentCategory AssignmentCategory_t;
	typedef IRegressor Regressor_t;
	typedef IDataFrame DataFrame_t;
	typedef IDataFrame::Influence_Factor_t InfluenceFactor_t;
public:
	IRegressionFormula(){;}
	virtual ~IRegressionFormula(){;}

	virtual void setDataFrame(boost::shared_ptr<DataFrame_t> dataFrame) = 0;

	virtual IDataFrame::Column_Container_t getInvolvedColumns() = 0;

	virtual void setBehaviorIndicatorColumn(boost::shared_ptr<IDataFrame::Behavior_Indicator_Column_t> behaviorIndicatorColumn) = 0;


	virtual const boost::shared_ptr<IDataFrame::Behavior_Indicator_Column_t> &getBehaviorIndicatorColumn() const = 0;

	virtual Rcpp::Formula buildFormula() = 0;

	virtual Rcpp::Formula buildFormulaForSingleRegressor(boost::shared_ptr<Regressor_t> regressor) = 0;

	virtual Rcpp::Formula buildFormulaForInfluenceFactorWithinAssignmentCategory(boost::shared_ptr<InfluenceFactor_t> influenceFactor, boost::shared_ptr<AssignmentCategory_t> assignmentCategory) = 0;


	virtual Rcpp::Formula buildFormulaForAssignmentCategory(boost::shared_ptr<AssignmentCategory_t> assignmentCategory) = 0;


	virtual int calculateOverallMaxStartupTime() = 0;

	virtual QString resolveRFormulaFor(boost::shared_ptr<Regressor_t> regressor) = 0;

	virtual std::string buildFormulaAsRStringForSingleRegressor(boost::shared_ptr<Regressor_t> regressor, QString delimiter = "+") = 0;

	virtual std::string buildFormulaAsRString(QString delimiter = "+") = 0;

	virtual std::string buildFormulaAsRStringForInfluenceFactorWithinAssignmentCategory(boost::shared_ptr<InfluenceFactor_t> influenceFactor, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, QString delimiter = "+") = 0;

	virtual std::string buildFormulaAsRStringForAssignmentCategory(boost::shared_ptr<AssignmentCategory_t> assignmentCategory, QString delimiter = "+") = 0;

	virtual QString toString(QString delimiter = " + ") = 0;

	virtual std::vector<boost::shared_ptr<RegressionModelTemplate_t> > getMethodDescriptions() = 0;

	/**
	 * Primarly used by the serialization process
	 */
	virtual const std::vector<boost::shared_ptr<RegressionModelTemplate_t> > &getAllMethodDescriptionsRegardlessOfTheMode() const = 0;

	/**
	 * Primarly used by the deserialization process
	 */
	virtual void setMethodDescriptions(std::vector<boost::shared_ptr<RegressionModelTemplate_t> > &methodDescriptions) = 0;

	/**
	 * Primarly used by the deserialization process
	 */
	virtual void setCustomRegressionModelDefinitions(std::vector<boost::shared_ptr<RegressionModelDefinition_t> > &regressionModelDefinitions) = 0;

	virtual void addCustomRegressionModelDefinition(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) = 0;

	/**
	 * Primarly used by the deserialization process
	 */
	virtual void setTemplatedRegressionModelDefinitions(std::vector<boost::shared_ptr<RegressionModelDefinition_t> > &regressionModelDefinitions) = 0;

	virtual void addTemplatedRegressionModelDefinition(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) = 0;

	virtual void removeRegressionModelDefinition(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) = 0;

	virtual const std::vector<boost::shared_ptr<RegressionModelDefinition_t> > &getCustomRegressionModelDefinitions() const = 0;

	virtual const std::vector<boost::shared_ptr<RegressionModelDefinition_t> > &getTemplatedRegressionModelDefinitions() const = 0;

	virtual std::vector<boost::shared_ptr<RegressionModelDefinition_t> > getAllRegressionModelDefinitions() = 0;

	virtual void releaseRegressionModelDefinitionForModification(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setRegressorFormulaBuilders(std::vector<boost::shared_ptr<RegressorFormulaBuilder_t> > &regressorFormulaBuilders) = 0;

	virtual void addRegressorFormulaBuilder(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) = 0;

	virtual const std::vector<boost::shared_ptr<RegressorFormulaBuilder_t> > &getRegressorFormulaBuilders() const = 0;

	virtual boost::shared_ptr<RegressorFormulaBuilder_t> findRegressorFormulaBuilderByIdentifier(QString identifier) = 0;

	virtual boost::shared_ptr<RegressorFormulaBuilder_t> findRegressorFormulaBuilderByFormula(QString formula) = 0;

	virtual QString getNewIdentifierFor(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) = 0;

	virtual QString getNewFormulaFor(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) = 0;

	virtual std::vector<boost::shared_ptr<Regressor_t> > getActiveRegressors() = 0;

	virtual std::vector<boost::shared_ptr<Regressor_t> > getRegressors() = 0;

	virtual boost::shared_ptr<Regressor_t> findRegressorByIndex(int index) = 0;

	virtual boost::shared_ptr<Regressor_t> findRegressorByFormula(QString formula) = 0;

	virtual std::vector<boost::shared_ptr<AssignmentCategory_t> > getInvolvedAssignmentCategoriesWithAnInfluenceFactor() = 0;

	virtual std::vector<boost::shared_ptr<AssignmentCategory_t> > findAssignmentCategoriesFor(std::vector<boost::shared_ptr<Regressor_t> > regressors) = 0;

	/**
	 * Returns a vector containing all (unique) influence factors of all regressors being assigned to a given assignmentcategory
	 */
	virtual std::vector<boost::shared_ptr<InfluenceFactor_t> > findInfluenceFactorsOfRegressorsAssignedTo(std::vector<boost::shared_ptr<Regressor_t> > regressors, boost::shared_ptr<AssignmentCategory_t> assignmentCategory) = 0;

	virtual boost::shared_ptr<RegressionModelDefinition_t> getRegressionModelDefinitionForRegressor(boost::shared_ptr<Regressor_t> regressor) = 0;

	virtual bool containsRegressor(boost::shared_ptr<Regressor_t> regressor) = 0;

	virtual const boost::shared_ptr<DataFrame_t> &getDataFrame() const = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace


#endif /* IREGRESSIONFORMULA_HPP_ */
