/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionModelDefinition.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONMODELDEFINITION_HPP_
#define IREGRESSIONMODELDEFINITION_HPP_

#include <boost/shared_ptr.hpp>
#include "IDataFrame.hpp"
#include "IRegressor.hpp"
#include "IExpressionRegressorFormulaBuilder.hpp"
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

namespace detail {

class RegressionModelDefinitionSignals : public communication::BoostSignalPolicy {
public:
	typedef IRegressor Regressor_t;
	typedef IExpressionRegressorFormulaBuilder RegressorFormulaBuilder_t;
public:
	RegressionModelDefinitionSignals(){;}
	virtual ~RegressionModelDefinitionSignals(){;}

	COM_SUBSCRIPTION_FUNCTION(RegressorAdded, communication::Signal<void(boost::shared_ptr<Regressor_t>)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressor, communication::Signal<void(boost::shared_ptr<Regressor_t>)>);
	COM_SUBSCRIPTION_FUNCTION(RegressorFormulaBuilderAdded, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder_t>)>);
	COM_SUBSCRIPTION_FUNCTION(ModificationChanged, communication::Signal<void()>);

	void fireRegressorAdded(boost::shared_ptr<Regressor_t> regressor) {
		communication::BoostSignalPolicy::fire();
		RegressorAddedSignal(regressor);
	}

	void fireRemoveRegressor(boost::shared_ptr<Regressor_t> regressor) {
		communication::BoostSignalPolicy::fire();
		RemoveRegressorSignal(regressor);
	}

	void fireRegressorFormulaBuilderAdded(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) {
		communication::BoostSignalPolicy::fire();
		RegressorFormulaBuilderAddedSignal(regressorFormulaBuilder);
	}

	void fireModificationChanged() {
		communication::BoostSignalPolicy::fire();
		ModificationChangedSignal();
	}

	void fireRemove() {
		communication::BoostSignalPolicy::fire();

	}
};

} // detail

class IRegressionModelDefinition : public detail::RegressionModelDefinitionSignals {
public:
	typedef IRegressor Regressor_t;
	typedef IExpressionRegressorFormulaBuilder RegressorFormulaBuilder_t;
	typedef IDataFrame DataFrame_t;
public:
	IRegressionModelDefinition(){;}
	virtual ~IRegressionModelDefinition(){;}

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setDataFrame(boost::shared_ptr<IRegressionModelDefinition::DataFrame_t> dataFrame) = 0;

	/**
	 * This is primarly used by the deserialization process. For other purposes, use #freezeModification
	 */
	virtual void setIsEditable(bool iseditable) = 0;

	virtual const bool &isEditable() const = 0;

	virtual void freezeModification() = 0;

	virtual void releaseForModification(std::vector<boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> > additionalFormulaBuilders) = 0;

	virtual void setName(QString name) = 0;

	virtual const QString &getName() const = 0;

	virtual const std::vector<boost::shared_ptr<IRegressionModelDefinition::Regressor_t> > &getRegressors() const = 0;

	/**
	 * This is primarly used by deserialization process. For other purposes, use #addRegressorFormulaBuilders
	 */
	virtual void setRegressorFormulaBuilders(std::vector<boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> > &formulaBuilders) = 0;

	virtual void addRegressorFormulaBuilders(std::vector<boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> > formulaBuilders) = 0;

	virtual void addRegressorFormulaBuilder(boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder) = 0;

	virtual void removeRegressorFormulaBuilder(boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder) = 0;

	virtual const std::vector<boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> > &getRegressorFormulaBuilders() const = 0;

	virtual boost::shared_ptr<IRegressionModelDefinition::RegressorFormulaBuilder_t> findRegressorFormulaBuilderByIdentifier(QString identifier) = 0;

	/**
	 * This is primary used by the deserialization process
	 */
	virtual void setRegressors(std::vector<boost::shared_ptr<IRegressionModelDefinition::Regressor_t> > regressors) = 0;

	virtual void addRegressor(boost::shared_ptr<IRegressionModelDefinition::Regressor_t> regressor) = 0;

	virtual bool containsRegressor(boost::shared_ptr<IRegressionModelDefinition::Regressor_t> regressor) = 0;

	virtual void removeRegressor(boost::shared_ptr<IRegressionModelDefinition::Regressor_t> regressor) = 0;

	virtual const boost::shared_ptr<IRegressionModelDefinition::DataFrame_t> &getDataFrame() const = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace

#endif /* IREGRESSIONMODELDEFINITION_HPP_ */
