/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * SeasonalFormulaBuilderDefinition.hpp
 *
 *  Created on: Aug 7, 2014
 *      Author: gerberma
 */

#ifndef SEASONALFORMULABUILDERDEFINITION_HPP_
#define SEASONALFORMULABUILDERDEFINITION_HPP_

#include <QString>
#include <boost/foreach.hpp>
#include <boost/algorithm/string/join.hpp>
#include <boost/shared_ptr.hpp>

namespace dambase {
namespace model {
namespace r {

template<typename UserParameterType>
class SeasonalFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	SeasonalFormulaBuilderDefinition(bool isSinus, int factor) : factor(factor) {
		if(isSinus) {
			seasonalName = "sin";
		}else{
			seasonalName = "cos";
		}

		userParameters["factor"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(factor), false));
	}
	virtual ~SeasonalFormulaBuilderDefinition(){;}

	QString getIdentifier() {
		return seasonalName + QString("(") + QString::number(factor) + QString("s)");
	}

	QString getFormula() {
		return getIdentifier();
	}

	QString getFormulaDefinition() {
		return QString("dambase.seasonal.") + seasonalName + QString("(date_, factor=arg$factor)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	QString seasonalName;
	int factor;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class ChebyshevFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	ChebyshevFormulaBuilderDefinition(int degree) : degree(degree) {
	}

	virtual ~ChebyshevFormulaBuilderDefinition(){;}

	QString getIdentifier() {
		return QString("Chebyshev polynom degree ") + QString::number(degree);
	}

	QString getFormula() {
		return QString("T") + QString::number(degree) + QString("($)");
	}

	QString getFormulaDefinition() {
		return QString("dambase.T") + QString::number(degree) + QString("(col_, container=arg$container)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}
private:
	int degree;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class PowerFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	PowerFormulaBuilderDefinition(int degree) : degree(degree) {
		userParameters["degree"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(degree), false));
	}

	virtual ~PowerFormulaBuilderDefinition(){;}

	QString getIdentifier() {
		return QString("Polynom degree ") + QString::number(degree);
	}

	QString getFormula() {
		QString formula = "h($)";

		if(degree > 1) {
			formula += QString("^") + QString::number(degree);
		}

		return formula;
	}

	QString getFormulaDefinition() {
		return QString("dambase.P(col_, degree=arg$degree, container=arg$container)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	int degree;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class ExponentialBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	ExponentialBuilderDefinition(int divisor) : divisor(divisor) {
		userParameters["divisor"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(divisor), false));
	}

	virtual ~ExponentialBuilderDefinition(){;}

	QString getIdentifier() {
		QString identifier = QString("Exponential");

		if(divisor > 1) {
			identifier += QString(" with divisor ") + QString::number(divisor);
		}

		return identifier;
	}

	QString getFormula() {
		QString formula = "exp(-t";
		if(divisor > 1) {
			formula = formula + QString("/") + QString::number(divisor);
		}
		formula = formula + QString(")");

		return formula;
	}

	QString getFormulaDefinition() {
		return QString("dambase.drift.exp(date_, divisor=arg$divisor)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	int divisor;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class SquareRootBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	SquareRootBuilderDefinition(){;}
	virtual ~SquareRootBuilderDefinition(){;}

	QString getIdentifier() {
		return QString("Square root");
	}

	QString getFormula() {
		return QString("sqrt(t)");
	}

	QString getFormulaDefinition() {
		return QString("dambase.drift.sqrt(date_)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};

template<typename UserParameterType>
class LogarithmusNaturalisBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	LogarithmusNaturalisBuilderDefinition(){;}
	virtual ~LogarithmusNaturalisBuilderDefinition(){;}

	QString getIdentifier() {
		return QString("ln");
	}

	QString getFormula() {
		return QString("ln(1+t)");
	}

	QString getFormulaDefinition() {
		return QString("dambase.drift.ln(date_)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class DriftPowerBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	DriftPowerBuilderDefinition(int power):power(power){
		userParameters["power"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(power), false));
	}

	virtual ~DriftPowerBuilderDefinition(){;}

	QString getIdentifier() {
		if(power == 1) {
			return QString("direct");
		}
		/*
		if(power == 2) {
			return QString("square");
		}
		*/

		QString formula ("t");
		if(power > 1) {
			formula += QString("^") + QString::number(power);
		}

		return formula;
	}

	QString getFormula() {
		QString formula ("t");
		if(power > 1) {
			formula += QString("^") + QString::number(power);
		}

		return formula;
	}

	QString getFormulaDefinition() {
		return QString("dambase.drift.power(date_, power=arg$power)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	int power;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};

template<
	typename UserParameterType,
	typename WaterLevelFunction>
class CreepFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	CreepFormulaBuilderDefinition(boost::shared_ptr<WaterLevelFunction> waterLevelFunction, double creepConstant, double convolutionTime, int startupTime)
		:waterLevelFunction(waterLevelFunction), creepConstant(creepConstant), convolutionTime(convolutionTime), startupTime(startupTime)
	{
		userParameters = waterLevelFunction->getUserParameters();

		userParameters["creepConstant"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(creepConstant), false));
		userParameters["convolutionTime"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(convolutionTime), false));
		userParameters["startup"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(startupTime), true));
	}

	QString getIdentifier() {
		return QString("Creep with ") + waterLevelFunction->getIdentifier();
	}

	QString getFormula() {
		return QString("Creep(") + waterLevelFunction->getFormula() + QString(")");
	}

	QString getFormulaDefinition() {
		return QString("dambase.creep(wlevelPreCalculated=")
				+ waterLevelFunction->getFormulaDefinition()
				+ QString(", dateColumn=date_, creepConstant=arg$creepConstant, convolutionTime=arg$convolutionTime)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	boost::shared_ptr<WaterLevelFunction> waterLevelFunction;
	double creepConstant;
	double convolutionTime;
	int startupTime;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};

template<typename UserParameterType>
class ExtraSqrtRampBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	ExtraSqrtRampBuilderDefinition(int subtrahend):subtrahend(subtrahend){
		userParameters["subtrahend"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(subtrahend), false));
	}

	virtual ~ExtraSqrtRampBuilderDefinition(){;}

	QString getIdentifier() {
		return QString("sqrt(ramp($-") + QString::number(subtrahend) + QString("))");
	}

	QString getFormula(){
		return QString("sqrt(ramp($-") + QString::number(subtrahend) + QString("))");
	}

	QString getFormulaDefinition() {
		return QString("dambase.extra.sqrtramp(col_, subtrahend=arg$subtrahend)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	int subtrahend;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};

template<typename UserParameterType>
class ExtraColumnTimesStepBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	ExtraColumnTimesStepBuilderDefinition(int subtrahend):subtrahend(subtrahend){
		userParameters["subtrahend"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(subtrahend), false));
	}

	virtual ~ExtraColumnTimesStepBuilderDefinition(){;}

	QString getIdentifier() {
		return QString("$*step($-") + QString::number(subtrahend) + QString(")");
	}

	QString getFormula(){
		return QString("$*step($-") + QString::number(subtrahend) + QString(")");
	}

	QString getFormulaDefinition() {
		return QString("dambase.extra.columntimesstep(col_, subtrahend=arg$subtrahend)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	int subtrahend;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class TemperatureOneDimensionalHeatEquation {
public:
	typedef UserParameterType UserParameterType_t;
public:
	TemperatureOneDimensionalHeatEquation(int depth, double thermalDiffusivity, int convolutionTimeSteps, int startupTime)
		:depth(depth),
		 thermalDiffusivity(thermalDiffusivity),
		 convolutionTimeSteps(convolutionTimeSteps),
		 startupTime(startupTime)
	{
		userParameters["startup"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(startupTime), true));
		userParameters["diffusivity"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(thermalDiffusivity), false));
		userParameters["convolution_time_steps"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(convolutionTimeSteps), false));
		userParameters["depth"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(depth), false));
	}

	virtual ~TemperatureOneDimensionalHeatEquation(){;}

	QString getIdentifier() {
		return QString("1d heat equation on depth ") + QString::number(depth);
	}

	QString getFormula() {
		return QString("temp1d($-") + QString::number(depth) + QString(")");
	}

	QString getFormulaDefinition() {
		return QString("dambase.temp.thermal.oned(date_, col_, depth=arg$depth, thermalDiffusivity=arg$diffusivity, convolutionTime=arg$convolution_time_steps)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	int depth;
	double thermalDiffusivity;
	int convolutionTimeSteps;
	int startupTime;
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class SplineExpressionFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	SplineExpressionFormulaBuilderDefinition(std::vector<double> x, std::vector<double> y) {
		userParameters["x"] = boost::shared_ptr<UserParameterType>(new UserParameterType(build(x), false));
		userParameters["y"] = boost::shared_ptr<UserParameterType>(new UserParameterType(build(y), false));
	}

	virtual ~SplineExpressionFormulaBuilderDefinition() {;}

	QString getIdentifier() {
		return QString("spline");
	}

	QString getFormula() {
		return QString("spline($)");
	}

	QString getFormulaDefinition() {
		return QString("dambase.extra.spline(col_, x=arg$x, y=arg$y)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	QString build(std::vector<double> x) {
		using namespace boost;
		return QString("c(") + QString::fromStdString(algorithm::join(resolve(x), ", ")) + QString(")");
	}

	std::vector<std::string> resolve(std::vector<double> x) {
		std::vector<std::string> xasstrings;
		BOOST_FOREACH(double xvalue, x) {
			xasstrings.push_back(QString::number(xvalue).toStdString());
		}

		return xasstrings;
	}

private:
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
};


template<typename UserParameterType>
class AverageExpressionFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	AverageExpressionFormulaBuilderDefinition(int averageTimesteps) : averageTimesteps(averageTimesteps) {
		userParameters["averageTimesteps"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(averageTimesteps), true));
	}

	virtual ~AverageExpressionFormulaBuilderDefinition() {;}

	QString getIdentifier() {
		return QString("Average with ") + QString::number(averageTimesteps) + QString(" timesteps");
	}

	QString getFormula() {
		return QString("avg($, ") + QString::number(averageTimesteps) + QString(")");
	}

	QString getFormulaDefinition() {
		return QString("dambase.average(date_, col_, averageTimesteps=arg$averageTimesteps)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
	int averageTimesteps;
};

template<typename UserParameterType>
class DelayExpressionFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	DelayExpressionFormulaBuilderDefinition(int delayTimesteps) : delayTimesteps(delayTimesteps) {
		userParameters["delayTimesteps"] = boost::shared_ptr<UserParameterType>(new UserParameterType(QString::number(delayTimesteps), true));
	}

	virtual ~DelayExpressionFormulaBuilderDefinition() {;}

	QString getIdentifier() {
		return QString("Delay with ") + QString::number(delayTimesteps) + QString(" timesteps");
	}

	QString getFormula() {
		return QString("delay($, ") + QString::number(delayTimesteps) + QString(")");
	}

	QString getFormulaDefinition() {
		return QString("dambase.delay(col_, timesteps=arg$delayTimesteps)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
	int delayTimesteps;
};

template<typename UserParameterType>
class DirectExpressionFormulaBuilderDefinition {
public:
	typedef UserParameterType UserParameterType_t;
public:
	DirectExpressionFormulaBuilderDefinition(QString identifierSuffix, QString formulaSuffix) : identifierSuffix(identifierSuffix), formulaSuffix(formulaSuffix) {
	}

	virtual ~DirectExpressionFormulaBuilderDefinition() {;}

	QString getIdentifier() {
		return QString("direct ") + identifierSuffix;
	}

	QString getFormula() {
		return QString("direct_") + formulaSuffix + QString("($)");
	}

	QString getFormulaDefinition() {
		return QString("dambase.direct(col_)");
	}

	std::map<QString, boost::shared_ptr<UserParameterType> > getUserParameters() {
		return userParameters;
	}

private:
	std::map<QString, boost::shared_ptr<UserParameterType> > userParameters;
	QString identifierSuffix;
	QString formulaSuffix;
};

}  // namespace r
}  // namespace model
}  // namespace dambase

#endif /* SEASONALFORMULABUILDERDEFINITION_HPP_ */
