/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//#define BOOST_TEST_MAIN
#define BOOST_TEST_MODULE ApplicationContextTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <iostream>
#include <stdexcept>
#include "app/GenericApplicationContext.hpp"


class ClassA {
public:
	ClassA()
		:value(0)
	{}

	int get() {
		return value;
	}

	void set(int value) {
		this->value = value;
	}

	bool operator==(ClassA &other) {
		return value == other.value;
	}

	bool operator!=(ClassA &other) {
		return value != other.value;
	}

private:
	int value;
};

class ClassB {
};

class ClassC {
public:
	ClassC(ClassA *a)
		:a(a){}

	ClassA *get() {
		return a;
	}
private:
	ClassA *a;
};

ClassA *classAFactory() {
	return new ClassA;
}

ClassB *classBFactory() {
	return new ClassB;
}

ClassC *classCFactory() {
	using namespace app::context;
	return new ClassC(GenericApplicationContext<ClassA, ThrowLogicalErrorPolicy>::get());
}

void classACleanupStrategy(ClassA *a) {
	delete a;
}

void classBCleanupStrategy(ClassB *b) {
	delete b;
}

void classCCleanupStrategy(ClassC *c) {
	delete c;
}


BOOST_AUTO_TEST_CASE(TestResolveSameObjectFromSingleton)
{
	using namespace app::context;

	typedef GenericApplicationContext<ClassA> Generic_Application_Context_t;

	//const GenericApplicationContextScope<Generic_Application_Context_t > contextScope = registerCtxWithContextScope<Generic_Application_Context_t, ClassA>();
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(ClassA);

	ClassA *a = Generic_Application_Context_t::get();

	ClassA *aa = Generic_Application_Context_t::get();

	BOOST_CHECK(a == aa);
}


BOOST_AUTO_TEST_CASE(TestCauseLogicalErrorIfClassIsNotRegisteredForDefault)
{
	using namespace app::context;
	typedef GenericApplicationContext<ClassA> Generic_Application_Context_t;

	class Testcase {
	public:
		void operator()() {
			Generic_Application_Context_t::get();
		}
	};

	GenericApplicationContextScope<Generic_Application_Context_t> contextScope;
	Testcase testcase;

	BOOST_CHECK_THROW(testcase(), std::logic_error);
}


BOOST_AUTO_TEST_CASE(TestSeparateTypesResultInSeparateObjects)
{
	using namespace app::context;

	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(ClassA);
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(ClassB);

	ClassA *a = FETCH_FROM_CTX(ClassA);
	ClassB *b = FETCH_FROM_CTX(ClassB);

	BOOST_CHECK((ClassB*)a != b);
}

BOOST_AUTO_TEST_CASE(TestChainOfObjectInstantiation){
	using namespace app::context;

	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(ClassA);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(ClassC, classCFactory, classCCleanupStrategy);

	ClassC *c = FETCH_FROM_CTX(ClassC);
	BOOST_CHECK(c->get() != 0);

	ClassA *a = FETCH_FROM_CTX(ClassA);
	BOOST_CHECK(c->get() == a);
}

class Cleanup {
public:
	Cleanup():cleanupCalled(false){}

	void cleanup(ClassA *a) {
		cleanupCalled = true;
		delete a;
	}

	bool cleanupCalled;
};


BOOST_AUTO_TEST_CASE(TestCallCleanupFunction) {
	using namespace app::context;
	typedef GenericApplicationContext<ClassA> Generic_Application_Context_t;

	Cleanup cleanup;
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(ClassA, classAFactory, boost::bind(&Cleanup::cleanup, boost::ref(cleanup), _1));


	Generic_Application_Context_t::get();
	BOOST_CHECK(cleanup.cleanupCalled == false);

	Generic_Application_Context_t::releaseObject();
	BOOST_CHECK(cleanup.cleanupCalled == true);
}

BOOST_AUTO_TEST_CASE(TestCallCleanupWhenContextScopeReleases) {
	using namespace app::context;

	Cleanup cleanup;

	{
		REGISTER_FULL_CTX_WITH_CTX_SCOPE(ClassA, classAFactory, boost::bind(&Cleanup::cleanup, boost::ref(cleanup), _1));
		BOOST_CHECK(cleanup.cleanupCalled == false);
		FETCH_FROM_CTX(ClassA);
		BOOST_CHECK(cleanup.cleanupCalled == false);
	}

	BOOST_CHECK(cleanup.cleanupCalled == true);
}

