/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisControllerTests.cpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE AnalysisControllerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/shared_ptr.hpp>
#include "controller/AnalysisController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

MOCK_CLASS(NullStruct){};

MOCK_CLASS(Analysis) {
};

MOCK_CLASS(AnalysisFactory) {
public:
	template<typename T>
	boost::shared_ptr<T> create() {
		return createDelegate();
	}

	MOCK_METHOD_EXT(createDelegate, 0, boost::shared_ptr<Analysis>(), createDelegate)
};

MOCK_CLASS(AnalysisWizard) {
	MOCK_METHOD_EXT(start, 0, void(), start);
	MOCK_METHOD_EXT(accepted, 0, bool(), accepted);
	MOCK_METHOD_EXT(editWaterLevelScaling, 0, bool(), editWaterLevelScaling);
};


MOCK_CLASS(AnalysisOverviewUi) {
	MOCK_METHOD_EXT(show, 0, void(), show);
	MOCK_METHOD_EXT(subscribeForEdit, 1, void(const boost::any &), subscribeForEdit);
	MOCK_METHOD_EXT(subscribeForEditWaterLevelScaling, 1, void(const boost::any &), subscribeForEditWaterLevelScaling);
};


MOCK_CLASS(WorkingAreaContainer) {
	MOCK_METHOD_EXT(setWorkingArea, 1, void(AnalysisOverviewUi *), setWorkingArea);
};

MOCK_CLASS(AnalysisUiManager) {
	MOCK_METHOD_EXT(getOverviewUiFor, 1, AnalysisOverviewUi*(boost::shared_ptr<Analysis>), getOverviewUiFor);
	MOCK_METHOD_EXT(getWizardFor, 1, AnalysisWizard*(boost::shared_ptr<Analysis>), getWizardFor);


	void destroyWizardFor(boost::shared_ptr<Analysis>analysis) {
		delegateDestroyWizardFor(analysis);
	}

	MOCK_METHOD_EXT(delegateDestroyWizardFor, 1, void(boost::shared_ptr<Analysis>), delegateDestroyWizardFor);

public:
	typedef AnalysisOverviewUi overview_ui_type;
	typedef AnalysisWizard wizard_ui_type;
};

MOCK_CLASS(Project){
	MOCK_METHOD_EXT(addAnalysis, 1, void(boost::shared_ptr<Analysis>), addAnalysis);
};

MOCK_CLASS(ActivityTracker){
	MOCK_METHOD_EXT(trackChanged, 0, void(), trackChanged);
};


typedef Project Project_t;
typedef Analysis Analysis_t;
typedef AnalysisFactory Analysis_Factory_t;
typedef AnalysisWizard Analysis_Wizard_t;
typedef WorkingAreaContainer Working_Area_Container_t;
typedef AnalysisOverviewUi Analysis_Overview_Ui_t;
typedef AnalysisUiManager Analysis_Ui_Manager_t;
typedef ActivityTracker Activity_Tracker_t;
typedef controller::AnalysisController<
		Analysis_t,
		Working_Area_Container_t,
		Analysis_Ui_Manager_t,
		Activity_Tracker_t
		> Analysis_Controller_t;

namespace test {

using namespace helper;


class ApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Analysis_Controller_t, boost::bind(&ApplicationContext::analysisControllerFactory, this)),
		GAC_DEFAULT_CTOR_INIT(Analysis_Factory_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Wizard_t),
		GAC_DEFAULT_CTOR_INIT(Working_Area_Container_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Ui_Manager_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Overview_Ui_t)
	{
		project = boost::shared_ptr<Project_t>(new Project_t);
		analysis = boost::shared_ptr<Analysis_t>(new Analysis_t);
		activityTracker = boost::shared_ptr<Activity_Tracker_t>(new Activity_Tracker_t);
	}

private:
	Analysis_Controller_t *analysisControllerFactory(){
		Analysis_Controller_t *analysisController = new Analysis_Controller_t(
				FETCH_FROM_CTX(Working_Area_Container_t),
				FETCH_FROM_CTX(Analysis_Ui_Manager_t),
				activityTracker
				);

		return analysisController;
	}

public:
	GAC_SCOPE(Analysis_Controller_t)
	GAC_SCOPE(Analysis_Factory_t)
	GAC_SCOPE(Analysis_Wizard_t)
	GAC_SCOPE(Working_Area_Container_t)
	GAC_SCOPE(Analysis_Ui_Manager_t)
	boost::shared_ptr<Project_t> project;
	boost::shared_ptr<Analysis_t> analysis;
	boost::shared_ptr<Activity_Tracker_t> activityTracker;
	GAC_SCOPE(Analysis_Overview_Ui_t)
};
}


BOOST_FIXTURE_TEST_CASE(CreateForProjectSetsAnalysisIfWizardTerminatesWithAccept, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Factory_t)->createDelegate).once().returns(analysis);

	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Manager_t)->getWizardFor).at_least(1).with(analysis).returns(FETCH_FROM_CTX(Analysis_Wizard_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Manager_t)->delegateDestroyWizardFor).at_least(1).with(analysis);
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Wizard_t)->start).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Wizard_t)->accepted).at_least(1).returns(true);
	MOCK_EXPECT(project->addAnalysis).once().with(analysis);

	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Manager_t)->getOverviewUiFor).at_least(1).with(analysis).returns(FETCH_FROM_CTX(Analysis_Overview_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Overview_Ui_t)->subscribeForEdit).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Overview_Ui_t)->subscribeForEditWaterLevelScaling).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Overview_Ui_t)->show).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Analysis_Overview_Ui_t));
	MOCK_EXPECT(activityTracker->trackChanged).once();

	Analysis_Controller_t *analysisController = FETCH_FROM_CTX(Analysis_Controller_t);
	analysisController->createFor(project, FETCH_FROM_CTX(Analysis_Factory_t));
}

BOOST_FIXTURE_TEST_CASE(CreateForProjectDoesNotSetAnalysisIfWizardTerminatesWithNotAccept, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Factory_t)->createDelegate).once().returns(analysis);

	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Manager_t)->getWizardFor).at_least(1).with(analysis).returns(FETCH_FROM_CTX(Analysis_Wizard_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Wizard_t)->start).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Wizard_t)->accepted).at_least(1).returns(false);
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Manager_t)->delegateDestroyWizardFor).at_least(1).with(analysis);
	MOCK_EXPECT(project->addAnalysis).never();

	Analysis_Controller_t *analysisController = FETCH_FROM_CTX(Analysis_Controller_t);
	analysisController->createFor(project, FETCH_FROM_CTX(Analysis_Factory_t));
}
