/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericControllerTests.cpp
 *
 *  Created on: Feb 21, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE GenericControllerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <string>
#include "controller/GenericController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"
#include "memory/OwnerhshipPolicies.hpp"

using namespace app::context;

MOCK_CLASS(AnyModel) {
};

MOCK_CLASS(ParentContainer) {
	void addRegressionModel(AnyModel *regressionModel) {
		delegateAddRegressionModel(regressionModel);
	}
	MOCK_METHOD_EXT(delegateAddRegressionModel, 1, void(AnyModel *), delegateAddRegressionModel);
};

MOCK_CLASS(AnyModelFactory) {
	AnyModel *create() {
		return delegateCreate();
	}

	MOCK_METHOD_EXT(delegateCreate, 0, AnyModel*(), delegateCreate);
};

MOCK_CLASS(PostCreateHandler) {
	void doAnything(AnyModel *anyModel) {
		delegateDoAnything(anyModel);
	}
	MOCK_METHOD_EXT(delegateDoAnything, 1, void(AnyModel *), delegateDoAnything);
};


MOCK_CLASS(Wizard) {
	MOCK_METHOD_EXT(start, 0, void(), start);
	MOCK_METHOD_EXT(accepted, 0, bool(), accepted);
};

MOCK_CLASS(OverviewUi) {
	MOCK_METHOD_EXT(show, 0, void(), show);
};

MOCK_CLASS(WorkingAreaContainerUi) {
	MOCK_METHOD_EXT(setWorkingArea, 1, void(OverviewUi *), setWorkingArea);
};

MOCK_CLASS(SubscriptionManager) {
};

MOCK_CLASS(UiManagerWithWizardAndWorkingAreaContainer) {
	typedef Wizard wizard_ui_t;
	typedef OverviewUi overview_ui_t;
	typedef WorkingAreaContainerUi working_area_container_ui_t;

	void destroyWizardFor(AnyModel *regressionModel) {
		delegateDestroyWizardFor(regressionModel);
	}

	MOCK_METHOD_EXT(getWizardFor, 2, Wizard *(AnyModel *, SubscriptionManager *), getWizardFor);
	MOCK_METHOD_EXT(delegateDestroyWizardFor, 1, void(AnyModel*), delegateDestroyWizardFor);

	MOCK_METHOD_EXT(getOverviewUiFor, 2, OverviewUi*(AnyModel*, SubscriptionManager *), getOverviewUiFor);
	MOCK_METHOD_EXT(getWorkingAreaContainerUi, 0, WorkingAreaContainerUi*(), getWorkingAreaContainerUi);
};

MOCK_CLASS(UiManagerWithNoFeaturesEnabled) {
	typedef OverviewUi overview_ui_t;

	MOCK_METHOD_EXT(getOverviewUiFor, 2, OverviewUi*(AnyModel*, SubscriptionManager *), getOverviewUiFor);
};


typedef controller::GenericController<AnyModel, UiManagerWithWizardAndWorkingAreaContainer, memory::CustomOwnershipPolicy<AnyModel, memory::IgnoreOnDeleteChildPolicy> > Generic_Controller_All_Features_Enabled_t;
typedef controller::GenericController<AnyModel, UiManagerWithNoFeaturesEnabled, memory::CustomOwnershipPolicy<AnyModel, memory::IgnoreOnDeleteChildPolicy> > Generic_Controller_All_Features_Disabled_t;

namespace test {
using namespace helper;
class IndependentApplicationContext {
public:
	ParentContainer parentContainer;
	AnyModel anyModel;
	AnyModelFactory anyModelFactory;
	PostCreateHandler postCreateHandler;
	OverviewUi overviewUi;
	WorkingAreaContainerUi workingAreaContainerUi;
	SubscriptionManager subscriptionManager;
};

class ApplicationContext : public IndependentApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Generic_Controller_All_Features_Enabled_t, boost::bind(&ApplicationContext::genericControllerFactoryAllFeaturesEnabled, this))
		,GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Generic_Controller_All_Features_Disabled_t, boost::bind(&ApplicationContext::genericControllerFactoryAllFeaturesDisabled, this))
		,GAC_DEFAULT_CTOR_INIT(Wizard)
	{;}

private:
	Generic_Controller_All_Features_Enabled_t *genericControllerFactoryAllFeaturesEnabled() {
		return new Generic_Controller_All_Features_Enabled_t(&uiManagerAllEnabled);
	}

	Generic_Controller_All_Features_Disabled_t *genericControllerFactoryAllFeaturesDisabled() {
		return new Generic_Controller_All_Features_Disabled_t(&uiManagerAllDisabled);
	}

private:
	GAC_SCOPE(Generic_Controller_All_Features_Enabled_t)
	GAC_SCOPE(Generic_Controller_All_Features_Disabled_t)
	GAC_SCOPE(Wizard)
public:
	UiManagerWithWizardAndWorkingAreaContainer uiManagerAllEnabled;
	UiManagerWithNoFeaturesEnabled uiManagerAllDisabled;
};

} /* namespace test */


BOOST_FIXTURE_TEST_CASE(CreateModelWithAcceptedWizardAndConfiguredWorkingAreaContainer, test::ApplicationContext)
{
	using namespace boost;

	MOCK_EXPECT(anyModelFactory.delegateCreate).once().returns(&anyModel);
	MOCK_EXPECT(postCreateHandler.delegateDoAnything).once().with(&anyModel);
	MOCK_EXPECT(uiManagerAllEnabled.getWizardFor).once().with(&anyModel, &subscriptionManager).returns(FETCH_FROM_CTX(Wizard));
	MOCK_EXPECT(uiManagerAllEnabled.delegateDestroyWizardFor).once().with(&anyModel);
	//MOCK_EXPECT(subscriptionManager.delegateSubscribeWizard).once().with(FETCH_FROM_CTX(Wizard));
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard)->start).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard)->accepted).once().returns(true);
	MOCK_EXPECT(parentContainer.delegateAddRegressionModel).once().with(&anyModel);
	MOCK_EXPECT(uiManagerAllEnabled.getOverviewUiFor).once().with(&anyModel, &subscriptionManager).returns(&overviewUi);
	MOCK_EXPECT(overviewUi.show).once();
	MOCK_EXPECT(uiManagerAllEnabled.getWorkingAreaContainerUi).once().returns(&workingAreaContainerUi);
	MOCK_EXPECT(workingAreaContainerUi.setWorkingArea).once().with(&overviewUi);

	FETCH_FROM_CTX(Generic_Controller_All_Features_Enabled_t)->createFor(
			bind(&ParentContainer::addRegressionModel, ref(parentContainer), _1),
			bind(&AnyModelFactory::create, ref(anyModelFactory)),
			&subscriptionManager,
			bind(&PostCreateHandler::doAnything, ref(postCreateHandler), _1));
}


BOOST_FIXTURE_TEST_CASE(DoNotCreateModelIfWizardHasNotAccepted, test::ApplicationContext) {
	using namespace boost;

	MOCK_EXPECT(anyModelFactory.delegateCreate).once().returns(&anyModel);
	MOCK_EXPECT(postCreateHandler.delegateDoAnything).once().with(&anyModel);
	MOCK_EXPECT(uiManagerAllEnabled.getWizardFor).once().with(&anyModel, &subscriptionManager).returns(FETCH_FROM_CTX(Wizard));
	//MOCK_EXPECT(subscriptionManager.delegateSubscribeWizard).once().with(FETCH_FROM_CTX(Wizard));
	MOCK_EXPECT(uiManagerAllEnabled.delegateDestroyWizardFor).once().with(&anyModel);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard)->start).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard)->accepted).once().returns(false);

	FETCH_FROM_CTX(Generic_Controller_All_Features_Enabled_t)->createFor(
			bind(&ParentContainer::addRegressionModel, ref(parentContainer), _1),
			bind(&AnyModelFactory::create, ref(anyModelFactory)),
			&subscriptionManager,
			bind(&PostCreateHandler::doAnything, ref(postCreateHandler), _1));
}

BOOST_FIXTURE_TEST_CASE(CreateModelWithAllFeaturesDisabled, test::ApplicationContext)
{
	using namespace boost;

	MOCK_EXPECT(anyModelFactory.delegateCreate).once().returns(&anyModel);
	MOCK_EXPECT(postCreateHandler.delegateDoAnything).once().with(&anyModel);
	MOCK_EXPECT(parentContainer.delegateAddRegressionModel).once().with(&anyModel);
	MOCK_EXPECT(uiManagerAllDisabled.getOverviewUiFor).once().with(&anyModel, &subscriptionManager).returns(&overviewUi);
	MOCK_EXPECT(overviewUi.show).once();

	FETCH_FROM_CTX(Generic_Controller_All_Features_Disabled_t)->createFor(
			bind(&ParentContainer::addRegressionModel, ref(parentContainer), _1),
			bind(&AnyModelFactory::create, ref(anyModelFactory)),
			&subscriptionManager,
			bind(&PostCreateHandler::doAnything, ref(postCreateHandler), _1));
}

BOOST_FIXTURE_TEST_CASE(TestDefaultPostCreateCallbackWithAllFeaturesDisabledButThisDoesNotMatterAndOnlySimplifiesTestCaseComplexity, test::ApplicationContext)
{
	using namespace boost;

	MOCK_EXPECT(anyModelFactory.delegateCreate).once().returns(&anyModel);
	MOCK_EXPECT(parentContainer.delegateAddRegressionModel).once().with(&anyModel);
	MOCK_EXPECT(uiManagerAllDisabled.getOverviewUiFor).once().with(&anyModel, &subscriptionManager).returns(&overviewUi);
	MOCK_EXPECT(overviewUi.show).once();

	FETCH_FROM_CTX(Generic_Controller_All_Features_Disabled_t)->createFor(
			bind(&ParentContainer::addRegressionModel, ref(parentContainer), _1),
			bind(&AnyModelFactory::create, ref(anyModelFactory)),
			&subscriptionManager);
}

BOOST_FIXTURE_TEST_CASE(ShowWithWorkingAreaEnabled, test::ApplicationContext) {
	using namespace boost;

	MOCK_EXPECT(uiManagerAllEnabled.getOverviewUiFor).once().with(&anyModel, &subscriptionManager).returns(&overviewUi);
	MOCK_EXPECT(overviewUi.show).once();
	MOCK_EXPECT(uiManagerAllEnabled.getWorkingAreaContainerUi).once().returns(&workingAreaContainerUi);
	MOCK_EXPECT(workingAreaContainerUi.setWorkingArea).once().with(&overviewUi);

	FETCH_FROM_CTX(Generic_Controller_All_Features_Enabled_t)->show(&anyModel, &subscriptionManager);
}
