/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectControllerTests.cpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE ProjectControllerTests
#include <boost/test/unit_test.hpp>
#include <QString>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include "controller/ProjectController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"
#include <exception>
#include <boost/bind.hpp>
#include <boost/any.hpp>

using namespace app::context;

MOCK_CLASS(NullStruct){};

MOCK_CLASS(Project){
	MOCK_METHOD_EXT(setParent, 1, void(QObject *), setParent);
};

MOCK_CLASS(Model) {
	MOCK_METHOD_EXT(setProject, 1, void(Project *), setProject);
	MOCK_METHOD_EXT(getProject, 0, Project*(), getProject);
};

MOCK_CLASS(ProjectWizard) {
	MOCK_METHOD_EXT(start, 1, void(Project*), start)
	MOCK_METHOD_EXT(editCommonProperties, 1, void(Project*), editCommonProperties)
};

MOCK_CLASS(ProjectFactory) {
public:
	template<typename T>
	T *create() {
		return createDelegate();
	}

	MOCK_METHOD_EXT(createDelegate, 0, Project*(), createDelegate)
};

MOCK_CLASS(ProjectOverviewUi) {
	MOCK_METHOD_EXT(show, 1, void(Project*), show)
};

MOCK_CLASS(NavigationUi) {
	MOCK_METHOD_EXT(show, 1, void(Project*), show)
};

MOCK_CLASS(WorkingAreaContainer) {
	MOCK_METHOD_EXT(setWorkingArea, 1, void(ProjectOverviewUi *), setWorkingArea);
	MOCK_METHOD_EXT(setNavigationArea, 1, void(NavigationUi *), setNavigationArea);
};

MOCK_CLASS(SerializationPolicy) {
	MOCK_METHOD_EXT(serialize, 2, void(Project*, QString), serialize);
	MOCK_METHOD_EXT(deserialize, 2, void(Project*, QString), deserialize);
};

MOCK_CLASS(FileDialogUi) {
	MOCK_STATIC_METHOD(getSaveFileName, 1, QString(QString), getSaveFileName);
	MOCK_STATIC_METHOD(getOpenFileName, 1, QString(QString), getOpenFileName);
};

MOCK_CLASS(Loader) {
	MOCK_METHOD_EXT(execute, 1, void(Project *), execute);
};

MOCK_CLASS(ExceptionPolicy) {
	MOCK_STATIC_METHOD(handleSerializationException, 2, void(mpl::Type2Type<Project>, QString), handleSerializationException);
	MOCK_STATIC_METHOD(handleDeserializationException, 2, void(mpl::Type2Type<Project>, QString), handleDeserializationException);
	MOCK_STATIC_METHOD(wrongCallOrder, 3, bool(void*, QString, QString), wrongCallOrder);
};

typedef Project Project_t;
typedef ProjectFactory Project_Factory_t;
typedef ProjectWizard Project_Wizard_t;
typedef ProjectOverviewUi Project_Overview_Ui_t;
typedef NavigationUi Navigation_Ui_t;
typedef Model Model_t;
typedef WorkingAreaContainer Working_Area_Container_t;
typedef FileDialogUi File_Dialog_Ui_t;
typedef SerializationPolicy Serialization_Policy_t;
typedef ExceptionPolicy Exception_Policy_t;
typedef Loader Loader_t;

typedef controller::ProjectController<
			Project_t,
			Project_Factory_t,
			Project_Wizard_t,
			Model_t,
			Working_Area_Container_t,
			Project_Overview_Ui_t,
			Navigation_Ui_t,
			File_Dialog_Ui_t,
			Serialization_Policy_t,
			Exception_Policy_t
		> Project_Controller_t;

namespace test {

using namespace helper;

class ApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Project_Controller_t, boost::bind(&ApplicationContext::projectControllerFactory, this)),
		GAC_DEFAULT_CTOR_INIT(Project_Factory_t),
		GAC_DEFAULT_CTOR_INIT(Project_Wizard_t),
		GAC_DEFAULT_CTOR_INIT(Model_t),
		GAC_DEFAULT_CTOR_INIT(Working_Area_Container_t),
		GAC_DEFAULT_CTOR_INIT(Project_Overview_Ui_t),
		GAC_DEFAULT_CTOR_INIT(Navigation_Ui_t),
		GAC_DEFAULT_CTOR_INIT(Project_t),
		GAC_DEFAULT_CTOR_INIT(Loader_t)
	{}

private:
	Project_Controller_t *projectControllerFactory() {
		Project_Controller_t *projectController = new Project_Controller_t(
				FETCH_FROM_CTX(Project_Factory_t),
				FETCH_FROM_CTX(Project_Wizard_t),
				FETCH_FROM_CTX(Model_t),
				FETCH_FROM_CTX(Working_Area_Container_t),
				FETCH_FROM_CTX(Project_Overview_Ui_t),
				FETCH_FROM_CTX(Navigation_Ui_t)
				);

		return projectController;
	}

private:
	GAC_SCOPE(Project_Controller_t)
	GAC_SCOPE(Project_Factory_t)
	GAC_SCOPE(Project_Wizard_t)
	GAC_SCOPE(Model_t)
	GAC_SCOPE(Working_Area_Container_t)
	GAC_SCOPE(Project_Overview_Ui_t)
	GAC_SCOPE(Navigation_Ui_t)
	GAC_SCOPE(Project_t)
	GAC_SCOPE(Loader_t)
};

}


void createTest() {
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);
	Project *project = FETCH_FROM_CTX(Project_t);
	Project_Factory_t *projectFactory = FETCH_FROM_CTX(Project_Factory_t);

	mock::sequence callOrder;
	MOCK_EXPECT(projectFactory->createDelegate).once().in(callOrder).returns(project);
	MOCK_EXPECT(project->setParent).once().with(projectController).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Project_Wizard_t)->start).once().with(project).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->setProject).never().in(callOrder);

	projectController->create();
}

BOOST_FIXTURE_TEST_CASE(CreateGetsNewProjectFromFactoryAndCallsProjectWizzardToStartAndDoesNotSetTheProjectOnTheModelYet, test::ApplicationContext)
{
	createTest();
}

BOOST_FIXTURE_TEST_CASE(CreatedWithoutCreateCalledReturnsWithExceptionPolicyBehavior, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	MOCK_EXPECT(ExceptionPolicy::wrongCallOrder).once().with(projectController, QString("create"), QString("created")).returns(false);

	projectController->created();
}


BOOST_FIXTURE_TEST_CASE(CreatedSetsTheProjectOnTheModelAndShowsTheProjectOverviewUiOnTheWorkingAreaContainer, test::ApplicationContext)
{
	createTest();
	mock::reset();

	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->getProject).at_least(1).returns(FETCH_FROM_CTX(Project_t));

	mock::sequence callOrder;
	MOCK_EXPECT(FETCH_FROM_CTX(Project_t)->setParent).once().with(projectController).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->setProject).once().with(FETCH_FROM_CTX(Project_t)).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Project_Overview_Ui_t)->show).once().with(FETCH_FROM_CTX(Project_t)).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Navigation_Ui_t)->show).once().with(FETCH_FROM_CTX(Project_t)).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Project_Overview_Ui_t)).in(callOrder);
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setNavigationArea).once().with(FETCH_FROM_CTX(Navigation_Ui_t)).in(callOrder);

	projectController->created();
}



BOOST_FIXTURE_TEST_CASE(EditCallsProjectWizard, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->getProject).returns(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Project_Wizard_t)->editCommonProperties).once().with(FETCH_FROM_CTX(Model_t)->getProject());

	projectController->edit();
}


BOOST_FIXTURE_TEST_CASE(EditedSetsProjectOverviewUiOnWorkingAreaContainer, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Project_Overview_Ui_t));
	//MOCK_EXPECT(model->getProject).returns(NULL); if any further calls to a given mock object happen

	projectController->edited();
}

BOOST_FIXTURE_TEST_CASE(SaveDoesntCallSerializeIfFileNameIsEmpty, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);
	/*
	MOCK_EXPECT(File_Dialog_Ui_t::getSaveFileName).once().returns(QString(""));
	MOCK_EXPECT(projectController->serialize).never();
	*/

	BOOST_CHECK_THROW(projectController->save(), std::exception);

	mock::verify();
	mock::reset();
}

BOOST_FIXTURE_TEST_CASE(SaveCallsSerializeWithValidFileName, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);
/*
	MOCK_EXPECT(File_Dialog_Ui_t::getSaveFileName).once().returns(QString("C:/dummyvalidprojectfile.xml"));
	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->getProject).once().returns(FETCH_FROM_CTX(Project));
	MOCK_EXPECT(projectController->serialize).once().with(FETCH_FROM_CTX(Project), QString("C:/dummyvalidprojectfile.xml"));
*/

	BOOST_CHECK_THROW(projectController->save(), std::exception);

	mock::verify();
	mock::reset();
}

BOOST_FIXTURE_TEST_CASE(IfSerializationFailsWithAnExceptionTheExceptionPolicyGetsCalledToNotifyTheException, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	/*
	MOCK_EXPECT(File_Dialog_Ui_t::getSaveFileName).once().returns(QString("C:/dummyinvalidprojectfile.xml"));
	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->getProject).once().returns(FETCH_FROM_CTX(Project));
	MOCK_EXPECT(projectController->serialize).once().throws(std::runtime_error("failed"));
	MOCK_EXPECT(ExceptionPolicy::handleSerializationException).once().with(mock::any, QString("C:/dummyinvalidprojectfile.xml"));
	*/

	BOOST_CHECK_THROW(projectController->save(), std::exception);

	mock::verify();
	mock::reset();
}

BOOST_FIXTURE_TEST_CASE(LoadDoesntCallDeserializeIfFileNameIsEmpty, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	/*
	MOCK_EXPECT(File_Dialog_Ui_t::getOpenFileName).once().returns(QString(""));
	MOCK_EXPECT(projectController->deserialize).never();
	*/

	BOOST_CHECK_THROW(projectController->load(FETCH_FROM_CTX(Loader_t)), std::exception);

	mock::verify();
	mock::reset();
}

BOOST_FIXTURE_TEST_CASE(LoadCallsDeserializeIfFileNameIsValid, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	/*
	MOCK_EXPECT(File_Dialog_Ui_t::getOpenFileName).once().returns(QString("C:/dummyinvalidprojectfile.xml"));
	MOCK_EXPECT(FETCH_FROM_CTX(Project_Factory_t)->createDelegate).once().returns(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Project_t)->setParent).once().with(projectController);
	MOCK_EXPECT(projectController->deserialize).once().with(FETCH_FROM_CTX(Project_t), QString("C:/dummyinvalidprojectfile.xml"));
	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->setProject).once().with(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->getProject).at_least(1).returns(FETCH_FROM_CTX(Project_t));

	MOCK_EXPECT(FETCH_FROM_CTX(Project_Overview_Ui_t)->show).once().with(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Navigation_Ui_t)->show).once().with(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Project_Overview_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setNavigationArea).once().with(FETCH_FROM_CTX(Navigation_Ui_t));

	MOCK_EXPECT(FETCH_FROM_CTX(Loader_t)->execute).once().with(FETCH_FROM_CTX(Project_t));
	*/

	BOOST_CHECK_THROW(projectController->load(FETCH_FROM_CTX(Loader_t)), std::exception);

	mock::verify();
	mock::reset();
}


BOOST_FIXTURE_TEST_CASE(IfDeserializationFailsWithAnExceptionTheExceptionPolicyGetsCalledToNotifyTheException, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	/*
	MOCK_EXPECT(File_Dialog_Ui_t::getOpenFileName).once().returns(QString("C:/dummyinvalidprojectfile.xml"));
	MOCK_EXPECT(FETCH_FROM_CTX(Project_Factory_t)->createDelegate).once().returns(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Project_t)->setParent).once().with(projectController);
	MOCK_EXPECT(projectController->deserialize).once().throws(std::runtime_error("failed"));
	MOCK_EXPECT(ExceptionPolicy::handleDeserializationException).once().with(mock::any, QString("C:/dummyinvalidprojectfile.xml"));
	*/

	BOOST_CHECK_THROW(projectController->load(FETCH_FROM_CTX(Loader_t)), std::exception);

	mock::verify();
	mock::reset();
}

BOOST_FIXTURE_TEST_CASE(ShowProjectCallsSetWorkingAreaOnWorkingAreaContainerContainer, test::ApplicationContext)
{
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Model_t)->getProject).at_least(1).returns(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Project_Overview_Ui_t)->show).once().with(FETCH_FROM_CTX(Project_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Project_Overview_Ui_t));

	projectController->show();
}
