/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactorsDisplayWidgetTest.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QUrl>
#include <QString>
#include <set>
#include <iostream>
#include <boost/bind.hpp>
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "gui/DataModelAnalysisInfluenceFactorsDisplayWidget.hpp"
#include "app/GenericApplicationContext.hpp"


namespace test {
class AnalysisToolDescription {
};

template<typename T>
struct ColumnComparator
{
	bool operator() (T* const &lhs, T* const &rhs) const
	{
		return lhs->getIndex() != rhs->getIndex();
	}
};

class Column {
public:
	Column(QString name, int index)
		:name(name), index(index)
	{}
	virtual ~Column(){}

	QString getName() {
		return name;
	}

	int getIndex() {
		return index;
	}

private:
	QString name;
	int index;
};

class BehaviorIndicator : public Column{
public:
	BehaviorIndicator(QString name, int index)
		:Column(name, index)
	{}
};

class WaterLevelInfluenceFactor : public Column {
public:
	WaterLevelInfluenceFactor(QString name, int index)
		:Column(name, index)
	{}
};

class TemperatureInfluenceFactor : public Column {
public:
	TemperatureInfluenceFactor(QString name, int index)
		:Column(name, index)
	{}
};

class InfluenceFactors {
public:
	typedef AnalysisToolDescription tool_description_t;
	typedef BehaviorIndicator behavior_indicator_column_t;
	typedef WaterLevelInfluenceFactor water_level_column_t;
	typedef TemperatureInfluenceFactor temperature_column_t;
	typedef Column Column_t;
	typedef std::set<BehaviorIndicator *, ColumnComparator<BehaviorIndicator> > Behavior_Indicator_Column_Container_t;
	typedef std::set<WaterLevelInfluenceFactor *, ColumnComparator<WaterLevelInfluenceFactor> > Water_Level_Column_Container_t;
	typedef std::set<TemperatureInfluenceFactor *, ColumnComparator<TemperatureInfluenceFactor> > Temperature_Column_Container_t;
public:
	InfluenceFactors(QString plotLocation, Behavior_Indicator_Column_Container_t behaviorIndicators, Water_Level_Column_Container_t waterLevels, Temperature_Column_Container_t temperatureLevels, Column *currentBehaviorIndicator, Column *currentInfluenceFactor)
		:plotLocation(plotLocation), behaviorIndicators(behaviorIndicators), waterLevels(waterLevels), temperatureLevels(temperatureLevels), currentBehaviorIndicator(currentBehaviorIndicator), currentInfluenceFactor(currentInfluenceFactor)
	{
	}

	std::vector<double> getInfluenceFactorColumnValues() {
		return influenceFactorValues;
	}

	std::vector<double> getBehaviorIndicatorColumnValues() {
		return behaviorIndicatorValues;
	}

	QString getPlotLocation() {
		return plotLocation;
	}

	AnalysisToolDescription *getAnalysisToolDescription() {
		return &analysisToolDescripion;
	}

	int getWidthInPixel() {
		return 500;
	}

	int getHeightInPixel() {
		return 500;
	}

	Behavior_Indicator_Column_Container_t &getBehaviorIndicatorColumns() {
		return behaviorIndicators;
	}

	Water_Level_Column_Container_t &getWaterLevelColumns() {
		return waterLevels;
	}

	Temperature_Column_Container_t &getTemperatureColumns() {
		return temperatureLevels;
	}

	Column *getCurrentBehaviorIndicatorColumn() {
		return currentBehaviorIndicator;
	}

	Column *getCurrentInfluenceFactorColumn() {
		return currentInfluenceFactor;
	}

	int getAnEstimatedBehaviorIndicator() {
		return 0;
	}

	int getAnEstimatedInfluenceFactor() {
		return 0;
	}

	COM_SUBSCRIPTION_FUNCTION(Loaded, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorsChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(AnyInfluenceFactorChanged, communication::Signal<void()>);

	QString plotLocation;
	AnalysisToolDescription analysisToolDescripion;
	Behavior_Indicator_Column_Container_t behaviorIndicators;
	Water_Level_Column_Container_t waterLevels;
	Temperature_Column_Container_t temperatureLevels;
	Column *currentBehaviorIndicator;
	Column *currentInfluenceFactor;
	std::vector<double> behaviorIndicatorValues;
	std::vector<double> influenceFactorValues;
};

} /* namespace test */


using namespace app::context;

typedef test::InfluenceFactors Influence_Factors_t;
typedef test::BehaviorIndicator Behavior_Indicator_t;
typedef test::WaterLevelInfluenceFactor Water_Level_Influence_Factor_t;
typedef test::TemperatureInfluenceFactor Temperature_Influence_Factor_t;
typedef gui::DataModelAnalysisInfluenceFactorsDisplayWidget<Influence_Factors_t> Widget_t;

template<typename T>
T *factory(QString name, int index) {
	return new T(name, index);
}

Influence_Factors_t *influenceFactorsFactory() {
	typename Influence_Factors_t::Behavior_Indicator_Column_Container_t behaviorIndicators;
	typename Influence_Factors_t::Water_Level_Column_Container_t waterLevels;
	typename Influence_Factors_t::Temperature_Column_Container_t temperatureLevels;

	behaviorIndicators.insert(FETCH_FROM_CTX(Behavior_Indicator_t));
	waterLevels.insert(FETCH_FROM_CTX(Water_Level_Influence_Factor_t));
	temperatureLevels.insert(FETCH_FROM_CTX(Temperature_Influence_Factor_t));

	return new Influence_Factors_t("./plot.svg", behaviorIndicators, waterLevels, temperatureLevels, FETCH_FROM_CTX(Behavior_Indicator_t), FETCH_FROM_CTX(Temperature_Influence_Factor_t));
}

Widget_t *widgetFactory() {
	return new Widget_t(FETCH_FROM_CTX(Influence_Factors_t));
}


void behaviorIndicatorChanged(Influence_Factors_t *influenceFactors, int index) {
	std::cout << __FUNCTION__ << std::endl;
}

void influenceFactorsChanged(Influence_Factors_t *influenceFactors, int index) {
	std::cout << __FUNCTION__ << std::endl;
}

void analyseButtonClicked(Influence_Factors_t *influenceFactors) {
	std::cout << __FUNCTION__ << std::endl;
}

int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Widget_t, widgetFactory, defaultCleanupStrategy<Widget_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Influence_Factors_t, influenceFactorsFactory, defaultCleanupStrategy<Influence_Factors_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Behavior_Indicator_t, boost::bind(factory<Behavior_Indicator_t>, QString("a"), 0), defaultCleanupStrategy<Behavior_Indicator_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Water_Level_Influence_Factor_t, boost::bind(factory<Water_Level_Influence_Factor_t>, QString("b"), 1), defaultCleanupStrategy<Water_Level_Influence_Factor_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Temperature_Influence_Factor_t, boost::bind(factory<Temperature_Influence_Factor_t>, QString("c"), 2), defaultCleanupStrategy<Temperature_Influence_Factor_t>);

	Influence_Factors_t *influenceFactors = FETCH_FROM_CTX(Influence_Factors_t);
	influenceFactors->behaviorIndicatorValues.push_back(12.5);
	influenceFactors->behaviorIndicatorValues.push_back(13.5);
	influenceFactors->behaviorIndicatorValues.push_back(10.5);
	influenceFactors->behaviorIndicatorValues.push_back(5.5);

	influenceFactors->influenceFactorValues.push_back(9.8);
	influenceFactors->influenceFactorValues.push_back(10.8);
	influenceFactors->influenceFactorValues.push_back(11.34);
	influenceFactors->influenceFactorValues.push_back(5.12);

	Widget_t *widget = FETCH_FROM_CTX(Widget_t);

	COM_SUBSCRIBE_FOR(widget, BehaviorIndicatorChanged, boost::bind(behaviorIndicatorChanged, _1, _2));
	COM_SUBSCRIBE_FOR(widget, InfluenceFactorChanged, boost::bind(influenceFactorsChanged, _1, _2));
	//COM_SUBSCRIBE_FOR(widget, ExecuteAnalysis, boost::bind(analyseButtonClicked, _1));


	mainWindow.setCentralWidget(widget);

	mainWindow.show();

 return app.exec();
}
