/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverviewConfigurationWidgetTest.cpp
 *
 *  Created on: Mar 12, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <set>
#include <boost/bind.hpp>
#include <iostream>
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "gui/DataModelAnalysisTimeSeriesOverviewConfigurationWidget.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {


template<typename T>
struct ColumnComparator
{
	bool operator() (T* const &lhs, T* const &rhs) const
	{
		return lhs->getIndex() != rhs->getIndex();
	}
};

class MissingDataStrategy {
public:
	QString getName() {
		return QString("fill with 0");
	}
};

class DataFrameHeader {
public:
	DataFrameHeader(QString name, int index)
		:name(name), index(index)
	{
	}

	int getIndex() const {
		return index;
	}

	const QString getName() const {
		return name;
	}

private:
	QString name;
	int index;
};

class Column {
public:
	Column();
	Column(DataFrameHeader &dataFrameHeader)
		:dataFrameHeader(dataFrameHeader)
	{
		index = dataFrameHeader.getIndex();
	}

	QString getName() {
		return dataFrameHeader.getName();
	}

	int getIndex() const {
		return index;
	}

	DataFrameHeader &getHeader() {
		return dataFrameHeader;
	}

private:
	DataFrameHeader dataFrameHeader;
	int index;
};

class DateColumn : public Column {
public:
	DateColumn();
	DateColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class BehaviorIndicator : public Column {
public:
	BehaviorIndicator();
	BehaviorIndicator(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};


class WaterLevelColumn : public Column {
public:
	WaterLevelColumn();
	WaterLevelColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class TemperatureColumn : public Column {
public:
	TemperatureColumn();
	TemperatureColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class DataModel : public ::communication::BoostSignalPolicy {
public:
	typedef MissingDataStrategy missing_data_strategy_t;
	typedef DataFrameHeader Header_t;
	typedef WaterLevelColumn Water_Level_Column_t;
	typedef BehaviorIndicator Behavior_Indicator_Column_t;
	typedef TemperatureColumn Temperature_Column_t;
	typedef std::vector<DataFrameHeader> Header_Container_t;
	typedef std::set<BehaviorIndicator *, ColumnComparator<BehaviorIndicator> > Behavior_Indicator_Column_Container_t;
	typedef std::set<Water_Level_Column_t *, ColumnComparator<WaterLevelColumn> > Water_Level_Column_Container_t;
	typedef std::set<Temperature_Column_t *, ColumnComparator<WaterLevelColumn> > Temperature_Column_Container_t;

	DataModel(Behavior_Indicator_Column_Container_t &behaviorIndicators, Water_Level_Column_Container_t &waterLevels)
		:behaviorIndicators(behaviorIndicators), waterLevels(waterLevels)
	{
	}

	Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns() {
		return behaviorIndicators;
	}

	Water_Level_Column_Container_t &getWaterLevelColumns() {
		return waterLevels;
	}

	Temperature_Column_Container_t &getTemperatureColumns() {
		return temperatureColumns;
	}

private:
	Behavior_Indicator_Column_Container_t behaviorIndicators;
	Water_Level_Column_Container_t waterLevels;
	Temperature_Column_Container_t temperatureColumns;
	MissingDataStrategy missingDataStrategy;
};


struct TimeSeriesOverview {
	typedef Column Column_t;
	TimeSeriesOverview(DataModel *dataModel, Column *valueColumn):dataModel(dataModel),valueColumn(valueColumn){;}

	DataModel *getDataModel() {
		return dataModel;
	}

	Column *getValueColumn() {
		return valueColumn;
	}

	DataModel *dataModel;
	Column *valueColumn;
};

} /* namespace test */

typedef test::WaterLevelColumn Water_Level_Column_t;
typedef test::BehaviorIndicator Behavior_Indicator_t;
typedef test::DataModel DataModel_t;
typedef test::TimeSeriesOverview TimeSeriesOverview_t;
typedef gui::DataModelAnalysisTimeSeriesOverviewConfigurationWidget<TimeSeriesOverview_t> Widget_t;

DataModel_t *dataModelFactory() {
	DataModel_t::Header_Container_t headerContainer;
	headerContainer.push_back(test::DataFrameHeader("MSVC0", 0));
	headerContainer.push_back(test::DataFrameHeader("MSVC1", 1));
	headerContainer.push_back(test::DataFrameHeader("date", 2));

	DataModel_t *dataModel;
	typename test::DataModel::Behavior_Indicator_Column_Container_t behaviorIndicators;
	typename test::DataModel::Water_Level_Column_Container_t waterLevelContainers;

	behaviorIndicators.insert(FETCH_FROM_CTX(Behavior_Indicator_t)); //new test::BehaviorIndicator(headerContainer.at(0)));
	waterLevelContainers.insert(FETCH_FROM_CTX(Water_Level_Column_t)); //new test::BehaviorIndicator(headerContainer.at(0)));

	dataModel = new DataModel_t(behaviorIndicators, waterLevelContainers);

	return dataModel;
}


Water_Level_Column_t *waterLevelFactory() {
	test::DataFrameHeader dataFrameHeader("MSVC1", 1);
	return new Water_Level_Column_t(dataFrameHeader);
}

Behavior_Indicator_t *behaviorIndicatorFactory() {
	//return new Behavior_Indicator_t(test::DataFrameHeader("MSVC0", 0));
	test::DataFrameHeader dataFrameHeader("MSVC0", 0);
	return new test::BehaviorIndicator(dataFrameHeader);
}

TimeSeriesOverview_t *factory() {
	return new TimeSeriesOverview_t(FETCH_FROM_CTX(DataModel_t), *FETCH_FROM_CTX(DataModel_t)->getBehaviorIndicatorColumns().begin());
}

Widget_t *widgetFactory() {
	return new Widget_t(FETCH_FROM_CTX(TimeSeriesOverview_t));
}


void valueColumnChanged(TimeSeriesOverview_t *t, int index) {
	std::cout << __FUNCTION__ << std::endl;
}


int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(Water_Level_Column_t, waterLevelFactory, defaultCleanupStrategy<Water_Level_Column_t>);
	REGISTER_FULL_CTX(Behavior_Indicator_t, behaviorIndicatorFactory, defaultCleanupStrategy<Behavior_Indicator_t>);
	REGISTER_FULL_CTX(DataModel_t, dataModelFactory, defaultCleanupStrategy<DataModel_t>);
	REGISTER_FULL_CTX(TimeSeriesOverview_t, factory, defaultCleanupStrategy<TimeSeriesOverview_t>);
	REGISTER_FULL_CTX(Widget_t, widgetFactory, defaultCleanupStrategy<Widget_t>);

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(Widget_t), ValueColumnChanged, (boost::bind(valueColumnChanged, _1, _2)));

	mainWindow.setCentralWidget(FETCH_FROM_CTX(Widget_t));

	mainWindow.show();
	return app.exec();
}
