/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisWidgetTest.cpp
 *
 *  Created on: Jan 21, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <vector>
#include <set>
#include <auto_ptr.h>
#include <iostream>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <QUuid>
#include <QLabel>
#include "gui/AnalysisToolToolBar.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "model/SignalPolicies.hpp"
#include "gui/DataModelAnalysisWidget.hpp"
#include "gui/AnalysisToolWidget.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {



class AnalysisToolDescription {
public:
	AnalysisToolDescription(QString name, QString id)
		:name(name), id(id)
	{
	}

	QString &getName() {
		return name;
	}

	QString &getId() {
		return id;
	}

private:
	QString name;
	QString id;
};

class DataModelAnalysisToolsDescription :
		public memory::CustomOwnershipPolicy<AnalysisToolDescription, memory::DeleteOnDeleteChildPolicy>
{
public:
	typedef AnalysisToolDescription tool_description_t;
	typedef std::vector<AnalysisToolDescription *> container_t;
public:
	DataModelAnalysisToolsDescription()
	{
		AnalysisToolDescription *influenceFactors(new AnalysisToolDescription(QString("influence factors"), QUuid::createUuid().toString()));
		overviewToolsDescription.push_back(influenceFactors);
		overtakeOwnership(overviewToolsDescription);

		AnalysisToolDescription *summary = new AnalysisToolDescription(QString("summary"), QUuid::createUuid().toString());
		overviewToolsDescription.push_back(summary);
		overtakeOwnership(summary);

		AnalysisToolDescription *distribution = new AnalysisToolDescription(QString("any distribution"), QUuid::createUuid().toString());
		distributionToolsDescription.push_back(distribution);
		overtakeOwnership(distribution);
	}

	virtual ~DataModelAnalysisToolsDescription(){

	}

	std::vector<AnalysisToolDescription *> &getOverviewAnalysisToolsDescription() {
		return overviewToolsDescription;
	}

	std::vector<AnalysisToolDescription *> &getDistributionAnalysisToolsDescription() {
		return distributionToolsDescription;
	}

private:
	std::vector<AnalysisToolDescription *> overviewToolsDescription;
	std::vector<AnalysisToolDescription *> distributionToolsDescription;
};


class DummyAnalysisToolWidget : public gui::AnalysisToolWidget {
public:
	DummyAnalysisToolWidget(AnalysisToolDescription *analysisToolDescription, QWidget *parent = 0)
		:gui::AnalysisToolWidget(parent)
	{
		setupLayout();
		connectLocal();
	}

	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysisTool, communication::Signal<void(DummyAnalysisToolWidget *)>);

private:
	void setupLayout() {
		QVBoxLayout *vBoxLayout = new QVBoxLayout(this);
		vBoxLayout->addWidget(gui::AnalysisToolWidget::getToolBar());
		vBoxLayout->addWidget(new QLabel("testcase", this));
		setLayout(vBoxLayout);
	}

	void connectLocal() {
		COM_SUBSCRIBE_FOR(gui::AnalysisToolWidget::getToolBar(), RemoveButtonClicked, (boost::bind(boost::ref(RemoveAnalysisToolSignal), this)));
	}
};


} /* namespace test */


typedef test::AnalysisToolDescription Analysis_Tool_Description_t;
typedef test::DataModelAnalysisToolsDescription DataModel_Analysis_Tools_Description_t;
typedef gui::DataModelAnalysisWidget<DataModel_Analysis_Tools_Description_t> DataModel_Analysis_Widget_t;
typedef test::DummyAnalysisToolWidget Dummy_Analysis_Tool_Widget_t;

DataModel_Analysis_Tools_Description_t *toolsDescriptionFactory() {
	/*
	std::vector<Analysis_Tool_Description_t> overviewDescriptions;

	overviewDescriptions.push_back(Analysis_Tool_Description_t(QString("influence factors"), QUuid::createUuid().toString()));
	overviewDescriptions.push_back(Analysis_Tool_Description_t(QString("summary"), QUuid::createUuid().toString()));

	std::vector<Analysis_Tool_Description_t> distributionDescriptions;
	distributionDescriptions.push_back(Analysis_Tool_Description_t(QString("any distribution"), QUuid::createUuid().toString()));
	*/

	//return new DataModel_Analysis_Tools_Description_t(overviewDescriptions, distributionDescriptions);
	return new DataModel_Analysis_Tools_Description_t;
}

DataModel_Analysis_Widget_t *widgetFactory() {
	DataModel_Analysis_Tools_Description_t *analysisToolsDescription = FETCH_FROM_CTX(DataModel_Analysis_Tools_Description_t);
	return new DataModel_Analysis_Widget_t(analysisToolsDescription);
}

Dummy_Analysis_Tool_Widget_t *dummyAnalysisToolWidgetFactory() {
	return new Dummy_Analysis_Tool_Widget_t(FETCH_FROM_CTX(DataModel_Analysis_Tools_Description_t)->getDistributionAnalysisToolsDescription().at(0));
}

void removeAnalysisTool(QWidget *analysisTool) {
	std::cout << __FUNCTION__ << std::endl;
	delete analysisTool;
}

void addAnalysisTool(Analysis_Tool_Description_t *toolDescription, boost::function<void(gui::AnalysisToolWidget *)> callback) {
	std::cout << __FUNCTION__ << std::endl;

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(Dummy_Analysis_Tool_Widget_t), RemoveAnalysisTool, boost::bind(removeAnalysisTool, _1));

	callback(FETCH_FROM_CTX(Dummy_Analysis_Tool_Widget_t));
}



int
main(int argc, char *argv[])
{
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_Analysis_Tools_Description_t, toolsDescriptionFactory, defaultCleanupStrategy<DataModel_Analysis_Tools_Description_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_Analysis_Widget_t, widgetFactory, defaultCleanupStrategy<DataModel_Analysis_Widget_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Dummy_Analysis_Tool_Widget_t, dummyAnalysisToolWidgetFactory, nullCleanupStrategy<Dummy_Analysis_Tool_Widget_t>);

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(DataModel_Analysis_Widget_t), AddAnalysisTool, boost::bind(addAnalysisTool, _1, _2));

	mainWindow.setCentralWidget(FETCH_FROM_CTX(DataModel_Analysis_Widget_t));
	mainWindow.show();

	return app.exec();
}
