/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAssignmentWizardPageTest.cpp
 *
 *  Created on: Jan 7, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <vector>
#include <set>
#include <iostream>
#include "model/SignalPolicies.hpp"
#include "gui/DataModelAssignmentWizardPage.hpp"
#include "gui/WizardNextIdStrategy.hpp"
#include "gui/DataModelWizardPages.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {

template<typename T>
struct ColumnComparator
{
	bool operator() (T* const &lhs, T* const &rhs) const
	{
		return lhs->getIndex() != rhs->getIndex();
	}
};

class DataFrameHeader {
public:
	DataFrameHeader(QString name, int index)
		:name(name), index(index)
	{
	}

	int getIndex() const {
		return index;
	}

	const QString getName() const {
		return name;
	}

private:
	QString name;
	int index;
};

class Column {
public:
	Column();
	Column(DataFrameHeader &dataFrameHeader)
		:dataFrameHeader(dataFrameHeader)
	{
		index = dataFrameHeader.getIndex();
	}

	QString getName() {
		return dataFrameHeader.getName();
	}

	int getIndex() const {
		return index;
	}

	DataFrameHeader &getDataFrameHeader();

private:
	DataFrameHeader dataFrameHeader;
	int index;
};

class BehaviorIndicator : public Column {
public:
	BehaviorIndicator();
	BehaviorIndicator(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};


class WaterLevelColumn : public Column {
public:
	WaterLevelColumn();
	WaterLevelColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class DataModel : public ::communication::BoostSignalPolicy {
public:
	typedef DataFrameHeader Header_t;
	typedef std::vector<DataFrameHeader> Header_Container_t;
	typedef std::set<BehaviorIndicator, ColumnComparator<BehaviorIndicator> > Behavior_Indicator_Column_Container_t;
	typedef std::set<WaterLevelColumn, ColumnComparator<WaterLevelColumn> > Water_Level_Column_Container_t;

	DataModel(Header_Container_t &headers, Behavior_Indicator_Column_Container_t &behaviorIndicators, Water_Level_Column_Container_t &waterLevels)
		:headers(headers), behaviorIndicators(behaviorIndicators), waterLevels(waterLevels)
	{
	}

	const Header_Container_t &getAssignableBehaviorIndicatorHeaders() {
		return headers;
	}

	const Header_Container_t &getAssignableWaterLevelHeaders() {
		return headers;
	}

	const Header_Container_t &getAssignableTemperatureHeaders() {
		return headers;
	}

	bool isBehaviorIndicatorColumnAssignedFor(int index) {
		return false;
	}

	bool isWaterLevelColumnAssignedFor(int index) {
		return true;
	}

	bool isTemperatureColumnAssignedFor(int index) {
		return true;
	}

	COM_SUBSCRIPTION_FUNCTION(DataModelLoaded, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(DateColumnChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(WaterLevelAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(TemperatureAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);

private:
	Header_Container_t headers;
	Behavior_Indicator_Column_Container_t behaviorIndicators;
	Water_Level_Column_Container_t waterLevels;
};
} /* namespace test */

typedef test::DataModel DataModel_t;
typedef gui::WizardNextIdStrategy<gui::DataModelWizardPages::Pages> Wizard_Next_Id_Strategy_t;
typedef gui::DataModelAssignmentWizardPage<DataModel_t, Wizard_Next_Id_Strategy_t > DataModel_Assignment_Wizard_Page_t;

DataModel_t *dataModelFactory() {
	DataModel_t::Header_Container_t headerContainer;
	headerContainer.push_back(test::DataFrameHeader("MSVC0", 0));
	headerContainer.push_back(test::DataFrameHeader("MSVC1", 1));
	headerContainer.push_back(test::DataFrameHeader("data", 2));

	DataModel_t *dataModel;
	typename test::DataModel::Behavior_Indicator_Column_Container_t behaviorIndicators;
	typename test::DataModel::Water_Level_Column_Container_t waterLevelContainers;
	dataModel = new DataModel_t(headerContainer, behaviorIndicators, waterLevelContainers);

	return dataModel;
}

DataModel_Assignment_Wizard_Page_t *dataModelAssignmentWizardPageFactory() {
	return new DataModel_Assignment_Wizard_Page_t(NULL, FETCH_FROM_CTX(DataModel_t), FETCH_FROM_CTX(Wizard_Next_Id_Strategy_t), gui::DataModelWizardPages::Assignment);
}

/*
bool operator==(test::DataFrameHeader const &header, test::BehaviorIndicator *dateColumn) {
	return dateColumn->getIndex() == header.getIndex();
}
*/

void behaviorIndicatorAssignmentChanged(DataModel_t *, int index, bool assignment) {
	std::cout << __FUNCTION__ << " index: " << index << " , assignement: " << assignment << std::endl;
}

int
main(int argc, char *argv[])
{
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_t, dataModelFactory, defaultCleanupStrategy<DataModel_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_Assignment_Wizard_Page_t, dataModelAssignmentWizardPageFactory, defaultCleanupStrategy<DataModel_Assignment_Wizard_Page_t>);
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(Wizard_Next_Id_Strategy_t);

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(DataModel_Assignment_Wizard_Page_t), BehaviorIndicatorAssignmentChanged, boost::bind(behaviorIndicatorAssignmentChanged, _1, _2, _3));

	mainWindow.setCentralWidget(FETCH_FROM_CTX(DataModel_Assignment_Wizard_Page_t));
	mainWindow.show();

	return app.exec();
}
