/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelOverviewWidgetTest.cpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QUrl>
#include <QString>
#include <QUuid>
#include <set>
#include <iostream>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "gui/DataModelOverviewWidget.hpp"
#include "gui/AnalysisToolWidget.hpp"
#include "app/GenericApplicationContext.hpp"


namespace test {


template<typename T>
struct ColumnComparator
{
	bool operator() (T* const &lhs, T* const &rhs) const
	{
		return lhs->getIndex() != rhs->getIndex();
	}
};

class MissingDataStrategy {
public:
	QString getName() {
		return QString("fill with 0");
	}
};

class DataFrameHeader {
public:
	DataFrameHeader(QString name, int index)
		:name(name), index(index)
	{
	}

	int getIndex() const {
		return index;
	}

	const QString getName() const {
		return name;
	}

private:
	QString name;
	int index;
};

class Column {
public:
	Column();
	Column(DataFrameHeader &dataFrameHeader)
		:dataFrameHeader(dataFrameHeader)
	{
		index = dataFrameHeader.getIndex();
	}

	QString getName() {
		return dataFrameHeader.getName();
	}

	int getIndex() const {
		return index;
	}

	DataFrameHeader &getHeader() {
		return dataFrameHeader;
	}

private:
	DataFrameHeader dataFrameHeader;
	int index;
};

class DateColumn : public Column {
public:
	DateColumn();
	DateColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class BehaviorIndicator : public Column {
public:
	BehaviorIndicator();
	BehaviorIndicator(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};


class WaterLevelColumn : public Column {
public:
	WaterLevelColumn();
	WaterLevelColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class TemperatureColumn : public Column {
public:
	TemperatureColumn();
	TemperatureColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class DataModel : public ::communication::BoostSignalPolicy {
public:
	typedef MissingDataStrategy missing_data_strategy_t;
	typedef DataFrameHeader Header_t;
	typedef WaterLevelColumn Water_Level_Column_t;
	typedef BehaviorIndicator Behavior_Indicator_Column_t;
	typedef TemperatureColumn Temperature_Column_t;
	typedef std::vector<DataFrameHeader> Header_Container_t;
	typedef std::set<BehaviorIndicator *, ColumnComparator<BehaviorIndicator> > Behavior_Indicator_Column_Container_t;
	typedef std::set<Water_Level_Column_t *, ColumnComparator<WaterLevelColumn> > Water_Level_Column_Container_t;
	typedef std::set<Temperature_Column_t *, ColumnComparator<WaterLevelColumn> > Temperature_Column_Container_t;

	DataModel(Header_Container_t &headers, DateColumn &dateColumn, Behavior_Indicator_Column_Container_t &behaviorIndicators, Water_Level_Column_Container_t &waterLevels)
		:headers(headers), dateColumn(dateColumn), behaviorIndicators(behaviorIndicators), waterLevels(waterLevels)
	{
	}

	QUrl getSourceUri() {
		return QUrl("c:/foo/bar");
	}

	MissingDataStrategy *getCurrentMissingDataStrategy() {
		return &missingDataStrategy;
	}

	DateColumn *getDateColumn() {
		return &dateColumn;
	}


	Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns() {
		return behaviorIndicators;
	}

	Water_Level_Column_Container_t &getWaterLevelColumns() {
		return waterLevels;
	}

	Temperature_Column_Container_t &getTemperatureColumns() {
		return temperatureColumns;
	}

	/*
	COM_SUBSCRIPTION_FUNCTION(DataModelLoaded, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(DateColumnChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(WaterLevelAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(TemperatureAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	*/

private:
	Header_Container_t headers;
	DateColumn dateColumn;
	Behavior_Indicator_Column_Container_t behaviorIndicators;
	Water_Level_Column_Container_t waterLevels;
	Temperature_Column_Container_t temperatureColumns;
	MissingDataStrategy missingDataStrategy;
};

class AnalysisToolDescription {
public:
	AnalysisToolDescription(QString name, QString id)
		:name(name), id(id)
	{
	}

	QString &getName() {
		return name;
	}

	QString &getId() {
		return id;
	}

private:
	QString name;
	QString id;
};

class DataModelAnalysisToolsDescription :
		public memory::CustomOwnershipPolicy<AnalysisToolDescription, memory::DeleteOnDeleteChildPolicy>
{
public:
	typedef AnalysisToolDescription tool_description_t;
	typedef std::vector<AnalysisToolDescription *> container_t;
public:
	DataModelAnalysisToolsDescription()
	{
		AnalysisToolDescription *influenceFactors(new AnalysisToolDescription(QString("influence factors"), QUuid::createUuid().toString()));
		overviewToolsDescription.push_back(influenceFactors);
		overtakeOwnership(overviewToolsDescription);

		AnalysisToolDescription *summary = new AnalysisToolDescription(QString("summary"), QUuid::createUuid().toString());
		overviewToolsDescription.push_back(summary);
		overtakeOwnership(summary);

		AnalysisToolDescription *distribution = new AnalysisToolDescription(QString("any distribution"), QUuid::createUuid().toString());
		distributionToolsDescription.push_back(distribution);
		overtakeOwnership(distribution);
	}

	virtual ~DataModelAnalysisToolsDescription(){

	}

	std::vector<AnalysisToolDescription *> &getOverviewAnalysisToolsDescription() {
		return overviewToolsDescription;
	}

	std::vector<AnalysisToolDescription *> &getDistributionAnalysisToolsDescription() {
		return distributionToolsDescription;
	}

private:
	std::vector<AnalysisToolDescription *> overviewToolsDescription;
	std::vector<AnalysisToolDescription *> distributionToolsDescription;
};

class AnyAnalysisWidget : public gui::AnalysisToolWidget{
public:
	AnyAnalysisWidget(QWidget *parent = 0)
		:gui::AnalysisToolWidget(parent)
	{
		setupLayout();
	}

private:
	void setupLayout() {
		QVBoxLayout *vboxLayout =  new QVBoxLayout(this);
		vboxLayout->addWidget(gui::AnalysisToolWidget::getToolBar());
		vboxLayout->addWidget(new QLabel("test", this));
		setMinimumHeight(300);
		setLayout(vboxLayout);
	}
};

} /* namespace test */


typedef test::AnyAnalysisWidget Any_Analysis_Widget_t;
typedef test::BehaviorIndicator Behavior_Indicator_t;
typedef test::DataModel DataModel_t;
typedef test::AnalysisToolDescription Analysis_Tool_Description_t;
typedef test::DataModelAnalysisToolsDescription DataModel_Analysis_Tools_Description_t;
typedef gui::DataModelOverviewWidget<DataModel_t, DataModel_Analysis_Tools_Description_t> DataModel_Overview_Widget_t;


using namespace app::context;


DataModel_t *dataModelFactory() {
	DataModel_t::Header_Container_t headerContainer;
	headerContainer.push_back(test::DataFrameHeader("MSVC0", 0));
	headerContainer.push_back(test::DataFrameHeader("MSVC1", 1));
	headerContainer.push_back(test::DataFrameHeader("date", 2));

	DataModel_t *dataModel;
	typename test::DataModel::Behavior_Indicator_Column_Container_t behaviorIndicators;
	typename test::DataModel::Water_Level_Column_Container_t waterLevelContainers;

	test::DateColumn dateColumn(headerContainer.at(2));
	behaviorIndicators.insert(FETCH_FROM_CTX(test::BehaviorIndicator)); //new test::BehaviorIndicator(headerContainer.at(0)));

	dataModel = new DataModel_t(headerContainer, dateColumn, behaviorIndicators, waterLevelContainers);

	return dataModel;
}

Behavior_Indicator_t *behaviorIndicatorFactory() {
	//return new Behavior_Indicator_t(test::DataFrameHeader("MSVC0", 0));
	test::DataFrameHeader dataFrameHeader("MSVC0", 0);
	return new test::BehaviorIndicator(dataFrameHeader);
}

DataModel_Analysis_Tools_Description_t *toolsDescriptionFactory() {
	return new DataModel_Analysis_Tools_Description_t();
}

DataModel_Overview_Widget_t *dataModelOverviewWidgetFactory() {
	return new DataModel_Overview_Widget_t(FETCH_FROM_CTX(DataModel_t), FETCH_FROM_CTX(DataModel_Analysis_Tools_Description_t));
}

Any_Analysis_Widget_t *anyAnalysisWidgetFactory() {
	return new Any_Analysis_Widget_t();
}


namespace conversion {

template<>
inline QString conversion::Converter<typename gui::DataModelCommonParameterDisplayWidget<DataModel_t>::Missing_Data_Strategy_Gui_t, QString>::convert(gui::DataModelCommonParameterDisplayWidget<DataModel_t>::Missing_Data_Strategy_Gui_t strategy)
{
	return strategy.value.getName();
}
} /* namespace conversion */


void editCommonParameters(DataModel_t *dataModel) {
	std::cout << __FUNCTION__ << std::endl;
}

void editTimeSeries(DataModel_t *dataModel) {
	std::cout << __FUNCTION__ << std::endl;
}

void editAssignment(DataModel_t *dataModel) {
	std::cout << __FUNCTION__ << std::endl;
}

void addAnalysisTool(Analysis_Tool_Description_t *analysisToolDescription, boost::function<void(test::AnyAnalysisWidget *)> callback) {
	std::cout << __FUNCTION__ << std::endl;
	callback(FETCH_FROM_CTX(Any_Analysis_Widget_t));
}

int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_Analysis_Tools_Description_t, toolsDescriptionFactory, defaultCleanupStrategy<DataModel_Analysis_Tools_Description_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Behavior_Indicator_t, behaviorIndicatorFactory, defaultCleanupStrategy<Behavior_Indicator_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_t, dataModelFactory, defaultCleanupStrategy<DataModel_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_Overview_Widget_t, dataModelOverviewWidgetFactory, defaultCleanupStrategy<DataModel_Overview_Widget_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Any_Analysis_Widget_t, anyAnalysisWidgetFactory, defaultCleanupStrategy<Any_Analysis_Widget_t>);

	DataModel_Overview_Widget_t *overviewWidget = FETCH_FROM_CTX(DataModel_Overview_Widget_t);

	COM_SUBSCRIBE_FOR(overviewWidget, EditCommonParameters, (boost::bind(editCommonParameters, _1)));
	COM_SUBSCRIBE_FOR(overviewWidget, EditTimeSeries, (boost::bind(editTimeSeries, _1)));
	COM_SUBSCRIBE_FOR(overviewWidget, EditAssignment, (boost::bind(editAssignment, _1)));

	COM_SUBSCRIBE_FOR(overviewWidget, AddAnalysisTool, (boost::bind(addAnalysisTool, _1, _2)));


	mainWindow.setCentralWidget(FETCH_FROM_CTX(DataModel_Overview_Widget_t));
	mainWindow.show();

	return app.exec();
}

