/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelDefinitionEditWidgetTests.cpp
 *
 *  Created on: Aug 6, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QString>
#include <vector>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include "gui/RegressionModelDefinitionEditWidget.hpp"
#include "app/GenericApplicationContext.hpp"
#include "communication/Signals.hpp"

using namespace app::context;

namespace test {

struct AssignmentCategory {
	int getIndex() {
		return 0;
	}

	QString getName() {
		return "the_assignment_category";
	}
};

struct RegressorFormulaBuilder {
	typedef AssignmentCategory AssignmentCategory_t;

	RegressorFormulaBuilder(QString identifier):identifier(identifier){;}

	QString getIdentifier() {
		return identifier;
	}

	QString resolveExtendedIdentifier() {
		return QString("extended ") + getIdentifier();
	}

	AssignmentCategory *getAssignmentCategory() {
		return &assignmentCategory;
	}

	QString identifier;
	AssignmentCategory assignmentCategory;
};

struct Regressor {
	Regressor(){;}

	Regressor(QString formula, QString name):formula(formula), name(name){;}

	QString getFormula() {
		return formula;
	}

	QString getName() {
		return name;
	}

	bool isEditable() {
		return true;
	}

	bool isActive() {
		return true;
	}

	QString formula;
	QString name;
};


struct ModelDefinition {
	typedef Regressor Regressor_t;
	typedef RegressorFormulaBuilder RegressorFormulaBuilder_t;

	COM_SUBSCRIPTION_FUNCTION(RegressorAdded, communication::Signal<void(Regressor_t *)>);
	COM_SUBSCRIPTION_FUNCTION(Changed, communication::Signal<void()>);

	QString getName() {
		return name;
	}

	bool isEditable() {
		return true;
	}

	std::vector<Regressor_t *> getRegressors() {
		std::vector<Regressor_t *> returnable;
		BOOST_FOREACH(Regressor_t &regressor, regressors) {
			returnable.push_back(&regressor);
		}

		return returnable;
	}

	std::vector<RegressorFormulaBuilder_t *> getRegressorFormulaBuilders() {
		std::vector<RegressorFormulaBuilder_t *> returnable;
		BOOST_FOREACH(RegressorFormulaBuilder_t &regressor, regressorFormulaBuilders) {
			returnable.push_back(&regressor);
		}

		return returnable;
	}

	QString name;
	std::vector<Regressor_t> regressors;
	std::vector<RegressorFormulaBuilder_t> regressorFormulaBuilders;
};

struct RegressionFormula {
};

}  // namespace test


typedef test::ModelDefinition model_t;
typedef test::RegressionFormula RegressionFormula_t;
typedef gui::RegressionModelDefinitionEditWidget<RegressionFormula_t, model_t> widget_t;

widget_t *widgetFactory() {
	return new widget_t(FETCH_FROM_CTX(RegressionFormula_t), FETCH_FROM_CTX(model_t));
}

void addRegressor(model_t *model, QString formulaBuilderIdentifier) {
	qDebug() << __FUNCTION__ << ":" << model->getName() << ":" << formulaBuilderIdentifier;
}

void removeModel(RegressionFormula_t *regressionFormula, model_t *model) {
	qDebug() << __FUNCTION__ << ":" << model->getName();
}

int
main(int argc, char *argv[]) {
	using namespace boost;
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(widget_t, widgetFactory, defaultCleanupStrategy<widget_t>);
	REGISTER_FULL_DEFAULT_CTX(model_t);
	REGISTER_FULL_DEFAULT_CTX(RegressionFormula_t);

	model_t *model = FETCH_FROM_CTX(model_t);
	model->name = "Super model definition";
	model->regressors.push_back(test::Regressor("T1($)", "T1(W_LEVEL)"));
	model->regressors.push_back(test::Regressor("T2($)", "T2(W_LEVEL)"));
	model->regressorFormulaBuilders.push_back(test::RegressorFormulaBuilder("T1 (Chebyshev)"));
	model->regressorFormulaBuilders.push_back(test::RegressorFormulaBuilder("T2 (Chebyshev)"));
	model->regressorFormulaBuilders.push_back(test::RegressorFormulaBuilder("T3 (Chebyshev)"));

	widget_t *widget = FETCH_FROM_CTX(widget_t);
	COM_SUBSCRIBE_FOR(widget, AddRegressor, bind(addRegressor, _1, _2));
	COM_SUBSCRIBE_FOR(widget, Remove, bind(removeModel, _1, _2));

	mainWindow.setCentralWidget(widget);
	mainWindow.show();

 return app.exec();
}
