/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelWizardTest.cpp
 *
 *  Created on: Feb 11, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QString>
#include <QWidget>
#include <QLabel>
#include <QVBoxLayout>
#include <iostream>
#include <set>
#include <vector>
#include <boost/bind.hpp>
#include "gui/RegressionModelWizard.hpp"
#include "gui/RegressorInfluenceFactorSelectionWidget.hpp"
#include "app/GenericApplicationContext.hpp"
#include "communication/Signals.hpp"
#include "model/SignalPolicies.hpp"

namespace test {

template<typename T>
struct ColumnComparator
{
	bool operator() (T* const &lhs, T* const &rhs) const
	{
		return lhs->getIndex() != rhs->getIndex();
	}
};

class DataFrameHeader {
public:
	DataFrameHeader(QString name, int index)
		:name(name), index(index)
	{
	}

	int getIndex() const {
		return index;
	}

	const QString getName() const {
		return name;
	}

private:
	QString name;
	int index;
};

class Column {
public:
	Column();
	Column(DataFrameHeader &dataFrameHeader)
		:dataFrameHeader(dataFrameHeader)
	{
		index = dataFrameHeader.getIndex();
	}

	QString getName() {
		return dataFrameHeader.getName();
	}

	int getIndex() const {
		return index;
	}

	DataFrameHeader &getHeader() {
		return dataFrameHeader;
	}

private:
	DataFrameHeader dataFrameHeader;
	int index;
};

struct DateDataPoint {
	DateDataPoint(QString date, int index) : date(date), index(index){;}

	DateDataPoint(){;}

	QString &getDate() {
		return date;
	}

	int getIndex() {
		return index;
	}

	QString date;
	int index;
};

class DateColumn : public Column {
public:
	DateColumn();
	DateColumn(DataFrameHeader &dataFrameHeader, std::vector<DateDataPoint> dateDataPoints)
		:Column(dataFrameHeader), dateDataPoints(dateDataPoints)
	{;}

	std::vector<DateDataPoint> &getValues() {
		return dateDataPoints;
	}

private:
	std::vector<DateDataPoint> dateDataPoints;
};

class BehaviorIndicator : public Column {
public:
	BehaviorIndicator();
	BehaviorIndicator(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};


class WaterLevelColumn : public Column {
public:
	WaterLevelColumn();
	WaterLevelColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class TemperatureColumn : public Column {
public:
	TemperatureColumn();
	TemperatureColumn(DataFrameHeader &dataFrameHeader)
		:Column(dataFrameHeader)
	{;}
};

class DataModel {
public:
	typedef DateDataPoint Date_Data_Point_t;
	typedef DataFrameHeader Header_t;
	typedef WaterLevelColumn Water_Level_Column_t;
	typedef BehaviorIndicator Behavior_Indicator_Column_t;
	typedef TemperatureColumn Temperature_Column_t;
	typedef std::vector<DataFrameHeader> Header_Container_t;
	typedef std::set<BehaviorIndicator *, ColumnComparator<BehaviorIndicator> > Behavior_Indicator_Column_Container_t;
	typedef std::set<Water_Level_Column_t *, ColumnComparator<WaterLevelColumn> > Water_Level_Column_Container_t;
	typedef std::set<Temperature_Column_t *, ColumnComparator<WaterLevelColumn> > Temperature_Column_Container_t;

	DataModel(Header_Container_t &headers, DateColumn &dateColumn, Behavior_Indicator_Column_Container_t &behaviorIndicators, Water_Level_Column_Container_t &waterLevels)
		:headers(headers), dateColumn(dateColumn), behaviorIndicators(behaviorIndicators), waterLevels(waterLevels)
	{
	}

	DateColumn *getDateColumn() {
		return &dateColumn;
	}


	Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns() {
		return behaviorIndicators;
	}

	Water_Level_Column_Container_t &getWaterLevelColumns() {
		return waterLevels;
	}

	Temperature_Column_Container_t &getTemperatureColumns() {
		return temperatureColumns;
	}

private:
	Header_Container_t headers;
	DateColumn dateColumn;
	Behavior_Indicator_Column_Container_t behaviorIndicators;
	Water_Level_Column_Container_t waterLevels;
	Temperature_Column_Container_t temperatureColumns;
};

struct RegressorFormulaBuilder {
	QString resolveExtendedIdentifier() {
		return "extended identifier";
	}
};

struct Regressor : public communication::BoostSignalPolicy {
	typedef Column Influence_Factor_t;

	Regressor(QString formula, Column *influenceFactor, int index)
		:formula(formula), influenceFactor(influenceFactor), index(index){;}

	QString &getFormula() {
		return formula;
	}

	void setFormula(QString formula) {
		this->formula = formula;
		communication::BoostSignalPolicy::fire();
	}

	Influence_Factor_t *getInfluenceFactor() {
		return influenceFactor;
	}

	RegressorFormulaBuilder *getRegressorFormulaBuilder() {
		return &regressorFormulaBuilder;
	}

	std::vector<Influence_Factor_t *> getAvailableInfluenceFactors() {
		std::vector<Influence_Factor_t *> influenceFactors;
		influenceFactors.push_back(influenceFactor);
		influenceFactors.push_back(influenceFactor);

		return influenceFactors;
	}

	int getIndex() {
		return index;
	}

	bool isActive() {
		return true;
	}


	QString formula;
	Influence_Factor_t *influenceFactor;
	RegressorFormulaBuilder regressorFormulaBuilder;
	int index;
};

struct RegressionFormulaMethodDescription {
	RegressionFormulaMethodDescription(QString methodId, QString name, QString author, QString description):name(name), author(author), description(description){;}

	QString getMethodId() {
		return methodId;
	}

	QString getName(){
		return name;
	}

	QString getAuthor() {
		return author;
	}

	QString getDescription() {
		return description;
	}

	QString methodId;
	QString name;
	QString author;
	QString description;
};

struct RegressionModelDefinition {
	QString getName() {
		return "The regression model definition";
	}
};


struct RegressionFormula {
	typedef Regressor Regressor_t;
	typedef RegressionFormulaMethodDescription Method_Description_t;
	typedef RegressionModelDefinition RegressionModelDefinition_t;
	typedef RegressorFormulaBuilder RegressorFormulaBuilder_t;

	RegressionFormula(Regressor *waterLevelRegressor, std::vector<RegressionFormulaMethodDescription *> &methodDescriptions) : waterLevelRegressor(waterLevelRegressor), methodDescriptions(methodDescriptions){;}

	std::vector<Regressor *> getWaterLevelRegressors() {
		std::vector<Regressor *> waterLevelRegressors;
		waterLevelRegressors.push_back(waterLevelRegressor);
		return waterLevelRegressors;
	}

	std::vector<Regressor *> getTemperatureRegressors() {
		std::vector<Regressor *> temperatureRegressors;
		return temperatureRegressors;
	}



	QString getWaterLevelFormula() {
		return waterLevelRegressor->getFormula();
	}

	Regressor_t *findRegressorByIndex(int index) {
		return waterLevelRegressor;
	}


	std::vector<RegressionModelDefinition_t *> getAllRegressionModelDefinitions() {
		//return getAllRegressionModelDefinitions();
		std::vector<RegressionModelDefinition_t *> returning;
		return returning;
	}

	std::vector<RegressionModelDefinition_t *> getRegressionModelDefinitions() {
		std::vector<RegressionModelDefinition_t *> returning;
		BOOST_FOREACH(RegressionModelDefinition_t &definition, regressionModelDefinitions) {
			returning.push_back(&definition);
		}

		return returning;
	}

	std::vector<RegressionFormulaMethodDescription *> getMethodDescriptions() {
		return methodDescriptions;
	}

	QString toString() {
		return QString("y ~ x + z");
	}

	COM_SUBSCRIPTION_FUNCTION(Changed, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(WaterLevelRegressorAdded, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(TemperatureRegressorAdded, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(RegressorRemoved, communication::Signal<void(int)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressionModelDefinition, communication::Signal<void(RegressionModelDefinition *)>);
	COM_SUBSCRIPTION_FUNCTION(RegressionModelDefinitionAdded, communication::Signal<void(RegressionModelDefinition *)>);

	Regressor *waterLevelRegressor;
	std::vector<RegressionFormulaMethodDescription *> methodDescriptions;
	std::vector<RegressionModelDefinition_t> regressionModelDefinitions;
};

class RegressionModel {
public:
	typedef DataModel DataModel_t;
	typedef RegressionFormula Regression_Formula_t;
public:
	COM_SUBSCRIPTION_FUNCTION(TimeValuesChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(AvailableRegressionTimeWindowChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(AlgorithmChanged, communication::Signal<void()>);
public:
	RegressionModel(DataModel *dataModel, RegressionFormula *regressionFormula)
		:dataModel(dataModel)
		, behaviorIndicator(*dataModel->getBehaviorIndicatorColumns().begin())
		, regressionFormula(regressionFormula)
	{
		if(dataModel->getDateColumn()->getValues().size() >= 2) {
			fromDateDataPoint = dataModel->getDateColumn()->getValues().at(1);
			toDateDataPoint = dataModel->getDateColumn()->getValues().at(2);
		}else{
			fromDateDataPoint = dataModel->getDateColumn()->getValues().at(0);
			toDateDataPoint = dataModel->getDateColumn()->getValues().at(1);
		}
	}

	enum Algorithm {
			Standard,
			PraisWinsten,
			Ridge,
			Pcr
		};

	bool isStandardAlgorithm() {
		return false;
	}

	bool isPraisWinstenAlgorithm() {
		return false;
	}

	bool isRidgeAlgorithm() {
		return false;
	}

	bool isPCRAlgorithm() {
		return true;
	}


	const QString& getDescription() const {
		return description;
	}

	void setDescription(const QString& description) {
		this->description = description;
	}

	const QString& getName() const {
		return name;
	}

	void setName(const QString& name) {
		this->name = name;
	}

	DataModel* getDataModel() {
		return dataModel;
	}

	std::vector<DateDataPoint> getAvailableRegressionTimeWindow() {
		std::vector<DateDataPoint> datePoints = dataModel->getDateColumn()->getValues();

		//return only the latest few
		std::vector<DateDataPoint> realDatePoints;
		if(datePoints.size() >= 2) {
			realDatePoints.push_back(datePoints.at(datePoints.size() - 2));
			realDatePoints.push_back(datePoints.at(datePoints.size() - 1));
		}else{
			realDatePoints = datePoints;
		}

		return realDatePoints;
	}

	DateDataPoint &getFromDate() {
		return fromDateDataPoint;
	}

	DateDataPoint &getToDate() {
		return toDateDataPoint;
	}

	BehaviorIndicator *getBehaviorIndicatorColumn() {
		return behaviorIndicator;
	}

	RegressionFormula *getRegressionFormula() {
		return regressionFormula;
	}

	unsigned getPraisWinstenIterations() {
		return 3;
	}

	double getRidgeKValue() {
		return 0.06;
	}

	unsigned getMaxPCRrValue() {
		return 4;
	}

	unsigned getPCRrValue() {
		return 3;
	}

private:
	QString name;
	QString description;
	DataModel *dataModel;
	DateDataPoint fromDateDataPoint;
	DateDataPoint toDateDataPoint;
	BehaviorIndicator *behaviorIndicator;
	RegressionFormula *regressionFormula;
};
} /* namespace test */

typedef test::Regressor Regressor_t;
typedef test::DataModel DataModel_t;
typedef test::RegressionFormulaMethodDescription Method_Description_t;
typedef test::RegressionFormula Regression_Formula_t;
typedef test::RegressionModel RegressionModel_t;
typedef gui::RegressionModelWizard<RegressionModel_t> RegressionModel_Wizard_t;
typedef gui::RegressorInfluenceFactorSelectionWidget<Regressor_t> RegressorInfluenceFactorSelectionWidget_t;

using namespace app::context;

DataModel_t *dataModelFactory(){
	DataModel_t::Header_Container_t headerContainer;
	headerContainer.push_back(test::DataFrameHeader("MSVC0", 0));
	headerContainer.push_back(test::DataFrameHeader("MSVC1", 1));
	headerContainer.push_back(test::DataFrameHeader("date", 2));

	DataModel_t *dataModel;
	typename test::DataModel::Behavior_Indicator_Column_Container_t behaviorIndicators;
	typename test::DataModel::Water_Level_Column_Container_t waterLevelContainers;

	std::vector<test::DateDataPoint> dateDataPoints;
	dateDataPoints.push_back(test::DateDataPoint("11-08-1983", 0));
	dateDataPoints.push_back(test::DateDataPoint("08-11-1983", 1));
	dateDataPoints.push_back(test::DateDataPoint("20-11-1983", 2));

	test::DateColumn dateColumn(headerContainer.at(2), dateDataPoints);

	behaviorIndicators.insert(FETCH_FROM_CTX(test::BehaviorIndicator)); //new test::BehaviorIndicator(headerContainer.at(0)));
	waterLevelContainers.insert(FETCH_FROM_CTX(test::WaterLevelColumn));

	dataModel = new DataModel_t(headerContainer, dateColumn, behaviorIndicators, waterLevelContainers);

	return dataModel;
}

test::BehaviorIndicator *behaviorIndicatorFactory() {
	test::DataFrameHeader dataFrameHeader("MSVC0", 0);
	return new test::BehaviorIndicator(dataFrameHeader);
}

test::WaterLevelColumn *waterLevelColumnFactory() {
	test::DataFrameHeader dataFrameHeader("MSVC1", 1);
	return new test::WaterLevelColumn(dataFrameHeader);
}

Method_Description_t *methodDescriptionFactory() {
	return new Method_Description_t("1526654b-afd6-4e64-bd5b-f212bde73f74", "Polynom 3th degree", "Lombardi 2004", "Polynom of 3th degree for water level");
}

test::Regressor *regressorFactory() {
	return new test::Regressor("sin(MSV1)", *FETCH_FROM_CTX(DataModel_t)->getWaterLevelColumns().begin(), 1);
}

Regression_Formula_t *regressionFormulaFactory() {
	std::vector<Method_Description_t *> methodDescriptions;
	methodDescriptions.push_back(FETCH_FROM_CTX(Method_Description_t));

	Regression_Formula_t *regressionFormula = new Regression_Formula_t(
				FETCH_FROM_CTX(test::Regressor),
				methodDescriptions
			);

	return regressionFormula;
}

RegressionModel_t *regressionModelFactory() {
	return new RegressionModel_t(FETCH_FROM_CTX(DataModel_t), FETCH_FROM_CTX(Regression_Formula_t));
}

RegressionModel_Wizard_t *regressionModelWizardFactory() {
	return new RegressionModel_Wizard_t(FETCH_FROM_CTX(RegressionModel_t));
}

RegressorInfluenceFactorSelectionWidget_t *influenceFactorSelectionWidgetFactory() {
	return new RegressorInfluenceFactorSelectionWidget_t(FETCH_FROM_CTX(test::Regressor));
}

void setName(RegressionModel_t *regressionModel, QString name) {
	std::cout << __FUNCTION__ << std::endl;
}

void setDescription(RegressionModel_t *regressionModel, QString name) {
	std::cout << __FUNCTION__ << std::endl;
}

void fromDateChanged(RegressionModel_t *regressionModel, int fromDateIndex) {
	std::cout << __FUNCTION__ << std::endl;
}

void toDateChanged(RegressionModel_t *regressionModel, int fromDateIndex) {
	std::cout << __FUNCTION__ << std::endl;
}

void behaviorIndicatorChanged(RegressionModel_t *regressionModel, int behaviorIndicatorColumnIndex) {
	std::cout << __FUNCTION__ << std::endl;
}

void createWaterLevelRegressor(RegressionModel_t *regressionModel) {
	std::cout << __FUNCTION__ << std::endl;

	regressionModel->getRegressionFormula()->WaterLevelRegressorAddedSignal(1);
}

void createTemperatureRegressor(RegressionModel_t *regressionModel) {
	std::cout << __FUNCTION__ << std::endl;

	regressionModel->getRegressionFormula()->TemperatureRegressorAddedSignal(1);
}

void waterLevelRegressorEditFinished(RegressionModel_t *regressionModel, int regressorIndex, QString formula) {
	qDebug() << __FUNCTION__ << " " << regressorIndex << " - " << formula;
}

void temperatureRegressorEditFinished(RegressionModel_t *regressionModel, int regressorIndex, QString formula) {
	qDebug() << __FUNCTION__ << " " << regressorIndex << " - " << formula;
}

void removeRegressor(RegressionModel_t *regressionModel, int regressorIndex) {
	qDebug() << __FUNCTION__ << " " << regressorIndex;
}

void addRegressionFormulaMethodInstance(RegressionModel_t *regressionModel, Method_Description_t *methodDescription) {
	qDebug() << __FUNCTION__;
}

void algorithmChanged(RegressionModel_t *, int algorithm) {
	qDebug() << __FUNCTION__ << " " << algorithm;
}

void praisWinstenIterationsChanged(RegressionModel_t *, int value) {
	qDebug() << __FUNCTION__ << " " << value;
}

void ridgeKValueChanged(RegressionModel_t *, double value) {
	qDebug() << __FUNCTION__ << " " << value;
}

void pcrrValueChanged(RegressionModel_t *, int value) {
	qDebug() << __FUNCTION__ << " " << value;
}

QWidget *regressionModelDefinitionWidgetFactory(test::RegressionFormula *, test::RegressionModelDefinition *regressionModelDefinition) {
	QWidget *widget = new QWidget;
	QLabel *label = new QLabel("testwidget");
	QVBoxLayout *layout = new QVBoxLayout();
	layout->addWidget(label);
	widget->setLayout(layout);

	return widget;
}

int
main(int argc, char *argv[]) {
	using namespace boost;
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(test::BehaviorIndicator, behaviorIndicatorFactory, defaultCleanupStrategy<test::BehaviorIndicator>);
	REGISTER_FULL_CTX(test::WaterLevelColumn, waterLevelColumnFactory, defaultCleanupStrategy<test::WaterLevelColumn>);
	REGISTER_FULL_CTX(DataModel_t, dataModelFactory, defaultCleanupStrategy<DataModel_t>);
	REGISTER_FULL_CTX(Method_Description_t, methodDescriptionFactory, defaultCleanupStrategy<Method_Description_t>);
	REGISTER_FULL_CTX(test::Regressor, regressorFactory, defaultCleanupStrategy<test::Regressor>);
	REGISTER_FULL_CTX(Regression_Formula_t, regressionFormulaFactory, defaultCleanupStrategy<Regression_Formula_t>);
	REGISTER_FULL_CTX(RegressionModel_t, regressionModelFactory, defaultCleanupStrategy<RegressionModel_t>);
	REGISTER_FULL_CTX(RegressionModel_Wizard_t, regressionModelWizardFactory, defaultCleanupStrategy<RegressionModel_Wizard_t>);
	REGISTER_FULL_CTX(RegressorInfluenceFactorSelectionWidget_t, influenceFactorSelectionWidgetFactory, defaultCleanupStrategy<RegressorInfluenceFactorSelectionWidget_t>);

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), SetName, (boost::bind(setName, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), SetDescription, (boost::bind(setDescription, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), FromDateChanged, (boost::bind(fromDateChanged, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), ToDateChanged, (boost::bind(toDateChanged, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), BehaviorIndicatorChanged, (boost::bind(behaviorIndicatorChanged, _1, _2)));
//	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), RemoveRegressor, (boost::bind(removeRegressor, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), AddRegressionFormulaMethodInstance, (boost::bind(addRegressionFormulaMethodInstance, _1, _2)));
	/*
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), CreateWaterLevelRegressor, (boost::bind(createWaterLevelRegressor, _1)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), CreateTemperatureRegressor, (boost::bind(createTemperatureRegressor, _1)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), WaterLevelRegressorFormulaEdited, (boost::bind(waterLevelRegressorEditFinished, _1, _2, _3)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), TemperatureRegressorFormulaEdited, (boost::bind(temperatureRegressorEditFinished, _1, _2, _3)));
	*/
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), AlgorithmChanged, (boost::bind(algorithmChanged, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), PraisWinstenIterationsChanged, (boost::bind(praisWinstenIterationsChanged, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), RidgeKValueChanged, (boost::bind(ridgeKValueChanged, _1, _2)));
	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(RegressionModel_Wizard_t), PCRrValueChanged, (boost::bind(pcrrValueChanged, _1, _2)));

	mainWindow.show(); //we need this to properly terminate the application

	FETCH_FROM_CTX(Regression_Formula_t)->regressionModelDefinitions.push_back(test::RegressionModelDefinition());
	FETCH_FROM_CTX(test::Regressor)->setFormula("newFormula()");


	FETCH_FROM_CTX(RegressorInfluenceFactorSelectionWidget_t)->exec();
	FETCH_FROM_CTX(RegressionModel_Wizard_t)->getRegressionModelDefinitionsWidget()->init(bind(regressionModelDefinitionWidgetFactory, _1, _2));
	FETCH_FROM_CTX(RegressionModel_Wizard_t)->start();



 return app.exec();
}
