/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderEditWidgetTests.cpp
 *
 *  Created on: 13.08.2014
 *      Author: gerberma
 */

#include <QDebug>
#include <QApplication>
#include <QMainWindow>
#include <QString>
#include <QInputDialog>
#include <vector>
#include <map>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include "gui/RegressorFormulaBuilderEditWidget.hpp"
#include "app/GenericApplicationContext.hpp"
#include "applicationcontext/scopes.hpp"
#include "communication/Signals.hpp"

using namespace app::context;

namespace test {

struct AssignmentCategory {
	AssignmentCategory(QString identifier, QString name):identifier(identifier), name(name){;}
	QString getIdentifier() {
		return identifier;
	}

	QString getName() {
		return name;
	}

	QString identifier;
	QString name;
};

struct UserParameterType {
	UserParameterType():value(""), isminimumleadtime(false){;}
	UserParameterType(QString value, bool isminimumleadtime)
		:value(value), isminimumleadtime(isminimumleadtime)
	{;}

	void setValue(QString value) {
		this->value = value;
	}

	QString getValue() {
		return value;
	}

	void setIsMinimumLeadTime(bool isminimumleadtime) {
		this->isminimumleadtime = isminimumleadtime;
	}

	bool isMinimumLeadTime() {
		return isminimumleadtime;
	}

	QString value;
	bool isminimumleadtime;
};

struct Preprocessor {
	Preprocessor(QString name, bool active, bool enabled):name(name), active(active), enabled(enabled){;}

	QString getName() {
		return name;
	}

	bool isActive() {
		return active;
	}

	bool isEnabled() {
		return enabled;
	}

	QString name;
	bool active;
	bool enabled;
};

struct RegressorFormulaBuilder {
	typedef AssignmentCategory AssignmentCategory_t;
	typedef std::pair<QString, UserParameterType *> UserParameter_t;
	typedef Preprocessor RegressorFormulaBuilderPreprocessor_t;

	COM_SUBSCRIPTION_FUNCTION(UserParameterAdded, communication::Signal<void(UserParameter_t)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveUserParameter, communication::Signal<void(UserParameter_t)>);

	QString getIdentifier() {
		return identifier;
	}

	QString getFormula() {
		return formula;
	}

	QString getFormulaDefinition() {
		return formulaDefinition;
	}

	AssignmentCategory *getAssignmentCategory() {
		return assignmentCategories.back();
	}

	std::vector<AssignmentCategory *> getAssignmentCategories() {
		return assignmentCategories;
	}

	bool isEditable() {
		return iseditable;
	}

	std::map<QString, UserParameterType *> getUserParameters() {
		return userParameters;
	}

	void removeUserParameter(std::pair<QString, UserParameterType *> userParameter) {
		RemoveUserParameterSignal(userParameter);
	}

	const std::vector<Preprocessor *> &getPreprocessors() {
		return preprocessors;
	}

	bool hasPreprocessors() {
		return preprocessors.empty();
	}

	QString identifier;
	QString formula;
	QString formulaDefinition;
	std::vector<AssignmentCategory *> assignmentCategories;
	std::vector<Preprocessor *> preprocessors;
	bool iseditable;
	std::map<QString, UserParameterType *> userParameters;
};

}  // namespace test

typedef test::RegressorFormulaBuilder RegressorFormulaBuilder_t;
typedef gui::RegressorFormulaBuilderEditWidget<test::RegressorFormulaBuilder> widget_t;

RegressorFormulaBuilder_t *regressorFormulaBuilderFactory() {
	RegressorFormulaBuilder_t *formulaBuilder = new RegressorFormulaBuilder_t;
	formulaBuilder->identifier = "T1 Chebyshev level 1";
	formulaBuilder->formula = "T1";
	formulaBuilder->formulaDefinition = "damreg.T1(col_);";

	return formulaBuilder;
}

widget_t *widgetFactory() {
	return new widget_t(FETCH_FROM_CTX(RegressorFormulaBuilder_t));
}


void setIdentifier(RegressorFormulaBuilder_t *, QString identifier) {
	qDebug() << __FUNCTION__ << ":" << identifier;
}

void setFormula(RegressorFormulaBuilder_t *, QString formula) {
	qDebug() << __FUNCTION__ << ":" << formula;
}

void setCategory(RegressorFormulaBuilder_t *, QString identifier) {
	qDebug() << __FUNCTION__ << ":" << identifier;
}

void setFormulaDefinition(RegressorFormulaBuilder_t *, QString formulaDefinition) {
	qDebug() << __FUNCTION__ << ":" << formulaDefinition;
}

void setUserParameterValue(std::pair<QString, test::UserParameterType *> userParameter, QString value) {
	qDebug() << __FUNCTION__ << " : for " << userParameter.first << " value " << value;
}

void setUserParameterIsMinimumLeadTime(std::pair<QString, test::UserParameterType *> userParameter, bool state) {
	qDebug() << __FUNCTION__ << " : for " << userParameter.first << " state " << state;
}

void createUserParameter(RegressorFormulaBuilder_t *formulaBuilder) {
	qDebug() << __FUNCTION__ << " : " << formulaBuilder->getIdentifier();
}

void removeUserParameter(RegressorFormulaBuilder_t *formulaBuilder, std::pair<QString, test::UserParameterType *> userParameter) {
	qDebug() << __FUNCTION__ << " : " << formulaBuilder->getIdentifier() << " - " << userParameter.first;
	formulaBuilder->removeUserParameter(userParameter);
}

void preprocessorStateChanged(test::Preprocessor *preprocessor, bool state) {
	qDebug() << __FUNCTION__ << " : " << preprocessor->getName() << " - " << state;
}

int
main(int argc, char *argv[]) {
	using namespace boost;
	using namespace app::context;
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(widget_t, widgetFactory, defaultCleanupStrategy<widget_t>);
	REGISTER_FULL_CTX(RegressorFormulaBuilder_t, regressorFormulaBuilderFactory, defaultCleanupStrategy<RegressorFormulaBuilder_t>);

	RegressorFormulaBuilder_t *formulaBuilder = FETCH_FROM_CTX(RegressorFormulaBuilder_t);

	test::AssignmentCategory waterLevel("water_level", "Water level");
	test::AssignmentCategory temperature("temperature", "Temperature");
	test::AssignmentCategory seasonal("seasonal", "Seasonal");

	std::vector<test::AssignmentCategory *> assignmentCategories;
	assignmentCategories.push_back(&waterLevel);
	assignmentCategories.push_back(&temperature);
	assignmentCategories.push_back(&seasonal);
	formulaBuilder->assignmentCategories = assignmentCategories;
	formulaBuilder->iseditable = false;


	LocalScope<test::UserParameterType> userParameterA(new test::UserParameterType("123", true));
	LocalScope<test::UserParameterType> userParameterB(new test::UserParameterType("hello world", false));
	LocalScope<test::UserParameterType> userParameterC(new test::UserParameterType("any value", false));
	formulaBuilder->userParameters["a"] = userParameterA.get();
	formulaBuilder->userParameters["b"] = userParameterB.get();
	formulaBuilder->userParameters["c"] = userParameterC.get();
	formulaBuilder->iseditable = false;


	LocalScope<test::Preprocessor> preprocessorA(new test::Preprocessor("preprocessor A", true, true));
	LocalScope<test::Preprocessor> preprocessorB(new test::Preprocessor("preprocessor B", false, false));
	formulaBuilder->preprocessors.push_back(preprocessorA.get());
	formulaBuilder->preprocessors.push_back(preprocessorB.get());

	widget_t *widget = FETCH_FROM_CTX(widget_t);


	using namespace boost;
	COM_SUBSCRIBE_FOR(widget, SetIdentifier, bind(setIdentifier, _1, _2));
	COM_SUBSCRIBE_FOR(widget, SetFormula, bind(setFormula, _1, _2));
	COM_SUBSCRIBE_FOR(widget, SetFormulaDefinition, bind(setFormulaDefinition, _1, _2));
	COM_SUBSCRIBE_FOR(widget, SetAssignmentCategory, bind(setCategory, _1, _2));
	COM_SUBSCRIBE_FOR(widget, UserParameterSetValue, bind(setUserParameterValue, _1, _2));
	COM_SUBSCRIBE_FOR(widget, UserParameterSetIsMinimumLeadTime, bind(setUserParameterIsMinimumLeadTime, _1, _2));
	COM_SUBSCRIBE_FOR(widget, CreateUserParameter, bind(createUserParameter, _1));
	COM_SUBSCRIBE_FOR(widget, RemoveUserParameter, bind(removeUserParameter, _1, _2));
	COM_SUBSCRIBE_FOR(widget, PreprocessorStateChanged, bind(preprocessorStateChanged, _1, _2));


	QInputDialog *inputDialog = new QInputDialog(&mainWindow);
	QString value = inputDialog->getText(0, "the title", "the label");
	qDebug() << value;
	mainWindow.setCentralWidget(widget);
	mainWindow.show();

	return app.exec();
}
