/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportWidgetTest.cpp
 *
 *  Created on: Jul 23, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QWidget>
#include <QLabel>
#include <QDebug>
#include <boost/bind.hpp>
#include "gui/ReportWidget.hpp"
#include "gui/SaveDialog.hpp"
#include "communication/Signals.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

namespace test {

struct ReportListElement {
	QString getName() {
		return name;
	}

	QString name;
};

struct TitleReportListElement : public ReportListElement {
	QString getTitle() {
		return title;
	}

	QString title;
};

struct Report {
	COM_SUBSCRIPTION_FUNCTION(MoveReportListElement, communication::Signal<void(int, int)>);
	COM_SUBSCRIPTION_FUNCTION(WriteSuccessful, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(WriteFailed, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(WriteTimedOut, communication::Signal<void(int)>);

	typedef ReportListElement ReportListElement_t;
	typedef TitleReportListElement TitleReportListElement_t;

	std::vector<ReportListElement_t *> getReportListElements() {
		return reportListElements;
	}

	TitleReportListElement_t *getTitleReportListElement() {
		return &titleReportListElement;
	}

	std::vector<double> getAvailablePlotScaleFactors() {
		return availablePlotScaleFactors;
	}

	double getPlotScaleFactor() {
		return availablePlotScaleFactors.at(2);
	}

	std::vector<int> getAvailableFontSizes() {
		return availableFontSizes;
	}

	double getFontSize() {
		return availableFontSizes.at(1);
	}

	QString getOutputFileName() {
		return "output.ext";
	}

	QString getFailMessage() {
		return "fail message";
	}


	TitleReportListElement titleReportListElement;
	std::vector<ReportListElement_t *> reportListElements;
	std::vector<double> availablePlotScaleFactors;
	std::vector<int> availableFontSizes;
};

}  // namespace test

typedef gui::ReportWidget<test::Report> ReportWidget_t;
typedef gui::SaveDialog<int> SaveDialog_t;

QWidget *reportListElementFactory(QWidget *parent, QString text) {
	QWidget *widgetOne = new QWidget(parent);
	QLabel *lblOne = new QLabel(text);
	widgetOne->setLayout(new QHBoxLayout);
	widgetOne->layout()->addWidget(lblOne);

	return widgetOne;
}

ReportWidget_t *reportWidgetFactory() {
	ReportWidget_t *reportWidget = new ReportWidget_t(FETCH_FROM_CTX(test::Report));

	reportWidget->addReportListElementWidget(reportListElementFactory(reportWidget, "one"));
	reportWidget->addReportListElementWidget(reportListElementFactory(reportWidget, "two"));
	reportWidget->addReportListElementWidget(reportListElementFactory(reportWidget, "three"));
	reportWidget->addReportListElementWidget(reportListElementFactory(reportWidget, "four"));

	return reportWidget;
}

SaveDialog_t *saveDialogWidgetFactory() {
	return new SaveDialog_t("title", "PDF (*.pdf);; Microsoft Word (*.docx);; Open Office (*.odt);; Rich Text (*.rtf);; Text (*.txt)");
}

void buildReport(test::Report *) {
	qDebug() << __FUNCTION__ << endl;
}

void plotScaleChanged(test::Report *, double value) {
	qDebug() << __FUNCTION__ << ":" << value;
}

void fontSizeChanged(test::Report *, int value) {
	qDebug() << __FUNCTION__ << ":" << value;
}

int
main(int argc, char *argv[]) {
	using namespace boost;

	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_DEFAULT_CTX(test::Report);
	REGISTER_FULL_CTX(ReportWidget_t, reportWidgetFactory, defaultCleanupStrategy<ReportWidget_t>);
	REGISTER_FULL_CTX(SaveDialog_t, saveDialogWidgetFactory, defaultCleanupStrategy<SaveDialog_t>);

	test::Report *report = FETCH_FROM_CTX(test::Report);
	report->titleReportListElement.name = "Title";
	report->titleReportListElement.title = "content";
	report->reportListElements.push_back(&report->titleReportListElement);
	report->availablePlotScaleFactors.push_back(0.5);
	report->availablePlotScaleFactors.push_back(1.0);
	report->availablePlotScaleFactors.push_back(2.0);
	report->availablePlotScaleFactors.push_back(3.0);
	report->availablePlotScaleFactors.push_back(4.0);
	report->availableFontSizes.push_back(10);
	report->availableFontSizes.push_back(11);
	report->availableFontSizes.push_back(12);

	ReportWidget_t *reportWidget = FETCH_FROM_CTX(ReportWidget_t);
	QWidget *widgetFive = reportListElementFactory(reportWidget, "five");
	reportWidget->addReportListElementWidget(widgetFive);
	reportWidget->moveReportListElementWidget(4, 3);

	COM_SUBSCRIBE_FOR(reportWidget, CreateReport, bind(buildReport, _1));
	COM_SUBSCRIBE_FOR(reportWidget, PlotScaleFactorChanged, bind(plotScaleChanged, _1, _2));
	COM_SUBSCRIBE_FOR(reportWidget, FontSizeChanged, bind(fontSizeChanged, _1, _2));

	mainWindow.show();
	reportWidget->show(); //use 'exec' to wait until the dialog has been closed or 'show' return immediately
	FETCH_FROM_CTX(SaveDialog_t)->getSaveLocation();

 return app.exec();
}
