/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * dataModelTimeSeriesWidgetTest.cpp
 *
 *  Created on: Jan 7, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QDebug>
#include <boost/bind.hpp>
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "gui/DataModelTimeSeriesWidget.hpp"
#include "gui/WizardNextIdStrategy.hpp"
#include "gui/DataModelWizardPages.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {

class DataFrameHeader {
public:
	DataFrameHeader(QString name, int index)
		:name(name), index(index)
	{
	}

	int getIndex() const {
		return index;
	}

	const QString getName() const {
		return name;
	}

private:
	QString name;
	int index;
};

struct DateDataPoint {
	DateDataPoint(QString date):date(date){;}

	QString getDate() {
		return date;
	}

	QString date;
};

class DateColumn {
public:
	DateColumn();
	DateColumn(DataFrameHeader &dataFrameHeader, int index)
		:dataFrameHeader(dataFrameHeader), index(index)
	{}

	int getIndex() const {
		return index;
	}

	DataFrameHeader &getHeader() {
		return dataFrameHeader;
	}

	std::vector<DateDataPoint> getValues() {
		std::vector<DateDataPoint> values;
		values.push_back(DateDataPoint("11.08.1983"));
		values.push_back(DateDataPoint("12.08.1983"));

		return values;
	}
private:
	DataFrameHeader dataFrameHeader;
	int index;
};

class DataModel : public ::communication::BoostSignalPolicy {
public:
	typedef DataFrameHeader Header_t;
	typedef DateColumn DateColumn_t;
	typedef std::vector<DataFrameHeader> Header_Container_t;

	DataModel(Header_Container_t &headers, DateColumn_t *dateColumn = NULL)
		:headers(headers), dateColumn(dateColumn)
	{}

	const Header_Container_t &getAssignableDateColumnHeaders() {
		return headers;
	}

	const Header_Container_t &getHeaders() {
		return headers;
	}

	bool isDateColumnEditable() {
		return false;
	}

	bool isDateColumnAssignmentComplete() {
		return true;
	}

	DateColumn *getDateColumn(QString format) {
		qDebug() << __FUNCTION__ << " : " << format;
		return dateColumn;
	}

	DateColumn *getDateColumn() {
		qDebug() << __FUNCTION__;
		return dateColumn;
	}

	QString getDateFormat() {
		return "%d.%m.%Y";
	}

	COM_SUBSCRIPTION_FUNCTION(DataModelLoaded, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(DateColumnChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(WaterLevelAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);
	COM_SUBSCRIPTION_FUNCTION(TemperatureAssignmentChanged, ::communication::BoostSignalPolicy::changed_signal_t);

private:
	Header_Container_t headers;
	DateColumn *dateColumn;
};
} /* namespace test */

typedef test::DataModel DataModel_t;
typedef gui::WizardNextIdStrategy<gui::DataModelWizardPages::Pages> Wizard_Next_Id_Strategy_t;
typedef gui::DataModelTimeSeriesWidget<DataModel_t, Wizard_Next_Id_Strategy_t > DataModel_Time_Series_Widget_t;

DataModel_t *dataModelFactory() {
	DataModel_t::Header_Container_t headerContainer;
	headerContainer.push_back(test::DataFrameHeader("MSVC1", 0));
	headerContainer.push_back(test::DataFrameHeader("data", 1));

	DataModel_t *dataModel;
	test::DateColumn *dateColumn = new test::DateColumn(headerContainer.at(1), 1);
	dataModel = new DataModel_t(headerContainer, dateColumn);

	return dataModel;
}

DataModel_Time_Series_Widget_t *dataModelAssignmentWizardPageFactory() {
	return new DataModel_Time_Series_Widget_t(NULL, FETCH_FROM_CTX(DataModel_t), FETCH_FROM_CTX(Wizard_Next_Id_Strategy_t), gui::DataModelWizardPages::TimeSeries);
}

bool operator==(test::DataFrameHeader const &header, test::DateColumn *dateColumn) {
	return dateColumn->getIndex() == header.getIndex();
}

void applyDateFormat(test::DataModel *, QString format) {
	qDebug() << __FUNCTION__ << " : " << format;
}

int
main(int argc, char *argv[])
{
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_t, dataModelFactory, defaultCleanupStrategy<DataModel_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_Time_Series_Widget_t, dataModelAssignmentWizardPageFactory, defaultCleanupStrategy<DataModel_Time_Series_Widget_t>);
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(Wizard_Next_Id_Strategy_t);

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(DataModel_Time_Series_Widget_t), ApplyDateFormat, boost::bind(applyDateFormat, _1, _2));

	mainWindow.setCentralWidget(FETCH_FROM_CTX(DataModel_Time_Series_Widget_t));
	mainWindow.show();

	return app.exec();
}
