/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelTests.cpp
 *
 *  Created on: Feb 19, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionModelTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <set>
#include "memory/OwnerhshipPolicies.hpp"
#include "model/api/IRegressionModel.hpp"
#include "model/RegressionModel.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

MOCK_CLASS(DateDataPoint) {
	bool operator==(DateDataPoint const &) const {
		return true;
	}
};

MOCK_CLASS(DateColumn) {
	MOCK_METHOD_EXT(getValueAt, 1, DateDataPoint(int), getValueAt);
};

MOCK_CLASS(BehaviorIndicatorColumn) {
};

MOCK_CLASS(RDataFrame) {
	bool operator==(RDataFrame const &other) const {
		return true;
	}
};

MOCK_CLASS(DataFrame) {
	MOCK_METHOD_EXT(getDataFrame, 0, RDataFrame &(), getDataFrame);
};

MOCK_CLASS(DataModel) {
	typedef DateDataPoint Date_Data_Point_t;
	typedef BehaviorIndicatorColumn Behavior_Indicator_Column_t;

	MOCK_METHOD_EXT(getDateColumn, 0, DateColumn *(), getDateColumn);
	MOCK_METHOD_EXT(getBehaviorIndicatorColumnWith, 1, BehaviorIndicatorColumn *(int), getBehaviorIndicatorColumnWith);
	MOCK_METHOD_EXT(getBehaviorIndicatorColumns, 0, std::set<BehaviorIndicatorColumn *>(), getBehaviorIndicatorColumns);
	MOCK_METHOD_EXT(getDataFrame, 0, DataFrame *(), getDataFrame);
};


MOCK_CLASS(TimeRange) {
	MOCK_METHOD_EXT(setFromDate, 1, void(DateDataPoint), setFromDate);
	MOCK_METHOD_EXT(getFromDate, 0, DateDataPoint &(), getFromDate);
	MOCK_METHOD_EXT(setToDate, 1, void(DateDataPoint), setToDate);
	MOCK_METHOD_EXT(getToDate, 0, DateDataPoint &(), getToDate);
};

MOCK_CLASS(AnalysisTool) {
};

MOCK_CLASS(RegressionFormula) {
	MOCK_METHOD_EXT(setOverallBestFitFormula, 0, void(), setOverallBestFitFormula);
	MOCK_METHOD_EXT(setBehaviorIndicatorColumn, 1, void(BehaviorIndicatorColumn *), setBehaviorIndicatorColumn);
	MOCK_METHOD_EXT(getBehaviorIndicatorColumn, 0, BehaviorIndicatorColumn *(), getBehaviorIndicatorColumn);
};

MOCK_CLASS(RegressionFormulaProvider) {
	typedef RegressionFormula Regression_Formula_type;

	MOCK_METHOD_EXT(load, 2, RegressionFormula *(DataFrame *, BehaviorIndicatorColumn *), load);
};

MOCK_CLASS(RegressionFit) {
};

MOCK_CLASS(RegressionFitProvider) {
	typedef RegressionFit RegressionFit_t;

	MOCK_METHOD_EXT(load, 2, RegressionFit *(RDataFrame &, RegressionFormula *), load);
};

typedef dambase::model::api::IRegressionModel RegressionModel_t;

using namespace app::context;
using namespace test::helper;

struct ApplicationContext {
	ApplicationContext()
	{
		behaviorIndicatorColumns.insert(&behaviorIndicatorColumn);
		MOCK_EXPECT(dataModel->getBehaviorIndicatorColumns).at_least(1).returns(behaviorIndicatorColumns);
		MOCK_EXPECT(regressionFormula->setBehaviorIndicatorColumn).once().with(&behaviorIndicatorColumn);

		//regressionModel = boost::shared_ptr<RegressionModel_t>(new dambase::model::RegressionModel(&dataModel, &timeRange, &regressionFormula, &regressionFitProvider));
		regressionModel = boost::shared_ptr<RegressionModel_t>(new dambase::model::RegressionModel());

	}

	TimeRange timeRange;
	DateDataPoint dateDataPoint;
	DateColumn dateColumn;
	BehaviorIndicatorColumn behaviorIndicatorColumn;
	DataFrame dataFrame;
	RDataFrame rDataFrame;
	boost::shared_ptr<DataModel> dataModel;
	boost::shared_ptr<RegressionFormula> regressionFormula;
	RegressionFit regressionFit;
	RegressionFitProvider regressionFitProvider;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	std::set<BehaviorIndicatorColumn *> behaviorIndicatorColumns;

	boost::shared_ptr<RegressionModel_t> regressionModel;
};


BOOST_FIXTURE_TEST_CASE(TestCtorThrowsExceptionIfDataModelHasNoBehaviorIndicatorSet, ApplicationContext) {
	mock::reset(dataModel);

	std::set<BehaviorIndicatorColumn *> emptyBehaviorIndicatorColumns;
	MOCK_EXPECT(dataModel.getBehaviorIndicatorColumns).at_least(1).returns(emptyBehaviorIndicatorColumns);
	BOOST_CHECK_THROW((new RegressionModel_t(&dataModel, &timeRange, &regressionFormula, &regressionFitProvider)), std::logic_error);
}


BOOST_FIXTURE_TEST_CASE(TestSetNameAndDescription, ApplicationContext) {
	QString name("name");
	QString description("description");

	regressionModel->setName(name);
	regressionModel->setDescription(description);

	BOOST_CHECK(regressionModel->getName() == name);
	BOOST_CHECK(regressionModel->getDescription() == description);
}


BOOST_FIXTURE_TEST_CASE(TestSetFromDate, ApplicationContext) {
	MOCK_EXPECT(dataModel.getDateColumn).at_least(1).returns(&dateColumn);
	MOCK_EXPECT(timeRange.setFromDate).once().with(dateDataPoint);
	MOCK_EXPECT(timeRange.getFromDate).once().returns(dateDataPoint);
	MOCK_EXPECT(dateColumn.getValueAt).once().with(0).returns(dateDataPoint);

	regressionModel->setFromDate(0);

	BOOST_CHECK(regressionModel->getFromDate() == dateDataPoint);
}

BOOST_FIXTURE_TEST_CASE(TestSetToDate, ApplicationContext) {
	MOCK_EXPECT(dataModel.getDateColumn).at_least(1).returns(&dateColumn);
	MOCK_EXPECT(timeRange.setToDate).once().with(dateDataPoint);
	MOCK_EXPECT(timeRange.getToDate).once().returns(dateDataPoint);
	MOCK_EXPECT(dateColumn.getValueAt).once().with(3).returns(dateDataPoint);

	regressionModel->setToDate(3);

	BOOST_CHECK(regressionModel->getToDate() == dateDataPoint);
}

BOOST_FIXTURE_TEST_CASE(TestSetBehaviorIndicatorThrowsAnExceptionIfNoBehaviorIndicatorForAGivenIndexExists, ApplicationContext) {
	MOCK_EXPECT(dataModel.getBehaviorIndicatorColumnWith).at_least(1).with(0).returns(NULL);

	BOOST_CHECK_THROW(regressionModel->setBehaviorIndicatorColumn(0), std::logic_error);
}

BOOST_FIXTURE_TEST_CASE(TestSetBehaviorIndicatorWithExistingBehaviorIndicator, ApplicationContext) {
	MOCK_EXPECT(dataModel.getBehaviorIndicatorColumnWith).at_least(1).with(0).returns(&behaviorIndicatorColumn);
	MOCK_EXPECT(regressionFormula.setBehaviorIndicatorColumn).at_least(1).with(&behaviorIndicatorColumn);

	regressionModel->setBehaviorIndicatorColumn(0);
}


BOOST_FIXTURE_TEST_CASE(TestSetupBestFittingRegressionFormulaHasNotBeenImplementedYet, ApplicationContext) {
	//MOCK_EXPECT(regressionFormula.setOverallBestFitFormula).once();

	BOOST_CHECK_THROW(regressionModel->setupBestFittingRegressionFormula(), std::exception);
}

BOOST_FIXTURE_TEST_CASE(CalculateRegressionFitUsesRegressionFitProvider, ApplicationContext) {
	MOCK_EXPECT(dataModel.getDataFrame).at_least(1).returns(&dataFrame);
	MOCK_EXPECT(dataFrame.getDataFrame).at_least(1).returns(rDataFrame);
	MOCK_EXPECT(regressionFitProvider.load).once().with(rDataFrame, &regressionFormula).returns(&regressionFit);

	regressionModel->calculateRegressionFit();
}

