/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrameOverviewStatisticsTests.cpp
 *
 *  Created on: Jan 29, 2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE DataFrameOverviewStatisticsTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <QString>
#include <QUrl>
#include <Rcpp.h>
#include "model/r/DataFrame.hpp"
#include "model/r/DataFrameProvider.hpp"
#include "model/r/DataFrameOverviewStatistics.hpp"
#include "model/r/DataFrameOverviewStatisticsProvider.hpp"
#include "RApplicationContext.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


typedef model::r::DataFrameProvider Data_Frame_Provider_t;
typedef model::r::DataFrameOverviewStatisticsProvider Data_Frame_Overview_Statistics_Provider_t;


class ApplicationContext : public test::RApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Provider_t, boost::bind(&ApplicationContext::dataFrameProviderFactory, this)),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Overview_Statistics_Provider_t, boost::bind(&ApplicationContext::dataFrameOverviewStatisticsProviderFactory, this))
	{;}

private:
	Data_Frame_Provider_t *dataFrameProviderFactory() {
		Data_Frame_Provider_t *dataFrameProvider = new Data_Frame_Provider_t;
		return dataFrameProvider;
	}

	Data_Frame_Overview_Statistics_Provider_t *dataFrameOverviewStatisticsProviderFactory() {
		return new Data_Frame_Overview_Statistics_Provider_t;
	}


private:
	GAC_SCOPE(Data_Frame_Provider_t)
	GAC_SCOPE(Data_Frame_Overview_Statistics_Provider_t)
};



BOOST_FIXTURE_TEST_CASE(TestTotalNumberOfDataPoints, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./overview_statistics_data.csv")));

	app::context::LocalScope<model::r::DataFrameOverviewStatistics> statistics(FETCH_FROM_CTX(Data_Frame_Overview_Statistics_Provider_t)->load(dataFrame.get()));

	BOOST_CHECK(statistics->getTotalNumberOfDataPoints() == 21);
}

BOOST_FIXTURE_TEST_CASE(TestAnyNumberOfDataPointsReturnZeroIfElementIsNotContainedInList, ApplicationContext) {
	Rcpp::List list = Rcpp::List::create();
	app::context::LocalScope<model::r::DataFrameOverviewStatistics> statistics(new model::r::DataFrameOverviewStatistics(list));

	BOOST_CHECK(statistics->getTotalNumberOfDataPoints() == 0);
	BOOST_CHECK(statistics->getNumberOfNotAvailableDataPoints() == 0);
	BOOST_CHECK(statistics->getNumberOfAvailableDataPoints() == 0);
}

BOOST_FIXTURE_TEST_CASE(TestNumberOfNotAvailableDataPoints, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data_with_empty_cells_data.csv")));

	app::context::LocalScope<model::r::DataFrameOverviewStatistics> statistics(FETCH_FROM_CTX(Data_Frame_Overview_Statistics_Provider_t)->load(dataFrame.get()));

	BOOST_CHECK(statistics->getNumberOfNotAvailableDataPoints() == 3);
}

BOOST_FIXTURE_TEST_CASE(TestTotalNumberOfDataPointsDoesNotDependOnNotAvailableDataPoints, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data_with_empty_cells_data.csv")));

	app::context::LocalScope<model::r::DataFrameOverviewStatistics> statistics(FETCH_FROM_CTX(Data_Frame_Overview_Statistics_Provider_t)->load(dataFrame.get()));

	BOOST_CHECK(statistics->getTotalNumberOfDataPoints() == 21);
}

BOOST_FIXTURE_TEST_CASE(TestAvailableNumberOfDataPointsIsTheTotalNumberOfDataPointsReducedByTheNotAvailableOnes, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data_with_empty_cells_data.csv")));

	app::context::LocalScope<model::r::DataFrameOverviewStatistics> statistics(FETCH_FROM_CTX(Data_Frame_Overview_Statistics_Provider_t)->load(dataFrame.get()));

	BOOST_CHECK(statistics->getNumberOfAvailableDataPoints() == statistics->getTotalNumberOfDataPoints() - statistics->getNumberOfNotAvailableDataPoints());
}
