/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrameTests.cpp
 *
 *  Created on: Dec 18, 2013
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RDataFrameTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <exception>
#include <QString>
#include <QUrl>
#include <QDebug>
#include "model/r/DateDataPoint.hpp"
#include "model/r/DataFrame.hpp"
#include "model/r/DataFrameProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "RApplicationContext.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


typedef model::r::DataFrameProvider Data_Frame_Provider_t;

class ApplicationContext : public test::RApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Provider_t, boost::bind(&ApplicationContext::dataFrameProviderFactory, this))
	{;}

private:
	Data_Frame_Provider_t *dataFrameProviderFactory() {
		Data_Frame_Provider_t *dataFrameProvider = new Data_Frame_Provider_t;
		return dataFrameProvider;
	}

private:
	GAC_SCOPE(Data_Frame_Provider_t)
};



BOOST_FIXTURE_TEST_CASE(TestCountNrOfRows, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getNrOfRows() == 7);
}

BOOST_FIXTURE_TEST_CASE(TestCountNrOfColumn, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getNrOfColumns() == 2);
}

BOOST_FIXTURE_TEST_CASE(TestHeaders, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(QString("date") == dataFrame->getHeader(0).getName());
	BOOST_CHECK(QString("MSV1") == dataFrame->getHeader(1).getName());
}

BOOST_FIXTURE_TEST_CASE(TestDateColumn, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getDateColumn() == NULL);
	dataFrame->setDateColumn(0);
	BOOST_CHECK(QString("date") == dataFrame->getDateColumn()->getHeader().getName());
	BOOST_CHECK(dataFrame->getDateColumn()->getIndex() == 0);
}

BOOST_FIXTURE_TEST_CASE(TestApplyDateFormatAndTestValidityOnProvider, ApplicationContext) {
	using namespace app::context;
	LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	dataFrame->setDateColumn(0);
	Rcpp::DataFrame newDataFrame = FETCH_FROM_CTX(Data_Frame_Provider_t)->applyDateFormat(dataFrame->getDataFrame(), dataFrame->getDateColumn()->getIndex(), QString("%d.%m.%Y"));
	Rcpp::CharacterVector newDateColumn = Rcpp::as<Rcpp::CharacterVector>(newDataFrame[dataFrame->getDateColumn()->getIndex()]);

	std::vector<std::string> values;
	BOOST_FOREACH(Rcpp::CharacterVector::value_type value, newDateColumn) {
		values.push_back(Rcpp::as<std::string>(value));
	}

	//test validity
	bool testDateColumnValidity = FETCH_FROM_CTX(Data_Frame_Provider_t)->testDateColumnValidity(newDataFrame, dataFrame->getDateColumn()->getIndex());


	BOOST_REQUIRE_EQUAL(7, values.size());
	BOOST_REQUIRE_EQUAL("1983-06-01", values.front());
	BOOST_CHECK(testDateColumnValidity);
}

BOOST_FIXTURE_TEST_CASE(TestInvalidDateFormatOnProvider, ApplicationContext) {
	using namespace app::context;
	LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	dataFrame->setDateColumn(0);
	Rcpp::DataFrame newDataFrame = FETCH_FROM_CTX(Data_Frame_Provider_t)->applyDateFormat(dataFrame->getDataFrame(), dataFrame->getDateColumn()->getIndex(), QString("%Y.%d.%m"));
	Rcpp::CharacterVector newDateColumn = Rcpp::as<Rcpp::CharacterVector>(newDataFrame[dataFrame->getDateColumn()->getIndex()]);

	std::vector<std::string> values;
	BOOST_FOREACH(Rcpp::CharacterVector::value_type value, newDateColumn) {
		values.push_back(Rcpp::as<std::string>(value));
	}

	//test validity
	bool testDateColumnValidity = FETCH_FROM_CTX(Data_Frame_Provider_t)->testDateColumnValidity(newDataFrame, dataFrame->getDateColumn()->getIndex());

	BOOST_REQUIRE_EQUAL(7, values.size());
	BOOST_REQUIRE_EQUAL("NA", values.front());
	BOOST_REQUIRE_EQUAL(false, testDateColumnValidity);
}

BOOST_FIXTURE_TEST_CASE(TestDateColumnDateDataPoints, ApplicationContext) {
	QString firstDateInDateColumn("1.6.1983");
	model::r::DateDataPoint firstDateDataPointInDateColumn(firstDateInDateColumn, 0);

	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));
	dataFrame->setDateColumn(0);

	BOOST_CHECK(dataFrame->getDateColumn()->getValues().size() == 7);
	BOOST_CHECK(dataFrame->getDateColumn()->getValues().at(0) == firstDateDataPointInDateColumn);
	BOOST_CHECK(dataFrame->getDateColumn()->getValueAt(0) == firstDateDataPointInDateColumn);
}

BOOST_FIXTURE_TEST_CASE(TestEstimateDateColumn, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	dataFrame->estimateDateColumn();
	BOOST_CHECK(dataFrame->getDateColumn() == 0);
}

BOOST_FIXTURE_TEST_CASE(TestGetSomeRawValues, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	std::vector<QString> values = dataFrame->getSomeRawValuesFor("date");
	BOOST_REQUIRE_EQUAL(6, values.size());
}


BOOST_FIXTURE_TEST_CASE(TestAddingTwiceTheSameBehaviorIndicatorReturnsFalseAndKeepsTheAlreadyExistingElement, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().empty() == true);
	BOOST_CHECK(dataFrame->assignBehaviorIndicatorColumn(1) == true);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 1);

	BOOST_CHECK(dataFrame->assignBehaviorIndicatorColumn(1) == false);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 1);
}


BOOST_FIXTURE_TEST_CASE(TestAddingTwoDifferentBehaviorIndicatorsReturnsTrueAndAddsBoth, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().empty() == true);
	BOOST_CHECK(dataFrame->assignBehaviorIndicatorColumn(0) == true);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 1);

	BOOST_CHECK(dataFrame->assignBehaviorIndicatorColumn(1) == true);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 2);
}

BOOST_FIXTURE_TEST_CASE(TestRemoveBehaviorIndicator, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->assignBehaviorIndicatorColumn(0) == true);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 1);
	BOOST_CHECK(dataFrame->removeBehaviorIndicatorColumn(0) == true);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 0);
}

BOOST_FIXTURE_TEST_CASE(TestRemoveTheSameBehaviorIndicatorTwiceReturnsFalse, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->assignBehaviorIndicatorColumn(0) == true);
	BOOST_CHECK(dataFrame->getBehaviorIndicatorColumns().size() == 1);
	BOOST_CHECK(dataFrame->removeBehaviorIndicatorColumn(0) == true);
	BOOST_CHECK(dataFrame->removeBehaviorIndicatorColumn(0) == false);
}


BOOST_FIXTURE_TEST_CASE(TestAddingTwiceTheSameWaterLevelReturnsFalseAndKeepsTheAlreadyExistingElement, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getWaterLevelColumns().empty() == true);
	BOOST_CHECK(dataFrame->assignWaterLevelColumn(1) == true);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 1);

	BOOST_CHECK(dataFrame->assignWaterLevelColumn(1) == false);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 1);
}

BOOST_FIXTURE_TEST_CASE(TestAddingTwoDifferentWaterLevelColumnsReturnsTrueAndAddsBoth, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getWaterLevelColumns().empty() == true);
	BOOST_CHECK(dataFrame->assignWaterLevelColumn(0) == true);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 1);

	BOOST_CHECK(dataFrame->assignWaterLevelColumn(1) == true);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 2);
}

BOOST_FIXTURE_TEST_CASE(TestRemoveWaterLevel, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->assignWaterLevelColumn(0) == true);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 1);
	BOOST_CHECK(dataFrame->removeWaterLevelColumn(0) == true);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 0);
}

BOOST_FIXTURE_TEST_CASE(TestRemoveTheSameWaterLevelTwiceReturnsFalse, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->assignWaterLevelColumn(0) == true);
	BOOST_CHECK(dataFrame->getWaterLevelColumns().size() == 1);
	BOOST_CHECK(dataFrame->removeWaterLevelColumn(0) == true);
	BOOST_CHECK(dataFrame->removeWaterLevelColumn(0) == false);
}



BOOST_FIXTURE_TEST_CASE(TestAddingTwiceTheSameTemperatureReturnsFalseAndKeepsTheAlreadyExistingElement, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getTemperatureColumns().empty() == true);
	BOOST_CHECK(dataFrame->assignTemperatureColumn(1) == true);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 1);

	BOOST_CHECK(dataFrame->assignTemperatureColumn(1) == false);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 1);
}


BOOST_FIXTURE_TEST_CASE(TestAddingTwoDifferentTemperaturesReturnsTrueAndAddsBoth, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->getTemperatureColumns().empty() == true);
	BOOST_CHECK(dataFrame->assignTemperatureColumn(0) == true);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 1);

	BOOST_CHECK(dataFrame->assignTemperatureColumn(1) == true);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 2);
}

BOOST_FIXTURE_TEST_CASE(TestRemoveTemperature, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->assignTemperatureColumn(0) == true);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 1);
	BOOST_CHECK(dataFrame->removeTemperatureColumn(0) == true);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 0);
}

BOOST_FIXTURE_TEST_CASE(TestRemoveTheSameTemperatureTwiceReturnsFalse, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	BOOST_CHECK(dataFrame->assignTemperatureColumn(0) == true);
	BOOST_CHECK(dataFrame->getTemperatureColumns().size() == 1);
	BOOST_CHECK(dataFrame->removeTemperatureColumn(0) == true);
	BOOST_CHECK(dataFrame->removeTemperatureColumn(0) == false);
}


BOOST_FIXTURE_TEST_CASE(TestAssignableColumnsContainAllHeadersIfNoHeaderHasBeenAssignedYet, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	size_t nrOfHeaders = dataFrame->getHeaders().size();

	BOOST_CHECK(dataFrame->getAssignableBehaviorIndicatorHeaders().size() == nrOfHeaders);
}

BOOST_FIXTURE_TEST_CASE(TestAssignableColumnsDoNotContainAlreadyAssignedColumns, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	size_t nrOfHeaders = dataFrame->getHeaders().size();

	dataFrame->setDateColumn(1);
	dataFrame->assignWaterLevelColumn(0);


	BOOST_CHECK(dataFrame->getAssignableBehaviorIndicatorHeaders().size() == nrOfHeaders - 2);
	BOOST_FOREACH(model::r::DataFrameHeader const &alreadyAssignedHeader, dataFrame->getAssignableBehaviorIndicatorHeaders()) {
		BOOST_CHECK(alreadyAssignedHeader.getIndex() != dataFrame->getDateColumn()->getHeader().getIndex());
	}
}

BOOST_FIXTURE_TEST_CASE(TestIsAssigned, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data.csv")));

	dataFrame->assignBehaviorIndicatorColumn(0);
	BOOST_CHECK(dataFrame->isBehaviorIndicatorColumnAssignedFor(0) == true);
	BOOST_CHECK(dataFrame->isBehaviorIndicatorColumnAssignedFor(1) == false);
}


BOOST_FIXTURE_TEST_CASE(MissingDataStrategyIgnoreDoesNotOmitAnyRowsWithMissingValues, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data_with_empty_cells_data.csv")));

	BOOST_CHECK(dataFrame->getNrOfRows() == 7);
}

BOOST_FIXTURE_TEST_CASE(MissingDataStrategySkipValuesOmitsRowsWithAtLeastOneMissingValue, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data_with_empty_cells_data.csv"), ",", 1));

	BOOST_CHECK(dataFrame->getNrOfRows() == 4); //3 rows with missing values
}

BOOST_FIXTURE_TEST_CASE(UnknownMissingDataStrategyThrowsAnException, ApplicationContext) {
	BOOST_CHECK_THROW(
			app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./data_with_empty_cells_data.csv"), ",", 999)),
			std::exception
	);
}


namespace test {

struct AssignmentCategory {
	QString getIdentifier() {
		return identifier;
	}

	QString identifier;
};

}  // namespace test
//About AssignmentCategory tests
BOOST_FIXTURE_TEST_CASE(GetAssignedInfluenceFactorsTests, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("./influence_factors_data.csv")));

	BOOST_CHECK(dataFrame->assignWaterLevelColumn(0));
	BOOST_CHECK(dataFrame->assignWaterLevelColumn(1));
	BOOST_CHECK(dataFrame->assignTemperatureColumn(2));

	test::AssignmentCategory waterLevelAssignmentCategory;
	waterLevelAssignmentCategory.identifier = "water_level";
	test::AssignmentCategory temperatureAssignmentCategory;
	temperatureAssignmentCategory.identifier = "temperature";

	model::r::DataFrame::Influence_Factor_Container_t assignedWaterLevelInfluenceFactors = dataFrame->getAssignedInfluenceFactorsFor(&waterLevelAssignmentCategory);
	BOOST_REQUIRE_EQUAL(2, assignedWaterLevelInfluenceFactors.size());

	model::r::DataFrame::Influence_Factor_Container_t assignedTemperatureInfluenceFactors = dataFrame->getAssignedInfluenceFactorsFor(&temperatureAssignmentCategory);
	BOOST_REQUIRE_EQUAL(1, assignedTemperatureInfluenceFactors.size());
}
