/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * EvaluateTests.cpp
 *
 *  Created on: Aug 4, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE EvaluateTests

#include <boost/test/unit_test.hpp>
#include <boost/foreach.hpp>
#include <Rcpp.h>
#include <stdexcept>
#include <QDebug>
#include "RApplicationContext.hpp"

namespace test {

struct ApplicationContext : public RApplicationContext {
};

}  // namespace test

BOOST_FIXTURE_TEST_CASE(TestValidExpression, test::ApplicationContext) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function evaluate = damBaseEnv["evaluate"];

	Rcpp::Environment argEnvir;
	argEnvir["x"] = Rcpp::NumericVector::create(12, 42);

	try {
		Rcpp::NumericVector result = Rcpp::as<Rcpp::NumericVector>(evaluate(
					Rcpp::Named("exp", "{x*2}"),
					Rcpp::Named("arg", argEnvir)
				));
		qDebug() << result.length();
		BOOST_FOREACH(Rcpp::NumericVector::value_type value, result) {
			qDebug() << "result: " << value;
		}
	}catch(std::exception &ex) {
		qDebug() << ex.what();
		BOOST_REQUIRE_EQUAL(true, false);
	}

	BOOST_REQUIRE_EQUAL(true, true);
}


BOOST_FIXTURE_TEST_CASE(TestParseExpression, test::ApplicationContext) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function testExpression = damBaseEnv["testExpression"];

	try {
		testExpression(Rcpp::Named("exp", "x+2;}"));
		BOOST_REQUIRE_EQUAL(true, false);
	}catch(std::exception &ex) {
		qDebug() << ex.what();
		BOOST_REQUIRE_EQUAL(true, true);
	}
}

BOOST_FIXTURE_TEST_CASE(TestEvaluateExpansionWithoutAnyPreprocessorsAndWithoutAnyUserParameters, test::ApplicationContext) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function evaluateExpansion = damBaseEnv["evaluateExpansion"];

	try {
		Rcpp::NumericVector column = Rcpp::NumericVector::create(1,2,3,4);
		Rcpp::CharacterVector dateColumn = Rcpp::CharacterVector::create("1995-01-01", "1995-01-02", "1995-01-03", "1995-01-04");
		Rcpp::NumericVector result = Rcpp::as<Rcpp::NumericVector>(evaluateExpansion(
					Rcpp::Named("exp", "{col_*2}"),
					Rcpp::Named("startDiff", 0),
					Rcpp::Named("finalCutDiff", 0),
					Rcpp::Named("dateColumn", dateColumn),
					Rcpp::Named("column", column)
				));
		std::vector<double> expectedResults;
		expectedResults.push_back(2);
		expectedResults.push_back(4);
		expectedResults.push_back(6);
		expectedResults.push_back(8);

		std::vector<double> results;
		BOOST_FOREACH(double resultValue, result) {
			results.push_back(resultValue);
		}
		BOOST_REQUIRE_EQUAL_COLLECTIONS(expectedResults.begin(), expectedResults.end(), results.begin(), results.end());
	}catch(std::exception &ex) {
		qDebug() << ex.what();
		BOOST_REQUIRE_EQUAL(true, false);
	}

	BOOST_REQUIRE_EQUAL(true, true);
}


BOOST_FIXTURE_TEST_CASE(TestEvaluateExpansionWithOnePreprocessorButWithoutAnyUserParameters, test::ApplicationContext) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function evaluateExpansion = damBaseEnv["evaluateExpansion"];

	try {
		Rcpp::NumericVector column = Rcpp::NumericVector::create(1,2,3,4);
		Rcpp::CharacterVector dateColumn = Rcpp::CharacterVector::create("1995-01-01", "1995-01-02", "1995-01-03", "1995-01-04");

		Rcpp::List preprocessors = Rcpp::List::create("{col_+1}");

		Rcpp::NumericVector result = Rcpp::as<Rcpp::NumericVector>(evaluateExpansion(
					Rcpp::Named("exp", "{col_*2}"),
					Rcpp::Named("startDiff", 0),
					Rcpp::Named("finalCutDiff", 0),
					Rcpp::Named("dateColumn", dateColumn),
					Rcpp::Named("column", column),
					Rcpp::Named("preprocessor", preprocessors)
				));
		qDebug() << result.length();
		BOOST_FOREACH(Rcpp::NumericVector::value_type value, result) {
			qDebug() << __FUNCTION__ << " - result: " << value;
		}
		std::vector<double> expectedResults;
		expectedResults.push_back(4);
		expectedResults.push_back(6);
		expectedResults.push_back(8);
		expectedResults.push_back(10);

		std::vector<double> results;
		BOOST_FOREACH(double resultValue, result) {
			results.push_back(resultValue);
		}
		BOOST_REQUIRE_EQUAL_COLLECTIONS(expectedResults.begin(), expectedResults.end(), results.begin(), results.end());
	}catch(std::exception &ex) {
		qDebug() << ex.what();
		BOOST_REQUIRE_EQUAL(true, false);
	}

	BOOST_REQUIRE_EQUAL(true, true);
}

BOOST_FIXTURE_TEST_CASE(TestEvaluateExpansionWithListOfPreprocessorsButWithoutAnyUserParameters, test::ApplicationContext) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function evaluateExpansion = damBaseEnv["evaluateExpansion"];

	try {
		Rcpp::NumericVector column = Rcpp::NumericVector::create(1,2,3,4);
		Rcpp::CharacterVector dateColumn = Rcpp::CharacterVector::create("1995-01-01", "1995-01-02", "1995-01-03", "1995-01-04");

		Rcpp::List preprocessors = Rcpp::List::create("{col_+1}", "{col_*3}"); //the second preprocessor works on the results of the first, so the preprocessors are executed from left to right

		Rcpp::NumericVector result = Rcpp::as<Rcpp::NumericVector>(evaluateExpansion(
					Rcpp::Named("exp", "{col_*2}"),
					Rcpp::Named("startDiff", 0),
					Rcpp::Named("finalCutDiff", 0),
					Rcpp::Named("dateColumn", dateColumn),
					Rcpp::Named("column", column),
					Rcpp::Named("preprocessors", preprocessors)
				));
		qDebug() << result.length();
		BOOST_FOREACH(Rcpp::NumericVector::value_type value, result) {
			qDebug() << __FUNCTION__ << " - result: " << value;
		}

		std::vector<double> expectedResults;
		expectedResults.push_back(6);
		expectedResults.push_back(12);
		expectedResults.push_back(18);
		expectedResults.push_back(24);

		std::vector<double> results;
		BOOST_FOREACH(double resultValue, result) {
			results.push_back(resultValue);
		}
		BOOST_REQUIRE_EQUAL_COLLECTIONS(expectedResults.begin(), expectedResults.end(), results.begin(), results.end());

	}catch(std::exception &ex) {
		qDebug() << ex.what();
		BOOST_REQUIRE_EQUAL(true, false);
	}

	BOOST_REQUIRE_EQUAL(true, true);
}

BOOST_FIXTURE_TEST_CASE(TestProvideEnvironment, test::ApplicationContext) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function environmentTest = damBaseEnv["environmentTest"];

	Rcpp::Environment otherEnvironment;

	{
		Rcpp::NumericVector result = Rcpp::as<Rcpp::NumericVector>(environmentTest(
				Rcpp::Named("env", otherEnvironment),
				Rcpp::Named("mode", "c")
			));

		std::vector<double> results;
		BOOST_FOREACH(double resultValue, result) {
			results.push_back(resultValue);
		}
		qDebug() << __FUNCTION__ << " (TestProvideEnvironment) : " << result.size() << " : " << results.at(0);
	}

	{
		Rcpp::NumericVector result = Rcpp::as<Rcpp::NumericVector>(environmentTest(
				Rcpp::Named("env", otherEnvironment),
				Rcpp::Named("mode", "p")
			));

		std::vector<double> results;
		BOOST_FOREACH(double resultValue, result) {
			results.push_back(resultValue);
		}
		qDebug() << __FUNCTION__ << " (TestProvideEnvironment) : " << result.size() << " : " << results.at(0);
	}
}
