/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalBehaviorTests.cpp
 *
 *  Created on: 21.10.2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionFitAnalysisPhysicalBehaviorTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>
#include <Rcpp.h>
#include <QDebug>
#include <QString>
#include "RApplicationContext.hpp"
#include "model/r/RegressionFitProvider.hpp"
#include "model/r/RegressionFitAnalysisPhysicalBehaviorProvider.hpp"
#include "model/r/RRegressionFitAnalysisPhysicalBehavior.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

namespace test {

struct NullStruct {};

struct Column {
	Column(){;}
	Column(QString name, int index):name(name){;}

	QString getName(){return name;}
	int getIndex(){return index;}

	QString name;
	int index;
};

struct DataFrame {
	DataFrame(){;}
	DataFrame(Column *dateColumn):dateColumn(dateColumn){;}

	Column *getDateColumn() {
		return dateColumn;
	}

	Column *dateColumn;
};

struct RegressionFitDataFrame {
	RegressionFitDataFrame(){;}
	RegressionFitDataFrame(Rcpp::DataFrame &dataFrame):dataFrame(dataFrame){;};
	Rcpp::DataFrame &getDataFrame() {
		return dataFrame;
	}
	Rcpp::DataFrame dataFrame;
};

struct RegressionFormula {
	typedef NullStruct Regressor_t;
	RegressionFormula(){;}

	std::string buildFormula() {
		return formula;
	}

	std::string formula;
};

}  // namespace test

typedef test::RegressionFormula Regression_Formula_t;
typedef test::RegressionFitDataFrame RegressionFitDataFrame_t;
typedef model::r::RegressionFit<Regression_Formula_t, RegressionFitDataFrame_t> Regression_Fit_t;
typedef model::r::RegressionFitProvider<Regression_Formula_t, RegressionFitDataFrame_t> Regression_Fit_Provider_t;
typedef model::r::RRegressionFitAnalysisPhysicalBehavior PhysicalBehavior_t;
typedef model::r::RegressionFitAnalysisPhysicalBehaviorProvider<PhysicalBehavior_t, Regression_Fit_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext  {
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Fit_Provider_t, boost::bind(&ApplicationContext::regressionFitProviderFactory, this)),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Formula_t, boost::bind(&ApplicationContext::regressionFormulaFactory, this))
	{;}

	void init(Rcpp::DataFrame rcppDataFrame) {
		/**
		 * Prepare date column on dataFrame
		 */
		Rcpp::Environment damBaseEnv("package:DamBASE");
		Rcpp::Function applyDateFormat = damBaseEnv["applyDateFormat"];

		regressionFitDataFrame.dataFrame = Rcpp::as<Rcpp::DataFrame>(applyDateFormat(
					Rcpp::Named("dataFrame", rcppDataFrame),
					Rcpp::Named("dateColumnNr", 1)
				));
	}

	Regression_Formula_t *regressionFormulaFactory() {
		return new Regression_Formula_t;
	}

	Regression_Fit_Provider_t *regressionFitProviderFactory() {
		Regression_Fit_Provider_t *regressionFitProvider = new Regression_Fit_Provider_t;

		return regressionFitProvider;
	}

	Regression_Fit_t *calculateRegressionFit() {
		return FETCH_FROM_CTX(Regression_Fit_Provider_t)->load(&regressionFitDataFrame, FETCH_FROM_CTX(Regression_Formula_t));
	}

	GAC_SCOPE(Regression_Fit_Provider_t)
	GAC_SCOPE(Regression_Formula_t)
	test::RegressionFitDataFrame regressionFitDataFrame;
	test::Column responseColumn;
	test::Column dateColumn;

};

struct CompleteApplicationContext : public ApplicationContext {
	void init() {
		dateColumn.index = 0;
		ApplicationContext::init(
				Rcpp::DataFrame::create(
						Rcpp::Named("DATE", Rcpp::CharacterVector::create("1.6.1983", "2.6.1983", "3.6.1983", "4.6.1983", "5.6.1983")),
						Rcpp::Named("y", Rcpp::NumericVector::create(1,4,9,16,32)),
						Rcpp::Named("x", Rcpp::NumericVector::create(1,2,3,4,5)),
						Rcpp::Named("z", Rcpp::NumericVector::create(10,14,19,16.8,32.9))
				)
		);
	}
};

BOOST_FIXTURE_TEST_CASE(TestCompletePolynomAppliedWithNumericResponse, CompleteApplicationContext) {
	using namespace app::context;

	init();
	responseColumn.name = "x";
	responseColumn.index = 2;

	std::string finalFormula("y ~ I(x) + I(z)");
	Rcpp::Formula applyedFormula(finalFormula);

	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula;
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	LocalScope<Provider_t> provider(new Provider_t());
	LocalScope<PhysicalBehavior_t> physicalBehavior(provider->load(
			regressionFit.get(),
			applyedFormula,
			regressionFitDataFrame.dataFrame,
			&dateColumn,
			"1983-6-1",
			"1983-6-5",
			0,
			&responseColumn,
			"test"
			//Rcpp::as<Rcpp::CharacterVector>(regressionFitDataFrame.dataFrame["DATE"])
			));


	typename PhysicalBehavior_t::ApplyedValuesContainer_t applyedValues = physicalBehavior->getApplyedValues();
	typename PhysicalBehavior_t::NumericResponseContainer_t numericResponse = physicalBehavior->getNumericResponse();

	BOOST_REQUIRE_EQUAL(5, applyedValues.size());
	BOOST_REQUIRE_CLOSE(-1.5259, applyedValues.at(0), 0.01);
	BOOST_REQUIRE_EQUAL("numeric", physicalBehavior->getResponseType().toStdString());
	BOOST_REQUIRE_EQUAL(5, numericResponse.size());
	BOOST_REQUIRE_EQUAL(0, physicalBehavior->getDateResponse().size());
}


BOOST_FIXTURE_TEST_CASE(TestCompletePolynomAppliedWithDateResponse, CompleteApplicationContext) {
	using namespace app::context;

	init();
	responseColumn.name = "DATE";
	responseColumn.index = 0;

	std::string finalFormula("y ~ I(x) + I(z)");
	Rcpp::Formula applyedFormula(finalFormula);

	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula;
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	LocalScope<Provider_t> provider(new Provider_t());
	LocalScope<PhysicalBehavior_t> physicalBehavior(provider->load(
			regressionFit.get(),
			applyedFormula,
			regressionFitDataFrame.dataFrame,
			&dateColumn,
			"1983-6-1",
			"1983-6-5",
			0,
			&responseColumn,
			"test"
			//Rcpp::as<Rcpp::CharacterVector>(regressionFitDataFrame.dataFrame["DATE"])
			));


	typename PhysicalBehavior_t::ApplyedValuesContainer_t applyedValues = physicalBehavior->getApplyedValues();
	typename PhysicalBehavior_t::NumericResponseContainer_t numericResponse = physicalBehavior->getNumericResponse();
	typename PhysicalBehavior_t::DateResponseContainer_t dateResponse = physicalBehavior->getDateResponse();

	BOOST_REQUIRE_EQUAL(5, applyedValues.size());
	BOOST_REQUIRE_CLOSE(-1.5259, applyedValues.at(0), 0.01);
	BOOST_REQUIRE_EQUAL("Date", physicalBehavior->getResponseType().toStdString());
	BOOST_REQUIRE_EQUAL(0, numericResponse.size());
	BOOST_REQUIRE_EQUAL(5, dateResponse.size());
}



BOOST_FIXTURE_TEST_CASE(TestPartialPolynomApplied, CompleteApplicationContext) {
	using namespace app::context;

	init();
	responseColumn.name = "x";
	responseColumn.index = 2;

	std::string finalFormula("y ~ I(x) + I(z)");
	Rcpp::Formula applyedFormula("y ~ I(x)");

	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula;
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	LocalScope<Provider_t> provider(new Provider_t());
	LocalScope<PhysicalBehavior_t> physicalBehavior(provider->load(
			regressionFit.get(),
			applyedFormula,
			regressionFitDataFrame.dataFrame,
			&dateColumn,
			"1983-6-1",
			"1983-6-5",
			0,
			&responseColumn,
			"test"
			//Rcpp::as<Rcpp::CharacterVector>(regressionFitDataFrame.dataFrame["DATE"])
			));


	typename PhysicalBehavior_t::ApplyedValuesContainer_t applyedValues = physicalBehavior->getApplyedValues();
	typename PhysicalBehavior_t::NumericResponseContainer_t numericResponse = physicalBehavior->getNumericResponse();
	typename PhysicalBehavior_t::DateResponseContainer_t dateResponse = physicalBehavior->getDateResponse();

	BOOST_REQUIRE_EQUAL(5, applyedValues.size());
	BOOST_REQUIRE_CLOSE(-8.9335289538295726, applyedValues.at(0), 0.01);
	BOOST_REQUIRE_EQUAL("numeric", physicalBehavior->getResponseType().toStdString());
	BOOST_REQUIRE_EQUAL(5, numericResponse.size());
	BOOST_REQUIRE_EQUAL(0, dateResponse.size());
}



BOOST_FIXTURE_TEST_CASE(TestCompleteComplexPolynomAppliedWithNumericResponse, CompleteApplicationContext) {
	using namespace app::context;

	init();
	responseColumn.name = "x";
	responseColumn.index = 2;

	std::string finalFormula("I(sin(y)) ~ I(cos(x)) + I(z)");
	Rcpp::Formula applyedFormula(finalFormula);

	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula;
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	LocalScope<Provider_t> provider(new Provider_t());
	LocalScope<PhysicalBehavior_t> physicalBehavior(provider->load(
			regressionFit.get(),
			applyedFormula,
			regressionFitDataFrame.dataFrame,
			&dateColumn,
			"1983-6-1",
			"1983-6-5",
			0,
			&responseColumn,
			"test"
			//Rcpp::as<Rcpp::CharacterVector>(regressionFitDataFrame.dataFrame["DATE"])
			));


	typename PhysicalBehavior_t::ApplyedValuesContainer_t applyedValues = physicalBehavior->getApplyedValues();
	typename PhysicalBehavior_t::NumericResponseContainer_t numericResponse = physicalBehavior->getNumericResponse();

	BOOST_REQUIRE_EQUAL(5, applyedValues.size());
	BOOST_REQUIRE_CLOSE(0.46054122902162309, applyedValues.at(0), 0.01);
	BOOST_REQUIRE_EQUAL("numeric", physicalBehavior->getResponseType().toStdString());
	BOOST_REQUIRE_EQUAL(5, numericResponse.size());
	BOOST_REQUIRE_EQUAL(0, physicalBehavior->getDateResponse().size());
}


BOOST_FIXTURE_TEST_CASE(TestCompleteComplexPolynomAppliedWithNumericResponseAndNoApplicationOfRegressionResults, CompleteApplicationContext) {
	using namespace app::context;

	init();
	responseColumn.name = "x";
	responseColumn.index = 2;

	std::string finalFormula("I(sin(y)) ~ I(cos(x)) + I(z)");
	Rcpp::Formula applyedFormula(finalFormula);

	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula;
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	LocalScope<Provider_t> provider(new Provider_t());
	LocalScope<PhysicalBehavior_t> physicalBehavior(provider->load(
			regressionFit.get(),
			applyedFormula,
			regressionFitDataFrame.dataFrame,
			&dateColumn,
			"1983-6-1",
			"1983-6-5",
			0,
			&responseColumn,
			"test",
			false
			//Rcpp::as<Rcpp::CharacterVector>(regressionFitDataFrame.dataFrame["DATE"])
			));


	typename PhysicalBehavior_t::ApplyedValuesContainer_t applyedValues = physicalBehavior->getApplyedValues();
	typename PhysicalBehavior_t::NumericResponseContainer_t numericResponse = physicalBehavior->getNumericResponse();

	BOOST_REQUIRE_EQUAL(5, applyedValues.size());
	BOOST_REQUIRE_CLOSE(10.54030230586814, applyedValues.at(0), 0.01);
	BOOST_REQUIRE_EQUAL("numeric", physicalBehavior->getResponseType().toStdString());
	BOOST_REQUIRE_EQUAL(5, numericResponse.size());
	BOOST_REQUIRE_EQUAL(0, physicalBehavior->getDateResponse().size());
}


BOOST_FIXTURE_TEST_CASE(TestCompletePolynomAppliedWithOverallMaxStartupTimeSet, CompleteApplicationContext) {
	using namespace app::context;

	init();
	responseColumn.name = "y";
	responseColumn.index = 1;

	std::string finalFormula("y ~ I(x)"); //we use a simple formula here and no applied fit related values to simplify the expectations
	Rcpp::Formula applyedFormula(finalFormula);

	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula;
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	LocalScope<Provider_t> provider(new Provider_t());
	LocalScope<PhysicalBehavior_t> physicalBehavior(provider->load(
			regressionFit.get(),
			applyedFormula,
			regressionFitDataFrame.dataFrame,
			&dateColumn,
			"1983-6-1",
			"1983-6-5",
			1,
			&responseColumn,
			"test",
			false //the test is easier to understand if we don't apply fit related values
			//Rcpp::as<Rcpp::CharacterVector>(regressionFitDataFrame.dataFrame["DATE"])
			));


	typename PhysicalBehavior_t::ApplyedValuesContainer_t applyedValues = physicalBehavior->getApplyedValues();
	typename PhysicalBehavior_t::NumericResponseContainer_t numericResponse = physicalBehavior->getNumericResponse();
	typename PhysicalBehavior_t::DateResponseContainer_t dateResponse = physicalBehavior->getDateResponse();

	std::vector<double> expectedApplyedValues;
	expectedApplyedValues.push_back(2);
	expectedApplyedValues.push_back(3);
	expectedApplyedValues.push_back(4);
	expectedApplyedValues.push_back(5);
	BOOST_REQUIRE_EQUAL(4, applyedValues.size());
	BOOST_REQUIRE_EQUAL_COLLECTIONS(
			expectedApplyedValues.begin(),
			expectedApplyedValues.end(),
			applyedValues.begin(),
			applyedValues.end());


	std::vector<double> expectedNumericResponseValues;
	expectedNumericResponseValues.push_back(4);
	expectedNumericResponseValues.push_back(9);
	expectedNumericResponseValues.push_back(16);
	expectedNumericResponseValues.push_back(32);
	BOOST_REQUIRE_EQUAL(4, numericResponse.size());
	BOOST_REQUIRE_EQUAL_COLLECTIONS(
			expectedNumericResponseValues.begin(),
			expectedNumericResponseValues.end(),
			numericResponse.begin(),
			numericResponse.end()
		);


	BOOST_REQUIRE_EQUAL(0, dateResponse.size());
}
