/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorOverviewTests.cpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionFitAnalysisRegressorOverviewTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <boost/math/special_functions/fpclassify.hpp>
#include <Rcpp.h>
#include "RApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "model/r/RRegressionFitAnalysisRegressorOverview.hpp"
#include "model/r/RegressionFitAnalysisRegressorOverviewProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

typedef typename test::RegressionFitApplicationContext::RegressionFit_t RegressionFit_t;
typedef model::r::RRegressionFitAnalysisRegressorOverview<typename RegressionFit_t::RegressionFormula_t> RegressionFitAnalysisRegressorOverview_t;
typedef model::r::RegressionFitAnalysisRegressorOverviewProvider<RegressionFit_t, RegressionFitAnalysisRegressorOverview_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Provider_t),
		DOUBLE_PRECISION(0.0001)
	{
		Rcpp::DataFrame rDataFrame = Rcpp::DataFrame::create(
					Rcpp::Named("date", Rcpp::CharacterVector::create("11.08.1983", "12.08.1983", "13.08.1983", "14.08.1983", "15.08.1983")),
					Rcpp::Named("y", Rcpp::NumericVector::create(1,2,3,4,5)),
					Rcpp::Named("x", Rcpp::NumericVector::create(2,4,6,8,10)),
					Rcpp::Named("w", Rcpp::NumericVector::create(2,4,8,10,11))
				);

		QString formula("x+w");
		fitCtx.waterLevelColumns.push_back(2);
		fitCtx.waterLevelColumns.push_back(3);
		fitCtx.init(rDataFrame, formula);
	}

	GAC_SCOPE(Provider_t);
	test::RegressionFitApplicationContext fitCtx;
	const double DOUBLE_PRECISION;
};


BOOST_FIXTURE_TEST_CASE(TestsLoadRegressorOverviewForRegressorItems, ApplicationContext) {
	using namespace app::context;
	LocalScope<RegressionFitAnalysisRegressorOverview_t> overview(FETCH_FROM_CTX(Provider_t)->load(fitCtx.regressionFit.get()));

	BOOST_CHECK(overview.get() != NULL);
	BOOST_REQUIRE_EQUAL(overview->getRegressorItems().size(), 2);
	typename RegressionFitAnalysisRegressorOverview_t::RegressorItem_t *firstRegressorItem = overview->getRegressorItems().at(0);

	/*
	qDebug() << firstRegressorItem->getCoefficient();
	qDebug() << firstRegressorItem->getTStat();
	qDebug() << firstRegressorItem->getPValue();
	qDebug() << firstRegressorItem->getStdError();
	qDebug() << firstRegressorItem->getVIF();
	qDebug() << firstRegressorItem->getStdCoefficient();
	qDebug() << "-";
	*/

	BOOST_CHECK(firstRegressorItem->getName() == "x");
	BOOST_REQUIRE_CLOSE(firstRegressorItem->getCoefficient(), 0.5f, DOUBLE_PRECISION);

	BOOST_REQUIRE_CLOSE(firstRegressorItem->getTStat(), 1.10714e+15, DOUBLE_PRECISION*100);
	BOOST_REQUIRE_CLOSE(firstRegressorItem->getPValue(), 8.15818e-31, DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(firstRegressorItem->getStdError(), 4.51613e-16, DOUBLE_PRECISION*10);
	BOOST_REQUIRE_CLOSE(firstRegressorItem->getVIF(), 25, DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(firstRegressorItem->getStdCoefficient(), 1, DOUBLE_PRECISION);
}

BOOST_FIXTURE_TEST_CASE(TestsLoadRegressorOverviewForInterceptItem, ApplicationContext) {
	using namespace app::context;
	LocalScope<RegressionFitAnalysisRegressorOverview_t> overview(FETCH_FROM_CTX(Provider_t)->load(fitCtx.regressionFit.get()));

	typename RegressionFitAnalysisRegressorOverview_t::InterceptItem_t *interceptItem = overview->getInterceptItem();
	BOOST_CHECK(interceptItem != NULL);
	BOOST_CHECK(interceptItem->getName() == "Intercept");

/*
	qDebug() << interceptItem->getCoefficient();
	qDebug() << interceptItem->getPValue();
	qDebug() << interceptItem->getStdError();
	qDebug() << interceptItem->getVIF();
*/

	BOOST_REQUIRE_CLOSE(interceptItem->getCoefficient(), 3, DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(interceptItem->getPValue(), 7.25171e-33, DOUBLE_PRECISION*10);
	BOOST_REQUIRE_CLOSE(interceptItem->getStdError(), 2.55471e-16, DOUBLE_PRECISION);
	BOOST_CHECK(boost::math::isnan(interceptItem->getVIF()));
	BOOST_CHECK(boost::math::isnan(interceptItem->getStdCoefficient()));
}
