/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFormulaTests.cpp
 *
 *  Created on: Feb 26, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RRegressionFormulaTests

#include <boost/shared_ptr.hpp>
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <set>
#include <Rcpp.h>
#include <exception>
#include <QDebug>
#include <QString>
#include "RApplicationContext.hpp"
#include "model/r/RegressionFormula.hpp"
#include "applicationcontext/scopes.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "communication/Signals.hpp"

using namespace app::context;
using namespace model::r;

namespace test {

MOCK_CLASS(Column) {
	MOCK_METHOD_EXT(getName, 0, QString(), getName);
};

MOCK_CLASS(AssignmentCategory) {
	MOCK_METHOD_EXT(getName, 0, QString(), getName);
};

MOCK_CLASS(RegressorFormulaBuilder) {
	typedef AssignmentCategory AssignmentCategory_t;
	MOCK_METHOD_EXT(calculateStartupTimesteps, 0, int(), calculateStartupTimesteps);
};

MOCK_CLASS(Regressor) {
	typedef Column Influence_Factor_t;

	MOCK_METHOD_EXT(getRegressorFormulaBuilder, 0, RegressorFormulaBuilder *(), getRegressorFormulaBuilder);
	MOCK_METHOD_EXT(getRFormula, 1, QString(int), getRFormula);
	MOCK_METHOD_EXT(getFormula, 0, QString(), getFormula);
	MOCK_METHOD_EXT(getAssignmentCategory, 0, AssignmentCategory *(), getAssignmentCategory);
	MOCK_METHOD_EXT(isActive, 0, bool(), isActive);
};



MOCK_CLASS(BehaviorIndicatorColumn) {
	MOCK_METHOD_EXT(getName, 0, QString(), getName);
};

MOCK_CLASS(MethodDescription) {
	typedef BehaviorIndicatorColumn Behavior_Indicator_Column_t;
};


MOCK_CLASS(DataFrame) {
	typedef BehaviorIndicatorColumn Behavior_Indicator_Column_t;
};

MOCK_CLASS(RegressionModelDefinition) {
	typedef Regressor Regressor_t;
	COM_SUBSCRIPTION_FUNCTION(Changed, communication::Signal<void()>);
	MOCK_METHOD_EXT(getRegressors, 0, std::vector<Regressor *>(), getRegressors);
};

MOCK_CLASS(IndexManager) {
	MOCK_METHOD_EXT(getNextAvailableIndex, 0, int(), getNextAvailableIndex);
};

typedef RegressionFormula<
			test::RegressionModelDefinition,
			test::RegressorFormulaBuilder,
			test::DataFrame,
			MethodDescription,
			memory::IgnoreOwnershipPolicy<MethodDescription>,
			memory::IgnoreOwnershipPolicy<test::RegressionModelDefinition>,
			memory::IgnoreOwnershipPolicy<test::RegressorFormulaBuilder>
		> Regression_Formula_t;

struct ApplicationContext : public RApplicationContext {
	ApplicationContext() {
		std::vector<MethodDescription *> methodDescriptions;
		std::vector<RegressorFormulaBuilder *> regressorFormulaBuilders;
		regressorFormulaBuilders.push_back(&formulaBuilderOne);
		regressorFormulaBuilders.push_back(&formulaBuilderTwo);
		regressionFormula = boost::shared_ptr<Regression_Formula_t>(new Regression_Formula_t(
				methodDescriptions,
				regressorFormulaBuilders,
				&dataFrame)
		);
		twoRegressors.push_back(&regressorOne);
		twoRegressors.push_back(&regressorTwo);
		regressionFormula->setBehaviorIndicatorColumn(&behaviorIndicatorColumn);
	}

	BehaviorIndicatorColumn behaviorIndicatorColumn;
	MethodDescription methodDescription;
	DataFrame dataFrame;
	Regressor regressorOne;
	Regressor regressorTwo;
	std::vector<test::Regressor *> twoRegressors;
	RegressionModelDefinition regressionModelDefinition;
	test::RegressorFormulaBuilder formulaBuilderOne;
	test::RegressorFormulaBuilder formulaBuilderTwo;
	boost::shared_ptr<Regression_Formula_t> regressionFormula;
};

} /* namespace test */

BOOST_FIXTURE_TEST_CASE(GetBuildFormulaAsRStringWithTwoActiveRegressorsTest, test::ApplicationContext) {
	MOCK_EXPECT(regressionModelDefinition.getRegressors).at_least(1).returns(twoRegressors);
	MOCK_EXPECT(regressorOne.getRFormula).with(0).returns("regrOne <- formula");
	MOCK_EXPECT(regressorTwo.getRFormula).with(0).returns("regrTwo <- formula");
	MOCK_EXPECT(regressorOne.isActive).returns(true);
	MOCK_EXPECT(regressorTwo.isActive).returns(true);
	MOCK_EXPECT(formulaBuilderOne.calculateStartupTimesteps).at_least(1).returns(0);
	MOCK_EXPECT(formulaBuilderTwo.calculateStartupTimesteps).at_least(1).returns(0);
	MOCK_EXPECT(regressorOne.getRegressorFormulaBuilder).at_least(1).returns(&formulaBuilderOne);
	MOCK_EXPECT(regressorTwo.getRegressorFormulaBuilder).at_least(1).returns(&formulaBuilderTwo);
	MOCK_EXPECT(behaviorIndicatorColumn.getName).returns("PENDULUM");

	regressionFormula->addCustomRegressionModelDefinition(&regressionModelDefinition);
	BOOST_REQUIRE_EQUAL("cutColumnByDiff(column = PENDULUM, cutDiff = 0) ~ regrOne <- formula+regrTwo <- formula", regressionFormula->buildFormulaAsRString());
}

BOOST_FIXTURE_TEST_CASE(GetBuildFormulaAsRStringUsesOnlyActiveRegressors, test::ApplicationContext) {
	MOCK_EXPECT(regressionModelDefinition.getRegressors).at_least(1).returns(twoRegressors);
	MOCK_EXPECT(regressorOne.getRFormula).with(0).returns("regrOne <- formula");
	MOCK_EXPECT(regressorTwo.getRFormula).with(0).returns("regrTwo <- formula");
	MOCK_EXPECT(regressorOne.isActive).returns(true);
	MOCK_EXPECT(regressorTwo.isActive).returns(false);
	MOCK_EXPECT(formulaBuilderOne.calculateStartupTimesteps).at_least(1).returns(0);
	MOCK_EXPECT(regressorOne.getRegressorFormulaBuilder).at_least(1).returns(&formulaBuilderOne);
	MOCK_EXPECT(behaviorIndicatorColumn.getName).returns("PENDULUM");

	regressionFormula->addCustomRegressionModelDefinition(&regressionModelDefinition);
	BOOST_REQUIRE_EQUAL("cutColumnByDiff(column = PENDULUM, cutDiff = 0) ~ regrOne <- formula", regressionFormula->buildFormulaAsRString());
}

BOOST_FIXTURE_TEST_CASE(ToStringUsesFormulaInsteadOfRFormulaForBuildingTheDefinitionAndOnlyActiveRegressors, test::ApplicationContext) {
	MOCK_EXPECT(regressionModelDefinition.getRegressors).at_least(1).returns(twoRegressors);
	MOCK_EXPECT(regressorOne.getFormula).returns("regrOne()");
	MOCK_EXPECT(regressorTwo.getFormula).returns("regrTwo()");
	MOCK_EXPECT(regressorOne.isActive).returns(true);
	MOCK_EXPECT(regressorTwo.isActive).returns(true);
	MOCK_EXPECT(behaviorIndicatorColumn.getName).returns("PENDULUM");

	regressionFormula->addCustomRegressionModelDefinition(&regressionModelDefinition);
	BOOST_REQUIRE_EQUAL("PENDULUM ~ regrOne() + regrTwo()", regressionFormula->toString().toStdString());
}

BOOST_FIXTURE_TEST_CASE(GetBuildFormulaAsRStringForAssignmentCategoryReturnsAFormulaOnlyWithRegressorsRelatedToTheGivenAssignmentCategory, test::ApplicationContext) {
	MOCK_EXPECT(regressionModelDefinition.getRegressors).at_least(1).returns(twoRegressors);
	MOCK_EXPECT(regressorOne.getRFormula).with(0).returns("regrOne");
	MOCK_EXPECT(regressorTwo.getRFormula).with(0).returns("regrTwo");
	MOCK_EXPECT(regressorOne.isActive).returns(true);
	MOCK_EXPECT(regressorTwo.isActive).returns(true);

	test::AssignmentCategory seasonalAssignmentCategory;
	test::AssignmentCategory notMatchingAssignmentCategory;
	MOCK_EXPECT(seasonalAssignmentCategory.getName).at_least(1).returns("seasonal");
	MOCK_EXPECT(notMatchingAssignmentCategory.getName).at_least(1).returns("not-matching");

	MOCK_EXPECT(regressorOne.getAssignmentCategory).at_least(1).returns(&seasonalAssignmentCategory);
	MOCK_EXPECT(regressorTwo.getAssignmentCategory).at_least(1).returns(&notMatchingAssignmentCategory);

	MOCK_EXPECT(formulaBuilderOne.calculateStartupTimesteps).at_least(1).returns(0);
	MOCK_EXPECT(formulaBuilderTwo.calculateStartupTimesteps).at_least(1).returns(0);
	MOCK_EXPECT(regressorOne.getRegressorFormulaBuilder).at_least(1).returns(&formulaBuilderOne);
	MOCK_EXPECT(regressorTwo.getRegressorFormulaBuilder).at_least(1).returns(&formulaBuilderTwo);
	MOCK_EXPECT(behaviorIndicatorColumn.getName).returns("PENDULUM");

	regressionFormula->addCustomRegressionModelDefinition(&regressionModelDefinition);
	BOOST_REQUIRE_EQUAL("cutColumnByDiff(column = PENDULUM, cutDiff = 0) ~ regrOne", regressionFormula->buildFormulaAsRStringForAssignmentCategory(&seasonalAssignmentCategory));
}
