/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionIntegrationTests.cpp
 *
 * Integration tests for regressors
 * Test each regressor with and without missing values
 *
 *  Created on: 13.10.2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionIntegrationTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <Rcpp.h>
#include <math.h>
#include <QDebug>
#include <QString>
#include "RApplicationContext.hpp"
#include "model/r/RegressionFitProvider.hpp"
#include "model/r/formulabuilder/ExpressionFormulaBuilderDefinitions.hpp"
#include "model/r/formulabuilder/ExpressionRegressorFormulaBuilder.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

namespace test {

struct NullStruct {};

struct Preprocessor {
	bool isActive() {
		return false;
	}

	QString resolveFormula() {
		return "";
	}
};

struct Column {
	Column(){;}
	Column(QString name):name(name){;}
	QString getName(){return name;}
	QString name;
};

struct DataFrame {
	DataFrame(){;}
	DataFrame(Column *dateColumn):dateColumn(dateColumn){;}

	Column *getDateColumn() {
		return dateColumn;
	}

	Column *dateColumn;
};

struct RegressionFitDataFrame {
	RegressionFitDataFrame(){;}
	RegressionFitDataFrame(Rcpp::DataFrame &dataFrame):dataFrame(dataFrame){;};
	Rcpp::DataFrame &getDataFrame() {
		return dataFrame;
	}
	Rcpp::DataFrame dataFrame;
};

struct RegressionFormula {
	typedef NullStruct Regressor_t;
	RegressionFormula(){;}

	std::string buildFormula() {
		return formula;
	}

	std::string formula;
};

}  // namespace test


typedef test::RegressionFormula Regression_Formula_t;
typedef test::RegressionFitDataFrame RegressionFitDataFrame_t;
typedef model::r::RegressionFit<Regression_Formula_t, RegressionFitDataFrame_t> Regression_Fit_t;
typedef model::r::RegressionFitProvider<Regression_Formula_t, RegressionFitDataFrame_t> Regression_Fit_Provider_t;
typedef model::r::ExpressionRegressorFormulaBuilderUserParameterType<QString> ExpressionRegressorFormulaBuilderUserParameterType_t;
typedef model::r::ExpressionRegressorFormulaBuilder<test::NullStruct, test::Preprocessor> erfb_t;
typedef test::NullStruct AssignmentCategory_t;


struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Fit_Provider_t, boost::bind(&ApplicationContext::regressionFitProviderFactory, this)),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Formula_t, boost::bind(&ApplicationContext::regressionFormulaFactory, this))
	{;}

	void init(Rcpp::DataFrame rcppDataFrame) {
		/**
		 * Prepare date column on dataFrame
		 */
		Rcpp::Environment damBaseEnv("package:DamBASE");
		Rcpp::Function applyDateFormat = damBaseEnv["applyDateFormat"];

		df = Rcpp::as<Rcpp::DataFrame>(applyDateFormat(
					Rcpp::Named("dataFrame", rcppDataFrame),
					Rcpp::Named("dateColumnNr", 1)
				));

		dateColumn.name = "DATE";
		influenceFactor.name = "x";
		dataFrame.dateColumn = &dateColumn;
		regressionFitDataFrame.dataFrame = df;

		er = boost::shared_ptr<erfb_t>(new erfb_t("id", assignmentCategories, &assignmentCategory, preprocessors, true));
	}

	Regression_Formula_t *regressionFormulaFactory() {
		return new Regression_Formula_t;
	}

	Regression_Fit_Provider_t *regressionFitProviderFactory() {
		Regression_Fit_Provider_t *regressionFitProvider = new Regression_Fit_Provider_t;

		return regressionFitProvider;
	}

	Regression_Fit_t *calculateRegressionFit() {
		return FETCH_FROM_CTX(Regression_Fit_Provider_t)->load(&regressionFitDataFrame, FETCH_FROM_CTX(Regression_Formula_t));
	}

	template<typename Definition>
	void setupBuilder(boost::shared_ptr<erfb_t> builder, Definition *definition) {

		typedef std::pair<QString, ExpressionRegressorFormulaBuilderUserParameterType_t> pair_t;
		BOOST_FOREACH(pair_t userParameter, definition->getUserParameters()) {
			builder->addUserParameter(userParameter.first, new ExpressionRegressorFormulaBuilderUserParameterType_t(userParameter.second.getValue(), userParameter.second.isMinimumLeadTime()));
		}

		builder->setFormula(definition->getFormula());
		builder->setFormulaDefinition(definition->getFormulaDefinition());
	}

	GAC_SCOPE(Regression_Fit_Provider_t)
	GAC_SCOPE(Regression_Formula_t)
	Rcpp::DataFrame df;
	test::DataFrame dataFrame;
	test::Column dateColumn;
	test::Column influenceFactor;
	test::RegressionFitDataFrame regressionFitDataFrame;
	std::vector<AssignmentCategory_t *> assignmentCategories;
	std::vector<test::Preprocessor *> preprocessors;
	AssignmentCategory_t assignmentCategory;
	boost::shared_ptr<erfb_t> er;
};

struct CompleteApplicationContext : public ApplicationContext {
	void init() {
		ApplicationContext::init(
				Rcpp::DataFrame::create(
						Rcpp::Named("DATE", Rcpp::CharacterVector::create("1.6.1983", "2.6.1983", "3.6.1983", "4.6.1983", "5.6.1983")),
						Rcpp::Named("x", Rcpp::NumericVector::create(1,2,3,4,5)),
						Rcpp::Named("y", Rcpp::NumericVector::create(1,4,9,16,32))
				)
		);
	}
};

struct NaNInInfluenceFactorApplicationContext : public ApplicationContext {
	void init() {
		ApplicationContext::init(
				Rcpp::DataFrame::create(
						Rcpp::Named("DATE", Rcpp::CharacterVector::create("1.6.1983", "2.6.1983", "3.6.1983", "4.6.1983", "5.6.1983")),
						Rcpp::Named("x", Rcpp::NumericVector::create(1,nan(NULL),3,4,5)),
						Rcpp::Named("y", Rcpp::NumericVector::create(1,4,9,16,32))
				)
		);
	}
};


/**
 * Chebyshev Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestChebyshevCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::ChebyshevFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(1));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	BOOST_REQUIRE_CLOSE(12.4, regressionFit->getIntercept(), 0.01);
}


/**
 * Chebyshev NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestChebyshevNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::ChebyshevFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(1));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	BOOST_REQUIRE_CLOSE(12.6857, regressionFit->getIntercept(), 0.1);
}

/**
 * Seasonal (depends only on the date column, so we cannot setup a testcase for nan)
 */

BOOST_FIXTURE_TEST_CASE(TestSeasonalCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::SeasonalFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(true, 2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();


	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(339.159, regressionFit->getIntercept(), 0.01);
}


/**
 * Power Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestPowerCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::PowerFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(8.4, regressionFit->getIntercept(), 0.01);
}


/**
 * Power NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestPowerNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::PowerFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(16.4257, regressionFit->getIntercept(), 0.01);
}


/**
 * ExponentialBuilderDefinition Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestExponentialBuilderDefinitionCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::ExponentialBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(5416.7, regressionFit->getIntercept(), 0.01);
}


/**
 * ExponentialBuilderDefinition NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestExponentialBuilderDefinitionNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::ExponentialBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(5416.7, regressionFit->getIntercept(), 0.01);
}


/**
 * SquareRootBuilderDefinition Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestSquareRootBuilderDefinitionCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::SquareRootBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t());
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-3.80461, regressionFit->getIntercept(), 0.01);
}

/**
 * SquareRootBuilderDefinition NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestSquareRootBuilderDefinitionNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::SquareRootBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t());
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-3.80461, regressionFit->getIntercept(), 0.01);
}


/**
 * LogarithmusNaturalisBuilderDefinition Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestLogarithmusNaturalisBuilderDefinitionCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::LogarithmusNaturalisBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t());
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-2.4125, regressionFit->getIntercept(), 0.01);
}


/**
 * LogarithmusNaturalisBuilderDefinition NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestLogarithmusNaturalisBuilderDefinitionNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::LogarithmusNaturalisBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t());
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-2.4125, regressionFit->getIntercept(), 0.01);
}


/**
 * DriftPowerBuilderDefinition Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestDriftPowerBuilderDefinitionCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::DriftPowerBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(1.22759, regressionFit->getIntercept(), 0.01);
}

/**
 * DriftPowerBuilderDefinition NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestDriftPowerBuilderDefinitionNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::DriftPowerBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(1.22759, regressionFit->getIntercept(), 0.01);
}


/**
 * ExtraSqrtRampBuilderDefinition Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestExtraSqrtRampBuilderDefinitionCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::ExtraSqrtRampBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(0.863105, regressionFit->getIntercept(), 0.01);
}


/**
 * ExtraSqrtRampBuilderDefinition NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestExtraSqrtRampBuilderDefinitionNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::ExtraSqrtRampBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-1.90129, regressionFit->getIntercept(), 0.01);
}

/**
 * ExtraColumnTimesStepBuilderDefinition Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestExtraColumnTimesStepBuilderDefinitionCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::ExtraColumnTimesStepBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-3.75676, regressionFit->getIntercept(), 0.01);
}

/**
 * ExtraColumnTimesStepBuilderDefinition NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestExtraColumnTimesStepBuilderDefinitionNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::ExtraColumnTimesStepBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2));
	setupBuilder(er, bd.get());
	QString finalFormula = "y ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 0);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(-2, regressionFit->getIntercept(), 0.01);
}

/**
 * TemperatureOneDimensionalHeatEquation Complete case
 */
BOOST_FIXTURE_TEST_CASE(TestTemperatureOneDimensionalHeatEquationCompleteCase, CompleteApplicationContext) {
	using namespace app::context;
	typedef model::r::TemperatureOneDimensionalHeatEquation<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2, 0.1, 365, 1));
	setupBuilder(er, bd.get());
	QString behaviorIndicatorFormula = QString("cutColumnByDiff(")
			+ QString("column = ") + "y"
			+ QString(", cutDiff = ") + QString::number(1)
			+ QString(")");
	QString finalFormula = behaviorIndicatorFormula + " ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 1);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(3.88927, regressionFit->getIntercept(), 0.01);
}

/**
 * TemperatureOneDimensionalHeatEquation NaN case
 */
BOOST_FIXTURE_TEST_CASE(TestTemperatureOneDimensionalHeatEquationNaNCase, NaNInInfluenceFactorApplicationContext) {
	using namespace app::context;
	typedef model::r::TemperatureOneDimensionalHeatEquation<ExpressionRegressorFormulaBuilderUserParameterType_t> bd_t;

	init();
	LocalScope<bd_t> bd(new bd_t(2, 0.1, 365, 1));
	setupBuilder(er, bd.get());
	QString behaviorIndicatorFormula = QString("cutColumnByDiff(")
			+ QString("column = ") + "y"
			+ QString(", cutDiff = ") + QString::number(1)
			+ QString(")");
	QString finalFormula = behaviorIndicatorFormula + " ~ " + er->resolveFormulaDefinition(&dataFrame, &influenceFactor, 1);
	FETCH_FROM_CTX(Regression_Formula_t)->formula = finalFormula.toStdString();
	LocalScope<Regression_Fit_t> regressionFit(calculateRegressionFit());

	//qDebug() << "Intercept: " << (double)regressionFit->getIntercept();
	BOOST_REQUIRE_CLOSE(0.788832, regressionFit->getIntercept(), 0.01);
}
