/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderManagerTests.cpp
 *
 *  Created on: Jun 3, 2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE RegressorFormulaBuilderManagerTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/shared_ptr.hpp>
#include <set>
#include <string>
#include <QString>
#include "RegressorFormulaBuilder.hpp"
#include "RegressorFormulaBuilderManager.hpp"

using namespace boost;

namespace test {

MOCK_CLASS(InfluenceFactor) {
	bool operator==(InfluenceFactor const &other) const {return true;}
};

MOCK_CLASS(TimeRange) {
	MOCK_METHOD_EXT(getDurationInDays, 0, double(), getDurationInDays);
};

MOCK_CLASS(DataFrame) {
	typedef InfluenceFactor Column_t;
	typedef InfluenceFactor DateColumn_t;
	//MOCK_METHOD_EXT(getWaterLevelColumns, 0, std::set<InfluenceFactor *>(), getWaterLevelColumns);
	MOCK_METHOD_EXT(findWaterLevelColumnByName, 1, Column_t *(QString), findWaterLevelColumnByName);
	MOCK_METHOD_EXT(getDateColumn, 0, Column_t *(), getDateColumn);
};

MOCK_CLASS(AnyRegressionFormulaBuilder) {
};

MOCK_CLASS(RegressorFormulaBuilderFactory) {
	typedef AnyRegressionFormulaBuilder value_type;

	template<typename T, typename _InfluenceFactor>
	AnyRegressionFormulaBuilder *createForId(QString regressorFormula, InfluenceFactor *influenceFactor) {
		return delegateCreateForId(regressorFormula, influenceFactor);
	}
	MOCK_METHOD_EXT(delegateCreateForId, 2, AnyRegressionFormulaBuilder *(QString, InfluenceFactor *), delegateCreateForId);

	template<typename T, typename _InfluenceFactor>
	AnyRegressionFormulaBuilder *createForPower(QString regressorFormula, InfluenceFactor *influenceFactor, QString power) {
		return delegateCreateForPower(regressorFormula, influenceFactor, power);
	}
	MOCK_METHOD_EXT(delegateCreateForPower, 3, AnyRegressionFormulaBuilder *(QString, InfluenceFactor *, QString), delegateCreateForPower);

	template<typename T, typename _InfluenceFactor>
	AnyRegressionFormulaBuilder *createForChebyshev(QString regressorFormula, InfluenceFactor *influenceFactor, QString level) {
		return delegateCreateForChebyshev(regressorFormula, influenceFactor, level);
	}
	MOCK_METHOD_EXT(delegateCreateForChebyshev, 3, AnyRegressionFormulaBuilder *(QString, InfluenceFactor *, QString), delegateCreateForChebyshev);



	template<typename T, typename _DateColumn>
	AnyRegressionFormulaBuilder *createForSinus(QString regressorFormula, InfluenceFactor *influenceFactor, QString factor) {
		return delegateCreateForSinus(regressorFormula, influenceFactor, factor);
	}
	MOCK_METHOD_EXT(delegateCreateForSinus, 3, AnyRegressionFormulaBuilder *(QString, InfluenceFactor *, QString), delegateCreateForSinus);

	template<typename T, typename _DateColumn>
	AnyRegressionFormulaBuilder *createForCosinus(QString regressorFormula, InfluenceFactor *influenceFactor, QString factor) {
		return delegateCreateForCosinus(regressorFormula, influenceFactor, factor);
	}
	MOCK_METHOD_EXT(delegateCreateForCosinus, 3, AnyRegressionFormulaBuilder *(QString, InfluenceFactor *, QString), delegateCreateForCosinus);
};

typedef model::r::RegressorFormulaBuilderManager<test::RegressorFormulaBuilderFactory, test::TimeRange, test::DataFrame> manager_t;

struct ApplicationContext {
	ApplicationContext(){
		manager = shared_ptr<manager_t>(new manager_t(&factory, &timeRange, &dataFrame));
	}

	QString formula;

	test::TimeRange timeRange;
	test::DataFrame dataFrame;
	test::AnyRegressionFormulaBuilder anyRegressorFormulaBuilder;
	test::RegressorFormulaBuilderFactory factory;
	InfluenceFactor column;
	shared_ptr<manager_t> manager;
};

} // namespace test


BOOST_FIXTURE_TEST_CASE(TestCreateForWaterLevelWithIdFormula, test::ApplicationContext) {
	formula = "MSVC1";
	MOCK_EXPECT(dataFrame.findWaterLevelColumnByName).at_least(1).with("MSVC1").returns(&column);
	MOCK_EXPECT(factory.delegateCreateForId).with(formula, &column).returns(&anyRegressorFormulaBuilder);

	manager->createForWaterLevel(formula);
}


BOOST_FIXTURE_TEST_CASE(TestCreateForWaterLevelWithPowerFormula, test::ApplicationContext) {
	formula = "MSVC1^2";
	QString power = "2";
	MOCK_EXPECT(dataFrame.findWaterLevelColumnByName).at_least(1).with("MSVC1").returns(&column);
	MOCK_EXPECT(factory.delegateCreateForPower).with(formula, &column, power).returns(&anyRegressorFormulaBuilder);

	manager->createForWaterLevel(formula);
}


BOOST_FIXTURE_TEST_CASE(TestCreateForWaterLevelWithChebyshevFormula, test::ApplicationContext) {
	formula = "T1(MSVC1)";
	QString level = "1";
	MOCK_EXPECT(dataFrame.findWaterLevelColumnByName).at_least(1).with("MSVC1").returns(&column);
	MOCK_EXPECT(factory.delegateCreateForChebyshev).with(formula, &column, level).returns(&anyRegressorFormulaBuilder);

	manager->createForWaterLevel(formula);
}

BOOST_FIXTURE_TEST_CASE(TestCreateForTemperatureWithSimpleSinusFormula, test::ApplicationContext) {
	formula = "sin(s)";
	QString factor = "1";
	MOCK_EXPECT(dataFrame.getDateColumn).returns(&column);
	MOCK_EXPECT(factory.delegateCreateForSinus).with(formula, &column, factor).returns(&anyRegressorFormulaBuilder);

	manager->createForTemperature(formula);
}
