/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderTests.cpp
 *
 *  Created on: Jun 3, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressorFormulaBuilderTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/shared_ptr.hpp>
#include "ExpressionRegressorFormulaBuilder.hpp"
#include "RegressorFormulaBuilder.hpp"
#include "IdRegressorFormulaBuilder.hpp"
#include "PowerRegressorFormulaBuilder.hpp"
#include "ChebyshevRegressorFormulaBuilder.hpp"
#include "SinusRegressorFormulaBuilder.hpp"

using namespace boost;

namespace test {

MOCK_CLASS(InfluenceFactor) {
	MOCK_METHOD_EXT(getName, 0, QString(), getName);
};

MOCK_CLASS(GenericFormulaBuilder) {
	MOCK_METHOD_EXT(getInfluenceFactor, 0, InfluenceFactor*(), getInfluenceFactor);
	MOCK_METHOD_EXT(getFormula, 0, QString(), getFormula);
};

typedef test::GenericFormulaBuilder grfb_t;
template<typename Builder>
struct ApplicationContext {
	QString formula;
	InfluenceFactor influenceFactor;
	grfb_t grfb;
	shared_ptr<Builder> builder;
};

} // namespace test

typedef model::r::IdRegressorFormulaBuilder<test::grfb_t, memory::IgnoreOwnershipPolicy<test::grfb_t> > id_builder_t;
BOOST_FIXTURE_TEST_CASE(TestIdBuilderGetRFormula, test::ApplicationContext<id_builder_t>) {
	MOCK_EXPECT(influenceFactor.getName).returns("MSVC1");
	MOCK_EXPECT(grfb.getInfluenceFactor).at_least(1).returns(&influenceFactor);
	builder = shared_ptr<id_builder_t>(new id_builder_t(&grfb));

	BOOST_REQUIRE_EQUAL(builder->getRFormula().toStdString(), std::string("I(scale(MSVC1, scale = FALSE))"));
}


BOOST_FIXTURE_TEST_CASE(TestIdBuilderGetFormula, test::ApplicationContext<id_builder_t>) {
	MOCK_EXPECT(grfb.getFormula).at_least(1).returns("MSVC1");
	builder = shared_ptr<id_builder_t>(new id_builder_t(&grfb));

	BOOST_REQUIRE_EQUAL(builder->getFormula().toStdString(), std::string("MSVC1"));
}


typedef model::r::PowerRegressorFormulaBuilder<test::grfb_t, memory::IgnoreOwnershipPolicy<test::grfb_t> > power_builder_t;
BOOST_FIXTURE_TEST_CASE(TestPowerBuilderGetRFormula, test::ApplicationContext<power_builder_t>) {
	MOCK_EXPECT(influenceFactor.getName).returns("MSVC1");
	MOCK_EXPECT(grfb.getInfluenceFactor).at_least(1).returns(&influenceFactor);
	builder = shared_ptr<power_builder_t>(new power_builder_t(&grfb, "2"));

	BOOST_REQUIRE_EQUAL(builder->getRFormula().toStdString(), std::string("I(scale(MSVC1, scale = FALSE)^2)"));
}


BOOST_FIXTURE_TEST_CASE(TestPowerBuilderGetFormula, test::ApplicationContext<power_builder_t>) {
	MOCK_EXPECT(grfb.getFormula).at_least(1).returns("MSVC1^2");
	builder = shared_ptr<power_builder_t>(new power_builder_t(&grfb, "2"));

	BOOST_REQUIRE_EQUAL(builder->getFormula().toStdString(), std::string("MSVC1^2"));
}


typedef model::r::ChebyshevRegressorFormulaBuilder<test::grfb_t, memory::IgnoreOwnershipPolicy<test::grfb_t> > cheby_builder_t;
BOOST_FIXTURE_TEST_CASE(TestChebyBuilderGetRFormula, test::ApplicationContext<cheby_builder_t>) {
	MOCK_EXPECT(influenceFactor.getName).returns("MSVC1");
	MOCK_EXPECT(grfb.getInfluenceFactor).at_least(1).returns(&influenceFactor);
	builder = shared_ptr<cheby_builder_t>(new cheby_builder_t(&grfb, "1"));

	BOOST_REQUIRE_EQUAL(builder->getRFormula().toStdString(), std::string("I(T1(scale(MSVC1, scale = FALSE)))"));
}

BOOST_FIXTURE_TEST_CASE(TestChebyshevBuilderGetFormula, test::ApplicationContext<cheby_builder_t>) {
	MOCK_EXPECT(grfb.getFormula).at_least(1).returns("T1(MSVC1)");
	builder = shared_ptr<cheby_builder_t>(new cheby_builder_t(&grfb, "1"));

	BOOST_REQUIRE_EQUAL(builder->getFormula().toStdString(), std::string("T1(MSVC1)"));
}


typedef model::r::SinusRegressorFormulaBuilder<test::grfb_t, memory::IgnoreOwnershipPolicy<test::grfb_t> > sinus_builder_t;
BOOST_FIXTURE_TEST_CASE(TestSinusBuilderGetRFormula, test::ApplicationContext<sinus_builder_t>) {
	MOCK_EXPECT(influenceFactor.getName).returns("DATE");
	MOCK_EXPECT(grfb.getInfluenceFactor).at_least(1).returns(&influenceFactor);
	QString factor = "1";
	builder = shared_ptr<sinus_builder_t>(new sinus_builder_t(&grfb, factor));

	BOOST_REQUIRE_EQUAL(builder->getRFormula().toStdString(), std::string("I(temp.sin(dateColumn = DATE, factor = 1))"));
}


BOOST_FIXTURE_TEST_CASE(TestSinusBuilderGetFormula, test::ApplicationContext<sinus_builder_t>) {
	QString factor = "1";
	MOCK_EXPECT(grfb.getFormula).at_least(1).returns("sin(s)");
	builder = shared_ptr<sinus_builder_t>(new sinus_builder_t(&grfb, factor));

	BOOST_REQUIRE_EQUAL(builder->getFormula().toStdString(), std::string("sin(s)"));
}
