/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * UiManagerTests.cpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE UiManagerTests
#include <vector>
#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <QObject>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "ui/UiManager.hpp"

using namespace app::context;

MOCK_CLASS(NullStruct){};


MOCK_CLASS(Analysis) {
	MOCK_METHOD_EXT(setParent, 1, void(QObject *), setParent);
};

MOCK_CLASS(AnalysisFactory) {
	MOCK_METHOD_EXT(create, 0, Analysis*(), create)
};

MOCK_CLASS(AnalysisOverviewUiFactory) {

};


MOCK_CLASS(WizardUi) {
	MOCK_METHOD_EXT(setParent, 1, void(QObject *), setParent);
};


MOCK_CLASS(SubscriptionManager) {
	void subscribe(WizardUi *wizard) {
		delegateSubscribe(wizard);
	}

	MOCK_METHOD_EXT(delegateSubscribe, 1, void(WizardUi *), delegateSubscribe);
};

MOCK_CLASS(AnalysisUiFactory) {
public:
	template<typename OverviewUi, typename A>
	OverviewUi *create(boost::shared_ptr<A> a);

	MOCK_METHOD_EXT(createUiDelegate, 1, WizardUi*(boost::shared_ptr<Analysis>), createUiDelegate);
};


template<>
WizardUi *AnalysisUiFactory::create(boost::shared_ptr<Analysis> analysis) {
	return createUiDelegate(analysis);
}



MOCK_CLASS(Project){
	MOCK_METHOD_EXT(addAnalysis, 1, void(Analysis *), addAnalysis);
};

typedef Analysis Analysis_t;
typedef AnalysisUiFactory Analysis_Ui_Factory_t;
typedef WizardUi Wizard_Ui_t;
typedef ui::UiManager<Analysis_t, Wizard_Ui_t, memory::CustomOwnershipPolicy<Wizard_Ui_t, memory::IgnoreOnDeleteChildPolicy> > Ui_Manager_t;

namespace test {

using namespace helper;

class ApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Ui_Manager_t, boost::bind(&ApplicationContext::uiManagerFactory, this)),
		GAC_DEFAULT_CTOR_INIT(Wizard_Ui_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Ui_Factory_t)
	{}

public:
	SubscriptionManager subscriptionManager;

private:
	Ui_Manager_t *uiManagerFactory(){
		Ui_Manager_t *analysisUiManager = new Ui_Manager_t();
		return analysisUiManager;
	}

public:
	GAC_SCOPE(Ui_Manager_t)
	GAC_SCOPE(Wizard_Ui_t)
	GAC_SCOPE(Analysis_Ui_Factory_t)
	//GAC_SCOPE(Analysis_t)
	boost::shared_ptr<Analysis_t> analysis;
};
} /* namespace test */





void testCreate(boost::shared_ptr<Analysis_t> analysis) {
	Ui_Manager_t *uiManager = FETCH_FROM_CTX(Ui_Manager_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Factory_t)->createUiDelegate).once().returns(FETCH_FROM_CTX(Wizard_Ui_t));
	BOOST_CHECK(uiManager->getFor(analysis, FETCH_FROM_CTX(Analysis_Ui_Factory_t)) == FETCH_FROM_CTX(Wizard_Ui_t));
}

BOOST_FIXTURE_TEST_CASE(GettingWizardWithoutHavingOneAlready, test::ApplicationContext)
{
	testCreate(analysis);
}


BOOST_FIXTURE_TEST_CASE(GettingWizardWithHavingOneAlready, test::ApplicationContext)
{
	//create one
	testCreate(analysis);
	mock::reset();

	//testcase
	Ui_Manager_t *uiManager = FETCH_FROM_CTX(Ui_Manager_t);
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Factory_t)->createUiDelegate).never();

	BOOST_CHECK(uiManager->getFor(analysis, FETCH_FROM_CTX(Analysis_Ui_Factory_t)) == FETCH_FROM_CTX(Wizard_Ui_t));
}


BOOST_FIXTURE_TEST_CASE(ContainsUiReturnsTrueIfOneAlreadyExists, test::ApplicationContext)
{
	//create one
	testCreate(analysis);
	mock::reset();

	Ui_Manager_t *uiManager = FETCH_FROM_CTX(Ui_Manager_t);

	BOOST_CHECK(uiManager->containsFor(analysis) == true);
}



BOOST_FIXTURE_TEST_CASE(destroyWizardCallsOwnershipPolicyToDeleteChild, test::ApplicationContext) {
	//create one
	testCreate(analysis);
	mock::reset();

	//testcase
	Ui_Manager_t *uiManager = FETCH_FROM_CTX(Ui_Manager_t);
	uiManager->destroyFor(analysis);
}



BOOST_FIXTURE_TEST_CASE(destroyWizardDoesNotCallsOwnershipPolicyToDeleteChildIfWizardHasAlreadyBeenDestroyedEarlier, test::ApplicationContext) {
	//we simulate an earlier destroying of the wizard just by not creating one
	Ui_Manager_t *uiManager = FETCH_FROM_CTX(Ui_Manager_t);
	uiManager->destroyFor(analysis);
}


BOOST_FIXTURE_TEST_CASE(testGetForWithFunctionApproach, test::ApplicationContext) {
	Ui_Manager_t *uiManager = FETCH_FROM_CTX(Ui_Manager_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Ui_Factory_t)->createUiDelegate).once().with(analysis).returns(FETCH_FROM_CTX(Wizard_Ui_t));
	MOCK_EXPECT(subscriptionManager.delegateSubscribe).once().with(FETCH_FROM_CTX(Wizard_Ui_t));

	uiManager->getFor(
			analysis,
				boost::bind<WizardUi *>(&Analysis_Ui_Factory_t::create<Wizard_Ui_t, Analysis_t>, FETCH_FROM_CTX(Analysis_Ui_Factory_t), _1),
				boost::bind(&SubscriptionManager::subscribe, &subscriptionManager, _1)
			);
}
