/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelUiManager.hpp
 *
 *  Created on: Feb 28, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELUIMANAGER_HPP_
#define REGRESSIONMODELUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include <boost/bind.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {

template
<
	typename RegressionModel,
	typename OverviewUi,
	typename CommonParameterUi,
	typename TimeRangeUi,
	typename BehaviorIndicatorUi,
	typename RegressionFormulaUi,
	typename AlgorithmUi,
	typename RegressionExecuteUi,
	typename WizardUi,
	typename InfoDialogUi,
	typename RegressorInfluenceFactorInstantiationMessageUi,
	typename InfluenceFactorSelectionUi, //used together with the wizard
	typename RegressionModelDefinitionEditUi, //used by the wizard
	typename RegressorFormulaBuilderEditUi, //used by the wizard
	typename UserParameterNameUi, //used by the wizard
	typename WorkingAreaContainerUi,
	typename UiFactory,
	typename ProgressManager,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class RegressionModelUiManager {
public:
	typedef WizardUi wizard_ui_t;
	typedef InfoDialogUi info_dialog_ui_type;
	typedef OverviewUi overview_ui_t;
	typedef WorkingAreaContainerUi working_area_container_ui_t;
	typedef InfluenceFactorSelectionUi influence_factor_selection_ui_t;
	typedef RegressorInfluenceFactorInstantiationMessageUi regressor_influence_factor_instantiation_message_ui_t;
	typedef UserParameterNameUi user_parameter_name_ui_t;
private:
	typedef typename OverviewUi::left_widget_t Data_Overview_Widget_t;
	typedef typename OverviewUi::right_widget_t Analysis_Widget_t;
	typedef typename Analysis_Widget_t::Physics_Category_Widget_t Physics_Category_Widget_t;
	typedef typename Analysis_Widget_t::Statistics_Category_Widget_t Statistics_Category_Widget_t;
	typedef typename RegressionModel::Regression_Formula_t::RegressionModelDefinition_t RegressionModelDefinition_t;
	typedef typename RegressionModel::Regression_Formula_t::RegressorFormulaBuilder_t RegressorFormulaBuilder_t;
	typedef typename RegressionModelDefinition_t::Regressor_t Regressor_t;
public:
	RegressionModelUiManager(UiFactory *uiFactory, ProgressManager *progressManager, WorkingAreaContainerUi *workingAreaContainerUi)
		:uiFactory(uiFactory), workingAreaContainerUi(workingAreaContainerUi), progressManager(progressManager)
	{
	}

	virtual ~RegressionModelUiManager(){;}

	ProgressManager *getProgressManager() {
		return progressManager;
	}

	template<typename SubscriptionManager>
	RegressionModelDefinitionEditUi *getRegressionModelDefinitionEditUiFor(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<typename RegressionModel::Regression_Formula_t> formula, boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition, SubscriptionManager *subscriptionManager) {
		using namespace boost;
		typedef typename RegressionModel::Regression_Formula_t RegressionFormula_t;
		bool newUiCreated;


		RegressionModelDefinitionEditUi *ui = regressionModelDefinitionEditUiManager.getFor(
				regressionModelDefinition,
				bind((RegressionModelDefinitionEditUi *(UiFactory::*)(boost::shared_ptr<RegressionFormula_t>, boost::shared_ptr<RegressionModelDefinition_t>))&UiFactory::template create<RegressionModelDefinitionEditUi, RegressionFormula_t, RegressionModelDefinition_t>, uiFactory, formula, regressionModelDefinition)
				, newUiCreated);
		if(newUiCreated) {
			subscriptionManager->template subscribe<RegressionModel>(ui, regressionModel);
		}

		return ui;
	}

	void destroyRegressionModelDefinitionEditUiFor(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition) {
		regressionModelDefinitionEditUiManager.destroyFor(regressionModelDefinition);
	}

	template<typename SubscriptionManager>
	RegressorFormulaBuilderEditUi *getRegressorFormulaBuilderEditUiFor(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		bool newUiCreated;

		RegressorFormulaBuilderEditUi *ui = regressorFormulaBuilderEditUiManager.getFor(regressorFormulaBuilder, uiFactory, newUiCreated);
		if(newUiCreated) {
			subscriptionManager->template subscribe<RegressionModel>(ui, regressionModel);
		}

		return ui;
	}

	void destroyRegressorFormulaBuilderEditUiFor(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) {
		regressorFormulaBuilderEditUiManager.destroyFor(regressorFormulaBuilder);
	}

	RegressorInfluenceFactorInstantiationMessageUi *getRegressorInfluenceFactorInstantiationMessageUiFor(boost::shared_ptr<Regressor_t> regressor) {
		return regressorInfluenceFactorInstantiationMessageUiManager.getFor(regressor, uiFactory);
	}

	void destroyRegressorInfluenceFactorInstantiationMessageUiFor(boost::shared_ptr<Regressor_t> regressor) {
		regressorInfluenceFactorInstantiationMessageUiManager.destroyFor(regressor);
	}


	template<typename SubscriptionManager>
	InfluenceFactorSelectionUi *getInfluenceFactorSelectionUiFor(boost::shared_ptr<Regressor_t> regressor, SubscriptionManager *subscriptionManager) {
		bool newUiCreated;

		InfluenceFactorSelectionUi *ui = influenceFactorSelectionUiManager.getFor(regressor, uiFactory, newUiCreated);
		if(newUiCreated) {
			subscriptionManager->subscribe(ui);
		}

		return ui;
	}

	void destroyInfluenceFactorSelectionUiFor(boost::shared_ptr<Regressor_t> regressor) {
		influenceFactorSelectionUiManager.destroyFor(regressor);
	}

	UserParameterNameUi *getUserParameterNameUiFor(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) {
		return userParameterNameUiManager.getFor(regressorFormulaBuilder, uiFactory);
	}

	void destroyUserParameterNameUiFor(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) {
		userParameterNameUiManager.destroyFor(regressorFormulaBuilder);
	}


	template<typename SubscriptionManager>
	WizardUi *getWizardFor(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		bool newUiCreated = false;
		WizardUi *wizard = wizardUiManager.getFor(regressionModel, uiFactory, newUiCreated);

		//call init on wizard's getRegressionModelDefinitionsWidget with an appropriate factory
		wizard->getRegressionModelDefinitionsWidget()->init(bind(&RegressionModelUiManager<
					RegressionModel,
					OverviewUi,
					CommonParameterUi,
					TimeRangeUi,
					BehaviorIndicatorUi,
					RegressionFormulaUi,
					AlgorithmUi,
					RegressionExecuteUi,
					WizardUi,
					InfoDialogUi,
					RegressorInfluenceFactorInstantiationMessageUi,
					InfluenceFactorSelectionUi,
					RegressionModelDefinitionEditUi,
					RegressorFormulaBuilderEditUi,
					UserParameterNameUi,
					WorkingAreaContainerUi,
					UiFactory,
					ProgressManager,
					OwnershipPolicy
				>::template getRegressionModelDefinitionEditUiFor<SubscriptionManager>, this, regressionModel, _1, _2, subscriptionManager)); //provide the subscriptionManager after _1

		//call init on wizard's getRegressionModelDefinitionsWidget with an appropriate factory
		wizard->getRegressorFormulaBuildersWidget()->init(bind(&RegressionModelUiManager<
							RegressionModel,
							OverviewUi,
							CommonParameterUi,
							TimeRangeUi,
							BehaviorIndicatorUi,
							RegressionFormulaUi,
							AlgorithmUi,
							RegressionExecuteUi,
							WizardUi,
							InfoDialogUi,
							RegressorInfluenceFactorInstantiationMessageUi,
							InfluenceFactorSelectionUi,
							RegressionModelDefinitionEditUi,
							RegressorFormulaBuilderEditUi,
							UserParameterNameUi,
							WorkingAreaContainerUi,
							UiFactory,
							ProgressManager,
							OwnershipPolicy
						>::template getRegressorFormulaBuilderEditUiFor<SubscriptionManager>, this, _1, regressionModel, subscriptionManager)); //provide the subscriptionManager after _1


		if(newUiCreated) {
			subscriptionManager->subscribe(wizard);
		}

		return wizard;
	}

	InfoDialogUi *getInfoDialogUiFor(boost::shared_ptr<RegressionModel> regressionModel) {
		return infoDialogUiManager.getFor(regressionModel, uiFactory);
	}

	void destroyInfoDialogUiFor(boost::shared_ptr<RegressionModel> regressionModel) {
		infoDialogUiManager.destroyFor(regressionModel);
	}

	void destroyWizardFor(boost::shared_ptr<RegressionModel> regressionModel) {
		wizardUiManager.destroyFor(regressionModel);

		BOOST_FOREACH(boost::shared_ptr<RegressionModelDefinition_t> regressionModelDefinition, regressionModel->getRegressionFormula()->getAllRegressionModelDefinitions()) {
			destroyRegressionModelDefinitionEditUiFor(regressionModelDefinition);
		}

		BOOST_FOREACH(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, regressionModel->getRegressionFormula()->getRegressorFormulaBuilders()) {
			destroyRegressorFormulaBuilderEditUiFor(regressorFormulaBuilder);
		}
	}

	template<typename SubscriptionManager>
	Physics_Category_Widget_t *getPhysicsCategoryWidgetFor(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace boost;

		Physics_Category_Widget_t *physicsCategoryWidget = physicsWidgetUiManager.getFor(
			regressionModel,
			bind((Physics_Category_Widget_t *(UiFactory::*)())&UiFactory::template create<Physics_Category_Widget_t>, uiFactory)
		);

		return physicsCategoryWidget;
	}

	template<typename SubscriptionManager>
	Statistics_Category_Widget_t *getStatisticsCategoryWidgetFor(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace boost;

		Statistics_Category_Widget_t *statisticsCategoryWidget = statisticsWidgetUiManager.getFor(
			regressionModel,
			bind((Statistics_Category_Widget_t *(UiFactory::*)())&UiFactory::template create<Statistics_Category_Widget_t>, uiFactory)
		);

		return statisticsCategoryWidget;
	}

	template<typename SubscriptionManager>
	Physics_Category_Widget_t *getLeftSidePhysicsCategoryWidgetFor(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace boost;

		Physics_Category_Widget_t *physicsCategoryWidget = leftSidePhysicsWidgetUiManager.getFor(
			regressionModel,
			bind((Physics_Category_Widget_t *(UiFactory::*)())&UiFactory::template create<Physics_Category_Widget_t>, uiFactory)
		);

		return physicsCategoryWidget;
	}

	template<typename SubscriptionManager>
	Statistics_Category_Widget_t *getLeftSideStatisticsCategoryWidgetFor(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace boost;

		Statistics_Category_Widget_t *statisticsCategoryWidget = leftSideStatisticsWidgetUiManager.getFor(
			regressionModel,
			bind((Statistics_Category_Widget_t *(UiFactory::*)())&UiFactory::template create<Statistics_Category_Widget_t>, uiFactory)
		);

		return statisticsCategoryWidget;
	}

	bool containsOverviewUiFor(boost::shared_ptr<RegressionModel> regressionModel) {
		return overviewUiManager.containsFor(regressionModel);
	}

	template<typename SubscriptionManager>
	OverviewUi *getOverviewUiFor(boost::shared_ptr<RegressionModel> regressionModel, SubscriptionManager *subscriptionManager) {
		using namespace boost;

		bool isNewOverviewUi = false;
		OverviewUi *overviewUi = overviewUiManager.getFor(
					regressionModel,
					bind((OverviewUi *(UiFactory::*)())&UiFactory::template create<OverviewUi>, uiFactory),
					isNewOverviewUi,
					bind((void(SubscriptionManager::*)(OverviewUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				);

		if(isNewOverviewUi == false) {
			return overviewUi;
		}

		//DATA part
		std::vector<QWidget *> dataOverviewWidgets;


		dataOverviewWidgets.push_back(commonParameterUiManager.getFor(
					regressionModel,
					bind((CommonParameterUi *(UiFactory::*)(boost::shared_ptr<RegressionModel>))&UiFactory::template create<CommonParameterUi, RegressionModel>, uiFactory, regressionModel),
					bind((void(SubscriptionManager::*)(CommonParameterUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				)
		);

		dataOverviewWidgets.push_back(timeRangeUiManager.getFor(
					regressionModel,
					bind((TimeRangeUi *(UiFactory::*)(boost::shared_ptr<RegressionModel>))&UiFactory::template create<TimeRangeUi, RegressionModel>, uiFactory, regressionModel),
					bind((void(SubscriptionManager::*)(TimeRangeUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				)
		);

		dataOverviewWidgets.push_back(behaviorIndicatorUiManager.getFor(
					regressionModel,
					bind((BehaviorIndicatorUi *(UiFactory::*)(boost::shared_ptr<RegressionModel>))&UiFactory::template create<BehaviorIndicatorUi, RegressionModel>, uiFactory, regressionModel),
					bind((void(SubscriptionManager::*)(BehaviorIndicatorUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				)
		);

		dataOverviewWidgets.push_back(regressionFormulaUiManager.getFor(
					regressionModel,
					bind((RegressionFormulaUi *(UiFactory::*)(boost::shared_ptr<RegressionModel>))&UiFactory::template create<RegressionFormulaUi, RegressionModel>, uiFactory, regressionModel),
					bind((void(SubscriptionManager::*)(RegressionFormulaUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				)
		);

		dataOverviewWidgets.push_back(algorithmUiManager.getFor(
					regressionModel,
					bind((AlgorithmUi *(UiFactory::*)(boost::shared_ptr<RegressionModel>))&UiFactory::template create<AlgorithmUi, RegressionModel>, uiFactory, regressionModel),
					bind((void(SubscriptionManager::*)(AlgorithmUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				)
		);

		dataOverviewWidgets.push_back(regressionExecuteUiManager.getFor(
					regressionModel,
					bind((RegressionExecuteUi *(UiFactory::*)(boost::shared_ptr<RegressionModel>))&UiFactory::template create<RegressionExecuteUi, RegressionModel>, uiFactory, regressionModel),
					bind((void(SubscriptionManager::*)(RegressionExecuteUi *))&SubscriptionManager::subscribe, subscriptionManager, _1)
				));


		Data_Overview_Widget_t *dataOverviewWidget = dataOverviewWidgetUiManager.getFor(
					regressionModel,
					bind((Data_Overview_Widget_t *(UiFactory::*)())&UiFactory::template create<Data_Overview_Widget_t>, uiFactory)
				);


		//ANALYSIS PART AT THE DATA (LEFT) SIDE
		Analysis_Widget_t *leftSideAnalysisWidget = leftSideAnalysisWidgetUiManager.getFor(
			regressionModel,
			bind((Analysis_Widget_t *(UiFactory::*)())&UiFactory::template create<Analysis_Widget_t>, uiFactory)
		);
		Physics_Category_Widget_t *leftSidePhysicsCategoryWidget = getLeftSidePhysicsCategoryWidgetFor(regressionModel, subscriptionManager);
		Statistics_Category_Widget_t *leftSideStatisticsCategoryWidget = getLeftSideStatisticsCategoryWidgetFor(regressionModel, subscriptionManager);
		leftSideAnalysisWidget->setPhysicsCategoryWidget(leftSidePhysicsCategoryWidget);
		leftSideAnalysisWidget->setStatisticsCategoryWidget(leftSideStatisticsCategoryWidget);
		leftSideAnalysisWidget->init();
		//FINISH ANALYSIS PART AT THE DATA (LEFT) SIDE


		dataOverviewWidgets.push_back(leftSideAnalysisWidget);
		dataOverviewWidget->setWidgets(dataOverviewWidgets);
		dataOverviewWidget->init();


		//ANALYSIS part at the analysis tools (right) side
		Analysis_Widget_t *analysisWidget = analysisWidgetUiManager.getFor(
					regressionModel,
					bind((Analysis_Widget_t *(UiFactory::*)())&UiFactory::template create<Analysis_Widget_t>, uiFactory)
				);

		Physics_Category_Widget_t *physicsCategoryWidget = getPhysicsCategoryWidgetFor(regressionModel, subscriptionManager);
		Statistics_Category_Widget_t *statisticsCategoryWidget = getStatisticsCategoryWidgetFor(regressionModel, subscriptionManager);

		analysisWidget->setPhysicsCategoryWidget(physicsCategoryWidget);
		analysisWidget->setStatisticsCategoryWidget(statisticsCategoryWidget);

		analysisWidget->init();


		//FIT everything together
		overviewUi->setLeftWidget(dataOverviewWidget);
		overviewUi->setRightWidget(analysisWidget);
		overviewUi->init();

		return overviewUi;
	}

	void destroyOverviewUiFor(boost::shared_ptr<RegressionModel> regressionModel) {
		overviewUiManager.destroyFor(regressionModel);
		commonParameterUiManager.destroyFor(regressionModel);
		timeRangeUiManager.destroyFor(regressionModel);
		behaviorIndicatorUiManager.destroyFor(regressionModel);
		regressionFormulaUiManager.destroyFor(regressionModel);
		algorithmUiManager.destroyFor(regressionModel);
		regressionExecuteUiManager.destroyFor(regressionModel);
		dataOverviewWidgetUiManager.destroyFor(regressionModel);

		analysisWidgetUiManager.destroyFor(regressionModel);
		physicsWidgetUiManager.destroyFor(regressionModel);
		statisticsWidgetUiManager.destroyFor(regressionModel);

		leftSideAnalysisWidgetUiManager.destroyFor(regressionModel);
		leftSidePhysicsWidgetUiManager.destroyFor(regressionModel);
		leftSideStatisticsWidgetUiManager.destroyFor(regressionModel);
	}

	WorkingAreaContainerUi *getWorkingAreaContainerUi() {
		return workingAreaContainerUi;
	}


private:
	UiFactory *uiFactory;
	UiManager<Regressor_t, RegressorInfluenceFactorInstantiationMessageUi, OwnershipPolicy<RegressorInfluenceFactorInstantiationMessageUi> > regressorInfluenceFactorInstantiationMessageUiManager;
	UiManager<Regressor_t, InfluenceFactorSelectionUi, OwnershipPolicy<InfluenceFactorSelectionUi> > influenceFactorSelectionUiManager;
	UiManager<RegressionModelDefinition_t, RegressionModelDefinitionEditUi, OwnershipPolicy<RegressionModelDefinitionEditUi, memory::IgnoreOnDeleteChildPolicy> > regressionModelDefinitionEditUiManager;
	UiManager<RegressorFormulaBuilder_t, RegressorFormulaBuilderEditUi, OwnershipPolicy<RegressorFormulaBuilderEditUi, memory::IgnoreOnDeleteChildPolicy> > regressorFormulaBuilderEditUiManager;
	UiManager<RegressorFormulaBuilder_t, user_parameter_name_ui_t, OwnershipPolicy<user_parameter_name_ui_t> > userParameterNameUiManager;
	UiManager<RegressionModel, WizardUi, OwnershipPolicy<WizardUi> > wizardUiManager;
	UiManager<RegressionModel, InfoDialogUi, OwnershipPolicy<InfoDialogUi> > infoDialogUiManager;
	UiManager<RegressionModel, OverviewUi, OwnershipPolicy<OverviewUi> > overviewUiManager;
	UiManager<RegressionModel, CommonParameterUi, OwnershipPolicy<CommonParameterUi, memory::IgnoreOnDeleteChildPolicy> > commonParameterUiManager;
	UiManager<RegressionModel, TimeRangeUi, OwnershipPolicy<TimeRangeUi, memory::IgnoreOnDeleteChildPolicy> > timeRangeUiManager;
	UiManager<RegressionModel, BehaviorIndicatorUi, OwnershipPolicy<BehaviorIndicatorUi, memory::IgnoreOnDeleteChildPolicy> > behaviorIndicatorUiManager;
	UiManager<RegressionModel, RegressionFormulaUi, OwnershipPolicy<RegressionFormulaUi, memory::IgnoreOnDeleteChildPolicy> > regressionFormulaUiManager;
	UiManager<RegressionModel, AlgorithmUi, OwnershipPolicy<AlgorithmUi, memory::IgnoreOnDeleteChildPolicy> > algorithmUiManager;
	UiManager<RegressionModel, RegressionExecuteUi, OwnershipPolicy<RegressionExecuteUi, memory::IgnoreOnDeleteChildPolicy> > regressionExecuteUiManager;
	UiManager<RegressionModel, Data_Overview_Widget_t, OwnershipPolicy<Data_Overview_Widget_t, memory::IgnoreOnDeleteChildPolicy> > dataOverviewWidgetUiManager;

	UiManager<RegressionModel, Analysis_Widget_t, OwnershipPolicy<Analysis_Widget_t, memory::IgnoreOnDeleteChildPolicy> > analysisWidgetUiManager;
	UiManager<RegressionModel, Physics_Category_Widget_t, OwnershipPolicy<Physics_Category_Widget_t, memory::IgnoreOnDeleteChildPolicy> > physicsWidgetUiManager;
	UiManager<RegressionModel, Statistics_Category_Widget_t, OwnershipPolicy<Statistics_Category_Widget_t, memory::IgnoreOnDeleteChildPolicy> > statisticsWidgetUiManager;

	UiManager<RegressionModel, Analysis_Widget_t, OwnershipPolicy<Analysis_Widget_t, memory::IgnoreOnDeleteChildPolicy> > leftSideAnalysisWidgetUiManager;
	UiManager<RegressionModel, Physics_Category_Widget_t, OwnershipPolicy<Physics_Category_Widget_t, memory::IgnoreOnDeleteChildPolicy> > leftSidePhysicsWidgetUiManager;
	UiManager<RegressionModel, Statistics_Category_Widget_t, OwnershipPolicy<Statistics_Category_Widget_t, memory::IgnoreOnDeleteChildPolicy> > leftSideStatisticsWidgetUiManager;
	WorkingAreaContainerUi *workingAreaContainerUi;
	ProgressManager *progressManager;
};

}


#endif /* REGRESSIONMODELUIMANAGER_HPP_ */
