/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportUiManager.hpp
 *
 *  Created on: Jul 24, 2014
 *      Author: gerberma
 */

#ifndef REPORTUIMANAGER_HPP_
#define REPORTUIMANAGER_HPP_

#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {

template<
	typename Report,
	typename WorkingDialogUi,
	typename ReportListTitleElementUi,
	typename ReportListDescriptionElementUi,
	typename ReportListProjectElementUi,
	typename ReportListAlgorithmElementUi,
	typename ReportListAnalysisToolElementUi,
	typename SaveDialogUi,
	typename UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class ReportUiManager {
public:
	typedef WorkingDialogUi working_dialog_ui_t;
	typedef SaveDialogUi save_dialog_ui_t;

	typedef typename Report::ReportListElement_t ReportListElement_t;
	typedef typename Report::TitleReportListElement_t TitleReportListElement_t;
	typedef typename Report::AnalysisToolReportListElement_t AnalysisToolReportListElement_t;
	typedef typename Report::ProjectReportListElement_t ProjectReportListElement_t;
	typedef typename Report::AlgorithmReportListElement_t AlgorithmReportListElement_t;
	typedef typename Report::DescriptionReportListElement_t DescriptionReportListElement_t;
public:
	ReportUiManager(UiFactory *uiFactory)
		:uiFactory(uiFactory)
	{;}

	template<typename SubscriptionManager>
	WorkingDialogUi *getWorkingDialogFor(boost::shared_ptr<Report> report, SubscriptionManager *subscriptionManager) {
		using namespace boost;
		bool newUiCreated = false;

		WorkingDialogUi *workingDialog = workingDialogUiManager.getFor(report, uiFactory, newUiCreated);

		if(newUiCreated) {
			subscriptionManager->subscribe(workingDialog);
		}

		//add report list element widgets
		BOOST_FOREACH(boost::shared_ptr<ReportListElement_t> reportListElement, report->getReportListElements()) {
			if(reportListElement == report->getTitleReportListElement()) {
				ReportListTitleElementUi *titleWidget = titleListElementUiManager.template getFor<UiFactory>(
							report->getTitleReportListElement(),
							uiFactory,
							bind((void(SubscriptionManager::*)(boost::shared_ptr<Report>, ReportListTitleElementUi *))&SubscriptionManager::subscribe, subscriptionManager, report, _1)
						);
				typename ReportListTitleElementUi::DynamicContentWidget_t *titleDynamicContentWidget = titleListElementDynamicContentUiManager.getFor(report->getTitleReportListElement(), uiFactory, subscriptionManager);
				titleWidget->setDynamicContentWidget(titleDynamicContentWidget);
				titleWidget->init();
				workingDialog->addReportListElementWidget(titleWidget);
			}else if(report->isAnalysisToolListElement(reportListElement)) {
				boost::shared_ptr<AnalysisToolReportListElement_t> analysisToolListElement = report->getAnalysisToolListElementFor(reportListElement);
				ReportListAnalysisToolElementUi *analysisToolWidget = analysisToolListElementUiManager.template getFor<UiFactory>(
							analysisToolListElement,
							uiFactory,
							bind((void(SubscriptionManager::*)(boost::shared_ptr<Report>, ReportListAnalysisToolElementUi *))&SubscriptionManager::subscribe, subscriptionManager, report, _1)
						);
				typename ReportListAnalysisToolElementUi::DynamicContentWidget_t *dynamicContentWidget = analysisToolListElementDynamicContentUiManager.getFor(analysisToolListElement, uiFactory, newUiCreated);
				analysisToolWidget->setDynamicContentWidget(dynamicContentWidget);
				analysisToolWidget->init();
				workingDialog->addReportListElementWidget(analysisToolWidget);
			}else if(report->isDescriptionReportListElement(reportListElement)) {
				boost::shared_ptr<DescriptionReportListElement_t> element = report->getDescriptionReportListElement();
				ReportListDescriptionElementUi *widget = descriptionListElementUiManager.template getFor<UiFactory>(
							element,
							uiFactory,
							bind((void(SubscriptionManager::*)(boost::shared_ptr<Report>, ReportListDescriptionElementUi *))&SubscriptionManager::subscribe, subscriptionManager, report, _1)
						);
				typename ReportListDescriptionElementUi::DynamicContentWidget_t *dynamicContentWidget = descriptionListElementDynamicContentUiManager.getFor(element, uiFactory, subscriptionManager);
				widget->setDynamicContentWidget(dynamicContentWidget);
				widget->init();
				workingDialog->addReportListElementWidget(widget);
			}else if(report->isProjectReportListElement(reportListElement)) {
				boost::shared_ptr<ProjectReportListElement_t> element = report->getProjectReportListElement();
				ReportListProjectElementUi *widget = projectListElementUiManager.template getFor<UiFactory>(
							element,
							uiFactory,
							bind((void(SubscriptionManager::*)(boost::shared_ptr<Report>, ReportListProjectElementUi *))&SubscriptionManager::subscribe, subscriptionManager, report, _1)
						);
				typename ReportListProjectElementUi::DynamicContentWidget_t *dynamicContentWidget = projectListElementDynamicContentUiManager.getFor(element, uiFactory, newUiCreated);
				widget->setDynamicContentWidget(dynamicContentWidget);
				widget->init();
				workingDialog->addReportListElementWidget(widget);
			}else if(report->isAlgorithmReportListElement(reportListElement)) {
				boost::shared_ptr<AlgorithmReportListElement_t> element = report->getAlgorithmReportListElement();
				ReportListAlgorithmElementUi *widget = algorithmListElementUiManager.template getFor<UiFactory>(
							element,
							uiFactory,
							bind((void(SubscriptionManager::*)(boost::shared_ptr<Report>, ReportListAlgorithmElementUi *))&SubscriptionManager::subscribe, subscriptionManager, report, _1)
						);
				typename ReportListAlgorithmElementUi::DynamicContentWidget_t *dynamicContentWidget = algorithmListElementDynamicContentUiManager.getFor(element, uiFactory, newUiCreated);
				widget->setDynamicContentWidget(dynamicContentWidget);
				widget->init();
				workingDialog->addReportListElementWidget(widget);
			}
		}


		return workingDialog;
	}

	void destroyWorkingDialogFor(boost::shared_ptr<Report> report) {
		workingDialogUiManager.destroyFor(report);
		titleListElementUiManager.destroyFor(report->getTitleReportListElement());
		titleListElementDynamicContentUiManager.destroyFor(report->getTitleReportListElement());

		projectListElementUiManager.destroyFor(report->getProjectReportListElement());
		projectListElementDynamicContentUiManager.destroyFor(report->getProjectReportListElement());

		algorithmListElementUiManager.destroyFor(report->getAlgorithmReportListElement());
		algorithmListElementDynamicContentUiManager.destroyFor(report->getAlgorithmReportListElement());

		BOOST_FOREACH(boost::shared_ptr<ReportListElement_t> reportListElement, report->getReportListElements()) {
			boost::shared_ptr<AnalysisToolReportListElement_t> analysisToolListElement = report->getAnalysisToolListElementFor(reportListElement);
			analysisToolListElementUiManager.destroyFor(analysisToolListElement);
			analysisToolListElementDynamicContentUiManager.destroyFor(analysisToolListElement);
		}

		descriptionListElementUiManager.destroyFor(report->getDescriptionReportListElement());
		descriptionListElementDynamicContentUiManager.destroyFor(report->getDescriptionReportListElement());
	}

	SaveDialogUi *getSaveDialogUiFor(boost::shared_ptr<Report> report) {
		return saveDialogUiManager.getFor(report, uiFactory);
	}

	void destroySaveDialogUiFor(boost::shared_ptr<Report> report) {
		saveDialogUiManager.destroyFor(report);
	}

private:
	UiFactory *uiFactory;
	UiManager<Report, WorkingDialogUi, OwnershipPolicy<WorkingDialogUi> > workingDialogUiManager;
	UiManager<Report, SaveDialogUi, OwnershipPolicy<SaveDialogUi> > saveDialogUiManager;
	UiManager<TitleReportListElement_t, ReportListTitleElementUi, OwnershipPolicy<ReportListTitleElementUi, memory::IgnoreOnDeleteChildPolicy> > titleListElementUiManager;
	UiManager<TitleReportListElement_t, typename ReportListTitleElementUi::DynamicContentWidget_t, OwnershipPolicy<typename ReportListTitleElementUi::DynamicContentWidget_t, memory::IgnoreOnDeleteChildPolicy> > titleListElementDynamicContentUiManager;
	UiManager<AnalysisToolReportListElement_t, ReportListAnalysisToolElementUi, OwnershipPolicy<ReportListAnalysisToolElementUi, memory::IgnoreOnDeleteChildPolicy> > analysisToolListElementUiManager;
	UiManager<AnalysisToolReportListElement_t, typename ReportListAnalysisToolElementUi::DynamicContentWidget_t, OwnershipPolicy<typename ReportListAnalysisToolElementUi::DynamicContentWidget_t, memory::IgnoreOnDeleteChildPolicy> > analysisToolListElementDynamicContentUiManager;
	UiManager<DescriptionReportListElement_t, ReportListDescriptionElementUi, OwnershipPolicy<ReportListDescriptionElementUi, memory::IgnoreOnDeleteChildPolicy> > descriptionListElementUiManager;
	UiManager<DescriptionReportListElement_t, typename ReportListDescriptionElementUi::DynamicContentWidget_t, OwnershipPolicy<typename ReportListDescriptionElementUi::DynamicContentWidget_t, memory::IgnoreOnDeleteChildPolicy> > descriptionListElementDynamicContentUiManager;
	UiManager<ProjectReportListElement_t, ReportListProjectElementUi, OwnershipPolicy<ReportListProjectElementUi, memory::IgnoreOnDeleteChildPolicy> > projectListElementUiManager;
	UiManager<ProjectReportListElement_t, typename ReportListProjectElementUi::DynamicContentWidget_t, OwnershipPolicy<typename ReportListProjectElementUi::DynamicContentWidget_t, memory::IgnoreOnDeleteChildPolicy> > projectListElementDynamicContentUiManager;
	UiManager<AlgorithmReportListElement_t, ReportListAlgorithmElementUi, OwnershipPolicy<ReportListAlgorithmElementUi, memory::IgnoreOnDeleteChildPolicy> > algorithmListElementUiManager;
	UiManager<AlgorithmReportListElement_t, typename ReportListAlgorithmElementUi::DynamicContentWidget_t, OwnershipPolicy<typename ReportListAlgorithmElementUi::DynamicContentWidget_t, memory::IgnoreOnDeleteChildPolicy> > algorithmListElementDynamicContentUiManager;
};

}  // namespace ui


#endif /* REPORTUIMANAGER_HPP_ */
